/*name: jqmodule

description: Provides methods to dynamically load JavaScript,
             and CSS files into the document.
             Provides methods to add and remove events.
             Provides methods for ajax.
Copyright (c) 2013, Generex GmbH. All rights reserved.
authors: said raqui
Last change: 08.06.2013
*/

var jqmodule = (function () {
  //Utils functions
  Array.prototype.remove = function (val) {
    var i = 0,
    len = this.length;
    for (; i < len; i += 1) {
      if (this[i] === val) {
        this.splice(i, 1);
        break;
      }
    }
    return this;
  }

  if (!Array.isArray) {
    Array.isArray = function (vArg) {
      return Object.prototype.toString.call(vArg) === "[object Array]";
    };
  }

  var trim = ''.trim ?
  function (s) { return s.trim(); } :
  function (s) { return s.replace(/^\s\s*/, '').replace(/\s\s*$/, ''); };

  String.prototype.empty = function () { return this == ''; };

  function extend(src, dist) //makes a "deep" copy of source.
  {
    for (var prop in src) {
      if (src.hasOwnProperty(prop)) // Filter Prototyp-prop
      {
        if (Array.isArray(src[prop]) && src[prop] !== null) {
          dist[prop] = dist[prop] || [];
          extend(src[prop], dist[prop]);
        } else if (typeof src[prop] === "object" && src[prop] !== null) {
          dist[prop] = dist[prop] || {};
          extend(src[prop], dist[prop]);
        } else
        dist[prop] = src[prop];
      }
    }
    return dist;
  }

  function serialize(object) //Serialize object for HTTP requests.
  {
    var results = [],
    key;
    if (!object) return;
    if (typeof object === 'string')
    return object;
    for (key in object)
    results.push(encodeURIComponent(key) + '=' + encodeURIComponent(object[key]));
    return results.join('&');
  }

  var create_node = function (name, attrs) {
    var node = document.createElement(name),
    attr;
    for (attr in attrs) {
      if (attrs.hasOwnProperty(attr)) {
        node.setAttribute(attr, attrs[attr]);
      }
    }
    return node;
  };

  var event = {
    add: null,
    remove: null,
    stop_propagation: function (evt) {
      if (evt.stopPropagation === 'function')
      evt.stopPropagation(); //W3C-DOM
      else
      evt.cancelBubble = true; //Internet Explorer < 9
    },
    prevent_default: function (evt) {
      if (evt.preventDefault === 'function') //W3C-DOM
      evt.preventDefault();
      else
      evt.returnValue = false; //Internet Explorer < 9
    },
    target: function (evt) {
      if (evt.target)
      return evt.target; //W3C-DOM
      return evt.srcElement; //Internet Explorer < 9
    }
  };
  //Init-Time Branching foor event
  if (typeof window.addEventListener === 'function') {
    event.add = function (el, type, fn) {
      el.addEventListener(type, fn, false);
    };
    event.remove = function (el, type, fn) {
      el.removeEventListener(type, fn, false);
    };
  } else if (typeof document.attachEvent === 'function') //IE
  {
    event.add = function (el, type, fn) {
      el.attachEvent('on' + type, fn);
    };
    event.remove = function (el, type, fn) {
      el.detachEvent('on' + type, fn);
    };
  } else //ältere Browser
  {
    event = function (el, type, fn) {
      el['on' + type] = fn;
    };
    event.remove = function (el, type, fn) {
      el['on' + type] = null;
    };
  }

  ///////////////////////begin ajax////////////////////////////////////////
  var ajax = function (url, options, settings) {
    var request,
    response = {},
    timeout_id, //timeout id
    _options = //defaults options.
    {
      data: {},
      data_type: null, //response data type
      username: null,
      password: null,
      success: null,
      error: null
    },
    _settings = //defaults settings.
    {
      timeout: 120000,
      cache: false,
      async: true,
      methode: 'POST',
      xhttp_methode_override: '',
      req_payload_json: false,
      content_type: "application/json",
      accepts: {
        text: "text/plain",
        html: "text/html",
        xml: "application/xml, text/xml",
        json: "application/json, text/javascript",
        all_types: "*/*"
      },
      converters: //Data converters
      {
        "text": function (response) { return response.responseText; },
        "html": function (response) { return response.responseText; },
        "json": parse_json,
        "xml": parse_xml
      }
    };

    options = options || {};
    extend(options, _options);
    setup(settings);
    send_request();

    function send_request() {
      try {
        request = xhr();
        request.open(_settings.methode, url, _settings.async);
        request.onreadystatechange = handle_response;
        set_headers(); //set Request headers
        send();
        if (_settings.async && _settings.timeout > 0)
        timeout_id = setTimeout(handle_timeout, _settings.timeout);
      } catch (e) {
        response.statusText = '';
        response.err_msg = e.name + ':' + e.message;
        return handle_error(response);
      }
    }

    function setup(settings) {
      settings = settings || {};
      extend(settings, _settings);
    }

    //private functions
    function handle_timeout() {
      request.abort();
      request.onreadystatechange = null;
      response.statusText = 'timeout';
      response.err_msg = 'Error:timeout';
      return handle_error(response);
    }

    function handle_error(response) {
      if (typeof _options.error === 'function')
      _options.error(response);
      return true;
    }

    function handle_response(evt) {
      var converter = null;
      if (request.readyState != 4)
      return;
      request.onreadystatechange = null;
      if (timeout_id)
      clearTimeout(timeout_id);
      response.status = request.status;
      response.statusText = request.statusText;
      response.responseText = request.responseText;
      response.responseXML = request.responseXML;
      response.content_type = request.getResponseHeader('Content-Type');
      response.responseURL = request.responseURL || window.location.href;

      if (!is_success(request)) {
        response.err_msg = 'something went wrong!';
        return handle_error(response);
      }
      //  This creates false Errors in Firefox and Chrome when run on Mac OS X
      //      if(!response.statusText || typeof response.statusText !== "string")
      //      {
      //        response.err_msg = 'something went wrong!';
      //        return handle_error(response);
      //      }
      try {
        converter = _settings.converters[_options.data_type];
        if (typeof converter === 'function') {
          if (response.status != 204) //No Content
          {
            response.data = converter(response);
          }
        } else {
          throw Error("Cannot find data converter for " + _options.data_type + "!");
        }

      } catch (e) {
        response.statusText = 'parsererror';
        response.err_msg = e.name + ':' + e.message;
        return handle_error(response);
      }
      if (typeof _options.success === 'function')
      _options.success(response.data, response.responseText);
    }

    //XMLHttpRequest functions
    function send() {
      if (_settings.methode === 'POST') {
        if (_settings.req_payload_json)
        _options.data = JSON.stringify(_options.data);
        else
        _options.data = serialize(_options.data);
        request.send(_options.data);
      } else
      request.send(null);

    }

    function is_success(request) //successful request?
    {
      return (request.status >= 200 && request.status < 300) ||
      request.status == 304 ||
      (request.status == 0 && request.responseText);
    }

    function set_headers() //set Request headers
    {
      var accepts = "";

      set_header(['Content-Type'], _settings.content_type);
      if (!_settings.xhttp_methode_override.empty()) {
        set_header(['X-HTTP-Method-Override'], _settings.xhttp_methode_override);
      }

      for (var name in _settings.accepts) {
        accepts += _settings.accepts[name] + (_settings.accepts[name] !== "*/*" ? ", " : "");;
      }
      request.setRequestHeader("Accept", accepts);
      request.setRequestHeader("Access-Control-Allow-Origin", '*');

      if (!_settings.cache) {
        request.setRequestHeader("Pragma", "no-cache");
        request.setRequestHeader("Cache-Control", "max-age=0,no-cache,no-store,post-check=0,pre-check=0");
        request.setRequestHeader("Expires", "Mon, 26 Jul 1997 05:00:00 GMT");
      }
    }

    function set_header(name, value) //set Request header
    {
      request.setRequestHeader(name, value);
    }

    function xhr() {
      var versions = [
        "MSXML2.XMLHttp.6.0", "MSXML2.XMLHttp.5.0",
        "MSXML2.XMLHttp.4.0", "MSXML2.XMLHttp.3.0",
        "MSXML2.XMLHttp", "Microsoft.XMLHttp"
      ],
      i = 0,
      len = versions.length,
      error_msg = null;

      if (typeof XMLHttpRequest !== "undefined")
      return new XMLHttpRequest();
      else if (window.ActiveXObject) {
        for (; i < len; i += 1) {
          try {
            var xhr = new ActiveXObject(versions[i]);
            return xhr;
          } catch (error) {}
        }
      }
      throw Error("ajax request object could not be created!");
    }

    //parse functions
    function parse_json(response) {
      var data = trim(response.responseText);
      if (typeof JSON !== 'undefined' && typeof JSON.parse === 'function')
      return JSON.parse(data);
      else
      return (new Function('return ' + data))();
    }

    function parse_xml(response) {
      var xml_doc = null,
      error_msg = null,
      versions = [
        'Msxml2.DOMDocument.6.0',
        'Msxml2.DOMDocument.4.0',
        'MSXML2.DOMDocument.3.0',
        'MSXML.DOMDocument' // 2.0
      ],
      i = 0,
      len = versions.length;
      if (response.responseXML) {
        xml_doc = response.responseXML;
      } else {
        var data = trim(response.responseText);
        if (window.DOMParser === 'function') {
          var dp = new DOMParser();
          xml_doc = dp.parseFromString(data, "application/xml");
        } else {
          for (; i < len; i += 1) {
            try {
              xml_doc = new ActiveXObject(versions[i]);
              xml_doc.async = 'false';
              xml_doc.loadXML(data);
              break;
            } catch (error) { xml_doc = null; }
          }
        }
      }

      if (!xml_doc)
      throw Error("Cannot create XMLDocument object.");
      if (xml_doc.parseError && xml_doc.parseError.errorCode != 0) //IE
      {
        error_msg = "XML Parsing Error: " + xml_doc.parseError.reason +
        " at line " + xml_doc.parseError.line +
        " at position " + xml_doc.parseError.linepos;
      } else {
        if (xml_doc.documentElement) {
          var parsererror = xml_doc.getElementsByTagName("parsererror")
          if (parsererror.length)
          error_msg = xml_doc.documentElement.childNodes[0].innerText;
        } else
        error_msg = "XML Parsing Error!";
      }

      if (error_msg)
      throw Error(error_msg);
      return xml_doc;
    }
  }
  /////////////////////////end ajax/////////////////////////////////////
  /////////////////////////////load on domande javascript and css///////
  var load = function () //TODO change
  {
    var head = document.head || document.getElementsByTagName('head')[0];

    var def_action = function (evt) {
      event.stop_propagation(evt);
      event.prevent_default(evt);
    };

    var success = function (evt, callbacks) {
      evt = evt || window.event;
      var target = event.target(evt);
      if (evt.type == 'readystatechange') //old IEs (readyState event)
      {
        if (target.readyState !== 'loaded' && target.readyState !== 'complete') {
          return;
        }
        if (target.readyState === 'loaded') {
          if (callbacks.onError && typeof (callbacks.onError) === 'function') {
            return error(evt, callbacks.onError);
          } else {
            def_action(evt);
            event.remove(target, evt.type, success);
            return false;
          }
        }
      }
      def_action(evt);
      event.remove(target, evt.type, success);
      callbacks.onLoad(evt, target);
    };

    return {
      javascript: function (source, callbacks) {
        var script = null;
        var that = this;
        callbacks = callbacks || {};
        var evt_type = 'load';
        //async></script> async is identical to defer, except that the script executes at the first opportunity after download
        //defer></script> deferred scripts are guaranteed to run in sequence

        script = create_node('script', { src: source, type: 'text/javascript', async: true, defer: 'defer' });
        if (callbacks.onLoad && typeof (callbacks.onLoad) === 'function') {
          if (typeof script.onload === 'undefined')
          evt_type = 'readystatechange'; //old IEs (readyState event)
          event.add(script, evt_type,
            function (evt) { success(evt, callbacks); },
            false
          );
        }
        if (callbacks.onError && typeof (callbacks.onError) === 'function') {
          event.add(script, 'error',
            function (evt) { error(evt, callbacks.onError); },
            false
          );
        }
        try {
          head.appendChild(script);
        } catch (e) {
          return handle_error();
        }
      }
    };
  }();
  ///////////////////////////// end load on domande javascript and css//////////////////////
  /////////////////////////////cross domaine request/////////////////////////////////////

  var cdr = function (origin, path) // change
  {
    var iframe = document.createElement("iframe"),
    success = function (evt) {
      //iframeWindow = iframe.contentWindow || iframe.contentDocument.parentWindow;
      iframe.contentWindow.postMessage('JSON.stringify(data.request)', origin);
    },
    error = function (evt) {
      alert('Event handling error');
    },
    handle_message = function (evt) {
      alert('Event handling Message received:' + evt.data);

      if (evt.origin == origin) {
        alert('Event handling Message received:' + evt.data + ' :' + evt.origin);
      }
    };

    event.add(iframe, "load", success, false);
    event.add(iframe, "error", error, false);
    event.add(window, "message", handle_message, false);

    iframe.style.cssText = "position:absolute;width:1px;height:1px;left:-9999px;";
    iframe.src = origin + "/" + path;
    document.body.appendChild(iframe);
  };

  return {
    event: event,
    load: load,
    ajax: ajax,
    cdr: cdr,
    extend: extend
  }
}());