/**

   panes.js : Functions for UPS screen.
   Copyright (c) 2012, Generex GmbH. All rights reserved.
   Said
**/
var panes_api = (function ($) {
  /*variables*/
  var panes = {};

  /*public functions*/
  panes.create_div = create_div;
  panes.create_log_event_pane = create_log_event_pane;
  panes.create_img_panel = create_img_panel;
  panes.create_loadbar_pane = create_loadbar_pane;
  panes.create_rom_temp_pane = create_rom_temp_pane;
  panes.set_rom_temp_values = set_rom_temp_values; //set rom temp values
  panes.set_loabar_value = set_loabar_value;
  panes.remove_loadbar_pane = remove_loadbar_pane;
  panes.create_err_panel = create_err_panel;
  panes.display_err = display_err;
  panes.create_br_check_panel = create_br_check_panel;
  panes.get_temperature = get_temperature;
  panes.create_autonomy_time_infos_panel = create_autonomy_time_infos_panel;
  panes.set_autonomy_time_infos = set_autonomy_time_infos;
  panes.toggle_visibility = toggle_visibility;
  return panes;

  function create_div(attrObj, css_obj, parent) {
    var div = $('<div/>').attr(attrObj).css(css_obj);
    parent = parent || document.body;
    $(div).appendTo(parent);
    return div;
  }

  function create_img_panel(img_id, src, left, top, w, h, border) {
    css_obj = { position: 'absolute', top: top, left: left, width: w, height: h, align: 'center', overflow: 'hidden' };
    var img_div = create_div({}, css_obj);
    $('<img />').attr({ 'id': img_id, width: w, 'height': h, 'src': src, border: border }).appendTo(img_div);
  }

  /*create log events pane*/
  function create_log_event_pane(log_entries, evt_attObj, evt_css_obj) {
    var log_div, th, tr, td, table, tbody, date;

    log_div = $('#' + evt_attObj['id']);

    if (log_div.length > 0)
      log_div.remove();

    log_div = create_div(evt_attObj, evt_css_obj);
    thead = $('<thead />');
    tbody = $('<tbody />');
    table = $('<table/>', {
      id: 'table_' + evt_attObj['id'],
      'class': evt_attObj['class'],
      'align': 'center',
      cellpadding: 0,
      cellspacing: 0,
      width: '200%',
      'text-align': 'center',
      'margin-left': '10px'
    });

    //log body
    tr = $('<tr />', { 'class': 'tdMediumWhiteBold2', height: 18, width: '100%' }).css({ 'background-color': '#405080', 'cellspacing': 0, 'color': 'white' });
    td = $('<td />', { align: 'left' }).append('&nbsp;&nbsp;&nbsp;Date');
    td.appendTo(tr);
    td = $('<td />', { align: 'left' }).append('&nbsp;&nbsp;&nbspTime');
    td.appendTo(tr);
    td = $('<td />', { align: 'left' }).append('&nbsp;&nbsp;&nbsp;&nbsp;&nbspEvent');
    td.appendTo(tr);
    tr.appendTo(tbody);

    $.each(log_entries, (function (i, log_entry) //log date, time and events
      {
        date = new Date(log_entry['time'] * 1000);
        tr = $('<tr />', { align: 'left' });
        tr.append($('<td />', { 'nowrap': true }).append('&nbsp;&nbsp;&nbsp' + utils_api.formatDate(date, "mm/dd/yy")));
        tr.append($('<td />').append('&nbsp;&nbsp;&nbsp' + utils_api.formatDate(date, "hh:nn:ss")));
        tr.append($('<td />', { id: 'xonl_evt_txt_' + i, nowrap: "yes", align: "left" }).css({ "padding-left": "16px" }).append(log_entry['event']));
        tr.appendTo(tbody);
        utils_api.rem_category_from_evt('xonl_evt_txt_' + i);
      }));

    //nowrap="yes" align="left" style="padding-left: 16px"
    tbody.appendTo(table);
    table.appendTo(log_div);

    $.each(log_entries, (function (i, log_entry) //log date, time and events
      {
        utils_api.rem_category_from_evt('xonl_evt_txt_' + i);
      }));

  }

  /* create loadbar pane*/
  function create_loadbar_pane(conf) //Load bar pane
  {
    var fragment = document.createDocumentFragment(),
      value = Math.round(conf.value);

    var val_green, val_yellow, val_red;

    if (value <= 80) {
      val_green = value;
      val_yellow = 0;
      val_red = 0;
    } else if (value > 80 && value <= 100) {
      val_green = 80;
      val_yellow = value - 80;
      val_red = 0;
    } else if (value > 80 && value > 100) {
      val_green = 80;
      val_yellow = 20;
      val_red = value - 100;
      if (val_red > 60)
        val_red = 60;
    }

    if (conf.scaler == 1) {
      css_obj = { position: 'absolute', top: conf.top - 10, left: conf.left + 49, width: 1, height: 10, 'background-color': 'black' };
      div = $('<div/>').attr({ id: 'scaler_1_l_' + conf.id }).css(css_obj);
      fragment.appendChild(div[0]);

      css_obj = { position: 'absolute', top: conf.top - 26, left: conf.left + 42, width: 20, height: 30 };
      div = $('<div/>').attr({ id: 'scaler_1_txt_' + conf.id }).css(css_obj);
      div.append('50%');
      fragment.appendChild(div[0]);

      css_obj = { position: 'absolute', top: conf.top - 10, left: conf.left + 99, width: 1, height: 10, 'background-color': 'black' };
      div = $('<div/>').attr({ id: 'scaler_2_l_' + conf.id }).css(css_obj);
      fragment.appendChild(div[0]);

      css_obj = { position: 'absolute', top: conf.top - 26, left: conf.left + 88, width: 20, height: 30 };
      div = $('<div/>').attr({ id: 'scaler_2_txt_' + conf.id }).css(css_obj);
      div.append('100%');
      fragment.appendChild(div[0]);

      css_obj = { position: 'absolute', top: conf.top - 10, left: conf.left + 149, width: 1, height: 10, 'background-color': 'black' };
      div = $('<div/>').attr({ id: 'scaler_3_l_' + conf.id }).css(css_obj);
      fragment.appendChild(div[0]);

      css_obj = { position: 'absolute', top: conf.top - 26, left: conf.left + 138, width: 20, height: 30 };
      div = $('<div/>').attr({ id: 'scaler_3_txt_' + conf.id }).css(css_obj);
      div.append('150%');
      fragment.appendChild(div[0]);
    }

    css_obj = { position: 'absolute', top: conf.top + 2, left: conf.left - 20, width: 20, height: 50 };
    div = $('<div/>').attr({ id: 'title_div_' + conf.id }).css(css_obj);
    div.append(conf.title);
    fragment.appendChild(div[0]);

    css_obj = { position: 'absolute', 'text-align': 'left', top: conf.top + 2, left: conf.left + 164, width: 60, height: 20 };
    div = $('<div/>').attr({ id: 'value_div_' + conf.id }).css(css_obj);
    div.append(value + ' %');
    fragment.appendChild(div[0]);

    css_obj = { position: 'absolute', top: conf.top, left: conf.left, width: 160, height: 20 };
    div_prgress = $('<div/>').attr({ id: 'bg_info_' + conf.id, 'class': 'progress' }).css(css_obj);
    fragment.appendChild(div_prgress[0]);

    css_obj = { width: val_green };
    div = $('<div/>').attr({ id: 'val_green_' + conf.id, 'class': 'progress-bar progress-bar-success' }).css(css_obj);
    div.appendTo(div_prgress);

    css_obj = { width: val_yellow };
    div = $('<div/>').attr({ id: 'val_yellow_' + conf.id, 'class': 'progress-bar progress-bar-warning' }).css(css_obj);
    div.appendTo(div_prgress);

    css_obj = { width: val_red };
    div = $('<div/>').attr({ id: 'val_red_' + conf.id, 'class': 'progress-bar progress-bar-danger' }).css(css_obj);
    div.appendTo(div_prgress);

    document.body.appendChild(fragment);
  }

  /* set loadbar value*/
  function set_loabar_value(value, id) {
    var val_green, val_yellow, val_red;
    var value = Math.round(value);

    if (value <= 80) {
      val_green = value;
      val_yellow = 0;
      val_red = 0;
    } else if (value > 80 && value <= 100) {
      val_green = 80;
      val_yellow = value - 80;
      val_red = 0;
    } else if (value > 80 && value > 100) {
      val_green = 80;
      val_yellow = 20;
      val_red = value - 100;
      if (val_red > 60)
        val_red = 60;
    }

    $('#value_div_' + id).text(value + ' %');
    $('#val_green_' + id).css('width', val_green);
    $('#val_yellow_' + id).css('width', val_yellow);
    $('#val_red_' + id).css('width', val_red);
  }

  function remove_loadbar_pane(id) {
    $('#scaler_1_l_' + id).remove();
    $('#scaler_1_txt_' + id).remove();
    $('#scaler_2_l_' + id).remove();
    $('#scaler_2_txt_' + id).remove();
    $('#scaler_3_l_' + id).remove();
    $('#scaler_3_txt_' + id).remove();
    $('#title_div_' + id).remove();
    $('#value_div_' + id).remove();
    $('#bg_info_' + id).remove();

    $('#val_green_' + id).remove();
    $('#val_yellow_' + id).remove();
    $('#val_red_' + id).remove();
  }

  function is_ge(_oemViewName) {
    var screen_name;
    if (_oemViewName == null)
      return false;
    var screen_name = _oemViewName.toLowerCase();

    if (screen_name === 'geml' || screen_name === 'gematch' ||
      screen_name === 'vh' || screen_name === 'delta' || screen_name === 'cp4')
      return true;
    return false;
  }

  function get_temperature(val, unit, precision) //unit = 0-->Celsius , unit = 1-->Fahrenheit
  {
    if (!unit) {
      return val.toFixed(precision) + ' &deg;C';
    }

    return ((val * 1.8) + 32.0).toFixed(precision) + ' &#176;F';
  }

  function create_rom_temp_pane(valtable_obj, attr_obj, temp_txt, ambtemp_txt) //rom_temp
  {
    var ups_protocol,
      tempdeg = valtable_obj.TEMPDEG,
      ambtemp = valtable_obj.AMBTEMP,
      comm2mode = valtable_obj.COM2MODE,
      temp1 = valtable_obj.TEMP1,
      temp2 = valtable_obj.TEMP2,
      val, tr, table, tbody, thead,
      content_count = 0;

    valtable_obj._tempUnit = valtable_obj._tempUnit || 0; //0 = Celsius

    if (tempdeg == null) {
      tempdeg = valtable_obj.upsBatteryTemperature;
    }

    if (valtable_obj._upsProtocol == null) {
      ups_protocol = valtable_obj.CONNECT;
      ups_protocol = (ups_protocol & 65535);
    } else {
      ups_protocol = valtable_obj._upsProtocol;
    }

    table = $('<table />', attr_obj),
      tbody = $('<tbody />').appendTo(table),
      thead = $('<thead />').appendTo(table);

    if (ups_protocol == 33031 || ups_protocol == 33032) {
      return table;
    }

    tr = $('<tr />').append($('<th />', { colspan: 3 }).append(i18n('Ext. Information'))).appendTo(thead);

    if (valtable_obj.EVO_BATTTEMPPROBE != null) {
      if (valtable_obj.EVO_BATTTEMPPROBE > 0) {
        tempdeg = get_temperature(tempdeg, valtable_obj._tempUnit, 1);
      } else {
        tempdeg = '--';
      }
      ++content_count;
    } else if (ups_protocol == 80E8 || ups_protocol == 33000) //Protect blue
    {
      if (tempdeg == 32768) {
        tempdeg = '???';
      } else if (valtable_obj.TEMPDEG == 32767) {
        tempdeg = '---';
      } else {
        tempdeg = get_temperature(tempdeg, valtable_obj._tempUnit, 1);
      }
      ++content_count;
    } else if (ups_protocol == 33021) //Centiel
    {
      if (tempdeg >= 120) {
        tempdeg = '---';
      } else {
        tempdeg = get_temperature(tempdeg, valtable_obj._tempUnit, 1);
      }
      ++content_count;
    } else {
      if ((tempdeg == null) || (tempdeg < -9998) || (tempdeg === "") ||
        (is_ge(valtable_obj._oemViewName) && tempdeg === -1)) {
        tempdeg = 'n/a';
      } else {
        tempdeg = get_temperature(tempdeg, valtable_obj._tempUnit, 1);
        ++content_count;
      }
    }

    if (temp_txt != null) {
      tr = $('<tr />').append($('<td />', { align: 'left' }).append(i18n(temp_txt) + ':'));
    } else {
      tr = $('<tr />').append($('<td />', { align: 'left' }).append(i18n("Device") + ' ' + i18n(lang_Temperature_short + '.') + ':'));
    }
    tr.append($('<td />', { 'id': 'rom_temp_id', align: 'left' }).append(tempdeg)).appendTo(tbody);

    if (ups_protocol != 33026) //JBUSDAKER
    {
      if ((ambtemp == null) || (ambtemp < -9998) || (ambtemp === "")) {
        ambtemp = 'n/a';
      } else {
        ambtemp = get_temperature(ambtemp, valtable_obj._tempUnit, 1);
        if (ambtemp_txt != null) {
          tr = $('<tr />').append($('<td />', { align: 'left' }).append(i18n(ambtemp_txt) + ':'));
        } else {
          tr = $('<tr />').append($('<td />', { align: 'left' }).append(i18n("Ambient") + " " + i18n(lang_Temperature_short + '.') + ':'));
        }
        tr.append($('<td />', { 'id': 'amb_temp_id', align: 'left' }).append(ambtemp)).appendTo(tbody);
        ++content_count;
      }
    }

    if (comm2mode != null && comm2mode !== '') //only cs121
    {
      if ((typeof object === 'string' && comm2mode.empty()))
        return false;

      tr = $('<tr />');
      val = comm2mode.toFixed(0);
      if (val == 14) {
        val = temp2.toFixed(0) + ' rel. H';
        $(tr).append($('<td />', { align: 'left' }).append(i18n(lang_SM_H) + ':'));
        tr.append($('<td />', { 'id': 'rom_sm_h_id', align: 'left' }).append(val)).appendTo(tbody);
        tr = $('<tr />').appendTo(tbody);
        $(tr).append($('<td />', { align: 'left' }).append(i18n(lang_SM_T) + ':'));
        val = temp1.toFixed(0) + ' &#176;C';
        tr.append($('<td />', { 'id': 'rom_sm_t_id', align: 'left' }).append(val)).appendTo(tbody);
        ++content_count;
      } else if (val == 11) {
        val = temp1.toFixed(0) + ' &#176;C';
        $(tr).append($('<td />', { align: 'left' }).append(i18n(lang_SM_T)));
        tr.append($('<td />', { 'id': 'rom_sm_t_id', align: 'left' }).append(val)).appendTo(tbody);
        ++content_count;
      }
    }

    if (valtable_obj.EX_NOM_VA != null && valtable_obj.EX_NOM_VA != 0) {
      tr = $('<tr />').append($('<td />', { align: 'left' }).append(i18n('Nominal Power') + ':'));
      tr.appendTo(tbody);
      td = $('<td />', { id: 'nominal_power', align: 'left' }).appendTo(tr);

      if (ups_protocol == 32781 || ups_protocol == 32818 || ups_protocol == 32912) // Meta: ECONT, EFACEC, ECONT2
      {
        td.append((valtable_obj.EX_NOM_VA).toFixed(0) + ' W');
        ++content_count;
      } else {
        if (valtable_obj.EX_NOM_VA >= 1000) {
          td.append((valtable_obj.EX_NOM_VA / 1000).toFixed(0) + ' kVA');
          ++content_count;
        } else {
          if (ups_protocol == 32963 || ups_protocol == 33031 || ups_protocol == 33032)   // PREXT, PREXTXANTO, PREXTZINTO
             td.append((valtable_obj.EX_NOM_VA).toFixed(0) + ' VA');
          else
            td.append((valtable_obj.EX_NOM_VA).toFixed(0) + ' kVA');
          ++content_count;
        }
      }
    }


    if (ups_protocol == 32988 || ups_protocol == 32962) //MBCUBE2 and MBCUBE
    {
      tr = $('<tr />').append($('<td />', { align: 'left' }).append(i18n('Inverter') + ' ' + i18n('Temp.') + ':')).appendTo(tbody);
      td = $('<td />', { id: 'inverter_temp', align: 'left' }).append(get_temperature(valtable_obj.CUBE_TEMPINV, valtable_obj._tempUnit, 1)).appendTo(tr);

      tr = $('<tr />').append($('<td />', { align: 'left' }).append(i18n('Rectifier') + ' ' + i18n('Temp.') + ':')).appendTo(tbody);
      td = $('<td />', { id: 'rectifier_temp', align: 'left' }).append(get_temperature(valtable_obj.CUBE_TEMPRECT, valtable_obj._tempUnit, 1)).appendTo(tr);
      content_count += 2;

    }

    if (content_count == 0) {
      table.empty();
    }
    return table;
  }

  function set_rom_temp_values(valtable_obj) //set rom temp values
  {
    var ups_protocol,
      tempdeg = valtable_obj.TEMPDEG,
      ambtemp = valtable_obj.AMBTEMP,
      comm2mode = valtable_obj.COM2MODE,
      temp1 = valtable_obj.TEMP1,
      temp2 = valtable_obj.TEMP2,
      val;

    valtable_obj._tempUnit = valtable_obj._tempUnit || 0; //0 = Celsius

    if (tempdeg == null)
      tempdeg = valtable_obj.upsBatteryTemperature;

    if (valtable_obj._upsProtocol == null) {
      ups_protocol = valtable_obj.CONNECT;
      ups_protocol = (ups_protocol & 65535);
    } else
      ups_protocol = valtable_obj._upsProtocol;

    if (ups_protocol == '33031' || ups_protocol == '33032') {
      return;
    }

    if (valtable_obj.EVO_BATTTEMPPROBE != null) {
      if (valtable_obj.EVO_BATTTEMPPROBE > 0)
        tempdeg = get_temperature(tempdeg, valtable_obj._tempUnit, 1);
      else
        tempdeg = '--';
    } else if (ups_protocol == '80E8' || ups_protocol == '33000') //Protect blue
    {
      if (tempdeg == 32768)
        tempdeg = '???';
      else if (valtable_obj.TEMPDEG == 32767)
        tempdeg = '---';
      else
        tempdeg = get_temperature(tempdeg, valtable_obj._tempUnit, 1);
    } else if (ups_protocol == 33021) //Centiel
    {
      if (tempdeg >= 120) {
        tempdeg = '---';
      } else {
        tempdeg = get_temperature(tempdeg, valtable_obj._tempUnit, 1);
      }
    } else {
      if ((tempdeg == null) || (tempdeg < -9998) || (tempdeg === "") ||
        (is_ge(valtable_obj._oemViewName) && tempdeg === -1))
        tempdeg = 'n/a';
      else
        tempdeg = get_temperature(tempdeg, valtable_obj._tempUnit, 1);
    }
    $('#rom_temp_id').html(tempdeg);

    if (ups_protocol != 33026) //JBUSDAKER
    {
      if ((ambtemp == null) || (ambtemp < -9998) || (ambtemp === ""))
        ambtemp = 'n/a';
      else {
        ambtemp = get_temperature(ambtemp, valtable_obj._tempUnit, 1);
        $('#amb_temp_id').html(ambtemp);
      }
    }

    if (comm2mode != null && comm2mode !== '') //only cs121
    {
      if ((typeof object === 'string' && comm2mode.empty()))
        return false;
      val = comm2mode.toFixed(0);
      if (val == 14) {
        val = temp2.toFixed(0) + ' rel. H';
        $('#rom_sm_h_id').html(val);
        val = temp1.toFixed(0) + ' &#176;C';
        $('#rom_sm_t_id').html(val);
      } else if (val == 11) {
        val = temp1.toFixed(0) + ' &#176;C';
        $('#rom_sm_t_id').html(val);
      }
    }

    if (valtable_obj.EX_NOM_VA != null && valtable_obj.EX_NOM_VA != 0) {
      if (ups_protocol == '32781' || ups_protocol == '32818' || ups_protocol == '32912') // Meta: ECONT, EFACEC, ECONT2
      {
        $('#nominal_power').text((valtable_obj.EX_NOM_VA).toFixed(0) + ' W');
      } else {
        if (valtable_obj.EX_NOM_VA >= 1000) {
          $('#nominal_power').text((valtable_obj.EX_NOM_VA / 1000).toFixed(0) + ' kVA');
        } else {
          if (ups_protocol == 32963 || ups_protocol == 33031 || ups_protocol == 33032)   // PREXT, PREXTXANTO, PREXTZINTO
            $('#nominal_power').text((valtable_obj.EX_NOM_VA).toFixed(0) + ' VA');
          else
            $('#nominal_power').text((valtable_obj.EX_NOM_VA).toFixed(0) + ' kVA');
        }
      }
    }

    if (ups_protocol == 32988 || ups_protocol == 32962) //MBCUBE2 and MBCUBE
    {
      $('#inverter_temp').html(get_temperature(valtable_obj.CUBE_TEMPINV, valtable_obj._tempUnit, 1));
      $('#rectifier_temp').html(get_temperature(valtable_obj.CUBE_TEMPRECT, valtable_obj._tempUnit, 1));
    }
  }

  function create_err_panel(attr_obj, css_obj) {
    var err_div = $('<div/>').attr(attr_obj).css(css_obj),
      button = $('<button/>', { 'class': 'close' }).append('&times;').appendTo(err_div),
      h = $('<strong/>').append('').appendTo(err_div),
      err_msg_div = $('<h6/>', { id: 'error_msg_div' }).append('').appendTo(err_div),
      fragment = document.createDocumentFragment();
    fragment.appendChild(err_div[0]);
    document.body.appendChild(fragment);
    button.click(function (event) {
      err_div.css('display', 'none');
      event.preventDefault();
    });
    return err_div;
  }

  //parse functions
  function parse_json(data) {
    if (typeof JSON !== 'undefined' && typeof JSON.parse === 'function')
      return JSON.parse(data);
    else
      return (new Function('return ' + data))();
  }

  function display_err(obj) {
    var message = null;
    obj = obj || {};
    /*
        if (obj.oid)
        {
          obj.response.responseText = obj.response.responseText.replace('../gxNMS.css', '../../../gxNMS.css');
          obj.response.responseText = obj.response.responseText.replace('../image/msgfail.gif', '../../../image/msgfail.gif');
        }
        */
    if (obj.oid) {
      window.setTimeout(function () { window.location.href = get_url('script/nmsView.dll?' + obj.oid); }, 10000);
      return true;
    } else
    if (obj.response.statusText == 'parsererror') {
      //document.body.innerHTML = "";
      document.open();
      document.write(obj.response.responseText);
      document.close();
      return true;
    }

    if (!obj.err_div)
      return true;
    obj.err_div.css('display', 'block');

    if (obj.response.responseText && !obj.response.responseText.empty()) {
      try {
        message = parse_json(obj.response.responseText);
        if (message.message) {
          var msg_div = $('#' + obj.err_msg_div_id);
          $('#' + obj.err_msg_div_id).html("Error(" + obj.response.status + "): " + message.message);
        } else {
          $('#' + obj.err_msg_div_id).html(obj.response.responseText);
        }
      } catch (e) {
        $('#' + obj.err_msg_div_id).html(obj.response.responseText);
      }
    } else if (obj.response.statusText && !obj.response.statusText.empty()) {
      $('#' + obj.err_msg_div_id).html(obj.response.statusText);
    } else {
      $('#' + obj.err_msg_div_id).html(obj.response.err_msg);
    }
  }

  function create_br_check_panel(obj, func) {
    var br_check_div, br_err_msg, h, a,
      fragment = document.createDocumentFragment();

    css_obj = { position: 'absolute', left: 10, width: '520px', margin: '20px auto', padding: '20px', 'z-index': 1000 };

    br_check_div = $('<div/>', { id: 'br_check_div_id', 'class': 'alert alert-warning' }).css(css_obj);

    h = $('<h4/>').append('Your Browser is no longer supported.').appendTo(br_check_div),
      br_err_msg = $('<h5/>', { id: 'error_msg_id' }).append('This application no longer supports the web browser version you are using.').appendTo(br_check_div);
    br_err_msg = $('<h5/>', { id: 'error_msg_id' }).append('Please upgrade your browser.').appendTo(br_check_div);
    a = $('<a/>', { href: 'javascript:window.location = "../sys_br_req.html"', title: 'System and Browser requirements' }).append('Learn more...').appendTo(br_err_msg);

    a = $('<a/>', { href: '#', title: 'Show Graphical Screen' }).append('Proceed to the application...').appendTo(br_check_div);
    a.click(function (event) {
      br_check_div.remove();
      if (typeof func === 'function')
        func.call(obj); //browser_check_ok = true
      event.preventDefault();
    });

    fragment.appendChild(br_check_div[0]);
    document.body.appendChild(fragment);
  }

  function toggle_visibility(element_id) {
    var elem = $('#' + element_id),
      display = 'none';

    if (elem.length > 0) {
      display = elem.css('display');
      elem.css('display', display == 'none' ? 'block' : 'none');
      localStorage.setItem(element_id + '_display', elem.css('display'));
    }
  }

  function create_autonomy_time_infos_panel(elements_id, valtable_obj) {
    var display = null,
      top = null,
      left = null,
      infos_div = $('<div/>').attr({ id: 'autonomy_time_infos', 'class': 'alert alert-info' })
    .css({
      position: 'absolute',
      width: '400px',
      display: "none",
      'z-index': 20,
      margin: '100px',
      'font-size': '10px'
    }),

    button = $('<button/>', { 'class': 'close' }).append('&times;').appendTo(infos_div),

    h4 = $('<h4/>').append(i18n('Autonomy Time Infos')).appendTo(infos_div),

    table = $('<table />', { 'class': 'table table-condensed table-hover', 'id': 'infostable' }).css('font-size', '12px').appendTo(infos_div),
    tbody = $('<tbody />').appendTo(table),

    tr = $('<tr />').append($('<td />', { align: 'left' }).css('white-space', 'nowrap').append(i18n('Autonomy Time') + ':')).appendTo(tbody),
    td = $('<td />', { id: 'autonomy_time', align: 'left' }).appendTo(tr);

    tr = $('<tr />').appendTo(tbody);
    td = $('<td />', { align: 'left', id: 'autonomy_time_src_label' }).append(i18n('Source') + ':').css('white-space', 'nowrap').appendTo(tr);
    td = $('<td />', { id: 'autonomy_time_src', align: 'left' }).css('align', 'left').appendTo(tr);

    tr = $('<tr />').append($('<td />', { align: 'left', id: 'info_calibration_label' }).append(i18n('Calibration') + ':')).appendTo(tbody);
    td = $('<td />', { id: 'info_calibration', align: 'left' }).appendTo(tr);

    fragment = document.createDocumentFragment();

    elements_id = Array.isArray(elements_id) ? elements_id : [elements_id];

    $.each(elements_id, (function (index, id) //add action to elements_id
      {
        var element = $('#' + id),
          html = element.html(),
          infos_butt_id = id + '_butt';

        if (element.length > 0) {
          element.html(element.html() + '<span>&nbsp;</span><a class="badge badge-info" id="' + infos_butt_id + '" style="background-color: #3a87ad">i</a>');
          $('#' + infos_butt_id).attr({
            title: i18n('Click here for information about the autonomy time.'),
            href: "javascript:panes_api.toggle_visibility(\'autonomy_time_infos\')"
          });
        }
      }));

    button.click(function (event) {
      toggle_visibility('autonomy_time_infos');
    });


    display = localStorage.getItem('autonomy_time_infos_display');
    top = localStorage.getItem('autonomy_time_infos_top');
    left = localStorage.getItem('autonomy_time_infos_left');
    if (display != null) {
      infos_div.css('display', display);
    }
    if (top != null && left != null) {
      infos_div.offset({
        top: top,
        left: left
      });
    }

    fragment.appendChild(infos_div[0]);
    window.document.body.appendChild(fragment);
    set_autonomy_time_infos(valtable_obj);

    new utils_api.draggable('autonomy_time_infos');

    return infos_div;
  }

  function set_autonomy_time_infos(valtable_obj) {
    if (valtable_obj.LASTTSTBUPT > 0 && valtable_obj.LASTTSTBUPLOAD > 0 &&
      !(valtable_obj.STATUS & 0x40) && valtable_obj.AUTONOMY_SRC == 3) {
      var txt = i18n("Last tested with") + " " + valtable_obj.LASTTSTBUPLOAD.toFixed(0) + "% " +
        i18n("load") + ": " + valtable_obj.LASTTSTBUPT.toFixed(0) + " " + i18n("Minutes");

      $('#autonomy_time').text(txt);
    } else {
      var val = valtable_obj.AUTONOMTIME;
      if (val == null || val == -9999)
        $('#autonomy_time').text('N/A');
      else
        $('#autonomy_time').text(valtable_obj.AUTONOMTIME.toFixed(0) + " " + i18n("Minutes"));
    }

    $('#autonomy_time_src').text(i18n(get_autonomy_time_src(valtable_obj.AUTONOMY_SRC)));
    if (valtable_obj.AUTONOMY_SRC == 0) {
      $('#autonomy_time_src').css('visibility', 'hidden');
      $('#autonomy_time_src_label').css('visibility', 'hidden');
    } else {
      $('#autonomy_time_src').css('visibility', 'visible');
      $('#autonomy_time_src_label').css('visibility', 'visible');
    }

    $('#info_calibration').text(i18n('Last tested Autonomytime is no longer valid because Load has increased. Please start a new FULLTEST to re-calibrate Autonomytime.'));
    if (valtable_obj.CALIBRATE == 0) {
      $('#info_calibration').css('visibility', 'hidden');
      $('#info_calibration_label').css('visibility', 'hidden');
    } else {
      $('#info_calibration').css('visibility', 'visible');
      $('#info_calibration_label').css('visibility', 'visible');
    }
  }

  function get_autonomy_time_src(autonomy_src) {
    switch (autonomy_src) {
      case 1: //AUTONOMY_UPS
        return "UPS";
      case 2: //AUTONOMY_CALC
        return "Calculation";
      case 3: //AUTONOMY_TEST
        return "FullTest";

      default:
        break;
    }
    return "Unknown";
  }
}(jQuery));