<?xml version="1.0"?>
<!--
   panes.xsl - XSL-templates for displaying UPS values (UNMS-II)
   Copyright (c) 2005, Generex GmbH. All rights reserved.
   heRo/klaus
   Last update: said 2010
-->

<xsl:stylesheet version="1.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform">
  <xsl:output method="html" indent="yes" omit-xml-declaration="yes" encoding="iso-8859-1"/>

  <!--  oem View Name-->
  <xsl:variable name="oem_view_name" select="//values/value[@name='_oemViewName']/@val"/>
  <!-- N/A number -->
  <xsl:variable name="na_number">
    <xsl:choose>
      <xsl:when test="string($oem_view_name)='GEML' or string($oem_view_name)='geml'
        or string($oem_view_name)='GEMATCH' or string($oem_view_name)='gematch'
        or string($oem_view_name)='VH' or string($oem_view_name)='vh'
        or string($oem_view_name)='DELTA' or string($oem_view_name)='delta'
        or string($oem_view_name)='CP4' or string($oem_view_name)='cp4'">
        <xsl:value-of select="'-1'"/>
      </xsl:when>
      <xsl:otherwise>
        <xsl:value-of select="'-9998'"/>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:variable>

  <!-- Pane Value -->
  <xsl:template name="paneValue">
    <xsl:param name="x" select="0" />
    <xsl:param name="y" select="0" />
    <xsl:param name="w" select="0" />
    <xsl:param name="h" select="0" />
    <xsl:param name="align" select="'left'" />
    <xsl:param name="value" select="0" />
    <xsl:param name="unit" select="''" />
    <xsl:param name="precision" select="-1" />
    <xsl:param name="visibility" select="'visible'" />
    <xsl:param name="name" select="'divValue'" />
    <xsl:element name="div">
      <xsl:attribute name="id">
        <xsl:value-of select="$name"/>
      </xsl:attribute>
      <xsl:attribute name="style">
        position:absolute;left:<xsl:value-of select="$x"/>px;top:<xsl:value-of select="$y"/>px;width:<xsl:value-of select="$w"/>px;height:<xsl:value-of select="$h"/>px;visibility:<xsl:value-of select="$visibility"/>; text-align:<xsl:value-of select="$align"/>;
      </xsl:attribute>

      <!-- if $precision is below zero we have to assume a string as value-->
      <xsl:choose>
        <xsl:when test="number($value) &lt;number($na_number) or number($value)= number($na_number)">
          <xsl:value-of select="'N/A'"/>
        </xsl:when>
        <xsl:when test="$precision &gt;= 0">
          <xsl:call-template name="round-decimal">
            <xsl:with-param name="value" select="$value"/>
            <xsl:with-param name="precision" select="$precision" />
          </xsl:call-template>
        </xsl:when>
        <xsl:otherwise>
          <xsl:value-of select="$value"/>
        </xsl:otherwise>
      </xsl:choose>
      <!-- Test if $unit is a non-empty string -->
      <xsl:if test="string-length($unit) !=0 and $value != -1">
        &#160;<xsl:value-of select="$unit"/>
      </xsl:if>
    </xsl:element>
  </xsl:template>

  <!-- Pane Text -->
  <xsl:template name="paneText">
    <xsl:param name="x" select="0" />
    <xsl:param name="y" select="0" />
    <xsl:param name="w" select="0" />
    <xsl:param name="h" select="0" />
    <xsl:param name="align" select="'left'" />
    <xsl:param name="value" select="0" />
    <xsl:param name="d-o-e" select="'no'" />
    <xsl:param name="unit" select="''" />
    <xsl:param name="color" select="'black'" />
    <xsl:param name="visibility" select="'visible'" />
    <xsl:param name="name" select="'divText'" />

    <xsl:element name="div">
      <xsl:attribute name="id">
        <xsl:value-of select="$name"/>
      </xsl:attribute>
      <xsl:attribute name="style">
        position:absolute;left:<xsl:value-of select="$x"/>px;top:<xsl:value-of select="$y"/>px;width:<xsl:value-of select="$w"/>px;height:<xsl:value-of select="$h"/>px;visibility:<xsl:value-of select="$visibility"/>; clip:rect(0px, <xsl:value-of select="$w"/>px, <xsl:value-of select="$h"/>px, 0px);color:<xsl:value-of select="$color"/>;text-align:<xsl:value-of select="$align"/>;
      </xsl:attribute>
      <xsl:choose>
        <xsl:when test="$d-o-e = 'yes'">
          <xsl:value-of disable-output-escaping="yes" select="$value"/>
        </xsl:when>
        <xsl:otherwise>
          <xsl:value-of disable-output-escaping="no" select="$value"/>
        </xsl:otherwise>
      </xsl:choose>
      <xsl:if test="string-length($unit) !=0">
        &#160;<xsl:value-of select="$unit"/>
      </xsl:if>
    </xsl:element>
  </xsl:template>

  <xsl:template name="paneTextColumn">
    <xsl:param name="rows" select="2" />
    <xsl:param name="x" select="0" />
    <xsl:param name="y" select="0" />
    <xsl:param name="w" select="0" />
    <xsl:param name="h" select="0" />
    <xsl:param name="yOffset" select="0" />
    <xsl:param name="align" select="'right'" />
    <xsl:param name="text1" select="''" />
    <xsl:param name="text2" select="''" />
    <xsl:param name="text3" select="''" />
    <xsl:param name="text4" select="''" />
    <xsl:param name="text5" select="''" />
    <xsl:param name="text6" select="''" />
    <xsl:param name="visibility" select="'visible'" />
    <xsl:param name="name" select="'div'" />

    <xsl:call-template name="paneColumn">
      <xsl:with-param name="rows" select="$rows"/>
      <xsl:with-param name="x" select="$x"/>
      <xsl:with-param name="y" select="$y"/>
      <xsl:with-param name="w" select="$w"/>
      <xsl:with-param name="h" select="$h"/>
      <xsl:with-param name="align" select="$align" />
      <xsl:with-param name="value1"  select="$text1" />
      <xsl:with-param name="unit1" select="''"/>
      <xsl:with-param name="precision1" select="-1"/>
      <xsl:with-param name="value2"  select="$text2" />
      <xsl:with-param name="unit2" select="''"/>
      <xsl:with-param name="precision2" select="-1"/>
      <xsl:with-param name="value3"  select="$text3" />
      <xsl:with-param name="unit3" select="''"/>
      <xsl:with-param name="precision3" select="-1"/>
      <xsl:with-param name="value4"  select="$text4" />
      <xsl:with-param name="unit4" select="''"/>
      <xsl:with-param name="precision4" select="-1"/>
      <xsl:with-param name="value5"  select="$text5" />
      <xsl:with-param name="unit5" select="''"/>
      <xsl:with-param name="precision5" select="-1"/>
      <xsl:with-param name="value6"  select="$text6" />
      <xsl:with-param name="unit6" select="''"/>
      <xsl:with-param name="precision6" select="-1"/>
      <xsl:with-param name="visibility" select="$visibility"/>
      <xsl:with-param name="name" select="$name"/>
    </xsl:call-template>

  </xsl:template>
  <!-- Pane Column -->
  <xsl:template name="paneColumn">
    <xsl:param name="rows" select="2" />
    <xsl:param name="x" select="0" />
    <xsl:param name="y" select="0" />
    <xsl:param name="w" select="0" />
    <xsl:param name="h" select="0" />
    <xsl:param name="yOffset" select="0" />
    <xsl:param name="align" select="'right'" />
    <xsl:param name="value1" select="0" />
    <xsl:param name="unit1" select="''" />
    <xsl:param name="precision1" select="-1" />
    <xsl:param name="value2" select="0" />
    <xsl:param name="unit2" select="''" />
    <xsl:param name="precision2" select="-1" />
    <xsl:param name="value3" select="0" />
    <xsl:param name="unit3" select="''" />
    <xsl:param name="precision3" select="-1" />
    <xsl:param name="value4" select="0" />
    <xsl:param name="unit4" select="''" />
    <xsl:param name="precision4" select="-1" />
    <xsl:param name="value5" select="0" />
    <xsl:param name="unit5" select="''" />
    <xsl:param name="precision5" select="-1" />
    <xsl:param name="value6" select="0" />
    <xsl:param name="unit6" select="''" />
    <xsl:param name="precision6" select="-1" />
    <xsl:param name="visibility" select="'visible'" />
    <xsl:param name="name" select="'div'" />

    <xsl:if test="$rows &gt; 0">
      <xsl:call-template name="paneValue">
        <xsl:with-param name="x" select="$x" />
        <xsl:with-param name="y" select="$y" />
        <xsl:with-param name="w" select="$w" />
        <xsl:with-param name="h" select="$h" />
        <xsl:with-param name="align" select="$align" />
        <xsl:with-param name="value" select="$value1" />
        <xsl:with-param name="unit" select="$unit1" />
        <xsl:with-param name="precision" select="$precision1" />
        <xsl:with-param name="visibility" select="$visibility" />
        <xsl:with-param name="name" select="concat($name, '1')" />
      </xsl:call-template>
    </xsl:if>
    <xsl:if test="$rows &gt; 1">
      <xsl:call-template name="paneValue">
        <xsl:with-param name="x" select="$x" />
        <xsl:with-param name="y" select="$y+$h+$yOffset" />
        <xsl:with-param name="w" select="$w" />
        <xsl:with-param name="h" select="$h" />
        <xsl:with-param name="align" select="$align" />
        <xsl:with-param name="value" select="$value2" />
        <xsl:with-param name="unit" select="$unit2" />
        <xsl:with-param name="precision" select="$precision2" />
        <xsl:with-param name="visibility" select="$visibility" />
        <xsl:with-param name="name" select="concat($name, '2')" />
      </xsl:call-template>
    </xsl:if>

    <xsl:if test="$rows &gt;2">
      <xsl:call-template name="paneValue">
        <xsl:with-param name="x" select="$x" />
        <xsl:with-param name="y" select="$y+$h+$yOffset+$h+$yOffset" />
        <xsl:with-param name="w" select="$w" />
        <xsl:with-param name="h" select="$h" />
        <xsl:with-param name="align" select="$align" />
        <xsl:with-param name="value" select="$value3" />
        <xsl:with-param name="unit" select="$unit3" />
        <xsl:with-param name="precision" select="$precision3" />
        <xsl:with-param name="visibility" select="$visibility" />
        <xsl:with-param name="name" select="concat($name, '3')" />
      </xsl:call-template>
    </xsl:if>
    <xsl:if test="$rows &gt;3">
      <xsl:call-template name="paneValue">
        <xsl:with-param name="x" select="$x" />
        <xsl:with-param name="y" select="$y+3*($h+$yOffset)" />
        <xsl:with-param name="w" select="$w" />
        <xsl:with-param name="h" select="$h" />
        <xsl:with-param name="align" select="$align" />
        <xsl:with-param name="value" select="$value4" />
        <xsl:with-param name="unit" select="$unit4" />
        <xsl:with-param name="precision" select="$precision4" />
        <xsl:with-param name="visibility" select="$visibility" />
        <xsl:with-param name="name" select="concat($name, '4')" />
      </xsl:call-template>
    </xsl:if>
    <xsl:if test="$rows &gt;4">
      <xsl:call-template name="paneValue">
        <xsl:with-param name="x" select="$x" />
        <xsl:with-param name="y" select="$y+4*($h+$yOffset)" />
        <xsl:with-param name="w" select="$w" />
        <xsl:with-param name="h" select="$h" />
        <xsl:with-param name="align" select="$align" />
        <xsl:with-param name="value" select="$value5" />
        <xsl:with-param name="unit" select="$unit5" />
        <xsl:with-param name="precision" select="$precision5" />
        <xsl:with-param name="visibility" select="$visibility" />
        <xsl:with-param name="name" select="concat($name, '5')" />
      </xsl:call-template>
    </xsl:if>
    <xsl:if test="$rows &gt;5">
      <xsl:call-template name="paneValue">
        <xsl:with-param name="x" select="$x" />
        <xsl:with-param name="y" select="$y+5*($h+$yOffset)" />
        <xsl:with-param name="w" select="$w" />
        <xsl:with-param name="h" select="$h" />
        <xsl:with-param name="align" select="$align" />
        <xsl:with-param name="value" select="$value6" />
        <xsl:with-param name="unit" select="$unit6" />
        <xsl:with-param name="precision" select="$precision6" />
        <xsl:with-param name="visibility" select="$visibility" />
        <xsl:with-param name="name" select="concat($name, '6')" />
      </xsl:call-template>
    </xsl:if>

  </xsl:template>

  <!-- Pane Row -->
  <xsl:template name="paneRow">
    <xsl:param name="xT" select="0" />
    <xsl:param name="yT" select="0" />
    <xsl:param name="wT" select="0" />
    <xsl:param name="hT" select="0" />
    <xsl:param name="alignT" select="'left'" />
    <xsl:param name="text" select="0" />
    <xsl:param name="d-o-e" select="'no'" />
    <xsl:param name="xOff" select="0" />
    <xsl:param name="w" select="0" />
    <xsl:param name="h" select="0" />
    <xsl:param name="align" select="'right'" />
    <xsl:param name="value" select="0" />
    <xsl:param name="unit" select="''" />
    <xsl:param name="precision" select="-1" />
    <xsl:param name="visibility" select="'visible'" />
    <xsl:param name="name" select="'div'" />

    <xsl:call-template name="paneText">
      <xsl:with-param name="x" select="$xT" />
      <xsl:with-param name="y" select="$yT" />
      <xsl:with-param name="w" select="$wT" />
      <xsl:with-param name="h" select="$hT" />
      <xsl:with-param name="align" select="$alignT" />
      <xsl:with-param name="value" select="concat($text,':')" />
      <xsl:with-param name="d-o-e" select="$d-o-e" />
      <xsl:with-param name="visibility" select="$visibility" />
      <xsl:with-param name="name" select="concat($name, '0')" />
    </xsl:call-template>

    <xsl:call-template name="paneValue">
      <xsl:with-param name="x" select="$xT+$wT+$xOff" />
      <xsl:with-param name="y" select="$yT" />
      <xsl:with-param name="w" select="$w" />
      <xsl:with-param name="h" select="$h" />
      <xsl:with-param name="align" select="$align" />
      <xsl:with-param name="value" select="$value" />
      <xsl:with-param name="unit" select="$unit" />
      <xsl:with-param name="precision" select="$precision" />
      <xsl:with-param name="visibility" select="$visibility" />
      <xsl:with-param name="name" select="concat($name, '1')" />
    </xsl:call-template>

  </xsl:template>


  <!-- Pane Row1 -->
  <xsl:template name="paneRow1">
    <xsl:param name="xT" select="0" />
    <xsl:param name="yT" select="0" />
    <xsl:param name="wT" select="0" />
    <xsl:param name="hT" select="0" />
    <xsl:param name="alignT" select="'left'" />
    <xsl:param name="text" select="0" />
    <xsl:param name="xOff" select="0" />
    <xsl:param name="w" select="0" />
    <xsl:param name="h" select="0" />
    <xsl:param name="align" select="'right'" />
    <xsl:param name="value" select="0" />
    <xsl:param name="unit" select="''" />
    <xsl:param name="precision" select="-1" />
    <xsl:param name="visibility" select="'visible'" />
    <xsl:param name="name" select="'div'" />

    <xsl:call-template name="paneText">
      <xsl:with-param name="x" select="$xT" />
      <xsl:with-param name="y" select="$yT" />
      <xsl:with-param name="w" select="$wT" />
      <xsl:with-param name="h" select="$hT" />
      <xsl:with-param name="align" select="$alignT" />
      <xsl:with-param name="value" select="concat($text,':')" />
      <xsl:with-param name="visibility" select="$visibility" />
      <xsl:with-param name="name" select="concat($name, '0')" />
    </xsl:call-template>

    <xsl:call-template name="paneValue">
      <xsl:with-param name="x" select="$xT+$wT+$xOff" />
      <xsl:with-param name="y" select="$yT" />
      <xsl:with-param name="w" select="$w" />
      <xsl:with-param name="h" select="$h" />
      <xsl:with-param name="align" select="$align" />
      <xsl:with-param name="value" select="$value" />
      <xsl:with-param name="unit" select="$unit" />
      <xsl:with-param name="precision" select="$precision" />
      <xsl:with-param name="visibility" select="$visibility" />
      <xsl:with-param name="name" select="concat($name, '1')" />
    </xsl:call-template>

  </xsl:template>

  <!-- Pane Row2 -->
  <xsl:template name="paneRow2">
    <xsl:param name="xT" select="0" />
    <xsl:param name="yT" select="0" />
    <xsl:param name="wT" select="0" />
    <xsl:param name="hT" select="0" />
    <xsl:param name="alignT" select="'left'" />
    <xsl:param name="text" select="0" />
    <xsl:param name="xOff" select="0" />
    <xsl:param name="xOff2" select="0" />
    <xsl:param name="w" select="0" />
    <xsl:param name="h" select="0" />
    <xsl:param name="align" select="'right'" />
    <xsl:param name="value" select="0" />
    <xsl:param name="value2" select="0" />
    <xsl:param name="unit" select="''" />
    <xsl:param name="unit2" select="''" />
    <xsl:param name="precision" select="-1" />
    <xsl:param name="precision2" select="-1" />
    <xsl:param name="visibility" select="'visible'" />
    <xsl:param name="name" select="'div'" />

    <xsl:call-template name="paneText">
      <xsl:with-param name="x" select="$xT" />
      <xsl:with-param name="y" select="$yT" />
      <xsl:with-param name="w" select="$wT" />
      <xsl:with-param name="h" select="$hT" />
      <xsl:with-param name="align" select="$alignT" />
      <xsl:with-param name="value" select="concat($text,':')" />
      <xsl:with-param name="visibility" select="$visibility" />
      <xsl:with-param name="name" select="concat($name, '0')" />
    </xsl:call-template>

    <xsl:variable name="new_unit">
      <xsl:choose>
        <xsl:when test="string-length($value) = 0 or string($value)&lt; 0">
          <xsl:value-of select="' '"/>
        </xsl:when>
        <xsl:otherwise>
          <xsl:value-of select="$unit"/>
        </xsl:otherwise>
      </xsl:choose>
    </xsl:variable>

    <xsl:call-template name="paneValue">
      <xsl:with-param name="x" select="$xT + $wT + $xOff" />
      <xsl:with-param name="y" select="$yT" />
      <xsl:with-param name="w" select="$w" />
      <xsl:with-param name="h" select="$h" />
      <xsl:with-param name="align" select="$align" />
      <xsl:with-param name="value" select="$value" />
      <xsl:with-param name="unit" select="$new_unit" />
      <xsl:with-param name="precision" select="$precision" />
      <xsl:with-param name="visibility" select="$visibility" />
      <xsl:with-param name="name" select="concat($name, '1')" />
    </xsl:call-template>

    <xsl:variable name="new_unit2">
      <xsl:choose>
        <xsl:when test="string-length($value2) = 0 or string($value2) = '-1'">
          <xsl:value-of select="' '"/>
        </xsl:when>
        <xsl:otherwise>
          <xsl:value-of select="$unit2"/>
        </xsl:otherwise>
      </xsl:choose>
    </xsl:variable>

    <xsl:call-template name="paneValue">
      <xsl:with-param name="x" select="$xT + $wT + $xOff + $w + $xOff2" />
      <xsl:with-param name="y" select="$yT" />
      <xsl:with-param name="w" select="$w" />
      <xsl:with-param name="h" select="$h" />
      <xsl:with-param name="align" select="$align" />
      <xsl:with-param name="value" select="$value2" />
      <xsl:with-param name="unit" select="$new_unit2" />
      <xsl:with-param name="precision" select="$precision2" />
      <xsl:with-param name="visibility" select="$visibility" />
      <xsl:with-param name="name" select="concat($name, '2')" />
    </xsl:call-template>

  </xsl:template>

  <!-- Pane Row3 -->
  <xsl:template name="paneRow3">
    <xsl:param name="xT" select="0" />
    <xsl:param name="yT" select="0" />
    <xsl:param name="wT" select="0" />
    <xsl:param name="hT" select="0" />
    <xsl:param name="alignT" select="'left'" />
    <xsl:param name="text" select="0" />
    <xsl:param name="xOff" select="0" />
    <xsl:param name="xOff2" select="0" />
    <xsl:param name="w" select="0" />
    <xsl:param name="h" select="0" />
    <xsl:param name="align" select="'right'" />
    <xsl:param name="value" select="0" />
    <xsl:param name="value2" select="0" />
    <xsl:param name="value3" select="0" />
    <xsl:param name="unit" select="''" />
    <xsl:param name="unit2" select="''" />
    <xsl:param name="unit3" select="''" />
    <xsl:param name="precision" select="-1" />
    <xsl:param name="precision2" select="-1" />
    <xsl:param name="precision3" select="-1" />
    <xsl:param name="visibility" select="'visible'" />
    <xsl:param name="name" select="'div'" />

    <xsl:call-template name="paneText">
      <xsl:with-param name="x" select="$xT" />
      <xsl:with-param name="y" select="$yT" />
      <xsl:with-param name="w" select="$wT" />
      <xsl:with-param name="h" select="$hT" />
      <xsl:with-param name="align" select="$alignT" />
      <xsl:with-param name="value" select="concat($text,':')" />
      <xsl:with-param name="visibility" select="$visibility" />
      <xsl:with-param name="name" select="concat($name, '0')" />
    </xsl:call-template>

    <xsl:call-template name="paneValue">
      <xsl:with-param name="x" select="$xT + $wT + $xOff" />
      <xsl:with-param name="y" select="$yT" />
      <xsl:with-param name="w" select="$w" />
      <xsl:with-param name="h" select="$h" />
      <xsl:with-param name="align" select="$align" />
      <xsl:with-param name="value" select="$value" />
      <xsl:with-param name="unit" select="$unit" />
      <xsl:with-param name="precision" select="$precision" />
      <xsl:with-param name="visibility" select="$visibility" />
      <xsl:with-param name="name" select="concat($name, '1')" />
    </xsl:call-template>

    <xsl:call-template name="paneValue">
      <xsl:with-param name="x" select="$xT + $wT + $xOff + $w + $xOff2" />
      <xsl:with-param name="y" select="$yT" />
      <xsl:with-param name="w" select="$w" />
      <xsl:with-param name="h" select="$h" />
      <xsl:with-param name="align" select="$align" />
      <xsl:with-param name="value" select="$value2" />
      <xsl:with-param name="unit" select="$unit2" />
      <xsl:with-param name="precision" select="$precision2" />
      <xsl:with-param name="visibility" select="$visibility" />
      <xsl:with-param name="name" select="concat($name, '2')" />
    </xsl:call-template>

    <xsl:call-template name="paneValue">
      <xsl:with-param name="x" select="$xT + $wT + $xOff + $xOff2 + $w + $xOff2 + $w + $xOff2" />
      <xsl:with-param name="y" select="$yT" />
      <xsl:with-param name="w" select="$w" />
      <xsl:with-param name="h" select="$h" />
      <xsl:with-param name="align" select="$align" />
      <xsl:with-param name="value" select="$value3" />
      <xsl:with-param name="unit" select="$unit3" />
      <xsl:with-param name="precision" select="$precision3" />
      <xsl:with-param name="visibility" select="$visibility" />
      <xsl:with-param name="name" select="concat($name, '3')" />
    </xsl:call-template>

  </xsl:template>
  <!-- Pane Tab3r1c -->
  <xsl:template name="paneTab3r1c">
    <xsl:param name="xT" select="0" />
    <xsl:param name="yT" select="0" />
    <xsl:param name="wT" select="0" />
    <xsl:param name="hT" select="0" />
    <xsl:param name="alignT" select="'left'" />
    <xsl:param name="text" select="0" />
    <xsl:param name="xOff" select="0" />
    <xsl:param name="w" select="0" />
    <xsl:param name="h" select="0" />
    <xsl:param name="align" select="'right'" />
    <xsl:param name="value" select="0" />
    <xsl:param name="unit" select="''" />
    <xsl:param name="precision" select="-1" />
    <xsl:param name="visibility" select="'visible'" />
    <xsl:param name="name" select="'div'" />

    <xsl:call-template name="paneRow">
      <xsl:with-param name="xT" select="$xT"/>
      <xsl:with-param name="yT" select="$yT"/>
      <xsl:with-param name="wT" select="$wT"/>
      <xsl:with-param name="hT" select="$hT"/>
      <xsl:with-param name="alignT" select="$alignT"/>
      <xsl:with-param name="text" select="concat($text,'1')" />
      <xsl:with-param name="xOff" select="$xOff"/>
      <xsl:with-param name="w" select="$w"/>
      <xsl:with-param name="h" select="$h"/>
      <xsl:with-param name="align" select="$align"/>
      <xsl:with-param name="precision" select="$precision"/>
      <xsl:with-param name="value" select="$value/@val"/>
      <xsl:with-param name="unit" select="$unit"/>
      <xsl:with-param name="visibility" select="$visibility" />
      <xsl:with-param name="name" select="concat($name, '0')" />
    </xsl:call-template>

    <xsl:call-template name="paneRow">
      <xsl:with-param name="xT" select="$xT"/>
      <xsl:with-param name="yT" select="$yT+12"/>
      <xsl:with-param name="wT" select="$wT"/>
      <xsl:with-param name="hT" select="$hT"/>
      <xsl:with-param name="alignT" select="$alignT"/>
      <xsl:with-param name="text" select="concat($text,'2')" />
      <xsl:with-param name="xOff" select="$xOff"/>
      <xsl:with-param name="w" select="$w"/>
      <xsl:with-param name="h" select="$h"/>
      <xsl:with-param name="align" select="$align"/>
      <xsl:with-param name="precision" select="$precision"/>
      <xsl:with-param name="value" select="$value/@val2"/>
      <xsl:with-param name="unit" select="$unit"/>
      <xsl:with-param name="visibility" select="$visibility" />
      <xsl:with-param name="name" select="concat($name, '1')" />
    </xsl:call-template>

    <xsl:call-template name="paneRow">
      <xsl:with-param name="xT" select="$xT"/>
      <xsl:with-param name="yT" select="$yT+12+12"/>
      <xsl:with-param name="wT" select="$wT"/>
      <xsl:with-param name="hT" select="$hT"/>
      <xsl:with-param name="alignT" select="$alignT"/>
      <xsl:with-param name="text" select="concat($text,'3')" />
      <xsl:with-param name="xOff" select="$xOff"/>
      <xsl:with-param name="w" select="$w"/>
      <xsl:with-param name="h" select="$h"/>
      <xsl:with-param name="align" select="$align"/>
      <xsl:with-param name="precision" select="$precision"/>
      <xsl:with-param name="value" select="$value/@val3"/>
      <xsl:with-param name="unit" select="$unit"/>
      <xsl:with-param name="visibility" select="$visibility" />
      <xsl:with-param name="name" select="concat($name, '2')" />
    </xsl:call-template>

  </xsl:template>

  <!-- Pane Tab3r2c -->
  <xsl:template name="paneTab3r2c">
    <xsl:param name="xT" select="0" />
    <xsl:param name="yT" select="0" />
    <xsl:param name="wT" select="0" />
    <xsl:param name="hT" select="0" />
    <xsl:param name="alignT" select="'left'" />
    <xsl:param name="text" select="0" />
    <xsl:param name="xOff" select="0" />
    <xsl:param name="xOff2" select="0" />
    <xsl:param name="w" select="0" />
    <xsl:param name="h" select="0" />
    <xsl:param name="align" select="'right'" />
    <xsl:param name="value" select="0" />
    <xsl:param name="unit" select="''" />
    <xsl:param name="precision" select="0" />
    <xsl:param name="value2" select="0" />
    <xsl:param name="unit2" select="''" />
    <xsl:param name="precision2" select="-1" />
    <xsl:param name="visibility" select="'visible'" />
    <xsl:param name="name" select="'div'" />

    <xsl:call-template name="paneRow2">
      <xsl:with-param name="xT" select="$xT"/>
      <xsl:with-param name="yT" select="$yT"/>
      <xsl:with-param name="wT" select="$wT"/>
      <xsl:with-param name="hT" select="$hT"/>
      <xsl:with-param name="alignT" select="$alignT"/>
      <xsl:with-param name="text" select="concat($text,'1')" />
      <xsl:with-param name="xOff" select="$xOff"/>
      <xsl:with-param name="xOff2" select="$xOff2"/>
      <xsl:with-param name="w" select="$w"/>
      <xsl:with-param name="h" select="$h"/>
      <xsl:with-param name="align" select="$align"/>
      <xsl:with-param name="precision" select="$precision"/>
      <xsl:with-param name="value" select="$value/@val"/>
      <xsl:with-param name="unit" select="$unit"/>
      <xsl:with-param name="precision2" select="$precision2"/>
      <xsl:with-param name="value2" select="$value2/@val"/>
      <xsl:with-param name="unit2" select="$unit2"/>
      <xsl:with-param name="visibility" select="$visibility" />
      <xsl:with-param name="name" select="concat($name, '0')" />
    </xsl:call-template>

    <xsl:call-template name="paneRow2">
      <xsl:with-param name="xT" select="$xT"/>
      <xsl:with-param name="yT" select="$yT+12"/>
      <xsl:with-param name="wT" select="$wT"/>
      <xsl:with-param name="hT" select="$hT"/>
      <xsl:with-param name="alignT" select="$alignT"/>
      <xsl:with-param name="text" select="concat($text,'2')" />
      <xsl:with-param name="xOff" select="$xOff"/>
      <xsl:with-param name="xOff2" select="$xOff2"/>
      <xsl:with-param name="w" select="$w"/>
      <xsl:with-param name="h" select="$h"/>
      <xsl:with-param name="align" select="$align"/>
      <xsl:with-param name="precision" select="$precision"/>
      <xsl:with-param name="value" select="$value/@val2"/>
      <xsl:with-param name="unit" select="$unit"/>
      <xsl:with-param name="precision2" select="$precision2"/>
      <xsl:with-param name="value2" select="$value2/@val2"/>
      <xsl:with-param name="unit2" select="$unit2"/>
      <xsl:with-param name="visibility" select="$visibility" />
      <xsl:with-param name="name" select="concat($name, '1')" />
    </xsl:call-template>

    <xsl:call-template name="paneRow2">
      <xsl:with-param name="xT" select="$xT"/>
      <xsl:with-param name="yT" select="$yT+12+12"/>
      <xsl:with-param name="wT" select="$wT"/>
      <xsl:with-param name="hT" select="$hT"/>
      <xsl:with-param name="alignT" select="$alignT"/>
      <xsl:with-param name="text" select="concat($text,'3')" />
      <xsl:with-param name="xOff" select="$xOff"/>
      <xsl:with-param name="xOff2" select="$xOff2"/>
      <xsl:with-param name="w" select="$w"/>
      <xsl:with-param name="h" select="$h"/>
      <xsl:with-param name="align" select="$align"/>
      <xsl:with-param name="precision" select="$precision"/>
      <xsl:with-param name="value" select="$value/@val3"/>
      <xsl:with-param name="unit" select="$unit"/>
      <xsl:with-param name="precision2" select="$precision2"/>
      <xsl:with-param name="value2" select="$value2/@val3"/>
      <xsl:with-param name="unit2" select="$unit2"/>
      <xsl:with-param name="visibility" select="$visibility" />
      <xsl:with-param name="name" select="concat($name, '2')" />
    </xsl:call-template>

  </xsl:template>

  <!-- Pane Tab3r2cOne -->
  <xsl:template name="paneTab3r2cOne">
    <xsl:param name="xT" select="0" />
    <xsl:param name="yT" select="0" />
    <xsl:param name="wT" select="0" />
    <xsl:param name="hT" select="0" />
    <xsl:param name="alignT" select="'left'" />
    <xsl:param name="text" select="0" />
    <xsl:param name="xOff" select="0" />
    <xsl:param name="xOff2" select="0" />
    <xsl:param name="w" select="0" />
    <xsl:param name="h" select="0" />
    <xsl:param name="align" select="'right'" />
    <xsl:param name="value" select="0" />
    <xsl:param name="unit" select="''" />
    <xsl:param name="precision" select="-1" />
    <xsl:param name="value2" select="0" />
    <xsl:param name="unit2" select="''" />
    <xsl:param name="precision2" select="-1" />
    <xsl:param name="visibility" select="'visible'" />
    <xsl:param name="scale1" select="1" />
    <xsl:param name="scale2" select="1" />

    <xsl:param name="name" select="'div'" />

    <xsl:call-template name="paneRow2">
      <xsl:with-param name="xT" select="$xT"/>
      <xsl:with-param name="yT" select="$yT"/>
      <xsl:with-param name="wT" select="$wT"/>
      <xsl:with-param name="hT" select="$hT"/>
      <xsl:with-param name="alignT" select="$alignT"/>
      <xsl:with-param name="text" select="concat($text,'1')" />
      <xsl:with-param name="xOff" select="$xOff"/>
      <xsl:with-param name="xOff2" select="$xOff2"/>
      <xsl:with-param name="w" select="$w"/>
      <xsl:with-param name="h" select="$h"/>
      <xsl:with-param name="align" select="$align"/>
      <xsl:with-param name="precision" select="$precision"/>
      <xsl:with-param name="value" select="$value/@val * $scale1"/>
      <xsl:with-param name="unit" select="$unit"/>
      <xsl:with-param name="precision2" select="$precision2"/>
      <xsl:with-param name="value2" select="$value2/@val * $scale2"/>
      <xsl:with-param name="unit2" select="$unit2"/>
      <xsl:with-param name="visibility" select="$visibility" />
      <xsl:with-param name="name" select="concat($name, '0')" />
    </xsl:call-template>

    <xsl:call-template name="paneRow2">
      <xsl:with-param name="xT" select="$xT"/>
      <xsl:with-param name="yT" select="$yT+12"/>
      <xsl:with-param name="wT" select="$wT"/>
      <xsl:with-param name="hT" select="$hT"/>
      <xsl:with-param name="alignT" select="$alignT"/>
      <xsl:with-param name="text" select="concat($text,'2')" />
      <xsl:with-param name="xOff" select="$xOff"/>
      <xsl:with-param name="xOff2" select="$xOff2"/>
      <xsl:with-param name="w" select="$w"/>
      <xsl:with-param name="h" select="$h"/>
      <xsl:with-param name="align" select="$align"/>
      <xsl:with-param name="precision" select="$precision"/>
      <xsl:with-param name="value" select="$value/@val2 *  $scale1"/>
      <xsl:with-param name="unit" select="$unit"/>
      <xsl:with-param name="precision2" select="$precision2"/>
      <xsl:with-param name="value2" select="$value2/@val2 * $scale2"/>
      <xsl:with-param name="unit2" select="$unit2"/>
      <xsl:with-param name="visibility" select="$visibility" />
      <xsl:with-param name="name" select="concat($name, '1')" />
    </xsl:call-template>

    <xsl:call-template name="paneRow2">
      <xsl:with-param name="xT" select="$xT"/>
      <xsl:with-param name="yT" select="$yT+12+12"/>
      <xsl:with-param name="wT" select="$wT"/>
      <xsl:with-param name="hT" select="$hT"/>
      <xsl:with-param name="alignT" select="$alignT"/>
      <xsl:with-param name="text" select="concat($text,'3')" />
      <xsl:with-param name="xOff" select="$xOff"/>
      <xsl:with-param name="xOff2" select="$xOff2"/>
      <xsl:with-param name="w" select="$w"/>
      <xsl:with-param name="h" select="$h"/>
      <xsl:with-param name="align" select="$align"/>
      <xsl:with-param name="precision" select="$precision"/>
      <xsl:with-param name="value" select="$value/@val3 * $scale1"/>
      <xsl:with-param name="unit" select="$unit"/>
      <xsl:with-param name="precision2" select="$precision2"/>
      <xsl:with-param name="value2" select="$value2/@val3 * $scale2"/>
      <xsl:with-param name="unit2" select="$unit2"/>
      <xsl:with-param name="visibility" select="$visibility" />
      <xsl:with-param name="name" select="concat($name, '2')" />
    </xsl:call-template>

  </xsl:template>

  <!-- Pane Tab2r2c -->
  <xsl:template name="paneTab2r2c">
    <xsl:param name="xT" select="0" />
    <xsl:param name="yT" select="0" />
    <xsl:param name="wT" select="0" />
    <xsl:param name="hT" select="0" />
    <xsl:param name="alignT" select="'left'" />
    <xsl:param name="text" select="0" />
    <xsl:param name="text2" select="0" />
    <xsl:param name="xOff" select="0" />
    <xsl:param name="w" select="0" />
    <xsl:param name="h" select="0" />
    <xsl:param name="align" select="'right'" />
    <xsl:param name="value" select="0" />
    <xsl:param name="value2" select="0" />
    <xsl:param name="unit" select="''" />
    <xsl:param name="unit2" select="''" />
    <xsl:param name="precision" select="-1" />
    <xsl:param name="visibility" select="'visible'" />
    <xsl:param name="name" select="'div'" />

    <xsl:call-template name="paneRow">
      <xsl:with-param name="xT" select="$xT"/>
      <xsl:with-param name="yT" select="$yT"/>
      <xsl:with-param name="wT" select="$wT"/>
      <xsl:with-param name="hT" select="$hT"/>
      <xsl:with-param name="alignT" select="$alignT"/>
      <xsl:with-param name="text" select="$text" />
      <xsl:with-param name="xOff" select="$xOff"/>
      <xsl:with-param name="w" select="$w"/>
      <xsl:with-param name="h" select="$h"/>
      <xsl:with-param name="align" select="$align"/>
      <xsl:with-param name="precision" select="$precision"/>
      <xsl:with-param name="value" select="$value/@val"/>
      <xsl:with-param name="unit" select="$unit"/>
      <xsl:with-param name="visibility" select="$visibility" />
      <xsl:with-param name="name" select="concat($name, '0')" />
    </xsl:call-template>

    <xsl:call-template name="paneRow">
      <xsl:with-param name="xT" select="$xT"/>
      <xsl:with-param name="yT" select="$yT+12"/>
      <xsl:with-param name="wT" select="$wT"/>
      <xsl:with-param name="hT" select="$hT"/>
      <xsl:with-param name="alignT" select="$alignT"/>
      <xsl:with-param name="text" select="$text2" />
      <xsl:with-param name="xOff" select="$xOff"/>
      <xsl:with-param name="w" select="$w"/>
      <xsl:with-param name="h" select="$h"/>
      <xsl:with-param name="align" select="$align"/>
      <xsl:with-param name="precision" select="$precision"/>
      <xsl:with-param name="value" select="$value2/@val"/>
      <xsl:with-param name="unit" select="$unit2"/>
      <xsl:with-param name="visibility" select="$visibility" />
      <xsl:with-param name="name" select="concat($name, '1')" />
    </xsl:call-template>

  </xsl:template>

  <!-- Pane Tab3r3c -->
  <xsl:template name="paneTab3r3c">
    <xsl:param name="xT" select="0" />
    <xsl:param name="yT" select="0" />
    <xsl:param name="wT" select="0" />
    <xsl:param name="hT" select="0" />
    <xsl:param name="alignT" select="'left'" />
    <xsl:param name="text" select="0" />
    <xsl:param name="xOff" select="0" />
    <xsl:param name="xOff2" select="0" />
    <xsl:param name="w" select="0" />
    <xsl:param name="h" select="0" />
    <xsl:param name="align" select="'right'" />
    <xsl:param name="value" select="0" />
    <xsl:param name="value2" select="0" />
    <xsl:param name="value3" select="0" />
    <xsl:param name="unit" select="''" />
    <xsl:param name="unit2" select="''" />
    <xsl:param name="unit3" select="''" />
    <xsl:param name="precision" select="-1" />
    <xsl:param name="precision2" select="-1" />
    <xsl:param name="precision3" select="-1" />
    <xsl:param name="visibility" select="'visible'" />
    <xsl:param name="name" select="'div'" />

    <xsl:call-template name="paneRow3">
      <xsl:with-param name="xT" select="$xT"/>
      <xsl:with-param name="yT" select="$yT"/>
      <xsl:with-param name="wT" select="$wT"/>
      <xsl:with-param name="hT" select="$hT"/>
      <xsl:with-param name="alignT" select="$alignT"/>
      <xsl:with-param name="text" select="concat($text,'1')" />
      <xsl:with-param name="xOff" select="$xOff"/>
      <xsl:with-param name="xOff2" select="$xOff2"/>
      <xsl:with-param name="w" select="$w"/>
      <xsl:with-param name="h" select="$h"/>
      <xsl:with-param name="align" select="$align"/>
      <xsl:with-param name="value" select="$value/@val"/>
      <xsl:with-param name="unit" select="$unit"/>
      <xsl:with-param name="precision" select="$precision"/>
      <xsl:with-param name="value2" select="$value2/@val"/>
      <xsl:with-param name="unit2" select="$unit2"/>
      <xsl:with-param name="precision2" select="$precision2"/>
      <xsl:with-param name="value3" select="$value3/@val"/>
      <xsl:with-param name="unit3" select="$unit3"/>
      <xsl:with-param name="precision3" select="$precision2"/>
      <xsl:with-param name="visibility" select="$visibility" />
      <xsl:with-param name="name" select="concat($name, '0')" />
    </xsl:call-template>

    <xsl:call-template name="paneRow3">
      <xsl:with-param name="xT" select="$xT"/>
      <xsl:with-param name="yT" select="$yT+12"/>
      <xsl:with-param name="wT" select="$wT"/>
      <xsl:with-param name="hT" select="$hT"/>
      <xsl:with-param name="alignT" select="$alignT"/>
      <xsl:with-param name="text" select="concat($text,'2')" />
      <xsl:with-param name="xOff" select="$xOff"/>
      <xsl:with-param name="xOff2" select="$xOff2"/>
      <xsl:with-param name="w" select="$w"/>
      <xsl:with-param name="h" select="$h"/>
      <xsl:with-param name="align" select="$align"/>
      <xsl:with-param name="value" select="$value/@val2"/>
      <xsl:with-param name="unit" select="$unit"/>
      <xsl:with-param name="precision" select="$precision"/>
      <xsl:with-param name="value2" select="$value2/@val2"/>
      <xsl:with-param name="unit2" select="$unit2"/>
      <xsl:with-param name="precision2" select="$precision2"/>
      <xsl:with-param name="value3" select="$value3/@val2"/>
      <xsl:with-param name="unit3" select="$unit3"/>
      <xsl:with-param name="precision3" select="$precision3"/>
      <xsl:with-param name="visibility" select="$visibility" />
      <xsl:with-param name="name" select="concat($name, '1')" />
    </xsl:call-template>

    <xsl:call-template name="paneRow3">
      <xsl:with-param name="xT" select="$xT"/>
      <xsl:with-param name="yT" select="$yT+12+12"/>
      <xsl:with-param name="wT" select="$wT"/>
      <xsl:with-param name="hT" select="$hT"/>
      <xsl:with-param name="alignT" select="$alignT"/>
      <xsl:with-param name="text" select="concat($text,'3')" />
      <xsl:with-param name="xOff" select="$xOff"/>
      <xsl:with-param name="xOff2" select="$xOff2"/>
      <xsl:with-param name="w" select="$w"/>
      <xsl:with-param name="h" select="$h"/>
      <xsl:with-param name="align" select="$align"/>
      <xsl:with-param name="precision" select="$precision"/>
      <xsl:with-param name="value" select="$value/@val3"/>
      <xsl:with-param name="unit" select="$unit"/>
      <xsl:with-param name="value2" select="$value2/@val3"/>
      <xsl:with-param name="unit2" select="$unit2"/>
      <xsl:with-param name="precision2" select="$precision2"/>
      <xsl:with-param name="value3" select="$value3/@val3"/>
      <xsl:with-param name="unit3" select="$unit3"/>
      <xsl:with-param name="precision3" select="$precision3"/>
      <xsl:with-param name="visibility" select="$visibility" />
      <xsl:with-param name="name" select="concat($name, '2')" />
    </xsl:call-template>

  </xsl:template>

  <!-- Pane for typical 3 Phase unit -->
  <xsl:template name="pane-standard-unit">
    <xsl:param name="x" select="0" />
    <xsl:param name="y" select="0" />
    <xsl:param name="w" select="0" />
    <xsl:param name="h" select="0" />
    <xsl:param name="title" select="''"/>
    <xsl:param name="power" select="0"/>
    <xsl:param name="freq" select="0"/>
    <xsl:param name="volt1" select="0"/>
    <xsl:param name="volt2" select="0"/>
    <xsl:param name="volt3" select="0"/>
    <xsl:param name="curr1" select="0"/>
    <xsl:param name="curr2" select="0"/>
    <xsl:param name="curr3" select="0"/>
    <xsl:param name="border" select="0"/>
    <xsl:param name="id" select="'div'"/>
    <xsl:param name="visibility" select="'visible'" />
    <xsl:param name="class" select="'small'" />

    <xsl:variable name="disp-power">
      <xsl:call-template name="get-value">
        <xsl:with-param name="value"      select="$power"/>
        <xsl:with-param name="precision"  select="0"/>
        <xsl:with-param name="unit"       select="'kVA'"/>
      </xsl:call-template>
    </xsl:variable>

    <xsl:variable name="disp-freq">
      <xsl:call-template name="get-value">
        <xsl:with-param name="value"      select="$freq"/>
        <xsl:with-param name="precision"  select="0"/>
        <xsl:with-param name="unit"       select="'Hz'"/>
      </xsl:call-template>
    </xsl:variable>
    <xsl:variable name="disp-volt1">
      <xsl:call-template name="get-value">
        <xsl:with-param name="value"      select="$volt1"/>
        <xsl:with-param name="precision"  select="0"/>
        <xsl:with-param name="unit"       select="'V'"/>
      </xsl:call-template>
    </xsl:variable>
    <xsl:variable name="disp-volt2">
      <xsl:call-template name="get-value">
        <xsl:with-param name="value"      select="$volt2"/>
        <xsl:with-param name="precision"  select="0"/>
        <xsl:with-param name="unit"       select="'V'"/>
      </xsl:call-template>
    </xsl:variable>
    <xsl:variable name="disp-volt3">
      <xsl:call-template name="get-value">
        <xsl:with-param name="value"      select="$volt3"/>
        <xsl:with-param name="precision"  select="0"/>
        <xsl:with-param name="unit"       select="'V'"/>
      </xsl:call-template>
    </xsl:variable>
    <xsl:variable name="disp-curr1">
      <xsl:call-template name="get-value">
        <xsl:with-param name="value"      select="$curr1"/>
        <xsl:with-param name="precision"  select="0"/>
        <xsl:with-param name="unit"       select="'A'"/>
      </xsl:call-template>
    </xsl:variable>
    <xsl:variable name="disp-curr2">
      <xsl:call-template name="get-value">
        <xsl:with-param name="value"      select="$curr2"/>
        <xsl:with-param name="precision"  select="0"/>
        <xsl:with-param name="unit"       select="'A'"/>
      </xsl:call-template>
    </xsl:variable>
    <xsl:variable name="disp-curr3">
      <xsl:call-template name="get-value">
        <xsl:with-param name="value"      select="$curr3"/>
        <xsl:with-param name="precision"  select="0"/>
        <xsl:with-param name="unit"       select="'A'"/>
      </xsl:call-template>
    </xsl:variable>

    <div id="{$id}" style="position:absolute; left:{$x}px; top:{$y}px; width:{$w}px; height:{$h}px: visibility:{$visibility}">
      <table cellspacing="1" cellpadding="1" border="{$border}" width="{$w}" class="{$class}" style="color:#000000">
        <tr>
          <td align="left"  class="{$class}" width="20" style="color:#000000">
            <xsl:value-of select="$title"/>:
          </td>
          <td align="right">
            <xsl:value-of select="$disp-power"/>
          </td>
          <td align="right">
            <xsl:value-of select="$disp-freq"/>
          </td>
        </tr>
        <tr>
          <td align="left">
            <xsl:value-of select="'L1:'"/>
          </td>
          <td align="right">
            <xsl:value-of select="$disp-volt1"/>
          </td>
          <td align="right">
            <xsl:value-of select="$disp-curr1"/>
          </td>
        </tr>
        <tr>
          <td align="left">
            <xsl:value-of select="'L2:'"/>
          </td>
          <td align="right">
            <xsl:value-of select="$disp-volt2"/>
          </td>
          <td align="right">
            <xsl:value-of select="$disp-curr2"/>
          </td>
        </tr>
        <tr>
          <td align="left">
            <xsl:value-of select="'L3:'"/>
          </td>
          <td align="right">
            <xsl:value-of select="$disp-volt3"/>
          </td>
          <td align="right">
            <xsl:value-of select="$disp-curr3"/>
          </td>
        </tr>
      </table>
    </div>
  </xsl:template>


  <!-- From this line to the end: Said extention 05/2008-->
  <xsl:template name="paneRow1Col3">
    <xsl:param name="x" select="0" />
    <xsl:param name="y" select="0" />
    <xsl:param name="w" select="0" />
    <xsl:param name="h" select="0" />
    <xsl:param name="title" select="''"/>
    <xsl:param name="value1" select="0"/>
    <xsl:param name="value2" select="0"/>
    <xsl:param name="value3" select="0"/>
    <xsl:param name="unit1" select="0"/>
    <xsl:param name="unit2" select="0"/>
    <xsl:param name="unit3" select="0"/>
    <xsl:param name="precision1" select="0"/>
    <xsl:param name="precision2" select="0"/>
    <xsl:param name="precision3" select="0"/>
    <xsl:param name="id" select="'div'"/>
    <xsl:param name="border" select="0"/>
    <xsl:param name="visibility" select="'visible'" />
    <xsl:param name="align" select="'left'" />
    <xsl:param name="f_weight" select="'bold'" />
    <xsl:param name="f_s" select="'9pt'" />
    <xsl:param name="class" select="'small'" />
    <xsl:variable name="var1">
      <xsl:call-template name="get-value">
        <xsl:with-param name="value"      select="$value1"/>
        <xsl:with-param name="precision"  select="$precision1"/>
        <xsl:with-param name="unit"       select="$unit1"/>
      </xsl:call-template>
    </xsl:variable>
    <xsl:variable name="var2">
      <xsl:call-template name="get-value">
        <xsl:with-param name="value"      select="$value2"/>
        <xsl:with-param name="precision"  select="$precision2"/>
        <xsl:with-param name="unit"       select="$unit2"/>
      </xsl:call-template>
    </xsl:variable>
    <xsl:variable name="var3">
      <xsl:call-template name="get-value">
        <xsl:with-param name="value"      select="$value3"/>
        <xsl:with-param name="precision"  select="$precision3"/>
        <xsl:with-param name="unit"       select="$unit3"/>
      </xsl:call-template>
    </xsl:variable>
    <div id="{$id}" style="position:absolute; left:{$x}px; top:{$y}px; width:{$w}px; height:{$h}px: visibility:{$visibility}">
      <table cellspacing="1" cellpadding="1" border="{$border}" width="{$w}" class="{$class}" style="color:#000000;font-weight:bold; font-size:9pt">
        <tr>
          <td align="{$align}"  class="{$class}" width="20" style="color:#000000;font-weight:($f_weight);font-size:9pt">
            <xsl:value-of select="$title"/>:
          </td>
          <td align="{$align}">
            <xsl:value-of select="$var1"/>
          </td>
          <xsl:if test="string($var2) != 'N/A'" >
            <td align="left">
              <xsl:value-of select="$var2"/>
            </td>
          </xsl:if>
          <xsl:if test="string($var3) != 'N/A'" >
            <td align="left">
              <xsl:value-of select="$var3"/>
            </td>
          </xsl:if>
        </tr>
      </table>
    </div>
  </xsl:template>
  <!-- Pane Row 1 -->
  <xsl:template name="paneRow1_ge">
    <xsl:param name="xT" select="0" />
    <xsl:param name="yT" select="0" />
    <xsl:param name="wT" select="0" />
    <xsl:param name="hT" select="0" />
    <xsl:param name="alignT" select="'left'" />
    <xsl:param name="text" select="0" />
    <xsl:param name="d-o-e" select="'no'" />
    <xsl:param name="xOff" select="0" />
    <xsl:param name="w" select="0" />
    <xsl:param name="h" select="0" />
    <xsl:param name="align" select="'left'" />
    <xsl:param name="value" select="0" />
    <xsl:param name="unit" select="''" />
    <xsl:param name="precision" select="-1" />
    <xsl:param name="power_fail"  select="0"/>
    <xsl:param name="visibility" select="'visible'" />
    <xsl:param name="name" select="'div'" />

    <xsl:variable name="var_decimal">
      <xsl:choose>
        <xsl:when test="$precision &lt;0  and contains($value, '.') ">
          <xsl:value-of select="substring-before($value,'.')"/>
        </xsl:when>
        <xsl:otherwise>
          <xsl:value-of select="$value"/>
        </xsl:otherwise>
      </xsl:choose>
    </xsl:variable>

    <xsl:variable name="var_v">
      <xsl:call-template name="var_value">
        <xsl:with-param name="value" select="$var_decimal"/>
        <xsl:with-param name="power_fail"  select="$power_fail"/>
      </xsl:call-template>
    </xsl:variable>
    <xsl:variable name="var_u">
      <xsl:call-template name="var_unit">
        <xsl:with-param name="value" select="$var_v"/>
        <xsl:with-param name="unit" select="$unit"/>
      </xsl:call-template>
    </xsl:variable>
    <xsl:call-template name="paneRow">
      <xsl:with-param name="xT" select="$xT"/>
      <xsl:with-param name="yT" select="$yT"/>
      <xsl:with-param name="wT" select="$wT"/>
      <xsl:with-param name="hT" select="$hT"/>
      <xsl:with-param name="xOff" select="$xOff"/>
      <xsl:with-param name="w" select="$w"/>
      <xsl:with-param name="h" select="$h"/>
      <xsl:with-param name="alignT" select="$alignT"/>
      <xsl:with-param name="text" select="$text" />
      <xsl:with-param name="d-o-e" select="$d-o-e" />
      <xsl:with-param name="value"  select="$var_v" />
      <xsl:with-param name="unit" select="$var_u"/>
      <xsl:with-param name="precision"  select="$precision"/>
      <xsl:with-param name="name" select="$name" />
    </xsl:call-template>
  </xsl:template>



  <xsl:template name="paneRow1Col3_vh">
    <xsl:param name="x" select="0" />
    <xsl:param name="y" select="0" />
    <xsl:param name="w" select="0" />
    <xsl:param name="h" select="0" />
    <xsl:param name="title" select="''"/>
    <xsl:param name="value1" select="0"/>
    <xsl:param name="value2" select="0"/>
    <xsl:param name="value3" select="0"/>
    <xsl:param name="unit1" select="0"/>
    <xsl:param name="unit2" select="0"/>
    <xsl:param name="unit3" select="0"/>
    <xsl:param name="precision1" select="0"/>
    <xsl:param name="precision2" select="0"/>
    <xsl:param name="precision3" select="0"/>
    <xsl:param name="id" select="'div'"/>
    <xsl:param name="border" select="0"/>
    <xsl:param name="align" select="'left'" />
    <xsl:param name="visibility" select="'visible'" />
    <xsl:param name="fweight" select="'bold'" />
    <xsl:param name="f_size" select="'9pt'" />
    <xsl:param  name="spaces" select="''" />
    <xsl:param name="class" select="'small'" />
    <!-- Variable1 -->
    <xsl:variable name="var_1">
      <xsl:call-template name="var_value">
        <xsl:with-param name="value" select="$value1"/>
      </xsl:call-template>
    </xsl:variable>
    <!-- Variable1 unit-->
    <xsl:variable name="var_1_unit">
      <xsl:call-template name="var_unit">
        <xsl:with-param name="value" select="$var_1"/>
        <xsl:with-param name="unit" select="$unit1"/>
        <xsl:with-param  name="spaces" select="$spaces" />
      </xsl:call-template>
    </xsl:variable>
    <!-- Variable2 -->
    <xsl:variable name="var_2">
      <xsl:call-template name="var_value">
        <xsl:with-param name="value" select="$value2"/>
      </xsl:call-template>
    </xsl:variable>
    <!-- Variable2 unit-->
    <xsl:variable name="var_2_unit">
      <xsl:call-template name="var_unit">
        <xsl:with-param name="value" select="$var_2"/>
        <xsl:with-param name="unit" select="$unit2"/>
        <xsl:with-param  name="spaces" select="$spaces" />
      </xsl:call-template>
    </xsl:variable>
    <!-- Variable3 -->
    <xsl:variable name="var_3">
      <xsl:call-template name="var_value">
        <xsl:with-param name="value" select="$value3"/>
      </xsl:call-template>
    </xsl:variable>
    <!-- Variable3 unit-->
    <xsl:variable name="var_3_unit">
      <xsl:call-template name="var_unit">
        <xsl:with-param name="value" select="$var_3"/>
        <xsl:with-param name="unit" select="$unit3"/>
        <xsl:with-param  name="spaces" select="$spaces" />
      </xsl:call-template>
    </xsl:variable>

    <xsl:call-template name="paneRow1Col3">
      <xsl:with-param name="x" select="$x" />
      <xsl:with-param name="y" select="$y" />
      <xsl:with-param name="w" select="$w" />
      <xsl:with-param name="h" select="$h" />
      <xsl:with-param name="title" select="$title"/>
      <xsl:with-param name="value1" select="$var_1"/>
      <xsl:with-param name="value2" select="$var_2"/>
      <xsl:with-param name="value3" select="$var_3"/>
      <xsl:with-param name="unit1" select="$var_1_unit"/>
      <xsl:with-param name="unit2" select="$var_2_unit"/>
      <xsl:with-param name="unit3" select="$var_3_unit"/>
      <xsl:with-param name="precision1" select="$precision1"/>
      <xsl:with-param name="precision2" select="$precision2"/>
      <xsl:with-param name="precision3" select="$precision3"/>
      <xsl:with-param  name="f_weight" select="$fweight" />
      <xsl:with-param  name="spaces" select="$spaces" />
    </xsl:call-template>

  </xsl:template>

  <xsl:template name="battery-values">
    <xsl:param name="batt-txt" select="'Battery'" />
    <xsl:param name="batt-volt" select="'0'" />
    <xsl:param name="battvolt-unit" select="'V'" />


    <xsl:param name="time-txt" select="'Time'" />
    <xsl:param name="time-val" select="'0'" />
    <xsl:param name="time-unit" select="'min'" />


    <xsl:param name="capacity-txt" select="'Capacity'" />
    <xsl:param name="capacity-val" select="'0'" />
    <xsl:param name="capacity-unit" select="'%'" />

    <xsl:param name="capacity" select="''" />

    <xsl:param name="left" select="136" />
    <xsl:param name="top" select="306" />
    <xsl:param name="w" select="300" />
    <xsl:param name="h" select="112" />
    <xsl:param name="align" select="'right'" />
    <xsl:param name="class" select="'small'" />

    <div  id="battery-values-id" style="position:absolute;left:{$left}px;top:{$top}px;width:{$w}px;height:{$h}px;text-align:{$align};direction:rtl;">
      <table  align="center" cellpadding="1"  cellspacing="1" style="border: 1px solid #405080;text-align:{$align};direction:rtl;">
        <tr>
          <td  align="{$align}" class="{$class}" style="color:black;font-family: arial; font-size: 10px;">
            <xsl:value-of select="concat($batt-volt,$battvolt-unit)"/>
          </td>
          <td align="{$align}" class="{$class}" style="color:black;font-family: arial; font-size: 10px;" >
            <xsl:value-of select="concat(':', $batt-txt)"/>
          </td>
        </tr>
        <tr >
          <td align="{$align}" class="{$class}" style="color:black;font-family: arial; font-size: 10px;" >
            <!--#f1f1f1-->
            <xsl:value-of select="concat($time-val,$time-unit)"/>
            <!---->
          </td>
          <td align="{$align}" class="{$class}" style="color:black;font-family: arial; font-size: 10px;" >
            <!--#f1f1f1-->
            <xsl:value-of select="concat(':', $time-txt)"/>
          </td>
        </tr>

        <tr >
          <td align="{$align}" class="{$class}" style="color:black;font-family: arial;font-size: 10px;" >
            <!--#f1f1f1-->
            <xsl:value-of select="concat($capacity-val, $capacity-unit)"/>
          </td>
          <td align="{$align}" class="{$class}" style="color:black;font-family: arial;font-size: 10px;" >
            <!--#f1f1f1-->
            <xsl:value-of select="concat(':', $capacity-txt)"/>
          </td>
        </tr>
      </table>
    </div>

  </xsl:template>
  <!-- Pane Rom Temp -->
  <xsl:template name="rom_temp">
    <xsl:param name="x" select="465" />
    <xsl:param name="y" select="306" />
    <xsl:param name="w" select="112" />
    <xsl:param name="align" select="'right'" />
    <xsl:param name="class" select="'small'" />
    <xsl:param name="temp" select="0" />
    <xsl:param name="temp_text" select="'Temp'" />
    <xsl:param name="temp_unit" select="'&#176;C'" />
    <xsl:param name="precision" select="0" />
    <xsl:param name="name" select="'div_romtemp'" />


    <xsl:variable name="com2_mode" select="value[@name='COM2MODE']/@val"/>
    <xsl:variable name="sm_t">
      <xsl:call-template name="round-decimal">
        <xsl:with-param name="value" select="value[@name='TEMP1']/@val"/>
        <xsl:with-param name="precision" select="$precision" />
      </xsl:call-template>
    </xsl:variable>
    <xsl:variable name="sm_h">
      <xsl:call-template name="round-decimal">
        <xsl:with-param name="value" select="value[@name='TEMP2']/@val"/>
        <xsl:with-param name="precision" select="$precision" />
      </xsl:call-template>
    </xsl:variable>
    <xsl:variable name="h">
      <xsl:choose>
        <xsl:when test="string-length($com2_mode) != 0  and '14'=$com2_mode">
          <xsl:value-of select="43"/>
        </xsl:when>
        <xsl:when test="string-length($com2_mode) != 0  and '11'=$com2_mode">
          <xsl:value-of select="28"/>
        </xsl:when>
        <xsl:otherwise>
          <xsl:value-of select="18"/>
        </xsl:otherwise>
      </xsl:choose>
    </xsl:variable>

    <xsl:element name="div">
      <xsl:attribute name="id">
        <xsl:value-of select="$name"/>
      </xsl:attribute>
      <xsl:attribute name="style">
        position:absolute;left:<xsl:value-of select="$x"/>px;top:<xsl:value-of select="$y"/>px;width:<xsl:value-of select="$w"/>px;height:<xsl:value-of select="$h"/>px;text-align:<xsl:value-of select="$align"/>;
      </xsl:attribute>
      <div style="position:absolute;left:0px;top:2px;width:{$w}px;height:{$h}px;">
        <!--<table border="0"  align="center" cellpadding="0"  cellspacing="0">-->
        <table align="center" cellpadding="1"  cellspacing="1" style="border: 1px solid #405080;text-align:{$align}">
          <tr >
            <td align="{$align}" class="{$class}" style="color:black;font-family: arial;font-size: 10px;" >
              <!--#f1f1f1-->
              <xsl:value-of select="concat($temp_text,':')"/>
            </td>
            <td align="{$align}" class="{$class}" style="color:black;font-family: arial;font-size: 10px;" >
              <!--#f1f1f1-->
              <xsl:value-of select="$temp"/>
              <xsl:if test="$temp != 'N/A'">
                <xsl:value-of select="concat('&#160;', $temp_unit)"/>
              </xsl:if>
            </td>
          </tr>
          <xsl:choose>
            <xsl:when test="string-length($com2_mode) != 0  and '14'=$com2_mode">
              <tr >
                <td align="{$align}" class="{$class}" style="color:black;font-family: arial;font-size: 10px;" >
                  <xsl:value-of select="concat($lang-SM_T, ':')"/>
                </td>
                <td align="{$align}" class="{$class}" style="color:black;font-family: arial;font-size: 10px;" >
                  <xsl:value-of select="$sm_t"/>
                  <xsl:value-of select="concat('&#160;',$temp_unit)"/>
                </td>

              </tr>
              <tr >
                <td align="{$align}" class="{$class}" style="color:black;font-family: arial;font-size: 10px;" >
                  <xsl:value-of select="concat($lang-SM_H,':')"/>
                </td>
                <td align="{$align}" class="{$class}" style="color:black;font-family: arial;font-size: 10px;" >
                  <xsl:value-of select="$sm_h"/>
                  <xsl:value-of select="concat('&#160;','%')"/>
                </td>
              </tr>
            </xsl:when>
            <xsl:when test="string-length($com2_mode) != 0  and '11'=$com2_mode">
              <tr >
                <td align="{$align}" class="{$class}" style="color:black;font-family: arial;font-size: 10px;" >
                  <xsl:value-of select="concat($lang-SM_T, ':')"/>
                </td>
                <td align="{$align}" class="{$class}" style="color:black;font-family: arial;font-size: 10px;" >
                  <xsl:value-of select="$sm_t"/>
                  <xsl:value-of select="concat('&#160;',$temp_unit)"/>
                </td>
              </tr>
            </xsl:when>
            <xsl:otherwise>
            </xsl:otherwise>
          </xsl:choose>
        </table>
      </div>
    </xsl:element>
  </xsl:template>

  <!-- Pane humidity-Temp -->
  <xsl:template name="humidity_temp">
    <xsl:param name="x" select="465" />
    <xsl:param name="y" select="306" />
    <xsl:param name="w" select="112" />
    <xsl:param name="h" select="120" />
    <xsl:param name="align" select="'right'" />
    <xsl:param name="class" select="'small'" />
    <xsl:param name="temp" select="0" />
    <xsl:param name="humid" select="0" />
    <xsl:param name="temp_text" select="'Temp'" />
    <xsl:param name="humidity_text" select="'Humidity'" />
    <xsl:param name="temp_unit" select="'&#176;C'" />
    <xsl:param name="precision" select="0" />
    <xsl:param name="name" select="'div_romtemp'" />

    <xsl:variable name="temperature">
      <xsl:choose>
        <xsl:when test="string-length($temp) != 0">
          <xsl:call-template name="round-decimal">
            <xsl:with-param name="value" select="$temp"/>
            <xsl:with-param name="precision" select="$precision" />
          </xsl:call-template>
        </xsl:when>
        <xsl:otherwise>
        </xsl:otherwise>
      </xsl:choose>
    </xsl:variable>

    <xsl:variable name="humidity">
      <xsl:choose>
        <xsl:when test="string-length($humid) != 0">
          <xsl:call-template name="round-decimal">
            <xsl:with-param name="value" select="$humid"/>
            <xsl:with-param name="precision" select="$precision" />
          </xsl:call-template>
        </xsl:when>
        <xsl:otherwise>
        </xsl:otherwise>
      </xsl:choose>
    </xsl:variable>


    <xsl:element name="div">
      <xsl:attribute name="id">
        <xsl:value-of select="$name"/>
      </xsl:attribute>
      <xsl:attribute name="style">
        position:absolute;left:<xsl:value-of select="$x"/>px;top:<xsl:value-of select="$y"/>px;width:<xsl:value-of select="$w"/>px;height:<xsl:value-of select="$h"/>px;text-align:<xsl:value-of select="$align"/>;
      </xsl:attribute>
      <xsl:if test="string-length($temperature) != 0 or string-length($humidity) != 0">
        <table align="center" cellpadding="1"  cellspacing="1" style="border: 1px solid #405080;text-align:{$align}">
          <xsl:if test="string-length($temperature) != 0 ">
            <tr align="{$align}" class="{$class}" style="color:black;font-family: arial;font-size: 10px;">
              <td>
                <xsl:value-of select="concat($temp_text,':')"/>
              </td>
              <td  >
                <xsl:value-of select="$temperature"/>
                <xsl:if test="$temp != 'N/A'">
                  <xsl:value-of select="concat('&#160;', $temp_unit)"/>
                </xsl:if>
              </td>
            </tr>
          </xsl:if>
          <xsl:if test="string-length($humidity) != 0 ">
            <tr align="{$align}" class="{$class}" style="color:black;font-family: arial;font-size: 10px;">
              <td>
                <xsl:value-of select="concat($humidity_text,':')"/>
              </td>
              <td>
                <xsl:value-of select="$humidity"/>
                <xsl:value-of select="concat('&#160;','%')"/>
              </td>
            </tr>
          </xsl:if>



        </table>

      </xsl:if>

      <!--</div>-->
    </xsl:element>


  </xsl:template>


  <!-- Pane Tab2r2c -->
  <xsl:template name="pane2r2c">
    <xsl:param name="x" select="0" />
    <xsl:param name="y" select="0" />
    <xsl:param name="w" select="0" />
    <xsl:param name="h" select="0" />
    <xsl:param name="text_1" select="''" />
    <xsl:param name="text_2" select="''" />
    <xsl:param name="value_1_1" select="0" />
    <xsl:param name="value_1_2" select="0" />
    <xsl:param name="value_2_1" select="0" />
    <xsl:param name="value_2_2" select="0" />
    <xsl:param name="unit_1_1" select="''" />
    <xsl:param name="unit_1_2" select="''" />
    <xsl:param name="unit_2_1" select="''" />
    <xsl:param name="unit_2_2" select="''" />
    <xsl:param name="precision_1_1" select="0" />
    <xsl:param name="precision_1_2" select="0" />
    <xsl:param name="precision_2_1" select="0" />
    <xsl:param name="precision_2_2" select="0" />
    <xsl:param name="visibility" select="'visible'" />
    <xsl:param name="class" select="'small'" />
    <xsl:param name="id" select="'div1'" />
    <xsl:param name="border" select="0"/>
    <xsl:param name="align" select="'left'" />

    <xsl:variable name="val_1_1">
      <xsl:call-template name="get-value">
        <xsl:with-param name="value"      select="$value_1_1"/>
        <xsl:with-param name="precision"  select="$precision_1_1"/>
        <xsl:with-param name="unit"       select="$unit_1_1"/>
      </xsl:call-template>
    </xsl:variable>


    <xsl:variable name="val_1_2">
      <xsl:call-template name="get-value">
        <xsl:with-param name="value"      select="$value_1_2"/>
        <xsl:with-param name="precision"  select="$precision_1_2"/>
        <xsl:with-param name="unit"       select="$unit_1_2"/>
      </xsl:call-template>
    </xsl:variable>

    <xsl:variable name="val_2_1">
      <xsl:call-template name="get-value">
        <xsl:with-param name="value"      select="$value_2_1"/>
        <xsl:with-param name="precision"  select="$precision_2_1"/>
        <xsl:with-param name="unit"       select="$unit_2_1"/>
      </xsl:call-template>
    </xsl:variable>

    <xsl:variable name="val_2_2">
      <xsl:call-template name="get-value">
        <xsl:with-param name="value"      select="$value_2_2"/>
        <xsl:with-param name="precision"  select="$precision_2_2"/>
        <xsl:with-param name="unit"       select="$unit_2_2"/>
      </xsl:call-template>
    </xsl:variable>

    <div id="{$id}" style="position:absolute; left:{$x}px; top:{$y}px; width:{$w}px; height:{$h}px; visibility:{$visibility}; background-color:white;">
      <table style="border: 1px solid #405080;" cellpadding="4" >
        <tr align="align" class="{$class}" style="background-color:#f1f1f1; color:black;" >
          <td>
            <xsl:value-of select="$text_1"/>:
          </td>
          <td>
            <xsl:value-of select="$val_1_1"/>
          </td>
          <td>
            <xsl:value-of select="$val_1_2"/>
          </td>
        </tr>
        <tr align="left" class="{$class}" style="background-color:#f1f1f1; color:black;" >
          <td>
            <xsl:value-of select="$text_2"/>:
          </td>
          <td>
            <xsl:value-of select="$val_2_1"/>
          </td>
          <td>
            <xsl:value-of select="$val_2_2"/>
          </td>
        </tr>
      </table>
    </div>
  </xsl:template>

  <!-- Pane Tab2r1c -->
  <xsl:template name="pane2r1c">
    <xsl:param name="x" select="0" />
    <xsl:param name="y" select="0" />
    <xsl:param name="w" select="0" />
    <xsl:param name="h" select="0" />
    <xsl:param name="text_1" select="''" />
    <xsl:param name="text_2" select="''" />
    <xsl:param name="value_1" select="0" />
    <xsl:param name="value_2" select="0" />
    <xsl:param name="unit_1" select="''" />
    <xsl:param name="unit_2" select="''" />
    <xsl:param name="precision_1" select="0" />
    <xsl:param name="precision_2" select="0" />
    <xsl:param name="visibility" select="'visible'" />
    <xsl:param name="class" select="'small'" />
    <xsl:param name="id" select="'div1'" />
    <xsl:param name="align" select="'left'" />

    <xsl:variable name="val_1">
      <xsl:call-template name="get-value">
        <xsl:with-param name="value"      select="$value_1"/>
        <xsl:with-param name="precision"  select="$precision_1"/>
        <xsl:with-param name="unit"       select="$unit_1"/>
      </xsl:call-template>
    </xsl:variable>

    <xsl:variable name="val_2">
      <xsl:call-template name="get-value">
        <xsl:with-param name="value"      select="$value_2"/>
        <xsl:with-param name="precision"  select="$precision_2"/>
        <xsl:with-param name="unit"       select="$unit_2"/>
      </xsl:call-template>
    </xsl:variable>


    <div id="{$id}" style="position:absolute; left:{$x}px; top:{$y}px; width:{$w}px; height:{$h}px; visibility:{$visibility}; background-color:white;">
      <table style="border: 1px solid #405080;" cellpadding="4" >
        <tr align="align" class="{$class}" style="background-color:#f1f1f1; color:black;" >
          <td>
            <xsl:value-of select="$text_1"/>:
          </td>
          <td>
            <xsl:value-of select="$val_1"/>&#160;&#160;&#160;&#160;&#160;&#160;
          </td>
        </tr>
        <tr align="left" class="{$class}" style="background-color:#f1f1f1; color:black;" >
          <td>
            <xsl:value-of select="$text_2"/>:
          </td>
          <td>
            <xsl:value-of select="$val_2"/>&#160;&#160;&#160;&#160;&#160;&#160;
          </td>
        </tr>
      </table>
    </div>
  </xsl:template>

  <!--Alarm Panel -->
  <xsl:template name="alarm_panel">
    <xsl:param name="upsAlarmBatteryBad" select="'0'" />
    <xsl:param name="upsAlarmOnBattery" select="'0'" />
    <xsl:param name="upsAlarmLowBattery" select="'0'" />
    <xsl:param name="upsAlarmDepletedBattery" select="'0'" />
    <xsl:param name="upsAlarmTempBad" select="'0'" />
    <xsl:param name="upsAlarmInputBad" select="'0'" />
    <xsl:param name="upsAlarmOutputBad" select="'0'" />
    <xsl:param name="upsAlarmOutputOverload" select="'0'" />
    <xsl:param name="upsAlarmOnBypass" select="'0'" />
    <xsl:param name="upsAlarmBypassBad" select="'0'" />
    <xsl:param name="upsAlarmOutputOffAsRequested" select="'0'" />
    <xsl:param name="upsAlarmUpsOffAsRequested" select="'0'" />
    <xsl:param name="upsAlarmChargerFailed" select="'0'" />
    <xsl:param name="upsAlarmUpsOutputOff" select="'0'" />
    <xsl:param name="upsAlarmUpsSystemOff" select="'0'" />
    <xsl:param name="upsAlarmFanFailure" select="'0'" />
    <xsl:param name="upsAlarmFuseFailure" select="'0'" />
    <xsl:param name="upsAlarmGeneralFault" select="'0'" />
    <xsl:param name="upsAlarmDiagnosticTestFailed" select="'0'" />
    <xsl:param name="upsAlarmCommunicationsLost" select="'0'" />
    <xsl:param name="upsAlarmAwaitingPower" select="'0'" />
    <xsl:param name="upsAlarmShutdownPending" select="'0'" />
    <xsl:param name="upsAlarmShutdownImminent" select="'0'" />
    <xsl:param name="upsAlarmTestInProgress" select="'0'" />
    <xsl:param name="upsAlarmReceptacleOff" select="'0'" />
    <xsl:param name="upsAlarmEPROMError" select="'0'" />
    <xsl:param name="upsAlarmVoltageOnOutput" select="'0'" />
    <xsl:param name="upsAlarmOvervoltageOnDCLink" select="'0'" />
    <xsl:param name="upsAlarmInputCircuitFailure" select="'0'" />
    <xsl:param name="upsAlarmChargeConverterError" select="'0'" />
    <xsl:param name="upsAlarmBypassDefect" select="'0'" />
    <xsl:param name="upsAlarmPhaseWrong" select="'0'" />
    <xsl:param name="upsAlarmHighspeedBusFailure" select="'0'" />
    <xsl:param name="upsAlarmHighspeedBusCRCFailureA" select="'0'" />
    <xsl:param name="upsAlarmConnectivityBusFailure" select="'0'" />
    <xsl:param name="upsAlarmHighspeedBusCRCFailureB" select="'0'" />
    <xsl:param name="upsAlarmHighCurrentSharing" select="'0'" />
    <xsl:param name="upsAlarmHighDCRipple" select="'0'" />

    <xsl:param name="left" select="'55'" />
    <xsl:param name="top" select="'430'" />
    <xsl:param name="width" select="'690'" />
    <xsl:param name="height" select="'30'" />
    <xsl:param name="class" select ="'XLarge_black'"/>
    <xsl:param name="border" select ="'1'"/>


    <script type="text/javascript">
      var alarm_txt = i18n("UPS Alarms") + ": ",
      alarm_count = 0,
      max_alarm_count = 4,
      alarm_txt_separator = ", ";

      function set_alarm_str(value, txt)
      {
      if(value != 1)
      return;
      if(alarm_count == max_alarm_count)
      {
      ++alarm_count;
      return;
      }
      else if(alarm_count == (max_alarm_count+1))
      return;

      if(alarm_count != 0)
      alarm_txt += alarm_txt_separator + txt;
      else
      alarm_txt += i18n(txt);
      ++alarm_count;
      }


      function get_alarm_str()
      {
      set_alarm_str('<xsl:value-of select="$upsAlarmBatteryBad"/>', '<xsl:value-of select="$BatteryBad"/>');
      set_alarm_str('<xsl:value-of select="$upsAlarmOnBattery"/>', '<xsl:value-of select="$OnBattery"/>');
      set_alarm_str('<xsl:value-of select="$upsAlarmLowBattery"/>', '<xsl:value-of select="$LowBattery"/>');
      set_alarm_str('<xsl:value-of select="$upsAlarmDepletedBattery"/>', '<xsl:value-of select="$DepletedBattery"/>');
      set_alarm_str('<xsl:value-of select="$upsAlarmTempBad"/>', '<xsl:value-of select="$TempBad"/>');
      set_alarm_str('<xsl:value-of select="$upsAlarmInputBad"/>', '<xsl:value-of select="$InputBad"/>');
      set_alarm_str('<xsl:value-of select="$upsAlarmOutputBad"/>', '<xsl:value-of select="$OutputBad"/>');
      set_alarm_str('<xsl:value-of select="$upsAlarmOutputOverload"/>', '<xsl:value-of select="$OutputOverload"/>');
      set_alarm_str('<xsl:value-of select="$upsAlarmBypassBad"/>', '<xsl:value-of select="$BypassBad"/>');
      set_alarm_str('<xsl:value-of select="$upsAlarmChargerFailed"/>', '<xsl:value-of select="$ChargerFailed"/>');
      set_alarm_str('<xsl:value-of select="$upsAlarmFanFailure"/>', '<xsl:value-of select="$FanFailure"/>');
      set_alarm_str('<xsl:value-of select="$upsAlarmFuseFailure"/>', '<xsl:value-of select="$FuseFailure"/>');
      set_alarm_str('<xsl:value-of select="$upsAlarmGeneralFault"/>', '<xsl:value-of select="$GeneralFault"/>');
      set_alarm_str('<xsl:value-of select="$upsAlarmCommunicationsLost"/>', '<xsl:value-of select="$CommunicationsLost"/>');
      set_alarm_str('<xsl:value-of select="$upsAlarmOnBypass"/>', '<xsl:value-of select="$OnBypass"/>');
      set_alarm_str('<xsl:value-of select="$upsAlarmUpsOutputOff"/>', '<xsl:value-of select="$UpsOutputOff"/>');
      set_alarm_str('<xsl:value-of select="$upsAlarmUpsSystemOff"/>', '<xsl:value-of select="$UpsSystemOff"/>');
      set_alarm_str('<xsl:value-of select="$upsAlarmDiagnosticTestFailed"/>', '<xsl:value-of select="$DiagnosticTestFailed"/>');
      set_alarm_str('<xsl:value-of select="$upsAlarmOutputOffAsRequested"/>', '<xsl:value-of select="$OutputOffAsRequested"/>');
      set_alarm_str('<xsl:value-of select="$upsAlarmUpsOffAsRequested"/>', '<xsl:value-of select="$UpsOffAsRequested"/>');
      set_alarm_str('<xsl:value-of select="$upsAlarmAwaitingPower"/>', '<xsl:value-of select="$AwaitingPower"/>');
      set_alarm_str('<xsl:value-of select="$upsAlarmShutdownPending"/>', '<xsl:value-of select="$ShutdownPending"/>');
      set_alarm_str('<xsl:value-of select="$upsAlarmShutdownImminent"/>', '<xsl:value-of select="$ShutdownImminent"/>');
      set_alarm_str('<xsl:value-of select="$upsAlarmTestInProgress"/>', '<xsl:value-of select="$TestInProgress"/>');
      set_alarm_str('<xsl:value-of select="$upsAlarmReceptacleOff"/>', '<xsl:value-of select="$ReceptacleOff"/>');
      set_alarm_str('<xsl:value-of select="$upsAlarmEPROMError"/>', '<xsl:value-of select="$EPROMError"/>');
      set_alarm_str('<xsl:value-of select="$upsAlarmVoltageOnOutput"/>', '<xsl:value-of select="$VoltageOnOutput"/>');
      set_alarm_str('<xsl:value-of select="$upsAlarmOvervoltageOnDCLink"/>', '<xsl:value-of select="$OvervoltageOnDCLink"/>');
      set_alarm_str('<xsl:value-of select="$upsAlarmInputCircuitFailure"/>', '<xsl:value-of select="$InputCircuitFailure"/>');
      set_alarm_str('<xsl:value-of select="$upsAlarmChargeConverterError"/>', '<xsl:value-of select="$ChargeConverterError"/>');
      set_alarm_str('<xsl:value-of select="$upsAlarmBypassDefect"/>', '<xsl:value-of select="$BypassDefect"/>');
      set_alarm_str('<xsl:value-of select="$upsAlarmPhaseWrong"/>', '<xsl:value-of select="$PhaseWrong"/>');
      set_alarm_str('<xsl:value-of select="$upsAlarmHighspeedBusFailure"/>', '<xsl:value-of select="$HighspeedBusFailure"/>');
      set_alarm_str('<xsl:value-of select="$upsAlarmHighspeedBusCRCFailureA"/>', '<xsl:value-of select="$HighspeedBusCRCFailureA"/>');
      set_alarm_str('<xsl:value-of select="$upsAlarmConnectivityBusFailure"/>', '<xsl:value-of select="$ConnectivityBusFailure"/>');
      set_alarm_str('<xsl:value-of select="$upsAlarmHighspeedBusCRCFailureB"/>', '<xsl:value-of select="$HighspeedBusCRCFailureB"/>');
      set_alarm_str('<xsl:value-of select="$upsAlarmHighCurrentSharing"/>', '<xsl:value-of select="$HighCurrentSharing"/>');
      set_alarm_str('<xsl:value-of select="$upsAlarmHighDCRipple"/>', '<xsl:value-of select="$HighDCRipple"/>');

      if(alarm_txt =="UPS Alarms: ")
      return " ";
      if(alarm_count == (max_alarm_count+1))
      alarm_txt += alarm_txt_separator + "...";
      return alarm_txt;
      }
    </script>

    <div id="alarm_div_3" class='{$class}' align="center" style="text-align:center; position:absolute; top:{$top}px;left:{$left}px;width:{$width}px;height:{$height}px;background-color:withe; border: 1px;color:#3341AA; font-weight:bold; font-size:9pt">
      <script type="text/javascript">
        document.getElementById('alarm_div_3').innerHTML += get_alarm_str();
      </script>
    </div>

  </xsl:template>

  <!--Available UPS'S Panel -->
  <xsl:template name="available_ups_panel">
    <xsl:param name="upsid_mask" select="'0'" />
    <xsl:param name="num_units" select="'0'" />
    <xsl:param name="is_multi" select="'0'" />
    <xsl:param name="act_ups_id" select="'0'" />
    <xsl:param name="selected_ups_id" select="'0'" />
    <xsl:param name="rawstatee" select ="'0'"/>
    <xsl:param name="rawstatef" select ="'0'"/>
    <xsl:param name="rawstateg" select ="'0'"/>
    <xsl:param name="left" select="'90'" />
    <xsl:param name="top" select="'400'" />
    <xsl:param name="class" select ="'XLarge_black'"/>
    <xsl:param name="border" select ="'1'"/>
    <xsl:param name="auxidmask" select ="'0'"/>
    <xsl:param name="upsid_mask_startup" select ="'0'"/>
    <xsl:param name="newprotocol" select ="'1'"/>

    <xsl:variable name="oid" select="//service/@oid" />

    <xsl:variable name="sys_txt">
      <xsl:choose>
        <xsl:when test="$is_multi = 1">
          <xsl:value-of select="'P'"/>
        </xsl:when>
        <xsl:otherwise>
          <xsl:value-of select="'S'"/>
        </xsl:otherwise>
      </xsl:choose>
    </xsl:variable>

    <xsl:if test="$newprotocol = '1'">

    <!--
  Background colors der buttons je nach Status
-->
    <xsl:variable name="color_sys">
      <xsl:choose>
        <xsl:when test="($rawstatef mod ($bit30 * 2)) - ($rawstatef mod $bit30)">
          <xsl:value-of select="'#FF00FF'"/>
        </xsl:when>
        <xsl:when test="($rawstatee mod ($bit30 * 2)) - ($rawstatee mod $bit30)">
          <xsl:value-of select="'#FFFF00'"/>
        </xsl:when>

        <xsl:otherwise>
          <xsl:value-of select="'#DDDDDD'"/>
        </xsl:otherwise>
      </xsl:choose>
    </xsl:variable>


    <!--
  Font size (button size): selected UPS hat groesseren button
-->

    <xsl:variable name="font_ssys">
      <xsl:choose>
        <xsl:when test="$selected_ups_id = 0">
          <xsl:value-of select="'11pt'"/>
        </xsl:when>
        <xsl:otherwise>
          <xsl:value-of select="'9pt'"/>
        </xsl:otherwise>
      </xsl:choose>
    </xsl:variable>

    <!--Available UPS ID's-->
    <xsl:variable name="ups1_available">
      <xsl:choose>
        <xsl:when test="($upsid_mask mod ($bit0 * 2)) - ($upsid_mask mod $bit0)">
          <xsl:value-of select="'1'"/>
        </xsl:when>
        <xsl:otherwise>
          <xsl:value-of select="'0'"/>
        </xsl:otherwise>
      </xsl:choose>
    </xsl:variable>
    <xsl:variable name="ups2_available">
      <xsl:choose>
        <xsl:when test="($upsid_mask mod ($bit1 * 2)) - ($upsid_mask mod $bit1)">
          <xsl:value-of select="'1'"/>
        </xsl:when>
        <xsl:otherwise>
          <xsl:value-of select="'0'"/>
        </xsl:otherwise>
      </xsl:choose>
    </xsl:variable>
    <xsl:variable name="ups3_available">
      <xsl:choose>
        <xsl:when test="($upsid_mask mod ($bit2 * 2)) - ($upsid_mask mod $bit2)">
          <xsl:value-of select="'1'"/>
        </xsl:when>
        <xsl:otherwise>
          <xsl:value-of select="'0'"/>
        </xsl:otherwise>
      </xsl:choose>
    </xsl:variable>
    <xsl:variable name="ups4_available">
      <xsl:choose>
        <xsl:when test="($upsid_mask mod ($bit3 * 2)) - ($upsid_mask mod $bit3)">
          <xsl:value-of select="'1'"/>
        </xsl:when>
        <xsl:otherwise>
          <xsl:value-of select="'0'"/>
        </xsl:otherwise>
      </xsl:choose>
    </xsl:variable>
    <xsl:variable name="ups5_available">
      <xsl:choose>
        <xsl:when test="($upsid_mask mod ($bit4 * 2)) - ($upsid_mask mod $bit4)">
          <xsl:value-of select="'1'"/>
        </xsl:when>
        <xsl:otherwise>
          <xsl:value-of select="'0'"/>
        </xsl:otherwise>
      </xsl:choose>
    </xsl:variable>
    <xsl:variable name="ups6_available">
      <xsl:choose>
        <xsl:when test="($upsid_mask mod ($bit5 * 2)) - ($upsid_mask mod $bit5)">
          <xsl:value-of select="'1'"/>
        </xsl:when>
        <xsl:otherwise>
          <xsl:value-of select="'0'"/>
        </xsl:otherwise>
      </xsl:choose>
    </xsl:variable>
    <xsl:variable name="ups7_available">
      <xsl:choose>
        <xsl:when test="($upsid_mask mod ($bit6 * 2)) - ($upsid_mask mod $bit6)">
          <xsl:value-of select="'1'"/>
        </xsl:when>
        <xsl:otherwise>
          <xsl:value-of select="'0'"/>
        </xsl:otherwise>
      </xsl:choose>
    </xsl:variable>
    <xsl:variable name="ups8_available">
      <xsl:choose>
        <xsl:when test="($upsid_mask mod ($bit7 * 2)) - ($upsid_mask mod $bit7)">
          <xsl:value-of select="'1'"/>
        </xsl:when>
        <xsl:otherwise>
          <xsl:value-of select="'0'"/>
        </xsl:otherwise>
      </xsl:choose>
    </xsl:variable>
    <xsl:variable name="ups9_available">
      <xsl:choose>
        <xsl:when test="($upsid_mask mod ($bit8 * 2)) - ($upsid_mask mod $bit8)">
          <xsl:value-of select="'1'"/>
        </xsl:when>
        <xsl:otherwise>
          <xsl:value-of select="'0'"/>
        </xsl:otherwise>
      </xsl:choose>
    </xsl:variable>
    <xsl:variable name="ups10_available">
      <xsl:choose>
        <xsl:when test="($upsid_mask mod ($bit9 * 2)) - ($upsid_mask mod $bit9)">
          <xsl:value-of select="'1'"/>
        </xsl:when>
        <xsl:otherwise>
          <xsl:value-of select="'0'"/>
        </xsl:otherwise>
      </xsl:choose>
    </xsl:variable>
    <xsl:variable name="ups11_available">
      <xsl:choose>
        <xsl:when test="($upsid_mask mod ($bit10 * 2)) - ($upsid_mask mod $bit10)">
          <xsl:value-of select="'1'"/>
        </xsl:when>
        <xsl:otherwise>
          <xsl:value-of select="'0'"/>
        </xsl:otherwise>
      </xsl:choose>
    </xsl:variable>
    <xsl:variable name="ups12_available">
      <xsl:choose>
        <xsl:when test="($upsid_mask mod ($bit11 * 2)) - ($upsid_mask mod $bit11)">
          <xsl:value-of select="'1'"/>
        </xsl:when>
        <xsl:otherwise>
          <xsl:value-of select="'0'"/>
        </xsl:otherwise>
      </xsl:choose>
    </xsl:variable>
    <xsl:variable name="ups13_available">
      <xsl:choose>
        <xsl:when test="($upsid_mask mod ($bit12 * 2)) - ($upsid_mask mod $bit12)">
          <xsl:value-of select="'1'"/>
        </xsl:when>
        <xsl:otherwise>
          <xsl:value-of select="'0'"/>
        </xsl:otherwise>
      </xsl:choose>
    </xsl:variable>
    <xsl:variable name="ups14_available">
      <xsl:choose>
        <xsl:when test="($upsid_mask mod ($bit13 * 2)) - ($upsid_mask mod $bit13)">
          <xsl:value-of select="'1'"/>
        </xsl:when>
        <xsl:otherwise>
          <xsl:value-of select="'0'"/>
        </xsl:otherwise>
      </xsl:choose>
    </xsl:variable>
    <xsl:variable name="ups15_available">
      <xsl:choose>
        <xsl:when test="($upsid_mask mod ($bit14 * 2)) - ($upsid_mask mod $bit14)">
          <xsl:value-of select="'1'"/>
        </xsl:when>
        <xsl:otherwise>
          <xsl:value-of select="'0'"/>
        </xsl:otherwise>
      </xsl:choose>
    </xsl:variable>
    <xsl:variable name="ups16_available">
      <xsl:choose>
        <xsl:when test="($upsid_mask mod ($bit15 * 2)) - ($upsid_mask mod $bit15)">
          <xsl:value-of select="'1'"/>
        </xsl:when>
        <xsl:otherwise>
          <xsl:value-of select="'0'"/>
        </xsl:otherwise>
      </xsl:choose>
    </xsl:variable>
    <xsl:variable name="ups17_available">
      <xsl:choose>
        <xsl:when test="($upsid_mask mod ($bit16 * 2)) - ($upsid_mask mod $bit16)">
          <xsl:value-of select="'1'"/>
        </xsl:when>
        <xsl:otherwise>
          <xsl:value-of select="'0'"/>
        </xsl:otherwise>
      </xsl:choose>
    </xsl:variable>
    <xsl:variable name="ups18_available">
      <xsl:choose>
        <xsl:when test="($upsid_mask mod ($bit17 * 2)) - ($upsid_mask mod $bit17)">
          <xsl:value-of select="'1'"/>
        </xsl:when>
        <xsl:otherwise>
          <xsl:value-of select="'0'"/>
        </xsl:otherwise>
      </xsl:choose>
    </xsl:variable>
    <xsl:variable name="ups19_available">
      <xsl:choose>
        <xsl:when test="($upsid_mask mod ($bit18 * 2)) - ($upsid_mask mod $bit18)">
          <xsl:value-of select="'1'"/>
        </xsl:when>
        <xsl:otherwise>
          <xsl:value-of select="'0'"/>
        </xsl:otherwise>
      </xsl:choose>
    </xsl:variable>
    <xsl:variable name="ups20_available">
      <xsl:choose>
        <xsl:when test="($upsid_mask mod ($bit19 * 2)) - ($upsid_mask mod $bit19)">
          <xsl:value-of select="'1'"/>
        </xsl:when>
        <xsl:otherwise>
          <xsl:value-of select="'0'"/>
        </xsl:otherwise>
      </xsl:choose>
    </xsl:variable>
    <xsl:variable name="ups21_available">
      <xsl:choose>
        <xsl:when test="($upsid_mask mod ($bit20 * 2)) - ($upsid_mask mod $bit20)">
          <xsl:value-of select="'1'"/>
        </xsl:when>
        <xsl:otherwise>
          <xsl:value-of select="'0'"/>
        </xsl:otherwise>
      </xsl:choose>
    </xsl:variable>
    <xsl:variable name="ups22_available">
      <xsl:choose>
        <xsl:when test="($upsid_mask mod ($bit21 * 2)) - ($upsid_mask mod $bit21)">
          <xsl:value-of select="'1'"/>
        </xsl:when>
        <xsl:otherwise>
          <xsl:value-of select="'0'"/>
        </xsl:otherwise>
      </xsl:choose>
    </xsl:variable>
    <xsl:variable name="ups23_available">
      <xsl:choose>
        <xsl:when test="($upsid_mask mod ($bit22 * 2)) - ($upsid_mask mod $bit22)">
          <xsl:value-of select="'1'"/>
        </xsl:when>
        <xsl:otherwise>
          <xsl:value-of select="'0'"/>
        </xsl:otherwise>
      </xsl:choose>
    </xsl:variable>
    <xsl:variable name="ups24_available">
      <xsl:choose>
        <xsl:when test="($upsid_mask mod ($bit23 * 2)) - ($upsid_mask mod $bit23)">
          <xsl:value-of select="'1'"/>
        </xsl:when>
        <xsl:otherwise>
          <xsl:value-of select="'0'"/>
        </xsl:otherwise>
      </xsl:choose>
    </xsl:variable>
    <xsl:variable name="ups25_available">
      <xsl:choose>
        <xsl:when test="($upsid_mask mod ($bit24 * 2)) - ($upsid_mask mod $bit24)">
          <xsl:value-of select="'1'"/>
        </xsl:when>
        <xsl:otherwise>
          <xsl:value-of select="'0'"/>
        </xsl:otherwise>
      </xsl:choose>
    </xsl:variable>

    <xsl:variable name="ups26_available">
      <xsl:choose>
        <xsl:when test="($upsid_mask mod ($bit25 * 2)) - ($upsid_mask mod $bit25)">
          <xsl:value-of select="'1'"/>
        </xsl:when>
        <xsl:otherwise>
          <xsl:value-of select="'0'"/>
        </xsl:otherwise>
      </xsl:choose>
    </xsl:variable>

    <xsl:variable name="ups27_available">
      <xsl:choose>
        <xsl:when test="($upsid_mask mod ($bit26 * 2)) - ($upsid_mask mod $bit26)">
          <xsl:value-of select="'1'"/>
        </xsl:when>
        <xsl:otherwise>
          <xsl:value-of select="'0'"/>
        </xsl:otherwise>
      </xsl:choose>
    </xsl:variable>

    <xsl:variable name="ups28_available">
      <xsl:choose>
        <xsl:when test="($upsid_mask mod ($bit27 * 2)) - ($upsid_mask mod $bit27)">
          <xsl:value-of select="'1'"/>
        </xsl:when>
        <xsl:otherwise>
          <xsl:value-of select="'0'"/>
        </xsl:otherwise>
      </xsl:choose>
    </xsl:variable>

    <xsl:variable name="ups29_available">
      <xsl:choose>
        <xsl:when test="($upsid_mask mod ($bit28 * 2)) - ($upsid_mask mod $bit28)">
          <xsl:value-of select="'1'"/>
        </xsl:when>
        <xsl:otherwise>
          <xsl:value-of select="'0'"/>
        </xsl:otherwise>
      </xsl:choose>
    </xsl:variable>

    <xsl:variable name="ups30_available">
      <xsl:choose>
        <xsl:when test="($upsid_mask mod ($bit29 * 2)) - ($upsid_mask mod $bit29)">
          <xsl:value-of select="'1'"/>
        </xsl:when>
        <xsl:otherwise>
          <xsl:value-of select="'0'"/>
        </xsl:otherwise>
      </xsl:choose>
    </xsl:variable>


    <div id="system_div" class='{$class}' align="center" style="text-align:left; position:absolute; top:{$top -25}px;left:{$left + 5}px;width:160px;height:10px; border: 1px;color:#3341AA; font-weight:bold; font-size:9pt">
      <span id="system_txt">
      </span>
      <a id="a_ups_sys" href="javascript:conce_api.switch_to_ups('0')" class="btn" style="font-size:{$font_ssys}">
        <span>
          <span style="background:{$color_sys}">
            <xsl:value-of select="$sys_txt"/>
          </span>
        </span>
      </a>
    </div>
    <script type="text/javascript"> translate_text('system_txt', ['System', ':&#160;&#160;&#160;']);</script>


    <xsl:if test="$num_units &gt; 0">
      <div id="available_ups_div" class='{$class}' align="center" style="text-align:left; position:absolute; top:{$top}px;left:{$left + 5}px;width:160px;height:10px; border: 1px;color:#3341AA; font-weight:bold; font-size:9pt">
        <br></br>
        <span id="available_ups_txt"></span>
      </div>
      <script type="text/javascript"> translate_text('available_ups_txt', ['Available UPS', ':']);</script>

      <div class='{$class}' align="center" style="text-align:center; position:absolute; top:{$top +20}px;left:{$left + 2}px;width:20px;height:20px;">
        <br></br>
        <table border="0" cellpadding="4">
          <tr>
            <xsl:if test="$ups1_available = '1'">
              <td>
                <a id="a_ups_1" href="javascript:conce_api.switch_to_ups(conce_api.get_ups_id(0))" class="btn" style="font-size:9pt;">
                  <span>
                    <span id="s_ups_1">
                      <xsl:value-of select="1"/>
                    </span>
                  </span>
                </a>
              </td>
            </xsl:if>
            <xsl:if test="$ups2_available = '1'">
              <td>
                <a id="a_ups_2" href="javascript:conce_api.switch_to_ups(conce_api.get_ups_id(1))" class="btn" style="font-size:9pt">
                  <span>
                    <span id="s_ups_2">
                      <xsl:value-of select="2"/>
                    </span>
                  </span>
                </a>
              </td>
            </xsl:if>
            <xsl:if test="$ups3_available = '1'">
              <td>
                <a id="a_ups_3" href="javascript:conce_api.switch_to_ups(conce_api.get_ups_id(2))" class="btn" style="font-size:9pt">
                  <span>
                    <span id="s_ups_3">
                      <xsl:value-of select="3"/>
                    </span>
                  </span>
                </a>
              </td>
            </xsl:if>
            <xsl:if test="$ups4_available = '1'">
              <td>
                <a id="a_ups_4" href="javascript:conce_api.switch_to_ups(conce_api.get_ups_id(3))" class="btn" style="font-size:9pt">
                  <span>
                    <span id="s_ups_4">
                      <xsl:value-of select="4"/>
                    </span>
                  </span>
                </a>
              </td>
            </xsl:if>
            <xsl:if test="$ups5_available = '1'">
              <td>
                <a id="a_ups_5" href="javascript:conce_api.switch_to_ups(conce_api.get_ups_id(4))" class="btn" style="font-size:9pt">
                  <span>
                    <span id="s_ups_5">
                      <xsl:value-of select="5"/>
                    </span>
                  </span>
                </a>
              </td>
            </xsl:if>
            <xsl:if test="$ups6_available = '1'">
              <td>
                <a id="a_ups_6" href="javascript:conce_api.switch_to_ups(conce_api.get_ups_id(5))" class="btn" style="font-size:9pt">
                  <span>
                    <span id="s_ups_6">
                      <xsl:value-of select="6"/>
                    </span>
                  </span>
                </a>
              </td>
            </xsl:if>
            <xsl:if test="$ups7_available = '1'">
              <td>
                <a id="a_ups_7" href="javascript:conce_api.switch_to_ups(conce_api.get_ups_id(6))" class="btn" style="font-size:9pt">
                  <span>
                    <span id="s_ups_7">
                      <xsl:value-of select="7"/>
                    </span>
                  </span>
                </a>
              </td>
            </xsl:if>
            <xsl:if test="$ups8_available = '1'">
              <td>
                <a id="a_ups_8" href="javascript:conce_api.switch_to_ups(conce_api.get_ups_id(7))" class="btn" style="font-size:9pt">
                  <span>
                    <span id="s_ups_8">
                      <xsl:value-of select="8"/>
                    </span>
                  </span>
                </a>
              </td>
            </xsl:if>
            <xsl:if test="$ups9_available = '1'">
              <td>
                <a id="a_ups_9" href="javascript:conce_api.switch_to_ups(conce_api.get_ups_id(8))" class="btn" style="font-size:9pt">
                  <span>
                    <span id="s_ups_9">
                      <xsl:value-of select="9"/>
                    </span>
                  </span>
                </a>
              </td>
            </xsl:if>
            <xsl:if test="$ups10_available = '1'">
              <td>
                <a id="a_ups_10" href="javascript:conce_api.switch_to_ups(conce_api.get_ups_id(9))" class="btn" style="font-size:9pt">
                  <span>
                    <span id="s_ups_10">
                      <xsl:value-of select="10"/>
                    </span>
                  </span>
                </a>
              </td>
            </xsl:if>
            <xsl:if test="$ups11_available = '1'">
              <td>
                <a id="a_ups_11" href="javascript:conce_api.switch_to_ups(conce_api.get_ups_id(10))" class="btn" style="font-size:9pt">
                  <span>
                    <span id="s_ups_11">
                      <xsl:value-of select="11"/>
                    </span>
                  </span>
                </a>
              </td>
            </xsl:if>
            <xsl:if test="$ups12_available = '1'">
              <td>
                <a id="a_ups_12" href="javascript:conce_api.switch_to_ups(conce_api.get_ups_id(11))" class="btn" style="font-size:9pt">
                  <span>
                    <span id="s_ups_12">
                      <xsl:value-of select="12"/>
                    </span>
                  </span>
                </a>
              </td>
            </xsl:if>
            <xsl:if test="$ups13_available = '1'">
              <td>
                <a id="a_ups_13" href="javascript:conce_api.switch_to_ups(conce_api.get_ups_id(12))" class="btn" style="font-size:9pt">
                  <span>
                    <span id="s_ups_13">
                      <xsl:value-of select="13"/>
                    </span>
                  </span>
                </a>
              </td>
            </xsl:if>
            <xsl:if test="$ups14_available = '1'">
              <td>
                <a id="a_ups_14" href="javascript:conce_api.switch_to_ups(conce_api.get_ups_id(13))" class="btn" style="font-size:9pt">
                  <span>
                    <span id="s_ups_14">
                      <xsl:value-of select="14"/>
                    </span>
                  </span>
                </a>
              </td>
            </xsl:if>
            <xsl:if test="$ups15_available = '1'">
              <td>
                <a id="a_ups_15" href="javascript:conce_api.switch_to_ups(conce_api.get_ups_id(14))" class="btn" style="font-size:9pt">
                  <span>
                    <span id="s_ups_15">
                      <xsl:value-of select="15"/>
                    </span>
                  </span>
                </a>
              </td>
            </xsl:if>
          </tr>

          <tr>
            <xsl:if test="$ups16_available = '1'">
              <td>
                <a id="a_ups_16" href="javascript:conce_api.switch_to_ups(conce_api.get_ups_id(15))" class="btn" style="font-size:9pt">
                  <span>
                    <span id="s_ups_16">
                      <xsl:value-of select="16"/>
                    </span>
                  </span>
                </a>
              </td>
            </xsl:if>
            <xsl:if test="$ups17_available = '1'">
              <td>
                <a id="a_ups_17" href="javascript:conce_api.switch_to_ups(conce_api.get_ups_id(16))" class="btn" style="font-size:9pt">
                  <span>
                    <span id="s_ups_17">
                      <xsl:value-of select="17"/>
                    </span>
                  </span>
                </a>
              </td>
            </xsl:if>
            <xsl:if test="$ups18_available = '1'">
              <td>
                <a id="a_ups_18" href="javascript:conce_api.switch_to_ups(conce_api.get_ups_id(17))" class="btn" style="font-size:9pt">
                  <span>
                    <span id="s_ups_18">
                      <xsl:value-of select="18"/>
                    </span>
                  </span>
                </a>
              </td>
            </xsl:if>
            <xsl:if test="$ups19_available = '1'">
              <td>
                <a id="a_ups_19" href="javascript:conce_api.switch_to_ups(conce_api.get_ups_id(18))" class="btn" style="font-size:9pt">
                  <span>
                    <span id="s_ups_19">
                      <xsl:value-of select="19"/>
                    </span>
                  </span>
                </a>
              </td>
            </xsl:if>
            <xsl:if test="$ups20_available = '1'">
              <td>
                <a id="a_ups_20" href="javascript:conce_api.switch_to_ups(conce_api.get_ups_id(19))" class="btn" style="font-size:9pt">
                  <span>
                    <span id="s_ups_20">
                      <xsl:value-of select="20"/>
                    </span>
                  </span>
                </a>
              </td>
            </xsl:if>
            <xsl:if test="$ups21_available = '1'">
              <td>
                <a id="a_ups_21" href="javascript:conce_api.switch_to_ups(conce_api.get_ups_id(20))" class="btn" style="font-size:9pt">
                  <span>
                    <span id="s_ups_21">
                      <xsl:value-of select="21"/>
                    </span>
                  </span>
                </a>
              </td>
            </xsl:if>
            <xsl:if test="$ups22_available = '1'">
              <td>
                <a id="a_ups_22" href="javascript:conce_api.switch_to_ups(conce_api.get_ups_id(21))" class="btn" style="font-size:9pt">
                  <span>
                    <span id="s_ups_22">
                      <xsl:value-of select="22"/>
                    </span>
                  </span>
                </a>
              </td>
            </xsl:if>
            <xsl:if test="$ups23_available = '1'">
              <td>
                <a id="a_ups_23" href="javascript:conce_api.switch_to_ups(conce_api.get_ups_id(22))" class="btn" style="font-size:9pt">
                  <span>
                    <span id="s_ups_23">
                      <xsl:value-of select="23"/>
                    </span>
                  </span>
                </a>
              </td>
            </xsl:if>
            <xsl:if test="$ups24_available = '1'">
              <td>
                <a id="a_ups_24" href="javascript:conce_api.switch_to_ups(conce_api.get_ups_id(23))" class="btn" style="font-size:9pt">
                  <span>
                    <span id="s_ups_24">
                      <xsl:value-of select="24"/>
                    </span>
                  </span>
                </a>
              </td>
            </xsl:if>
            <xsl:if test="$ups25_available = '1'">
              <td>
                <a id="a_ups_25" href="javascript:conce_api.switch_to_ups(conce_api.get_ups_id(24))" class="btn" style="font-size:9pt">
                  <span>
                    <span id="s_ups_25">
                      <xsl:value-of select="25"/>
                    </span>
                  </span>
                </a>
              </td>
            </xsl:if>

            <xsl:if test="$ups26_available = '1'">
              <td>
                <a id="a_ups_26" href="javascript:conce_api.switch_to_ups(conce_api.get_ups_id(25))" class="btn" style="font-size:9pt">
                  <span>
                    <span id="s_ups_26">
                      <xsl:value-of select="26"/>
                    </span>
                  </span>
                </a>
              </td>
            </xsl:if>

            <xsl:if test="$ups27_available = '1'">
              <td>
                <a id="a_ups_27" href="javascript:conce_api.switch_to_ups(conce_api.get_ups_id(26))" class="btn" style="font-size:9pt">
                  <span>
                    <span id="s_ups_27">
                      <xsl:value-of select="27"/>
                    </span>
                  </span>
                </a>
              </td>
            </xsl:if>

            <xsl:if test="$ups28_available = '1'">
              <td>
                <a id="a_ups_28" href="javascript:conce_api.switch_to_ups(conce_api.get_ups_id(27))" class="btn" style="font-size:9pt">
                  <span>
                    <span id="s_ups_28">
                      <xsl:value-of select="28"/>
                    </span>
                  </span>
                </a>
              </td>
            </xsl:if>

            <xsl:if test="$ups29_available = '1'">
              <td>
                <a id="a_ups_29" href="javascript:conce_api.switch_to_ups(conce_api.get_ups_id(28))" class="btn" style="font-size:9pt">
                  <span>
                    <span id="s_ups_29">
                      <xsl:value-of select="29"/>
                    </span>
                  </span>
                </a>
              </td>
            </xsl:if>

            <xsl:if test="$ups30_available = '1'">
              <td>
                <a id="a_ups_30" href="javascript:conce_api.switch_to_ups(conce_api.get_ups_id(29))" class="btn" style="font-size:9pt">
                  <span>
                    <span id="s_ups_30">
                      <xsl:value-of select="30"/>
                    </span>
                  </span>
                </a>
              </td>
            </xsl:if>

          </tr>
        </table>
      </div>
    
      <script type="text/javascript"> conce_api.disable_module_button('<xsl:value-of select="//service/@type"/>');</script>
    
    
    
    </xsl:if>

    <script type="text/javascript">
      conce_api.set_ups_id('<xsl:value-of select="$num_units"/>', '<xsl:value-of select="$upsid_mask_startup"/>');
      conce_api.set_selected_ups_id('<xsl:value-of select="$selected_ups_id"/>');
      conce_api.set_ups_background_color('<xsl:value-of select="$upsid_mask"/>','<xsl:value-of select="$rawstatee"/>', '<xsl:value-of select="$rawstatef"/>');
      conce_api.set_line1_source('<xsl:value-of select="$selected_ups_id"/>', '<xsl:value-of select="$auxidmask"/>');
      conce_api.set_auxinp_font('<xsl:value-of select="$auxidmask"/>');
    </script>
    </xsl:if>
  </xsl:template>

  <xsl:template name="ups_remote_controle">
    <xsl:param name="top" select="'70'"/>
    <xsl:param name="left" select="'40'"/>
    <xsl:param name="w" select="'480'"/>
    <xsl:param name="h" select="'80'"/>
    <xsl:param name="border" select="'1'"/>
    <xsl:param name="class" select="'XLarge'"/>
    <xsl:param name="title" select="'UPS&#160;Remote&#160;Control'"/>
    <xsl:param name="id" select="'u_r_c'"/>

    <script type="text/javascript">
      <xsl:comment>
        <![CDATA[

          outlet_settings_win = null;

          function ups_remote_ctl(titel,w,h,args)
          {
             x=screen.availWidth/2-w/2;
             y=screen.availHeight/2-h/2;
             index = location.href.indexOf("?");
            if (index != -1)
            {
              href =location.href.substring(0, index+1 );
              href = href.concat(args);
              var size = 'width='+w+',height='+h;
              var pos = 'left='+x+',top='+y+',screenX='+x+',screenY='+y;
              location.replace(href,'titel',size+pos);
            }
          }
          // Popup window code
          function newPopup(url,titel,arg)
          {
            popupWindow = window.open(url,'title',arg);
            popupWindow.focus();
          }
          function outlet_settings(titel,w,h,args)
          {
            var x=screen.availWidth/2-w/2;
            var y=screen.availHeight/2-h/2;
            var index = location.href.indexOf("?");
            if(index != -1)
            {
              href =location.href.substring(0, index+1 );
              href = href.concat(args);
              if(outlet_settings_win!=null && !outlet_settings_win.closed)
                outlet_settings_win.close();
              var size = 'width='+w+',height='+h;
              var pos = 'left='+x+',top='+y+',screenX='+x+',screenY='+y;
              newPopup(href,titel, size+pos);
              //outlet_settings_win = window.open(href,'titel',size+" "+pos);
              //alert("outlet_settings:");
              //outlet_settings_win.focus();
            }
          }

          function err_funct(response)
          {

          }

          function success_funct(data)
          {

          }

         function buzzer_cmd() //create switch off on ups panel
         {
            var url = "",ups_cmd_option,
              href = window.location.href,
              index = href.indexOf("?");

            if(index != -1)
            {
              url = href.substring(0, index) ;
            }

            ups_cmd_option = {
              data: { type: 'ups_cmd', cmd: 'UPS_BUZZER', p1:'OFF' },
              data_type: 'json',
              success: success_funct, //Callback on success
              error: err_funct //Callback on error
            };
            jqmodule.ajax(url, ups_cmd_option);
         }

        ]]>
      </xsl:comment>
    </script>

    <div id="{$id}" style="position:absolute; top:{$top}px;left:{$left}px;width:{$w}px;height:{$h}px;border: {$border}px solid black;">
      <div id="div_1" class ="{$class}" style="position:absolute; top:8px;left:155px;width:80px;height:40px;text-align:center;">
        <xsl:value-of select="$title"/>
      </div>
      <div id="div_2" class ="{$class}" style="position:absolute; top:40px;left:50px;width:80px;height:40px;text-align:left;">
        <a href="javascript:ups_remote_ctl('Extended&#160;Commands',570,520,'restart_sys')" class="btn">
          <span>
            <span>
              <xsl:value-of select="'Restart&#160;System'"/>
            </span>
          </span>
        </a>
      </div>
      <div id="div_3" class ="Medium" style="position:absolute; top:40px;left:310px;width:80px;height:40px;text-align:left;">
        <a href="javascript:ups_remote_ctl('Extended&#160;Commands',570,520,'shutdown_sys')" class="btn">
          <span>
            <span>
              <xsl:value-of select="'Switch&#160;Off'"/>
            </span>
          </span>
        </a>
      </div>

      <div id="div_4" class ="Medium" style="position:absolute; top:68px;left:10px;width:80px;height:40px;text-align:left;">
        <xsl:value-of select="'Restart&#160;will&#160;switch&#160;off&#160;UPS&#160;load&#160;off/on '"/>
      </div>

      <div id="div_5" class ="Medium" style="position:absolute; top:68px;left:255px;width:80px;height:40px;text-align:left;">
        <xsl:value-of select="'Switch&#160;will&#160;switch&#160;off&#160;UPS&#160;(no&#160;restart)'"/>
      </div>

      <div id="div_6" class ="Medium" style="position:absolute; top:110px;left:50px;width:80px;height:40px;text-align:left;">
        <a href="javascript:outlet_settings('Programmable&#160;Outlet&#160;Settings','530','580','outlet_settings')" class="btn">
          <span>
            <span>
              <xsl:value-of select="'Programmable&#160;Outlet&#160;Settings'"/>
            </span>
          </span>
        </a>
      </div>

      <div id="div_7" class ="Medium" style="position:absolute; top:138px;left:50px;width:80px;height:40px;text-align:left;">
        <xsl:value-of select="'Configure&#160;the&#160;programmable&#160;outlets'"/>
      </div>

      <div id="div_8" class ="Medium" style="position:absolute; top:110px;left:310px;width:80px;height:40px;text-align:left;">
        <a href=" javascript:buzzer_cmd();" class="btn">
          <span>
            <span>
              <xsl:value-of select="'Toggle&#160;Buzzer'"/>
            </span>
          </span>
        </a>
      </div>

      <div id="div_9" class ="Medium" style="position:absolute; top:138px;left:320px;width:80px;height:40px;text-align:left;">
        <xsl:value-of select="'Toggle&#160;Buzzer'"/>
      </div>

    </div>

  </xsl:template>

  <xsl:template name="manual_outlet_switches">
    <xsl:param name="top" select="'70'"/>
    <xsl:param name="left" select="'40'"/>
    <xsl:param name="w" select="'480'"/>
    <xsl:param name="h" select="'80'"/>
    <xsl:param name="border" select="'1'"/>
    <xsl:param name="class" select="'XLarge'"/>
    <xsl:param name="title" select="'Manual&#160;Outlet&#160;Switches'"/>
    <xsl:param name="id" select="'m_o_s'"/>

    <script type="text/javascript">
      <xsl:comment>
        <![CDATA[

     /* function startRefresh(timeout)
      {
        refreshTimer = window.setTimeout("refresh()", timeout);
      }
      function refresh(timeout)
      {
        window.clearTimeout(refreshTimer);
        document.write('<div style="position:absolute;left:100px;top:20px;width:100px;height:1px;background-color:red;overflow:hidden;"/>');
        alert("after");

      }*/

          function outlet_switches(titel,w,h,args)
          {
             x=screen.availWidth/2-w/2;
             y=screen.availHeight/2-h/2;
             index = location.href.indexOf("?");
            if (index != -1)
            {
              href =location.href.substring(0, index+1 );
              href = href.concat(args);
              var size = 'width='+w+',height='+h;
              var pos = 'left='+x+',top='+y+',screenX='+x+',screenY='+y;
              temp = this.document;
              location.replace(href,'titel',size+pos);
            }
          }
        ]]>
      </xsl:comment>
    </script>

    <div id="{$id}" style="position:absolute; top:{$top}px;left:{$left}px;width:{$w}px;height:{$h}px;border: {$border}px solid black;">
      <div id="div_m1" class ="{$class}" style="position:absolute; top:8px;left:155px;width:80px;height:40px;text-align:center;">
        <xsl:value-of select="$title"/>
      </div>
      <div id="div_m2" class ="Medium" style="position:absolute; top:42px;left:50px;width:80px;height:40px;text-align:left;">
        <xsl:value-of select="'Outlet1:'"/>
      </div>
      <div id="div_m3" class ="Medium" style="position:absolute; top:40px;left:110px;width:80px;height:40px;text-align:left;">
        <a href="javascript:outlet_switches('Extended&#160;Commands',570,520,'outlet_switches 1 0')" class="btn">
          <span>
            <span>
              <xsl:value-of select="'Off'"/>
            </span>
          </span>
        </a>
      </div>
      <div id="div_m4" class ="Medium" style="position:absolute; top:40px;left:160px;width:80px;height:40px;text-align:left;">
        <a href="javascript:outlet_switches('Extended&#160;Commands',570,520,'outlet_switches 1 1')" class="btn">
          <span>
            <span>
              <xsl:value-of select="'On'"/>
            </span>
          </span>
        </a>
      </div>

      <div id="div_m5" class ="Medium" style="position:absolute; top:42px;left:270px;width:80px;height:40px;text-align:left;">
        <xsl:value-of select="'Outlet2:'"/>
      </div>
      <div id="div_m6" class ="Medium" style="position:absolute; top:40px;left:330px;width:80px;height:40px;text-align:left;">
        <a href="javascript:outlet_switches('Extended&#160;Commands',570,520,'outlet_switches 2 0');" class="btn">
          <span>
            <span>
              <xsl:value-of select="'Off'"/>
            </span>
          </span>
        </a>
      </div>
      <div id="div_m7" class ="Medium" style="position:absolute; top:40px;left:380px;width:80px;height:40px;text-align:left;">
        <a href="javascript:outlet_switches('Extended&#160;Commands',570,520,'outlet_switches 2 1')" class="btn">
          <span>
            <span>
              <xsl:value-of select="'On'"/>
            </span>
          </span>
        </a>
      </div>
    </div>
  </xsl:template>

  <xsl:template name="special_features">
    <xsl:param name="top" select="'70'"/>
    <xsl:param name="left" select="'40'"/>
    <xsl:param name="w" select="'480'"/>
    <xsl:param name="h" select="'80'"/>
    <xsl:param name="border" select="'1'"/>
    <xsl:param name="class" select="'XLarge'"/>
    <xsl:param name="title" select="'Special&#160;Features'"/>
    <xsl:param name="id" select="'s_f'"/>

    <script type="text/javascript">
      <xsl:comment>
        <![CDATA[
          function special_feature(titel,w,h,args)
          {
             x=screen.availWidth/2-w/2;
             y=screen.availHeight/2-h/2;
             index = location.href.indexOf("?");
            if (index != -1)
            {
              href =location.href.substring(0, index+1 );
              href = href.concat(args);
              var size = 'width='+w+',height='+h;
              var pos = 'left='+x+',top='+y+',screenX='+x+',screenY='+y;
              location.replace(href,'titel',size+pos);
            }
          }
        ]]>
      </xsl:comment>
    </script>

    <div id="{$id}" style="position:absolute; top:{$top}px;left:{$left}px;width:{$w}px;height:{$h}px;border: {$border}px solid black;">
      <div id="div_s1" class ="{$class}" style="position:absolute; top:8px;left:155px;width:80px;height:40px;text-align:center;">
        <xsl:value-of select="$title"/>
      </div>
      <div id="div_s2" class ="Medium" style="position:absolute; top:40px;left:90px;width:80px;height:40px;text-align:left;">
        <xsl:value-of select="'Check&#160;of&#160;grounding:'"/>
      </div>
      <div id="div_s3" class ="Medium" style="position:absolute; top:40px;left:300px;width:80px;height:40px;text-align:left;">
        <a href="javascript:special_feature('Extended&#160;Commands',570,520,'check_of_grounding 0')" class="btn">
          <span>
            <span>
              <xsl:value-of select="'Off'"/>
            </span>
          </span>
        </a>
      </div>
      <div id="div_s4" class ="Medium" style="position:absolute; top:40px;left:340px;width:80px;height:40px;text-align:left;">
        <a href="javascript:special_feature('Extended&#160;Commands',570,520,'check_of_grounding 1')" class="btn">
          <span>
            <span>
              <xsl:value-of select="'On'"/>
            </span>
          </span>
        </a>
      </div>

      <div id="div_s5" class ="Medium" style="position:absolute; top:80px;left:90px;width:240px;height:40px;text-align:left;">
        <xsl:value-of select="'Automatic&#160;shutdown(Load &lt; 5%):'"/>
      </div>
      <div id="div_s6" class ="Medium" style="position:absolute; top:80px;left:300px;width:80px;height:40px;text-align:left;">
        <a href="javascript:special_feature('Extended&#160;Commands',570,520,'auto_shutdown 0')" class="btn">
          <span>
            <span>
              <xsl:value-of select="'Off'"/>
            </span>
          </span>
        </a>
      </div>
      <div id="div_s7" class ="Medium" style="position:absolute; top:80px;left:340px;width:80px;height:40px;text-align:left;">
        <a href="javascript:special_feature('Extended&#160;Commands',570,520,'auto_shutdown 1')" class="btn">
          <span>
            <span>
              <xsl:value-of select="'On'"/>
            </span>
          </span>
        </a>
      </div>

    </div>
  </xsl:template>

  <xsl:template name="programmable_outlet">
    <xsl:param name="top" select="'70'"/>
    <xsl:param name="left" select="'40'"/>
    <xsl:param name="w" select="'480'"/>
    <xsl:param name="h" select="'80'"/>
    <xsl:param name="t1" select="100"/>
    <xsl:param name="t2" select="2"/>
    <xsl:param name="t3" select="3"/>
    <xsl:param name="t4" select="4"/>
    <xsl:param name="c1" select="1"/>
    <xsl:param name="c2" select="2"/>
    <xsl:param name="c3" select="3"/>
    <xsl:param name="c4" select="4"/>
    <xsl:param name="border" select="'1'"/>
    <xsl:param name="class" select="'XLarge'"/>
    <xsl:param name="title" select="'Programmable&#160;Outlets&#160;1'"/>
    <xsl:param name="id" select="'outlet1_id'"/>
    <xsl:param name="outlet_nr" select="'prg_outlet1'"/>

    <script type="text/javascript">
      <xsl:comment>
        <![CDATA[

          function isInteger(n)
          {
            return (Math.floor(n) == n);
          }

          function check_input(outlet_nr)
          {
            var input1 = document.forms[outlet_nr].t1.value;
            var input2 = document.forms[outlet_nr].t2.value;
            var input3 = document.forms[outlet_nr].t3.value;
            var input4 = document.forms[outlet_nr].t4.value;
            if(!isInteger(input1) || !isInteger(input2) ||!isInteger(input3) ||!isInteger(input4))
            {
                alert("Please enter a valid number!");
                return false;
            }
            if((parseInt(input1) < 0 | parseInt(input1) > 3600) || (parseInt(input2) < 0 | parseInt(input2) > 3600) || (parseInt(input3) < 0 | parseInt(input3) > 3600))
            {
                alert("Please enter a valid number between 0 and 3600 !");
                return false;
            }
            if((parseInt(input4) < 20 | parseInt(input4) > 80))
            {
                alert("Please enter a valid number between 20 and 80 !");
                return false;
            }
            return true;
          }

          function prog_outlet(titel,w,h, outlet_nr)
          {
             if(!check_input(outlet_nr))
              return false;
             var args = outlet_nr+" "+document.forms['prg_outlet1'].t1.value + " " +document.forms['prg_outlet1'].t2.value + " "+document.forms['prg_outlet1'].t3.value + " "
                      +document.forms['prg_outlet1'].t4.value;
             args += document.forms['prg_outlet1']['cb'][1].checked?" 1":" 0";
             args += document.forms['prg_outlet1']['cb'][2].checked?" 1":" 0";
             args += document.forms['prg_outlet1']['cb'][3].checked?" 1":" 0";
             args += document.forms['prg_outlet1']['cb'][4].checked?" 1":" 0";

              args += " "+document.forms['prg_outlet2'].t1.value + " " +document.forms['prg_outlet2'].t2.value + " "+document.forms['prg_outlet2'].t3.value + " "
                      +document.forms['prg_outlet2'].t4.value;
             args += document.forms['prg_outlet2']['cb'][1].checked?" 1":" 0";
             args += document.forms['prg_outlet2']['cb'][2].checked?" 1":" 0";
             args += document.forms['prg_outlet2']['cb'][3].checked?" 1":" 0";
             args += document.forms['prg_outlet2']['cb'][4].checked?" 1":" 0";

             x=screen.availWidth/2-w/2;
             y=screen.availHeight/2-h/2;
             index = location.href.indexOf("?");
            if (index != -1)
            {
              href =location.href.substring(0, index+1 );
              href = href.concat(args);
              var size = 'width='+w+',height='+h;
              var pos = 'left='+x+',top='+y+',screenX='+x+',screenY='+y;
              location.replace(href,'titel',size+pos);
            }
          }
      ]]>
      </xsl:comment>
    </script>

    <div id="{$id}" style="position:absolute; top:{$top}px;left:{$left}px;width:{$w}px;height:{$h}px;border:{$border}px solid black;">
      <div id="pr_outlets_1" class ="{$class}" style="position:absolute; top:8px;left:132px;width:180px;height:20px;text-align:center;">
        <xsl:value-of select="'Programmable Outlet 1'"/>
        <br></br>
      </div>
      <!--<script type="text/javascript"> translate_text('pr_outlets_1', ['Programmable Outlet', ' 1']);</script>-->

      <br></br>
      <br></br>
      <br></br>
      <form name="{$outlet_nr}">
        <table class ="Medium" border="0" width="100%" cellpadding="1" cellspacing="2">
          <tr>
            <td align="left">
              <input id="pr_outlets_1_turn_on_1" type="checkbox" name="cb" value="c1" checked="true" disabled="true">
                <!--checked="false" disabled="true"-->
                <xsl:value-of select="'Turn outlet ON after UPS is on'"/>
                <!--<script type="text/javascript"> translate_text('pr_outlets_1_turn_on_1', ['Turn outlet ON after UPS is on', '  ']);</script>-->
              </input>
            </td>
            <td align="left">
              <input type="text" name="t1" size="6" value="{$t1}" />
            </td>
            <td align="left">
              <xsl:value-of select="'Seconds&#160;(0-3600)'"/>
            </td>
          </tr>
          <tr>
            <td align="left">
              <xsl:if test="$c1=1">
                <input id="pr_outlets_1_turn_off_1" type="checkbox" name="cb" value="c2" checked="true">
                  <!--<script type="text/javascript"> translate_text('pr_outlets_1_turn_off_1', ['Turn outlet OFF after AC failure', '  ']);</script>-->
                  <xsl:value-of select="'Turn outlet OFF after AC failure'"/>
                </input>
              </xsl:if>
              <xsl:if test="$c1=0">
                <input id="pr_outlets_1_turn_off_2" type="checkbox" name="cb" value="c2">
                  <!--<script type="text/javascript"> translate_text('pr_outlets_1_turn_off_2', ['Turn outlet OFF after AC failure', '  ']);</script>-->
                  <xsl:value-of select="'Turn outlet OFF after AC failure'"/>
                </input>
              </xsl:if>
            </td>
            <td align="left">
              <input type="text" name="t2" size="6" value="{$t2}" />
            </td>
            <td align="left">
              <xsl:value-of select="'Seconds&#160;(0-3600)'"/>
            </td>
          </tr>
          <tr>
            <td align="left">
              <xsl:if test="$c2=1">
                <input id="pr_outlets_1_turn_off_3" type="checkbox" name="cb" value="c3" checked="true">
                  <!--<script type="text/javascript"> translate_text('pr_outlets_1_turn_off_3', ['Turn outlet ON after AC recovery', '  ']);</script>-->
                  <xsl:value-of select="'Turn outlet ON after AC recovery'"/>
                </input>
              </xsl:if>
              <xsl:if test="$c2=0">
                <input id="pr_outlets_1_turn_off_4" type="checkbox" name="cb" value="c3">
                  <xsl:value-of select="'Turn outlet ON after AC recovery'"/>
                  <!--<script type="text/javascript"> translate_text('pr_outlets_1_turn_off_4', ['Turn outlet ON after AC recovery', '  ']);</script>-->
                </input>
              </xsl:if>
            </td>
            <td align="left">
              <input type="text" name="t3" size="6" value="{$t3}" />
            </td>
            <td align="left">
              <xsl:value-of select="'Seconds&#160;(0-3600)'"/>
            </td>
          </tr>
          <tr>
            <td align="left">
              <xsl:if test="$c3=1">
                <input id="pr_outlets_1_turn_off_5" type="checkbox" name="cb" value="c4" checked="true">
                  <!--<script type="text/javascript"> translate_text('pr_outlets_1_turn_off_5', ['Turn outlet OFF when battery lower', '  ']);</script>-->
                  <xsl:value-of select="'Turn outlet OFF when battery lower'"/>
                </input>
              </xsl:if>
              <xsl:if test="$c3=0">
                <input id="pr_outlets_1_turn_off_6" type="checkbox" name="cb" value="c4">
                  <!--<script type="text/javascript"> translate_text('pr_outlets_1_turn_off_6', ['Turn outlet OFF when battery lower', '  ']);</script>-->
                  <xsl:value-of select="'Turn outlet OFF when battery lower'"/>
                </input>
              </xsl:if>
            </td>
            <td align="left">
              <input type="text" name="t4" size="6" value="{$t4}" />
            </td>
            <td align="left">
              <xsl:value-of select="'%(20-80)'"/>
            </td>
          </tr>
          <tr>
            <td align="left">
              <xsl:if test="$c4=1">
                <input id="pr_outlets_1_turn_off_7" type="checkbox" name="cb" value="c5" checked="true">
                  <!--<script type="text/javascript"> translate_text('pr_outlets_1_turn_off_7', ['Turn outlet OFF when UPS overload', '  ']);</script>-->
                  <xsl:value-of select="'Turn outlet OFF when UPS overload'"/>
                </input>
              </xsl:if>
              <xsl:if test="$c4=0">
                <input id="pr_outlets_1_turn_off_8" type="checkbox" name="cb" value="c5">
                  <!--<script type="text/javascript"> translate_text('pr_outlets_1_turn_off_8', ['Turn outlet OFF when UPS overload', '  ']);</script>-->
                  <xsl:value-of select="'Turn outlet OFF when UPS overload'"/>
                </input>
              </xsl:if>
            </td>
          </tr>
        </table>
      </form>

      <div id="w_s" style="position:absolute; top:174px;left:170px;width:120px;height:40px;">
        <a href="javascript:prog_outlet('Programmable&#160;Outlet&#160;Settings','530','580', '{$outlet_nr}')" class="btn">
          <span>
            <span id="write_settings_id" >
              <!--<script type="text/javascript"> translate_text('write_settings_id', 'write settings');</script>-->
              <xsl:value-of select="'write settings'"/>
            </span>
          </span>
        </a>
      </div>
    </div>
  </xsl:template>

  <xsl:template name="extcmd_infos">
    <xsl:param name="top" select="'460'"/>
    <xsl:param name="left" select="'40'"/>
    <xsl:param name="w" select="'16'"/>
    <xsl:param name="h" select="'16'"/>
    <xsl:param name="border" select="'1'"/>
    <xsl:param name="class" select="'XLarge_black'"/>
    <xsl:param name="infos" select="''"/>
    <xsl:param name="color" select="'#52B245'"/>
    <script type="text/javascript">
      <xsl:comment>
        <![CDATA[

     function startTimer(timeout)
      {
        Timer = window.setTimeout("hide()", timeout);
      }
      function hide(timeout)
      {
        document.getElementById('ext_infos_id').style.visibility="hidden";
        document.getElementById('infos').style.visibility="hidden";
        window.clearTimeout(Timer);
      }
      startTimer(3000);
        ]]>
      </xsl:comment>
    </script>

    <div id="ext_infos_id" class = "{$class}" style="position:absolute; top:{$top}px;left:{$left}px;width:{$w}px;height:{$h}px;background-color:{$color};text-align:left;">
    </div>
    <div id="infos_txt" class = "{$class}" style="position:absolute; top:{$top}px;left:{$left + 24}px;width:300px;height:{$h}px;text-align:left;">
      <script type="text/javascript">
        translate_text('infos_txt', '<xsl:value-of select="$infos"/>');
      </script>
    </div>
  </xsl:template>

  <!-- draw horisontal line-->
  <xsl:template name="draw_line">
    <xsl:param name="x" select="592"/>
    <xsl:param name="y" select="536"/>
    <xsl:param name="w" select="10"/>
    <xsl:param name="h" select="30"/>
    <xsl:param name="color" select="'#2129AB'"/>
    <xsl:param name="border_bottom" select="'1'"/>
    <xsl:param name="border_right" select="'1'"/>
    <xsl:param name="border_left" select="'1'"/>
    <xsl:param name="border_top" select="'1'"/>

    <div style ="position:absolute;left:{$x}px;top:{$y}px;width:{$w}px;height:{$h}px;">
      <hr style="overflow:hidden;color:{$color}; background: {$color}; height:{$h}px; width:{$w}px;border-bottom :{$border_bottom}pt solid #395A84;border-right :{$border_right}pt solid #395A84; border-left :{$border_left}pt solid #395A84;border-top :{$border_top}pt solid #395A84"/>

      <!---->

    </div>
  </xsl:template>

  <xsl:template name="ups-history">
    <xsl:param name="top" select="458"/>
    <xsl:param name="left" select="30"/>
    <xsl:param name="w" select="120"/>
    <xsl:param name="h" select="34"/>
    <xsl:param name="service_oid" select="''"/>
    <xsl:param name="type" select="'maildev'"/>

    <script type="text/javascript">
      <!--<xsl:comment>-->
      <![CDATA[
          function get_ups_history(service_oid)
          {
              var cmd = 'script/nmsView.dll?upshist=UPS_GET_HISTORY&refoid=' + service_oid,
                url = get_url(cmd);
              window.open(url);
          }
      ]]>
      <!--</xsl:comment>-->
    </script>

    <xsl:if test="string-length($service_oid) != 0 and $type = 'maildev'">
      <div id="ups_history_div" align="center" style="text-align:center; position:absolute; top:{$top}px;left:{$left}px;width:{$w}px;height:{$h}px;color: black; font-family: arial; font-size: 14px;font-weight: bold;">
        <!--<xsl:value-of select="'System:&#160;&#160;&#160;'"/>-->
        <a href="javascript:get_ups_history('{$service_oid}')" class="btn">
          <span>
            <span>
              <xsl:value-of select="'UPS History'"/>
            </span>
          </span>
        </a>
      </div>
    </xsl:if>
  </xsl:template>
</xsl:stylesheet>
