/*
   apcsts_panel.js - APC ATS screen.
   Copyright (c) 2016, Generex GmbH. All rights reserved.
   UW.
*/

var apcsts_api = (function ($) {
  var oid = get_oid_from_url(),
  cb = null,
  panes_created = false,
  err_div = null,
  check_bowser = true,
  css_obj, attr_obj,
  valtable_obj, report_obj, unms_obj,
  x_pos = 10,
  y_pos = 10;

  var req_params = null;

  /*public functions*/
  return {
    init: init
  };

  function init() {
    if (check_bowser && !utils_api.check_bowser()) {
      $('#loading_id').css('display', 'none');
      panes_api.create_br_check_panel(this, browser_check_ok);
      return true;
    }
    create_err_msg_panel();

    req_params = utils_api.get_req_params(utils_api.device_ups, oid);
    cb = jqmodule.subscribe(this, { update: update, error: err_funct }, req_params);
  }

  function browser_check_ok() {
    check_bowser = false;
    init();
  }

  function create_err_msg_panel() {
    css_obj = { position: 'absolute', top: y_pos, left: x_pos, width: 791, display: 'none', 'z-index': 22 };
    err_div = panes_api.create_err_panel({ id: 'error_div', 'class': 'alert alert-danger' }, css_obj);
  }

  function err_funct(response) {
    jqmodule.refresh(cb, 10 * 1000);
    panes_api.display_err({ response: response, oid: oid, err_div: err_div, err_msg_div_id: 'error_msg_div' });
  }

  function update(data) {
    if (data.ups == null) {
      return;
    }

    var refresh
    valtable_obj = data.ups.valtable;
    report_obj = data.report;

    jqmodule.refresh(cb, report_obj.refresh * 1000);
    if (err_div)
    err_div.css('display', 'none');

    if (window.menu_api != null)
    menu_api.update(data);

    unms_obj = get_unms_obj(data);
    if (unms_obj != null) {
      info_head_api.update(unms_obj);
    }

    if (!panes_created) {
      panes_created = true;
      create_panes();
      return false;
    }

    set_sts_led();
    set_input_src_1(); //Set Input Source 1
    set_input_src_2(); //Set Input Source 2
    set_warnings();
    set_outpower_loadbars(); //outpower loadbars
    set_output();
    set_footer();
  }

  function create_footer_pane() {
    if (!oid || !unms_obj)
    return true;
    footer_api.create({ //Attr
        id: 'panel2'
        //'class':'unmsFooter'
      }, { //css
        position: 'absolute',
        top: '470px',
        left: '74px'
      },
      unms_obj
    );
  }

  function set_footer() {
    if (!oid || !unms_obj)
    return true;
    footer_api.set(unms_obj);
  }

  function create_panes() {
    utils_api.setTitle(valtable_obj._platform);
    create_bg(); //background
    create_model_panes();
    create_sts_led();
    create_input_src_1_pane() //Input Source 1
    create_input_src_2_pane() //Input Source 2
    create_warnings_pane();
    create_outpower_loadbars(); //outpower loadbars
    create_output_pane() //Output
    create_footer_pane();
  }

  function create_bg() {
    var fragment = document.createDocumentFragment(),
    img_div;
    css_obj = { position: 'absolute', top: y_pos, left: x_pos, width: 950, height: 446, border: 'solid 1px #808080', align: 'center' };
    img_div = $('<div/>', { 'class': 'img-rounded' }).css(css_obj);

    $('<img />').attr({ 'id': 'bg_img_id', 'class': 'img-rounded', src: './image/sts/apcats.png' }).css({ display: 'block', 'margin': 'auto' }).appendTo(img_div);

    fragment.appendChild(img_div[0]);
    document.body.appendChild(fragment);
  }

  function create_model_panes() {
    var model_div, a;
    css_obj = { position: 'absolute', top: y_pos, left: x_pos, align: 'center', width: 950 };
    attr_obj = { id: 'model_div_id', 'class': 'list-group' };
    model_div = panes_api.create_div(attr_obj, css_obj);
    a = $('<a/>', { 'class': 'list-group-item active' }).append(valtable_obj.atsIdentModelNumber).appendTo(model_div);
  }

  function create_sts_led() {
    var fragment = document.createDocumentFragment(),
    div, img, src;
    //Source 1
    css_obj = { position: 'absolute', top: y_pos + 106, left: x_pos + 235, width: 80, height: 18, color: '#3F5987', 'font-size': ' 13px', 'font-weight': 'bold' };
    div = $('<div/>').css(css_obj);
    div.append('Source A');
    fragment.appendChild(div[0]);

    css_obj.top = y_pos + 126;
    css_obj.left = x_pos + 260;
    css_obj.width = 17;
    css_obj.height = 17;
    div = $('<div/>').css(css_obj);
    $('<img />').attr({ 'id': 'source_1_img', 'class': 'img-rounded', width: 16, 'height': 16, src: src }).appendTo(div);
    fragment.appendChild(div[0]);

    //Source 2
    css_obj = { position: 'absolute', top: y_pos + 340, left: x_pos + 235, width: 80, height: 18, color: '#3F5987', 'font-size': ' 13px', 'font-weight': 'bold' };
    div = $('<div/>').css(css_obj);
    div.append('Source B');
    fragment.appendChild(div[0]);

    css_obj.top = y_pos + 314;
    css_obj.left = x_pos + 260;
    css_obj.width = 17;
    css_obj.height = 17;
    div = $('<div/>').css(css_obj);
    $('<img />').attr({ 'id': 'source_2_img', 'class': 'img-rounded', width: 16, 'height': 16, src: src }).appendTo(div);
    fragment.appendChild(div[0]);

    //switch 1
    css_obj = { position: 'absolute', top: y_pos + 106, left: x_pos + 417, width: 80, height: 18, color: '#3F5987', 'font-size': ' 13px', 'font-weight': 'bold' };
    div = $('<div/>').css(css_obj);
    div.append(i18n('Preferred A'));
    fragment.appendChild(div[0]);

    css_obj.top = y_pos + 126;
    css_obj.left = x_pos + 450;
    css_obj.width = 17;
    css_obj.height = 17;
    div = $('<div/>').css(css_obj);
    $('<img />').attr({ 'id': 'switch_1_img', 'class': 'img-rounded', width: 16, 'height': 16, src: src }).appendTo(div);
    fragment.appendChild(div[0]);

    //switch 2
    css_obj = { position: 'absolute', top: y_pos + 340, left: x_pos + 417, width: 80, height: 18, color: '#3F5987', 'font-size': ' 13px', 'font-weight': 'bold' };
    div = $('<div/>').css(css_obj);
    div.append(i18n('Preferred B'));
    fragment.appendChild(div[0]);

    css_obj.top = y_pos + 314;
    css_obj.left = x_pos + 450;
    css_obj.width = 17;
    css_obj.height = 17;
    div = $('<div/>').css(css_obj);
    $('<img />').attr({ 'id': 'switch_2_img', 'class': 'img-rounded', width: 16, 'height': 16, src: src }).appendTo(div);
    fragment.appendChild(div[0]);

    //output led
    css_obj.top = y_pos + 224;
    css_obj.left = x_pos + 730;
    css_obj.width = 17;
    css_obj.height = 17;
    div = $('<div/>').css(css_obj);
    $('<img />').attr({ 'id': 'output_img', 'class': 'img-rounded', width: 16, 'height': 16, src: src }).appendTo(div);
    fragment.appendChild(div[0]);

    document.body.appendChild(fragment);
    set_sts_led();
  }

  function set_sts_led() {
    var src1, src2, ledgrey, ledgreen, ledred;

    ledgrey = './image/bacs2/b_grey.gif';
    ledgreen = './image/bacs2/b_green.gif';
    ledred = './image/bacs2/b_red.gif';

    src1 = src2 = ledgrey;
    if (valtable_obj.atsStatusSelectedSource == 1)
    src1 = ledgreen;
    else if (valtable_obj.atsStatusSelectedSource == 2)
    src2 = ledgreen;
    $('#switch_1_img').attr('src', src1);
    $('#switch_2_img').attr('src', src2);

    src1 = src2 = ledgrey;
    if (valtable_obj.atsStatusSourceAStatus == 2)
    src1 = ledgreen;
    if (valtable_obj.atsStatusSourceBStatus == 2)
    src2 = ledgreen;
    $('#source_1_img').attr('src', src1);
    $('#source_2_img').attr('src', src2);

    src1 = ledgrey;
    if (valtable_obj.atsStatusVoltageOutStatus == 2)
    src1 = ledgreen;
    if (valtable_obj.atsStatusOverCurrentState == 1)
    src1 = ledred;
    $('#output_img').attr('src', src1);
  }

  function create_input_src_1_pane() //Input Source 1
  {
    var fragment = document.createDocumentFragment(),
    val, tr, table, tbody, thead, div;

    css_obj = { position: 'absolute', top: y_pos + 100, left: x_pos + 28 };

    div = $('<div/>', { id: 'input_src_1' }).css(css_obj);
    table = $('<table />', { 'class': 'table table-condensed table-hover' }).appendTo(div);
    tbody = $('<tbody />').appendTo(table);
    thead = $('<thead />').appendTo(table);

    tr = $('<tr />').append($('<th />', { colspan: 3 }).append(i18n(lang_Input) + ' ' + i18n('Source A'))).appendTo(thead);

    tr = $('<tr />').append($('<td />').append("L1:")).appendTo(tbody);

    td = $('<td />', { id: 'src_1_volt_1', align: 'left' }).
    append(get_value({ values: [valtable_obj.atsInputVoltageA], index: 0, unit: ' V', precision: 1 })).appendTo(tr);

    td = $('<td />', { id: 'src_1_freq_1', align: 'left' }).
    append(get_value({ values: [valtable_obj.atsInputFrequencyA], index: 0, unit: ' Hz', precision: 1 })).appendTo(tr);

    fragment.appendChild(div[0]);
    document.body.appendChild(fragment);
  }

  function set_input_src_1() //Set Input Source 1
  {
    $('#src_1_volt_1').
    text(get_value({ values: [valtable_obj.atsInputVoltageA], index: 0, unit: ' V', precision: 1 }));

    $('#src_1_freq_1').
    text(get_value({ values: [valtable_obj.atsInputFrequencyA], index: 0, unit: ' Hz', precision: 1 }));

  }

  function create_input_src_2_pane() //Input Source 2
  {
    var fragment = document.createDocumentFragment(),
    val, tr, table, tbody, thead, div;

    css_obj = { position: 'absolute', top: y_pos + 290, left: x_pos + 28 };

    div = $('<div/>', { id: 'input_src_2' }).css(css_obj);
    table = $('<table />', { 'class': 'table table-condensed table-hover' }).appendTo(div);
    tbody = $('<tbody />').appendTo(table);
    thead = $('<thead />').appendTo(table);

    tr = $('<tr />').append($('<th />', { colspan: 3 }).append(i18n(lang_Input) + ' ' + i18n('Source B'))).appendTo(thead);

    tr = $('<tr />').append($('<td />').append("L1:")).appendTo(tbody);

    td = $('<td />', { id: 'src_2_volt_1', align: 'left' }).
    append(get_value({ values: [valtable_obj.atsInputVoltageB], index: 0, unit: ' V', precision: 1 })).appendTo(tr);

    td = $('<td />', { id: 'src_2_freq_1', align: 'left' }).
    append(get_value({ values: [valtable_obj.atsInputFrequencyB], index: 0, unit: ' Hz', precision: 1 })).appendTo(tr);

    fragment.appendChild(div[0]);
    document.body.appendChild(fragment);
  }

  function set_input_src_2() //Set Input Source 2
  {
    $('#src_2_volt_1').
    text(get_value({ values: [valtable_obj.atsInputVoltageB], index: 0, unit: ' V', precision: 1 }));

    $('#src_2_freq_1').
    text(get_value({ values: [valtable_obj.atsInputFrequencyB], index: 0, unit: ' Hz', precision: 1 }));
  }

  function create_output_pane() //Output
  {
    var fragment = document.createDocumentFragment(),
    val, td, tr, table, tbody, thead, div;

    css_obj = { position: 'absolute', top: y_pos + 86, left: x_pos + 700 };

    div = $('<div/>', { id: 'output' }).css(css_obj);
    table = $('<table />', { 'class': 'table table-condensed table-hover' }).appendTo(div);
    tbody = $('<tbody />').appendTo(table);
    thead = $('<thead />').appendTo(table);

    tr = $('<tr />').append($('<th />', { colspan: 2 }).append(i18n(lang_Output))).appendTo(thead);

    tr = $('<tr />').appendTo(tbody);
    td = $('<td />', { id: 'output_volt_1', align: 'left' }).
    append(get_value({ values: [valtable_obj.atsOutputVoltage], index: 0, unit: ' V', precision: 1 })).appendTo(tr);
    td = $('<td />', { id: 'output_freq_1', align: 'left' }).
    append(get_value({ values: [valtable_obj.atsOutputFrequency], index: 0, unit: ' Hz', precision: 1 })).appendTo(tr);

    tr = $('<tr />').appendTo(tbody);
    td = $('<td />', { id: 'output_curr_1', align: 'left' }).
    append(get_value({ values: [valtable_obj.atsOutputCurrent], index: 0, unit: ' A', precision: 1 })).appendTo(tr);
    td = $('<td />', { id: 'output_watt_1', align: 'left' }).
    append(get_value({ values: [valtable_obj.atsOutputPower], index: 0, unit: ' W', precision: 1 })).appendTo(tr);

    fragment.appendChild(div[0]);
    document.body.appendChild(fragment);
  }

  function set_output() //Set Output
  {
    $('#output_volt_1').
    text(get_value({ values: [valtable_obj.atsOutputVoltage], index: 0, unit: ' V', precision: 1 }));

    $('#output_freq_1').
    text(get_value({ values: [valtable_obj.atsOutputFrequency], index: 0, unit: ' Hz', precision: 1 }));

    $('#output_curr_1').
    text(get_value({ values: [valtable_obj.atsOutputCurrent], index: 0, unit: ' A', precision: 1 }));

    $('#output_watt_1').
    text(get_value({ values: [valtable_obj.atsOutputPower], index: 0, unit: ' W', precision: 1 }));
  }

  function create_warnings_pane() {
    var fragment = document.createDocumentFragment(),
    val, tr, table, tbody, thead, div;

    css_obj = { position: 'absolute', top: y_pos + 170, left: x_pos + 260 };

    div = $('<div/>', { id: 'output' }).css(css_obj);
    table = $('<table />', { 'class': 'table table-condensed table-hover' }).appendTo(div);
    tbody = $('<tbody />').appendTo(table);
    thead = $('<thead />').appendTo(table);

    tr = $('<tr />').append($('<td />', { align: 'left' }).append(i18n("Hardware status" + ":"))).appendTo(tbody);
    td = $('<td />', { id: 'hardware_status', align: 'left' }).appendTo(tr);

    tr = $('<tr />').append($('<td />', { align: 'left' }).append(i18n("Redundancy status" + ":"))).appendTo(tbody);
    td = $('<td />', { id: 'redundancy_state', align: 'left' }).appendTo(tr);

    tr = $('<tr />').append($('<td />', { align: 'left' }).append(i18n("Overcurrent status" + ":"))).appendTo(tbody);
    td = $('<td />', { id: 'overcurrent_state', align: 'left' }).appendTo(tr);

    tr = $('<tr />').append($('<td />', { align: 'left' }).append(i18n("Outvoltage status" + ":"))).appendTo(tbody);
    td = $('<td />', { id: 'outvolt_state', align: 'left' }).appendTo(tr);

    tr = $('<tr />').append($('<td />', { align: 'left' }).append(i18n("Phasesync status" + ":"))).appendTo(tbody);
    td = $('<td />', { id: 'phasesync_state', align: 'left' }).appendTo(tr);

    fragment.appendChild(div[0]);
    document.body.appendChild(fragment);
    set_warnings();
  }

  function set_warnings() {
    var txt;
    if (valtable_obj.atsStatusHardwareStatus == 1)
    txt = "Fail";
    else
    txt = "Ok";
    $('#hardware_status').text(txt);

    if (valtable_obj.atsStatusRedundancyState == 1)
    txt = "Lost";
    else
    txt = "Ok";
    $('#redundancy_state').text(i18n(txt));

    if (valtable_obj.atsStatusOverCurrentState == 1)
    txt = "Fail";
    else
    txt = "Ok";
    $('#overcurrent_state').text(i18n(txt));

    if (valtable_obj.atsStatusVoltageOutStatus == 1)
    txt = "Fail";
    else
    txt = "Ok";
    $('#outvolt_state').text(i18n(txt));

    if (valtable_obj.atsStatusPhaseSyncStatus == 2)
    txt = "Out of sync";
    else
    txt = "Ok";
    $('#phasesync_state').text(i18n(txt));

  }

  function create_outpower_loadbars() //outpower loadbars
  {
    if (valtable_obj.atsOutputPercentLoad == null)
    return true;
    outpower_loadbars_created = true;

    if (valtable_obj.atsOutputPercentLoad != -1)
    panes_api.create_loadbar_pane({ value: valtable_obj.atsOutputPercentLoad, id: 'loadbar1_id', left: x_pos + 700, top: y_pos + 350, title: '', scaler: 1 });
  }

  function set_outpower_loadbars() //outpower loadbars
  {
    if (valtable_obj.atsOutputPercentLoad == null)
    return true;
    if (valtable_obj.atsOutputPercentLoad != -1)
    panes_api.set_loabar_value(valtable_obj.atsOutputPercentLoad, 'loadbar1_id');
  }

  function get_value(val_obj) {
    if (!val_obj.values || (val_obj.values.length <= val_obj.index))
    return 'N/A';
    var val = val_obj.values[val_obj.index];
    if (val == null || val_obj.values[val_obj.index] == -9999)
    return 'N/A';

    val = val.toFixed(val_obj.precision);
    val += val_obj.unit;
    return val;
  }

  function create_rom_temp() //rom temp
  {
    var fragment = document.createDocumentFragment(),
    rom_temp_div;
    css_obj = { position: 'absolute', top: y_pos + 350, left: x_pos + 170 };
    rom_temp_div = $('<div/>').attr({ id: 'rom_temp_div_id' }).css(css_obj);
    panes_api.create_rom_temp_pane(valtable_obj, { id: 'rom_temp_table_id', 'class': 'table table-condensed table-hover' }).appendTo(rom_temp_div);
    fragment.appendChild(rom_temp_div[0]);
    document.body.appendChild(fragment);
  }

}(jQuery));