/*
   apoconv_panel.js - APOCONV screen.
   Copyright (c) 2013-2020, Generex GmbH. All rights reserved.
   Said.
   Markus.
   UW
*/

var apoconv_api = (function ($) {
  var oid = get_oid_from_url(),
    cb = null,
    valtable_obj,
    report_obj, unms_obj,
    pannes_created = false,
    check_bowser = true,
    err_div = null,
    css_obj,
    css_val = { 'text-align': 'right' },
    css_unit = { 'padding-left': '2px', 'text-align': 'left' },
    curr_select = 0,
    panels_id = ['#rec_div_id', '#inv_div_id', '#out1_div_id', '#out2_div_id', '#def_div_id', '#dc_div_id', '#kwh_div_id', '#kwh2_div_id'];

  var LED_OFF = 0;
  var LED_ON = 1;
  var LED_LOW = 2;
  var LED_MED = 3;
  var LED_HI = 4;
  var PANEL_TABLE_STYLE = { 'class': 'XMedium_blue', align: 'left', cellpadding: 0, cellspacing: 0, width: '217px', height: '67px' };
  var CSS_PANEL_TABLE = { 'margin-left': '4px', 'margin-top': '4px' };
  var css_panel_div = {
    position: 'absolute',
    top: 137,
    left: 162,
    width: 225,
    height: 75,
    'background-color': '#A5C3CE',
    'align': 'center',
    visibility: 'hidden',
    overflow: 'hidden'
  };
  var css_oldpanel_div = {
    position: 'absolute',
    top: 195,
    left: 250,
    width: 225,
    height: 75,
    'background-color': '#FFFFCC',
    'align': 'center',
    visibility: 'hidden',
    overflow: 'hidden'
  };

  var FIRST_OF_FIVE_COLUM_STYLE = { 'width': '50px' };

  var req_params = null;
  var newpanel = false;
  var out2avail = false;

  function err_funct(response) {
    jqmodule.refresh(cb, 10 * 1000);
    panes_api.display_err({ response: response, oid: oid, err_div: err_div, err_msg_div_id: 'error_msg_div' });
  }

  return {
    init: init
  }

  function init() {
    if (check_bowser && !utils_api.check_bowser()) {
      $('#loading_id').css('display', 'none');
      panes_api.create_br_check_panel(this, browser_check_ok);
      return true;
    }
    create_err_msg_panel();

    req_params = utils_api.get_req_params(utils_api.device_ups, oid);
    cb = jqmodule.subscribe(this, { update: update, error: err_funct }, req_params);
  }

  function browser_check_ok() {
    check_bowser = false;
    init();
  }

  function create_err_msg_panel() {
    css_obj = { position: 'absolute', top: 30, left: 40, width: 790, display: 'none', 'z-index': 10 };
    err_div = panes_api.create_err_panel({ id: 'error_div', 'class': 'alert alert-danger' }, css_obj);
  }

  function update(data) {
    if (data.ups == null) {
      return;
    }

    if (err_div)
      err_div.css('display', 'none');

    valtable_obj = data.ups.valtable;
    report_obj = data.report;
    jqmodule.refresh(cb, report_obj.refresh * 1000);
    if (window.menu_api != null)
      menu_api.update(data);

    unms_obj = get_unms_obj(data);
    if (unms_obj != null) {
      info_head_api.update(unms_obj);
    }

    utils_api.setTitle(valtable_obj._platform);

    if (!pannes_created) {
      out2avail = valtable_obj.ADR99 == 1;
      newpanel = valtable_obj.ADRC2 == 1;
      pannes_created = true;
      create_pannes(); //create screen.
      return;
    }

    set_to_bg_panel();
    set_mains_panel();
    set_inverter_panel();
    set_output1_panel();
    set_output2_panel();
    set_default_panel();
    set_dc_panel();
    set_energy_meter_panel();
    set_intermediate_meter_panel();
    set_footer();
  }

  function create_pannes() //create screen.
  {
    create_two_output_panel();
  }

  function create_two_output_panel() {
    create_to_bg_panel();
    create_mains_panel();
    create_inverter_panel();
    create_output1_panel();
    create_output2_panel();
    create_default_panel();
    create_select_panel();
    create_dc_panel();
    create_energy_meter_panel();
    create_intermediate_meter_panel();
    create_footer_pane();
  }

  // Helper functions
  function get_led_source(color, state) {
    var led = "./image/piller/led";
    var ledsize = "14x14";

    if (newpanel)
      ledsize = "10x10";
    if (null == color || null == state)
      return led + "grau" + ledsize + ".gif";
    switch (state) {
      case LED_OFF:
        led += "grau" + ledsize + ".gif";
        break;
      case LED_ON:
        led += color + ledsize + ".gif";
        break;
      case LED_LOW:
        led += color + ledsize + "_flash_l.gif";
        break;
      case LED_MED:
        led += color + ledsize + "_flash_m.gif";
        break;
      case LED_HI:
        led += color + ledsize + "_flash_h.gif";
        break;
      default:
        led += "grau" + ledsize + ".gif";
    }
    return led;
  }

  function get_led_source2(color1, color2, state1, state2) {
    var ledsize = "14x14";
    if (newpanel)
      ledsize = "10x10";
    var led = "./image/piller/ledrot_gruen" + ledsize + "_flash_";
    if (null == color1) return get_led_source(color2, state2);
    if (null == color2) return get_led_source(color1, state1);
    if (null == state1) return get_led_source(color2, state2);
    if (null == state2) return get_led_source(color1, state1);
    if (0x0 == state1) return get_led_source(color2, state2);
    if (0x0 == state2) return get_led_source(color1, state1);
    if (state1 == state2) return get_led_source("gelb", state1);
    switch (state1) {
      case LED_ON:
        return get_led_source("gelb", state1);
      case LED_LOW:
        led += "l";
        break;
      case LED_MED:
        led += "m";
        break;
      case LED_HI:
        led += "h";
        break;
    }
    led += ".gif";
    return led;
  }

  function toCurrResolution(value) {
    var apoconv_precision = valtable_obj.ADRC1 == 9;
    var val = apoconv_precision ? value.toFixed(1) : value.toFixed(0);
    return val;
  }

  function toFreqResolution(value) {
    var hi_freq_precision = valtable_obj.ADR176 < 100;
    var val = hi_freq_precision ? (value / 10).toFixed(1) : value.toFixed(0);
    return val;
  }

  function get_voltage(volt_obj) {
    if (!volt_obj ||
      undefined == volt_obj.phase_switch ||
      !volt_obj.arr_n ||
      !volt_obj.arr_ph ||
      undefined == volt_obj.arr_id) {
      return "";
    }
    if (3 != volt_obj.arr_n.length ||
      3 != volt_obj.arr_ph.length) {
      return "";
    }

    //return volt_obj.phase_switch ? volt_obj.arr_ph[volt_obj.arr_id] : volt_obj.arr_n[volt_obj.arr_id];
    var val = 0;
    if (volt_obj.phase_switch) {
      val = volt_obj.arr_ph[volt_obj.arr_id];
    } else {
      val = volt_obj.arr_n[volt_obj.arr_id];
    }
    return val;
  }

  function mains_volt(l) {
    if (!l) { return ""; }
    var vo = {};
    vo.arr_id = l - 1;
    vo.phase_switch = valtable_obj.ADRA2;
    vo.arr_n = new Array(valtable_obj.ADRB1, valtable_obj.ADRB2, valtable_obj.ADRB3);
    vo.arr_ph = new Array(valtable_obj.ADR65, valtable_obj.ADR66, valtable_obj.ADR67);
    return get_voltage(vo);
  }

  function inv_volt(l) {
    if (!l) { return ""; }
    var vo = {};
    vo.arr_id = l - 1;
    /* If valtable_obj.ADRA3 == 1, then show values for phase-phase! */
    vo.phase_switch = valtable_obj.ADRA3;
    vo.arr_n = new Array(valtable_obj.ADR6E, valtable_obj.ADR6F, valtable_obj.ADR70);
    vo.arr_ph = new Array(valtable_obj.ADRB4, valtable_obj.ADRB5, valtable_obj.ADRB6);
    return get_voltage(vo);
  }

  function out1_volt(l) {
    if (!l) { return ""; }
    var vo = {};
    vo.arr_id = l - 1;
    /* If valtable_obj.ADRA4 == 1, then show values for phase-phase! */
    vo.phase_switch = valtable_obj.ADRA4;
    vo.arr_n = new Array(valtable_obj.ADR76, valtable_obj.ADR77, valtable_obj.ADR78);
    vo.arr_ph = new Array(valtable_obj.ADRB7, valtable_obj.ADRB8, valtable_obj.ADRB9);
    return get_voltage(vo);
  }

  function out2_volt(l) {
    if (!l) { return ""; }
    var vo = {};
    vo.arr_id = l - 1;
    vo.phase_switch = valtable_obj.ADRA4;
    vo.arr_n = new Array(valtable_obj.ADR7F, valtable_obj.ADR80, valtable_obj.ADR81);
    vo.arr_ph = new Array(valtable_obj.ADRBE, valtable_obj.ADRBF, valtable_obj.ADRC0);
    return get_voltage(vo);
  }

  /**
   * Overwrites obj1's values with obj2's and adds obj2's if non existent in obj1
   * @param obj1
   * @param obj2
   * @returns obj3 a new object based on obj1 and obj2
   */
  function merge_options(obj1, obj2) {
    var obj3 = {};
    for (var attrname in obj1) {
      obj3[attrname] = obj1[attrname];
    }
    for (var attrname in obj2) {
      obj3[attrname] = obj2[attrname];
    }
    return obj3;
  }

  function get2DigNum(number) {
    return (10 > number ? "0" : "") + number;
  }

  //panels

  function create_to_bg_panel() {
    var fragment = document.createDocumentFragment(),
      div, link, img, led;

    //background img
    if (newpanel) {
      if (out2avail) {
        css_obj = { position: 'absolute', top: 10, left: 10, width: 600, height: 600, align: 'center', 'z-index': 0 };
        div = $('<div/>').attr({ id: 'bg_div_id', 'class': 'LargeBold' }).css(css_obj);
        div.append($('<img border="0" src="./image/piller/ajs2a.png" width="600" height="600"/>'));
      } else {
        css_obj = { position: 'absolute', top: 10, left: 10, width: 600, height: 600, align: 'center', 'z-index': 0 };
        div = $('<div/>').attr({ id: 'bg_div_id', 'class': 'LargeBold' }).css(css_obj);
        div.append($('<img border="0" src="./image/piller/ajs1a.png" width="600" height="600"/>'));
      }
    } else {
      css_obj = { position: 'absolute', top: 10, left: 10, width: 785, height: 488, align: 'center', 'z-index': 0 };
      div = $('<div/>').attr({ id: 'bg_div_id', 'class': 'LargeBold' }).css(css_obj);
      div.append($('<img border="0" src="./image/piller/ajs_II.png" width="785" height="490"/>'));
      fragment.appendChild(div[0]);

      css_obj = { position: 'absolute', top: 40, left: 10, width: 785, height: 30, 'text-align': 'center', align: 'center', 'z-index': 1 };
      div = $('<div/>').attr({ 'class': 'XLargeBold' }).css(css_obj);
      div.text(valtable_obj.LOCATION);
      fragment.appendChild(div[0]);

      css_obj = { position: 'absolute', top: 310, left: 260, width: 200, height: 30, 'text-align': 'center', align: 'center', 'z-index': 1 };
      div = $('<div/>').attr({ 'class': 'LargeBold' }).css(css_obj);
      div.text(valtable_obj.MODEL);
    }
    fragment.appendChild(div[0]);

    led = '';
    if (newpanel) {
      //LED Output1
      css_obj = { position: 'absolute', top: 326, left: 409, width: 10, height: 10, 'z-index': 1 };
      div = $('<div/>').attr({ id: 'out1_led_div_id' }).css(css_obj);
      img = $('<img />', { 'src': led }).appendTo(div);
      fragment.appendChild(div[0]);

      //LED Output2
      if (out2avail) {
        css_obj['top'] = 387;
        div = $('<div/>').attr({ id: 'out2_led_div_id' }).css(css_obj);
        img = $('<img />', { 'src': led }).appendTo(div);
        fragment.appendChild(div[0]);
      }

      //LED Horn
      css_obj['top'] = 183;
      css_obj['left'] = 522;
      div = $('<div/>').attr({ id: 'horn_led_div_id' }).css(css_obj);
      img = $('<img />', { 'src': led }).appendTo(div);
      fragment.appendChild(div[0]);

      //LED Reset
      css_obj['top'] = 122;
      div = $('<div/>').attr({ id: 'reset_led_div_id' }).css(css_obj);
      img = $('<img />', { 'src': led }).appendTo(div);
      fragment.appendChild(div[0]);

      //LED Rectifier
      css_obj['top'] = 312;
      css_obj['left'] = 268;
      div = $('<div/>').attr({ id: 'rec_led_div_id' }).css(css_obj);
      img = $('<img />', { 'src': led }).appendTo(div);
      fragment.appendChild(div[0]);

      //LED Inverter
      css_obj['left'] = 339;
      div = $('<div/>').attr({ id: 'inv_led_div_id' }).css(css_obj);
      img = $('<img />', { 'src': led }).appendTo(div);
      fragment.appendChild(div[0]);

      //LED Mains
      css_obj['top'] = 326;
      css_obj['left'] = 202;
      div = $('<div/>').attr({ id: 'mains_led_div_id' }).css(css_obj);
      img = $('<img />', { 'src': led }).appendTo(div);
      fragment.appendChild(div[0]);
    } else {
      css_obj = { position: 'absolute', top: 120, left: 550, width: 14, height: 14, 'z-index': 1 };
      //LED Mains
      div = $('<div/>').attr({ id: 'mains_led_div_id' }).css(css_obj);
      img = $('<img />', { 'src': led }).appendTo(div);
      fragment.appendChild(div[0]);

      //LED Motor
      css_obj['top'] = 174;
      div = $('<div/>').attr({ id: 'motor_led_div_id' }).css(css_obj);
      img = $('<img />', { 'src': led }).appendTo(div);
      fragment.appendChild(div[0]);

      //LED generator
      css_obj['top'] = 227;
      div = $('<div/>').attr({ id: 'gen_led_div_id' }).css(css_obj);
      img = $('<img />', { 'src': led }).appendTo(div);
      fragment.appendChild(div[0]);

      //LED Output
      css_obj['top'] = 280;
      div = $('<div/>').attr({ id: 'out1_led_div_id' }).css(css_obj);
      img = $('<img />', { 'src': led }).appendTo(div);
      fragment.appendChild(div[0]);

      //LED Warn
      css_obj['top'] = 333;
      div = $('<div/>').attr({ id: 'warn_led_div_id' }).css(css_obj);
      img = $('<img />', { 'src': led }).appendTo(div);
      fragment.appendChild(div[0]);
    }

    document.body.appendChild(fragment);

    set_to_bg_panel();
  }

  function set_to_bg_panel() {
    var led, led_state, led2_state;

    if (newpanel) {
      //update LED Output1
      led_state = (valtable_obj.ADR157 >>> 12) & 0xF;
      led = get_led_source("gruen", led_state);
      $('#out1_led_div_id img').attr('src', led);
      //update LED Output2
      led_state = (valtable_obj.ADR157 >>> 8) & 0xF;
      led = get_led_source("gruen", led_state);
      $('#out2_led_div_id img').attr('src', led);
      //update LED Horn
      led_state = (valtable_obj.ADR157 >>> 4) & 0xF;
      led = get_led_source("rot", led_state);
      $('#horn_led_div_id img').attr('src', led);
      //update LED Reset
      led_state = (valtable_obj.ADR157 >>> 0) & 0xF;
      led = get_led_source("rot", led_state);
      $('#reset_led_div_id img').attr('src', led);
      //update LED Rectifier
      led_state = (valtable_obj.ADR158 >>> 12) & 0xF;
      led2_state = (valtable_obj.ADR158 >>> 8) & 0xF;
      led = get_led_source2("rot", "gruen", led2_state, led_state);
      $('#rec_led_div_id img').attr('src', led);
      //update LED Inverter
      led_state = (valtable_obj.ADR158 >>> 4) & 0xF;
      led2_state = (valtable_obj.ADR158 >>> 0) & 0xF;
      led = get_led_source2("rot", "gruen", led2_state, led_state);
      $('#inv_led_div_id img').attr('src', led);
      //update LED Mains
      led_state = (valtable_obj.ADR159 >>> 4) & 0xF;
      led2_state = (valtable_obj.ADR159 >>> 0) & 0xF;
      led = get_led_source2("rot", "gruen", led2_state, led_state);
      $('#mains_led_div_id img').attr('src', led);
    } else {
      //update LED Mains
      led_state = (valtable_obj.ADR154 >>> 4) & 0xF;
      led2_state = (valtable_obj.ADR154 >>> 0) & 0xF;
      led = get_led_source2("rot", "gruen", led_state, led2_state);
      $('#mains_led_div_id img').attr('src', led);

      //update LED Motor
      led_state = (valtable_obj.ADR154 >>> 12) & 0xF;
      led2_state = (valtable_obj.ADR154 >>> 8) & 0xF;
      led = get_led_source2("rot", "gruen", led_state, led2_state);
      $('#motor_led_div_id img').attr('src', led);

      //update LED generator
      led_state = (valtable_obj.ADR155 >>> 4) & 0xF;
      led2_state = (valtable_obj.ADR155 >>> 0) & 0xF;
      led = get_led_source2("rot", "gruen", led_state, led2_state);
      $('#gen_led_div_id img').attr('src', led);

      //update LED output
      led_state = (valtable_obj.ADR155 >>> 12) & 0xF;
      led2_state = (valtable_obj.ADR155 >>> 8) & 0xF;
      led = get_led_source2("rot", "gruen", led_state, led2_state);
      $('#out1_led_div_id img').attr('src', led);

      //update LED warning
      led_state = (valtable_obj.ADR156 >>> 4) & 0xF;
      led2_state = (valtable_obj.ADR156 >>> 0) & 0xF;
      led = get_led_source2("rot", "gruen", led_state, led2_state);
      $('#warn_led_div_id img').attr('src', led);
    }
  }

  function create_mains_panel() {
    var fragment = document.createDocumentFragment(),
      rec_div, tbody, table, tr, td, panel_css;

    //mains div
    if (newpanel)
      panel_css = css_panel_div;
    else
      panel_css = css_oldpanel_div;

    panel_css['visibility'] = 'visible';
    rec_div = $('<div/>').attr({ id: 'rec_div_id', 'class': 'XBold_black' }).css(panel_css);
    panel_css['visibility'] = 'hidden';

    table = $('<table/>', PANEL_TABLE_STYLE).css(CSS_PANEL_TABLE);
    tbody = $('<tbody />');
    tr = $('<tr />');
    td = $('<td />').css(merge_options(FIRST_OF_FIVE_COLUM_STYLE, css_unit)).append('IN:').appendTo(tr);
    td = $('<td />', { id: 'rec_kva_id' }).css(css_val).appendTo(tr);
    td = $('<td />').css(css_unit).append(' kVA').appendTo(tr);
    td = $('<td />', { id: 'rec_infreq_id' }).css(css_val).appendTo(tr);
    td = $('<td />').css(css_unit).append(' Hz').appendTo(tr);
    tr.appendTo(tbody);

    tr = $('<tr />');
    td = $('<td />').css(css_unit).append('L1').appendTo(tr);
    td = $('<td />', { id: 'rec_inv_1_id' }).css(css_val).appendTo(tr);
    td = $('<td />').css(css_unit).append(' V').appendTo(tr);
    td = $('<td />', { id: 'rec_curr_1_id' }).css(css_val).appendTo(tr);
    td = $('<td />').css(css_unit).append(' A').appendTo(tr);
    tr.appendTo(tbody);

    tr = $('<tr />');
    td = $('<td />').css(css_unit).append('L2').appendTo(tr);
    td = $('<td />', { id: 'rec_inv_2_id' }).css(css_val).appendTo(tr);
    td = $('<td />').css(css_unit).append(' V').appendTo(tr);
    td = $('<td />', { id: 'rec_curr_2_id' }).css(css_val).appendTo(tr);
    td = $('<td />').css(css_unit).append(' A').appendTo(tr);
    tr.appendTo(tbody);

    tr = $('<tr />');
    td = $('<td />').css(css_unit).append('L3').appendTo(tr);
    td = $('<td />', { id: 'rec_inv_3_id' }).css(css_val).appendTo(tr);
    td = $('<td />').css(css_unit).append(' V').appendTo(tr);
    td = $('<td />', { id: 'rec_curr_3_id' }).css(css_val).appendTo(tr);
    td = $('<td />').css(css_unit).append(' A').appendTo(tr);
    tr.appendTo(tbody);
    tbody.appendTo(table);
    table.appendTo(rec_div);

    fragment.appendChild(rec_div[0]);
    document.body.appendChild(fragment);

    set_mains_panel();
  }

  function set_mains_panel() {
    $('#rec_kva_id').text(toCurrResolution(valtable_obj.ADR63));
    $('#rec_infreq_id').text(valtable_obj.INFREQ[0].toFixed(1));
    $('#rec_inv_1_id').text(mains_volt(1));
    $('#rec_curr_1_id').text(toCurrResolution(valtable_obj.ADR68));
    $('#rec_inv_2_id').text(mains_volt(2));
    $('#rec_curr_2_id').text(toCurrResolution(valtable_obj.ADR69));
    $('#rec_inv_3_id').text(mains_volt(3));
    $('#rec_curr_3_id').text(toCurrResolution(valtable_obj.ADR6A));
  }

  function create_inverter_panel() {
    var fragment = document.createDocumentFragment(),
      inv_div, tbody, table, tr, td, panel_css;

    if (newpanel)
      panel_css = css_panel_div;
    else
      panel_css = css_oldpanel_div;

    //inverter div
    inv_div = $('<div/>').attr({ id: 'inv_div_id', 'class': 'XBold_black' }).css(panel_css);

    table = $('<table/>', PANEL_TABLE_STYLE).css(CSS_PANEL_TABLE);
    tbody = $('<tbody />');
    tr = $('<tr />');
    td = $('<td />').css(merge_options(FIRST_OF_FIVE_COLUM_STYLE, css_unit)).append(i18n('INV') + ':').appendTo(tr);
    td = $('<td />', { id: 'inv_kva_id' }).css(css_val).appendTo(tr);
    td = $('<td />').css(css_unit).append(' kVA').appendTo(tr);
    td = $('<td />', { id: 'inv_infreq_id' }).css(css_val).appendTo(tr);
    td = $('<td />').css(css_unit).append(' Hz').appendTo(tr);
    tr.appendTo(tbody);
    tr = $('<tr />');
    td = $('<td />').css(css_unit).append('L1').appendTo(tr);
    td = $('<td />', { id: 'inv_inv_1_id' }).css(css_val).appendTo(tr);
    td = $('<td />').css(css_unit).append(' V').appendTo(tr);
    td = $('<td />', { id: 'inv_curr_1_id' }).css(css_val).appendTo(tr);
    td = $('<td />').css(css_unit).append(' A').appendTo(tr);
    tr.appendTo(tbody);
    tr = $('<tr />');
    td = $('<td />').css(css_unit).append('L2').appendTo(tr);
    td = $('<td />', { id: 'inv_inv_2_id' }).css(css_val).appendTo(tr);
    td = $('<td />').css(css_unit).append(' V').appendTo(tr);
    td = $('<td />', { id: 'inv_curr_2_id' }).css(css_val).appendTo(tr);
    td = $('<td />').css(css_unit).append(' A').appendTo(tr);
    tr.appendTo(tbody);
    tr = $('<tr />');
    td = $('<td />').css(css_unit).append('L3').appendTo(tr);
    td = $('<td />', { id: 'inv_inv_3_id' }).css(css_val).appendTo(tr);
    td = $('<td />').css(css_unit).append(' V').appendTo(tr);
    td = $('<td />', { id: 'inv_curr_3_id' }).css(css_val).appendTo(tr);
    td = $('<td />').css(css_unit).append(' A').appendTo(tr);
    tr.appendTo(tbody);
    tbody.appendTo(table);
    table.appendTo(inv_div);
    fragment.appendChild(inv_div[0]);
    document.body.appendChild(fragment);

    set_inverter_panel();
  }

  function set_inverter_panel() {
    $('#inv_kva_id').text(toCurrResolution(valtable_obj.ADR6C));
    $('#inv_infreq_id').text(toFreqResolution(valtable_obj.ADR6D));
    $('#inv_inv_1_id').text(inv_volt(1));
    $('#inv_curr_1_id').text(toCurrResolution(valtable_obj.ADR71));
    $('#inv_inv_2_id').text(inv_volt(2));
    $('#inv_curr_2_id').text(toCurrResolution(valtable_obj.ADR72));
    $('#inv_inv_3_id').text(inv_volt(3));
    $('#inv_curr_3_id').text(toCurrResolution(valtable_obj.ADR73));
  }

  function create_output1_panel() {
    var fragment = document.createDocumentFragment(),
      out1_div, tbody, table, tr, td, panel_css;

    //out1 div
    if (newpanel)
      panel_css = css_panel_div;
    else
      panel_css = css_oldpanel_div;
    out1_div = $('<div/>').attr({ id: 'out1_div_id', 'class': 'XBold_black' }).css(panel_css);

    table = $('<table/>', PANEL_TABLE_STYLE).css(CSS_PANEL_TABLE);
    tbody = $('<tbody />');
    tr = $('<tr />');
    if (1 == valtable_obj.ADR99 && 1 != valtable_obj.ADRA8) {
      td = $('<td />').css(merge_options(FIRST_OF_FIVE_COLUM_STYLE, css_unit)).append(i18n('OUT1:')).appendTo(tr);
    } else {
      td = $('<td />').css(merge_options(FIRST_OF_FIVE_COLUM_STYLE, css_unit)).append(i18n('OUT:')).appendTo(tr);
    }
    td = $('<td />', { id: 'out1_kva_id' }).css(css_val).appendTo(tr);
    td = $('<td />').css(css_unit).append(' kVA').appendTo(tr);
    td = $('<td />', { id: 'out1_freq_id' }).css(css_val).appendTo(tr);
    td = $('<td />').css(css_unit).append(' Hz').appendTo(tr);
    tr.appendTo(tbody);
    tr = $('<tr />');
    td = $('<td />').css(css_unit).append('L1').appendTo(tr);
    td = $('<td />', { id: 'out1_v_1_id' }).css(css_val).appendTo(tr);
    td = $('<td />').css(css_unit).append(' V').appendTo(tr);
    td = $('<td />', { id: 'out1_curr_1_id' }).css(css_val).appendTo(tr);
    td = $('<td />').css(css_unit).append(' A').appendTo(tr);
    tr.appendTo(tbody);
    tr = $('<tr />');
    td = $('<td />').css(css_unit).append('L2').appendTo(tr);
    td = $('<td />', { id: 'out1_v_2_id' }).css(css_val).appendTo(tr);
    td = $('<td />').css(css_unit).append(' V').appendTo(tr);
    td = $('<td />', { id: 'out1_curr_2_id' }).css(css_val).appendTo(tr);
    td = $('<td />').css(css_unit).append(' A').appendTo(tr);
    tr.appendTo(tbody);
    tr = $('<tr />');
    td = $('<td />').css(css_unit).append('L3').appendTo(tr);
    td = $('<td />', { id: 'out1_v_3_id' }).css(css_val).appendTo(tr);
    td = $('<td />').css(css_unit).append(' V').appendTo(tr);
    td = $('<td />', { id: 'out1_curr_3_id' }).css(css_val).appendTo(tr);
    td = $('<td />').css(css_unit).append(' A').appendTo(tr);
    tr.appendTo(tbody);
    tbody.appendTo(table);
    table.appendTo(out1_div);
    fragment.appendChild(out1_div[0]);
    document.body.appendChild(fragment);

    set_output1_panel();
  }

  function set_output1_panel() {
    $('#out1_kva_id').text(toCurrResolution(valtable_obj.ADR74));
    $('#out1_freq_id').text(toFreqResolution(valtable_obj.ADR75));
    $('#out1_v_1_id').text(out1_volt(1));
    $('#out1_curr_1_id').text(toCurrResolution(valtable_obj.ADR79));
    $('#out1_v_2_id').text(out1_volt(2));
    $('#out1_curr_2_id').text(toCurrResolution(valtable_obj.ADR7A));
    $('#out1_v_3_id').text(out1_volt(3));
    $('#out1_curr_3_id').text(toCurrResolution(valtable_obj.ADR7B));
  }

  function create_output2_panel() {
    var fragment = document.createDocumentFragment(),
      out2_div, tbody, table, tr, td, panel_css;

    //out2 div
    if (newpanel)
      panel_css = css_panel_div;
    else
      panel_css = css_oldpanel_div;
    out2_div = $('<div/>').attr({ id: 'out2_div_id', 'class': 'XBold_black' }).css(panel_css);

    table = $('<table/>', PANEL_TABLE_STYLE).css(CSS_PANEL_TABLE);
    tbody = $('<tbody />');
    tr = $('<tr />');
    td = $('<td />').css(merge_options(FIRST_OF_FIVE_COLUM_STYLE, css_unit)).append(i18n('OUT2:')).appendTo(tr);
    td = $('<td />', { id: 'out2_kva_id' }).css(css_val).appendTo(tr);
    td = $('<td />').css(css_unit).append(' kVA').appendTo(tr);
    td = $('<td />', { id: 'out2_freq_id' }).css(css_val).appendTo(tr);
    td = $('<td />').css(css_unit).append(' Hz').appendTo(tr);
    tr.appendTo(tbody);
    tr = $('<tr />');
    td = $('<td />').css(css_unit).append('L1').appendTo(tr);
    td = $('<td />', { id: 'out2_v_1_id' }).css(css_val).appendTo(tr);
    td = $('<td />').css(css_unit).append(' V').appendTo(tr);
    td = $('<td />', { id: 'out2_curr_1_id' }).css(css_val).appendTo(tr);
    td = $('<td />').css(css_unit).append(' A').appendTo(tr);
    tr.appendTo(tbody);
    tr = $('<tr />');
    td = $('<td />').css(css_unit).append('L2').appendTo(tr);
    td = $('<td />', { id: 'out2_v_2_id' }).css(css_val).appendTo(tr);
    td = $('<td />').css(css_unit).append(' V').appendTo(tr);
    td = $('<td />', { id: 'out2_curr_2_id' }).css(css_val).appendTo(tr);
    td = $('<td />').css(css_unit).append(' A').appendTo(tr);
    tr.appendTo(tbody);
    tr = $('<tr />');
    td = $('<td />').css(css_unit).append('L3').appendTo(tr);
    td = $('<td />', { id: 'out2_v_3_id' }).css(css_val).appendTo(tr);
    td = $('<td />').css(css_unit).append(' V').appendTo(tr);
    td = $('<td />', { id: 'out2_curr_3_id' }).css(css_val).appendTo(tr);
    td = $('<td />').css(css_unit).append(' A').appendTo(tr);
    tr.appendTo(tbody);
    tbody.appendTo(table);
    table.appendTo(out2_div);
    fragment.appendChild(out2_div[0]);
    document.body.appendChild(fragment);

    set_output2_panel();
  }

  function set_output2_panel() {
    $('#out2_kva_id').text(toCurrResolution(valtable_obj.ADR7D));
    $('#out2_freq_id').text(toFreqResolution(valtable_obj.ADR7E));
    $('#out2_v_1_id').text(out2_volt(1));
    $('#out2_curr_1_id').text(toCurrResolution(valtable_obj.ADR82));
    $('#out2_v_2_id').text(out2_volt(2));
    $('#out2_curr_2_id').text(toCurrResolution(valtable_obj.ADR83));
    $('#out2_v_3_id').text(out2_volt(3));
    $('#out2_curr_3_id').text(toCurrResolution(valtable_obj.ADR84));
  }

  function create_default_panel() {
    var fragment = document.createDocumentFragment(),
      def_div, thead, tbody, table, tr, td;
    var panel_css, css_left = { 'text-align': 'left' };

    //default div
    if (newpanel)
      panel_css = css_panel_div;
    else
      panel_css = css_oldpanel_div;
    def_div = $('<div/>').attr({ id: 'def_div_id', 'class': 'XBold_black' }).css(panel_css);

    table = $('<table/>', PANEL_TABLE_STYLE).css(CSS_PANEL_TABLE);
    tbody = $('<tbody />');
    tr = $('<tr />');
    td = $('<td />', { id: 'def_date_id' }).css(css_left).appendTo(tr);
    td = $('<td />', { id: 'def_time_id' }).css(css_val).appendTo(tr);
    tr.appendTo(tbody);
    tr = $('<tr />');
    td = $('<td />', { id: 'def_hours_txt_id' }).css(css_left).append(i18n('HOURS ')).appendTo(tr);
    td = $('<td />', { id: 'def_hours_id' }).css(css_val).appendTo(tr);
    tr.appendTo(tbody);
    tr = $('<tr />');
    td = $('<td/>').append("&nbsp;").appendTo(tr);
    tr.appendTo(tbody);
    tr = $('<tr />');
    td = $('<td />').append(i18n('OUTPUT')).css(css_left).appendTo(tr);
    td = $('<td />', { id: 'def_output_id' }).css(css_val).appendTo(tr);

    tr.appendTo(tbody);
    tbody.appendTo(table);
    table.appendTo(def_div);

    fragment.appendChild(def_div[0]);
    document.body.appendChild(fragment);

    set_default_panel();
  }

  function set_default_panel() {
    var date, time, day, month, year, second, hour, minute;

    day = get2DigNum(valtable_obj.ADR184);
    month = get2DigNum(valtable_obj.ADR185) - 1;
    year = valtable_obj.ADR186;
    second = get2DigNum(valtable_obj.ADR181);
    minute = get2DigNum(valtable_obj.ADR182);
    hour = get2DigNum(valtable_obj.ADR183);

    var d = new Date(Date.UTC(year, month, day, hour, minute, second));
    var options = { year: 'numeric', month: 'numeric', day: 'numeric' };
    var loc = 'de-DE';
    date = d.toLocaleDateString(loc, options);
    //    date = utils_api.formatDate(new Date(year, month - 1, day), "dd.MMM.yyyy");
    time = hour + ":" + minute + ":" + second;
    $('#def_date_id').text(date);
    $('#def_time_id').text(time);
    $('#def_hours_id').text((valtable_obj.ADR9B * 10000.0 + valtable_obj.ADR9C).toFixed(0) + ' H');
    if (valtable_obj.ADRC1 != 9) {
      $('#def_output_id').text(valtable_obj.ADR7C.toFixed(0) + ' kW');
    } else {
      $('#def_output_id').text(valtable_obj.ADR7C.toFixed(1) + ' kW');
    }
  }

  function create_dc_panel() {
    var fragment = document.createDocumentFragment();
    var css_left = { 'text-align': 'left' };
    var panel_css, loc_css = {
      'text-align': 'left',
      'vertical-align': 'top',
      'width': '70px'
    };

    //dc div
    if (newpanel)
      panel_css = css_panel_div;
    else
      panel_css = css_oldpanel_div;
    def_div = $('<div/>').attr({ id: 'dc_div_id', 'class': 'XBold_black' }).css(panel_css);

    table = $('<table/>', PANEL_TABLE_STYLE).css(CSS_PANEL_TABLE);
    tbody = $('<tbody />');
    tr = $('<tr />');
    td = $('<td />', { id: 'dc_txt_id' }).css(loc_css).append('OUT DC:').appendTo(tr);
    td = $('<td />').append('&nbsp;').appendTo(tr);
    loc_css['text-align'] = 'right';
    delete loc_css.width;
    td = $('<td />', { id: 'dc_kw_id' }).css(loc_css).appendTo(tr);

    loc_css['text-align'] = 'left';
    loc_css.width = '25px';
    td = $('<td />').css(loc_css).append(' kW').appendTo(tr);
    tr.appendTo(tbody);
    tr = $('<tr />').appendTo(tbody);
    tr = $('<tr />');

    loc_css['text-align'] = 'right';
    delete loc_css.width;
    td = $('<td />', { id: 'dc_v_id' }).css(loc_css).appendTo(tr);
    loc_css['text-align'] = 'left';
    td = $('<td />').css(loc_css).append(' V').appendTo(tr);

    loc_css['text-align'] = 'right';
    td = $('<td />', { id: 'dc_a_id' }).css(loc_css).appendTo(tr);
    loc_css['text-align'] = 'left';
    td = $('<td />').css(loc_css).append(' A').appendTo(tr);
    tr.appendTo(tbody);
    tbody.appendTo(table);
    table.appendTo(def_div);

    fragment.appendChild(def_div[0]);
    document.body.appendChild(fragment);

    set_dc_panel();
  }

  function set_dc_panel() {
    $('#dc_kw_id').text((valtable_obj.ADR97 / 10).toFixed(1));
    $('#dc_a_id').text((valtable_obj.ADR96).toFixed(0));
    $('#dc_v_id').text((valtable_obj.ADR95 / 10).toFixed(1));
  }

  function create_energy_meter_panel() {
    var fragment = document.createDocumentFragment();
    var css_left = { 'text-align': 'left' };
    var panel_css, loc_css = {
      'text-align': 'center',
      'vertical-align': 'top'
    };

    //energy meter div
    if (newpanel)
      panel_css = css_panel_div;
    else
      panel_css = css_oldpanel_div;
    def_div = $('<div/>').attr({ id: 'kwh_div_id', 'class': 'XBold_black' }).css(panel_css);

    table = $('<table/>', PANEL_TABLE_STYLE).css(CSS_PANEL_TABLE);
    tbody = $('<tbody />');
    tr = $('<tr />');
    td = $('<td />', { id: 'kwh_txt_id', colspan: 2 }).css(loc_css).append('**  ' + i18n('ENERGY METER') + '  **').appendTo(tr);
    tr.appendTo(tbody);
    tr = $('<tr />');
    loc_css['text-align'] = 'right';
    td = $('<td />', { id: 'kwh_val_id', colspan: 2 }).css(loc_css).appendTo(tr);
    tr.appendTo(tbody);

    tr = $('<tr />');
    loc_css['text-align'] = 'left';
    td = $('<td />', { colspan: 2 }).css(loc_css).append(i18n('Since')).appendTo(tr);
    tr.appendTo(tbody);
    tr = $('<tr />');
    loc_css['text-align'] = 'left';
    td = $('<td />', { id: 'kwh_date_id', colspan: 1 }).css(loc_css).appendTo(tr);
    loc_css['text-align'] = 'right';
    td = $('<td />', { id: 'kwh_time_id', colspan: 1 }).css(loc_css).appendTo(tr);
    tr.appendTo(tbody);
    tbody.appendTo(table);
    table.appendTo(def_div);

    fragment.appendChild(def_div[0]);
    document.body.appendChild(fragment);

    set_energy_meter_panel();
  }

  function set_energy_meter_panel() {
    var day, month, year, hour, minute, second;
    day = get2DigNum(valtable_obj.ADRE7);
    month = get2DigNum(valtable_obj.ADRE8);
    year = valtable_obj.ADRE9;
    hour = get2DigNum(valtable_obj.ADREA);
    minute = get2DigNum(valtable_obj.ADREB);
    second = get2DigNum(valtable_obj.ADREC);
    $('#kwh_val_id').text((valtable_obj.ADRE3 * 10000 + valtable_obj.ADRE4) + ' kWh');
    $('#kwh_date_id').text(day + '.' + month + '.' + year);
    $('#kwh_time_id').text(hour + ':' + minute + ':' + second);
  }

  function create_intermediate_meter_panel() {
    var fragment = document.createDocumentFragment();
    var css_left = { 'text-align': 'left' };
    var panel_css, loc_css = {
      'text-align': 'center',
      'vertical-align': 'top'
    };

    //intermediate meter div
    if (newpanel)
      panel_css = css_panel_div;
    else
      panel_css = css_oldpanel_div;
    def_div = $('<div/>').attr({ id: 'kwh2_div_id', 'class': 'XBold_black' }).css(panel_css);
    table = $('<table/>', PANEL_TABLE_STYLE).css(CSS_PANEL_TABLE);
    tbody = $('<tbody />');
    tr = $('<tr />');
    td = $('<td />', { id: 'kwh2_txt_id', colspan: 2 }).css(loc_css).append('**  ' + i18n('INTERMEDIATE METER') + '  **').appendTo(tr);
    tr.appendTo(tbody);
    tr = $('<tr />');
    loc_css['text-align'] = 'right';
    td = $('<td />', { id: 'kwh2_val_id', colspan: 2 }).css(loc_css).appendTo(tr);
    tr.appendTo(tbody);

    tr = $('<tr />');
    loc_css['text-align'] = 'left';
    td = $('<td />', { colspan: 2 }).css(loc_css).append(i18n('Since')).appendTo(tr);
    tr.appendTo(tbody);
    tr = $('<tr />');
    loc_css['text-align'] = 'left';
    td = $('<td />', { id: 'kwh2_date_id', colspan: 1 }).css(loc_css).appendTo(tr);
    loc_css['text-align'] = 'right';
    td = $('<td />', { id: 'kwh2_time_id', colspan: 1 }).css(loc_css).appendTo(tr);
    tr.appendTo(tbody);
    tbody.appendTo(table);
    table.appendTo(def_div);

    fragment.appendChild(def_div[0]);
    document.body.appendChild(fragment);

    set_intermediate_meter_panel();
  }

  function set_intermediate_meter_panel() {
    var day, month, year, hour, minute, second;
    day = get2DigNum(valtable_obj.ADRF2);
    month = get2DigNum(valtable_obj.ADRF3);
    year = valtable_obj.ADRF4;
    hour = get2DigNum(valtable_obj.ADRF5);
    minute = get2DigNum(valtable_obj.ADRF6);
    second = get2DigNum(valtable_obj.ADRF7);
    $('#kwh2_val_id').text((valtable_obj.ADREE * 10000 + valtable_obj.ADREF) + ' kWh');
    $('#kwh2_date_id').text(day + '.' + month + '.' + year);
    $('#kwh2_time_id').text(hour + ':' + minute + ':' + second);
  }

  function create_select_panel() {
    var fragment = document.createDocumentFragment(),
      sel_up_div, sel_down_div, img;

    if (newpanel) {
      css_obj = {
        position: 'absolute',
        top: 120,
        left: 429,
        width: 48,
        height: 48,
        'background-color': 'transparent',
        align: 'center',
        visibility: 'visible',
        overflow: 'hidden',
        cursor: 'pointer'
      };
      //sel_up div
      sel_up_div = $('<div/>').attr({ id: 'sel_up_div_id' }).css(css_obj);
      img = $('<img />', { 'src': "./image/piller/selectUpAPOCONV.png" }).appendTo(sel_up_div);

      sel_up_div.on("click",function () { selection_up(); });
      //sel_down div
      css_obj['top'] = 181;
      sel_down_div = $('<div/>').attr({ id: 'sel_down_div_id' }).css(css_obj);
      img = $('<img />', { 'src': "./image/piller/selectDownAPOCONV.png" }).appendTo(sel_down_div);
      sel_down_div.on("click",function () { selection_down(); });
    } else {
      css_obj = {
        position: 'absolute',
        top: 383,
        left: 387,
        width: 31,
        height: 22,
        'background-color': 'transparent',
        align: 'center',
        visibility: 'visible',
        overflow: 'hidden',
        cursor: 'pointer'
      };
      //sel_up div
      sel_up_div = $('<div/>').attr({ id: 'sel_up_div_id' }).css(css_obj);
      img = $('<img />', { 'src': "./image/piller/selectUpAJR2.gif" }).appendTo(sel_up_div);
      sel_up_div.on("click",function () { selection_up(); });
      //sel_down div
      css_obj['left'] = 453;
      sel_down_div = $('<div/>').attr({ id: 'sel_down_div_id' }).css(css_obj);
      img = $('<img />', { 'src': "./image/piller/selectDownAJR2.gif" }).appendTo(sel_down_div);
      sel_down_div.on("click",function () { selection_down(); });
    }

    fragment.appendChild(sel_up_div[0]);
    fragment.appendChild(sel_down_div[0]);
    document.body.appendChild(fragment);
  }

  function selection_up() {
    ++curr_select;
    if (curr_select >= panels_id.length)
      curr_select = 0;

    // compare documentation MODBUS_Piller_v12_130611.docx Page 7
    // -> if(1==valtable_obj.ADR99 && 1!=valtable_obj.ADRA8){/*show OUTPUT2!!!*/}
    if ((1 != valtable_obj.ADR99 || 1 == valtable_obj.ADRA8) &&
      ('#out2_div_id' == panels_id[curr_select])) {
      ++curr_select;
      if (curr_select >= panels_id.length)
        curr_select = 0;
    }
    if ((6 != valtable_obj.ADRA5) &&
      ('#dc_div_id' == panels_id[curr_select])) {
      ++curr_select;
      if (curr_select >= panels_id.length)
        curr_select = 0;
    }
    if ((1 != valtable_obj.ADRED && 1 != valtable_obj.ADREE) &&
      ('#kwh_div_id' == panels_id[curr_select])) {
      ++curr_select;
      if (curr_select >= panels_id.length)
        curr_select = 0;
    }
    if ((1 != valtable_obj.ADRF8) &&
      ('#kwh2_div_id' == panels_id[curr_select])) {
      ++curr_select;
      if (curr_select >= panels_id.length)
        curr_select = 0;
    }
    select_panel();
  }

  function selection_down() {
    --curr_select;
    if (curr_select < 0)
      curr_select = panels_id.length - 1;

    if ((1 != valtable_obj.ADRF8) &&
      ('#kwh2_div_id' == panels_id[curr_select])) {
      --curr_select;
      if (curr_select < 0)
        curr_select = panels_id.length - 1;
    }

    if ((1 != valtable_obj.ADRED && 1 != valtable_obj.ADREE) &&
      ('#kwh_div_id' == panels_id[curr_select])) {
      --curr_select;
      if (curr_select < 0)
        curr_select = panels_id.length - 1;
    }    

    if ((6 != valtable_obj.ADRA5) &&
      ('#dc_div_id' == panels_id[curr_select])) {
      --curr_select;
      if (curr_select < 0)
        curr_select = panels_id.length - 1;
    }  

    if ((1 != valtable_obj.ADR99 || 1 == valtable_obj.ADRA8) &&
      ('#out2_div_id' == panels_id[curr_select])) {
      --curr_select;
      if (curr_select < 0)
        curr_select = panels_id.length - 1;
    }    

    select_panel();
  }

  function select_panel(panel_nr) {
    if (curr_select >= panels_id.length) curr_select = panels_id.length - 1;
    if (curr_select < 0) curr_select = 0;
    $.each(panels_id, function (i, el) { $(el).css('visibility', 'hidden'); });
    $(panels_id[curr_select]).css('visibility', 'visible');
  }

  function create_footer_pane() {
    if (!oid || !unms_obj)
      return true;
    if (newpanel) {
      footer_api.create({ //Attr
          id: 'panel2'
        }, { //css
          position: 'absolute',
          top: '640px'
        },
        unms_obj
      );
    } else {
      footer_api.create({ //Attr
          id: 'panel2'
        }, { //css
          position: 'absolute',
          top: '470px',
          left: '80px'
        },
        unms_obj
      );
    }
  }

  function set_footer() {
    if (!oid || !unms_obj)
      return true;
    footer_api.set(unms_obj);
  }
}(jQuery));