/*
    bacs2_panel - Bacs 2 screen.
    Copyright (c) 2011, Generex GmbH. All rights reserved.
    Said
    Last change: 15.08.2013
*/

var bacs_api = (function ($) {
  var bacs_obj = null,
    unms_obj = null,
    cb = null,
    err_div = null,
    module_tables = [],
    current_sort = [],
    sort_dir = [],
    pannes_created = false,
    check_bowser = true,
    temp_unit = '&#176;C',
    bacs_oid = get_bacs_oid(),
    bacs_impc_baseline_alarm = 1,
    bacs_impc_baseline_warning = 2,
    bacs_state_string_voltage_high = 0x00200000,
    bacs_state_string_voltage_low = 0x00400000,
    bacs_state_string_voltage_warning_high = 0x00800000,
    bacs_state_string_voltage_warning_low = 0x01000000,
    bacs_alarm2_current_sensor_high = 0x0001,
    bacs_alarm2_current_sensor_low = 0x0002,
    bacs_alarm2_current_sensor_warn_high = 0x0004,
    bacs_alarm2_current_sensor_warn_low = 0x0008,
    bacs_alarm2_equalizing_system_disable = 0x0010,
    bacs_alarm2_current_deviation_discharge = 0x0020;
    bacs_alarm2_current_deviation_charge = 0x0040;
    bacs_alarm2_current_deviation_warn_discharge = 0x0080;
    bacs_alarm2_current_deviation_warn_charge = 0x0100;
    bacs_alarm_modrev_incompatible = 0x8000;

  var req_params = null;

  /*public functions*/
  return {
    init: init
  };

  function init() {
    if (check_bowser && !utils_api.check_bowser()) {
      $('#loading_id').css('display', 'none');
      panes_api.create_br_check_panel(this, browser_check_ok);
      return true;
    }
    create_err_msg_panel();

    req_params = utils_api.get_req_params(utils_api.device_bacs, bacs_oid);
    cb = jqmodule.subscribe(this, { update: update, error: err_funct }, req_params);
  }

  function browser_check_ok() {
    check_bowser = false;
    init();
  }

  function create_err_msg_panel() {
    css_obj = { position: 'absolute', top: 10, left: 10, width: 724, display: 'none', 'z-index': 10 };
    err_div = panes_api.create_err_panel({ id: 'error_div', 'class': 'alert alert-danger' }, css_obj);
  }

  function err_funct(response) {
    jqmodule.refresh(cb, 10 * 1000);
    panes_api.display_err({ response: response, oid: bacs_oid, err_div: err_div, err_msg_div_id: 'error_msg_div' });
  }

  function get_bacs_oid() {
    return get_oid_from_url();
  }

  function update(data) {
    if (data.bacs2 == 0) {
      return;
    }

    bacs_obj = data.bacs2;
    bacs_obj.systemValues.displayName = bacs_obj.systemValues.displayName || "BACS";
    bacs_obj.systemValues.location = bacs_obj.systemValues.location || "";

    if (err_div)
      err_div.css('display', 'none');

    if (bacs_obj.unms != null) {
      unms_obj = bacs_obj.unms;
      unms_obj.fetchtime = unms_obj.service.fetchtime;
      unms_obj.dll_path = '../../';
      unms_obj.html_path = '../../../';
      unms_obj.img_path = '../../../image/';
      unms_obj.device_type = 'BACS';
      unms_obj._upsProtocol = bacs_obj.systemValues._upsProtocol;
      info_head_api.update(unms_obj);
    }

    jqmodule.refresh(cb, bacs_obj.report.refresh * 1000); //Millisekunden
    if (bacs_obj.systemValues.initModulesActive == 1) {
      pannes_created = false;
      create_init_state_pane(); // create modules init state pane
      return;
    } else // remove modules init state pane
    {
      var init_div = $('#init_div_id');
      if (init_div != null)
        init_div.remove();
    }

    if (bacs_obj.systemValues.bMetricSystem != null && bacs_obj.systemValues.bMetricSystem == 0)
      temp_unit = '&#176;F';

    bacs_obj.systemValues.mac = bacs_obj.systemValues.mac || "";
    bacs_obj.systemValues.ip = bacs_obj.systemValues.ip || "";


    if (!pannes_created) {
      create_panes();
      pannes_created = true;
      return;
    }
    set_bacs_alarm_warn_state_panel(); //set bacs alarm warning and state values
    set_time_server_warning_value(); //create time_server_warning panel
    set_bacs_error_hw(); //set Error: BACS type HW Revision mismatch
    set_strings_values(); //set strings values
    set_miscellaneous_values(); //set Miscellaneous values
    set_footer();
  }

  function create_init_state_pane() // create modules init state pane
  {
    var fragment = document.createDocumentFragment(),
      init_div, tr, td, tbody, table, h, title, string_colspan, title_txt;

    title_txt = bacs_obj.systemValues.displayName;
    if (!bacs_obj.systemValues.location.empty()) {
      title_txt += " - " + bacs_obj.systemValues.location;
    }

    init_div = $('#init_div_id');
    if (init_div != null)
      init_div.remove(); // remove modules init state pane

    init_div = $('<div/>', { id: 'init_div_id', align: 'center' });
    table = $('<table/>', { 'class': 'table-condensed', align: 'center' }).appendTo(init_div);
    tbody = $('<tbody />').appendTo(table);

    tr = $('<tr />');
    td = $('<td />').appendTo(tr);
    h = $('<h3 />').append($('<span />', { 'class': 'label label-info' }).append(title_txt));
    h.appendTo(td);
    tr.appendTo(tbody);
    tr = $('<tr />').append($('<td />').append($('<h4 />').append(i18n(lang_Loading_BACS) + '...')));
    tr.appendTo(tbody);

    $('<br />').appendTo(init_div);

    table = $('<table/>', { id: 'init_table_id', 'class': 'table-condensed', align: 'center' });
    table.appendTo(init_div);
    tbody = $('<tbody />').appendTo(table);

    if (bacs_obj.moduleInitStatus != null)
      $.each(bacs_obj.moduleInitStatus, (function (i, string) {
        string_colspan = string.modules.length >= 6 ? 6 : string.modules.length;

        /*tr = $('<tr />').appendTo(tbody);
        tr.append($('<td />', {colspan:string_colspan, align:'center'}).append($('<span />', {'class':'label label-info'})
                    .append('String '+ string.string)));*/


        tr = $('<tr />').appendTo(tbody);
        td = $('<td />', { colspan: string_colspan, align: 'center' }).appendTo(tr);
        h = $('<h4 />').append($('<span />', { 'class': 'label label-info' }).append('String ' + string.name));
        h.appendTo(td);


        $.each(string.modules, (function (j, module) {
          if ((j % 6) == 0)
            tr = $('<tr />').appendTo(tbody);

          title = "String" + string.string + " Module" + module.module + " :";
          if (module.initialised == 0)
            title += lang_Not_Initialised;
          else if (module.initialised == 1)
            title += lang_Initialised;
          else
            title += lang_Unreachable_Module;
          td = $('<td />', { width: 30 }).append($('<span />', { 'class': get_init_module_class(module.initialised), title: title }).css('display', 'block').append(getJarString(module.module)));
          tr.append(td);
        }));
        tr = $('<tr />').appendTo(tbody);
        $('<td />').append($('<br />')).appendTo(tr);
      }));

    fragment.appendChild(init_div[0]);
    document.body.appendChild(fragment);
  };

  function getJarString(moduleNo) {
    var nJars = bacs_obj.systemValues.JarsPerModule,
        jarString = "";
    
    if (undefined == nJars) {
      return moduleNo; //hack: use this as default, if value in json is wrong.
    }

    if (1 >= nJars) {
      jarString = moduleNo;
    } else {
      var i = moduleNo * nJars - (nJars - 1);
      for (var j = 0; j < nJars; j++) {
        if (0 < j) {
          jarString += "|"
        }
        jarString += (i + j);
      }
    }
    return jarString;
  };

  function get_init_module_class(initialised) {
    if (initialised == 0)
      return 'label label-default';
    else if (initialised == 1)
      return 'label label-success';
    else
      return 'label label-danger';
  };

  function create_panes() {
    create_bacs_title_panel(); //bacs title panel
    create_bacs_alarm_warn_state_panel(); //create bacs alarm warning and state panel
    create_time_server_warning_panel(); //create time_server_warning panel
    create_strings_panes(); //create strings panes
    create_sys_infos_panel(); //create system infos panel
    create_footer_pane();
    update_alarmTT();
  }

  function update_alarmTT() {
    var tt_span = $('#sysWarnTT');
    var tt = i18n("indicates a problem with the") + " " + bacs_obj.systemValues.displayName + " " +
      i18n("wiring or a bad") + " " + bacs_obj.systemValues.displayName + " " + i18n("configuration") + ".";
    tt_span.empty().text(tt);
    tt_span = $('#sysAlarmTT');
    tt_span.empty().text(tt);
  }

  function create_bacs_title_panel() //bacs title panel
  {
    var title_div, tr, table, tbody, title_txt;

    title_txt = bacs_obj.systemValues.displayName;
    if (!bacs_obj.systemValues.location.empty()) {
      title_txt += " - " + bacs_obj.systemValues.location;
    }

    title_div = $('<div/>', { id: 'bacs_title_id', align: 'center' });
    tbody = $('<tbody />');
    table = $('<table/>', { 'class': 'table-condensed', align: 'center' });
    $(table).css('margin-top', '4px');

    tr = $('<tr />');
    td = $('<td />').appendTo(tr);
    h = $('<h3 />').append($('<span />', { 'class': 'label label-info' }).append(title_txt));
    h.appendTo(td);
    tr.appendTo(tbody);
    tbody.appendTo(table);
    table.appendTo(title_div);
    $(title_div).appendTo(document.body);
  }

  function create_bacs_alarm_warn_state_panel() //create bacs alarm warning and state panel
  {
    var fragment = document.createDocumentFragment(),
      alarm_div, tr, td, table, tbody, alarm_txt, warn_txt, status_txt;

    alarm_div = $('<div/>', { id: 'div_alarm_id', align: 'center' });

    table = $('<table/>', { id: 'table_alarm_id', 'class': 'table-condensed', align: 'center' });
    table.appendTo(alarm_div);
    tbody = $('<tbody />').appendTo(table);

    alarm_txt = get_alarm_txt(bacs_obj.systemValues.alarm, bacs_obj.systemValues.status, bacs_obj.systemValues.impc_baseline_alarm); //get Alarm text
    alarm_txt = appendAlarmText(alarm_txt, get_alarm2_txt(bacs_obj.systemValues.alarm2));
    if (alarm_txt.length > 0) {
      tr = $('<tr />', { id: 'tr_alarm_id' }).appendTo(tbody);
      td = $('<td />').appendTo(tr);
      $('<img />').attr({ id: 'alarm_img_id', width: 16, height: 16, src: './image/bacs2/b_red.gif' }).appendTo(td);
      tr.append($('<td />', { align: 'left' }).append(i18n(lang_bacs_alarm) + ": " + i18n(alarm_txt)));
    }

    warn_txt = get_warn_txt(bacs_obj.systemValues.alarm, bacs_obj.systemValues.status, bacs_obj.systemValues.impc_baseline_alarm); //get warnings text
    warn_txt = appendAlarmText(warn_txt, get_alarm2_warn_txt(bacs_obj.systemValues.alarm2));
    if (warn_txt.length > 0) {
      tr = $('<tr />', { id: 'tr_warn_id' });
      td = $('<td />');
      $('<img />').attr({ id: 'warn_img_id', width: 16, height: 16, src: './image/bacs2/b_brightorange.gif' }).appendTo(td);
      td.appendTo(tr);
      tr.append($('<td />', { align: 'left' }).append(i18n(lang_bacs_warning) + ": " + i18n(warn_txt)));
      tr.appendTo(tbody);
    }

    status_txt = get_status_txt(bacs_obj.systemValues.status); //get status text
    tr = $('<tr />', { id: 'tr_status_id' });
    td = $('<td />');
    $('<img />').attr({ id: 'status_img_id', width: 16, height: 16, src: get_status_img(bacs_obj.systemValues.status) }).appendTo(td);
    td.appendTo(tr);
    tr.append($('<td />', { align: 'left' }).append(i18n(lang_bacs_status) + ": " + i18n(status_txt)));
    tr.appendTo(tbody);
    create_bacs_error_hw(tbody); //Error: BACS type HW Revision mismatch
    fragment.appendChild(alarm_div[0]);
    document.body.appendChild(fragment);
  }

  function set_bacs_alarm_warn_state_panel() //set bacs alarm warning and state values
  {
    var alarm_div, tr, td, alarm_txt, warn_txt, status_txt, baseline_alarm_txt;

    alarm_txt = get_alarm_txt(bacs_obj.systemValues.alarm, bacs_obj.systemValues.status, bacs_obj.systemValues.impc_baseline_alarm); //get Alarm text
    alarm_txt = appendAlarmText(alarm_txt, get_alarm2_txt(bacs_obj.systemValues.alarm2));
    if (alarm_txt.length > 0) {
      if ($('#tr_alarm_id').children().length > 0) {
        $('#tr_alarm_id').find('td:eq(1)').text(i18n(lang_bacs_alarm) + ": " + i18n(alarm_txt));
      } else {
        tr = $('<tr />', { id: 'tr_alarm_id' });
        td = $('<td />');
        $('<img />').attr({ id: 'alarm_img_id', width: 16, height: 16, src: './image/bacs2/b_red.gif' }).appendTo(td);
        td.appendTo(tr);
        tr.append($('<td />').append(i18n(lang_bacs_alarm) + ": " + i18n(alarm_txt)));
        if ($('#tr_warn_id').children().length > 0)
          tr.insertBefore($('#tr_warn_id'));
        else
          tr.insertBefore($('#tr_status_id'));
      }
    } else {
      $('#tr_alarm_id').remove();
    }

    warn_txt = get_warn_txt(bacs_obj.systemValues.alarm, bacs_obj.systemValues.status, bacs_obj.systemValues.impc_baseline_alarm); //get warnings text
    warn_txt = appendAlarmText(warn_txt, get_alarm2_warn_txt(bacs_obj.systemValues.alarm2));
    if (warn_txt.length > 0) {
      if ($('#tr_warn_id').children().length > 0) {
        $('#tr_warn_id').find('td:eq(1)').text(i18n(lang_bacs_warning) + ": " + i18n(warn_txt));
      } else {
        tr = $('<tr />', { id: 'tr_warn_id' });
        td = $('<td />');
        $('<img />').attr({ id: 'warn_img_id', width: 16, height: 16, src: './image/bacs2/b_brightorange.gif' }).appendTo(td);
        td.appendTo(tr);
        tr.append($('<td />').append(i18n(lang_bacs_warning) + ": " + i18n(warn_txt)));
        tr.insertBefore($('#tr_status_id'));
      }
    } else {
      $('#tr_warn_id').remove();
    }

    status_txt = get_status_txt(bacs_obj.systemValues.status); //get status text
    $('#tr_status_id').find('td:eq(1)').text(i18n(lang_bacs_status) + ": " + i18n(status_txt));
    $('#status_img_id').attr({ src: get_status_img(bacs_obj.systemValues.status) });
  }

  function create_bacs_error_hw(tbody) //Error: BACS type HW Revision mismatch
  {
    var tr, td;
    var hw_error_created = 0;

    $.each(bacs_obj.strings, (function (i, string) {
      $.each(string.modules, (function (j, module) {
        if (((module.alarm & bacs_alarm_modrev_incompatible) != 0) && (hw_error_created == 0)) {
          tr = $('<tr />', { id: 'hw_revision_mismatch_id' });
          td = $('<td />');
          $('<img />').attr({ id: 'hw_err_img_id', width: 16, height: 16, src: './image/bacs2/b_blue.gif' }).appendTo(td);
          td.appendTo(tr);
          tr.append($('<td />').append(i18n(lang_Bacs_Error_HW_Revision_mismatch)));
          tr.appendTo(tbody);
          hw_error_created = 1;
        }
      }));
    }));
  }

  function set_bacs_error_hw() //set Error: BACS type HW Revision mismatch
  {
    var tr = $('#hw_revision_mismatch_id');
    var hw_error_created = 0;

    $.each(bacs_obj.strings, (function (i, string) {
      $.each(string.modules, (function (j, module) {
        if (((module.alarm & bacs_alarm_modrev_incompatible) != 0) && (hw_error_created == 0)) {
          if (tr.children().length > 0) {
            tr.find('td:eq(1)').text(i18n(lang_Bacs_Error_HW_Revision_mismatch));
            hw_error_created = 1;
          } else {
            tr = $('<tr />', { id: 'hw_revision_mismatch_id' });
            td = $('<td />');
            $('<img />').attr({ id: 'hw_err_img_id', width: 16, height: 16, src: './image/bacs2/b_blue.gif' }).appendTo(td);
            td.appendTo(tr);
            tr.append($('<td />').append(i18n(lang_Bacs_Error_HW_Revision_mismatch)));
            tr.insertAfter($('#tr_status_id'));
            hw_error_created = 1;
          }
        }
      }));
    }));

    if (hw_error_created == 0)
      tr.remove();
  }

  function create_time_server_warning_panel() //create time_server_warning panel
  {
    var time_server_div = $('<div/>', { id: 'time_server_warning_id', align: 'center' }),
      tr, td, table, tbody;

    table = $('<table/>', { id: 'tb_time_server_warning_id', 'class': 'table-condensed', align: 'center' });
    table.appendTo(time_server_div);
    tbody = $('<tbody />').appendTo(table);
    tr = $('<tr />').append($('<td />').append(i18n(lang_time_server_warning))).appendTo(tbody);

    time_server_div.appendTo(document.body);
    if (bacs_obj.systemValues.time_server_warning == "0")
      time_server_div.hide();
  }

  function set_time_server_warning_value() //create time_server_warning panel
  {
    if (bacs_obj.systemValues.time_server_warning == "1")
      $('#time_server_warning_id').show("slow");
    else
      $('#time_server_warning_id').hide("slow");
  }

  function create_strings_panes() //create strings panes
  {
    var strings_div, tr, td, table, tbody, tfoot, module_table, span,
      current = null,
      currentAC = "n.a",
      color = "black";

    $('<br />').appendTo(document.body);
    strings_div = $('<div/>', { id: 'strings_div_id', align: 'center', overflow: 'hidden' });

    table = $('<table/>', { id: 'strings_panes_id', 'class': 'table-condensed' });
    tbody = $('<tbody />').appendTo(table);
    tfoot = $('<tfoot />', { 'class': 'active' }).css({ 'width': '50%', border: '1px solid #dddddd' }).appendTo(table);

    tr = $('<tr />').appendTo(tbody);
    $.each(bacs_obj.strings, (function (i, string) {
      module_table = get_string_panel(string);
      td = $('<td />').append(module_table);
      td.appendTo(tr);
      module_tables.push(module_table);

      if (current == null) //Sum Current
      {
        current = string.current;
        color = get_string_current_color(string.alarm2);
        if (current == -1 && bacs_obj.systemValues.status & 0x100000) // sensor_lost
          current = "n.a";
      }
      if ("n.a" == currentAC) {
        if (undefined != string.currentAC) {
          currentAC = string.currentAC;
        }
        color = get_string_current_color(string.alarm2);
        if (-1 == currentAC && bacs_obj.systemValues.status & 0x100000) { // sensor_lost
          currentAC = "n.a";
        }
      }
    }));

    if (bacs_obj.systemValues.current_sensor_attached === '2' && bacs_obj.systemValues.string_count > 1) {
      tr = $('<tr />').appendTo(tfoot);
      td = $('<td />', { align: 'center', colspan: bacs_obj.systemValues.string_count }).appendTo(tr);

      if (current !== "n.a") {
        span = $('<span />', { id: 'strings_sum_current' })
        .css('color', color)
        .append(" " + i18n("Bacs CS Sensor Total Current") + ": " + current + " [A]").appendTo(td);
      } else {
        span = $('<span />', { id: 'strings_sum_current' }).
        append(" " + i18n("Bacs CS Sensor Total Current") + ": " + current).appendTo(td);
      }
      if ("n.a" !== currentAC) {
        tr = $('<tr />').appendTo(tfoot);
        td = $('<td />', { align: 'center', colspan: bacs_obj.systemValues.string_count }).appendTo(tr);
        span = $('<span />', { id: 'strings_sum_currentAC' })
        .css('color', color)
        .append(" " + i18n("Bacs CS Sensor Total AC Current") + ": " + currentAC + " [A]").appendTo(td);
      }
    }

    table.appendTo(strings_div);
    strings_div.appendTo(document.body);
    $('<br />').appendTo(document.body);
  }

  function get_string_panel(string) //get string panel
  {
    var alarm, equalizing, current, temperature, voltage, impedance, init, id,
      thead, th, tr, td, table, tbody, tfoot, img, alarm_alt, string_voltage_txt,
      sum_voltage = 0, //Summe String Voltage
      act_power = 0, //active Power
      equalizing_img_w = 26,
      hardware_version = parseFloat(bacs_obj.systemValues.hardware_version),
      baseline_impc_flags,
      currentAC = "n.a";

    table = $('<table/>', {
      'class': 'tablesorter table table-condensed table-striped table-hover table-bordered',
      id: "id_table_String_" + string.string,
      align: 'center'
    }).css('margin-bottom', '4px');
    tbody = $('<tbody />').appendTo(table);
    tfoot = $('<tfoot />', { 'class': 'active' }).appendTo(table);

    current = string.current;
    if (current == -1 && bacs_obj.systemValues.status & 0x100000) // sensor_lost
    {
      current = "n.a";
    }
    if (undefined != string.currentAC) {
      currentAC = string.currentAC;
    }
    if (-1 == currentAC && bacs_obj.systemValues.status & 0x100000) { // sensor_lost!
      currentAC = "n.a";
    }
    thead = $('<thead />').appendTo(table);

    tr = $('<tr />').appendTo(thead);

    td = $('<td />', { colspan: 6, id: "id_String_name_" + string.string, 'class': get_string_class(string) })
    .css('text-align', 'center').append("String " + string.name).appendTo(tr);

    tr = $('<tr />').appendTo(thead);
    tr.append($('<th />', { 'class': 'header' }).append("No.<br></br>"));



    tr.append($('<th />', { 'class': 'header' }).append(i18n("Volt.") + "</br>&nbsp;&nbsp;[V]")
    .on({
      mouseover: function (event) {
        var tooltipX = event.pageX - 8,
          tooltipY = event.pageY + 8;
        $('#bacs_volt_hdr').css({ top: tooltipY, left: tooltipX, display: 'block' });
      },
      mouseout: function (event) {
        $('#bacs_volt_hdr').css({ display: 'none' });
      }
    }));


    tr.append($('<th />', { 'class': 'header' }).append(i18n("Temp.") + "</br>&nbsp;&nbsp;[" + temp_unit + "]")
    .on({
      mouseover: function (event) {
        var tooltipX = event.pageX - 8,
          tooltipY = event.pageY + 8;
        $('#bacs_temp_hdr').css({ top: tooltipY, left: tooltipX, display: 'block' });
      },
      mouseout: function (event) {
        $('#bacs_temp_hdr').css({ display: 'none' });
      }
    }));


    if (bacs_obj.systemValues.has_impc == 1) {
      tr.append($('<th />', { 'class': 'header' }).append(i18n("Ri.") + "</br>[m&#937;]")
      .on({
        mouseover: function (event) {
          var tooltipX = event.pageX - 8,
            tooltipY = event.pageY + 8;
          $('#bacs_resist_hdr').css({ top: tooltipY, left: tooltipX, display: 'block' });
        },
        mouseout: function (event) {
          $('#bacs_resist_hdr').css({ display: 'none' });
        }
      }));
    }

    if (bacs_obj.systemValues.eq_active == 1)
      tr.append($('<th />', { 'id': 'id_string_equalizing_' + string.string, 'class': 'header' })
      .append("Equalize<br></br>")
      .on({
        mouseover: function (event) {
          var tooltipX = event.pageX - 8,
            tooltipY = event.pageY + 8;
          $('#bacs_bp_hdr').css({ top: tooltipY, left: tooltipX, display: 'block' });
        },
        mouseout: function (event) {
          $('#bacs_bp_hdr').css({ display: 'none' });
        }
      }));


    tr.append($('<th />', { 'class': 'header' }).append("Status<br></br>")
    .on({
      mouseover: function (event) {
        var tooltipX = event.pageX - 8,
          tooltipY = event.pageY + 8;
        $('#bacs_state_hdr').css({ top: tooltipY, left: tooltipX, display: 'block' });
      },
      mouseout: function (event) {
        $('#bacs_state_hdr').css({ display: 'none' });
      }
    }));

    $.each(string.modules, (function (i, module) {
      alarm = module.alarm;
      init = module.init; //TODO Fehlt !!!!
      baseline_impc_flags = module.impc_baseline_alarm;
      tr = $('<tr />').append($('<td />', { id: 'id_module_nr_' + string.string + "_" + module.module, align: 'center' }).append(getJarString(module.module)));

      if (module.impedance != null) {
        impedance = module.impedance.toFixed(2);

        if (module.impedance == -1)
          impedance = "n.a";
        else if (bacs_obj.systemValues.module_type == "C40") {
          if (module.impedance < -2 + .1) //do not compare == -2.0
            impedance = ">6.00";
          if (module.impedance < -3 + .1) // means -3
            impedance = "<0.02";
        }
      }
      if (module.temperature != null) {
        if (module.temperature == -1)
          temperature = "n.a";
        else
          temperature = module.temperature.toFixed(1);
      }
      if (module.voltage != null) {
        if (module.voltage == -1)
          voltage = "n.a";
        else
          voltage = module.voltage.toFixed(2);
      }
      if (module.voltage != null) {
        tr.append($('<td />', { id: 'id_module_volt_' + string.string + "_" + module.module, align: 'right' }).css('color', get_volt_color(alarm)).append(voltage));
      }
      if (module.temperature != null) {
        tr.append($('<td />', { id: 'id_module_tmp_' + string.string + "_" + module.module, align: 'center' }).css('color', get_temp_color(alarm)).append(temperature));
      }

      if (module.impedance != null) {
        if (bacs_obj.systemValues.has_impc == 1)
          tr.append($('<td />', { id: 'id_module_imp_' + string.string + "_" + module.module, align: 'center' }).css('color', get_impd_color(alarm, baseline_impc_flags)).append(impedance));
      }
      if (module.equalizing != null) {
        if (bacs_obj.systemValues.eq_active == 1) {
          equalizing = module.equalizing;
          if (module.equalizing == -1) {
            equalizing = "n.a";
          }
          td = $('<td />', { align: 'center' }).appendTo(tr);

          img = $('<img />').attr({
            id: 'id_equalizing_img_' + string.string + '_' + module.module,
            alt: equalizing,
            width: get_equalizing_img_width(module.equalizing, module.alarm),
            height: 16,
            src: get_equalizing_img_src(module.equalizing, module.alarm),
            border: 0
          });
          td.append(img);
          create_equalize_tt(string, module);
          img.on({
            mouseover: function (event) {
              var tooltipX = event.pageX - 8,
                tooltipY = event.pageY + 8,
                id = 'id_eq_module_' + module.module + '_' + string.string;
              $('#' + id).css({ top: tooltipY, left: tooltipX, display: 'block' });
            },
            mouseout: function (event) {
              var id = 'id_eq_module_' + module.module + '_' + string.string;
              $('#' + id).css({ display: 'none' });
            }
          });
        }
      }
      if (module.voltage != null) {
        if ((module.voltage != -1) && ((module.alarm & 2) == 0))
          sum_voltage += module.voltage; //Summe String Voltage
      }

      if (alarm == 0)
        alarm_alt = 1;
      else if (get_alarm_txt(alarm, string.status, baseline_impc_flags).length > 0 ||
        get_alarm2_txt(string.alarm2).length > 0)
        alarm_alt = 3;
      else if (get_warn_txt(alarm, string.status, baseline_impc_flags).length > 0 ||
        get_alarm2_warn_txt(string.alarm2).length > 0)
        alarm_alt = 2;
      else
        alarm_alt = 0;

      td = $('<td />', { align: 'center' }).appendTo(tr);
      create_state_tt(string, module);

      img = $('<img />').attr({
        id: 'id_state_img_' + string.string + '_' + module.module,
        alt: alarm_alt,
        width: 16,
        height: 16,
        src: get_state_img_src(alarm, init, baseline_impc_flags, string.alarm2),
        'border': 0
      });
      img.appendTo(td);
      img.on({
        mouseover: function (event) {
          var tooltipX = event.pageX - 8,
            tooltipY = event.pageY + 8,
            id = 'id_state_module_' + module.module + '_' + string.string;
          $('#' + id).css({ top: tooltipY, left: tooltipX, display: 'block' });
        },
        mouseout: function (event) {
          var id = 'id_state_module_' + module.module + '_' + string.string;
          $('#' + id).css({ display: 'none' });
        }
      });
      tr.appendTo(tbody);
    }));

    //footer
    if (hardware_version >= 3.0) //
    {
      var voltVal;
      tr = $('<tr />').appendTo(tfoot);
      td = $('<td />', { colspan: 6 }).appendTo(tr);
      voltVal = $('<span />', { id: 'id_String_sum_voltage_val_' + string.string }).css('color', get_string_volt_color(string.status)).append(sum_voltage.toFixed(2));
      span = $('<span />', { id: "id_String_sum_voltage_" + string.string }).append("&sum; " + i18n("Voltage") + " ").append(voltVal).append(' V').appendTo(td);
    }

    if (string.targetVolt != null) {
      string_voltage_txt = bacs_obj.systemValues.eq_active == 1 ? lang_Target_Voltage : lang_Average_Voltage;
      tr = $('<tr />').appendTo(tfoot);
      td = $('<td />', { colspan: 6 }).appendTo(tr);
      span = $('<span />', { id: "id_String_voltage_" + string.string }).
      append(parseFloat(string.targetVolt).toFixed(2) + '&nbsp;[V]&nbsp;' + i18n(string_voltage_txt)).appendTo(td);
    }

    if (bacs_obj.systemValues.current_sensor_attached === '1' ||
      (bacs_obj.systemValues.current_sensor_attached === '2' && bacs_obj.systemValues.string_count == 1)) {
      if (current !== "n.a")
        act_power = ((sum_voltage * current) / 1000).toFixed(2); //active Power
      else
        act_power = "n.a";

      tr = $('<tr />').appendTo(tfoot);
      td = $('<td />', { colspan: 6 }).appendTo(tr);

      if (current != "n.a") {
        span = $('<span />', { id: 'id_String_current_power_' + string.string })
        .css('color', get_string_current_color(string.alarm2))
        .append(current + "&nbsp;[A]&nbsp;" + i18n("DC Current") + "&nbsp;&nbsp;&nbsp;&nbsp;" + act_power + "&nbsp;[KW]&nbsp;" + i18n("Real Power") + " ")
        .appendTo(td);
      } else {
        span = $('<span />', { id: 'id_String_current_power_' + string.string })
        .append(current + "&nbsp;&nbsp;" + i18n("DC Current") + "&nbsp;&nbsp;&nbsp;&nbsp;" + act_power + "&nbsp;&nbsp;" + i18n("Real Power") + " ")
        .appendTo(td);
      }
      span.on({
        mouseover: function (event) {
          var tooltipX = event.pageX - 8,
            tooltipY = event.pageY + 8;
          $('#bacs_str_current').css({ top: tooltipY, left: tooltipX, display: 'block' });
        },
        mouseout: function (event) {
          $('#bacs_str_current').css({ display: 'none' });
        }
      });

      if ("n.a" != currentAC) {
        tr = $('<tr />').appendTo(tfoot);
        td = $('<td />', { colspan: 6 }).appendTo(tr);
        span = $('<span/>', { id: 'id_String_currentAC_power_' + string.string })
        .css('color', get_string_current_color(string.alarm2))
        .append(currentAC + "&nbsp;[A]&nbsp;" + i18n("AC Current"))
        .appendTo(td);
      }
    }

    $('thead th', table).each(function (index) {
      var thead = $(this);
      var dir = 0;

      //thead.unbind('click');
      thead.on("click", function (event) {
        current_sort = [];
        if (sort_dir[index] == -1 || sort_dir[index] == 1)
          dir = 0;
        else if (sort_dir[index] == 0)
          dir = 1;
        sort_dir = [-1, -1, -1, -1, -1, -1];
        sort_dir[index] = dir;
        current_sort.push(index);
        current_sort.push(dir);
        if (module_tables.length > 1)
          sort_module_tables(); //sort module tables
        // stop normal event by returning false
        return false;
      });
    });

    if (bacs_obj.systemValues.has_impc == 1)
      table.tablesorter({ headers: { 4: { sorter: 'img_sort' }, 5: { sorter: 'img_sort' } } });
    else
      table.tablesorter({ headers: { 3: { sorter: 'img_sort' }, 4: { sorter: 'img_sort' } } });

    /*table.data("tablesorter").widgets = ["zebra"];
    // there is a method to apply widgets!
    table.trigger("applyWidgets");*/
    $('tbody td', table).css('text-align', 'center');
    return table;
  }

  function sort_module_tables() //sort module tables
  {
    if (current_sort.length > 0) {
      $.each(module_tables, (function (i, table) {
        table.trigger('update');
        table.get(0).config.sortList = [current_sort];
        table.trigger('reSort');
      }));
    }
  }

  function set_strings_values() //set strings values
  {
    var current = null,
      currentAC = "n.a",
      color = "black";
    $.each(bacs_obj.strings, (function (i, string) {
      set_string_values(string);

      if (current == null) //Sum Current
      {
        current = string.current;
        color = get_string_current_color(string.alarm2);
        if (current == -1 && bacs_obj.systemValues.status & 0x100000) // sensor_lost
          current = "n.a";
      }
      if ("n.a" == currentAC) {
        if (undefined != string.currentAC) {
          currentAC = string.currentAC;
        }
        // color = get_string_current_color(string.alarm2);
        if (-1 == currentAC && bacs_obj.systemValues.status & 0x100000) // sensor_lost
          currentAC = "n.a";
      }
    }));

    if (bacs_obj.systemValues.current_sensor_attached === '2' && bacs_obj.systemValues.string_count > 1) {
      if (current !== "n.a") {
        $('#strings_sum_current')
        .css('color', color)
        .html(" " + i18n("Bacs CS Sensor Total Current") + ": " + current + " [A]");
      } else {
        $('#strings_sum_current').html(" " + i18n("Bacs CS Sensor Total Current") + ": " + current);
      }
      $('#string_sum_currentAC')
      .css('color', color)
      .html(" " + i18n("Bacs CS Sensor Total AC Current") + ": " + currentAC + " [A]");
    }

    //sort module tables
    if (navigator.userAgent.indexOf("Chrome") != -1)
      window.setTimeout(function () { sort_module_tables(); }, 3000);
    else
      sort_module_tables();
  }

  function set_string_values(string) //set string values
  {
    var alarm, equalizing, current, impedance, temperature, voltage, init, string_voltage_txt,
      module_id, id_String_name, alarm_alt,
      sum_voltage = 0, //Summe String Voltage
      act_power = 0, //active Power
      hardware_version = parseFloat(bacs_obj.systemValues.hardware_version),
      baseline_impc_flags,
      currentAC = "n.a";

    current = string.current;
    if (current == -1 && bacs_obj.systemValues.status & 0x100000) // sensor_lost
      current = "n.a";
    if (undefined != string.currentAC) {
      currentAC = string.currentAC;
    }
    if (-1 == currentAC && bacs_obj.systemValues.status & 0x100000) { // sensor_lost
      currentAC = "n.a";
    }

    id_String_name = "#id_String_name_" + string.string;
    $(id_String_name).text('String ' + string.name);
    $(id_String_name).attr('class', get_string_class(string));

    $.each(string.modules, (function (i, module) {
      alarm = module.alarm;
      init = module.init;
      baseline_impc_flags = module.impc_baseline_alarm;

      if (module.equalizing != null) {
        if (bacs_obj.systemValues.eq_active == 1) {
          equalizing = module.equalizing;
          if (module.equalizing == -1) {
            equalizing = "n.a";
          }
          $('#id_equalizing_img_' + string.string + '_' + module.module).
          attr({ src: get_equalizing_img_src(module.equalizing, module.alarm), alt: equalizing, width: get_equalizing_img_width(module.equalizing, module.alarm) });
          set_equalize_tt(string, module);
        }
      }

      if (alarm == 0)
        alarm_alt = 1;
      else if (get_alarm_txt(alarm, undefined, baseline_impc_flags).length > 0 ||
        get_alarm2_txt(string.alarm2).length > 0)
        alarm_alt = 3;
      else if (get_warn_txt(alarm, undefined, baseline_impc_flags).length > 0 ||
        get_alarm2_warn_txt(string.alarm2).length > 0)
        alarm_alt = 2;
      else
        alarm_alt = 0;

      if (module.impedance != null) {
        impedance = module.impedance.toFixed(2);

        if (module.impedance == -1)
          impedance = "n.a";
        else if (bacs_obj.systemValues.module_type == "C40") {
          if (module.impedance < -2 + .1) //do not compare == -2.0
            impedance = ">6.00";
          if (module.impedance < -3 + .1) // means -3
            impedance = "<0.02";
        }

        if (bacs_obj.systemValues.has_impc == 1)
          $('#id_module_imp_' + string.string + "_" + module.module).css('color', get_impd_color(alarm, baseline_impc_flags)).text(impedance);
      }

      if (module.temperature != null) {
        if (module.temperature == -1)
          temperature = "n.a";
        else
          temperature = module.temperature.toFixed(1);
        $('#id_module_tmp_' + string.string + "_" + module.module).css('color', get_temp_color(alarm)).text(temperature);
      }

      if (module.voltage != null) {
        if (module.voltage == -1)
          voltage = "n.a";
        else {
          voltage = module.voltage.toFixed(2);
          sum_voltage += module.voltage;
        }
        $('#id_module_volt_' + string.string + "_" + module.module).css('color', get_volt_color(alarm)).text(voltage);
      }

      $('#id_module_nr_' + string.string + "_" + module.module).text(getJarString(module.module));

      $('#id_state_img_' + string.string + '_' + module.module).attr({ src: get_state_img_src(alarm, init, baseline_impc_flags, string.alarm2), alt: alarm_alt });
      set_state_tt(string, module);
    }));


    string_voltage_txt = bacs_obj.systemValues.eq_active == 1 ? lang_Target_Voltage : lang_Average_Voltage;

    $('#id_String_voltage_' + string.string).
    html(parseFloat(string.targetVolt).toFixed(2) + '&nbsp;[V]&nbsp;' + string_voltage_txt);

    if (bacs_obj.systemValues.current_sensor_attached === '1' ||
      (bacs_obj.systemValues.current_sensor_attached === '2' && bacs_obj.systemValues.string_count == 1)) {
      if (current !== "n.a")
        act_power = ((sum_voltage * current) / 1000).toFixed(2); //active Power
      else
        act_power = "n.a";

      if (current != "n.a") {
        $('#id_String_current_power_' + string.string)
        .css('color', get_string_current_color(string.alarm2))
        .html(current + "&nbsp;[A]&nbsp;" + i18n("DC Current") + "&nbsp;&nbsp;&nbsp;&nbsp;" + act_power +
          "&nbsp;[KW]&nbsp;" + i18n("Real Power") + " ");
      } else {
        $('#id_String_current_power_' + string.string).html(current + "&nbsp;&nbsp;" + i18n("DC Current") + "&nbsp;&nbsp;&nbsp;&nbsp;" + act_power +
          "&nbsp;&nbsp;" + i18n("Real Power") + " ");
      }
      $('#id_String_currentAC_power_' + string.string)
      .css('color', get_string_current_color(string.alarm2))
      .html(currentAC + "&nbsp;[A]&nbsp;" + i18n("AC Current"))
    }

    if (hardware_version >= 3.0) //Summe String Voltage
    {
      var voltVal;
      voltVal = $('<span />', { id: 'id_String_sum_voltage_val_' + string.string }).css('color', get_string_volt_color(string.status)).append(sum_voltage.toFixed(2));
      $('#id_String_sum_voltage_' + string.string).empty().append("&sum; " + i18n("Voltage") + " ").append(voltVal).append(' V');
    }

  }

  function create_sys_infos_panel() //create system infos panel
  {
    var tr, th, table, tbody, thead, css = { 'border-left-width': '2px' };
    $('<br />').appendTo(document.body);
    table = $('<table/>', { id: 'sys_infos_id', 'class': 'table table-condensed table-striped table-hover table-bordered', align: 'center', width: '80%' }); //
    tbody = $('<tbody />').appendTo(table);
    thead = $('<thead />').appendTo(table);
    tr = $('<tr />').appendTo(thead);

    th = ($('<th />', { colspan: 2 }).append(lang_module_info)).appendTo(tr);
    th = ($('<th />', { colspan: 2 }).css(css).append(lang_batt_info)).appendTo(tr);
    th = ($('<th />', { colspan: 2 }).css(css).append(lang_contact_info)).appendTo(tr);
    th = ($('<th />', { colspan: 2 }).css(css).append(lang_level_info)).appendTo(tr);
    th = ($('<th />', { colspan: 2 }).css(css).append(lang_miscellaneous)).appendTo(tr);

    // row1
    tr = $('<tr />').append($('<td />', { align: 'left' }).append(i18n(lang_module_type))).append($('<td />', { align: 'left' }). // col1
      append(bacs_obj.systemValues.module_type));

    tr.append($('<td />', { align: 'left' }).css(css).append(lang_manufacturer)).append($('<td />', { align: 'left' }). // col2
      append(bacs_obj.systemValues.manufacturer));

    tr.append($('<td />', { align: 'left' }).css(css).append(i18n(lang_contact_person))).append($('<td />', { align: 'left' }). // col3
      append(bacs_obj.systemValues.contact_person));

    tr.append($('<td />', { align: 'left' }).css(css).append(i18n(lang_voltage_low_high))).append($('<td />', { align: 'left' }). // col4
      append(bacs_obj.systemValues.volt_low_alarm + " V/" + bacs_obj.systemValues.volt_high_alarm + " V"));

    tr.append($('<td />', { align: 'left' }).css(css).append(i18n("Firmware Version"))).append($('<td />', { align: 'left' }). // col5
      append(bacs_obj.systemValues.firmware_version));

    tr.appendTo(tbody);

    // row2
    tr = $('<tr />').append($('<td />', { align: 'left' }).append(i18n(lang_hardware_version))).append($('<td />', { align: 'left' }). // col1
      append(bacs_obj.systemValues.hardware_version));

    tr.append($('<td />', { align: 'left' }).css(css).append(i18n(lang_type))).append($('<td />', { align: 'left' }). // col2
      append(bacs_obj.systemValues.battery_technology));

    tr.append($('<td />', { align: 'left' }).css(css).append(i18n(lang_phone_number))).append($('<td />', { align: 'left' }). // col3
      append(bacs_obj.systemValues.phone_number));

    tr.append($('<td />', { align: 'left' }).css(css).append(i18n(lang_temp_low_high))).append($('<td />', { align: 'left' }). // col4
      append(bacs_obj.systemValues.temp_low_alarm + temp_unit + '/' + bacs_obj.systemValues.temp_high_alarm + temp_unit));

    tr.append($('<td />', { align: 'left' }).css(css).append(i18n(lang_discharge_counter))).append($('<td />', { id: 'id_infos_discharge_counter', align: 'left' }). // col5
      append(bacs_obj.systemValues.discharge_counter));

    tr.appendTo(tbody);

    // row 3
    tr = $('<tr />').append($('<td />', { align: 'left' }).append(i18n(lang_software_version))).append($('<td />', { align: 'left' }). // col1
      append(bacs_obj.systemValues.software_version));

    tr.append($('<td />', { align: 'left' }).css(css).append(i18n(lang_capacity_c_1_0))).append($('<td />', { align: 'left' }). // col2
      append(bacs_obj.systemValues.battery_capacity + " Ah"));

    tr.append($('<td />', { align: 'left' }).css(css).append(i18n('IP Address'))).append($('<td />', { align: 'left' }). // col3
      append(bacs_obj.systemValues.ip));

    tr.append($('<td />', { align: 'left' }).css(css).append(i18n(lang_resistance_min_max))).append($('<td />', { align: 'left' }). // col4
      append(round_max_decimals(Number(bacs_obj.systemValues.impc_low_alarm),2) + " m&#937;/" + round_max_decimals(Number(bacs_obj.systemValues.impc_high_alarm), 2) + " m&#937;"));

    var rippleVoltage = "n.a";
    if (-1 != bacs_obj.systemValues.rippleVoltage) {
      rippleVoltage = bacs_obj.systemValues.rippleVoltage + " mV";
    }
    tr.append($('<td />', { align: 'left' }).css(css).append(i18n(lang_ripple_voltage))).
    append($('<td />', { id: 'id_ripple_voltage', align: 'left' }).append(rippleVoltage)); // col5

    tr.appendTo(tbody);

    // row4
    tr = $('<tr />').append($('<td />', { align: 'left' }).append(i18n(lang_number_of_blocks))).append($('<td />', { align: 'left' }). // col1
      append(bacs_obj.systemValues.module_count));

    tr.append($('<td />', { align: 'left' }).css(css).append(i18n(lang_installation_date))).append($('<td />', { align: 'left' }). // col2
      append(bacs_obj.systemValues.installation_date));

    tr.append($('<td />', { align: 'left' }).css(css).append(i18n('Mac Address'))).append($('<td />', { align: 'left' }). // col3
      append(bacs_obj.systemValues.mac));

    tr.append($('<td />', { align: 'left' }).css(css).append(i18n(lang_equalizing_range_min_max))).append($('<td />', { align: 'left' }). // col4
      append(bacs_obj.systemValues.eq_range_min + " V/" + bacs_obj.systemValues.eq_range_max + " V"));

    tr.append($('<td />', { align: 'left' }).css(css).append(i18n(lang_equalizing_paused))).
    append($('<td />', { id: 'id_equalizing_paused', align: 'left' }).append(bacs_obj.systemValues.eq_paused_sec_remain)); // col5

    tr.appendTo(tbody);

    // row5
    tr = $('<tr />').append($('<td />', { align: 'left' }).append(i18n(lang_number_of_strings))).append($('<td />', { align: 'left' }). // col1
      append(bacs_obj.systemValues.string_count));

    tr.append($('<td />', { align: 'left' }).css(css).append(i18n(lang_location))).append($('<td />', { align: 'left' }). // col2
      append(bacs_obj.systemValues.location));

    tr.append($('<td />').css(css)).append($('<td />')) // col3
    .append($('<td />').css(css)).append($('<td />')) // col4

    if (!bacs_oid) { // col5
      tr.append($('<td />', { align: 'left' }).css(css).append(i18n(lang_page_generated))).append($('<td />', { id: 'id_page_generated', align: 'left' }).append(bacs_obj.systemValues.page_generated));
    } else {
      tr.append($('<td />', { align: 'left' }).css(css).append(i18n(lang_page_generated))).append($('<td />', { id: 'id_page_generated', align: 'left' }).append(bacs_obj.report.time));
    }

    tr.appendTo(tbody);

    table.appendTo(document.body);
  }

  function set_miscellaneous_values() //set Miscellaneous values
  {
    var rippleVoltage = "n.a";

    if (-1 != bacs_obj.systemValues.rippleVoltage) {
      rippleVoltage = bacs_obj.systemValues.rippleVoltage + " mV";
    }
    $('#id_ripple_voltage').text(rippleVoltage);

    $('#id_infos_discharge_counter').text(bacs_obj.systemValues.discharge_counter);

    $('#id_equalizing_paused').text(bacs_obj.systemValues.eq_paused_sec_remain);

    if (!bacs_oid) {
      $('#id_page_generated').text(bacs_obj.systemValues.page_generated);
    } else {
      $('#id_page_generated').text(bacs_obj.report.time);
    }
  }

  function get_alarm_txt(alarm, status, baseline_impc) //get Alarm text
  {
    var alarm_txt = "",
      array = new Array();
    
    if (-1 == baseline_impc) {
      baseline_impc = 0; //hack: suppress baseline_impc value, if value in json is wrong.
    }

    if ((alarm & 1) == 1)
      array.push(i18n(lang_general_alarm));
    if ((alarm & 2) == 2)
      array.push(i18n(lang_comm_lost));
    if ((alarm & 4) == 4)
      array.push(i18n(lang_voltage_high));
    if ((alarm & 8) == 8)
      array.push(i18n(lang_voltage_low));
    if ((alarm & 16) == 16)
      array.push(i18n(lang_temp_high));
    if ((alarm & 32) == 32)
      array.push(i18n(lang_temp_low));
    if ((alarm & 64) == 64)
      array.push(i18n(lang_impedance_high));
    if ((alarm & 128) == 128)
      array.push(i18n(lang_impedance_low));
    if ((alarm & 256) == 256)
      array.push(i18n(lang_equalizing_error));
    if (typeof status != 'undefined') {
      if ((status & 0x40000) == 0x40000)
        array.push(i18n(lang_batt_breaker_open));
      if ((status & 0x80000) == 0x80000)
        array.push((lang_thermal_runaway));
      if ((status & 0x100000) == 0x100000)
        array.push(i18n(lang_sensor_lost));
      if (status & bacs_state_string_voltage_high)
        array.push(i18n("High String Voltage"));
      if (status & bacs_state_string_voltage_low)
        array.push(i18n("Low String Voltage"));
    }
    if ((baseline_impc & bacs_impc_baseline_alarm) == bacs_impc_baseline_alarm) {
      array.push(i18n("Baseline Impedance"));
    }

    if (array.length > 0)
      alarm_txt = array.join(", ");
    return alarm_txt;
  }

  function get_warn_txt(alarm, status, baseline_impc) //get warnings text
  {
    var warn_txt = "",
      array = new Array();
      
    if (-1 == baseline_impc) {
      baseline_impc = 0; //hack: suppress baseline_impc value, if value in json is wrong.
    }

    if ((alarm & 512) == 512)
      array.push(i18n(lang_voltage_high));
    if ((alarm & 1024) == 1024)
      array.push(i18n(lang_voltage_low));
    if ((alarm & 2048) == 2048)
      array.push(i18n(lang_temp_high));
    if ((alarm & 4096) == 4096)
      array.push(i18n(lang_temp_low));
    if ((alarm & 8192) == 8192)
      array.push(i18n(lang_impedance_high));
    if ((alarm & 16384) == 16384)
      array.push(i18n(lang_impedance_low));
    if (typeof status != 'undefined') {
      if ((status & 0x20000) == 0x20000)
        array.push(i18n(lang_volt_diff_high));
      if (0 == (status & bacs_state_string_voltage_high)) {
        if (status & bacs_state_string_voltage_warning_high)
          array.push(i18n("High String Voltage"));
      }
      if (0 == (status & bacs_state_string_voltage_low)) {
        if (status & bacs_state_string_voltage_warning_low)
          array.push(i18n("Low String Voltage"));
      }
    }
    if ((baseline_impc & bacs_impc_baseline_warning) == bacs_impc_baseline_warning && (baseline_impc & bacs_impc_baseline_alarm) == 0) {
      array.push(i18n("Baseline Impedance"));
    }

    if (array.length > 0)
      warn_txt = array.join(", ");
    return warn_txt;
  }

  function int_get_alarm2_txt(alarm2, msgArr) {
    var warn_txt = "",
      array = new Array();

    if (-1 == alarm2) {
      alarm2 = 0; // hack: suppress alarm2 value, if value in json is wrong.
    }

    for (var i = 0; i < msgArr.length; ++i) {
      if (alarm2 & msgArr[i][0]) {
        array.push(msgArr[i][1]);
      }
    }

    if (array.length > 0) {
      warn_txt = array.join(", ");
    }
    return warn_txt;
  }

  function get_alarm2_txt(alarm2) {
    var alarmTxt = [
      [bacs_alarm2_current_sensor_high, i18n(lang_bacs_alarm2_current_sensor_high)],
      [bacs_alarm2_current_sensor_low, i18n(lang_bacs_alarm2_current_sensor_low)],
      [bacs_alarm2_current_deviation_discharge, i18n(lang_bacs_alarm2_current_deviation_discharge)],
      [bacs_alarm2_current_deviation_charge, i18n(lang_bacs_alarm2_current_deviation_charge)]
    ];
    return int_get_alarm2_txt(alarm2, alarmTxt);
  }

  function get_alarm2_warn_txt(alarm2) {
    var alarmTxt = [
      [bacs_alarm2_current_sensor_warn_high, i18n(lang_bacs_alarm2_current_sensor_warn_high)],
      [bacs_alarm2_current_sensor_warn_low, i18n(lang_bacs_alarm2_current_sensor_warn_low)],
      [bacs_alarm2_equalizing_system_disable, i18n(lang_bacs_alarm2_equalizing_cutoff)],
      [bacs_alarm2_current_deviation_warn_discharge, i18n(lang_bacs_alarm2_current_deviation_warn_discharge)],
      [bacs_alarm2_current_deviation_warn_charge, i18n(lang_bacs_alarm2_current_deviation_warn_charge)]
    ];
    return int_get_alarm2_txt(alarm2, alarmTxt);
  }

  function appendAlarmText(alarm_txt, appendText) {
    if (0 === alarm_txt.length) {
      return appendText;
    }
    if (0 === appendText.length) {
      return alarm_txt;
    }
    return alarm_txt + ", " + appendText;
  }

  function get_status_txt(status) //get status text
  {
    var status_txt = "",
      array = new Array();
    if ((status & 8) == 8)
      array.push(i18n(lang_discharging));
    if ((status & 16) == 16)
      array.push(i18n(lang_charging));
    if ((status & 32) == 32)
      array.push(i18n(lang_discharge_stop));
    if ((status & 64) == 64)
      array.push(i18n(lang_float_charging));
    if ((status & 16384) == 16384)
      array.push(i18n(lang_initialising_modules));
    if (array.length > 0)
      status_txt = array.join(", ");
    else
      status_txt = i18n(lang_unknown_status);
    return status_txt;
  }

  function get_status_img(status) //get status img-src
  {
    var img_src = "./image/bacs2/b_grey.gif";
    if ((status & 8) == 8)
      img_src = "./image/bacs2/b_brightorange.gif";
    if ((status & 16) == 16)
      img_src = "./image/bacs2/b_green.gif";
    if ((status & 32) == 32)
      img_src = "./image/bacs2/b_brightorange.gif";
    if ((status & 64) == 64)
      img_src = "./image/bacs2/b_green.gif";
    if ((status & 16384) == 16384)
      img_src = "./image/bacs2/b_red.gif";
    return img_src;
  }

  function get_state_img_src(alarm, init, baseline_impc, string_alarm2) {
    if (-1 == baseline_impc) {
      baseline_impc = 0; //hack: suppress baseline_impc value, if value in json is wrong.
    }
    if (-1 == string_alarm2) {
      string_alarm2 = 0;//hack: suppress string_alarm2 value, if value in json is wrong.
    }
    if ((init == 0) || (alarm & 2) == 2 || (alarm & bacs_alarm_modrev_incompatible) != 0)
      return "./image/bacs2/b_blue.gif";
    if ((alarm & 4) == 4 || (alarm & 8) == 8 || (alarm & 16) == 16 ||
      (alarm & 32) == 32 || (alarm & 64) == 64 || (alarm & 128) == 128 ||
      (baseline_impc & bacs_impc_baseline_alarm) == bacs_impc_baseline_alarm)
      return "./image/bacs2/b_red.gif";
    if ((alarm & 512) == 512 || (alarm & 1024) == 1024 || (alarm & 2048) == 2048 ||
      (alarm & 4096) == 4096 || (alarm & 8192) == 8192 || (alarm & 16384) == 16384 ||
      (baseline_impc & bacs_impc_baseline_warning) == bacs_impc_baseline_warning ||
      0 != (string_alarm2 & bacs_alarm2_equalizing_system_disable))
      return "./image/bacs2/b_brightorange.gif";
    return "./image/bacs2/b_green.gif";
  }

  function get_string_current_color(alarm) {
    if (alarm & (bacs_alarm2_current_sensor_high 
                 | bacs_alarm2_current_sensor_low
                 | bacs_alarm2_current_deviation_discharge
                 | bacs_alarm2_current_deviation_charge)) 
    {
      return "red";
    }
    if (alarm & (bacs_alarm2_current_sensor_warn_high 
                 | bacs_alarm2_current_sensor_warn_low
                 | bacs_alarm2_current_deviation_warn_discharge
                 | bacs_alarm2_current_deviation_warn_charge)) 
    {
      return "orange";
    }
    return "black";
  }

  function get_string_volt_color(alarm) {
    if (alarm & (bacs_state_string_voltage_high | bacs_state_string_voltage_low)) {
      return "red";
    }
    if (alarm & (bacs_state_string_voltage_warning_high | bacs_state_string_voltage_warning_low)) {
      return "orange";
    }
    return "black";
  }

  function get_volt_color(alarm) {
    if ((alarm & 2) == 2 || (alarm & 4) == 4 || (alarm & 8) == 8)
      return "red";
    if ((alarm & 512) == 512 || (alarm & 1024) == 1024)
      return "orange";
    return "black";
  }

  function get_temp_color(alarm) {
    if ((alarm & 2) == 2 || (alarm & 16) == 16 || (alarm & 32) == 32)
      return "red";
    if ((alarm & 2048) == 2048 || (alarm & 4096) == 4096)
      return "orange";
    return "black";
  }

  function get_impd_color(alarm, baseline_impc) {
    if (-1 == baseline_impc) {
      baseline_impc = 0; //hack: suppress baseline_impc value, if value in json is wrong.
    }
    if ((alarm & 2) == 2 || (alarm & 64) == 64 || (alarm & 128) == 128 || (baseline_impc & bacs_impc_baseline_alarm) == bacs_impc_baseline_alarm)
      return "red";
    if ((alarm & 8192) == 8192 || (alarm & 16384) == 16384 || (baseline_impc & bacs_impc_baseline_warning) == bacs_impc_baseline_warning)
      return "orange";
    return "black";
  }

  function get_equalizing_img_src(equalizing, alarm) {
    if ((alarm & 2) == 2 || (equalizing == -1))
      return "./image/bacs2/b_red.gif";
    if (equalizing == 0)
      return "./image/bacs2/beam0.gif";
    if (equalizing >= 1 && equalizing <= 20)
      return "./image/bacs2/beam1.gif";
    if (equalizing >= 21 && equalizing <= 40)
      return "./image/bacs2/beam2.gif";
    if (equalizing >= 41 && equalizing <= 60)
      return "./image/bacs2/beam3.gif";
    if (equalizing >= 61 && equalizing <= 80)
      return "./image/bacs2/beam4.gif";
    if (equalizing >= 81 && equalizing <= 100)
      return "./image/bacs2/beam5.gif";
    return "./image/bacs2/beam0.gif";
  }

  function get_equalizing_img_width(equalizing, alarm) {
    if ((alarm & 2) == 2 || (equalizing == -1))
      return 16;
    return 26;
  }

  function get_string_class(string) {
    var string_w_level = 0,
      alarm = 0,
      impc_baseline_alarm;
    if (string.warninglevel != null) {
      if (string.warninglevel == "alarm")
        return "danger";
      if (string.warninglevel == "warning")
        return "warning";
      return "success";
    }

    $.each(string.modules, (function (i, module) {
      alarm = module.alarm;
      impc_baseline_alarm = module.impc_baseline_alarm;
      if ((alarm & 2) == 2)
        string_w_level = 3; //"./image/bacs2/b_blue.gif";
      else if ((alarm & 4) == 4 || (alarm & 8) == 8 || (alarm & 16) == 16 ||
        (alarm & 32) == 32 || (alarm & 64) == 64 || (alarm & 128) == 128 ||
        (impc_baseline_alarm & bacs_impc_baseline_alarm) == bacs_impc_baseline_alarm) {
        if (string_w_level < 2)
          string_w_level = 2; //"./image/bacs2/b_red.gif";
      } else if ((alarm & 512) == 512 || (alarm & 1024) == 1024 || (alarm & 2048) == 2048 ||
        (alarm & 4096) == 4096 || (alarm & 8192) == 8192 || (alarm & 16384) == 16384 ||
        (impc_baseline_alarm & bacs_impc_baseline_warning) == bacs_impc_baseline_warning) {
        if (string_w_level < 1)
          string_w_level = 1; //"./image/bacs2/b_brightorange.gif";
      }
    }));

    if (string_w_level == 3)
      return "danger";
    if (string_w_level == 2)
      return "danger";
    if (string_w_level == 1)
      return "warning";
    return "success";
  }

  function create_equalize_tt(string, module) {
    var eq_div, eq_div_id, equalizing, css_obj;
    eq_div_id = 'id_eq_module_' + module.module + '_' + string.string;

    css_obj = { position: 'absolute', display: 'none', 'z-index': 10 };
    eq_div = $('<div />', { 'id': eq_div_id, 'class': 'alert alert-info' }).css(css_obj);
    eq_div.append($('<span />', { 'id': eq_div_id + '_nr' }).append(i18n('Module') + ' ' + module.module));

    eq_div.append($('<br />'));
    equalizing = module.equalizing;
    if (equalizing == -1 || ((module.alarm & 2) == 2)) {
      equalizing = "n.a";
      eq_div.append($('<span />', { 'id': eq_div_id + '_eq' }).append(i18n('equalize') + ': ' + equalizing));
    } else
      eq_div.append($('<span />', { 'id': eq_div_id + '_eq' }).append(i18n('equalize') + ': ' + equalizing + '%'));
    eq_div.appendTo(document.body);
  }

  function set_equalize_tt(string, module) {
    var eq_div_id = 'id_eq_module_' + module.module + '_' + string.string;

    $('#' + eq_div_id + '_nr').text('Module ' + module.module);

    equalizing = module.equalizing;
    if (equalizing == -1 || ((module.alarm & 2) == 2)) {
      equalizing = "n.a";
      $('#' + eq_div_id + '_eq').text(i18n('equalize') + ': ' + equalizing);
    } else
      $('#' + eq_div_id + '_eq').text(i18n('equalize') + ': ' + equalizing + '%');
  }

  function create_state_tt(string, module) {
    var state_div, state_div_id, alarm_txt, warn_txt, css_obj;
    state_div_id = 'id_state_module_' + module.module + '_' + string.string;

    css_obj = { position: 'absolute', display: 'none', 'z-index': 10, 'text-align': 'left' };

    state_div = $('<div />', { 'id': state_div_id, 'class': 'alert alert-info' }).css(css_obj);
    state_div.append($('<span />', { 'id': state_div_id + '_nr' }).append('Module ' + module.module));
    state_div.append($('<br />'));
    if (module.init == 0) {
      alarm_txt = lang_Lost_communication_to_this_module;
    } else {
      alarm_txt = get_alarm_txt(module.alarm, undefined, module.impc_baseline_alarm); //get Alarm text
      alarm_txt = appendAlarmText(alarm_txt, get_alarm2_txt(string.alarm2));
    }
    if (module.alarm & bacs_alarm_modrev_incompatible) {
      if (0 < alarm_txt.length) {
        alarm_txt += ", ";
      }
      alarm_txt += lang_Bacs_Error_HW_Revision_mismatch;
    }
    if (alarm_txt.length > 0) {
      state_div.append($('<span />', { 'id': state_div_id + '_alarm' }).append(i18n('Alarm Status') + ': ' + alarm_txt));
      state_div.append($('<br />'));
    } else {
      state_div.append($('<span />', { 'id': state_div_id + '_alarm' }).append(i18n('Alarm Status') + ': ' + i18n(lang_ok)));
      state_div.append($('<br />'));
    }
    warn_txt = get_warn_txt(module.alarm, undefined, module.impc_baseline_alarm); //get warnings text
    warn_txt = appendAlarmText(warn_txt, get_alarm2_warn_txt(string.alarm2));
    if (warn_txt.length > 0)
      state_div.append($('<span />', { 'id': state_div_id + '_warn' }).append(i18n('Warning') + ': ' + warn_txt));
    else
      state_div.append($('<span />', { 'id': state_div_id + '_warn' }).append(i18n('Warning') + ': ' + lang_No_Warning));

    state_div.append($('<br />'));
    state_div.append($('<br />'));
    state_div.append($('<span />', { 'id': state_div_id + '_sw_version' }).append(i18n('Software Version') + ': ' + getSoftwareVer(module)));



    state_div.append($('<br />'));
    state_div.append($('<span />', { 'id': state_div_id + '_hw_version' }).append(i18n('Hardware Version') + ': ' + getHardwareVer(module)));

    state_div.appendTo(document.body);
  }

  function set_state_tt(string, module) {
    var state_div_id, alarm_txt, warn_txt;
    state_div_id = 'id_state_module_' + module.module + '_' + string.string;

    $('#' + state_div_id + '_nr').text('Module ' + module.module);

    if (module.init == 0) {
      alarm_txt = lang_Lost_communication_to_this_module;
    } else {
      alarm_txt = get_alarm_txt(module.alarm, undefined, module.impc_baseline_alarm); //get Alarm text
      alarm_txt = appendAlarmText(alarm_txt, get_alarm2_txt(string.alarm2));
    }
    if (module.alarm & bacs_alarm_modrev_incompatible) {
      if (0 < alarm_txt.length) {
        alarm_txt += ", ";
      }
      alarm_txt += lang_Bacs_Error_HW_Revision_mismatch;
    }
    if (alarm_txt.length > 0)
      $('#' + state_div_id + '_alarm').text(i18n('Alarm Status') + ': ' + alarm_txt);
    else
      $('#' + state_div_id + '_alarm').text(i18n('Alarm Status') + ': ' + i18n(lang_ok));

    warn_txt = get_warn_txt(module.alarm, undefined, module.impc_baseline_alarm); //get warnings text
    warn_txt = appendAlarmText(warn_txt, get_alarm2_warn_txt(string.alarm2));
    if (warn_txt.length > 0)
      $('#' + state_div_id + '_warn').text(i18n('Warning') + ': ' + warn_txt);
    else
      $('#' + state_div_id + '_warn').text(i18n('Warning') + ': ' + i18n(lang_No_Warning));

    $('#' + state_div_id + '_sw_version').text(i18n('Software Version') + ': ' + getSoftwareVer(module));
    $('#' + state_div_id + '_hw_version').text(i18n('Hardware Version') + ': ' + getHardwareVer(module));

  }

  function getSoftwareVer(module) {
    var major = module.software_version_major < 10 ? '0' + module.software_version_major : module.software_version_major,
      minor = module.software_version_minor < 10 ? '0' + module.software_version_minor : module.software_version_minor,
      build = module.software_version_build < 10 ? '0' + module.software_version_build : module.software_version_build;
    return major + '.' + minor + '.' + build;
  }

  function getHardwareVer(module) {
    var major = module.hardware_version_major < 10 ? '0' + module.hardware_version_major : module.hardware_version_major,
      minor = module.hardware_version_minor < 10 ? '0' + module.hardware_version_minor : module.hardware_version_minor;
    return major + '.' + minor;
  }

  function create_footer_pane() {
    var table = document.getElementById('sys_infos_id');
    if (!bacs_oid || !unms_obj)
      return true;
    footer_api.create({ //Attr
        id: 'panel2'
      }, { //css
        position: 'absolute',
        top: table.offsetTop + table.offsetHeight + 30 + 'px',
        left: (table.offsetLeft) + 'px',
        align: 'center'
      },
      unms_obj
    );
    document.body.onresize = function () {
      var table = document.getElementById('sys_infos_id');
      var footer_div = document.getElementById('panel2');
      footer_div.style.left = (table.offsetLeft) + 'px';
      footer_div.style.top = table.offsetTop + table.offsetHeight + 30 + 'px';
    };
  }

  function set_footer() {
    if (!bacs_oid || !unms_obj)
      return true;
    footer_api.set(unms_obj);
  }

  function round_max_decimals(val, decimals) {
    var x = (val*10).toFixed();
    if (x == val*10) {
      return val;
    }
    return val.toFixed(decimals);
  }
}(jQuery));
