/*
   cet_panel.js - CET (cet_tsi) screen.
   Copyright (c) 2016, Generex GmbH. All rights reserved.
   Said.
*/

var cet_api = (function ($) {
  var oid = get_oid_from_url(),
  ctl_url = get_url("api/devices/ups/control"),
  plattform = get_plattform(),
  cb = null,
  pannes_created = false,
  err_div = null,
  alarm_history_win = null,
  check_bowser = true,
  css_obj,
  valtable_obj, report_obj, unms_obj,
  inphases = null,
  outphases = null,
  x_pos = 5,
  y_pos = 5,
  req_params = null,
  alarm_title = '';


  /*public functions*/
  return {
    init: init,
    send_alarm_history_cmd: send_alarm_history_cmd
  };

  function init() {
    if (check_bowser && !utils_api.check_bowser()) {
      $('#loading_id').css('display', 'none');
      panes_api.create_br_check_panel(this, browser_check_ok);
      return true;
    }
    create_err_msg_panel();

    req_params = utils_api.get_req_params(utils_api.device_ups, oid);
    cb = jqmodule.subscribe(this, { update: update, error: err_funct }, req_params);
  }

  function browser_check_ok() {
    check_bowser = false;
    init();
  }

  function create_err_msg_panel() {
    css_obj = { position: 'absolute', top: y_pos, left: x_pos, width: 791, display: 'none', 'z-index': 122 };
    err_div = panes_api.create_err_panel({ id: 'error_div', 'class': 'alert alert-danger' }, css_obj);
  }

  function err_funct(response) {
    jqmodule.refresh(cb, 10 * 1000);
    panes_api.display_err({ response: response, oid: oid, err_div: err_div, err_msg_div_id: 'error_msg_div' });
  }

  function update(data) {
    if (data.ups == null) {
      return;
    }

    set_variable(data);
    jqmodule.refresh(cb, report_obj.refresh * 1000);
    if (err_div)
    err_div.css('display', 'none');

    if (window.menu_api != null)
    menu_api.update(data);

    unms_obj = get_unms_obj(data);
    if (unms_obj != null) {
      info_head_api.update(unms_obj);
    }

    if (!pannes_created) {
      pannes_created = true;
      create_panes();
      return false;
    }
    set_info(); // set Location
    set_typ_module_redundancy(); // set Type, Modules configured and Redundancy
    set_src_pos_temp(); // set Source Position and Temperature
    set_alarm(); // set Major, Minor alarm ..
    set_leds(); // set leds
    set_ac_inp_groups();
    set_ac_inp_groups_info();
    set_phases_grb();
    set_phases_grb_info();
    set_dc_inp_groups();
    set_dc_inp_groups_info();
    set_modules_panel();
    set_modules_value();

    set_footer();
  }

  function set_footer() {
    if (!oid || !unms_obj)
    return true;
    footer_api.set(unms_obj);
  }

  function set_variable(data) {
    valtable_obj = data.ups.valtable;
    report_obj = data.report;

    if (valtable_obj.INPHASES != null)
    inphases = valtable_obj.INPHASES;
    else
    inphases = valtable_obj.PHASES;

    if (valtable_obj.OUTPHASES != null)
    outphases = valtable_obj.OUTPHASES;
    else
    outphases = valtable_obj.PHASES;

    valtable_obj._tempUnit = valtable_obj._tempUnit || 0; //0 = Celsius
  }

  function create_panes() {
    utils_api.setTitle(valtable_obj._platform);
    create_bg();
    create_info_panel(); // create Model, Location and Serial Number
    create_typ_module_redundancy_panel(); // create Type, Modules configured and Redundancy
    create_src_pos_temp_panel(); // create Source Position and Temperature
    create_alarm_panel(); // create Major, Minor alarm ..
    create_leds_panel(); //create leds  panel

    create_ac_inp_groups_bnt();
    create_ac_inp_groups_panel();

    create_ac_inp_groups_info_panel();
    create_ac_inp_groups_info_bnt();

    create_modules_bnt();
    create_modules_panel();

    create_phases_grb_bnt();
    create_phases_grb_panel();

    create_dc_inp_groups_info_bnt();
    create_dc_inp_groups_info_panel();

    create_dc_inp_groups_bnt();
    create_dc_inp_groups_panel();

    create_phases_grb_info_bnt();
    create_phases_grb_info_panel();

    create_modules_value_panel();

    create_footer_pane();
  }

  function create_bg() {
    var fragment = document.createDocumentFragment(),
    img_div;
    css_obj = { position: 'absolute', top: y_pos, left: x_pos, width: 798, height: 598, border: 'solid 1px #808080', align: 'center' };
    img_div = $('<div/>', { 'class': 'img-rounded' }).css(css_obj);
    $('<img />').css({ display: 'block', 'margin-left': 'auto', 'margin-right': 'auto' }).
    attr({ 'id': 'bg_img_id', 'class': 'img-rounded', src: './image/ce_t/cet_tsi.png', align: 'middle' }).appendTo(img_div);

    fragment.appendChild(img_div[0]);
    document.body.appendChild(fragment);
  }

  function create_info_panel() // create Model, Location and Serial Number
  {
    var fragment = document.createDocumentFragment(),
    tr, table, tbody, thead, info_div;

    css_obj = {
      position: 'absolute',
      top: y_pos + 80,
      left: x_pos + 55,
      height: 68,
      align: 'center',
      'background-color': '#cccccc',
      border: 'solid 1px white'
    };

    info_div = $('<div/>', { id: 'info_div_id', 'class': 'img-rounded' }).css(css_obj);

    table = $('<table />', { id: 'info_table', 'class': 'table table-condensed table-hover' }).appendTo(info_div);
    table.css({ 'font-size': '9pt', 'font-weight': 'bold', 'font-family': 'arial', align: 'center' })
    tbody = $('<tbody />').appendTo(table);
    thead = $('<thead />').appendTo(table);

    tr = $('<tr />').appendTo(tbody);
    $('<td />', { align: 'left' }).append(i18n('System Model') + ':').appendTo(tr);
    $('<td />', { id: 'sys_model_id', align: 'left' }).append(valtable_obj.MODEL).appendTo(tr);

    tr = $('<tr />').appendTo(tbody);
    $('<td />', { align: 'left' }).append(i18n('Location') + ':').appendTo(tr);
    $('<td />', { id: 'location_id', align: 'left' }).append(valtable_obj.LOCATION).appendTo(tr);

    tr = $('<tr />').appendTo(tbody);
    $('<td />', { align: 'left' }).append('T2S' + i18n('Serial Number') + ':').appendTo(tr);
    $('<td />', { id: 'ser_nr_id', align: 'left' }).append(valtable_obj.CETMISC_SERNO).appendTo(tr);

    fragment.appendChild(info_div[0]);
    document.body.appendChild(fragment);
  }

  function set_info() // set Location
  {
    $('#location_id').val(valtable_obj.LOCATION);
  }

  function create_typ_module_redundancy_panel() // create Type, Modules configured and Redundancy
  {
    var fragment = document.createDocumentFragment(),
    tr, table, tbody, typ_module_redun_div;

    css_obj = {
      position: 'absolute',
      top: y_pos + 80,
      left: x_pos + 555,
      height: 68,
      align: 'center',
      'background-color': '#cccccc',
      border: 'solid 1px white'
    };

    typ_module_redun_div = $('<div/>', { id: 'typ_module_redun_div_id', 'class': 'img-rounded' }).css(css_obj);

    table = $('<table />', { id: 'typ_module_redun_table', 'class': 'table table-condensed table-hover' }).appendTo(typ_module_redun_div);
    table.css({ 'font-size': '9pt', 'font-weight': 'bold', 'font-family': 'arial', align: 'center' })
    tbody = $('<tbody />').appendTo(table);

    tr = $('<tr />').appendTo(tbody);
    $('<td />', { align: 'left' }).append('TSI ' + i18n('Type') + ':').appendTo(tr);
    $('<td />', { id: 'tsi_type_id', align: 'left' }).append(get_tsi_type()).appendTo(tr);

    tr = $('<tr />').appendTo(tbody);
    $('<td />', { align: 'left' }).append(i18n('Modules configured') + ':').appendTo(tr);
    $('<td />', { id: 'modules_configured_id', align: 'left' }).append(valtable_obj.CET_NUMMODCONF).appendTo(tr);

    tr = $('<tr />').appendTo(tbody);
    $('<td />', { align: 'left' }).append(i18n('Redundancy') + ':').appendTo(tr);
    $('<td />', { id: 'redundancy_id', align: 'left' }).append(valtable_obj.CET_NUMREDUNDANCY).appendTo(tr);

    fragment.appendChild(typ_module_redun_div[0]);
    document.body.appendChild(fragment);
  }

  function set_typ_module_redundancy() // set Type, Modules configured and Redundancy
  {
    $('#tsi_type_id').val(get_tsi_type());
    $('#modules_configured_id').val(valtable_obj.CET_NUMMODCONF);
    $('#redundancy_id').val(valtable_obj.CET_NUMREDUNDANCY);
  }

  function create_src_pos_temp_panel() // create Source Position and Temperature
  {
    var fragment = document.createDocumentFragment(),
    tr, table, tbody, src_pos_temp_div;

    css_obj = {
      position: 'absolute',
      top: y_pos + 400,
      left: x_pos + 55,
      height: 48,
      align: 'center',
      'background-color': '#cccccc',
      border: 'solid 1px white'
    };

    src_pos_temp_div = $('<div/>', { id: 'src_pos_temp_div_id', 'class': 'img-rounded' }).css(css_obj);

    table = $('<table />', { id: 'src_pos_temp_table', 'class': 'table table-condensed table-hover' }).appendTo(src_pos_temp_div);
    table.css({ 'font-size': '9pt', 'font-weight': 'bold', 'font-family': 'arial', align: 'center' })
    tbody = $('<tbody />').appendTo(table);

    tr = $('<tr />').appendTo(tbody);
    $('<td />', { align: 'left' }).append(i18n('Source Position') + ':').appendTo(tr);
    $('<td />', { id: 'src_pos_id', align: 'left' }).append(get_src_pos()).appendTo(tr);

    tr = $('<tr />').appendTo(tbody);
    $('<td />', { align: 'left' }).append(i18n('Temperature') + ':').appendTo(tr);
    $('<td />', { id: 'temperature_id', align: 'left' }).append(panes_api.get_temperature(valtable_obj.TEMPDEG, valtable_obj._tempUnit, 1)).appendTo(tr);

    fragment.appendChild(src_pos_temp_div[0]);
    document.body.appendChild(fragment);
  }

  function set_src_pos_temp() // set Source Position and Temperature
  {
    $('#temperature_id').html(panes_api.get_temperature(valtable_obj.TEMPDEG, valtable_obj._tempUnit, 1));
    $('#src_pos_id').val(get_src_pos());
  }

  function create_alarm_panel() // create Major, Minor alarm ..
  {
    var fragment = document.createDocumentFragment(),
    tr, table, tbody, alarm_div, a, img;

    css_obj = {
      position: 'absolute',
      top: y_pos + 400,
      left: x_pos + 555,
      height: 134,
      align: 'center',
      'background-color': '#cccccc',
      border: 'solid 1px white'
    };

    alarm_div = $('<div/>', { id: 'alarm_div_id', 'class': 'img-rounded' }).css(css_obj);

    table = $('<table />', { id: 'typ_module_redun_table', 'class': 'table table-condensed table-hover' }).appendTo(alarm_div);
    table.css({ 'font-size': '9pt', 'font-weight': 'bold', 'font-family': 'arial', align: 'center' })
    tbody = $('<tbody />').appendTo(table);

    tr = $('<tr />').appendTo(tbody);
    img = $('<img />').attr({
      id: 'major_alarm_img_id',
      'class': 'img-rounded',
      width: 16,
      'height': 16,
      src: get_major_minor_alarm_img(valtable_obj.CETMISC_NUMMAJORAL)
    });
    $('<td />', { align: 'left' }).append(img).appendTo(tr);

    //Priorty nach dem 1 pipe

    if (plattform === utils_api.plattform_cs141) {
      a = $('<a />', { href: "#", onclick: "javascript:cet_api.send_alarm_history_cmd('2'); return false;", title: i18n('Show Major Alarms') }).text(i18n('Major Alarm'));
    } else {
      a = $('<a />', { href: "/create_cetlogfile.cgi?cmd=2001&p1=2", title: i18n('Show Major Alarms') }).text(i18n('Major Alarm'));
    }

    $('<td />', { align: 'left' }).append(a).appendTo(tr);

    tr = $('<tr />').appendTo(tbody);
    img = $('<img />').attr({
      id: 'minor_alarm_img_id',
      'class': 'img-rounded',
      width: 16,
      'height': 16,
      src: get_major_minor_alarm_img(valtable_obj.CETMISC_NUMMINORAL)
    });
    $('<td />', { align: 'left' }).append(img).appendTo(tr);
    if (plattform === utils_api.plattform_cs141) {
      a = $('<a />', { href: "#", onclick: "javascript:cet_api.send_alarm_history_cmd('1'); return false;", title: i18n('Show Minor Alarms') }).html(i18n('Minor Alarm'));
    } else {
      a = $('<a />', { href: "/create_cetlogfile.cgi?cmd=2001&p1=1", title: i18n('Show Minor Alarms') }).text(i18n('Minor Alarm'));
    }
    $('<td />', { align: 'left' }).append(a).appendTo(tr);

    tr = $('<tr />').appendTo(tbody);
    img = $('<img />').attr({
      id: 'prog_alarm_img_id',
      'class': 'img-rounded',
      width: 16,
      'height': 16,
      src: get_prog_alarm_img()
    });
    $('<td />', { align: 'left' }).append(img).appendTo(tr);
    $('<td />', { align: 'left' }).append(i18n('User selectable Alarm')).appendTo(tr);

    tr = $('<tr />').appendTo(tbody);
    img = $('<img />').attr({
      id: 'aux1_img_id',
      'class': 'img-rounded',
      width: 16,
      'height': 16,
      src: get_aux_img(valtable_obj.CET_1AUX)
    });
    $('<td />', { align: 'left' }).append(img).appendTo(tr);
    $('<td />', { id: 'aux1_ev_id', align: 'left' }).append(valtable_obj.CET_EV1AUX).appendTo(tr);

    tr = $('<tr />').appendTo(tbody);
    img = $('<img />').attr({
      id: 'aux2_img_id',
      'class': 'img-rounded',
      width: 16,
      'height': 16,
      src: get_aux_img(valtable_obj.CET_2AUX)
    });
    $('<td />', { align: 'left' }).append(img).appendTo(tr);
    $('<td />', { id: 'aux2_ev_id', align: 'left' }).append(valtable_obj.CET_EV2AUX).appendTo(tr);

    fragment.appendChild(alarm_div[0]);
    document.body.appendChild(fragment);
  }

  function set_alarm() // set Major, Minor alarm ..
  {
    $('#major_alarm_img_id').attr('src', get_major_minor_alarm_img(valtable_obj.CETMISC_NUMMAJORAL));
    $('#minor_alarm_img_id').attr('src', get_major_minor_alarm_img(valtable_obj.CETMISC_NUMMINORAL));
    $('#prog_alarm_img_id').attr('src', get_prog_alarm_img());
    $('#aux1_img_id').attr('src', get_aux_img(valtable_obj.CET_1AUX));
    $('#aux1_ev_id').val(valtable_obj.CET_EV1AUX);
    $('#aux2_img_id').attr('src', get_aux_img(valtable_obj.CET_2AUX));
    $('#aux2_ev_id').val(valtable_obj.CET_EV2AUX);
  }

  function create_leds_panel() //create leds panel
  {
    var fragment = document.createDocumentFragment(),
    img_div;

    //Led 1
    css_obj = { position: 'absolute', top: y_pos + 208, left: x_pos + 90, width: 16, height: 16 };
    img_div = $('<div/>', { 'class': 'img-rounded' }).css(css_obj);
    $('<img />').attr({ 'id': 'led1_img', 'class': 'img-rounded', width: 16, 'height': 16, src: get_led_1_5_img(valtable_obj.CETGLOBAL_ACIPLED) }).appendTo(img_div);
    fragment.appendChild(img_div[0]);

    //Led 2
    css_obj = { position: 'absolute', top: y_pos + 195, left: x_pos + 315, width: 16, height: 16 };
    img_div = $('<div/>', { 'class': 'img-rounded' }).css(css_obj);
    $('<img />').attr({ 'id': 'led2_img', 'class': 'img-rounded', width: 16, 'height': 16, src: get_led_2_3_4_6_img(valtable_obj.CETGLOBAL_ACIPLED) }).appendTo(img_div);
    fragment.appendChild(img_div[0]);

    //Led 3
    css_obj = { position: 'absolute', top: y_pos + 195, left: x_pos + 465, width: 16, height: 16 };
    img_div = $('<div/>', { 'class': 'img-rounded' }).css(css_obj);
    $('<img />').attr({ 'id': 'led3_img', 'class': 'img-rounded', width: 16, 'height': 16, src: get_led_2_3_4_6_img(valtable_obj.CETGLOBAL_ACOPLED) }).appendTo(img_div);
    fragment.appendChild(img_div[0]);

    //Led 4
    css_obj = { position: 'absolute', top: y_pos + 208, left: x_pos + 565, width: 16, height: 16 };
    img_div = $('<div/>', { 'class': 'img-rounded' }).css(css_obj);
    $('<img />').attr({ 'id': 'led4_img', 'class': 'img-rounded', width: 16, 'height': 16, src: get_led_2_3_4_6_img(valtable_obj.CETGLOBAL_LOADLED) }).appendTo(img_div);
    fragment.appendChild(img_div[0]);

    //Led 5
    css_obj = { position: 'absolute', top: y_pos + 405, left: x_pos + 325, width: 16, height: 16 };
    img_div = $('<div/>', { 'class': 'img-rounded' }).css(css_obj);
    $('<img />').attr({ 'id': 'led5_img', 'class': 'img-rounded', width: 16, 'height': 16, src: get_led_1_5_img(valtable_obj.CETGLOBAL_DCIPLED) }).appendTo(img_div);
    fragment.appendChild(img_div[0]);

    //Led 6
    css_obj = { position: 'absolute', top: y_pos + 258, left: x_pos + 390, width: 16, height: 16 };
    img_div = $('<div/>', { 'class': 'img-rounded' }).css(css_obj);
    $('<img />').attr({ 'id': 'led6_img', 'class': 'img-rounded', width: 16, 'height': 16, src: get_led_2_3_4_6_img(valtable_obj.CETGLOBAL_DCIPLED) }).appendTo(img_div);
    fragment.appendChild(img_div[0]);

    document.body.appendChild(fragment);
  }

  function set_leds() // set leds
  {
    $('#led1_img').attr('src', get_led_1_5_img(valtable_obj.CETGLOBAL_ACIPLED));
    $('#led2_img').attr('src', get_led_2_3_4_6_img(valtable_obj.CETGLOBAL_ACIPLED));
    $('#led3_img').attr('src', get_led_2_3_4_6_img(valtable_obj.CETGLOBAL_ACOPLED));
    $('#led4_img').attr('src', get_led_2_3_4_6_img(valtable_obj.CETGLOBAL_LOADLED));
    $('#led5_img').attr('src', get_led_1_5_img(valtable_obj.CETGLOBAL_DCIPLED));
    $('#led6_img').attr('src', get_led_2_3_4_6_img(valtable_obj.CETGLOBAL_DCIPLED));
  }

  function create_ac_inp_groups_bnt() {
    var fragment = document.createDocumentFragment(),
    div;

    css_obj = {
      position: 'absolute',
      top: y_pos + 199,
      left: x_pos + 81,
      width: 158,
      height: 102,
      'background-color': 'transparent',
      cursor: 'pointer'
    };
    div = $('<div/>', { 'class': 'img-rounded', title: i18n('Show AC Groups') }).css(css_obj);

    div.click(function (event) {
      var inp_groups_div = $('#inp_groups_div_id');
      inp_groups_div.css('display',
        inp_groups_div.css('display') == 'none' ? 'block' : 'none');
      event.preventDefault();
    });

    fragment.appendChild(div[0]);
    document.body.appendChild(fragment);
  }

  function create_ac_inp_groups_panel() {
    var fragment = document.createDocumentFragment(),
    tr, table, tbody, thead, inp_groups_div;

    css_obj = {
      position: 'absolute',
      top: y_pos + 225,
      left: x_pos + 81,
      'display': 'none',
      border: 'solid 1px #808080',
      'z-index': '44',
      'background-color': '#cccccc'
    };

    inp_groups_div = $('<div/>', { id: 'inp_groups_div_id', 'class': 'img-rounded' }).css(css_obj);

    table = $('<table />', { id: 'inp_groups_table', 'class': 'table table-condensed table-hover  table-bordered' }).appendTo(inp_groups_div);
    table.css({ 'font-size': '9pt', 'font-weight': 'bold', 'font-family': 'arial', 'border-collapse': 'initial' });

    tbody = $('<tbody />').appendTo(table);
    thead = $('<thead />').appendTo(table);

    span_x = $('<span>X</span>').
    css({
      cursor: 'pointer',
      color: 'white',
      border: '1px solid #AEAEAE',
      'border-radius': '30px',
      background: '#605F61',
      'font-size': '12px',
      'font-weight': 'bold',
      display: 'inline-block',
      'line-height': '0px',
      padding: '8px 6px',
      'margin-left': '10px'
    });
    tr = $('<tr />').append($('<td />', { colspan: 3, align: 'center', 'text-align': 'center', title: i18n('Close AC Input Groups') }).click(function (event) {
      $('#inp_groups_div_id').css({ display: 'none' });
      event.preventDefault();
    }).css({ color: 'white', 'background-color': '#C11E29', cursor: 'pointer' }).append(i18n("AC Input Groups"))).appendTo(thead);


    tr = $('<tr />').append($('<td />', { align: 'left', 'text-align': 'center' }).append(" ")).appendTo(thead);
    $('<td />', { align: 'left' }).append("Voltage").appendTo(tr);
    $('<td />', { align: 'left' }).append("Frequency").appendTo(tr);

    for (var index = 0; index < valtable_obj.CET_NUMMODCONF; ++index) {
      tr = $('<tr />').append($('<td />', { align: 'left' }).append(index + 1 + ':')).appendTo(tbody);
      $('<td />', { id: 'ac_inp_volt_' + (index + 1), align: 'right' }).append(get_value({ value: valtable_obj.CETAC_ACIPVOLT[index] * 0.1, unit: ' V', precision: 1 })).appendTo(tr);
      $('<td />', { id: 'ac_inp_freq_' + (index + 1), align: 'right' }).append(get_value({ value: valtable_obj.CETAC_ACIPFREQ[index] * 0.1, unit: ' Hz', precision: 1 })).appendTo(tr);
    }

    fragment.appendChild(inp_groups_div[0]);
    document.body.appendChild(fragment);

    new utils_api.draggable('inp_groups_div_id');
  }

  function set_ac_inp_groups() {
    for (var index = 0; index < valtable_obj.CET_NUMMODCONF; ++index) {
      $('#ac_inp_volt_' + (index + 1)).val(get_value({ value: valtable_obj.CETAC_ACIPVOLT[index] * 0.1, unit: ' V', precision: 1 }));
      $('#ac_inp_freq_' + (index + 1)).val(get_value({ value: valtable_obj.CETAC_ACIPFREQ[index] * 0.1, unit: ' Hz', precision: 1 }));
    }
  }

  // AC Groups Transfer
  function create_ac_inp_groups_info_panel() {
    var fragment = document.createDocumentFragment(),
    tr, table, tbody, thead, inp_groups_info_div;

    css_obj = {
      position: 'absolute',
      top: y_pos + 252,
      left: x_pos + 264,
      'display': 'none',
      border: 'solid 1px #808080',
      'z-index': '44',
      'background-color': '#cccccc'
    };

    inp_groups_info_div = $('<div/>', { id: 'inp_groups_div_info_id', 'class': 'img-rounded' }).css(css_obj);

    table = $('<table />', { id: 'inp_groups_info_table', 'class': 'table table-condensed table-hover  table-bordered' }).appendTo(inp_groups_info_div);
    table.css({ 'font-size': '9pt', 'font-weight': 'bold', 'font-family': 'arial', 'border-collapse': 'initial' });

    tbody = $('<tbody />').appendTo(table);
    thead = $('<thead />').appendTo(table);

    tr = $('<tr />').append($('<td />', { colspan: 5, align: 'center', 'text-align': 'center', title: i18n('Close AC Input Groups') }).click(function (event) {
      $('#inp_groups_div_info_id').css({ display: 'none' });
      event.preventDefault();
    }).css({ color: 'white', 'background-color': '#C11E29', cursor: 'pointer' }).append(i18n("AC Input Groups"))).appendTo(thead);

    tr = $('<tr />').append($('<td />', { align: 'left', 'text-align': 'center' }).append(" ")).appendTo(thead);
    $('<td />', { align: 'left', colspan: 2 }).append(i18n("Power")).appendTo(tr);
    $('<td />', { align: 'left' }).append(i18n("Current")).appendTo(tr);

    for (var index = 0; index < 4; ++index) {
      tr = $('<tr />').append($('<td />', { align: 'left' }).append(index + 1 + ':')).appendTo(tbody);
      $('<td />', { id: 'ac_inp_powerw_' + (index + 1), align: 'right' }).append(get_value({ value: valtable_obj.CETAC_ACIPPOWERW[index], unit: ' W', precision: 0 })).appendTo(tr);
      $('<td />', { id: 'ac_inp_powerva_' + (index + 1), align: 'right' }).append(get_value({ value: valtable_obj.CETAC_ACIPPOWERVA[index], unit: ' VA', precision: 0 })).appendTo(tr);
      $('<td />', { id: 'ac_inp_curr_' + (index + 1), align: 'right' }).append(get_value({ value: valtable_obj.CETAC_ACIPCURR[index] * 0.1, unit: ' A', precision: 1 })).appendTo(tr);
    }

    fragment.appendChild(inp_groups_info_div[0]);
    document.body.appendChild(fragment);

    new utils_api.draggable('inp_groups_div_info_id');
  }

  function set_ac_inp_groups_info() {
    for (var index = 0; index < 4; ++index) {
      $('#ac_inp_powerw_' + (index + 1)).val(get_value({ value: valtable_obj.CETAC_ACIPPOWERW[index], unit: ' WV', precision: 1 }));
      $('#ac_inp_powerva_' + (index + 1)).val(get_value({ value: valtable_obj.CETAC_ACIPPOWERVA[index], unit: ' VA', precision: 1 }));
      $('#ac_inp_curr_' + (index + 1)).val(get_value({ value: valtable_obj.CETAC_ACIPCURR[index] * 0.1, unit: ' A', precision: 1 }));
    }
  }

  function create_ac_inp_groups_info_bnt() {
    var fragment = document.createDocumentFragment(),
    div;

    css_obj = {
      position: 'absolute',
      top: y_pos + 222,
      left: x_pos + 244,
      width: 50,
      height: 50,
      'background-color': 'transparent',
      cursor: 'pointer'
    };
    div = $('<div/>', { 'class': 'img-rounded', title: i18n('Show AC Input Groups') }).css(css_obj);

    div.click(function (event) {
      var inp_groups_info_div = $('#inp_groups_div_info_id');
      inp_groups_info_div.css('display',
        inp_groups_info_div.css('display') == 'none' ? 'block' : 'none');
      event.preventDefault();
    });


    fragment.appendChild(div[0]);
    document.body.appendChild(fragment);
  }

  function create_modules_bnt() {
    var fragment = document.createDocumentFragment(),
    div;

    css_obj = {
      position: 'absolute',
      top: y_pos + 185,
      left: x_pos + 301,
      width: 194,
      height: 126,
      'background-color': 'transparent',
      cursor: 'pointer'
    };
    div = $('<div/>', { 'class': 'img-rounded', title: i18n('Show Modules') }).css(css_obj);

    div.click(function (event) {
      var modules_div = $('#modules_div_id');
      modules_div.css('display',
        modules_div.css('display') == 'none' ? 'block' : 'none');
      event.preventDefault();
    });

    fragment.appendChild(div[0]);
    document.body.appendChild(fragment);
  }

  function create_modules_panel() {
    var fragment = document.createDocumentFragment(),
    modules_div, div;
    css_obj = {
      position: 'absolute',
      top: y_pos + 300,
      left: x_pos + 200,
      width: 798,
      height: 598,
      border: 'solid 1px #808080',
      align: 'center',
      'display': 'none',
      'z-index': '55'
    };
    modules_div = $('<div/>', { id: 'modules_div_id', 'class': 'img-rounded' }).css(css_obj);
    $('<img />').css({ display: 'block', 'margin-left': 'auto', 'margin-right': 'auto' }).
    attr({ 'id': 'modules_img_id', 'class': 'img-rounded', src: './image/ce_t/ac_dc.png', align: 'middle' }).appendTo(modules_div);

    //close button
    css_obj = {
      position: 'absolute',
      top: 0,
      left: 776,
      width: 20,
      height: 16,
      border: 'solid 1px white',
      align: 'center',
      'z-index': '55',
      'background-color': 'white',
      cursor: 'pointer',
      'text-align': 'center',
      'font-weight': 'bold',
      'font-size': 14
    };
    div = $('<div/>').attr({ 'class': 'img-rounded', title: 'Close' }).css(css_obj).append('X').appendTo(modules_div);
    div.click(function (event) {
      var modules_div = $('#modules_div_id');
      modules_div.css('display', 'none');
      event.preventDefault();
    });

    //MPB img
    css_obj = { position: 'absolute', top: y_pos + 87, left: x_pos + 357, width: 44, height: 44 };
    div = $('<div/>').css(css_obj).appendTo(modules_div);
    $('<img />').attr({ 'id': 'modules_mpb_img_id', 'class': 'img-rounded', src: get_mpb_img() }).appendTo(div);

    //Modules led1
    css_obj = { position: 'absolute', top: y_pos + 183, left: x_pos + 307, width: 16, height: 16 };
    div = $('<div/>').css(css_obj).appendTo(modules_div);
    $('<img />').attr({ 'id': 'modules_led1_img', 'class': 'img-rounded', src: get_led_2_3_4_6_img(valtable_obj.CETGLOBAL_ACIPLED) }).appendTo(div);

    //Modules led2
    css_obj = { position: 'absolute', top: y_pos + 183, left: x_pos + 461, width: 16, height: 16 };
    div = $('<div/>').css(css_obj).appendTo(modules_div);
    $('<img />').attr({ 'id': 'modules_led2_img', 'class': 'img-rounded', src: get_led_2_3_4_6_img(valtable_obj.CETGLOBAL_ACOPLED) }).appendTo(div);

    //Modules led3
    css_obj = { position: 'absolute', top: y_pos + 295, left: x_pos + 307, width: 16, height: 16 };
    div = $('<div/>').css(css_obj).appendTo(modules_div);
    $('<img />').attr({ 'id': 'modules_led3_img', 'class': 'img-rounded', src: get_led_2_3_4_6_img(valtable_obj.CETGLOBAL_DCIPLED) }).appendTo(div);

    css_obj = {
      position: 'absolute',
      top: y_pos + 199,
      left: x_pos + 318,
      width: 144,
      height: 98,
      'background-color': 'transparent',
      cursor: 'pointer',
      'z-index': '66'
    };
    div = $('<div/>', { 'class': 'img-rounded', title: 'Show Modules' }).css(css_obj).appendTo(modules_div);

    div.click(function (event) {
      var modules_value_div = $('#modules_value_div_id');
      modules_value_div.css('display',
        modules_value_div.css('display') == 'none' ? 'block' : 'none');
      event.preventDefault();
    });

    fragment.appendChild(modules_div[0]);
    document.body.appendChild(fragment);

    new utils_api.draggable('modules_div_id');
  }

  function set_modules_panel() {
    //MPB img
    $('#modules_mpb_img_id').attr('src', get_mpb_img());
    //Modules led1
    $('#modules_led1_img').attr('src', get_led_2_3_4_6_img(valtable_obj.CETGLOBAL_ACIPLED));
    //Modules led2
    $('#modules_led2_img').attr('src', get_led_2_3_4_6_img(valtable_obj.CETGLOBAL_ACOPLED));
    //Modules led3
    $('#modules_led3_img').attr('src', get_led_2_3_4_6_img(valtable_obj.CETGLOBAL_DCIPLED));
  }

  function create_modules_value_panel() {
    var fragment = document.createDocumentFragment(),
    tr, table, tbody, thead, img, modules_value_div;

    css_obj = {
      position: 'absolute',
      top: y_pos + 225,
      left: x_pos + 175,
      'display': 'none',
      border: 'solid 1px #808080',
      'z-index': '77',
      'background-color': '#cccccc'
    };

    modules_value_div = $('<div/>', { id: 'modules_value_div_id', 'class': 'img-rounded' }).css(css_obj);

    table = $('<table />', { id: 'modules_value_div_table', 'class': 'table table-condensed table-hover table-bordered' }).appendTo(modules_value_div);
    table.css({ 'font-size': '9pt', 'font-weight': 'bold', 'font-family': 'arial', 'border-collapse': 'initial' });

    tbody = $('<tbody />').appendTo(table);
    thead = $('<thead />').appendTo(table);

    tr = $('<tr />').append($('<td />', { colspan: 16, align: 'center', 'text-align': 'center', title: i18n('Close Modules') }).click(function (event) {
      $('#modules_value_div_id').css({ display: 'none' });
      event.preventDefault();
    }).css({ color: 'white', 'background-color': '#C11E29', cursor: 'pointer' }).append("Modules")).appendTo(thead);

    tr = $('<tr />').append($('<td />', { align: 'left', 'text-align': 'center' }).append("Addr.")).appendTo(thead);
    $('<td />', { align: 'center' }).append("Group number<br>ACin-DCin-ACout").appendTo(tr);
    $('<td />', { align: 'left' }).append(i18n("Vers.")).appendTo(tr);
    $('<td />', { align: 'left' }).append(i18n("Serial")).appendTo(tr);
    $('<td />', { align: 'left' }).append(i18n("DC In")).appendTo(tr);
    $('<td />', { align: 'left' }).append(i18n("AC In")).appendTo(tr);
    $('<td />', { align: 'left' }).append(i18n("AC Out")).appendTo(tr);
    $('<td />', { colspan: 3, align: 'center' }).append(i18n("Output")).appendTo(tr);
    $('<td />', { colspan: 2, align: 'center' }).append("AC " + i18n("Input")).appendTo(tr);
    $('<td />', { colspan: 2, align: 'center' }).append("DC " + i18n("Input")).appendTo(tr);
    $('<td />', { align: 'left' }).append(i18n("Freq." + " " + i18n("Input"))).appendTo(tr);
    $('<td />', { align: 'left' }).append(i18n("Temp.")).appendTo(tr);

    for (var index = 0; index < 32; ++index) {
      if (valtable_obj.CETMOD_PRESENT[index] == 0) {
        continue;
      }
      tr = $('<tr />').append($('<td />', { id: 'modules_value_addr_' + index, align: 'left' }).append(valtable_obj.CETMOD_CONFADDR[index])).appendTo(tbody);

      $('<td />', { id: 'modules_value_grp_number_' + index, align: 'left' }).
      append(valtable_obj.CETMOD_GROUPAC[index] + '-' + valtable_obj.CETMOD_GROUPDC[index] + '-' + valtable_obj.CETMOD_PHASENUMBER[index]).appendTo(tr);

      $('<td />', { id: 'modules_value_swver_' + index, align: 'left' }).append(valtable_obj.CETMOD_SWVER[index]).appendTo(tr);

      $('<td />', { id: 'modules_value_serno_' + index, align: 'left' }).append(valtable_obj.CETMOD_SERNO[index]).appendTo(tr);

      img = $('<img />').attr({ 'id': 'modules_value_dc_in_img_' + index, 'class': 'img-rounded', src: get_led_2_3_4_6_img(valtable_obj.CETMOD_DCIPLED[index]) });
      $('<td />', { align: 'center' }).append(img).appendTo(tr);


      img = $('<img />').attr({ 'id': 'modules_value_ac_in_img_' + index, 'class': 'img-rounded', src: get_led_2_3_4_6_img(valtable_obj.CETMOD_ACIPLED[index]) });
      $('<td />', { align: 'center' }).append(img).appendTo(tr);

      img = $('<img />').attr({ 'id': 'modules_value_ac_out_img_' + index, 'class': 'img-rounded', src: get_led_2_3_4_6_img(valtable_obj.CETMOD_ACOPLED[index]) });
      $('<td />', { align: 'center' }).append(img).appendTo(tr);


      $('<td />', { id: 'modules_value_output_volt_' + index, align: 'left' }).
      append(get_value({ value: valtable_obj.CETMOD_OPVOLT[index] * 0.1, unit: ' V', precision: 1 })).appendTo(tr);

      $('<td />', { id: 'modules_value_output_curr_' + index, align: 'left' }).
      append(get_value({ value: valtable_obj.CETMOD_OPCURR[index] * 0.1, unit: ' A', precision: 1 })).appendTo(tr);

      $('<td />', { id: 'modules_value_output_powerva_' + index, align: 'left' }).
      append(get_value({ value: valtable_obj.CETMOD_OPPOWERVA[index], unit: ' VA', precision: 1 })).appendTo(tr);

      $('<td />', { id: 'modules_value_acinput_volt_' + index, align: 'left' }).
      append(get_value({ value: valtable_obj.CETMOD_ACIPVOLT[index] * 0.1, unit: ' V', precision: 1 })).appendTo(tr);

      $('<td />', { id: 'modules_value_acinput_power_' + index, align: 'left' }).
      append(get_value({ value: valtable_obj.CETMOD_ACIPPOWERVA[index], unit: ' VA', precision: 1 })).appendTo(tr);

      $('<td />', { id: 'modules_value_dcinput_volt_' + index, align: 'left' }).
      append(get_value({ value: valtable_obj.CETMOD_DCIPVOLT[index] * 0.1, unit: ' V', precision: 1 })).appendTo(tr);

      $('<td />', { id: 'modules_value_dcinput_curr_' + index, align: 'left' }).
      append(get_value({ value: valtable_obj.CETMOD_DCIPCURR[index] * 0.1, unit: ' A', precision: 1 })).appendTo(tr);

      $('<td />', { id: 'modules_value_input_freq_' + index, align: 'left' }).
      append(get_value({ value: valtable_obj.CETMOD_ACIPFREQ[index] * 0.1, unit: ' Hz', precision: 1 })).appendTo(tr);

      $('<td />', { id: 'modules_value_temp_' + index, align: 'left' }).
      append(get_value({ value: valtable_obj.CETMOD_TEMP[index], unit: ' &deg;C', precision: 1 })).appendTo(tr);
    }

    fragment.appendChild(modules_value_div[0]);
    document.body.appendChild(fragment);

    new utils_api.draggable('modules_value_div_id');
  }

  function set_modules_value() {
    for (var index = 0; index < 32; ++index) {
      if (valtable_obj.CETMOD_PRESENT[index] == 0) {
        continue;
      }
      $('#modules_value_addr_' + index).val(valtable_obj.CETMOD_CONFADDR[index]);

      $('#modules_value_grp_number_' + index).
      val(valtable_obj.CETMOD_GROUPAC[index] + '-' + valtable_obj.CETMOD_GROUPDC[index] + '-' + valtable_obj.CETMOD_PHASENUMBER[index]);

      $('#modules_value_swver_' + index).val(valtable_obj.CETMOD_SWVER[index]);

      $('#modules_value_serno_' + index).val(valtable_obj.CETMOD_SERNO[index]);

      $('#modules_value_dc_in_img_' + index).attr('src', get_led_2_3_4_6_img(valtable_obj.CETMOD_DCIPLED[index]));

      $('#modules_value_ac_in_img_' + index).attr('src', get_led_2_3_4_6_img(valtable_obj.CETMOD_ACIPLED[index]));

      $('#modules_value_ac_out_img_' + index).attr('src', get_led_2_3_4_6_img(valtable_obj.CETMOD_ACOPLED[index]));

      $('#modules_value_output_volt_' + index).
      val(get_value({ value: valtable_obj.CETMOD_OPVOLT[index] * 0.1, unit: ' V', precision: 1 }));

      $('#modules_value_output_curr_' + index).
      val(get_value({ value: valtable_obj.CETMOD_OPCURR[index] * 0.1, unit: ' A', precision: 1 }));

      $('#modules_value_output_powerva_' + index).
      val(get_value({ value: valtable_obj.CETMOD_OPPOWERVA[index], unit: ' VA', precision: 1 }));

      $('#modules_value_acinput_volt_' + index).
      val(get_value({ value: valtable_obj.CETMOD_ACIPVOLT[index] * 0.1, unit: ' V', precision: 1 }));

      $('#modules_value_acinput_power_' + index).
      val(get_value({ value: valtable_obj.CETMOD_ACIPPOWERVA[index], unit: ' VA', precision: 1 }));

      $('#modules_value_dcinput_volt_' + index).
      val(get_value({ value: valtable_obj.CETMOD_DCIPVOLT[index] * 0.1, unit: ' V', precision: 1 }));

      $('#modules_value_dcinput_curr_' + index).
      val(get_value({ value: valtable_obj.CETMOD_DCIPCURR[index] * 0.1, unit: ' A', precision: 1 }));

      $('#modules_value_input_freq_' + index).
      val(get_value({ value: valtable_obj.CETMOD_ACIPFREQ[index] * 0.1, unit: ' Hz', precision: 1 }));

      $('#modules_value_temp_' + index).
      html(get_value({ value: valtable_obj.CETMOD_TEMP[index], unit: ' &deg;C', precision: 1 }));
    }
  }

  function send_alarm_history_cmd(priority) {

    if (priority == "2") {
      alarm_title = "T2S " + i18n("Major Alarms");
    } else
    alarm_title = "T2S " + i18n("Minor Alarms");

    return get_alarm_page_from_ups({ unitid: '', priority: priority, sortmode: '', outputType: 1, inputType: 1, lines: '' }, { title: alarm_title });
  }

  function get_mpb_img() {
    var value = null;

    if (valtable_obj.CET_EV1AUX == 'MBP ENGAGED') {
      value = valtable_obj.CET_1AUX;
    } else if (valtable_obj.CET_EV2AUX == 'MBP ENGAGED') {
      value = valtable_obj.CET_2AUX;
    }

    if (value == 1) {
      return './image/ce_t/mbp_on.jpg';
    } else if (value == 0) {
      return './image/ce_t/mbp_off.jpg';
    }

    return './image/ce_t/mbp.png';
  }

  function create_phases_grb_bnt() {
    var fragment = document.createDocumentFragment(),
    div;

    css_obj = {
      position: 'absolute',
      top: y_pos + 199,
      left: x_pos + 557,
      width: 158,
      height: 102,
      'background-color': 'transparent',
      cursor: 'pointer'
    };
    div = $('<div/>', { 'class': 'img-rounded', title: i18n('Show Phases Groups') }).css(css_obj);

    div.click(function (event) {
      var phases_div = $('#phases_div_id');
      phases_div.css('display',
        phases_div.css('display') == 'none' ? 'block' : 'none');
      event.preventDefault();
    });

    fragment.appendChild(div[0]);
    document.body.appendChild(fragment);
  }

  function create_phases_grb_panel() {
    var fragment = document.createDocumentFragment(),
    tr, table, tbody, thead, phases_div;

    css_obj = {
      position: 'absolute',
      top: y_pos + 225,
      left: x_pos + 540,
      'display': 'none',
      border: 'solid 1px #808080',
      'z-index': '44',
      'background-color': '#cccccc'
    };

    phases_div = $('<div/>', { id: 'phases_div_id', 'class': 'img-rounded' }).css(css_obj);

    table = $('<table />', { id: 'phases_div_table', 'class': 'table table-condensed table-hover  table-bordered' }).appendTo(phases_div);
    table.css({ 'font-size': '9pt', 'font-weight': 'bold', 'font-family': 'arial', 'border-collapse': 'initial' });

    tbody = $('<tbody />').appendTo(table);
    thead = $('<thead />').appendTo(table);

    tr = $('<tr />').append($('<td />', { colspan: 5, align: 'center', 'text-align': 'center', title: i18n('Close Phases Groups') }).click(function (event) {
      $('#phases_div_id').css({ display: 'none' });
      event.preventDefault();
    }).css({ color: 'white', 'background-color': '#C11E29', cursor: 'pointer' }).append(i18n("Phases Groups"))).appendTo(thead);

    tr = $('<tr />').append($('<td />', { align: 'left', 'text-align': 'center' }).append(" ")).appendTo(thead);
    $('<td />', { align: 'left' }).append("Voltage").appendTo(tr);
    $('<td />', { align: 'left' }).append("Frequency").appendTo(tr);
    $('<td />', { align: 'left' }).append("Load W").appendTo(tr);
    $('<td />', { align: 'left' }).append("Load VA").appendTo(tr);

    for (var index = 0; index < 8; ++index) {
      tr = $('<tr />').append($('<td />', { align: 'left' }).append(index + 1 + ':')).appendTo(tbody);
      $('<td />', { id: 'phases_grp_volt_' + (index + 1), align: 'right' }).append(get_value({ value: valtable_obj.CETPHASE_OPVOLT[index] * 0.1, unit: ' V', precision: 1 })).appendTo(tr);
      $('<td />', { id: 'phases_grp_freq_' + (index + 1), align: 'right' }).append(get_value({ value: valtable_obj.CETPHASE_OPFREQ[index] * 0.1, unit: ' Hz', precision: 1 })).appendTo(tr);
      $('<td />', { id: 'phases_grp_loadW_' + (index + 1), align: 'right' }).append(get_value({ value: valtable_obj.CETPHASE_RATIOINSTW[index], unit: ' %', precision: 0 })).appendTo(tr);
      $('<td />', { id: 'phases_grp_loadva_' + (index + 1), align: 'right' }).append(get_value({ value: valtable_obj.CETPHASE_RATIOINSTVA[index], unit: ' %', precision: 0 })).appendTo(tr);
    }

    fragment.appendChild(phases_div[0]);
    document.body.appendChild(fragment);

    new utils_api.draggable('phases_div_id');
  }

  function set_phases_grb() {
    for (var index = 0; index < 8; ++index) {
      $('#phases_grp_volt_' + (index + 1)).val(get_value({ value: valtable_obj.CETPHASE_OPVOLT[index] * 0.1, unit: ' V', precision: 1 }));
      $('#phases_grp_freq_' + (index + 1)).val(get_value({ value: valtable_obj.CETPHASE_OPFREQ[index] * 0.1, unit: ' Hz', precision: 1 }));
      $('#phases_grp_loadW_' + (index + 1)).val(get_value({ value: valtable_obj.CETPHASE_RATIOINSTW[index], unit: ' %', precision: 0 }));
      $('#phases_grp_loadva_' + (index + 1)).val(get_value({ value: valtable_obj.CETPHASE_RATIOINSTVA[index], unit: ' %', precision: 0 }));
    }
  }

  function create_dc_inp_groups_bnt() {
    var fragment = document.createDocumentFragment(),
    div;

    css_obj = {
      position: 'absolute',
      top: y_pos + 395,
      left: x_pos + 317,
      width: 160,
      height: 102,
      'background-color': 'transparent',
      cursor: 'pointer'
    };
    div = $('<div/>', { 'class': 'img-rounded', title: i18n('Show DC Groups') }).css(css_obj);

    div.click(function (event) {
      var dc_inp_grp_div = $('#dc_inp_grp_div_id');
      dc_inp_grp_div.css('display',
        dc_inp_grp_div.css('display') == 'none' ? 'block' : 'none');
      event.preventDefault();
    });


    fragment.appendChild(div[0]);
    document.body.appendChild(fragment);
  }

  function create_dc_inp_groups_panel() {
    var fragment = document.createDocumentFragment(),
    tr, table, tbody, thead, dc_inp_grp_div;

    css_obj = {
      position: 'absolute',
      top: y_pos + 405,
      left: x_pos + 345,
      'display': 'none',
      border: 'solid 1px #808080',
      'z-index': '44',
      'background-color': '#cccccc'
    };

    dc_inp_grp_div = $('<div/>', { id: 'dc_inp_grp_div_id', 'class': 'img-rounded' }).css(css_obj);

    table = $('<table />', { id: 'dc_inp_grp_table', 'class': 'table table-condensed table-hover  table-bordered' }).appendTo(dc_inp_grp_div);
    table.css({ 'font-size': '9pt', 'font-weight': 'bold', 'font-family': 'arial', 'border-collapse': 'initial' });

    tbody = $('<tbody />').appendTo(table);
    thead = $('<thead />').appendTo(table);

    tr = $('<tr />').append($('<td />', { colspan: 2, align: 'center', 'text-align': 'center', title: i18n('Close DC Input Groups') }).click(function (event) {
      $('#dc_inp_grp_div_id').css({ display: 'none' });
      event.preventDefault();
    }).css({ color: 'white', 'background-color': '#C11E29', cursor: 'pointer' }).append(i18n("DC Input Groups"))).appendTo(thead);

    tr = $('<tr />').appendTo(thead);
    $('<td />', { align: 'center', colspan: 2 }).append("Voltage").appendTo(tr);

    for (var index = 0; index < 8; ++index) {
      tr = $('<tr />').append($('<td />', { align: 'left' }).append(index + 1 + ':')).appendTo(tbody);
      $('<td />', { id: 'dc_inp_grp_volt_' + (index + 1), align: 'right' }).append(get_value({ value: valtable_obj.CETDC_DCIPVOLT[index] * 0.1, unit: ' V', precision: 1 })).appendTo(tr);
    }

    fragment.appendChild(dc_inp_grp_div[0]);
    document.body.appendChild(fragment);

    new utils_api.draggable('dc_inp_grp_div_id');
  }

  function set_dc_inp_groups() {
    for (var index = 0; index < 8; ++index) {
      $('#dc_inp_grp_volt_' + (index + 1)).val(get_value({ value: valtable_obj.CETDC_DCIPVOLT[index] * 0.1, unit: ' V', precision: 1 }));
    }
  }

  function create_dc_inp_groups_info_panel() {
    var fragment = document.createDocumentFragment(),
    tr, table, tbody, thead, dc_inp_grp_info_div;

    css_obj = {
      position: 'absolute',
      top: y_pos + 336,
      left: x_pos + 426,
      'display': 'none',
      border: 'solid 1px #808080',
      'z-index': '44',
      'background-color': '#cccccc'
    };

    dc_inp_grp_info_div = $('<div/>', { id: 'dc_inp_grp_info_div_id', 'class': 'img-rounded' }).css(css_obj);

    table = $('<table />', { id: 'dc_inp_grp_info_table', 'class': 'table table-condensed table-hover  table-bordered' }).appendTo(dc_inp_grp_info_div);
    table.css({ 'font-size': '9pt', 'font-weight': 'bold', 'font-family': 'arial', 'border-collapse': 'initial' });

    tbody = $('<tbody />').appendTo(table);
    thead = $('<thead />').appendTo(table);

    tr = $('<tr />').append($('<td />', { colspan: 3, align: 'center', 'text-align': 'center', title: i18n('Close DC Input Groups') }).click(function (event) {
      $('#dc_inp_grp_info_div_id').css({ display: 'none' });
      event.preventDefault();
    }).css({ color: 'white', 'background-color': '#C11E29', cursor: 'pointer' }).append(i18n("DC Input Groups"))).appendTo(thead);


    tr = $('<tr />').append($('<td />', { align: 'left', 'text-align': 'center' }).append(" ")).appendTo(thead);
    $('<td />', { align: 'center' }).append("Power").appendTo(tr);
    $('<td />', { align: 'center' }).append("Current").appendTo(tr);

    for (var index = 0; index < 8; ++index) {
      tr = $('<tr />').append($('<td />', { align: 'left' }).append(index + 1 + ':')).appendTo(tbody);
      $('<td />', { id: 'dc_inp_grp_power_' + (index + 1), align: 'right' }).append(get_value({ value: valtable_obj.CETDC_DCIPPOWERW[index], unit: ' W', precision: 0 })).appendTo(tr);
      $('<td />', { id: 'dc_inp_grp_curr_' + (index + 1), align: 'right' }).append(get_value({ value: valtable_obj.CETDC_DCIPCURR[index] * 0.1, unit: ' A', precision: 1 })).appendTo(tr);
    }

    fragment.appendChild(dc_inp_grp_info_div[0]);
    document.body.appendChild(fragment);

    new utils_api.draggable('dc_inp_grp_info_div_id');
  }

  function set_dc_inp_groups_info() {
    for (var index = 0; index < 8; ++index) {
      $('#dc_inp_grp_power_' + (index + 1)).val(get_value({ value: valtable_obj.CETDC_DCIPPOWERW[index], unit: ' W', precision: 1 }));
      $('#dc_inp_grp_curr_' + (index + 1)).val(get_value({ value: valtable_obj.CETDC_DCIPCURR[index] * 0.1, unit: 'AV', precision: 1 }));
    }
  }

  function create_dc_inp_groups_info_bnt() {
    var fragment = document.createDocumentFragment(),
    div;

    css_obj = {
      position: 'absolute',
      top: y_pos + 330,
      left: x_pos + 380,
      width: 50,
      height: 50,
      'background-color': 'transparent',
      cursor: 'pointer'
    };
    div = $('<div/>', { 'class': 'img-rounded', title: i18n('Show DC Input Groups') }).css(css_obj);

    div.click(function (event) {
      var inp_groups_info_div = $('#dc_inp_grp_info_div_id');
      inp_groups_info_div.css('display',
        inp_groups_info_div.css('display') == 'none' ? 'block' : 'none');
      event.preventDefault();
    });


    fragment.appendChild(div[0]);
    document.body.appendChild(fragment);
  }

  function create_phases_grb_info_panel() {
    var fragment = document.createDocumentFragment(),
    tr, table, tbody, thead, phases_grb_info_div;

    css_obj = {
      position: 'absolute',
      top: y_pos + 260,
      left: x_pos + 542,
      'display': 'none',
      border: 'solid 1px #808080',
      'z-index': '44',
      'background-color': '#cccccc'
    };

    phases_grb_info_div = $('<div/>', { id: 'phases_grb_info_div_id', 'class': 'img-rounded' }).css(css_obj);

    table = $('<table />', { id: 'dc_inp_grp_info_table', 'class': 'table table-condensed table-hover  table-bordered' }).appendTo(phases_grb_info_div);
    table.css({ 'font-size': '9pt', 'font-weight': 'bold', 'font-family': 'arial', 'border-collapse': 'initial' });

    tbody = $('<tbody />').appendTo(table);
    thead = $('<thead />').appendTo(table);

    tr = $('<tr />').append($('<td />', { colspan: 4, align: 'center', 'text-align': 'center', title: i18n('Close Phases Groups') }).click(function (event) {
      $('#phases_grb_info_div_id').css({ display: 'none' });
      event.preventDefault();
    }).css({ color: 'white', 'background-color': '#C11E29', cursor: 'pointer' }).append(i18n("Phases Groups"))).appendTo(thead);


    tr = $('<tr />').append($('<td />', { align: 'left', 'text-align': 'center' }).append(" ")).appendTo(thead);
    $('<td />', { align: 'center', colspan: 2 }).append("Power").appendTo(tr);
    $('<td />', { align: 'center' }).append("Current").appendTo(tr);

    for (var index = 0; index < 8; ++index) {
      tr = $('<tr />').append($('<td />', { align: 'left' }).append(index + 1 + ':')).appendTo(tbody);
      $('<td />', { id: 'phases_grb_info_powerw_' + (index + 1), align: 'right' }).append(get_value({ value: valtable_obj.CETPHASE_OPPOWERW[index], unit: ' W', precision: 0 })).appendTo(tr);
      $('<td />', { id: 'phases_grb_info_powerva_' + (index + 1), align: 'right' }).append(get_value({ value: valtable_obj.CETPHASE_OPPOWERVA[index], unit: ' VA', precision: 0 })).appendTo(tr);
      $('<td />', { id: 'phases_grb_info_curr_' + (index + 1), align: 'right' }).append(get_value({ value: valtable_obj.CETPHASE_OPCURR[index] * 0.1, unit: ' A', precision: 1 })).appendTo(tr);
    }

    fragment.appendChild(phases_grb_info_div[0]);
    document.body.appendChild(fragment);

    new utils_api.draggable('phases_grb_info_div_id');
  }

  function set_phases_grb_info() {
    for (var index = 0; index < 8; ++index) {
      $('#phases_grb_info_powerw_' + (index + 1)).val(get_value({ value: valtable_obj.CETPHASE_OPPOWERW[index], unit: ' W', precision: 1 }));
      $('#phases_grb_info_powerva_' + (index + 1)).val(get_value({ value: valtable_obj.CETPHASE_OPPOWERVA[index], unit: 'AV', precision: 1 }));
      $('#phases_grb_info_curr_' + (index + 1)).val(get_value({ value: valtable_obj.CETPHASE_OPCURR[index] * 0.1, unit: 'A', precision: 1 }));
    }
  }

  function create_phases_grb_info_bnt() {
    var fragment = document.createDocumentFragment(),
    div;

    css_obj = {
      position: 'absolute',
      top: y_pos + 224,
      left: x_pos + 498,
      width: 50,
      height: 50,
      'background-color': 'transparent',
      cursor: 'pointer'
    };
    div = $('<div/>', { 'class': 'img-rounded', title: 'Show Phases Groups' }).css(css_obj);

    div.click(function (event) {
      var phases_grb_info_div = $('#phases_grb_info_div_id');
      phases_grb_info_div.css('display',
        phases_grb_info_div.css('display') == 'none' ? 'block' : 'none');
      event.preventDefault();
    });

    fragment.appendChild(div[0]);
    document.body.appendChild(fragment);
  }

  function get_led_2_3_4_6_img(value) {
    if (value == 1)
    return './image/ce_t/led_gray.gif';
    if (value == 2)
    return './image/ce_t/led_green.gif';
    if (value == 3)
    return './image/ce_t/led_blgreen.gif';
    if (value == 4)
    return './image/ce_t/led_blorange_green.gif';
    if (value == 5)
    return './image/ce_t/led_orange.gif';
    if (value == 6)
    return './image/ce_t/led_blorange.gif';
    if (value == 7)
    return './image/ce_t/led_blred.gif';
    if (value == 8)
    return './image/ce_t/led_red.gif';

    return './image/ce_t/led_gray.gif'
  }

  function get_led_1_5_img(value) {
    if (value == 2)
    return './image/ce_t/led_green.gif';
    return './image/ce_t/led_red.gif'
  }


  function get_major_minor_alarm_img(value) {
    if (value == 0)
    return './image/ce_t/led_green.gif';
    return './image/ce_t/led_red.gif'
  }

  function get_prog_alarm_img() {
    var value = valtable_obj.CET_PROGALARM;

    if (value == 255)
    return './image/ce_t/led_gray.gif';
    else if (value == 0)
    return './image/ce_t/led_green.gif';
    else
    return './image/ce_t/led_red.gif';
  }


  function get_aux_img(value) {
    if (value == 0)
    return './image/ce_t/aux_off.gif';
    return './image/ce_t/aux_on.gif'
  }

  function get_src_pos() {
    var value = valtable_obj.CETMISC_SYSLOADPOS;

    if (value == 0)
    return i18n("Load on") + " AC";
    else if (value == 100)
    return i18n("Load on") + " DC";
    else if (value == 50)
    return i18n("Load Mixed");
    else
    return i18n("Load Unknown");
  }

  function get_tsi_type() {
    var value = valtable_obj.CETMOD_NOMOPPOWERVA[0];

    if (value == 600)
    return "Veda";
    else if (value == 750)
    return "Nova";
    else if (value == 1500)
    return "Media";
    else if (value == 2500)
    return "Bravo";
    else if (value == 20000)
    return "Agil";
    else
    return "unknown (" + value + " VA)";
  }

  function create_footer_pane() {
    if (!oid || !unms_obj)
    return true;
    footer_api.create({ //Attr
        id: 'panel2'
        //'class':'unmsFooter'
      }, { //css
        position: 'absolute',
        top: '620px',
        left: '76px'
      },
      unms_obj
    );
  }

  function get_value(val_obj) {
    var val = null;
    if (val_obj.value == null || val_obj.value == -9999)
    return 'n/a';

    if (val_obj.unit == ' &deg;C') //Temperature?
    {
      return panes_api.get_temperature(val_obj.value, valtable_obj._tempUnit, val_obj.precision);
    }

    /*if(Math.round(val_obj.value) == val_obj.value)
       val = val.toString();
    else*/
    val = val_obj.value.toFixed(val_obj.precision);
    val += val_obj.unit;
    return val;
  }


}(jQuery));