/*
   cyber5_conf_panel.js - Cyber 5 Configuration screen.
   Copyright (c) 2014, Generex GmbH. All rights reserved.
   Said.
*/
var cyber5_conf_api = (function ($) {
  var cb = null,
  cb_cmd = null,
  valtable_obj,
  report_obj,
  oid = null,
  err_div = null,
  pannes_created = false,
  check_bowser = true,
  css_obj, attr_obj,
  ups_protocol,
  curr_ups_cmd = null,
  CYBER5_hex = '80D9',
  CYBER5_dec = '32985',
  x_pos = 10,
  y_pos = 10;

  var vorhang = null;

  var req_params = {
    url: get_url("data.cgi"),
    data: { "json": "ups" },
    data_type: 'json',
    refresh_tm: 10
  };

  var ups_cmd_option = {
    data: {},
    data_type: 'json',
    success: ups_cmd_update,
    error: err_funct
  };

  /*public functions*/
  return {
    init: init
  };

  function init() {
    if (check_bowser && !utils_api.check_bowser()) {
      $('#loading_id').css('display', 'none');
      panes_api.create_br_check_panel(this, browser_check_ok);
      return true;
    }
    create_err_msg_panel();
    cb = jqmodule.subscribe(this, { update: update, error: err_funct }, req_params);
  }

  function browser_check_ok() {
    check_bowser = false;
    init();
  }

  function create_err_msg_panel() {
    css_obj = { position: 'absolute', top: 10, left: 10, width: 724, height: 62, display: 'none', 'z-index': 10 };
    err_div = panes_api.create_err_panel({ id: 'error_div', 'class': 'alert alert-error' }, css_obj);
  }

  function err_funct(response) {
    jqmodule.refresh(cb, 10 * 1000);
    panes_api.display_err({ response: response, oid: oid, err_div: err_div, err_msg_div_id: 'error_msg_div' });
  }

  function update(data) {
    if (data.length == 0) {
      return;
    }

    jqmodule.unsubscribe(cb);
    valtable_obj = data.ups.valtable;
    if (valtable_obj._upsProtocol == null) {
      ups_protocol = valtable_obj.CONNECT;
      ups_protocol = (ups_protocol & 65535);
    } else
    ups_protocol = valtable_obj._upsProtocol;
    report_obj = data.report;

    if (err_div)
    err_div.css('display', 'none');

    jqmodule.refresh(cb, report_obj.refresh * 1000);
    if (!pannes_created) {
      window.resizeTo(760, 520);
      pannes_created = true;
      utils_api.setTitle(valtable_obj._platform);
      create_pannes(); //create screen.
      return false;
    }

    document.body.style.cursor = 'auto';
    $("span").prop("disabled", false);
    $("input").prop("disabled", false);
    $("button").prop("disabled", false);
    set_values(); //set values.
  }

  function create_bg() {
    var fragment = document.createDocumentFragment(),
    img_div;
    css_obj = { position: 'absolute', top: y_pos, left: x_pos, width: 712, height: 380, border: 'solid 1px #808080' };
    img_div = $('<div/>', { 'class': 'img-rounded' }).css(css_obj);
    fragment.appendChild(img_div[0]);
    document.body.appendChild(fragment);
  }

  function create_conf_title() {
    var title_div, a;
    css_obj = { position: 'absolute', top: y_pos, left: x_pos, align: 'center', width: 712 }; // 'background-color': '#3a87ad'
    attr_obj = { id: 'model_div_id', 'class': 'list-group' };
    title_div = panes_api.create_div(attr_obj, css_obj);
    a = $('<a/>', { 'class': 'list-group-item active' }).append(i18n('UPS Configuration')).appendTo(title_div);
  }

  function create_pannes() //create screen.
  {
    css_obj = { position: 'absolute', top: y_pos, left: x_pos, 'width': 712, 'height': 490, 'background-color': 'transparent', 'visibility': 'hidden', 'z-index': 100 };
    attr_obj = { id: 'vorhang_div_id' };
    vorhang = panes_api.create_div(attr_obj, css_obj);

    create_bg();
    create_conf_title();
    create_cmd_info_panel(); //create UPS command info panel
    create_utilty_pow_fail_cond_panel(); //create Utilty Power Failure Condition
    create_battery_panel(); //create Battery panel
    create_system_panel(); //create System panel
    create_ncl_outtlet_panel(); //create NCL Outlet panel
  }

  function set_values() //set values.
  {
    $('#vorhang_div_id').css('visibility', 'hidden');
    set_utilty_pow_fail_cond(); //Set Utilty Power Failure Condition
    set_battery(); //set Battery
    set_system(); //set System
    set_ncl_outtlet(); //set NCL Outlet
  }

  function create_cmd_info_panel() //create UPS command info panel
  {
    var cmd_state_info_div, cmd_txt_info_div;

    css_obj = { position: 'absolute', 'top': y_pos + 390, 'left': x_pos, 'width': 712, 'text-align': 'center', display: 'none', 'z-index': 10 };
    attr_obj = { id: 'cmd_state_div_id', 'class': 'alert alert-error' };
    cmd_state_info_div = panes_api.create_div(attr_obj, css_obj);
    cmd_state_info_div.append(i18n('Infos'));
  }

  function set_cmd_info(error) //set UPS command info
  {
    var cmd_state_info_div = $('#cmd_state_div_id');

    if (error != 0) {
      cmd_state_info_div.removeClass('alert-success');
      cmd_state_info_div.addClass('alert-danger');
    } else {
      cmd_state_info_div.removeClass('alert-danger');
      cmd_state_info_div.addClass('alert-success');
    }
    cmd_state_info_div.text(curr_ups_cmd);
    cmd_state_info_div.css({ 'display': 'block' });

    setTimeout(function () { cmd_state_info_div.css({ 'display': 'none' }); }, 6000);
  }

  function send_ups_cmd(ups_cmd_parameters) //send UPS Command
  {
    $("span").attr("disabled", "disabled");
    $("input").attr("disabled", "disabled");
    $("button").attr("disabled", "disabled");
    $('#vorhang_div_id').css('visibility', 'visible');
    document.body.disable = true;
    document.body.style.cursor = 'wait';
    var url = get_url("data.cgi");
    ups_cmd_option.data = ups_cmd_parameters;
    jqmodule.ajax(url, ups_cmd_option);
  }

  function ups_cmd_update(data) //set UPS command info
  {
    set_cmd_info(data.ups_cmd.error); //set UPS command info
    cb = jqmodule.subscribe(this, { update: update, error: err_funct }, req_params);
  }

  function create_utilty_pow_fail_cond_panel() //create Utilty Power Failure Condition
  {
    var fragment = document.createDocumentFragment(),
    div, tr, td, table, tbody, button, input;

    css_obj = { 'position': 'absolute', 'top': y_pos + 50, 'left': x_pos + 10, 'width': 340, 'height': 154, 'text-align': 'center', 'border': '1px solid black', 'background-color': 'transparent' };
    div = $('<div/>').attr({ 'id': 'utilty_pow_fail_cond_id' }).css(css_obj);

    table = $('<table/>', {
      'class': 'table table-condensed',
      'align': 'center',
      cellpadding: 4,
      cellspacing: 1,
      'text-align': 'center',
      'width': '100%',
      height: '100%'
    }).css({ 'margin-top': '4px' }).appendTo(div);
    tbody = $('<tbody />').appendTo(table);
    tr = $('<tr />').appendTo(tbody);
    td = $('<td />', { 'align': 'center', 'colspan': 2 }).
    css({ 'font-weight': 'bold', 'text-align': 'center' }).append(i18n('Utilty Power Failure Condition')).appendTo(tr);

    //High Voltage Threshold
    tr = $('<tr />').appendTo(tbody);
    td = $('<td />', { align: 'left' }).css('vertical-align', 'top').append(i18n('High Voltage Threshold') + '[V]').appendTo(tr);
    td = $('<td />', { align: 'center' }).appendTo(tr);
    input = $('<input />', { id: 'hi_volt_thres_id', type: 'text', maxlength: 3 }).css({ width: '80px', 'text-align': 'center' }).
    val(valtable_obj.HIGHVOLTTHRES).appendTo(td);
    input.on("keypress", function (event) { return utils_api.onKeyPressNumbers(event); });

    //low Voltage Threshold
    tr = $('<tr />').appendTo(tbody);
    td = $('<td />', { align: 'left' }).css('vertical-align', 'top').append(i18n('Low Voltage Threshold') + '[V]').appendTo(tr);
    td = $('<td />', { align: 'center' }).appendTo(tr);
    input = $('<input />', { id: 'low_volt_thres_id', type: 'text', maxlength: 3 }).css({ width: '80px', 'text-align': 'center' }).
    val(valtable_obj.LOWVOLTTHRES).appendTo(td);
    input.on("keypress", function (event) { return utils_api.onKeyPressNumbers(event); });

    //Voltage Threshold button
    tr = $('<tr />');
    td = $('<td />', { 'align': 'center', 'colspan': 2 }).append('').appendTo(tr);
    button = $('<button>', { id: "ups_set_volt_thres_id", 'class': "btn btn-default btn-sm" }).append(i18n('Apply')).appendTo(td);
    button.css({ width: 115 });
    button.on("click",function (event) {
      var hi_volt_thres = $('#hi_volt_thres_id').val(),
      low_volt_thres = $('#low_volt_thres_id').val();

      if (hi_volt_thres.empty() || low_volt_thres.empty())
      return false;

      curr_ups_cmd = 'Set Voltage Threshold';
      send_ups_cmd({ type: "ups_cmd", cmd: 'UPS_EXE_COMMAND', p1: 2109, p2: low_volt_thres + '|' + hi_volt_thres });
      event.preventDefault();
    });
    tr.appendTo(tbody);

    fragment.appendChild(div[0]);
    document.body.appendChild(fragment);
  }

  function set_utilty_pow_fail_cond() //Set Utilty Power Failure Condition
  {
    $('#hi_volt_thres_id').val(valtable_obj.HIGHVOLTTHRES);
    $('#low_volt_thres_id').val(valtable_obj.LOWVOLTTHRES);
  }

  function create_battery_panel() //create Battery panel
  {
    var fragment = document.createDocumentFragment(),
    div, tr, td, table, tbody, button, input;

    css_obj = { 'position': 'absolute', 'top': y_pos + 210, 'left': x_pos + 10, 'width': 340, 'height': 154, 'text-align': 'center', 'border': '1px solid black', 'background-color': 'transparent' };
    div = $('<div/>').attr({ 'id': 'battery_id' }).css(css_obj);

    table = $('<table/>', {
      'class': 'table table-condensed',
      'align': 'center',
      cellpadding: 4,
      cellspacing: 1,
      'text-align': 'center',
      'width': '100%',
      height: '100%'
    }).css({ 'margin-top': '4px' }).appendTo(div);
    tbody = $('<tbody />').appendTo(table);
    tr = $('<tr />').appendTo(tbody);
    td = $('<td />', { 'align': 'center', 'colspan': 2 }).
    css({ 'font-weight': 'bold', 'text-align': 'center' }).append(i18n('Battery')).appendTo(tr);

    //Low Battery Threshold
    tr = $('<tr />').appendTo(tbody);
    td = $('<td />', { align: 'left' }).css('vertical-align', 'top').append(i18n('Low Battery Threshold') + '[%]').appendTo(tr);
    td = $('<td />', { align: 'center' }).appendTo(tr);
    input = $('<input />', { id: 'low_battery_thres_id', type: 'text', maxlength: 3 }).css({ width: '80px', 'text-align': 'center' }).
    val(valtable_obj.LOWBATTTHRESPERCENT).appendTo(td);
    input.on("keypress", function (event) { return utils_api.onKeyPressNumbers(event); });

    //Extended Battery Pack
    tr = $('<tr />').appendTo(tbody);
    td = $('<td />', { align: 'left' }).css('vertical-align', 'top').append(i18n('Extended Battery Pack')).appendTo(tr);
    td = $('<td />', { align: 'center' }).appendTo(tr);
    input = $('<input />', { id: 'ext_battery_pack_id', type: 'text', maxlength: 3 }).css({ width: '80px', 'text-align': 'center' }).
    val(valtable_obj.EXTBATTPACKS).appendTo(td);
    input.on("keypress", function (event) { return utils_api.onKeyPressNumbers(event); });

    //Voltage Threshold button
    tr = $('<tr />');
    td = $('<td />', { 'align': 'center', 'colspan': 2 }).append('').appendTo(tr);
    button = $('<button>', { id: "ups_set_volt_thres_id", 'class': "btn btn-default btn-sm" }).append(i18n('Apply')).appendTo(td);
    button.css({ width: 115 });
    button.on("click",function (event) {
      var low_battery_thres = $('#low_battery_thres_id').val(),
      battery_pack = $('#ext_battery_pack_id').val();

      if (low_battery_thres.empty() || battery_pack.empty())
      return false;

      curr_ups_cmd = 'Set Battery Threshold | Packs';
      send_ups_cmd({ type: "ups_cmd", cmd: 'UPS_EXE_COMMAND', p1: 2110, p2: low_battery_thres + '|' + battery_pack });
      event.preventDefault();
    });
    tr.appendTo(tbody);

    fragment.appendChild(div[0]);
    document.body.appendChild(fragment);
  }

  function set_battery() //set Battery
  {
    $('#low_battery_thres_id').val(valtable_obj.LOWBATTTHRESPERCENT),
    $('#ext_battery_pack_id').val(valtable_obj.EXTBATTPACKS);
  }

  function create_system_panel() //create System panel
  {
    var fragment = document.createDocumentFragment(),
    div, tr, td, table, tbody, button, text;

    css_obj = { 'position': 'absolute', 'top': y_pos + 50, 'left': x_pos + 360, 'width': 340, 'height': 116, 'text-align': 'center', 'border': '1px solid black', 'background-color': 'transparent' };
    div = $('<div/>').attr({ 'id': 'battery_id' }).css(css_obj);

    table = $('<table/>', {
      'class': 'table table-condensed',
      'align': 'center',
      cellpadding: 4,
      cellspacing: 1,
      'text-align': 'center',
      'width': '100%',
      height: '100%'
    }).css({ 'margin-top': '4px' }).appendTo(div);
    tbody = $('<tbody />').appendTo(table);
    tr = $('<tr />').appendTo(tbody);
    td = $('<td />', { 'align': 'center', 'colspan': 2 }).
    css({ 'font-weight': 'bold', 'text-align': 'center' }).append(i18n('System')).appendTo(tr);

    //Cold Start
    tr = $('<tr />').appendTo(tbody);
    td = $('<td />', { align: 'left' }).css('vertical-align', 'Middle').append(i18n('Cold Start')).appendTo(tr);
    //Cold Start button
    td = $('<td />', { 'align': 'center' }).appendTo(tr);
    button = $('<button>', { id: "sys_cold_start_id", 'class': "btn btn-default btn-sm" }).appendTo(td);
    button.css({ width: 115 });

    if (valtable_obj.COLDSTARTONOFF == 0) {
      button.removeClass('btn-success');
      button.addClass('btn-default');
      button.text(i18n('Enable'));
    } else {
      button.removeClass('btn-default');
      button.addClass('btn-success');
      button.text('Disable');
    }

    button.on("click",function (event) {
      var p2 = valtable_obj.COLDSTARTONOFF == 1 ? 0 : 1,
      p3 = valtable_obj.PWRFAILALRONOFF;
      curr_ups_cmd = 'Set Cold Start';
      send_ups_cmd({ type: "ups_cmd", cmd: 'UPS_EXE_COMMAND', p1: 2111, p2: p2 + '|' + p3 });
      event.preventDefault();
    });

    //Utility Power Faillure Alarm
    tr = $('<tr />').appendTo(tbody);
    td = $('<td />', { align: 'left' }).css('vertical-align', 'Middle').append(i18n('Utility Power Faillure Alarm')).appendTo(tr);
    //Utility Power Faillure Alarm button
    td = $('<td />', { 'align': 'center' }).appendTo(tr);
    button = $('<button>', { id: "sys_utility_fail_alarm_id", 'class': "btn btn-default btn-sm" }).appendTo(td);
    button.css({ width: 115 });

    if (valtable_obj.PWRFAILALRONOFF == 0) {
      button.removeClass('btn-success');
      button.addClass('btn-default');
      button.text('Enable');
    } else {
      button.removeClass('btn-default');
      button.addClass('btn-success');
      button.text('Disable');
    }

    button.on("click",function (event) {
      var p3 = valtable_obj.PWRFAILALRONOFF == 1 ? 0 : 1,
      p2 = valtable_obj.COLDSTARTONOFF;
      curr_ups_cmd = 'Set Utility Power Faillure Alarm';
      send_ups_cmd({ type: "ups_cmd", cmd: 'UPS_EXE_COMMAND', p1: 2111, p2: p2 + '|' + p3 });
      event.preventDefault();
    });

    fragment.appendChild(div[0]);
    document.body.appendChild(fragment);
  }

  function set_system() //set System
  {
    var button = $('#sys_cold_start_id');
    if (valtable_obj.COLDSTARTONOFF == 0) {
      button.removeClass('btn-success');
      button.addClass('btn-default');
      button.text('Enable');
    } else {
      button.removeClass('btn-default');
      button.addClass('btn-success');
      button.text('Disable');
    }

    button = $('#sys_utility_fail_alarm_id');
    if (valtable_obj.PWRFAILALRONOFF == 0) {
      button.removeClass('btn-success');
      button.addClass('btn-default');
      button.text('Enable');
    } else {
      button.removeClass('btn-default');
      button.addClass('btn-success');
      button.text('Disable');
    }
  }

  function create_ncl_outtlet_panel() //create NCL Outlet panel
  {
    var fragment = document.createDocumentFragment(),
    div, tr, td, table, tbody, button, input;

    css_obj = { 'position': 'absolute', 'top': y_pos + 172, 'left': x_pos + 360, 'width': 340, 'height': 192, 'text-align': 'center', 'border': '1px solid black', 'background-color': 'transparent' };
    div = $('<div/>').attr({ 'id': 'battery_id' }).css(css_obj);

    table = $('<table/>', {
      'class': 'table table-condensed',
      'align': 'center',
      cellpadding: 4,
      cellspacing: 1,
      'text-align': 'center',
      'width': '100%',
      height: '100%'
    }).css({ 'margin-top': '4px' }).appendTo(div);
    tbody = $('<tbody />').appendTo(table);
    tr = $('<tr />').appendTo(tbody);
    td = $('<td />', { 'align': 'center', 'colspan': 2 }).
    css({ 'font-weight': 'bold', 'text-align': 'center' }).append('NCL ' + i18n('Outlet')).appendTo(tr);

    //Turn Off Threshold
    tr = $('<tr />').appendTo(tbody);
    td = $('<td />', { align: 'left' }).css('vertical-align', 'top').append(i18n('Turn Off Threshold') + '[%]').appendTo(tr);
    td = $('<td />', { align: 'center' }).appendTo(tr);
    input = $('<input />', { id: 'turn_off_thres_id', type: 'text', maxlength: 3 }).css({ width: '80px', 'text-align': 'center' }).
    val(valtable_obj.OFFTHRESPERCENT[1]).appendTo(td);
    input.on("keypress", function (event) { return utils_api.onKeyPressNumbers(event); });

    //Turn Off Delay
    tr = $('<tr />').appendTo(tbody);
    td = $('<td />', { align: 'left' }).css('vertical-align', 'top').append(i18n('Turn Off Delay') + '[Min]').appendTo(tr);
    td = $('<td />', { align: 'center' }).appendTo(tr);
    input = $('<input />', { id: 'turn_off_delay_id', type: 'text', maxlength: 3 }).css({ width: '80px', 'text-align': 'center' }).
    val(valtable_obj.OFFDELAY[1]).appendTo(td);
    input.on("keypress", function (event) { return utils_api.onKeyPressNumbers(event); });

    //Turn On Delay
    tr = $('<tr />').appendTo(tbody);
    td = $('<td />', { align: 'left' }).css('vertical-align', 'top').append(i18n('Turn On Delay') + '[Sec]').appendTo(tr);
    td = $('<td />', { align: 'center' }).appendTo(tr);
    input = $('<input />', { id: 'turn_on_delay_id', type: 'text', maxlength: 3 }).css({ width: '80px', 'text-align': 'center' }).
    val(valtable_obj.ONDELAY[1]).appendTo(td);
    input.on("keypress", function (event) { return utils_api.onKeyPressNumbers(event); });

    //NCL Outlet Button
    tr = $('<tr />');
    td = $('<td />', { 'align': 'center', 'colspan': 2 }).appendTo(tr);
    button = $('<button>', { id: "ups_set_ncl_outlet_id", 'class': "btn btn-default btn-sm" }).append(i18n('Apply')).appendTo(td);
    button.css({ width: 115 });
    button.on("click",function (event) {
      var outlet = valtable_obj.OUTLET_EXISTMASK & 2,
      on_delay = $('#turn_on_delay_id').val(),
      off_delay = $('#turn_off_delay_id').val(),
      off_thres = $('#turn_off_thres_id').val();

      if (on_delay.empty() || off_delay.empty() || off_thres.empty())
      return false;

      curr_ups_cmd = 'Set NCL Outlet';
      send_ups_cmd({ type: "ups_cmd", cmd: 'UPS_EXE_COMMAND', p1: 2225, p2: outlet + '|' + on_delay + '|' + off_delay + '|' + off_thres });
      event.preventDefault();
    });
    tr.appendTo(tbody);

    fragment.appendChild(div[0]);
    document.body.appendChild(fragment);
  }

  function set_ncl_outtlet() //set NCL Outlet
  {
    $('#turn_on_delay_id').val(valtable_obj.ONDELAY[1]),
    $('#turn_off_delay_id').val(valtable_obj.OFFDELAY[1]),
    $('#turn_off_thres_id').val(valtable_obj.OFFTHRESPERCENT[1]);
  }

}(jQuery));