/*
   dkmt_panel.js - DK Europe MT50 screen.
   Copyright (c) 2019, Generex GmbH. All rights reserved.
   UW.
*/

var dkmt_api = (function ($) {
  var oid = get_oid_from_url(),
    cb = null,
    panes_created = false,
    err_div = null,
    check_browser = true,
    css_obj, attr_obj,
    valtable_obj = null,
    report_obj = null,
    ups_obj = null,
    unms_obj = null,
    status, overload = 0,
    plattform = get_plattform(),
    ups_protocol = 0,
    prot_snmp = 0x803A,
    bg_height, bg_width,
    x_pos = 10, y_pos = 10;

  var status_obj = [
      { bit: 0x0001, txt: "The Bypass is presently engaged on the UPS" },
      { bit: 0x0002, txt: "UPS is in shutdown mode" },
      { bit: 0x0004, txt: "UPS Output active" },
      { bit: 0x0008, txt: "The UPS is drawing power from the batteries" },
      { bit: 0x0010, txt: "The remaining battery run-time is low" },
      { bit: 0x0020, txt: "A temperature is out of tolerance" },
      { bit: 0x0040, txt: "A test is in progress" },
      { bit: 0x0080, txt: "Input high" },
      { bit: 0x0100, txt: "The output load exceeds the UPS output capacity" },
      { bit: 0x0200, txt: "Inverter error" },
      { bit: 0x0400, txt: "One or more batteries have been determined to require replacement" },
      { bit: 0x0800, txt: "UPS in ECO mode" },
      { bit: 0x1000, txt: "Inverter warning" },
      { bit: 0x2000, txt: "A general fault in the UPS has been detected" },
      { bit: 0x4000, txt: "A problem has been encountered in the communications between the agent and the UPS" }
    ],
    ext_status_obj = [
      { bit: 0x00000008, txt: "The UPS will be unable to sustain the present load when and if the utility power is lost" },
      { bit: 0x00000040, txt: "An output condition is out of tolerance" },
      { bit: 0x00000200, txt: "The Bypass is out of tolerance" },
      { bit: 0x00000400, txt: "The UPS has shutdown as requested, i.e., the output is off" },
      { bit: 0x00000800, txt: "The entire UPS has shutdown as commanded" },
      { bit: 0x00001000, txt: "An uncorrected problem has been detected within the UPS charger subsystem" },
      { bit: 0x00002000, txt: "The output of the UPS is in the off state" },
      { bit: 0x00004000, txt: "The UPS system is in the off state" },
      { bit: 0x00008000, txt: "The failure of one or more fans in the UPS has been detected" },
      { bit: 0x00010000, txt: "The failure of one or more fuses has been detected" },
      { bit: 0x00040000, txt: "The result of the last diagnostic test indicates a failure" },
      { bit: 0x00100000, txt: "The UPS output is off and the UPS is awaiting the return of input power" },
      { bit: 0x00200000, txt: "An UPS Shutdown After Delay countdown is underway" },
      { bit: 0x00400000, txt: "The UPS will turn off power to the load immediatly" }
    ];

  var req_params = null;

  /*public functions*/
  return {
    init: init
  };

  function init() {
    if (check_browser && !utils_api.check_bowser()) {
      $('#loading_id').css('display', 'none');
      panes_api.create_br_check_panel(this, browser_check_ok);
      return true;
    }
    create_err_msg_panel();

    req_params = utils_api.get_req_params(utils_api.device_ups, oid);
    cb = jqmodule.subscribe(this, { update: update, error: err_funct }, req_params);
  }

  function browser_check_ok() {
    check_browser = false;
    init();
  }

  function create_err_msg_panel() {
    css_obj = { position: 'absolute', top: y_pos, left: x_pos, width: 791, display: 'none', 'z-index': 62 };
    err_div = panes_api.create_err_panel({ id: 'error_div', 'class': 'alert alert-danger' }, css_obj);
  }

  function err_funct(response) {
    jqmodule.refresh(cb, 10 * 1000);
    panes_api.display_err({ response: response, oid: oid, err_div: err_div, err_msg_div_id: 'error_msg_div' });
  }

  function update(data) {
    if (data.ups == null) {
      return;
    }

    set_variable(data);
    jqmodule.refresh(cb, report_obj.refresh * 1000);
    if (err_div)
      err_div.css('display', 'none');

    if (window.menu_api != null)
      menu_api.update(data);

    unms_obj = get_unms_obj(data);
    if (unms_obj != null) {
      info_head_api.update(unms_obj);
    }

    if (!panes_created) {
      panes_created = true;
      create_panes();
      return true;
    }

    set_values();
  }

  function set_values() {
    set_input_values();
    set_output_values();
    set_battery_value();
    set_byp_values();
    set_temp_values();
    set_fan_values();
    set_inf_values();
    set_event_values();
    set_synoptics();
    set_ups_status();
    set_outpower_loadbars();
    set_model_name();
    set_sys_mod_status();
    set_footer();
    panes_api.set_autonomy_time_infos(valtable_obj);
  }

  function create_footer_pane() {
    if (!oid || !unms_obj)
      return true;
    footer_api.create({ //Attr
        id: 'panel2'
      }, { //css
        position: 'absolute',
        top: '710px',
        left: '74px'
      },
      unms_obj
    );
  }

  function set_footer() {
    if (!oid || !unms_obj)
      return true;
    footer_api.set(unms_obj);
  }

  function set_variable(data) {
    ups_obj = data.ups;
    report_obj = data.report;
    valtable_obj = ups_obj.valtable;
    status = valtable_obj.STATUS;

    if (valtable_obj._upsProtocol == null) {
      ups_protocol = valtable_obj.CONNECT & 0XFFFF;
    } else {
      ups_protocol = valtable_obj._upsProtocol;
    }

    bg_height = 690;
    bg_width = 850;

    if ((valtable_obj.OUTPOWER[0] > 100) || (valtable_obj.OUTPOWER[1] > 100) || (valtable_obj.OUTPOWER[2] > 100))
      overload = 1;
  }

  function create_panes() {
    utils_api.setTitle(valtable_obj._platform);
    create_bg();
    create_input_pane();
    create_output_pane();
    create_battery_pane();
    create_synoptics();
    create_byp_pane();
    create_temp_pane();
    create_fan_pane();
    create_inf_pane();
    create_event_pane();
    create_status_pane();
    create_model_panes();
    create_outpower_loadbars();
    create_footer_pane();
    create_sys_mod_status_panel();
    panes_api.create_autonomy_time_infos_panel('autonomy_time_label', valtable_obj);
  }

  function create_bg() {
    var fragment = document.createDocumentFragment(),
      img_div, bg_img, bg_img_src = './image/ups/bg_byp_ups.png';

    css_obj = { position: 'absolute', 'text-align': 'center', top: y_pos, left: x_pos, width: bg_width, height: bg_height, border: 'solid 1px #808080' };
    img_div = $('<div/>', { 'class': 'img-rounded' }).css(css_obj);
    bg_img = $('<img />').attr({ 'id': 'bg_img_id', 'class': 'img-rounded', width: 816, 'height': 459, src: bg_img_src }).appendTo(img_div);
    fragment.appendChild(img_div[0]);
    document.body.appendChild(fragment);
  }

  function create_model_panes() {
    var model_div, a;
    css_obj = { position: 'absolute', top: y_pos, left: x_pos, align: 'center', width: bg_width, 'z-index': 40 }; // 'background-color': '#3a87ad'
    attr_obj = { id: 'model_div_id', 'class': 'list-group' };
    model_div = panes_api.create_div(attr_obj, css_obj);
    a = $('<a/>', { 'class': 'list-group-item active', id: 'model_name_id' }).appendTo(model_div);
    set_model_name();
  }

  function create_input_pane()
  {
    var i, fragment = document.createDocumentFragment(),
      tr, table, tbody, thead, in_div;

    css_obj = { position: 'absolute', top: y_pos + 270, left: x_pos + 8 };

    in_div = $('<div/>', { id: 'input_div_id' }).css(css_obj);
    table = $('<table />', { 'class': 'table table-condensed table-hover', 'id': 'inptable' }).appendTo(in_div); // table-hover
    table.css('display', 'none');
    tbody = $('<tbody />').appendTo(table);
    thead = $('<thead />').appendTo(table);
    tr = $('<tr />').append($('<th />', { colspan: 6 }).append(i18n(lang_Input))).appendTo(thead);

    for (i = 1; i <= 3; i++)
    {
      tr = $('<tr />', { 'id': 'inptrl' + i }).append($('<td />').append("L" + i + ":")).appendTo(tbody);
      $('<td />', { id: 'input_volt' + i + '_id' }).appendTo(tr);
      $('<td />', { id: 'input_cur' + i + '_id' }).appendTo(tr);
      $('<td />', { id: 'input_watt' + i + '_id' }).appendTo(tr);
      $('<td />', { id: 'input_freq' + i + '_id' }).appendTo(tr);
    }

    fragment.appendChild(in_div[0]);
    document.body.appendChild(fragment);
    set_input_values();
  }

  function set_input_values() {
    var i, input_val = get_ph_1_3_vals(valtable_obj.INVOLT, valtable_obj.INCURR, valtable_obj.INFREQ, valtable_obj.INPHASES, valtable_obj.EX_INP_WATT );

    $('#inptable').css('display', 'block');

    for (i = 1; i <= 3; i++)
    {
      $('#inptrl' + i).css('display', 'block');
      $('#input_volt' + i + '_id').text(input_val.val_objs[0].values[i - 1]);
      $('#input_cur' + i + '_id').text(input_val.val_objs[1].values[i - 1]);
      $('#input_watt' + i + '_id').text(input_val.val_objs[3].values[i - 1]);
      $('#input_freq' + i + '_id').text(input_val.val_objs[2].values[i - 1]);
    }
  }

  function create_output_pane()
  {
    var i, output_div = null,
      tr, table, tbody, thead,
      fragment = document.createDocumentFragment();

    css_obj = { position: 'absolute', top: y_pos + 270, left: x_pos + 470 };

    output_div = $('<div/>', { id: 'output_div_id' }).css(css_obj);
    table = $('<table />', { 'class': 'table table-condensed table-hover', 'id': 'outtable' }).appendTo(output_div);
    table.css('display', 'none');
    tbody = $('<tbody />').appendTo(table);
    thead = $('<thead />').appendTo(table);
    tr = $('<tr />').append($('<th />', { colspan: 7 }).append(i18n(lang_Output))).appendTo(thead);

    for (i = 1; i <= 3; i++)
    {
      tr = $('<tr />', { 'id': 'outtrl' + i }).append($('<td />').append("L" + i + ":")).appendTo(tbody);
      $('<td />', { 'id': 'out_volt' + i + '_id' }).appendTo(tr);
      $('<td />', { id: 'out_cur' + i + '_id' }).appendTo(tr);
      $('<td />', { id: 'out_watt' + i + '_id' }).appendTo(tr);
      $('<td />', { id: 'out_va' + i + '_id' }).appendTo(tr);
      $('<td />', { id: 'out_freq' + i + '_id' }).appendTo(tr);
      $('<td />', { id: 'out_pwr' + i + '_id' }).appendTo(tr);
    }

    fragment.appendChild(output_div[0]);
    document.body.appendChild(fragment);
    set_output_values();
  }

  function set_output_values()
  {
    var i, output_val = get_ph_1_3_vals(valtable_obj.OUTPUT_VOLT, valtable_obj.EX_OUT_CURR, valtable_obj.OUTFREQ, valtable_obj.OUTPHASES, valtable_obj.EX_OUT_WATT,valtable_obj.EX_OUT_VA);

    $('#outtable').css('display', 'block');

    for (i = 1; i <= 3; i++)
    {
      $('#outtrl' + i).css('display', 'block');
      $('#out_volt' + i + '_id').text(output_val.val_objs[0].values[i - 1]);
      $('#out_cur' + i + '_id').text(output_val.val_objs[1].values[i - 1]);
      $('#out_watt' + i + '_id').text(output_val.val_objs[3].values[i - 1]);
      $('#out_va' + i + '_id').text(output_val.val_objs[4].values[i - 1]);
      $('#out_freq' + i + '_id').text(output_val.val_objs[2].values[i - 1]);
      $('#out_pwr' + i + '_id').text('Pf ' + get_value({ values: [valtable_obj.MBOUTPWRFACT[i - 1]], index: 0, unit: '', precision: 2 }));
    }
  }

  function create_byp_pane()
  {
    var i, byp_div, tr, table, tbody, thead,
      fragment = document.createDocumentFragment();

    css_obj = { position: 'absolute', top: y_pos + 50, left: x_pos + 8 };

    byp_div = $('<div/>', { id: 'bypass_div_id' }).css(css_obj);
    table = table = $('<table />', { 'class': 'table table-condensed table-hover', 'id': 'byptable' }).appendTo(byp_div);
    table.css('display', 'none');
    tbody = $('<tbody />').appendTo(table);
    thead = $('<thead />').appendTo(table);
    tr = $('<tr />').append($('<th />', { colspan: 6 }).append(i18n(lang_Bypass))).appendTo(thead);

    for (i = 1; i <= 3; i++)
    {
      tr = $('<tr />', { 'id': 'byptrl' + i }).append($('<td />').append("L" + i + ":")).appendTo(tbody);
      $('<td />', { 'id': 'byp_volt' + i + '_id' }).appendTo(tr);
      $('<td />', { id: 'byp_cur' + i + '_id' }).appendTo(tr);
      $('<td />', { id: 'byp_watt' + i + '_id' }).appendTo(tr);
      $('<td />', { id: 'byp_va' + i + '_id' }).appendTo(tr);
      $('<td />', { id: 'byp_freq' + i + '_id' }).appendTo(tr);
    }

    fragment.appendChild(byp_div[0]);
    document.body.appendChild(fragment);
    set_byp_values();
  }

  function set_byp_values()
  {
    var i, bypass_val = get_ph_1_3_vals(valtable_obj.EX_BYP_VOLT, valtable_obj.EX_BYP_CURR, valtable_obj.EX_BYP_FREQ, valtable_obj.EX_BYP_NUMLINES,valtable_obj.EX_BYP_WATT, valtable_obj.BYPASS_APPPOW);

    $('#bypass_div_id').css('display', 'block');
    $('#bypass_led_id').css('display', 'block');
    $('#byptable').css('display', 'block');
    $('#bypass_img_id').css('display', 'block');

    for (i = 1; i <= 3; i++)
    {
      $('#byptrl' + i).css('display', 'block');
      $('#byp_volt' + i + '_id').text(bypass_val.val_objs[0].values[i - 1]);
      $('#byp_cur' + i + '_id').text(bypass_val.val_objs[1].values[i - 1]);
      $('#byp_watt' + i + '_id').text(bypass_val.val_objs[3].values[i - 1]);
      $('#byp_va' + i + '_id').text(bypass_val.val_objs[4].values[i - 1]);
      $('#byp_freq' + i + '_id').text(bypass_val.val_objs[2].values[i - 1]);
    }
  }

  function create_inf_pane()
  {
    var tr, table, tbody, thead, div,
      fragment = document.createDocumentFragment(),
      td_attr = {align: 'right', width:40};

    css_obj = { position: 'absolute', top: y_pos + 520, left: x_pos + 600 };
    attr_obj = { id: 'inf_div_id' };
    div = $('<div/>').attr(attr_obj).css(css_obj);
    table = table = $('<table />', { 'class': 'table table-condensed table-hover' }).appendTo(div);
    tbody = $('<tbody />').appendTo(table);
    thead = $('<thead />').appendTo(table);
    tr = $('<tr />').append($('<th />').append(i18n('Information'))).appendTo(thead);

    tr = $('<tr />').append($('<td />', { align: 'left' }).append(i18n('Firmware version') + ":")).appendTo(tbody);
    $('<td />', { id: 'fwver_id'}).attr(td_attr).appendTo(tr);
    tr = $('<tr />').append($('<td />', { align: 'left' }).append(i18n('C28 Hash') + ":")).appendTo(tbody);
    $('<td />', { id: 'c28hash_id'}).attr(td_attr).appendTo(tr);
    tr = $('<tr />').append($('<td />', { align: 'left' }).append(i18n('M3 Hash') + ":")).appendTo(tbody);
    $('<td />', { id: 'm3hash_id'}).attr(td_attr).appendTo(tr);

    fragment.appendChild(div[0]);
    document.body.appendChild(fragment);
    set_inf_values();
  }

  function set_inf_values()
  {
    var c28, m3, i;

    c28 = valtable_obj.DKC_C28HASH.toString(16);
    for (i = c28.length; i < 8; i++)
      c28 = '0' + c28;
    m3 = valtable_obj.DKC_M3HASH.toString(16);
    for (i = m3.length; i < 8; i++)
      m3 = '0' + m3;

    $('#fwver_id').text(valtable_obj.FIRMWARE);
    $('#c28hash_id').text('0x' + c28);
    $('#m3hash_id').text('0x' + m3);
  }

  function create_fan_pane()
  {
    var i, tr, table, tbody, thead, div,
      fragment = document.createDocumentFragment(),
      td_attr = {align: 'right', width:40};

    css_obj = { position: 'absolute', top: y_pos + 390, left: x_pos + 600 };
    attr_obj = { id: 'temp_div_id' };
    div = $('<div/>').attr(attr_obj).css(css_obj);
    table = table = $('<table />', { 'class': 'table table-condensed table-hover' }).appendTo(div);
    tbody = $('<tbody />').appendTo(table);
    thead = $('<thead />').appendTo(table);
    tr = $('<tr />').append($('<th />').append(i18n('Fan speed'))).appendTo(thead);

    for (i = 1; i <= 3; i++)
    {
      tr = $('<tr />').append($('<td />', { align: 'left' }).append(i18n('Group ' + i) + ":")).appendTo(tbody);
      $('<td />', { id: 'fan' + i + '_id'}).attr(td_attr).appendTo(tr);
    }

    fragment.appendChild(div[0]);
    document.body.appendChild(fragment);
    set_fan_values();
  }

  function set_fan_values()
  {
    var i;
    for (i = 1; i <= 3; i++)
    {
      $('#fan' + i + '_id').text(get_value({ values: [valtable_obj.FAN_SPEED[i - 1]], index: 0, unit: '%', precision: 0 }));
    }
  }

  function append_temp_line(tbody, title, id)
  {
    var td_attr = {align: 'right', width:40};
    var tr = $('<tr />').append($('<td />', { align: 'left' }).append(i18n(title) + ":")).appendTo(tbody);
    $('<td />', { id: id + '_curr_id'}).attr(td_attr).appendTo(tr);
    $('<td />', { id: id + '_mean_id'}).attr(td_attr).appendTo(tr);
    $('<td />', { id: id + '_max_id'}).attr(td_attr).appendTo(tr);
  }

  function create_temp_pane()
  {
    var i, tr, table, tbody, thead, div,
      fragment = document.createDocumentFragment();

    css_obj = { position: 'absolute', top: y_pos + 390, left: x_pos + 8 };
    attr_obj = { id: 'temp_div_id' };
    div = $('<div/>').attr(attr_obj).css(css_obj);
    table = table = $('<table />', { 'class': 'table table-condensed table-hover' }).appendTo(div);
    tbody = $('<tbody />').appendTo(table);
    thead = $('<thead />').appendTo(table);

    tr = $('<tr />').append($('<th />').append(i18n(lang_Temperature) + " &#176;C")).appendTo(thead);
    $('<th />').css('text-align','right').append(i18n('Curr.')).appendTo(tr);
    $('<th />').css('text-align','right').append(i18n('Mean')).appendTo(tr);
    $('<th />').css('text-align','right').append(i18n('Max')).appendTo(tr);

    for (i = 1; i <= 3; i++)
    {
      append_temp_line(tbody, 'Rectifier L' + i, 'rec_l' + i);
    }
    append_temp_line(tbody, 'Positive battery booster', 'posbatt');
    append_temp_line(tbody, 'Negative battery booster', 'negbatt');
    append_temp_line(tbody, 'Battery charger', 'charger');
    for (i = 1; i <= 3; i++)
    {
      append_temp_line(tbody, 'Inverter L' + i, 'inv_l' + i);
    }
    append_temp_line(tbody, 'Battery shelf', 'battshelf');
    append_temp_line(tbody, 'UPS shelf', 'upsshelf');

    fragment.appendChild(div[0]);
    document.body.appendChild(fragment);
    set_temp_values();
  }

  function set_temp_line(id, val)
  {
      $('#' + id + '_curr_id').text(get_value({ values: [val[0]], index: 0, unit: '', precision: 0 }));
      $('#' + id + '_mean_id').text(get_value({ values: [val[1]], index: 0, unit: '', precision: 0 }));
      $('#' + id + '_max_id').text(get_value({ values: [val[2]], index: 0, unit: '', precision: 0 }));
  }

  function set_temp_values()
  {
      set_temp_line('rec_l1', valtable_obj.TEMP_RECT_R);
      set_temp_line('rec_l2', valtable_obj.TEMP_RECT_S);
      set_temp_line('rec_l3', valtable_obj.TEMP_RECT_T);
      set_temp_line('posbatt', valtable_obj.TEMP_BOOSTPOS);
      set_temp_line('negbatt', valtable_obj.TEMP_BOOSTNEG);
      set_temp_line('charger', valtable_obj.TEMP_CHARGER);
      set_temp_line('inv_l1', valtable_obj.TEMP_INV_R);
      set_temp_line('inv_l2', valtable_obj.TEMP_INV_S);
      set_temp_line('inv_l3', valtable_obj.TEMP_INV_T);
      set_temp_line('battshelf', valtable_obj.TEMP_BATT);
      set_temp_line('upsshelf', valtable_obj.TEMP_UPS);
  }

  function create_event_pane()
  {
    var val, tr, table, tbody, thead, batt_div,
      fragment = document.createDocumentFragment();

    css_obj = { position: 'absolute', top: y_pos + 630, left: x_pos + 360 };
    attr_obj = { id: 'event_div_id' };
    batt_div = $('<div/>').attr(attr_obj).css(css_obj);
    table = table = $('<table />', { 'class': 'table table-condensed table-hover' }).appendTo(batt_div);
    tbody = $('<tbody />').appendTo(table);
    thead = $('<thead />').appendTo(table);
    tr = $('<tr />').append($('<th />', { colspan: 2 }).append(i18n('Last Event'))).appendTo(thead);

    tr = $('<tr />').append($('<td />', { align: 'left' }).append(i18n('Data') + ":")).appendTo(tbody);
    $('<td />', { id: 'lastevent_id', align: 'left' }).appendTo(tr);

    fragment.appendChild(batt_div[0]);
    document.body.appendChild(fragment);
    set_event_values();
  }

  function set_event_values()
  {
    var i, ev = '';

    for (i = 0; i < 16; i++)
    {
      ev += valtable_obj.DKC_CURREVENT[i].toString(16) + ' ';
    }
    $('#lastevent_id').text(ev);
  }

  function create_battery_pane()
  {
    var val, tr, table, tbody, thead, batt_div,
      fragment = document.createDocumentFragment();

    css_obj = { position: 'absolute', top: y_pos + 390, left: x_pos + 360 };
    attr_obj = { id: 'battery_div_id' };
    batt_div = $('<div/>').attr(attr_obj).css(css_obj);
    table = table = $('<table />', { 'class': 'table table-condensed table-hover' }).appendTo(batt_div);
    tbody = $('<tbody />').appendTo(table);
    thead = $('<thead />').appendTo(table);
    tr = $('<tr />').append($('<th />', { colspan: 2 }).append(i18n(lang_Battery))).appendTo(thead);

    tr = $('<tr />').append($('<td />', { align: 'left' }).append(i18n('Mode') + ":")).appendTo(tbody);
    $('<td />', { id: 'batt_mode_id', align: 'left' }).appendTo(tr);
    tr = $('<tr />').append($('<td />', { align: 'left' }).append(i18n(lang_Voltage) + " pos:")).appendTo(tbody);
    $('<td />', { id: 'batt_volt_pos_id', align: 'left' }).appendTo(tr);
    tr = $('<tr />').append($('<td />', { align: 'left' }).append(i18n(lang_Voltage) + " neg:")).appendTo(tbody);
    $('<td />', { id: 'batt_volt_neg_id', align: 'left' }).appendTo(tr);
    tr = $('<tr />').append($('<td />', { align: 'left' }).append(i18n(lang_Current_pos) + ":")).appendTo(tbody);
    $('<td />', { id: 'batt_currpos_id', align: 'left' }).appendTo(tr);
    tr = $('<tr />').append($('<td />', { align: 'left' }).append(i18n(lang_Current_neg) + ":")).appendTo(tbody);
    $('<td />', { id: 'batt_currneg_id', align: 'left' }).appendTo(tr);
    tr = $('<tr />').append($('<td />', { align: 'left' }).append(i18n(lang_Battery_Capacity_short) + " pos:")).appendTo(tbody);
    $('<td />', { id: 'batt_cap_id', align: 'left' }).appendTo(tr);
    tr = $('<tr />').append($('<td />', { align: 'left' }).append(i18n(lang_Battery_Capacity_short) + " neg:")).appendTo(tbody);
    $('<td />', { id: 'batt_capneg_id', align: 'left' }).appendTo(tr);
    tr = $('<tr />').append($('<td />', { align: 'left', id: 'autonomy_time_label' }).append(i18n(lang_Time) + ":")).appendTo(tbody);
    $('<td />', { id: 'batt_autotime_id', align: 'left' }).appendTo(tr);
    tr = $('<tr />').append($('<td />', { align: 'left' }).append(i18n('On Battery') + ":")).appendTo(tbody);
    $('<td />', { id: 'batt_seconds_id', align: 'left' }).appendTo(tr);

    fragment.appendChild(batt_div[0]);
    document.body.appendChild(fragment);
    set_battery_value();
  }

  function set_battery_value()
  {
    var battmode = i18n('Unknown');

    if (valtable_obj.BATTERY_STATE == 0)
      battmode = i18n('No battery');
    else if (valtable_obj.BATTERY_STATE == 1)
      battmode = i18n('Current charge');
    else if (valtable_obj.BATTERY_STATE == 2)
      battmode = i18n('Flat charge');
    else if (valtable_obj.BATTERY_STATE == 3)
      battmode = i18n('Discharge');
    $('#batt_mode_id').text(i18n(battmode));

    $('#batt_volt_pos_id').text(get_value({ values: [valtable_obj.EX_BATT_VOLTPOS], index: 0, unit: ' V', precision: 2 }));
    $('#batt_volt_neg_id').text(get_value({ values: [valtable_obj.EX_BATT_VOLTNEG], index: 0, unit: ' V', precision: 2 }));
    $('#batt_currpos_id').text(get_value({ values: [valtable_obj.EX_BATT_CURRPOS], index: 0, unit: ' A', precision: 1 }));
    $('#batt_currneg_id').text(get_value({ values: [valtable_obj.EX_BATT_CURRNEG], index: 0, unit: ' A', precision: 1 }));
    $('#batt_autotime_id').text(get_value({ values: [valtable_obj.AUTONOMTIME], index: 0, unit: ' Min', precision: 0 }));
    $('#batt_cap_id').text(get_value({ values: [valtable_obj.BATTCAP], index: 0, unit: ' %', precision: 0 }));
    $('#batt_capneg_id').text(get_value({ values: [valtable_obj.EX_BATT_CAPNEG], index: 0, unit: ' %', precision: 0 }));
    $('#batt_seconds_id').text(get_value({ values: [valtable_obj.EX_BATT_SECONBATT], index: 0, unit: ' Sec', precision: 0 }));
  }

  function get_ph_1_3_vals(volt, cur, freq, phases, watt, va) {
    var val_obj, val;
    l_1_3_vals = {
      availables: [0, 0, 0], //L1-L3
      val_objs: [
        { unit: ' V', precision: 0, values: ['', '', ''] },
        { unit: ' A', precision: 1, values: ['', '', ''] },
        { unit: ' Hz', precision: 1, values: ['', '', ''] },
        { unit: ' kW', precision: 2, values: ['', '', ''] },
        { unit: ' kVA', precision: 2, values: ['', '', ''] }
      ]
    };
    volt = volt || [];
    cur = cur || [], freq = freq || [];
    watt = watt || [-9999,-9999,-9999];
    va = va || [-9999,-9999,-9999];
    $.each([volt, cur, freq, watt, va], function (index, values) {
      val_obj = l_1_3_vals.val_objs[index];
      $.each(values, function (i, val) {
        if (i < phases) {
          val = values[i];
          if (val != -9999) {
            // do some nice output:
            // if watt or va are bigger than 1000 display
            // them as kVA and kW
            if (values === watt) {
              if (val >= 1000) {
                val_obj.unit = ' kW';
                val_obj.precision = 1;
                val = val / 1000.0;
              } else {
                val_obj.unit = ' W';
                val_obj.precision = 0;
              }
            }
            if (values === va) {
              if (val >= 1000) {
                val_obj.unit = ' kVA';
                val_obj.precision = 1;
                val = val / 1000.0;
              } else {
                val_obj.unit = ' VA';
                val_obj.precision = 0;
              }
            }
            l_1_3_vals.availables[i] = 1;
            val = val.toFixed(val_obj.precision);
            val += val_obj.unit;
            val_obj.values[i] = val;
          }
        }
      });
    });
    return l_1_3_vals;
  }

  function get_value(val_obj, na_string) {
    var val = null;
    var a = null;

    if (!val_obj.values || (val_obj.values.length <= val_obj.index))
      return na_string == null ? 'N/A' : na_string;

    val = val_obj.values[val_obj.index];
    if (val == null || val == -9999)
      return na_string == null ? 'N/A' : na_string;
    if (ups_protocol == prot_snmp && val == -1)
      return na_string == null ? 'N/A' : na_string;

    val = val.toFixed(val_obj.precision);
    val += val_obj.unit;
    return val;
  }

  function create_synoptics()
  {
    var div, fragment = document.createDocumentFragment();

    // leds
    css_obj = { position: 'absolute', top: y_pos + 105, left: x_pos + 470, width: 24, height: 10, overflow: 'hidden' };
    div = $('<div/>').attr({ id: 'bypassbreaker_led_id', 'class': 'img-rounded' }).css(css_obj);
    fragment.appendChild(div[0]);

    css_obj = { position: 'absolute', top: y_pos + 229, left: x_pos + 564, width: 24, height: 10, overflow: 'hidden' };
    div = $('<div/>').attr({ id: 'output_led_id', 'class': 'img-rounded' }).css(css_obj);
    fragment.appendChild(div[0]);

    css_obj = { position: 'absolute', top: y_pos + 229, left: x_pos + 236, width: 24, height: 10, overflow: 'hidden' };
    div = $('<div/>').attr({ id: 'input_led_id', 'class': 'img-rounded' }).css(css_obj);
    fragment.appendChild(div[0]);

    css_obj = { position: 'absolute', top: y_pos + 150, left: x_pos + 236, width: 24, height: 10, overflow: 'hidden' };
    div = $('<div/>').attr({ id: 'bypass_led_id', 'class': 'img-rounded' }).css(css_obj);
    fragment.appendChild(div[0]);

    document.body.appendChild(fragment);

    // symbols
    panes_api.create_img_panel('rec_img_id', './image/ups/grw.gif', x_pos + 302, y_pos + 210, 50, 50, 0);
    panes_api.create_img_panel('bypass_img_id', './image/ups/ssww.gif', x_pos + 388, y_pos + 130, 50, 50, 0);
    panes_api.create_img_panel('inv_img_id', './image/ups/wrr.gif', x_pos + 474, y_pos + 210, 50, 50, 0);
    panes_api.create_img_panel('batt_img_id', './image/ups/Batgrv.gif', x_pos + 388, y_pos + 294, 50, 50, 0);

    set_synoptics();
  }

  function set_synoptics()
  {
    // input led
    if ((valtable_obj.RAWSTATEA & 0xF) == 0x0)
      $('#input_led_id').css('background-color', '#C1C1C1'); // grey
    else if ((valtable_obj.RAWSTATEA & 0xF) == 0x1)
      $('#input_led_id').css('background-color', '#54B245'); // green
    else if ((valtable_obj.RAWSTATEA & 0xF) == 0x2)
      $('#input_led_id').css('background-color', '#FFF500'); // yellow
    else if ((valtable_obj.RAWSTATEA & 0xF) == 0x3)
      $('#input_led_id').css('background-color', '#DC3632'); // red

    // bypass breaker symbol
    if ((valtable_obj.RAWSTATEB & 0xF0) == 0x0)
      $('#bypassbreaker_led_id').css('background-color', '#C1C1C1'); // grey
    else if ((valtable_obj.RAWSTATEB & 0xF0) == 0x10)
      $('#bypassbreaker_led_id').css('background-color', '#54B245'); // green

    // bypass led
    if ((valtable_obj.RAWSTATEA & 0xF0) == 0x0)
      $('#bypass_led_id').css('background-color', '#C1C1C1'); // grey
    else if ((valtable_obj.RAWSTATEA & 0xF0) == 0x10)
      $('#bypass_led_id').css('background-color', '#54B245'); // green
    else if ((valtable_obj.RAWSTATEA & 0xF0) == 0x20)
      $('#bypass_led_id').css('background-color', '#FFF500'); // yellow
    else if ((valtable_obj.RAWSTATEA & 0xF0) == 0x30)
      $('#bypass_led_id').css('background-color', '#DC3632'); // red

    // static bypass symbol
    if ((valtable_obj.RAWSTATEB & 0xF) == 0x0)
      $('#bypass_img_id').attr('src', './image/ups/ssww.gif');
    else if ((valtable_obj.RAWSTATEB & 0xF) == 0x1)
      $('#bypass_img_id').attr('src', './image/ups/sswgr.gif');
    else if ((valtable_obj.RAWSTATEB & 0xF) == 0x2)
      $('#bypass_img_id').attr('src', './image/ups/sswge.gif');
    else if ((valtable_obj.RAWSTATEB & 0xF) == 0x3)
      $('#bypass_img_id').attr('src', './image/ups/sswr.gif');

    // rectifier symbol
    if ((valtable_obj.RAWSTATEA & 0xF00) == 0x0)
      $('#rec_img_id').attr('src', './image/ups/grw.gif');
    else if ((valtable_obj.RAWSTATEA & 0xF00) == 0x100)
      $('#rec_img_id').attr('src', './image/ups/grgr.gif');
    else if ((valtable_obj.RAWSTATEA & 0xF00) == 0x200)
      $('#rec_img_id').attr('src', './image/ups/grge.gif');
    else if ((valtable_obj.RAWSTATEA & 0xF00) == 0x300)
      $('#rec_img_id').attr('src', './image/ups/grr.gif');

    // battery symbol
    if ((valtable_obj.RAWSTATEA & 0xF00000) == 0x0)
      $('#batt_img_id').attr('src', './image/ups/Batwv.gif');
    else if ((valtable_obj.RAWSTATEA & 0xF00000) == 0x100000)
      $('#batt_img_id').attr('src', './image/ups/Batgrv.gif');
    else if ((valtable_obj.RAWSTATEA & 0xF00000) == 0x200000)
      $('#batt_img_id').attr('src', './image/ups/Batgev.gif');
    else if ((valtable_obj.RAWSTATEA & 0xF00000) == 0x300000)
      $('#batt_img_id').attr('src', './image/ups/Batrv.gif');

    // inverter symbol
    if ((valtable_obj.RAWSTATEA & 0xF000000) == 0x0)
      $('#inv_img_id').attr('src', './image/ups/wrw.gif');
    else if ((valtable_obj.RAWSTATEA & 0xF000000) == 0x1000000)
      $('#inv_img_id').attr('src', './image/ups/wrgr.gif');
    else if ((valtable_obj.RAWSTATEA & 0xF000000) == 0x2000000)
      $('#inv_img_id').attr('src', './image/ups/wrge.gif');
    else if ((valtable_obj.RAWSTATEA & 0xF000000) == 0x3000000)
      $('#inv_img_id').attr('src', './image/ups/wrr.gif');

    // output led
    if ((valtable_obj.RAWSTATEA & 0xF0000000) == 0x0)
      $('#output_led_id').css('background-color', '#C1C1C1'); // grey
    else if ((valtable_obj.RAWSTATEA & 0xF0000000) == 0x10000000)
      $('#output_led_id').css('background-color', '#54B245'); // green
    else if ((valtable_obj.RAWSTATEA & 0xF0000000) == 0x20000000)
      $('#output_led_id').css('background-color', '#FFF500'); // yellow
    else if ((valtable_obj.RAWSTATEA & 0xF0000000) == 0x30000000)
      $('#output_led_id').css('background-color', '#DC3632'); // red
  }

  function create_status_pane()
  {
    var status_div = $('<div/>'),
      status_span;
    css_obj = { position: 'absolute', top: y_pos + 50, left: x_pos + 365, align: 'center', overflow: 'hidden' };

    $(status_div).css(css_obj);
    status_span = $('<span />', { id: 'ups_status_id', 'class': get_status_color(), title: i18n('Click to see Status Text') }).append(i18n('Status'));
    status_span.css('cursor', 'pointer');
    status_span.click(function (event) //visibility: hidden
      {
        var sys_mod_status_txt_div = $('#sys_mod_status_txt_div'),
          display = sys_mod_status_txt_div.css('display') == 'none' ? 'block' : 'none';
        sys_mod_status_txt_div.css({ display: display });
      });

    $('<h4 />').append(status_span).appendTo(status_div);
    status_div.appendTo(document.body);
    set_ups_status();
  }

  function set_ups_status()
  {
    var workmode = i18n('Unknown');

    if (valtable_obj.UPS_WORKMODE == 0)
      workmode = i18n('OFF');
    else if (valtable_obj.UPS_WORKMODE == 1)
      workmode = i18n('Standby');
    else if (valtable_obj.UPS_WORKMODE == 2)
      workmode = i18n('Starting');
    else if (valtable_obj.UPS_WORKMODE == 3)
      workmode = i18n('Online');
    else if (valtable_obj.UPS_WORKMODE == 4)
      workmode = i18n('Battery');
    else if (valtable_obj.UPS_WORKMODE == 5)
      workmode = i18n('Bypass');
    else if (valtable_obj.UPS_WORKMODE == 6)
      workmode = i18n('Service');
    else if (valtable_obj.UPS_WORKMODE == 7)
      workmode = i18n('Testing');
    if (workmode.length > 0)
      $('#ups_status_id').text(i18n('Status') + ': ' + workmode);

    $('#ups_status_id').attr('class', get_status_color());
  }

  function get_status_color() {
    if ((status & 0xF7BA) || (valtable_obj.EX_EXTSTATUS & 0x75FE48))
      return 'label label-danger';

    if (status & 0x41)
      return 'label label-warning';

    return 'label label-success';
  }

  function create_outpower_loadbars()
  {
    var i, fragment = document.createDocumentFragment();

    for (i = 1; i <= 3; i++)
    {
       panes_api.create_loadbar_pane({ value: valtable_obj.OUTPOWER[i - 1], id: 'loadbar' + i + '_id', left: x_pos + 630, top: y_pos + 70 + (i * 30), title: 'L' + i, scaler: i == 1 ? 1 : 0 });
    }

    document.body.appendChild(fragment);
    set_outpower_loadbars();
  }

  function set_outpower_loadbars()
  {
    var i;

    for (i = 1; i <= 3; i++)
    {
       panes_api.set_loabar_value(valtable_obj.OUTPOWER[i - 1], 'loadbar' + i + '_id');
    }
  }

  function create_sys_mod_status_panel() {
    var fragment = document.createDocumentFragment(),
      tr, tr, table, tbody, a,
      title = i18n("Status") + ":" + get_model_name(),
      status_txt_array = get_sys_mod_status_txt(),
      sys_mod_status_txt_div;

    css_obj = {
      position: 'absolute',
      top: y_pos + 72,
      left: x_pos + 36,
      align: 'center',
      'background-color': '#F1F1F1',
      display: 'none',
      border: 'solid 1px black'
    };

    sys_mod_status_txt_div = $('<div/>', { id: 'sys_mod_status_txt_div', 'class': 'img-rounded' }).css(css_obj);

    a = $('<a/>', { 'class': 'list-group-item active', id: 'status_model_name_id' }).append(title).appendTo(sys_mod_status_txt_div);
    a.css('height', 30);
    table = $('<table />', { 'class': 'table table-condensed', id: 'sys_mod_status_txt_table' }).appendTo(sys_mod_status_txt_div); // table-hover
    table.css({ 'background-color': '#F1F1F1' })
    tbody = $('<tbody />', { id: 'sys_mod_status_txt_table_body' }).appendTo(table);
    $.each(status_txt_array, function (index, status_txt) {
      tr = $('<tr />').appendTo(tbody);
      td = $('<td />', { 'align': 'left' }).append(status_txt).appendTo(tr);
    });

    fragment.appendChild(sys_mod_status_txt_div[0]);
    document.body.appendChild(fragment);
    new utils_api.draggable('sys_mod_status_txt_div');
  }

  function set_sys_mod_status() {
    var tr, td,
      title = i18n("Status") + ":" + get_model_name(),
      tbody = $('#sys_mod_status_txt_table_body'),
      status_txt_array = get_sys_mod_status_txt();

    $('#status_model_name_id').text(title);

    tbody.empty();
    $.each(status_txt_array, function (index, status_txt) {
      tr = $('<tr />').appendTo(tbody);
      td = $('<td />', { 'align': 'left' }).append(status_txt).appendTo(tr);
    });
  }

  function get_sys_mod_status_txt() {
    var status_txt_array = [];
    $.each(status_obj, function (index, status_obj) {
      if (status_obj.bit & valtable_obj.STATUS) {
        status_txt_array.push(i18n(status_obj.txt));
      }
    });

    if (valtable_obj.EX_EXTSTATUS != null) {
      $.each(ext_status_obj, function (index, status_obj) {
        if (status_obj.bit & valtable_obj.EX_EXTSTATUS) {
          status_txt_array.push(i18n(status_obj.txt));
        }
      });
    }

    return status_txt_array;
  }

  function get_module_status_class(module_index) {
    var bit_val = 1 << module_index;
    if ((valtable_obj.RAWSTATEG & bit_val) == bit_val)
      return "btn btn-sm btn-danger";
    return "btn btn-sm btn-default";
  }

function get_model_name()
{
   var model = "TRIO XT" + valtable_obj.PREXT_NOMMODEL;
   return model;
}

  function set_model_name() {
    $('#model_name_id').html(get_model_name());
  }
}(jQuery));
