/*
   elidiesel_panel.js - Elinex IG200/IL3 screen
   Copyright (c) 2020, Generex GmbH. All rights reserved.
   UW.
*/
var elinexdiesel_api = (function ($) {
  var oid = get_oid_from_url(),
  cb = null, //Callback Obj
  pannes_created = false,
  err_div = null,
  check_bowser = true,
  css_obj,
  valtable_obj, report_obj, unms_obj,
  inphases = null,
  outphases = null,
  x_pos = 5,
  y_pos = 5;

  var req_params = null;

  /*public functions*/
  return {
    init: init
  };

  function init() {
    if (check_bowser && !utils_api.check_bowser()) {
      $('#loading_id').css('display', 'none');
      panes_api.create_br_check_panel(this, browser_check_ok);
      return true;
    }
    create_err_msg_panel();

    req_params = utils_api.get_req_params(utils_api.device_ups, oid);
    cb = jqmodule.subscribe(this, { update: update, error: err_funct }, req_params);
  }

  function browser_check_ok() {
    check_bowser = false;
    init();
  }

  function create_err_msg_panel() {
    css_obj = { position: 'absolute', top: y_pos, left: x_pos, width: 791, display: 'none', 'z-index': 22 };
    err_div = panes_api.create_err_panel({ id: 'error_div', 'class': 'alert alert-danger' }, css_obj);
  }

  function err_funct(response) {
    jqmodule.refresh(cb, 10 * 1000);
    panes_api.display_err({ response: response, oid: oid, err_div: err_div, err_msg_div_id: 'error_msg_div' });
  }

  function update(data) {
    if (data.ups == null) {
      return;
    }

    set_variable(data);
    jqmodule.refresh(cb, report_obj.refresh * 1000);
    if (err_div)
    err_div.css('display', 'none');

    if (window.menu_api != null)
    menu_api.update(data);

    unms_obj = get_unms_obj(data);
    if (unms_obj != null) {
      info_head_api.update(unms_obj);
    }

    if (!pannes_created) {
      pannes_created = true;
      create_panes();
      return false;
    }
    set_output_values();
    set_measure_values();
    set_ups_status_info();
    set_footer();
  }

  function create_footer_pane() {
    if (!oid || !unms_obj)
    return true;
    footer_api.create({ //Attr
        id: 'panel2'
        //'class':'unmsFooter'
      }, { //css
        position: 'absolute',
        top: '660px',
        left: '230px'
      },
      unms_obj
    );
  }

  function set_footer() {
    if (!oid || !unms_obj)
    return true;
    footer_api.set(unms_obj);
  }

  function set_variable(data) {
    valtable_obj = data.ups.valtable;
    report_obj = data.report;

    if (valtable_obj.INPHASES != null)
    inphases = valtable_obj.INPHASES;
    else
    inphases = valtable_obj.PHASES;

    if (valtable_obj.OUTPHASES != null)
    outphases = valtable_obj.OUTPHASES;
    else
    outphases = valtable_obj.PHASES;
  }

  function create_panes() {
    utils_api.setTitle(valtable_obj._platform);
    create_footer_pane();
    create_bg(); //background
    create_model_panes();
    create_output_pane();
    create_measure_pane();
    create_ups_status_info_pane();
  }

  function create_bg() {
    var fragment = document.createDocumentFragment(),
    img_div, img;
    css_obj = { position: 'absolute', top: y_pos + 30, left: x_pos, width: 1100, height: 600, border: 'solid 1px #808080', align: 'left' };
    img_div = $('<div/>', { 'class': 'img-rounded' }).css(css_obj);
    img = $('<img />').attr({ 'id': 'bg_img_id', 'class': 'img-rounded'/*, width: 650, 'height': 325*/, src: './image/elinex/diesel.png', align: 'left' }).appendTo(img_div);
    img.css({ 'margin-top': y_pos + 50, 'margin-left': x_pos + 0 });
    fragment.appendChild(img_div[0]);
    document.body.appendChild(fragment);
  }

  function create_model_panes() {
    var model_div, a;
    css_obj = { position: 'absolute', top: y_pos, left: x_pos, align: 'center', width: 1100, 'z-index': 40 }; // 'background-color': '#3a87ad'
    attr_obj = { id: 'model_div_id', 'class': 'list-group' };
    model_div = panes_api.create_div(attr_obj, css_obj);
    a = $('<a/>', { 'class': 'list-group-item active', id: 'model_name_id' }).append(valtable_obj.MODEL).appendTo(model_div);
  }

  function create_measure_pane() {
    var fragment = document.createDocumentFragment(),
    tr, table, tbody, thead, out_div;

    css_obj = { position: 'absolute', top: y_pos + 60, left: x_pos + 430 };

    out_div = $('<div/>', { id: 'output_div_id' }).css(css_obj);
    table = $('<table />', { 'class': 'table table-condensed', 'id': 'meastable' }).appendTo(out_div);
    tbody = $('<tbody />').appendTo(table);
    thead = $('<thead />').appendTo(table);
    tr = $('<tr />').append($('<th />', { colspan: 2 }).append(i18n('Measures'))).appendTo(thead);

    tr = $('<tr />', { 'id': 'batt_volt', align: 'left'  }).append($('<td />').append(i18n("Batt.Voltage") + ":")).appendTo(tbody);
    $('<td />', { id: 'batt_volt_id', align: 'left' }).appendTo(tr);
    tr = $('<tr />', { 'id': 'rpm', align: 'left'  }).append($('<td />').append(i18n("RPM") + ":")).appendTo(tbody);
    $('<td />', { id: 'rpm_id', align: 'left' }).appendTo(tr);
    tr = $('<tr />', { 'id': 'num_starts', align: 'left'  }).append($('<td />').append(i18n("Starts") + ":")).appendTo(tbody);
    $('<td />', { id: 'num_starts_id', align: 'left' }).appendTo(tr);
    tr = $('<tr />', { 'id': 'hours_on', align: 'left'  }).append($('<td />').append(i18n("Hours On") + ":")).appendTo(tbody);
    $('<td />', { id: 'hours_on_id', align: 'left' }).appendTo(tr);
    tr = $('<tr />', { 'id': 'contr_mode', align: 'left'  }).append($('<td />').append(i18n("Controler mode") + ":")).appendTo(tbody);
    $('<td />', { id: 'contr_mode_id', align: 'left' }).appendTo(tr);
    tr = $('<tr />', { 'id': 'gen_mode', align: 'left'  }).append($('<td />').append(i18n("Generator") + ":")).appendTo(tbody);
    $('<td />', { id: 'gen_mode_id', align: 'left' }).appendTo(tr);

    fragment.appendChild(out_div[0]);
    document.body.appendChild(fragment);
    set_measure_values();
  }

  function set_measure_values()
  {
    $('#batt_volt_id').text(get_value({ values: [valtable_obj.BATTVOLT], index: 0, unit: ' V', precision: 1 }));
    $('#rpm_id').text(get_value({ values: [valtable_obj.RPM], index: 0, unit: '', precision: 1 }));
    $('#num_starts_id').text(get_value({ values: [valtable_obj.NUM_STARTS], index: 0, unit: '', precision: 0 }));
    $('#hours_on_id').text(get_value({ values: [valtable_obj.HOURS_ON], index: 0, unit: '', precision: 0 }));

    var cmode = "OFF";
    if (valtable_obj.CONTROLERMODE == 1)
      cmode = "MAN";
    else if (valtable_obj.CONTROLERMODE == 2)
      cmode = "AUTO";
    else if (valtable_obj.CONTROLERMODE == 3)
      cmode = "TEST";
    $('#contr_mode_id').text(cmode);

    var genmode = "OFF";
    if (valtable_obj.RAWSTATEA & 0x4)
      genmode = "ON";
    $('#gen_mode_id').text(genmode);
  }

  function create_output_pane() {
    var fragment = document.createDocumentFragment(),
    tr, table, tbody, thead, out_div;

    css_obj = { position: 'absolute', top: y_pos + 60, left: x_pos + 70 };

    out_div = $('<div/>', { id: 'output_div_id' }).css(css_obj);
    table = $('<table />', { 'class': 'table table-condensed', 'id': 'outputtable' }).appendTo(out_div);
    tbody = $('<tbody />').appendTo(table);
    thead = $('<thead />').appendTo(table);
    tr = $('<tr />').append($('<th />', { colspan: 4 }).append(i18n('Output'))).appendTo(thead);

    tr = $('<tr />', { 'id': 'out_volt', align: 'left'  }).append($('<td />').append(i18n("Voltage") + ":")).appendTo(tbody);
    $('<td />', { id: 'out_volt1_id', align: 'left'  }).appendTo(tr);
    $('<td />', { id: 'out_volt2_id', align: 'left'  }).appendTo(tr);
    $('<td />', { id: 'out_volt3_id', align: 'left'  }).appendTo(tr);

    tr = $('<tr />', { 'id': 'out_curr', align: 'left'  }).append($('<td />').append(i18n("Current") + ":")).appendTo(tbody);
    $('<td />', { id: 'out_curr1_id', align: 'left'  }).appendTo(tr);
    $('<td />', { id: 'out_curr2_id', align: 'left'  }).appendTo(tr);
    $('<td />', { id: 'out_curr3_id', align: 'left'  }).appendTo(tr);

    table = $('<table />', { 'class': 'table table-condensed', 'id': 'outputtable' }).appendTo(out_div);
    tbody = $('<tbody />').appendTo(table);

    tr = $('<tr />', { 'id': 'out_freq', align: 'left'  }).append($('<td />').append(i18n("Frequency") + ":")).appendTo(tbody);
    $('<td />', { id: 'out_freq1_id', align: 'left'  }).appendTo(tr);

    tr = $('<tr />', { 'id': 'out_pwr', align: 'left'  }).append($('<td />').append(i18n("kW") + ":")).appendTo(tbody);
    $('<td />', { id: 'out_watt1_id', align: 'left' }).appendTo(tr);

    tr = $('<tr />', { 'id': 'out_kva', align: 'left'  }).append($('<td />').append(i18n("kVA") + ":")).appendTo(tbody);
    $('<td />', { id: 'out_va1_id', align: 'left' }).appendTo(tr);

    tr = $('<tr />', { 'id': 'out_load', align: 'left'  }).append($('<td />').append(i18n("Load") + ":")).appendTo(tbody);
    $('<td />', { id: 'out_load1_id', align: 'left' }).appendTo(tr);

    tr = $('<tr />', { 'id': 'out_cp', align: 'left'  }).append($('<td />').append(i18n("CosPhi") + ":")).appendTo(tbody);
    $('<td />', { id: 'out_cosphi1_id', align: 'left' }).appendTo(tr);

    fragment.appendChild(out_div[0]);
    document.body.appendChild(fragment);
    set_output_values();
  }

  function set_output_values() {
    var output_val = get_ph_1_3_vals(valtable_obj.OUTPUT_VOLT, valtable_obj.EX_OUT_CURR, valtable_obj.OUTFREQ, outphases, valtable_obj.EX_OUT_WATT, valtable_obj.EX_OUT_VA);

    if (output_val.availables[0] != 0) {
      $('#out_volt1_id').text(output_val.val_objs[0].values[0]);
      $('#out_curr1_id').text(output_val.val_objs[1].values[0]);
      $('#out_watt1_id').text(output_val.val_objs[3].values[0]);
      $('#out_va1_id').text(output_val.val_objs[4].values[0]);
      $('#out_freq1_id').text(output_val.val_objs[2].values[0]);
    }
    if (output_val.availables[1] != 0) {
      $('#out_volt2_id').text(output_val.val_objs[0].values[1]);
      $('#out_curr2_id').text(output_val.val_objs[1].values[1]);
    }
    if (output_val.availables[2] != 0) {
      $('#out_volt3_id').text(output_val.val_objs[0].values[2]);
      $('#out_curr3_id').text(output_val.val_objs[1].values[2]);
    }
    $('#out_load1_id').text(get_value({ values: [valtable_obj.LOAD_KVAR], index: 0, unit: ' kVAr', precision: 1 }));
    $('#out_cosphi1_id').text(get_value({ values: valtable_obj.OUTPUT_COSPHI, index: 0, unit: '', precision: 1 }));

  }

  function create_ups_status_info_pane() {
    var fragment = document.createDocumentFragment(),
    td, tr, table, tbody, thead, info_div;

    var css_obj = { position: 'absolute', top: y_pos + 60, left: x_pos + 840 };
    var src = './image/bacs2/b_grey.gif';

    info_div = $('<div/>', { id: 'ups_info_div_id' }).css(css_obj);
    table = $('<table />', { 'class': 'table table-condensed table-hover', 'id': 'infotable' }).appendTo(info_div);
    tbody = $('<tbody />').appendTo(table);
    thead = $('<thead />').appendTo(table);
    tr = $('<tr />').append($('<th />', { colspan: 2 }).append(i18n('Alarm Information'))).appendTo(thead);

    tr = $('<tr />').append($('<td />', { align: 'left' }).append($('<img />').attr({ 'id': 'al_epo_img', 'class': 'img-rounded', width: 16, 'height': 16, src: src }))).appendTo(tbody);
    $('<td />', { align: 'left' }).append(i18n("Emergency Stop")).appendTo(tr);

    tr = $('<tr />').append($('<td />', { align: 'left' }).append($('<img />').attr({ 'id': 'al_battchrg_img', 'class': 'img-rounded', width: 16, 'height': 16, src: src }))).appendTo(tbody);
    $('<td />', { align: 'left' }).append(i18n("Battery Charging Current")).appendTo(tr);

    tr = $('<tr />').append($('<td />', { align: 'left' }).append($('<img />').attr({ 'id': 'al_battvolt_img', 'class': 'img-rounded', width: 16, 'height': 16, src: src }))).appendTo(tbody);
    $('<td />', { align: 'left' }).append(i18n("Battery Voltage High/Low")).appendTo(tr);

    tr = $('<tr />').append($('<td />', { align: 'left' }).append($('<img />').attr({ 'id': 'al_genvolt_img', 'class': 'img-rounded', width: 16, 'height': 16, src: src }))).appendTo(tbody);
    $('<td />', { align: 'left' }).append(i18n("Generator Voltage High/Low")).appendTo(tr);

    tr = $('<tr />').append($('<td />', { align: 'left' }).append($('<img />').attr({ 'id': 'al_genfreq_img', 'class': 'img-rounded', width: 16, 'height': 16, src: src }))).appendTo(tbody);
    $('<td />', { align: 'left' }).append(i18n("Generator Frequency High/Low")).appendTo(tr);

    tr = $('<tr />').append($('<td />', { align: 'left' }).append($('<img />').attr({ 'id': 'al_hightemp_img', 'class': 'img-rounded', width: 16, 'height': 16, src: src }))).appendTo(tbody);
    $('<td />', { align: 'left' }).append(i18n("Cooling Water Temp High")).appendTo(tr);

    tr = $('<tr />').append($('<td />', { align: 'left' }).append($('<img />').attr({ 'id': 'al_rmplow_img', 'class': 'img-rounded', width: 16, 'height': 16, src: src }))).appendTo(tbody);
    $('<td />', { align: 'left' }).append(i18n("RPM Low")).appendTo(tr);

    tr = $('<tr />').append($('<td />', { align: 'left' }).append($('<img />').attr({ 'id': 'al_rmphigh_img', 'class': 'img-rounded', width: 16, 'height': 16, src: src }))).appendTo(tbody);
    $('<td />', { align: 'left' }).append(i18n("RPM High")).appendTo(tr);

    tr = $('<tr />').append($('<td />', { align: 'left' }).append($('<img />').attr({ 'id': 'al_overload_img', 'class': 'img-rounded', width: 16, 'height': 16, src: src }))).appendTo(tbody);
    $('<td />', { align: 'left' }).append(i18n("Overload")).appendTo(tr);

    tr = $('<tr />').append($('<td />', { align: 'left' }).append($('<img />').attr({ 'id': 'al_oilpress_img', 'class': 'img-rounded', width: 16, 'height': 16, src: src }))).appendTo(tbody);
    $('<td />', { align: 'left' }).append(i18n("Low Oil Pressure")).appendTo(tr);

    tr = $('<tr />').append($('<td />', { align: 'left' }).append($('<img />').attr({ 'id': 'al_lowfuel_img', 'class': 'img-rounded', width: 16, 'height': 16, src: src }))).appendTo(tbody);
    $('<td />', { align: 'left' }).append(i18n("Fuel Level Warning Alarm")).appendTo(tr);

    tr = $('<tr />').append($('<td />', { align: 'left' }).append($('<img />').attr({ 'id': 'al_general_img', 'class': 'img-rounded', width: 16, 'height': 16, src: src }))).appendTo(tbody);
    $('<td />', { align: 'left' }).append(i18n("General Alarm")).appendTo(tr);

    fragment.appendChild(info_div[0]);
    document.body.appendChild(fragment);
    set_ups_status_info();
  }

  function set_ups_status_info() {
    var grey = './image/bacs2/b_grey.gif';
    var red = './image/bacs2/b_red.gif';
    var src = grey;

    if (valtable_obj.RAWSTATEA & 0x1)
      src = red;
    $('#al_epo_img').attr('src', src);

    src = grey;
    if (valtable_obj.RAWSTATEA & 0x100)
      src = red;
    $('#al_battchrg_img').attr('src', src);

    src = grey;
    if (valtable_obj.RAWSTATEA & 0x200)
      src = red;
    $('#al_battvolt_img').attr('src', src);

    src = grey;
    if (valtable_obj.RAWSTATEA & 0x8)
      src = red;
    $('#al_genvolt_img').attr('src', src);

    src = grey;
    if (valtable_obj.RAWSTATEA & 0x10)
      src = red;
    $('#al_genfreq_img').attr('src', src);

    src = grey;
    if (valtable_obj.RAWSTATEA & 0x800)
      src = red;
    $('#al_hightemp_img').attr('src', src);

    src = grey;
    if (valtable_obj.RAWSTATEA & 0x80)
      src = red;
    $('#al_rmplow_img').attr('src', src);

    src = grey;
    if (valtable_obj.RAWSTATEA & 0x40)
      src = red;
    $('#al_rmphigh_img').attr('src', src);

    src = grey;
    if (valtable_obj.RAWSTATEA & 0x20)
      src = red;
    $('#al_overload_img').attr('src', src);

    src = grey;
    if (valtable_obj.RAWSTATEA & 0x2)
      src = red;
    $('#al_oilpress_img').attr('src', src);

    src = grey;
    if (valtable_obj.RAWSTATEA & 0x1000)
      src = red;
    $('#al_lowfuel_img').attr('src', src);

    src = grey;
    if (valtable_obj.STATUS & 0x2000)
      src = red;
    $('#al_general_img').attr('src', src);
  }

  function get_ph_1_3_vals(volt, cur, freq, phases, watt, va) {
    var val_obj, val;
    l_1_3_vals = {
      availables: [0, 0, 0], //L1-L3
      val_objs: [
        { unit: ' V', precision: 0, values: ['', '', ''] },
        { unit: ' A', precision: 1, values: ['', '', ''] },
        { unit: ' Hz', precision: 1, values: ['', '', ''] },
        { unit: ' kW', precision: 1, values: ['', '', ''] },
        { unit: ' kVA', precision: 1, values: ['', '', ''] }
      ]
    };
    volt = volt || [];
    cur = cur || [], freq = freq || [];
    watt = watt || [];
    va = va || [];
    $.each([volt, cur, freq, watt, va], function (index, values) {
      val_obj = l_1_3_vals.val_objs[index];
      $.each(values, function (i, val) {
        if (i < phases) {
          val = values[i];
          if (val != -9999) {
            if (values === freq && (i == 1 || i == 2)) {
              l_1_3_vals.availables[i] = 0;
            } else {
              l_1_3_vals.availables[i] = 1;
            }
            val = val.toFixed(val_obj.precision);
            val += val_obj.unit;
            val_obj.values[i] = val;
          }
        }
      });
    });
    return l_1_3_vals;
  }

  function get_value(val_obj) {
    if (!val_obj.values || (val_obj.values.length <= val_obj.index))
    return 'N/A';
    var val = val_obj.values[val_obj.index];
    if (val == null || val_obj.values[val_obj.index] == -9999)
    return 'N/A';

    val = val.toFixed(val_obj.precision);
    val += val_obj.unit;
    return val;
  }

}(jQuery));