<?xml version="1.0" ?>
<xsl:stylesheet version="1.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform" >
  <xsl:output method="html" encoding="ISO-8859-1" indent="yes"/>

  <xsl:template match="/">
    <html>
      <head>
        <meta name="robots" content="noindex" />
        <meta name="generator" content="GENEREX GmbH" />
        <meta http-equiv="Content-Type" content="text/html; charset=ISO-8859-1"/>
        <style type="text/css">
          .tdLargeBold { font-family: verdana,arial,serif; font-size: 10pt; font-weight: bold; color:#ffffff;text-align:center; background-color:#405080;}
          .tdMedium { font-family: verdana,arial,serif; font-size: 8pt; background-color:#f1f1f1; color:#405080;text-align:center;}
        </style>
        <title>UPS Alert History</title>
      </head>
      <body>
        <xsl:apply-templates />
      </body>
    </html>
  </xsl:template>

 	<xsl:variable name="lcletters">abcdefghijklmnopqrstuvwxyz</xsl:variable>
  <xsl:variable name="ucletters">ABCDEFGHIJKLMNOPQRSTUVWXYZ</xsl:variable>

  <xsl:template name="to-uppercase">
  	<xsl:param name="toconvert" select="''"/>
    <xsl:value-of select="translate($toconvert,$lcletters,$ucletters)"/>
  </xsl:template>

  <xsl:template name="to-lowercase">
  	<xsl:param name="toconvert" select="''"/>
    <xsl:value-of select="translate($toconvert,$ucletters,$lcletters)"/>
  </xsl:template>

  <xsl:template match="eventlog">
    <center>
      <h2 style="color:#405080;font-size:15pt; font-weight:bold; font-family: verdana,arial,serif; ">UPS Alert History</h2>
      <table cellspacing="2" cellpadding="1" border="0" borderColor="#000000">
        <tbody>
          <tr>
            <th class="tdLargeBold" width="200" align="center">Time</th>
            <th class="tdLargeBold" width="20" align="center">UpsID</th>
            <th class="tdLargeBold" width="20" align="center">Code</th>
            <th class="tdLargeBold" width="200" align="center">Event</th>
            <th class="tdLargeBold" width="20" align="center">State</th>
          </tr>
          <xsl:for-each select="event">
            <!--
            <xsl:sort select="@time" />
          -->
             <xsl:sort select="@upsid" order="ascending" data-type="number" />
            <tr>
							<xsl:variable name="ucode">
								<xsl:call-template name="to-uppercase">
						    	<xsl:with-param name="toconvert" select="@code"/>
						    </xsl:call-template>
						  </xsl:variable>

							<xsl:variable name="ustatus">
								<xsl:call-template name="to-uppercase">
						    	<xsl:with-param name="toconvert" select="@status"/>
						    </xsl:call-template>
						  </xsl:variable>

              <td align="center" width="200" class="tdMedium" >
                <xsl:value-of select='@time'/>
              </td>
              <td align="center" width="20" class="tdMedium">
                <xsl:value-of select='@upsid'/>
              </td>
              <td align="center" width="20" class="tdMedium">
                <xsl:value-of select='$ucode'/>
              </td>
              <td align="center" width="200"  class="tdMedium" style="text-align:center">
                <xsl:call-template name="text">
                  <xsl:with-param name="code" select='$ucode'/>
                </xsl:call-template>
              </td>
              <td align="center" width="20"  class="tdMedium" >
                <xsl:value-of select='$ustatus'/>
              </td>
            </tr>
          </xsl:for-each>

        </tbody>
      </table>
      <br/>
    </center>
  </xsl:template>

  <xsl:template name="text">
    <xsl:param name="code" select="'0'"/>
    <xsl:choose>
      <xsl:when test="$code = '1'">
        <xsl:text>Input Grid FreqNowOk</xsl:text>
      </xsl:when>
      <xsl:when test="$code = '2'">
        <xsl:text>Saved Parameters</xsl:text>
      </xsl:when>
      <xsl:when test="$code = '3'">
        <xsl:text>InpGridModelNowSync</xsl:text>
      </xsl:when>
      <xsl:when test="$code = '4'">
        <xsl:text>Input Grid not ok</xsl:text>
      </xsl:when>
      <xsl:when test="$code = '5'">
        <xsl:text>Battery connected</xsl:text>
      </xsl:when>
      <xsl:when test="$code = '6'">
        <xsl:text>Q_Out is open</xsl:text>
      </xsl:when>
      <xsl:when test="$code = '7'">
        <xsl:text>Q_Out is closed</xsl:text>
      </xsl:when>
      <xsl:when test="$code = '8'">
        <xsl:text>ServiceByPass open</xsl:text>
      </xsl:when>
      <xsl:when test="$code = '9'">
        <xsl:text>Service request notice</xsl:text>
      </xsl:when>
      <xsl:when test="$code = '101'">
        <xsl:text>PhaseSequenceError</xsl:text>
      </xsl:when>
      <xsl:when test="$code = '102'">
        <xsl:text>ServiceByPass closed</xsl:text>
      </xsl:when>
      <xsl:when test="$code = '201'">
        <xsl:text>Service Mode Active</xsl:text>
      </xsl:when>
      <xsl:when test="$code = '202'">
        <xsl:text>Output Overload</xsl:text>
      </xsl:when>
      <xsl:when test="$code = '203'">
        <xsl:text>Fan failure</xsl:text>
      </xsl:when>
      <xsl:when test="$code = '301'">
        <xsl:text>Configuration Alarm</xsl:text>
      </xsl:when>
      <xsl:when test="$code = '302'">
        <xsl:text>Service request alarm</xsl:text>
      </xsl:when>
      <xsl:when test="$code = '1001'">
        <xsl:text>FactoryTest Rect Off</xsl:text>
      </xsl:when>
      <xsl:when test="$code = '1101'">
        <xsl:text>Booster ShortCircuit</xsl:text>
      </xsl:when>
      <xsl:when test="$code = '1102'">
        <xsl:text>Rectifier measure fail</xsl:text>
      </xsl:when>
      <xsl:when test="$code = '1103'">
        <xsl:text>Rectifier fuse fault</xsl:text>
      </xsl:when>
      <xsl:when test="$code = '1201'">
        <xsl:text>Startup BST Short</xsl:text>
      </xsl:when>
      <xsl:when test="$code = '1202'">
        <xsl:text>Rectifier not synch</xsl:text>
      </xsl:when>
      <xsl:when test="$code = '1203'">
        <xsl:text>K_Rectifier not open</xsl:text>
      </xsl:when>
      <xsl:when test="$code = '2001'">
        <xsl:text>Input Grid ok</xsl:text>
      </xsl:when>
      <xsl:when test="$code = '2201'">
        <xsl:text>Input Grid not ok</xsl:text>
      </xsl:when>
      <xsl:when test="$code = '2202'">
        <xsl:text>InputGrid Freq NotOk</xsl:text>
      </xsl:when>
      <xsl:when test="$code = '2203'">
        <xsl:text>Check neutral connection</xsl:text>
      </xsl:when>
      <xsl:when test="$code = '3001'">
        <xsl:text>ByPass Grid not ok</xsl:text>
      </xsl:when>
      <xsl:when test="$code = '3101'">
        <xsl:text>ByPass measure fail</xsl:text>
      </xsl:when>
      <xsl:when test="$code = '3102'">
        <xsl:text>Load frequency NotOk</xsl:text>
      </xsl:when>
      <xsl:when test="$code = '3103'">
        <xsl:text>BP open due BPThyr open</xsl:text>
      </xsl:when>
      <xsl:when test="$code = '3104'">
        <xsl:text>Error phase sequence BP</xsl:text>
      </xsl:when>
      <xsl:when test="$code = '3201'">
        <xsl:text>ByPass freq.not ok</xsl:text>
      </xsl:when>
      <xsl:when test="$code = '3202'">
        <xsl:text>ByPass Grid not ok</xsl:text>
      </xsl:when>
      <xsl:when test="$code = '3203'">
        <xsl:text>K_ByPass not open</xsl:text>
      </xsl:when>
      <xsl:when test="$code = '3204'">
        <xsl:text>Load NotSynchWith BP</xsl:text>
      </xsl:when>
      <xsl:when test="$code = '3205'">
        <xsl:text>Check neutral connection BP</xsl:text>
      </xsl:when>
      <xsl:when test="$code = '4001'">
        <xsl:text>Service Mode end</xsl:text>
      </xsl:when>
      <xsl:when test="$code = '4002'">
        <xsl:text>Module Switched Off</xsl:text>
      </xsl:when>
      <xsl:when test="$code = '4003'">
        <xsl:text>Module Switched On</xsl:text>
      </xsl:when>
      <xsl:when test="$code = '4004'">
        <xsl:text>OffsetCal InProgress</xsl:text>
      </xsl:when>
      <xsl:when test="$code = '4005'">
        <xsl:text>OffsetCal stopped</xsl:text>
      </xsl:when>
      <xsl:when test="$code = '4006'">
        <xsl:text>OffsetCalSuccessful</xsl:text>
      </xsl:when>
      <xsl:when test="$code = '4007'">
        <xsl:text>ParamFromBackupCopy</xsl:text>
      </xsl:when>
      <xsl:when test="$code = '4008'">
        <xsl:text>From EcoMode to INV</xsl:text>
      </xsl:when>
      <xsl:when test="$code = '4009'">
        <xsl:text>Comm cmd to ByPass</xsl:text>
      </xsl:when>
      <xsl:when test="$code = '400A'">
        <xsl:text>Comm cmd to Inverter</xsl:text>
      </xsl:when>
      <xsl:when test="$code = '400B'">
        <xsl:text>EcoMode back to BP</xsl:text>
      </xsl:when>
      <xsl:when test="$code = '400C'">
        <xsl:text>BackToINV after fail</xsl:text>
      </xsl:when>
      <xsl:when test="$code = '400D'">
        <xsl:text>Panel Cmd to ByPass</xsl:text>
      </xsl:when>
      <xsl:when test="$code = '400E'">
        <xsl:text>Panel Cmd toInverter</xsl:text>
      </xsl:when>
      <xsl:when test="$code = '400F'">
        <xsl:text>Remote Cmd to ByPass</xsl:text>
      </xsl:when>
      <xsl:when test="$code = '4010'">
        <xsl:text>Remote CmdToInverter</xsl:text>
      </xsl:when>
      <xsl:when test="$code = '4011'">
        <xsl:text>Temper.within limit</xsl:text>
      </xsl:when>
      <xsl:when test="$code = '4012'">
        <xsl:text>FactRedTestInProg</xsl:text>
      </xsl:when>
      <xsl:when test="$code = '4013'">
        <xsl:text>InvOffNotForBattDsch</xsl:text>
      </xsl:when>
      <xsl:when test="$code = '4014'">
        <xsl:text>Generator ON</xsl:text>
      </xsl:when>
      <xsl:when test="$code = '4015'">
        <xsl:text>Generator OFF</xsl:text>
      </xsl:when>
      <xsl:when test="$code = '4016'">
        <xsl:text>GenON / BattChargOFF</xsl:text>
      </xsl:when>
      <xsl:when test="$code = '4017'">
        <xsl:text>Shut-down from SNMP</xsl:text>
      </xsl:when>
      <xsl:when test="$code = '4018'">
        <xsl:text>INV Off due EnergyMngmt</xsl:text>
      </xsl:when>
      <xsl:when test="$code = '4019'">
        <xsl:text>Restart after sleep-mode</xsl:text>
      </xsl:when>
      <xsl:when test="$code = '4101'">
        <xsl:text>BattRect unavailable</xsl:text>
      </xsl:when>
      <xsl:when test="$code = '4102'">
        <xsl:text>DSPComm: CheckSum</xsl:text>
      </xsl:when>
      <xsl:when test="$code = '4103'">
        <xsl:text>DSPComm: CheckCode</xsl:text>
      </xsl:when>
      <xsl:when test="$code = '4104'">
        <xsl:text>DspComm: Dsp1 no com</xsl:text>
      </xsl:when>
      <xsl:when test="$code = '4105'">
        <xsl:text>DspComm: BufferType</xsl:text>
      </xsl:when>
      <xsl:when test="$code = '4106'">
        <xsl:text>DspComm: FastDataRcv</xsl:text>
      </xsl:when>
      <xsl:when test="$code = '4107'">
        <xsl:text>DspComm: PntRcvOvf</xsl:text>
      </xsl:when>
      <xsl:when test="$code = '4108'">
        <xsl:text>Startup INV failure</xsl:text>
      </xsl:when>
      <xsl:when test="$code = '4109'">
        <xsl:text>DspComm: INV Alarm</xsl:text>
      </xsl:when>
      <xsl:when test="$code = '410A'">
        <xsl:text>Invalid ID Module</xsl:text>
      </xsl:when>
      <xsl:when test="$code = '410A'">
        <xsl:text>Invalid ID Module</xsl:text>
      </xsl:when>
      <xsl:when test="$code = '410B'">
        <xsl:text>INV DSPCommCheckSum</xsl:text>
      </xsl:when>
      <xsl:when test="$code = '410C'">
        <xsl:text>INV DspCommDsp2NoCom</xsl:text>
      </xsl:when>
      <xsl:when test="$code = '410D'">
        <xsl:text>INV BuffTypeManager</xsl:text>
      </xsl:when>
      <xsl:when test="$code = '410E'">
        <xsl:text>INV FastDataRcvComm</xsl:text>
      </xsl:when>
      <xsl:when test="$code = '410F'">
        <xsl:text>INV PntRcvOutOfRange</xsl:text>
      </xsl:when>
      <xsl:when test="$code = '4110'">
        <xsl:text>Parallel Logic reset</xsl:text>
      </xsl:when>
      <xsl:when test="$code = '4111'">
        <xsl:text>INV DspComFatalAlarm</xsl:text>
      </xsl:when>
      <xsl:when test="$code = '4112'">
        <xsl:text>MismatchMasterSignal</xsl:text>
      </xsl:when>
      <xsl:when test="$code = '4113'">
        <xsl:text>CAN:CntrlLifeTimeout</xsl:text>
      </xsl:when>
      <xsl:when test="$code = '4114'">
        <xsl:text>Q_OutAcknowledged</xsl:text>
      </xsl:when>
      <xsl:when test="$code = '4115'">
        <xsl:text>CAN: TxPassiveError</xsl:text>
      </xsl:when>
      <xsl:when test="$code = '4116'">
        <xsl:text>CAN:BusOff</xsl:text>
      </xsl:when>
      <xsl:when test="$code = '4201'">
        <xsl:text>Dsp Comm not started</xsl:text>
      </xsl:when>
      <xsl:when test="$code = '4202'">
        <xsl:text>CAN transm.not exec.</xsl:text>
      </xsl:when>
      <xsl:when test="$code = '4203'">
        <xsl:text>INV DspComNotStarted</xsl:text>
      </xsl:when>
      <xsl:when test="$code = '4204'">
        <xsl:text>CAN: RxPassiveError</xsl:text>
      </xsl:when>
      <xsl:when test="$code = '4205'">
        <xsl:text>Module HighTemperat.</xsl:text>
      </xsl:when>
      <xsl:when test="$code = '4206'">
        <xsl:text>CAN: master fail</xsl:text>
      </xsl:when>
      <xsl:when test="$code = '4301'">
        <xsl:text>Read Parameters Fail</xsl:text>
      </xsl:when>
      <xsl:when test="$code = '4302'">
        <xsl:text>BST OffsetCal Failed</xsl:text>
      </xsl:when>
      <xsl:when test="$code = '4303'">
        <xsl:text>INV OffsetCal Failed</xsl:text>
      </xsl:when>
      <xsl:when test="$code = '4304'">
        <xsl:text>Gain Calibr.Failed</xsl:text>
      </xsl:when>
      <xsl:when test="$code = '4305'">
        <xsl:text>BST AlarmSwitchCase</xsl:text>
      </xsl:when>
      <xsl:when test="$code = '4306'">
        <xsl:text>CalCodeOutOfRange</xsl:text>
      </xsl:when>
      <xsl:when test="$code = '4307'">
        <xsl:text>RegPntrOutOfRange</xsl:text>
      </xsl:when>
      <xsl:when test="$code = '4308'">
        <xsl:text>BST WatchDogAlarm</xsl:text>
      </xsl:when>
      <xsl:when test="$code = '4309'">
        <xsl:text>BST WatchDogMain</xsl:text>
      </xsl:when>
      <xsl:when test="$code = '430A'">
        <xsl:text>BST WatchDogTime0Isr</xsl:text>
      </xsl:when>
      <xsl:when test="$code = '430B'">
        <xsl:text>BST WatchDogTime1Isr</xsl:text>
      </xsl:when>
      <xsl:when test="$code = '430C'">
        <xsl:text>BST WatchDogAdc1Isr</xsl:text>
      </xsl:when>
      <xsl:when test="$code = '430D'">
        <xsl:text>DspComm: FatalAlarm</xsl:text>
      </xsl:when>
      <xsl:when test="$code = '430E'">
        <xsl:text>BST WDogIllegal_Isr</xsl:text>
      </xsl:when>
      <xsl:when test="$code = '430F'">
        <xsl:text>CheckParameterAlarm</xsl:text>
      </xsl:when>
      <xsl:when test="$code = '4310'">
        <xsl:text>Flash Write Error</xsl:text>
      </xsl:when>
      <xsl:when test="$code = '4311'">
        <xsl:text>SetDefaultParameters</xsl:text>
      </xsl:when>
      <xsl:when test="$code = '4312'">
        <xsl:text>FWCompatibilityCheck</xsl:text>
      </xsl:when>
      <xsl:when test="$code = '4313'">
        <xsl:text>INV AlarmSwitchCase</xsl:text>
      </xsl:when>
      <xsl:when test="$code = '4314'">
        <xsl:text>INV WatchDogAlarm</xsl:text>
      </xsl:when>
      <xsl:when test="$code = '4315'">
        <xsl:text>INV WatchDogMain</xsl:text>
      </xsl:when>
      <xsl:when test="$code = '4316'">
        <xsl:text>INV WatchDogTime0Isr</xsl:text>
      </xsl:when>
      <xsl:when test="$code = '4317'">
        <xsl:text>INV WatchDogTime1Isr</xsl:text>
      </xsl:when>
      <xsl:when test="$code = '4318'">
        <xsl:text>INV WDogAdcInt1Isr</xsl:text>
      </xsl:when>
      <xsl:when test="$code = '4319'">
        <xsl:text>Q_Out GlitchAlarm</xsl:text>
      </xsl:when>
      <xsl:when test="$code = '431A'">
        <xsl:text>INVPntMeasOutOfRange</xsl:text>
      </xsl:when>
      <xsl:when test="$code = '431B'">
        <xsl:text>INV WDogIllegal_Isr</xsl:text>
      </xsl:when>
      <xsl:when test="$code = '431C'">
        <xsl:text>External UPS OFF</xsl:text>
      </xsl:when>
      <xsl:when test="$code = '431D'">
        <xsl:text>BST downLoad aborted</xsl:text>
      </xsl:when>
      <xsl:when test="$code = '431E'">
        <xsl:text>BST downLoad requested</xsl:text>
      </xsl:when>
      <xsl:when test="$code = '431F'">
        <xsl:text>INV downLoad refused</xsl:text>
      </xsl:when>
      <xsl:when test="$code = '4320'">
        <xsl:text>INV downLoad requested</xsl:text>
      </xsl:when>
      <xsl:when test="$code = '4321'">
        <xsl:text>INV downLoad ended</xsl:text>
      </xsl:when>
      <xsl:when test="$code = '4322'">
        <xsl:text>from BST:downLoad ended</xsl:text>
      </xsl:when>
      <xsl:when test="$code = '4323'">
        <xsl:text>from BST:downLoad requested</xsl:text>
      </xsl:when>
      <xsl:when test="$code = '4324'">
        <xsl:text>from BST:downLoad ended</xsl:text>
      </xsl:when>
      <xsl:when test="$code = '4325'">
        <xsl:text>INV downLoad aborted</xsl:text>
      </xsl:when>
      <xsl:when test="$code = '4326'">
        <xsl:text>INV downLoad requested</xsl:text>
      </xsl:when>
      <xsl:when test="$code = '4327'">
        <xsl:text>INV configuration alarm</xsl:text>
      </xsl:when>
      <xsl:when test="$code = '4328'">
        <xsl:text>BatteryIsNotCharging</xsl:text>
      </xsl:when>
      <xsl:when test="$code = '5101'">
        <xsl:text>U_DcMax fault</xsl:text>
      </xsl:when>
      <xsl:when test="$code = '5102'">
        <xsl:text>PosThreshold U_DcMax</xsl:text>
      </xsl:when>
      <xsl:when test="$code = '5103'">
        <xsl:text>NegThreshold U_DcMax</xsl:text>
      </xsl:when>
      <xsl:when test="$code = '5104'">
        <xsl:text>Positive BST Fault</xsl:text>
      </xsl:when>
      <xsl:when test="$code = '5105'">
        <xsl:text>Negative BST Fault</xsl:text>
      </xsl:when>
      <xsl:when test="$code = '5106'">
        <xsl:text>DcLink Short Circuit</xsl:text>
      </xsl:when>
      <xsl:when test="$code = '5107'">
        <xsl:text>INV bridge fuse fail</xsl:text>
      </xsl:when>
      <xsl:when test="$code = '5201'">
        <xsl:text>U_Dc Supply Problem</xsl:text>
      </xsl:when>
      <xsl:when test="$code = '6101'">
        <xsl:text>UpsOffForDcOutAlarm</xsl:text>
      </xsl:when>
      <xsl:when test="$code = '6102'">
        <xsl:text>U_Dc INVERTER Alarm</xsl:text>
      </xsl:when>
      <xsl:when test="$code = '6103'">
        <xsl:text>U_Dc on LOAD alarm</xsl:text>
      </xsl:when>
      <xsl:when test="$code = '7001'">
        <xsl:text>ByPass Freq in range</xsl:text>
      </xsl:when>
      <xsl:when test="$code = '7002'">
        <xsl:text>ByPass Grid is avail</xsl:text>
      </xsl:when>
      <xsl:when test="$code = '7003'">
        <xsl:text>Load Frequency NowOk</xsl:text>
      </xsl:when>
      <xsl:when test="$code = '7004'">
        <xsl:text>ModelFrequency NowOk</xsl:text>
      </xsl:when>
      <xsl:when test="$code = '7005'">
        <xsl:text>Load BP Synchronized</xsl:text>
      </xsl:when>
      <xsl:when test="$code = '7006'">
        <xsl:text>Load Model now Syncr</xsl:text>
      </xsl:when>
      <xsl:when test="$code = '7101'">
        <xsl:text>ByPassThyristor open</xsl:text>
      </xsl:when>
      <xsl:when test="$code = '7102'">
        <xsl:text>ByPass config.failed</xsl:text>
      </xsl:when>
      <xsl:when test="$code = '7201'">
        <xsl:text>BypThyShort</xsl:text>
      </xsl:when>
      <xsl:when test="$code = '8001'">
        <xsl:text>BatteryTest Aborted</xsl:text>
      </xsl:when>
      <xsl:when test="$code = '8002'">
        <xsl:text>BattDeepTestEnded</xsl:text>
      </xsl:when>
      <xsl:when test="$code = '8003'">
        <xsl:text>BattQuickTestInProgr</xsl:text>
      </xsl:when>
      <xsl:when test="$code = '8004'">
        <xsl:text>BattQuickTest OK</xsl:text>
      </xsl:when>
      <xsl:when test="$code = '8005'">
        <xsl:text>Batt.Test only Master</xsl:text>
      </xsl:when>
      <xsl:when test="$code = '8101'">
        <xsl:text>BattMeasureFail</xsl:text>
      </xsl:when>
      <xsl:when test="$code = '8102'">
        <xsl:text>BatteryLowInvOff</xsl:text>
      </xsl:when>
      <xsl:when test="$code = '8103'">
        <xsl:text>BattDeepTestInProgr</xsl:text>
      </xsl:when>
      <xsl:when test="$code = '8104'">
        <xsl:text>BattQuickTestFailed</xsl:text>
      </xsl:when>
      <xsl:when test="$code = '8201'">
        <xsl:text>High voltage battery</xsl:text>
      </xsl:when>
      <xsl:when test="$code = '8202'">
        <xsl:text>Batt.connect.failed</xsl:text>
      </xsl:when>
      <xsl:when test="$code = '8203'">
        <xsl:text>*** Not used ***</xsl:text>
      </xsl:when>
      <xsl:when test="$code = '8204'">
        <xsl:text>BattRelais ActFailed</xsl:text>
      </xsl:when>
      <xsl:when test="$code = '8205'">
        <xsl:text>BattNotConnected</xsl:text>
      </xsl:when>
      <xsl:when test="$code = '8206'">
        <xsl:text>BattLow_3MinAutonomy</xsl:text>
      </xsl:when>
      <xsl:when test="$code = '8207'">
        <xsl:text>BattLowUpsOff</xsl:text>
      </xsl:when>
      <xsl:when test="$code = '8208'">
        <xsl:text>Battery not charged</xsl:text>
      </xsl:when>
      <xsl:when test="$code = '8209'">
        <xsl:text>Battery problem</xsl:text>
      </xsl:when>
      <xsl:when test="$code = '820A'">
        <xsl:text>I charge check fail</xsl:text>
      </xsl:when>
      <xsl:when test="$code = '820B'">
        <xsl:text>BattHighTemp</xsl:text>
      </xsl:when>
      <xsl:when test="$code = '8301'">
        <xsl:text>BatteryRelaisNotOpen</xsl:text>
      </xsl:when>
      <xsl:when test="$code = 'A001'">
        <xsl:text>K_INV close problem</xsl:text>
      </xsl:when>
      <xsl:when test="$code = 'A002'">
        <xsl:text>Model restored</xsl:text>
      </xsl:when>
      <xsl:when test="$code = 'A003'">
        <xsl:text>System Is Sync</xsl:text>
      </xsl:when>
      <xsl:when test="$code = 'A004'">
        <xsl:text>Sync Signal Not Ok</xsl:text>
      </xsl:when>
      <xsl:when test="$code = 'A005'">
        <xsl:text>Sync Signal Is Ok</xsl:text>
      </xsl:when>
      <xsl:when test="$code = 'A101'">
        <xsl:text>RegulModeFailure</xsl:text>
      </xsl:when>
      <xsl:when test="$code = 'A102'">
        <xsl:text>INV ContactNotClose</xsl:text>
      </xsl:when>
      <xsl:when test="$code = 'A103'">
        <xsl:text>Load short circuit</xsl:text>
      </xsl:when>
      <xsl:when test="$code = 'A104'">
        <xsl:text>FrequencyModelNotOk</xsl:text>
      </xsl:when>
      <xsl:when test="$code = 'A105'">
        <xsl:text>INV off for overload</xsl:text>
      </xsl:when>
      <xsl:when test="$code = 'A106'">
        <xsl:text>INV off for HiTemper</xsl:text>
      </xsl:when>
      <xsl:when test="$code = 'A107'">
        <xsl:text>INV voltage not ok</xsl:text>
      </xsl:when>
      <xsl:when test="$code = 'A108'">
        <xsl:text>INV bridge fail</xsl:text>
      </xsl:when>
      <xsl:when test="$code = 'A109'">
        <xsl:text>INV current failure</xsl:text>
      </xsl:when>
      <xsl:when test="$code = 'A10A'">
        <xsl:text>Model not ok</xsl:text>
      </xsl:when>
      <xsl:when test="$code = 'A10B'">
        <xsl:text>Load measure error</xsl:text>
      </xsl:when>
      <xsl:when test="$code = 'A201'">
        <xsl:text>INV ContactorNotOpen</xsl:text>
      </xsl:when>
      <xsl:when test="$code = 'A202'">
        <xsl:text>Switch time too long</xsl:text>
      </xsl:when>
      <xsl:when test="$code = 'A203'">
        <xsl:text>Load short circuit</xsl:text>
      </xsl:when>
      <xsl:when test="$code = 'A204'">
        <xsl:text>System Not Sync</xsl:text>
      </xsl:when>
      <xsl:when test="$code = 'B001'">
        <xsl:text>SysRedund NotAvailab</xsl:text>
      </xsl:when>
      <xsl:when test="$code = 'B002'">
        <xsl:text>LackOfInvSystemRed</xsl:text>
      </xsl:when>
      <xsl:when test="$code = 'B003'">
        <xsl:text>Redundancy N-1 ok</xsl:text>
      </xsl:when>
      <xsl:when test="$code = 'B004'">
        <xsl:text>Ready for MasterMode</xsl:text>
      </xsl:when>
      <xsl:when test="$code = 'B005'">
        <xsl:text>Not ready for Master</xsl:text>
      </xsl:when>
      <xsl:when test="$code = 'B101'">
        <xsl:text>Q_Out CheckFailure</xsl:text>
      </xsl:when>
      <xsl:when test="$code = 'B102'">
        <xsl:text>Q_Out check failure</xsl:text>
      </xsl:when>
      <xsl:when test="$code = 'B103'">
        <xsl:text>Model load not sync</xsl:text>
      </xsl:when>
      <xsl:when test="$code = 'B104'">
        <xsl:text>Parallel BUS-C failure</xsl:text>
      </xsl:when>
      <xsl:when test="$code = 'B105'">
        <xsl:text>Parallel BUS-A failure</xsl:text>
      </xsl:when>
      <xsl:when test="$code = 'B106'">
        <xsl:text>Parallel BUS-B failure</xsl:text>
      </xsl:when>
      <xsl:when test="$code = 'B201'">
        <xsl:text>Redundancy N-1 NotOk</xsl:text>
      </xsl:when>

      <xsl:otherwise>
        <xsl:text>Unknown Code</xsl:text>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:template>
</xsl:stylesheet>
