/*
   invertronic3ph_panel.js - INVERTRONIC 3PH screen.
   Copyright (c) 2017-18, Generex GmbH. All rights reserved.
   Said/UW.
*/

var invertronic3ph_api = (function ($) {
  var oid = get_oid_from_url(),
  cb = null,
  valtable_obj,
  report_obj, unms_obj,
  outphases = null,
  pannes_created = false,
  check_bowser = true,
  outpower_loadbars_created = false,
  err_div = null,
  css_obj,
  curr_select = 0,
  panels_id = ['#def_div_id', '#dc_div_id', '#byp_div_id', /*'#rec_div_id',*/ '#ph_div_id', '#inv_div_id'],
  req_params = null;

  function err_funct(response) {
    jqmodule.refresh(cb, 10 * 1000);
    panes_api.display_err({ response: response, oid: oid, err_div: err_div, err_msg_div_id: 'error_msg_div' });
  }

  return {
    init: init
  }

  function init() {
    if (check_bowser && !utils_api.check_bowser()) {
      $('#loading_id').css('display', 'none');
      panes_api.create_br_check_panel(this, browser_check_ok);
      return true;
    }
    create_err_msg_panel();

    req_params = utils_api.get_req_params(utils_api.device_ups, oid);
    cb = jqmodule.subscribe(this, { update: update, error: err_funct }, req_params);
  }

  function browser_check_ok() {
    check_bowser = false;
    init();
  }

  function create_err_msg_panel() {
    css_obj = { position: 'absolute', top: 30, left: 40, width: 790, display: 'none', 'z-index': 10 };
    err_div = panes_api.create_err_panel({ id: 'error_div', 'class': 'alert alert-danger' }, css_obj);
  }

  function update(data) {
    if (data.ups == null) {
      return;
    }

    if (err_div)
    err_div.css('display', 'none');

    valtable_obj = data.ups.valtable;
    report_obj = data.report;
    jqmodule.refresh(cb, report_obj.refresh * 1000);
    if (window.menu_api != null)
    menu_api.update(data);

    if (valtable_obj.OUTPHASES != null)
    outphases = valtable_obj.OUTPHASES;
    else
    outphases = valtable_obj.PHASES;

    unms_obj = get_unms_obj(data);
    if (unms_obj != null) {
      info_head_api.update(unms_obj);
    }
    utils_api.setTitle(valtable_obj._platform);

    if (!pannes_created) {
      pannes_created = true;
      create_pannes(); //create screen.
      return;
    }
    set_bypass_panel();
    set_inverter_panel();
    set_phase_panel();
    set_dc_panel();
    set_default_panel();
    set_inv_led_panel();
    set_out_led_panel();
    set_man_byp_led_panel();
    set_byp_led_panel();
    set_dc_in_led_panel();
    set_byp_ac_in_led_panel();
    set_leds_panel();
    set_footer();
  }

  function create_pannes() //create screen.
  {
    create_bg_panel();
    create_model_panel();
    create_select_panel();
//    create_rectifier_panel();
    create_bypass_panel();
    create_inverter_panel();
    create_phase_panel();
    create_dc_panel();
    create_default_panel();
    create_inv_led_panel();
    create_out_led_panel();
    create_man_byp_led_panel();
    create_byp_led_panel();
    create_dc_in_led_panel();
    create_byp_ac_in_led_panel();
    create_leds_panel();
    create_footer_pane();
  }

  function create_bg_panel() {
    var fragment = document.createDocumentFragment(),
    div;
    //background img
    css_obj = { position: 'absolute', top: 0, left: 0, width: 932, height: 518, align: 'center', 'z-index': 0 };
    div = $('<div/>').attr({ id: 'bg_div_id', 'class': 'LargeBold' }).css(css_obj);
    div.append($('<img border="0" src="./image/benning/backinv.gif" width="930" height="516"/>'));
    fragment.appendChild(div[0]);
    //Led Test
    css_obj = { position: 'absolute', top: 432, left: 66, width: 180, height: 30, align: 'center', 'text-align': 'center', 'background-color': 'transparent' };
    div = $('<div/>').attr({ id: 'led_test_div_id', 'class': 'LargeBold' }).css(css_obj);
    div.append(i18n('Led Test'));
    fragment.appendChild(div[0]);
    //bypass
    css_obj = {
      position: 'absolute',
      top: 434,
      left: 505,
      width: 180,
      height: 30,
      align: 'center',
      'text-align': 'center',
      'font-family': 'arial',
      'font-size': '7pt',
      'background-color': 'transparent'
    };
    div = $('<div/>').attr({ id: 'bypass_txt_div_id' }).css(css_obj);
    div.append(i18n('BYPASS'));
    fragment.appendChild(div[0]);
    //RESET
    css_obj = {
      position: 'absolute',
      top: 200,
      left: 552,
      width: 180,
      height: 30,
      align: 'center',
      'text-align': 'center',
      'background-color': 'transparent',
      'font-family': 'arial',
      'font-size': '7pt',
      'color': 'white'
    };
    div = $('<div/>').attr({ id: 'reset_txt_div_id' }).css(css_obj);
    fragment.appendChild(div[0]);
    div.append(i18n('RESET'));
    document.body.appendChild(fragment);
  }

  function create_model_panel() {
    var model_div;
    css_obj = { position: 'absolute', top: 88, left: 294, width: 406, 'height': 30, align: 'center', 'text-align': 'center', 'background-color': 'transparent' };
    model_div = panes_api.create_div({ id: 'model_div_id', 'class': 'XLargeBold_black' }, css_obj);
    model_div.append(valtable_obj.MODEL);
  }

  function setval(id, val, dez, scale, unit) {
    if (val == 32767)
    $(id).text('---' + unit);
    else
    $(id).text((val / scale).toFixed(dez) + unit);
  }

  function create_bypass_panel() {
    var fragment = document.createDocumentFragment(),
    byp_div, tbody, table, tr, td;
    css_obj = {
      position: 'absolute',
      top: 175,
      left: 339,
      width: 182,
      height: 64,
      'background-color': '#A5C3CE',
      'align': 'center',
      display: 'none',
      overflow: 'hidden'
    };
    //bypass div
    byp_div = $('<div/>').attr({ id: 'byp_div_id', 'class': 'XBold_black' }).css(css_obj);
    table = $('<table/>', { 'class': 'XBold_black', align: 'left', cellpadding: 0, cellspacing: 0, 'width': '94%' });
    table.css('margin-left', '4px');
    tbody = $('<tbody />');

    tr = $('<tr />');
    td = $('<td />', { 'align': 'left' }).append(i18n('BYP') + ':').appendTo(tr);
    td = $('<td />', { id: 'byp_kva_id', 'align': 'right' }).appendTo(tr);
    td = $('<td />', { id: 'byp_infreq_id', 'align': 'right' }).appendTo(tr);
    tr.appendTo(tbody);

    tr = $('<tr />');
    td = $('<td />', { 'align': 'left' }).append('L1:').appendTo(tr);
    td = $('<td />', { id: 'byp_inv_1_id', 'align': 'right' }).appendTo(tr);
    td = $('<td />', { id: 'byp_curr_1_id', 'align': 'right' }).appendTo(tr);
    tr.appendTo(tbody);

    tr = $('<tr />');
    td = $('<td />', { 'align': 'left' }).append('L2:').appendTo(tr);
    td = $('<td />', { id: 'byp_inv_2_id', 'align': 'right' }).appendTo(tr);
    td = $('<td />', { id: 'byp_curr_2_id', 'align': 'right' }).appendTo(tr);
    tr.appendTo(tbody);

    tr = $('<tr />');
    td = $('<td />', { 'align': 'left' }).append('L3:').appendTo(tr);
    td = $('<td />', { id: 'byp_inv_3_id', 'align': 'right' }).appendTo(tr);
    td = $('<td />', { id: 'byp_curr_3_id', 'align': 'right' }).appendTo(tr);
    tr.appendTo(tbody);

    tbody.appendTo(table);
    table.appendTo(byp_div);

    fragment.appendChild(byp_div[0]);
    document.body.appendChild(fragment);
    set_bypass_panel();
  }

  function set_bypass_panel() {
    setval('#byp_kva_id', valtable_obj.ADR73, 0, 1.0, ' kVA');
    if (valtable_obj.ADR6F == 240)
      $('#byp_infreq_id').text('---');
    else
      setval('#byp_infreq_id', valtable_obj.ADR6F, 1, 10.0, ' Hz');
    if (valtable_obj.ADRA3 != 0)
    {
      setval('#byp_inv_1_id', valtable_obj.ADR80, 0, 1.0, ' V');
      setval('#byp_inv_2_id', valtable_obj.ADR81, 0, 1.0, ' V');
      setval('#byp_inv_3_id', valtable_obj.ADR82, 0, 1.0, ' V');
    }
    else
    {
      var sqrt3 = 1.732;
      setval('#byp_inv_1_id', valtable_obj.ADR80 / sqrt3, 0, 1.0, ' V');
      setval('#byp_inv_2_id', valtable_obj.ADR81 / sqrt3, 0, 1.0, ' V');
      setval('#byp_inv_3_id', valtable_obj.ADR82 / sqrt3, 0, 1.0, ' V');
    }
    setval('#byp_curr_1_id', valtable_obj.ADR70, 0, 1.0, ' A');
    setval('#byp_curr_2_id', valtable_obj.ADR71, 0, 1.0, ' A');
    setval('#byp_curr_3_id', valtable_obj.ADR72, 0, 1.0, ' A');
  }

  function create_inverter_panel() {
    var fragment = document.createDocumentFragment(),
    inv_div, tbody, table, tr, td;
    css_obj = {
      position: 'absolute',
      top: 175,
      left: 339,
      width: 182,
      height: 64,
      'background-color': '#A5C3CE',
      'align': 'center',
      display: 'none',
      overflow: 'hidden'
    };
    //inverter div
    inv_div = $('<div/>').attr({ id: 'inv_div_id', 'class': 'XBold_black' }).css(css_obj);

    table = $('<table/>', { 'class': 'XBold_black', align: 'left', cellpadding: 0, cellspacing: 0, 'width': '94%' });
    table.css('margin-left', '4px');
    tbody = $('<tbody />');
    tr = $('<tr />');
    td = $('<td />', { 'align': 'left' }).append(i18n('INV') + ':').appendTo(tr);
    td = $('<td />', { id: 'inv_kva_id', 'align': 'right' }).append(valtable_obj.ADR87 + ' kVA').appendTo(tr);
    td = $('<td />', { id: 'inv_infreq_id', 'align': 'right' }).append((valtable_obj.ADR86 / 10).toFixed(1) + ' Hz').appendTo(tr);
    tr.appendTo(tbody);
    tr = $('<tr />');
    td = $('<td />', { 'align': 'left' }).append('L1:').appendTo(tr);
    td = $('<td />', { id: 'inv_inv_1_id', 'align': 'right' }).append(valtable_obj.ADR77.toFixed(0) + ' V').appendTo(tr);
    td = $('<td />', { id: 'inv_curr_1_id', 'align': 'right' }).append(valtable_obj.ADR83.toFixed(0) + ' A').appendTo(tr);
    tr.appendTo(tbody);
    tr = $('<tr />');
    td = $('<td />', { 'align': 'left' }).append('L2:').appendTo(tr);
    td = $('<td />', { id: 'inv_inv_2_id', 'align': 'right' }).append(valtable_obj.ADR78.toFixed(0) + ' V').appendTo(tr);
    td = $('<td />', { id: 'inv_curr_2_id', 'align': 'right' }).append(valtable_obj.ADR84.toFixed(0) + ' A').appendTo(tr);
    tr.appendTo(tbody);
    tr = $('<tr />');
    td = $('<td />', { 'align': 'left' }).append('L3:').appendTo(tr);
    td = $('<td />', { id: 'inv_inv_3_id', 'align': 'right' }).append(valtable_obj.ADR79.toFixed(0) + ' V').appendTo(tr);
    td = $('<td />', { id: 'inv_curr_3_id', 'align': 'right' }).append(valtable_obj.ADR85.toFixed(0) + ' A').appendTo(tr);
    tr.appendTo(tbody);

    tbody.appendTo(table);
    table.appendTo(inv_div);

    fragment.appendChild(inv_div[0]);
    document.body.appendChild(fragment);
    set_inverter_panel();
  }

  function set_inverter_panel() {
    setval('#inv_kva_id', valtable_obj.ADR87, 0, 1.0, ' kVA');
    setval('#inv_infreq_id', valtable_obj.ADR86, 1, 10.0, ' Hz');
    if (valtable_obj.ADRA2 != 0)
    {
      setval('#inv_inv_1_id', valtable_obj.ADR77, 0, 1.0, ' V');
      setval('#inv_inv_2_id', valtable_obj.ADR78, 0, 1.0, ' V');
      setval('#inv_inv_3_id', valtable_obj.ADR79, 0, 1.0, ' V');
    }
    else
    {
      var sqrt3 = 1.732;
      setval('#inv_inv_1_id', valtable_obj.ADR77 / sqrt3, 0, 1.0, ' V');
      setval('#inv_inv_2_id', valtable_obj.ADR78 / sqrt3, 0, 1.0, ' V');
      setval('#inv_inv_3_id', valtable_obj.ADR79 / sqrt3, 0, 1.0, ' V');
    }
    setval('#inv_curr_1_id', valtable_obj.ADR83, 0, 1.0, ' A');
    setval('#inv_curr_2_id', valtable_obj.ADR84, 0, 1.0, ' A');
    setval('#inv_curr_3_id', valtable_obj.ADR85, 0, 1.0, ' A');
  }

  function create_phase_panel() {
    var fragment = document.createDocumentFragment(),
    ph_div, tbody, table, tr, td;

    css_obj = {
      position: 'absolute',
      top: 175,
      left: 339,
      width: 182,
      height: 64,
      'background-color': '#A5C3CE',
      'align': 'center',
      display: 'none',
      overflow: 'hidden'
    };
    //phase div
    ph_div = $('<div/>').attr({ id: 'ph_div_id', 'class': 'XBold_black' }).css(css_obj);
    table = $('<table/>', { 'class': 'XBold_black', align: 'left', cellpadding: 0, cellspacing: 0, width: '94%' }).appendTo(ph_div);
    table.css('margin-left', '4px');
    tbody = $('<tbody />').appendTo(table);

    tr = $('<tr />').appendTo(tbody);
    td = $('<td />', { 'align': 'center', colspan: 2 }).append(i18n('PHASE ')).appendTo(tr);
    td = $('<td />').append(' &#160;').appendTo(tr);

    tr = $('<tr />').appendTo(tbody);
    td = $('<td />', { 'align': 'left' }).append(i18n('INV - BYP') + ':').appendTo(tr);
    td = $('<td />', { id: 'ph_inv_byp_id', 'align': 'right' }).appendTo(tr);

    tr = $('<tr />').appendTo(tbody);
    td = $('<td />', { 'align': 'left' }).append(i18n('INV - BUS') + ':').appendTo(tr);
    td = $('<td />', { id: 'ph_inv_bus_id', 'align': 'right' }).appendTo(tr);

    tr = $('<tr />').appendTo(tbody);
    td = $('<td />', { 'align': 'left' }).append(i18n('INV - EXT') + ':').appendTo(tr);
    td = $('<td />', { id: 'ph_inv_ext_id', 'align': 'right' }).appendTo(tr);

    fragment.appendChild(ph_div[0]);
    document.body.appendChild(fragment);
    set_phase_panel();
  }

  function set_phase_panel() {
    setval('#ph_inv_byp_id', valtable_obj.ADR9F, 0, 1.0, ' ' + i18n('DGR'));
    setval('#ph_inv_bus_id', valtable_obj.ADRA0, 0, 1.0, ' ' + i18n('DGR'));
    setval('#ph_inv_ext_id', valtable_obj.ADRB5, 0, 1.0, ' ' + i18n('DGR'));
  }

  function create_dc_panel() {
    var fragment = document.createDocumentFragment(),
    dc_div, tbody, table, tr, td;

    css_obj = {
      position: 'absolute',
      top: 175,
      left: 339,
      width: 182,
      height: 64,
      'background-color': '#A5C3CE',
      'align': 'center',
      display: 'none',
      overflow: 'hidden'
    };
    //DC div
    dc_div = $('<div/>').attr({ id: 'dc_div_id', 'class': 'XBold_black' }).css(css_obj);
    table = $('<table/>', { 'class': 'XBold_black', align: 'left', cellpadding: 0, cellspacing: 0, height: '100%', width: '94%' }).appendTo(dc_div);
    table.css('margin-left', '4px');

    tbody = $('<tbody />').appendTo(table);
    tr = $('<tr />').appendTo(tbody);
    td = $('<td />', { 'align': 'left' }).append(i18n('U BATT') + ':').appendTo(tr);
    td = $('<td />', { id: 'dc_u1_batt_id', 'align': 'right' }).appendTo(tr);

    tr = $('<tr />').appendTo(tbody);
    td = $('<td />', { 'align': 'left' }).append(i18n('I BATT') + ':').appendTo(tr);
    td = $('<td />', { id: 'dc_i1_batt_id', 'align': 'right' }).appendTo(tr);

    tr = $('<tr />').appendTo(tbody);
    td = $('<td />', { 'align': 'left' }).append(i18n('SWITCH OFF AT') + ':').appendTo(tr);
    td = $('<td />', { id: 'dc_sw_off_at_id', 'align': 'right' }).appendTo(tr);

    tr = $('<tr />').appendTo(tbody);
    td = $('<td />', { 'align': 'left' }).append(i18n('DC-LINK') + ':').appendTo(tr);
    td = $('<td />', { id: 'dc_link_id', 'align': 'right' }).appendTo(tr);

    fragment.appendChild(dc_div[0]);
    document.body.appendChild(fragment);
    set_dc_panel();
  }

  function set_dc_panel() {
    setval('#dc_u1_batt_id', valtable_obj.ADRB4, 0, 1.0, ' V');
    setval('#dc_i1_batt_id', valtable_obj.ADR95, 0, 1.0, ' A');
    setval('#dc_sw_off_at_id', valtable_obj.ADR97, 0, 1.0, ' V');
    setval('#dc_link_id', valtable_obj.ADR96, 0, 1.0, ' V');
  }

  function create_default_panel() {
    var fragment = document.createDocumentFragment(),
    def_div, tbody, table, tr, td;
    css_obj = {
      position: 'absolute',
      top: 175,
      left: 339,
      width: 182,
      height: 64,
      'background-color': '#A5C3CE',
      'align': 'left',
      display: 'block',
      overflow: 'hidden'
    };
    //default div
    def_div = $('<div/>').attr({ id: 'def_div_id', 'class': 'XBold_black' }).css(css_obj);

    table = $('<table/>', { 'class': 'XBold_black', align: 'left', cellpadding: 0, cellspacing: 0, height: '100%', width: '94%' });
    table.css('margin-left', '4px');
    tbody = $('<tbody />');
    date = new Date();
    tr = $('<tr />');
    td = $('<td />', { id: 'def_date_id', 'align': 'left' }).appendTo(tr);
    td = $('<td />', { id: 'def_time_id', 'align': 'right' }).appendTo(tr);
    tr.appendTo(tbody);
    tr = $('<tr />');
    if (valtable_obj.ADRC8 == 1)
    td = $('<td />', { id: 'def_hours_txt_id', 'align': 'left' }).append(i18n('MAINT. DUE IN') + ':').appendTo(tr);
    else
    td = $('<td />', { id: 'def_hours_txt_id', 'align': 'left' }).append(i18n('HOURS ') + ':').appendTo(tr);
    td = $('<td />', { id: 'def_hours_id', 'align': 'right' }).appendTo(tr);
    tr.appendTo(tbody);
    tr = $('<tr />');
    td = $('<td />', { 'align': 'left' }).append(i18n('COUNTER') + ':').appendTo(tr);
    td = $('<td />', { id: 'def_counter_id', 'align': 'right' }).appendTo(tr);
    tr.appendTo(tbody);
    tr = $('<tr />');
    td = $('<td />', { 'align': 'left' }).append(i18n('OUTPUT') + ':').appendTo(tr);
    td = $('<td />', { id: 'def_output_id', 'align': 'right' }).appendTo(tr);
    tr.appendTo(tbody);
    tbody.appendTo(table);
    table.appendTo(def_div);

    fragment.appendChild(def_div[0]);
    document.body.appendChild(fragment);
    set_default_panel();
  }

  function set_default_panel() {

    $('#def_date_id').text(valtable_obj.DATE);
    $('#def_time_id').text(valtable_obj.TIME);
    if (valtable_obj.ADRC8 == 1)
    $('#def_hours_txt_id').text(i18n('MAINT. DUE IN') + ':');
    else
    $('#def_hours_txt_id').text(i18n('HOURS ') + ':');
    if (valtable_obj.ADRC8 == 0)
    setval('#def_hours_id', (valtable_obj.ADR9B * 65536.0) + valtable_obj.ADR9C, 0, 1.0, ' h');
    else
    setval('#def_hours_id', valtable_obj.ADRC7, 0, 1.0, ' h');
    setval('#def_counter_id', valtable_obj.ADR9A, 0, 1.0, '');
    setval('#def_output_id', valtable_obj.ADR7C, 0, 1.0, ' kVA');
  }

  function get_inv_led_src(addr456, addr457) {
    if ((addr457 >> 11) & 1)
    return "./image/benning/ledrot10x10.gif";
    if ((addr456 >> 7) & 1)
    return "./image/benning/ledrot10x10_flash_l.gif";
    if ((addr456 >> 11) & 1)
    return "./image/benning/ledgruen10x10.gif";
    if ((addr457 >> 10) & 1)
    return "./image/benning/ledgruen10x10_flash_l.gif";
    if ((addr457 >> 8) & 1)
    return "./image/benning/ledgelb10x10.gif";
    if ((addr456 >> 10) & 1)
    return "./image/benning/ledgelb10x10_flash_l.gif";
    if ((addr456 >> 9) & 1)
    return "./image/benning/ledgruen_gelb10x10_flash_l.gif";
    if ((addr456 >> 8) & 1)
    return "./image/benning/ledrot_gelb10x10_flash_l.gif";

    return "./image/benning/ledgrau10x10.gif";
  }

  function create_inv_led_panel() //bypass in prot
  {
    //inverter div
    var fragment = document.createDocumentFragment(),
    inv_led_div, img;
    css_obj = { position: 'absolute', top: 391, left: 472, width: 14, height: 14, align: 'center', overflow: 'hidden' };
    inv_led_div = $('<div/>').attr({ id: 'inv_led_div_id' }).css(css_obj);
    img = $('<img />', { id: 'inv_led_img_id', src: get_inv_led_src(valtable_obj.ADR1C8, valtable_obj.ADR1C9) }).appendTo(inv_led_div);
    fragment.appendChild(inv_led_div[0]);
    document.body.appendChild(fragment);
  }

  function set_inv_led_panel() { $('#inv_led_img_id').attr('src', get_inv_led_src(valtable_obj.ADR1C8, valtable_obj.ADR1C9)); }

  function create_out_led_panel() {
    var fragment = document.createDocumentFragment(),
    out_led_div, img;
    css_obj = { position: 'absolute', top: 378, left: 630, width: 14, height: 14, align: 'center', overflow: 'hidden' };
    out_led_div = $('<div/>').attr({ id: 'out_led_div_id' }).css(css_obj);
    img = $('<img />', { id: 'out_led_img_id', src: get_out_led_src(valtable_obj.ADR1C7) }).appendTo(out_led_div);
    fragment.appendChild(out_led_div[0]);
    document.body.appendChild(fragment);
  }

  function set_out_led_panel() { $('#out_led_img_id').attr('src', get_out_led_src(valtable_obj.ADR1C7)); }

  function create_man_byp_led_panel() {
    //Bypass div
    var fragment = document.createDocumentFragment(),
    man_byp_led_div, img;
    css_obj = { position: 'absolute', top: 286, left: 630, width: 14, height: 14, align: 'center', overflow: 'hidden' };
    man_byp_led_div = $('<div/>').attr({ id: 'man_byp_led_div_id' }).css(css_obj);
    img = $('<img />', { id: 'man_byp_led_img_id', src: get_man_byp_led_src() }).appendTo(man_byp_led_div);
    fragment.appendChild(man_byp_led_div[0]);
    document.body.appendChild(fragment);
  }

  function set_man_byp_led_panel() { $('#man_byp_led_img_id').attr('src', get_man_byp_led_src()); }

  function get_byp_led_src(addr456, addr457) {
    if ((addr456 >> 12) & 1)
    return "./image/benning/ledrot10x10.gif";
    if ((addr456 >> 13) & 1)
    return "./image/benning/ledrot10x10_flash_l.gif";
    if ((addr456 >> 15) & 1)
    return "./image/benning/ledgruen10x10.gif";
    if ((addr457 >> 9) & 1)
    return "./image/benning/ledgruen10x10_flash_l.gif";
    if ((addr457 >> 0) & 1)
    return "./image/benning/ledgelb10x10.gif";
    if ((addr456 >> 14) & 1)
    return "./image/benning/ledgelb10x10_flash_l.gif";
    if ((addr456 >> 2) & 1)
    return "./image/benning/ledgruen_gelb10x10_flash_l.gif";
    if ((addr457 >> 5) & 1)
    return "./image/benning/ledrot_gelb10x10_flash_l.gif";

    return "./image/benning/ledgrau10x10.gif";
  }

  function create_byp_led_panel() {
    //static byp div
    var fragment = document.createDocumentFragment(),
    byp_led_div, img;
    css_obj = { position: 'absolute', top: 339, left: 472, width: 14, height: 14, align: 'center', overflow: 'hidden' };
    byp_led_div = $('<div/>').attr({ id: 'byp_led_div_id' }).css(css_obj);
    img = $('<img />', { id: 'byp_led_img_id', src: get_byp_led_src(valtable_obj.ADR1C8, valtable_obj.ADR1C9) }).appendTo(byp_led_div);
    fragment.appendChild(byp_led_div[0]);
    document.body.appendChild(fragment);
  }

  function set_byp_led_panel() { $('#byp_led_img_id').attr('src', get_byp_led_src(valtable_obj.ADR1C8, valtable_obj.ADR1C9)); }

  function get_batt_led_src(addr456, addr457) {
    if ((addr457 >> 3) & 1)
    return "./image/benning/ledrot10x10.gif";
    if ((addr456 >> 3) & 1)
    return "./image/benning/ledrot10x10_flash_l.gif";
    if ((addr456 >> 5) & 1)
    return "./image/benning/ledgruen10x10.gif";
    if ((addr456 >> 6) & 1)
    return "./image/benning/ledgruen10x10_flash_l.gif";
    if ((addr456 >> 4) & 1)
    return "./image/benning/ledgelb10x10.gif";
    if ((addr457 >> 4) & 1)
    return "./image/benning/ledgelb10x10_flash_l.gif";
    if ((addr457 >> 13) & 1)
    return "./image/benning/ledgruen_gelb10x10_flash_l.gif";
    if ((addr457 >> 14) & 1)
    return "./image/benning/ledrot_gelb10x10_flash_l.gif";

    return "./image/benning/ledgrau10x10.gif";
  }

  function create_dc_in_led_panel() {
    //DC in-LED  div
    var fragment = document.createDocumentFragment(),
    batt_led_div, img;
    css_obj = { position: 'absolute', top: 378, left: 312, width: 14, height: 14, align: 'center', overflow: 'hidden' };
    batt_led_div = $('<div/>').attr({ id: 'dc_in_led_div_id' }).css(css_obj);
    img = $('<img />', { id: 'dc_in_led_img_id', src: get_batt_led_src(valtable_obj.ADR1C8, valtable_obj.ADR1C9) }).appendTo(batt_led_div);
    fragment.appendChild(batt_led_div[0]);
    document.body.appendChild(fragment);
  }

  function set_dc_in_led_panel() { $('#dc_in_led_img_id').attr('src', get_batt_led_src(valtable_obj.ADR1C8, valtable_obj.ADR1C9)); }

  function get_rec_led_src(addr455, addr456, addr457) {
    if ((addr455 >> 12) & 1)
    return "./image/benning/ledrot10x10.gif";
    if ((addr455 >> 13) & 1)
    return "./image/benning/ledrot10x10_flash_l.gif";
    if ((addr456 >> 1) & 1)
    return "./image/benning/ledgruen10x10.gif";
    if ((addr457 >> 12) & 1)
    return "./image/benning/ledgruen10x10_flash_l.gif";
    if ((addr456 >> 0) & 1)
    return "./image/benning/ledgelb10x10.gif";
    if ((addr455 >> 14) & 1)
    return "./image/benning/ledgelb10x10_flash_l.gif";
    if ((addr455 >> 15) & 1)
    return "./image/benning/ledgruen_gelb10x10_flash_l.gif";

    return "./image/benning/ledgrau10x10.gif";
  }

  function create_byp_ac_in_led_panel() {
    //DC in-LED  div
    var fragment = document.createDocumentFragment(),
    batt_led_div, img;
    css_obj = { position: 'absolute', top: 325, left: 312, width: 14, height: 14, align: 'center', overflow: 'hidden' };
    batt_led_div = $('<div/>').attr({ id: 'byp_ac_in_led_div_id' }).css(css_obj);
    img = $('<img />', { id: 'byp_ac_in_led_img_id', src: get_rec_led_src(valtable_obj.ADR1C7, valtable_obj.ADR1C8, valtable_obj.ADR1C9) }).appendTo(batt_led_div);
    fragment.appendChild(batt_led_div[0]);
    document.body.appendChild(fragment);
  }

  function set_byp_ac_in_led_panel() { $('#byp_ac_in_led_img_id').attr('src', get_rec_led_src(valtable_obj.ADR1C7, valtable_obj.ADR1C8, valtable_obj.ADR1C9)); }

  function get_man_byp_led_src() {
    // addr 457, bit 1
    if (valtable_obj.ADR1C9 & (1 << 1))
    return "./image/benning/ledgruen10x10.gif";
    return "./image/benning/ledgrau10x10.gif";
  }

  function get_out_led_src(addr) {
    if (addr & (1 << 10))
    return "./image/benning/ledgruen10x10_flash_m.gif";
    if (addr & (1 << 11))
    return "./image/benning/ledgruen10x10.gif";

    return "./image/benning/ledgrau10x10.gif";
  }

  function create_select_panel() {
    var fragment = document.createDocumentFragment(),
    sel_up_div, sel_down_div, img;
    css_obj = {
      position: 'absolute',
      top: 165,
      left: 554,
      width: 41,
      height: 39,
      'background-color': 'transparent',
      align: 'center',
      display: 'block',
      overflow: 'hidden',
      cursor: 'pointer'
    };
    //sel_up div
    sel_up_div = $('<div/>').attr({ id: 'sel_up_div_id' }).css(css_obj);
    img = $('<img />', { 'src': "./image/benning/up.png" }).appendTo(sel_up_div);

    sel_up_div.click(function () { selction_up(); });
    //sel_down div
    css_obj['top'] = 208;
    sel_down_div = $('<div/>').attr({ id: 'sel_down_div_id' }).css(css_obj);
    img = $('<img />', { 'src': "./image/benning/down.png" }).appendTo(sel_down_div);
    sel_down_div.click(function () { selction_down(); });

    fragment.appendChild(sel_up_div[0]);
    fragment.appendChild(sel_down_div[0]);
    document.body.appendChild(fragment);
  }

  function selction_up() {
    ++curr_select;
    select_panel();
  }

  function selction_down() {
    --curr_select;
    select_panel();
  }

  function select_panel(panel_nr) {
    if (curr_select >= panels_id.length) curr_select = 0;
    if (curr_select < 0) curr_select = panels_id.length - 1;
    $.each(panels_id, function (i, el) { $(el).css('display', 'none'); });
    $('#dc_div1_id').css('display', 'none');
    $(panels_id[curr_select]).css('display', 'block');
    if (curr_select == 4)
    $('#dc_div1_id').css('display', 'block');
  }

  function create_leds_panel() {
    var fragment = document.createDocumentFragment(),
    led_div, img, tbody, table, tr, td;
    css_obj = { position: 'absolute', 'top': 74, left: 56, width: 212, height: 316, align: 'center', overflow: 'hidden', 'z-index': 2 };
    led_div = $('<div/>').attr({ id: 'leds_div_id' }).css(css_obj);

    table = $('<table/>', { 'class': 'XBold_black', align: 'center', cellpadding: 2, cellspacing: 8, 'text-align': 'center', height: '100%', width: '100%' });
    tbody = $('<tbody />').appendTo(table);
    table.appendTo(led_div);

    tr = $('<tr />').appendTo(tbody);
    td = $('<td />').appendTo(tr);
    img = $('<img />', { id: 'led_1_img', src: './image/benning/ledgruen14x14.gif', 'z-index': 2 }).appendTo(td);
    td = $('<td />', { id: 'led_1_txt', align: 'left' }).append('&nbsp;' + i18n('UPS internal power supply')).appendTo(tr);
    td.css('border', '2px solid #7C9BB7');

    tr = $('<tr />').appendTo(tbody);
    td = $('<td />').appendTo(tr);
    img = $('<img />', { id: 'led_2_img', src: './image/benning/ledgruen14x14.gif', 'z-index': 2 }).appendTo(td);
    td = $('<td />', { id: 'led_2_txt', align: 'left' }).append("").appendTo(tr);
    td.css({ 'border': '2px solid #7C9BB7' });

    tr = $('<tr />').appendTo(tbody);
    td = $('<td />').appendTo(tr);
    img = $('<img />', { id: 'led_3_img', src: './image/benning/ledgruen14x14.gif', 'z-index': 2 }).appendTo(td);
    td = $('<td />', { id: 'led_3_txt', align: 'left' }).append("").appendTo(tr);
    td.css('border', '2px solid #7C9BB7');

    tr = $('<tr />').appendTo(tbody);
    td = $('<td />').appendTo(tr);
    img = $('<img />', { id: 'led_4_img', src: './image/benning/ledgruen14x14.gif', 'z-index': 2 }).appendTo(td);
    td = $('<td />', { id: 'led_4_txt', align: 'left' }).append("").appendTo(tr);
    td.css('border', '2px solid #7C9BB7');

    tr = $('<tr />').appendTo(tbody);
    td = $('<td />').appendTo(tr);
    img = $('<img />', { id: 'led_5_img', src: './image/benning/ledgruen14x14.gif', 'z-index': 2 }).appendTo(td);
    td = $('<td />', { id: 'led_5_txt', align: 'left' }).append("").appendTo(tr);
    td.css('border', '2px solid #7C9BB7');

    tr = $('<tr />').appendTo(tbody);
    td = $('<td />').appendTo(tr);
    img = $('<img />', { id: 'led_6_img', src: './image/benning/ledgruen14x14.gif', 'z-index': 2 }).appendTo(td);
    td = $('<td />', { id: 'led_6_txt', align: 'left' }).append("").appendTo(tr);
    td.css('border', '2px solid #7C9BB7');

    tr = $('<tr />').appendTo(tbody);
    td = $('<td />').appendTo(tr);
    img = $('<img />', { id: 'led_7_img', src: './image/benning/ledgruen14x14.gif', 'z-index': 2 }).appendTo(td);
    td = $('<td />', { id: 'led_7_txt', align: 'left' }).append("").appendTo(tr);
    td.css('border', '2px solid #7C9BB7');

    tr = $('<tr />').appendTo(tbody);
    td = $('<td />').appendTo(tr);
    img = $('<img />', { id: 'led_8_img', src: './image/benning/ledgruen14x14.gif', 'z-index': 2 }).appendTo(td);
    td = $('<td />', { id: 'led_8_txt', align: 'left' }).append("").appendTo(tr);
    td.css('border', '2px solid #7C9BB7');

    tr = $('<tr />').appendTo(tbody);
    td = $('<td />').appendTo(tr);
    img = $('<img />', { id: 'led_9_img', src: './image/benning/ledgruen14x14.gif', 'z-index': 2 }).appendTo(td);
    td = $('<td />', { id: 'led_9_txt', align: 'left' }).append("").appendTo(tr);
    td.css('border', '2px solid #7C9BB7');

    tr = $('<tr />').appendTo(tbody);
    td = $('<td />').appendTo(tr);
    img = $('<img />', { id: 'led_10_img', src: './image/benning/ledgruen14x14.gif', 'z-index': 2 }).appendTo(td);
    td = $('<td />', { id: 'led_10_txt', align: 'left' }).append("").appendTo(tr);
    td.css('border', '2px solid #7C9BB7');

    tr = $('<tr />').appendTo(tbody);
    td = $('<td />').appendTo(tr);
    img = $('<img />', { id: 'led_11_img', src: './image/benning/ledgruen14x14.gif', 'z-index': 2 }).appendTo(td);
    td = $('<td />', { id: 'led_11_txt', align: 'left' }).append("").appendTo(tr);
    td.css('border', '2px solid #7C9BB7');

    tr = $('<tr />').appendTo(tbody);
    td = $('<td />').appendTo(tr);
    img = $('<img />', { id: 'led_12_img', src: './image/benning/ledgruen14x14.gif', 'z-index': 2 }).appendTo(td);
    td = $('<td />', { id: 'led_12_txt', align: 'left' }).append("").appendTo(tr);
    td.css('border', '2px solid #7C9BB7');

    tr = $('<tr />').appendTo(tbody);
    td = $('<td />').appendTo(tr);
    img = $('<img />', { id: 'led_13_img', src: './image/benning/ledgruen14x14.gif', 'z-index': 2 }).appendTo(td);
    td = $('<td />', { id: 'led_13_txt', align: 'left' }).append("").appendTo(tr);
    td.css('border', '2px solid #7C9BB7');

    fragment.appendChild(led_div[0]);
    document.body.appendChild(fragment);

    set_leds_panel();
  }

  function set_leds_panel() {
    set_led_panel('led_2', get_led_txt(valtable_obj.PILLER_IO_TEXTS23), get_led_src(0x8000, valtable_obj.PILLER_IO_TEXTS23));
    set_led_panel('led_3', get_led_txt(valtable_obj.PILLER_IO_TEXTS21), get_led_src(0x4000, valtable_obj.PILLER_IO_TEXTS21));
    set_led_panel('led_4', get_led_txt(valtable_obj.PILLER_IO_TEXTS19), get_led_src(0x2000, valtable_obj.PILLER_IO_TEXTS19));
    set_led_panel('led_5', get_led_txt(valtable_obj.PILLER_IO_TEXTS17), get_led_src(0x1000, valtable_obj.PILLER_IO_TEXTS17));
    set_led_panel('led_6', get_led_txt(valtable_obj.PILLER_IO_TEXTS15), get_led_src(0x800, valtable_obj.PILLER_IO_TEXTS15));
    set_led_panel('led_7', get_led_txt(valtable_obj.PILLER_IO_TEXTS13), get_led_src(0x400, valtable_obj.PILLER_IO_TEXTS13));
    set_led_panel('led_8', get_led_txt(valtable_obj.PILLER_IO_TEXTS11), get_led_src(0x200, valtable_obj.PILLER_IO_TEXTS11));
    set_led_panel('led_9', get_led_txt(valtable_obj.PILLER_IO_TEXTS09), get_led_src(0x100, valtable_obj.PILLER_IO_TEXTS09));
    set_led_panel('led_10', get_led_txt(valtable_obj.PILLER_IO_TEXTS07), get_led_src(0x80, valtable_obj.PILLER_IO_TEXTS07));
    set_led_panel('led_11', get_led_txt(valtable_obj.PILLER_IO_TEXTS05), get_led_src(0x40, valtable_obj.PILLER_IO_TEXTS05));
    set_led_panel('led_12', get_led_txt(valtable_obj.PILLER_IO_TEXTS03), get_led_src(0x20, valtable_obj.PILLER_IO_TEXTS03));
    set_led_panel('led_13', get_led_txt(valtable_obj.PILLER_IO_TEXTS01), get_led_src(0x10, valtable_obj.PILLER_IO_TEXTS01));
  }

  function set_led_panel(id, text, img_src) {
    $('#' + id + '_img').attr('src', img_src);
    $('#' + id + '_txt').html(text);
  }

  function get_led_src(led_maske, led_txt) {
    var led_color = "";

    if (!(valtable_obj.ADR1E1 & led_maske))
    return "./image/benning/ledgrau14x14.gif";

    if (led_txt.length == 0)
    return "./image/benning/ledweiss14x14.gif";

    led_color = led_txt.substring(0, 1);
    if (led_color == '0')
    return "./image/benning/ledgrau14x14.gif";
    if (led_color == '1')
    return "./image/benning/ledrot14x14.gif";
    if (led_color == '2')
    return "./image/benning/ledgelb14x14.gif";
    if (led_color == '3')
    return "./image/benning/ledgruen14x14.gif";

    return "./image/benning/ledweiss14x14.gif";
  }

  function get_led_txt(led_txt) {
    if (led_txt.length > 0) {
      return '&nbsp;' + i18n(led_txt.substring(1));
    }
    return '';
  }

  function create_footer_pane() {
    if (!oid || !unms_obj)
    return true;
    footer_api.create({ //Attr
        id: 'panel2'
      }, { //css
        position: 'absolute',
        top: '540px'
      },
      unms_obj
    );
  }

  function set_footer() {
    if (!oid || !unms_obj)
    return true;
    footer_api.set(unms_obj);
  }
}(jQuery));