/*
   keormod_panel.js - Legrand KEOR MOD
   Copyright (c) 2019, Generex GmbH. All rights reserved.
   UW.
*/

var keormod_api = (function ($) {
  var oid = get_oid_from_url(),
    cb = null,
    pannes_created = false,
    err_div = null,
    inphases = null,
    outphases = null,
    bypphases = null,
    check_bowser = true,
    css_obj, attr_obj,
    valtable_obj = null,
    report_obj = null,
    ups_obj = null,
    unms_obj = null,
    status, tempmax, tempdeg, overload = 0,
    plattform = get_plattform(),
    ups_id = 0,
    ups_protocol = 0,
    prot_snmp = 0x803A,
    bg_height, bg_width;
    x_pos = 10, y_pos = 10;

  var status_obj = [

      { bit: 0x0010, txt: "Low Battery" },
      { bit: 0x0020, txt: "Overheat" },
      { bit: 0x0100, txt: "Overload" },
      { bit: 0x0400, txt: "Verify Batteries" },
      { bit: 0x2000, txt: "Hardware fault" },
      { bit: 0x4000, txt: "Communications problems between agent and UPS" }
    ],
    rawstatea_obj = [
      { bit: 0x00000020, txt: "Battery charger fault" },
      { bit: 0x00000040, txt: "Fan error" },
      { bit: 0x00000400, txt: "Battery fuse broken" },
      { bit: 0x00000800, txt: "Mains fuse broken" },
      { bit: 0x00002000, txt: "Make maintenance" }
    ];

  var req_params = null;

  /*public functions*/
  return {
    init: init
  };

  function init() {
    if (check_bowser && !utils_api.check_bowser()) {
      $('#loading_id').css('display', 'none');
      panes_api.create_br_check_panel(this, browser_check_ok);
      return true;
    }
    create_err_msg_panel();

    req_params = utils_api.get_req_params(utils_api.device_ups, oid);
    cb = jqmodule.subscribe(this, { update: update, error: err_funct }, req_params);
  }

  function browser_check_ok() {
    check_bowser = false;
    init();
  }

  function create_err_msg_panel() {
    css_obj = { position: 'absolute', top: y_pos, left: x_pos, width: 791, display: 'none', 'z-index': 62 };
    err_div = panes_api.create_err_panel({ id: 'error_div', 'class': 'alert alert-danger' }, css_obj);
  }

  function err_funct(response) {
    jqmodule.refresh(cb, 10 * 1000);
    panes_api.display_err({ response: response, oid: oid, err_div: err_div, err_msg_div_id: 'error_msg_div' });
  }

  function update(data) {
    if (data.ups == null) {
      return;
    }

    set_variable(data);
    jqmodule.refresh(cb, report_obj.refresh * 1000);
    if (err_div)
      err_div.css('display', 'none');

    if (window.menu_api != null)
      menu_api.update(data);

    unms_obj = get_unms_obj(data);
    if (unms_obj != null) {
      info_head_api.update(unms_obj);
    }

    if (!pannes_created) {
      pannes_created = true;
      create_panes();
      return true;
    }

    set_values();
  }

  function set_values() {
    set_input_values(); //Input values
    set_bypass_values(); //Bypass values
    set_output_values(); //Output values
    set_ext_values();   // ext. info
    set_symbols();
    set_battery_values(); //Battery value
    set_ups_status(); //set UPS status
    set_ups_alarm();
    set_outpower_loadbars(); //outpower loadbars
    set_model_name();
    set_sys_mod_status();
    set_sys_mod_alarm();
    set_footer();
    panes_api.set_autonomy_time_infos(valtable_obj);
  }

  function create_footer_pane() {
    if (!oid || !unms_obj)
      return true;
    footer_api.create({ //Attr
        id: 'panel2'
        //'class':'unmsFooter'
      }, { //css
        position: 'absolute',
        top:  '540px',
        left: '74px'
      },
      unms_obj
    );
  }

  function set_footer() {
    if (!oid || !unms_obj)
      return true;
    footer_api.set(unms_obj);
  }

  function set_variable(data) {
    ups_obj = data.ups;
    report_obj = data.report;
    valtable_obj = ups_obj.valtable;
    status = valtable_obj.STATUS;
    tempmax = valtable_obj.TMPMAX;
    tempdeg = valtable_obj.TEMPDEG;
    level = valtable_obj.LEVEL;

    if (valtable_obj._upsProtocol == null) {
      ups_protocol = valtable_obj.CONNECT & 0XFFFF;
    } else {
      ups_protocol = valtable_obj._upsProtocol;
    }

    if (valtable_obj.INPHASES != null)
      inphases = valtable_obj.INPHASES;
    else
      inphases = valtable_obj.PHASES;

    if (valtable_obj.OUTPHASES != null)
      outphases = valtable_obj.OUTPHASES;
    else
      outphases = valtable_obj.PHASES;

    if (valtable_obj.EX_BYP_NUMLINES != null && valtable_obj.EX_BYP_NUMLINES != -9999)
      bypphases = valtable_obj.EX_BYP_NUMLINES;
    else
      bypphases = inphases;

    bg_height = 700;
    bg_width = 800;

    if (valtable_obj.OUTPOWER != null)
    {
      if (valtable_obj.OUTPOWER[0] > 100)
        overload = 1;
      else
        overload = 0;
    }
  }

  function create_panes() {
    utils_api.setTitle(valtable_obj._platform);
    create_bg(); //background
    create_input_pane(); //Input table
    create_bypass_pane(); //Input table
    create_output_pane(); //Outnput   panel
    create_battery_pane(); //Battery panel
    create_ext_pane();  // ext. info
    create_symbols(); //create symbols
    create_status_pane(); //UPS status
    create_alarm_pane();
    create_model_panes();
    create_outpower_loadbars(); //outpower loadbars
    create_footer_pane();
    create_sys_mod_status_panel();
    create_sys_mod_alarm_panel();
    panes_api.create_autonomy_time_infos_panel('autonomy_time_label', valtable_obj);
  }

  function create_bg() {
    var fragment = document.createDocumentFragment(),
      img_div, bg_img, bg_img_src = './image/legrand/background.png';

    css_obj = { position: 'absolute', top: y_pos, left: x_pos, width: bg_width, height: bg_height, border: 'solid 1px #808080' };
    img_div = $('<div/>', { 'class': 'img-rounded' }).css(css_obj);
    bg_img = $('<img />').attr({ 'id': 'bg_img_id', 'class': 'img-rounded', width: 800, 'height': 590, src: bg_img_src }).appendTo(img_div);
    fragment.appendChild(img_div[0]);
    document.body.appendChild(fragment);
  }

  function create_model_panes() {
    var model_div, a;
    css_obj = { position: 'absolute', top: y_pos, left: x_pos, align: 'center', width: bg_width, 'z-index': 40 }; // 'background-color': '#3a87ad'
    attr_obj = { id: 'model_div_id', 'class': 'list-group' };
    model_div = panes_api.create_div(attr_obj, css_obj);
    a = $('<a/>', { 'class': 'list-group-item active', id: 'model_name_id' }).appendTo(model_div);
    set_model_name();
  }

  function create_input_pane() //Input table
  {
    var fragment = document.createDocumentFragment(),
        tr, table, tbody, thead, in_div;

    css_obj = { position: 'absolute', top: y_pos + 20, left: x_pos + 120 };

    in_div = $('<div/>', { id: 'input_div_id' }).css(css_obj);
    table = $('<table />', { 'class': 'table table-condensed table-hover', 'id': 'inptable' }).appendTo(in_div); // table-hover
    tbody = $('<tbody />').appendTo(table);
    thead = $('<thead />').appendTo(table);
    tr = $('<tr />').append($('<th />', { colspan: 4 }).append(i18n(lang_Input))).appendTo(thead);
    table.css('margin-top', '40px');

    tr = $('<tr />', { 'id': 'inptrl1' }).append($('<td />').append("L1:")).appendTo(tbody);
    $('<td />', { id: 'input_volt1_id', 'align': 'right'}).appendTo(tr);
    $('<td />', { id: 'input_watt1_id', 'align': 'right' }).appendTo(tr);
    $('<td />', { id: 'input_freq1_id', 'align': 'right' }).appendTo(tr);

    tr = $('<tr />', { 'id': 'inptrl2' }).append($('<td />').append("L2:")).appendTo(tbody);
    $('<td />', { id: 'input_volt2_id', 'align': 'right' }).appendTo(tr);
    $('<td />', { id: 'input_watt2_id', 'align': 'right' }).appendTo(tr);

    tr = $('<tr />', { 'id': 'inptrl3' }).append($('<td />').append("L3:")).appendTo(tbody);
    $('<td />', { id: 'input_volt3_id', 'align': 'right' }).appendTo(tr);
    $('<td />', { id: 'input_watt3_id', 'align': 'right' }).appendTo(tr);

    fragment.appendChild(in_div[0]);
    document.body.appendChild(fragment);
    set_input_values();
  }

  function set_input_values()
  {
    var input_val = get_ph_1_3_vals(valtable_obj.INVOLT, valtable_obj.EX_INP_WATT, valtable_obj.INFREQ, inphases);

   $('#input_volt1_id').text(input_val.val_objs[0].values[0]);
   $('#input_watt1_id').text(input_val.val_objs[1].values[0]);
   $('#input_freq1_id').text(input_val.val_objs[2].values[0]);

   $('#input_volt2_id').text(input_val.val_objs[0].values[1]);
   $('#input_watt2_id').text(input_val.val_objs[1].values[1]);

   $('#input_volt3_id').text(input_val.val_objs[0].values[2]);
   $('#input_watt3_id').text(input_val.val_objs[1].values[2]);
  }

  function create_bypass_pane() //Input table
  {
    var fragment = document.createDocumentFragment(),
        tr, table, tbody, thead, in_div;

    css_obj = { position: 'absolute', top: y_pos + 20, left: x_pos + 500 };

    byp_div = $('<div/>', { id: 'bypass_div_id' }).css(css_obj);
    table = $('<table />', { 'class': 'table table-condensed table-hover', 'id': 'byptable' }).appendTo(byp_div); // table-hover
    tbody = $('<tbody />').appendTo(table);
    thead = $('<thead />').appendTo(table);
    tr = $('<tr />').append($('<th />', { colspan: 4 }).append(i18n(lang_Bypass))).appendTo(thead);
    table.css('margin-top', '40px');

    tr = $('<tr />', { 'id': 'byptrl1' }).append($('<td />').append("L1:")).appendTo(tbody);
    $('<td />', { id: 'byp_volt1_id', 'align': 'right' }).appendTo(tr);
    $('<td />', { id: 'byp_watt1_id', 'align': 'right' }).appendTo(tr);
    $('<td />', { id: 'byp_freq1_id', 'align': 'right' }).appendTo(tr);

    tr = $('<tr />', { 'id': 'inptrl2' }).append($('<td />').append("L2:")).appendTo(tbody);
    $('<td />', { id: 'byp_volt2_id', 'align': 'right' }).appendTo(tr);
    $('<td />', { id: 'byp_watt2_id', 'align': 'right' }).appendTo(tr);

    tr = $('<tr />', { 'id': 'inptrl3' }).append($('<td />').append("L3:")).appendTo(tbody);
    $('<td />', { id: 'byp_volt3_id', 'align': 'right' }).appendTo(tr);
    $('<td />', { id: 'byp_watt3_id', 'align': 'right' }).appendTo(tr);

    fragment.appendChild(byp_div[0]);
    document.body.appendChild(fragment);
    set_bypass_values();
  }

  function set_bypass_values()
  {
    var val = get_ph_1_3_vals(valtable_obj.EX_BYP_VOLT, valtable_obj.EX_BYP_WATT, valtable_obj.EX_BYP_FREQ, bypphases);

   $('#byp_volt1_id').text(val.val_objs[0].values[0]);
   $('#byp_watt1_id').text(val.val_objs[1].values[0]);
   $('#byp_freq1_id').text(val.val_objs[2].values[0]);

   $('#byp_volt2_id').text(val.val_objs[0].values[1]);
   $('#byp_watt2_id').text(val.val_objs[1].values[1]);

   $('#byp_volt3_id').text(val.val_objs[0].values[2]);
   $('#byp_watt3_id').text(val.val_objs[1].values[2]);
  }

  function create_output_pane() //Output
  {
    var output_div = null,
      tr, table, tbody, thead,
      fragment = document.createDocumentFragment();

    css_obj = { position: 'absolute', top: y_pos + 450, left: x_pos + 90 };
    output_div = $('<div/>', { id: 'output_div_id' }).css(css_obj);
    table = $('<table />', { 'class': 'table table-condensed table-hover', 'id': 'outtable' }).appendTo(output_div);
    tbody = $('<tbody />').appendTo(table);
    thead = $('<thead />').appendTo(table);
    tr = $('<tr />').append($('<th />', { colspan: 4 }).append(i18n(lang_Output))).appendTo(thead);
    table.css('margin-top', '40px');

    tr = $('<tr />', { 'id': 'outtrl1' }).append($('<td />').append("L1:")).appendTo(tbody);
    $('<td />', { id: 'out_volt1_id', 'align': 'right' }).appendTo(tr);
    $('<td />', { id: 'out_watt1_id', 'align': 'right' }).appendTo(tr);
    $('<td />', { id: 'out_freq1_id', 'align': 'right' }).appendTo(tr);

    tr = $('<tr />', { 'id': 'outtrl2' }).append($('<td />').append("L2:")).appendTo(tbody);
    $('<td />', { id: 'out_volt2_id', 'align': 'right' }).appendTo(tr);
    $('<td />', { id: 'out_watt2_id', 'align': 'right' }).appendTo(tr);

    tr = $('<tr />', { 'id': 'outtrl3' }).append($('<td />').append("L3:")).appendTo(tbody);
    $('<td />', { id: 'out_volt3_id', 'align': 'right' }).appendTo(tr);
    $('<td />', { id: 'out_watt3_id', 'align': 'right' }).appendTo(tr);

    fragment.appendChild(output_div[0]);
    document.body.appendChild(fragment);

    css_obj = { /*'background-color': 'red',*/ 'text-align': 'center', position: 'absolute', top: y_pos + 430, left: x_pos + 360, 'z-index': 5, color: 'white','font-size': '20px', width:80 };
    attr_obj = { id: 'loadstr_div_id' };
    load_div = panes_api.create_div(attr_obj, css_obj).append("Load");
    css_obj = { 'text-align': 'center', position: 'absolute', top: y_pos + 470, left: x_pos + 350, 'z-index': 5, color: 'white','font-size': '36px', width:100 };
    attr_obj = { id: 'load_div_id' };
    load_div = panes_api.create_div(attr_obj, css_obj);

    set_output_values();
  }

  function set_output_values() //Output values
  {
    var val = get_ph_1_3_vals(valtable_obj.OUTPUT_VOLT, valtable_obj.EX_OUT_WATT, valtable_obj.OUTFREQ, outphases),
        maxload = 0, i;

   $('#out_volt1_id').text(val.val_objs[0].values[0]);
   $('#out_watt1_id').text(val.val_objs[1].values[0]);
   $('#out_freq1_id').text(val.val_objs[2].values[0]);

   $('#out_volt2_id').text(val.val_objs[0].values[1]);
   $('#out_watt2_id').text(val.val_objs[1].values[1]);

   $('#out_volt3_id').text(val.val_objs[0].values[2]);
   $('#out_watt3_id').text(val.val_objs[1].values[2]);

   for (i = 0; i < outphases; i++)
   {
      if (valtable_obj.OUTPOWER[i] > maxload)
         maxload = valtable_obj.OUTPOWER[i];
   }
    $('#load_div_id').html(get_value({ values: [maxload], index: 0, unit: ' %', precision: 0 }));
  }

  function create_battery_pane() //Battery panel
  {
    var val, tr, table, tbody, thead, batt_div, cap_div,
      fragment = document.createDocumentFragment();

    css_obj = { position: 'absolute', top: y_pos + 190, left: x_pos + 450 }; //, width:130, height:100
    attr_obj = { id: 'battery_div_id' };
    batt_div = $('<div/>').attr(attr_obj).css(css_obj);
    table = table = $('<table />', { 'class': 'table table-condensed table-hover' }).appendTo(batt_div);
    tbody = $('<tbody />').appendTo(table);
    thead = $('<thead />').appendTo(table);
    tr = $('<tr />').append($('<th />', { colspan: 2 }).append(i18n(lang_Battery))).appendTo(thead);

    tr = $('<tr />').append($('<td />', { align: 'left', id: 'autonomy_time_label' }).append(i18n(lang_Time) + ":")).appendTo(tbody);
    $('<td />', { id: 'batt_autotime_id', align: 'left' }).appendTo(tr);

    tr = $('<tr />').append($('<td />', { align: 'left' }).append(i18n('On Battery') + ":")).appendTo(tbody);
    $('<td />', { id: 'batt_seconds_id', align: 'left' }).appendTo(tr);

    tr = $('<tr />').append($('<td />', { align: 'left' }).append('Pos.' + i18n(lang_Voltage) + ":")).appendTo(tbody);
    $('<td />', { id: 'batt_voltpos_id', align: 'left' }).appendTo(tr);

    tr = $('<tr />').append($('<td />', { align: 'left' }).append(i18n('Neg.' + lang_Voltage) + ":")).appendTo(tbody);
    $('<td />', { id: 'batt_voltneg_id', align: 'left' }).appendTo(tr);

    fragment.appendChild(batt_div[0]);
    document.body.appendChild(fragment);

    css_obj = { 'text-align': 'center', position: 'absolute', top: y_pos + 245, left: x_pos + 370, 'z-index': 5, color: 'white','font-size': '26px', width:68 };
    attr_obj = { id: 'cap_div_id' };
    cap_div = panes_api.create_div(attr_obj, css_obj);

    set_battery_values();
  }

  function set_battery_values() //set Battery value
  {
    $('#batt_autotime_id').text(get_value({ values: [valtable_obj.AUTONOMTIME], index: 0, unit: ' Min', precision: 0 }));
    $('#batt_seconds_id').text(get_value({ values: [valtable_obj.EX_BATT_SECONBATT], index: 0, unit: ' Sec', precision: 0 }));
    $('#batt_voltpos_id').text(get_value({ values: [valtable_obj.EX_BATT_VOLTPOS], index: 0, unit: ' V', precision: 2 }));
    $('#batt_voltneg_id').text(get_value({ values: [valtable_obj.EX_BATT_VOLTNEG], index: 0, unit: ' V', precision: 2 }));
    $('#cap_div_id').html(get_value({ values: [valtable_obj.BATTCAP], index: 0, unit: ' <br><br>%', precision: 0 }));
  }

  function create_ext_pane() // ext. info
  {
    var val, tr, table, tbody, thead, div,
      fragment = document.createDocumentFragment();

    css_obj = { position: 'absolute', top: y_pos + 590, left: x_pos + 300 };
    attr_obj = { id: 'battery_div_id' };
    div = $('<div/>').attr(attr_obj).css(css_obj);
    table = table = $('<table />', { 'class': 'table table-condensed table-hover' }).appendTo(div);
    tbody = $('<tbody />').appendTo(table);
    thead = $('<thead />').appendTo(table);
    tr = $('<tr />').append($('<th />', { colspan: 2 }).append(i18n('Ext. Information'))).appendTo(thead);

    tr = $('<tr />').append($('<td />', { align: 'left' }).append(i18n('Nominal Apparent Power') + ":")).appendTo(tbody);
    $('<td />', { id: 'nom_kva_id', align: 'left' }).appendTo(tr);

    tr = $('<tr />').append($('<td />', { align: 'left' }).append(i18n('Nominal Active Power') + ":")).appendTo(tbody);
    $('<td />', { id: 'nom_kw_id', align: 'left' }).appendTo(tr);

    tr = $('<tr />').append($('<td />', { align: 'left' }).append(i18n('Device Temperature') + ":")).appendTo(tbody);
    $('<td />', { id: 'dev_temp_id', align: 'left' }).appendTo(tr);

    fragment.appendChild(div[0]);
    document.body.appendChild(fragment);
    set_ext_values();
  }

  function set_ext_values()
  {
    $('#nom_kva_id').text(get_value({ values: [valtable_obj.EX_NOM_VA / 1000], index: 0, unit: ' kVA', precision: 0 }));
    $('#nom_kw_id').text(get_value({ values: [valtable_obj.EX_NOM_WATT / 1000], index: 0, unit: ' kW', precision: 0 }));
    $('#dev_temp_id').html(get_value({ values: [valtable_obj.TEMPDEG], index: 0, unit: ' &deg;C', precision: 0 }));
  }

  function get_value(val_obj, na_string) {
    var val = null;
    var a = null;

    if (!val_obj.values || (val_obj.values.length <= val_obj.index))
      return na_string == null ? 'N/A' : na_string;

    val = val_obj.values[val_obj.index];
    if (val == null || val == -9999)
      return na_string == null ? 'N/A' : na_string;
    if (ups_protocol == prot_snmp && val == -1)
      return na_string == null ? 'N/A' : na_string;

    val = val.toFixed(val_obj.precision);
    val += val_obj.unit;

    return val;
  }

  function get_ph_1_3_vals(volt, watt, freq, phases) {
    var val_obj, val;
    l_1_3_vals = {
      availables: [0, 0, 0], //L1-L3
      val_objs: [
        { unit: ' V', precision: 0, values: ['', '', ''] },
        { unit: ' W', precision: 0, values: ['', '', ''] },
        { unit: ' Hz', precision: 2, values: ['', '', ''] }
      ]
    };
    volt = volt || [];
    watt = watt || [],
    freq = freq || [];
    $.each([volt, watt, freq], function (index, values) {
      val_obj = l_1_3_vals.val_objs[index];
      $.each(values, function (i, val) {
        if (i < phases) {
          val = values[i];
          if ((ups_protocol != prot_snmp && val != -9999) || (ups_protocol == prot_snmp && val != -1)) {
            if (values === freq && (i == 1 || i == 2)) {} else
              l_1_3_vals.availables[i] = 1;

            if (Math.round(val) == val)
              val = val.toString();
            else
              val = val.toFixed(val_obj.precision);
            val += val_obj.unit;
            val_obj.values[i] = val;
          }
        }
      });
    });
    return l_1_3_vals;
  }


  function create_symbols()
  {
    panes_api.create_img_panel('1_img_id', './image/legrand/1_gray.png', x_pos + 44, y_pos + 80, 72, 72, 0);   // mains
    panes_api.create_img_panel('2_img_id', './image/legrand/2_gray.png', x_pos + 44, y_pos + 190, 72, 72, 0);  // rectifier
    panes_api.create_img_panel('3_img_id', './image/legrand/3_gray.png', x_pos + 44, y_pos + 348, 72, 72, 0);  // inverter
    panes_api.create_img_panel('4_img_id', './image/legrand/4_gray__left.png', x_pos + 209, y_pos + 270, 72, 72, 0); // booster/charger
    panes_api.create_img_panel('5_img_id', './image/legrand/5_gray.png', x_pos + 366, y_pos + 195, 73, 142, 0);   // battery
    panes_api.create_img_panel('6_img_id', './image/legrand/6_gray.png', x_pos + 322, y_pos + 380, 160, 160, 0);   // load
    panes_api.create_img_panel('7_img_id', './image/legrand/7_gray.png', x_pos + 690, y_pos + 80, 72, 72, 0);   // bypass inp
    panes_api.create_img_panel('8_img_id', './image/legrand/8_gray.png', x_pos + 690, y_pos + 270, 72, 72, 0);   // bypass out

    set_symbols();
  }

  function set_symbols()
  {
      // Mains input icon
      if (valtable_obj.UPS_RUNNING == 1 && (valtable_obj.STATUS & 0x8) && !(valtable_obj.STATUS & 0x40))
         $('#1_img_id').attr('src', './image/legrand/1_red.png');
      else if (valtable_obj.UPS_RUNNING == 1 && !(valtable_obj.STATUS & 0x8))
         $('#1_img_id').attr('src', './image/legrand/1_green.png');
      else if (valtable_obj.UPS_RUNNING == 0 || (valtable_obj.STATUS & 0x8))
         $('#1_img_id').attr('src', './image/legrand/1_gray.png');

      // rectifier icon
      if (valtable_obj.UPS_RUNNING == 1 && !(valtable_obj.STATUS & 0x40) && ((valtable_obj.RAWSTATEA & 0x8) || (valtable_obj.RAWSTATEA & 0x800)))
         $('#2_img_id').attr('src', './image/legrand/2_red.png');
      else if (valtable_obj.UPS_RUNNING == 1 && (valtable_obj.RAWSTATEB & 0x1))
         $('#2_img_id').attr('src', './image/legrand/2_green.png');
      else if (valtable_obj.UPS_RUNNING == 0 || (valtable_obj.STATUS & 0x8))
         $('#2_img_id').attr('src', './image/legrand/2_gray.png');

      // inverter
      if (valtable_obj.UPS_RUNNING == 1 && !(valtable_obj.STATUS & 0x40) && (valtable_obj.RAWSTATEA & 0x4))
         $('#3_img_id').attr('src', './image/legrand/3_red.png');
      else if (valtable_obj.UPS_RUNNING == 1 && !(valtable_obj.RAWSTATEA & 0x4) && !(valtable_obj.STATUS & 0x1) && !(valtable_obj.RAWSTATEB & 0x8) && !(valtable_obj.RAWSTATEB & 0x10) && (valtable_obj.OPERATIONMODE != 2))
         $('#3_img_id').attr('src', './image/legrand/3_green.png');
      else if (valtable_obj.UPS_RUNNING == 0 || (valtable_obj.STATUS & 0x1) || (valtable_obj.RAWSTATEB & 0x8) || (valtable_obj.RAWSTATEB & 0x10) || (valtable_obj.OPERATIONMODE == 2))
         $('#3_img_id').attr('src', './image/legrand/3_gray.png');

      // booster
      if (valtable_obj.UPS_RUNNING == 1 && !(valtable_obj.STATUS & 0x40) && (valtable_obj.STATUS & 0x8) && (valtable_obj.RAWSTATEA & 0x10))
         $('#4_img_id').attr('src', './image/legrand/4_red__left.png');
      else if (valtable_obj.UPS_RUNNING == 1 && (valtable_obj.STATUS & 0x8) && !(valtable_obj.RAWSTATEA & 0x10))
         $('#4_img_id').attr('src', './image/legrand/4_green__left.png');

      // charger
      else if (!(valtable_obj.STATUS & 0x40) && !(valtable_obj.STATUS & 0x8) && (valtable_obj.RAWSTATEA & 0x20))
         $('#4_img_id').attr('src', './image/legrand/4_red__right.png');
      else if (!(valtable_obj.STATUS & 0x8) && (valtable_obj.UPS_BATTERYCHARGE == 1) && !(valtable_obj.RAWSTATEA & 0x20))
         $('#4_img_id').attr('src', './image/legrand/4_green__right.png');
      else if (!(valtable_obj.STATUS & 0x8) && (valtable_obj.UPS_BATTERYCHARGE == 0) && !(valtable_obj.RAWSTATEA & 0x20))
         $('#4_img_id').attr('src', './image/legrand/4_gray__right.png');

      // battery
      if (valtable_obj.UPS_RUNNING == 1 && !(valtable_obj.STATUS & 0x40) &&
         ((valtable_obj.RAWSTATEA & 0x400) || (valtable_obj.RAWSTATEA & 0x200) ||
         ((valtable_obj.STATUS & 0x8) && (valtable_obj.BATTCAP <= 20 || (valtable_obj.STATUS & 0x10)))))
            $('#5_img_id').attr('src', './image/legrand/5_red.png');
      else if (valtable_obj.UPS_RUNNING == 1 &&
         (((valtable_obj.STATUS & 0x8) && (valtable_obj.BATTCAP <= 40)) ||
         (valtable_obj.STATUS & 0x40) || (valtable_obj.RAWSTATEA & 0x4000)))
            $('#5_img_id').attr('src', './image/legrand/5_yellow.png');
      else if (valtable_obj.UPS_RUNNING == 1 && !(valtable_obj.STATUS & 0x8) && !(valtable_obj.STATUS & 0x40) && !(valtable_obj.RAWSTATEA & 0x4000))
         $('#5_img_id').attr('src', './image/legrand/5_green.png');
      else if (valtable_obj.UPS_RUNNING == 0)
         $('#5_img_id').attr('src', './image/legrand/5_gray.png');

      // load
      if (valtable_obj.UPS_RUNNING == 1 && !(valtable_obj.STATUS & 0x40) && (valtable_obj.LOAD90GT == 1 || (valtable_obj.RAWSTATEA & 0x1000)))
         $('#6_img_id').attr('src', './image/legrand/6_red.png');
      else if (valtable_obj.UPS_RUNNING == 1 && !(valtable_obj.STATUS & 0x40) &&
         !(valtable_obj.RAWSTATEB & 0x40) && !(valtable_obj.RAWSTATEB & 0x80) && !(valtable_obj.RAWSTATEB & 0x100) &&
         valtable_obj.LOAD80GT == 1 && valtable_obj.LOAD90GT == 0)
            $('#6_img_id').attr('src', './image/legrand/6_yellow.png');
      else if (valtable_obj.UPS_RUNNING == 1 &&
         !(valtable_obj.RAWSTATEB & 0x40) && !(valtable_obj.RAWSTATEB & 0x80) && !(valtable_obj.RAWSTATEB & 0x100) &&
         valtable_obj.LOAD80GT == 0)
            $('#6_img_id').attr('src', './image/legrand/6_green.png');
      else if (valtable_obj.UPS_RUNNING == 0 || (valtable_obj.RAWSTATEB & 0x40) || (valtable_obj.RAWSTATEB & 0x80) || (valtable_obj.RAWSTATEB & 0x100))
            $('#6_img_id').attr('src', './image/legrand/6_gray.png');

      // bypass inp
      if (valtable_obj.UPS_RUNNING == 1 && (valtable_obj.EX_EXTSTATUS & 0x200))
         $('#7_img_id').attr('src', './image/legrand/7_yellow.png');
      else if (valtable_obj.UPS_RUNNING == 1 && !(valtable_obj.EX_EXTSTATUS & 0x200))
         $('#7_img_id').attr('src', './image/legrand/7_green.png');
      else if (valtable_obj.UPS_RUNNING == 0)
         $('#7_img_id').attr('src', './image/legrand/7_gray.png');

      // bypass out
      if (valtable_obj.UPS_RUNNING == 1 && ((valtable_obj.STATUS & 0x1) || (valtable_obj.RAWSTATEB & 0x8) || (valtable_obj.RAWSTATEB & 0x10) || valtable_obj.OPERATIONMODE == 2))
         $('#8_img_id').attr('src', './image/legrand/8_green.png');
      else
         $('#8_img_id').attr('src', './image/legrand/8_gray.png');
  }

  function create_status_pane() //UPS status
  {
    var status_div = $('<div/>'),
      status_span;
    css_obj = { position: 'absolute', top: y_pos + 630, left: x_pos + 50, width: 100, height: 50, align: 'center', overflow: 'hidden' };

    $(status_div).css(css_obj);
    status_span = $('<span />', { id: 'ups_status_id', 'class': get_status_class(), title: i18n('Click to see Status Text') }).append(i18n('Status'));
    status_span.css({'cursor':'pointer', width: 100});
    status_span.click(function (event) //visibility: hidden
      {
        var sys_mod_status_txt_div = $('#sys_mod_status_txt_div'),
          display = sys_mod_status_txt_div.css('display') == 'none' ? 'block' : 'none';
        sys_mod_status_txt_div.css({ display: display });
      });

    $('<h4 />').append(status_span).appendTo(status_div);
    status_div.appendTo(document.body);

  }

  function set_ups_status() //set UPS status
  {
    $('#ups_status_id').attr('class', get_status_class());
  }

  function get_status_class()
  {
    if (valtable_obj.UPS_RUNNING == 1 && (valtable_obj.RAWSTATEB & 0x1))
       return 'label label-success';
    return 'label label-danger';
  }

  //////////////////////////////////////////////////////////////

  function create_alarm_pane()
  {
    var status_div = $('<div/>'),
      status_span;
    css_obj = { position: 'absolute', top: y_pos + 630, left: x_pos + 650, width: 100, height: 50, align: 'center', overflow: 'hidden' };

    $(status_div).css(css_obj);
    status_span = $('<span />', { id: 'ups_alarm_id', 'class': get_alarm_class(), title: i18n('Click to see Alarms') }).append(i18n('Alarms'));
    status_span.css({'cursor':'pointer', width: 100});
    status_span.click(function (event) //visibility: hidden
      {
        var sys_mod_alarm_txt_div = $('#sys_mod_alarm_txt_div'),
          display = sys_mod_alarm_txt_div.css('display') == 'none' ? 'block' : 'none';
        sys_mod_alarm_txt_div.css({ display: display });
      });

    $('<h4 />').append(status_span).appendTo(status_div);
    status_div.appendTo(document.body);
  }

  function set_ups_alarm()
  {
    $('#ups_alarm_id').attr('class', get_alarm_class());
  }

  function get_alarm_class() {
   if (valtable_obj.STATUS & 0x6530)
      return 'label label-danger';
   if (valtable_obj.RAWSTATEA & 0x2C60)
      return 'label label-danger';
   return 'label label-success';
  }

  ////////////////////////////////////////////////////////////

  function create_outpower_loadbars() //outpower loadbars
  {
    if (valtable_obj.OUTPOWER == null)
      return true;

    panes_api.create_loadbar_pane({ value: valtable_obj.OUTPOWER[0], id: 'loadbar1_id', left: x_pos + 524, top: y_pos + 510, title: 'L1', scaler: 1 });
    panes_api.create_loadbar_pane({ value: valtable_obj.OUTPOWER[1], id: 'loadbar2_id', left: x_pos + 524, top: y_pos + 540, title: 'L2', scaler: 0 });
    panes_api.create_loadbar_pane({ value: valtable_obj.OUTPOWER[2], id: 'loadbar3_id', left: x_pos + 524, top: y_pos + 570, title: 'L3', scaler: 0 });
    set_outpower_loadbars();
  }

  function set_outpower_loadbars() //outpower loadbars
  {
    if (valtable_obj.OUTPOWER == null)
      return true;
    if (valtable_obj.OUTPOWER[0] != -9999) {
      panes_api.set_loabar_value(valtable_obj.OUTPOWER[0], 'loadbar1_id');
    }
    if (valtable_obj.OUTPOWER[1] != -9999) {
      panes_api.set_loabar_value(valtable_obj.OUTPOWER[1], 'loadbar2_id');
    }
    if (valtable_obj.OUTPOWER[2] != -9999) {
      panes_api.set_loabar_value(valtable_obj.OUTPOWER[2], 'loadbar3_id');
    }
  }

  function create_sys_mod_status_panel()
  {
    var fragment = document.createDocumentFragment(),
      tr, tr, table, tbody, a,
      title = i18n("Status") + ": " + valtable_obj.MODEL,
      status_txt_array = get_sys_mod_status_txt(),
      sys_mod_status_txt_div;

    css_obj = {
      position: 'absolute',
      top: y_pos + 72,
      left: x_pos + 36,
      align: 'center',
      'background-color': '#F1F1F1',
      display: 'none',
      border: 'solid 1px black'
    };

    sys_mod_status_txt_div = $('<div/>', { id: 'sys_mod_status_txt_div', 'class': 'img-rounded' }).css(css_obj);

    a = $('<a/>', { 'class': 'list-group-item active', id: 'status_model_name_id' }).append(title).appendTo(sys_mod_status_txt_div);
    a.css('height', 30);
    table = $('<table />', { 'class': 'table table-condensed', id: 'sys_mod_status_txt_table' }).appendTo(sys_mod_status_txt_div); // table-hover
    table.css({ 'background-color': '#F1F1F1' })
    tbody = $('<tbody />', { id: 'sys_mod_status_txt_table_body' }).appendTo(table);
    $.each(status_txt_array, function (index, status_txt) {
      tr = $('<tr />').appendTo(tbody);
      td = $('<td />', { 'align': 'left' }).append(status_txt).appendTo(tr);
    });

    fragment.appendChild(sys_mod_status_txt_div[0]);
    document.body.appendChild(fragment);
    new utils_api.draggable('sys_mod_status_txt_div');
  }

  function set_sys_mod_status() {
    var tr, td,
      title = i18n("Status") + ": " + valtable_obj.MODEL,
      tbody = $('#sys_mod_status_txt_table_body'),
      status_txt_array = get_sys_mod_status_txt();

    $('#status_model_name_id').text(title);

    tbody.empty();
    $.each(status_txt_array, function (index, status_txt) {
      tr = $('<tr />').appendTo(tbody);
      td = $('<td />', { 'align': 'left' }).append(status_txt).appendTo(tr);
    });
  }


  function get_sys_mod_status_txt()
  {
    var status_txt_array = [];
    if (valtable_obj.UPS_IN_STANDBY == 1)
      status_txt_array.push(i18n('Standby'));
    if (valtable_obj.UPS_ON == 1)
      status_txt_array.push(i18n('Switching ON'));
    if (valtable_obj.UPS_RUNNING == 1)
    {
      status_txt_array.push(i18n('Running'));
      if (valtable_obj.RAWSTATEB & 0x1)
         status_txt_array.push('- ' + i18n('Mains'));
      else if (valtable_obj.STATUS & 0x8)
         status_txt_array.push('- ' + i18n('Battery'));
      if (valtable_obj.STATUS & 0x1)
         status_txt_array.push('- ' + i18n('Bypass'));
      if (valtable_obj.RAWSTATEB & 0x8)
         status_txt_array.push('- ' + i18n('Forced Bypass'));
      if (valtable_obj.RAWSTATEB & 0x10)
         status_txt_array.push('- ' + i18n('Manual Bypass'));
      if (valtable_obj.STATUS & 0x40)
         status_txt_array.push('- ' + i18n('Test Running'));
      if (valtable_obj.UPS_BATTERYCHARGE == 1)
         status_txt_array.push('- ' + i18n('Battery Charger Running'));
      if (valtable_obj.OPERATIONMODE == 2)
         status_txt_array.push('- ' + i18n('ECO Mode'));
    }
    if (valtable_obj.UPS_OFF == 1)
      status_txt_array.push(i18n('Switching OFF'));

    return status_txt_array;
  }

///////////////////////////////////////////////////////////////////////

  function create_sys_mod_alarm_panel()
  {
    var fragment = document.createDocumentFragment(),
      tr, tr, table, tbody, a,
      title = i18n("Alarms") + ": " + valtable_obj.MODEL,
      status_txt_array = get_sys_mod_alarm_txt(),
      sys_mod_alarm_txt_div;

    css_obj = {
      position: 'absolute',
      top: y_pos + 72,
      left: x_pos + 500,
      align: 'center',
      'background-color': '#F1F1F1',
      display: 'none',
      border: 'solid 1px black'
    };

    sys_mod_alarm_txt_div = $('<div/>', { id: 'sys_mod_alarm_txt_div', 'class': 'img-rounded' }).css(css_obj);

    a = $('<a/>', { 'class': 'list-group-item active', id: 'alarm_model_name_id' }).append(title).appendTo(sys_mod_alarm_txt_div);
    a.css('height', 30);
    table = $('<table />', { 'class': 'table table-condensed', id: 'sys_mod_alarm_txt_table' }).appendTo(sys_mod_alarm_txt_div); // table-hover
    table.css({ 'background-color': '#F1F1F1' })
    tbody = $('<tbody />', { id: 'sys_mod_alarm_txt_table_body' }).appendTo(table);
    $.each(status_txt_array, function (index, status_txt) {
      tr = $('<tr />').appendTo(tbody);
      td = $('<td />', { 'align': 'left' }).append(status_txt).appendTo(tr);
    });

    fragment.appendChild(sys_mod_alarm_txt_div[0]);
    document.body.appendChild(fragment);
    new utils_api.draggable('sys_mod_alarm_txt_div');
  }

  function set_sys_mod_alarm() {
    var tr, td,
      title = i18n("Alarms") + ": " + valtable_obj.MODEL,
      tbody = $('#sys_mod_alarm_txt_table_body'),
      status_txt_array = get_sys_mod_alarm_txt();

    $('#alarm_model_name_id').text(title);

    tbody.empty();
    $.each(status_txt_array, function (index, status_txt) {
      tr = $('<tr />').appendTo(tbody);
      td = $('<td />', { 'align': 'left' }).append(status_txt).appendTo(tr);
    });
  }

  function get_sys_mod_alarm_txt() {
    var status_txt_array = [];
    $.each(status_obj, function (index, status_obj) {
      if (status_obj.bit & valtable_obj.STATUS) {
        status_txt_array.push(i18n(status_obj.txt));
      }
    });

    if (valtable_obj.RAWSTATEA != null) {
      $.each(rawstatea_obj, function (index, rawstatea_obj) {
        if (rawstatea_obj.bit & valtable_obj.RAWSTATEA) {
          status_txt_array.push(i18n(rawstatea_obj.txt));
        }
      });
    }

    if (status_txt_array.length == 0)
      status_txt_array.push(i18n('No alarms'));

    return status_txt_array;
  }

  function set_model_name() {
    var model_name = valtable_obj.MODEL;
    $('#model_name_id').html(model_name);
  }
}(jQuery));
