/*
   kstar_panel.js - KSTAR screen.
   Copyright (c) 2013-2017, Generex GmbH. All rights reserved.
   Said/UW.
*/

var kstar_api = (function ($) {
  var oid = get_oid_from_url(),
  cb = null,
  pannes_created = false,
  err_div = null,
  check_bowser = true,
  outpower_loadbars_created = false,
  css_obj, attr_obj,
  valtable_obj, report_obj, unms_obj,
  inphases = null,
  outphases = null,
  bypphases = null,
  status, tempmax, tempdeg, overload = 0,
  modules_exist_maske = null,
  plattform = get_plattform(),
  ups_id = 0,
  ups_protocol = 0,
  prot_snmp = 0x803A,
  bg_height, bg_width,
  x_pos = 10, y_pos = 10;

  var status_obj = [

    // {bit:0x0008, txt:"The UPS is drawing power from the batteries"},
    { bit: 0x0010, txt: "The remaining battery run-time is low" },
    { bit: 0x0400, txt: "One or more batteries have been determined to require replacement" },
    { bit: 0x0020, txt: "A temperature is out of tolerance" },
    { bit: 0x0080, txt: "Input high" },
    { bit: 0x0100, txt: "The output load exceeds the UPS output capacity" },
    // {bit:0x2000, txt:"A general fault in the UPS has been detected"},
    { bit: 0x4000, txt: "A problem has been encountered in the communications between the agent and the UPS" },
    { bit: 0x0200, txt: "Inverter error" },
    { bit: 0x1000, txt: "Inverter warning" },
    // {bit:0x0002, txt:"UPS is in shutdown mode"},
    // {bit:0x0001, txt:"The Bypass is presently engaged on the UPS"},
    // {bit:0x0040, txt:"A test is in progress"},
    { bit: 0x0004, txt: "UPS Output active" }
    // {bit:0x0800, txt:"UPS in ECO mode"}
  ],
  ext_status_obj = [
    { bit: 0x00000008, txt: "The UPS will be unable to sustain the present load when and if the utility power is lost" },
    { bit: 0x00000040, txt: "An output condition (other than OutputOverload) is out of tolerance" },
    { bit: 0x00000200, txt: "The Bypass is out of tolerance" },
    { bit: 0x00000400, txt: "The UPS has shutdown as requested, i.e., the output is off" },
    { bit: 0x00000800, txt: "The entire UPS has shutdown as commanded" },
    { bit: 0x00001000, txt: "An uncorrected problem has been detected within the UPS charger subsystem" },
    { bit: 0x00002000, txt: "The output of the UPS is in the off state" },
    { bit: 0x00004000, txt: "The UPS system is in the off state" },
    { bit: 0x00008000, txt: "The failure of one or more fans in the UPS has been detected" },
    { bit: 0x00010000, txt: "The failure of one or more fuses has been detected" },
    { bit: 0x00040000, txt: "The result of the last diagnostic test indicates a failure" },
    { bit: 0x00100000, txt: "The UPS output is off and the UPS is awaiting the return of input power" },
    { bit: 0x00200000, txt: "An UPS Shutdown After Delay countdown is underway" },
    { bit: 0x00400000, txt: "The UPS will turn off power to the load immediatly" }
  ],
  rawstatec = [
    { bit: 0x80000000, txt: "Rectifier fault" },
    { bit: 0x40000000, txt: "Rectifier over temperature" },
    { bit: 0x20000000, txt: "Inverter over temperature" },
    { bit: 0x10000000, txt: "Rectfier over current" },
    { bit: 0x0C000000, txt: "Auxiliary power fault" },
    { bit: 0x00400000, txt: "Fan fault" },
    { bit: 0x00100000, txt: "DC Bus over voltage" },
    { bit: 0x00080000, txt: "DC Bus below voltage" },
    { bit: 0x00040000, txt: "DC Bus unbalance" },
    { bit: 0x00020000, txt: "Mains site wiring fault" },
    { bit: 0x00010000, txt: "Soft start failed" },
    { bit: 0x00004000, txt: "Battery reverse" },
    { bit: 0x00002000, txt: "No battery" },
    { bit: 0x00001000, txt: "P-Battery Charger fault" },
    { bit: 0x00000800, txt: "N-Battery Charger fault" },
    { bit: 0x00000200, txt: "Battery over voltage" },
    { bit: 0x00000400, txt: "Battery under voltage" },
    { bit: 0x00000100, txt: "Battery low pre-warning" },
    { bit: 0x00000080, txt: "Mains freq abnormal" },
    { bit: 0x00000040, txt: "Mains volt abnormal" }
  ],
  rawstatec2 = [
    { bit: 0x00000001, txt: "Rectifier fault" },
    { bit: 0x00000002, txt: "Inverter fault" },
    { bit: 0x00000004, txt: "Auxiliary power fault" },
    { bit: 0x00000010, txt: "Input thyristor failed" },
    { bit: 0x00000020, txt: "Invert thyristor failed" },
    { bit: 0x00000040, txt: "Bypass thyristor failed" },
    { bit: 0x00000100, txt: "DC bus volt abnormal" },
    { bit: 0x00000200, txt: "Initialization fault" },
    { bit: 0x00000400, txt: "Battery thyristor failed" },
    { bit: 0x00000800, txt: "Charger fault" },
    { bit: 0x00001000, txt: "Parallel fault" },
    { bit: 0x00002000, txt: "Parallel on invalid" },
    { bit: 0x00004000, txt: "DC component over limit" },
    { bit: 0x00008000, txt: "Mains site wiring fault" },
    { bit: 0x00010000, txt: "Input neutral line missing" },
    { bit: 0x00020000, txt: "Bypass site wiring fault" },
    { bit: 0x00040000, txt: "Battery reverse" },
    { bit: 0x00080000, txt: "No Battery" },
    { bit: 0x01000000, txt: "Parallel overload" },
    { bit: 0x02000000, txt: "Battery under voltage" },
    { bit: 0x04000000, txt: "Battery over voltage" },
    { bit: 0x08000000, txt: "Battery low pre-warning" },
    { bit: 0x10000000, txt: "Mains freq abnormal" },
    { bit: 0x20000000, txt: "Mains volt abnormal" },
    { bit: 0x40000000, txt: "Bypass unable to trace" },
    { bit: 0x80000000, txt: "Bypass not available" }
  ],
  rawstated = [
    { bit: 0x80000000, txt: "Inverter fault" },
    { bit: 0x20000000, txt: "Inverter Thyristor short" },
    { bit: 0x10000000, txt: "Inverter Thyristor broken" },
    { bit: 0x08000000, txt: "Bypass Thyristor short" },
    { bit: 0x04000000, txt: "Bypass Thyristor broken" },
    { bit: 0x02000000, txt: "CAN comm fault" },
    { bit: 0x01000000, txt: "Parallel load sharing fault" },
    { bit: 0x00800000, txt: "Bypass site wiring fault" },
    { bit: 0x00200000, txt: "Bypass unable to trace" },
    { bit: 0x00100000, txt: "Bypass not available" },
    { bit: 0x00040000, txt: "Fuse broken" },
    { bit: 0x00020000, txt: "Cable connection error" },
    { bit: 0x00010000, txt: "Parallel relay fault" },
    { bit: 0x00004000, txt: "Initialization fault" },
    { bit: 0x00000800, txt: "Parallel overload" },
    { bit: 0x00000400, txt: "DC component over limit" },
    { bit: 0x00000200, txt: "Bypass over current" }
  ],
  rawstated2 = [
    { bit: 0x00000002, txt: "LBS not synchronized" },
    { bit: 0x00000004, txt: "System not synchronized" },
    { bit: 0x00000008, txt: "Ext fire alarm" },
    { bit: 0x00000010, txt: "Ext smog alarm" }
  ],
  rawstatee = [
    { val: 0x0, txt: "Initializing" },
    { val: 0x1, txt: "Standby mode" },
    { val: 0x2, txt: "No output" },
    { val: 0x3, txt: "Bypass mode" },
    { val: 0x4, txt: "Online mode" },
    { val: 0x5, txt: "Battery mode" },
    { val: 0x6, txt: "Battery self diagnostigs" },
    { val: 0x7, txt: "Inverter is starting up" },
    { val: 0x8, txt: "ECO mode" },
    { val: 0x9, txt: "EPO mode" },
    { val: 0xA, txt: "Maintenance bypass mode" },
    { val: 0xB, txt: "General fault" }
  ];

  var req_params = null;

  /*public functions*/
  return {
    init: init
  };

  function init() {
    if (check_bowser && !utils_api.check_bowser()) {
      $('#loading_id').css('display', 'none');
      panes_api.create_br_check_panel(this, browser_check_ok);
      return true;
    }
    create_err_msg_panel();

    req_params = utils_api.get_req_params(utils_api.device_ups, oid);
    cb = jqmodule.subscribe(this, { update: update, error: err_funct }, req_params);
  }

  function browser_check_ok() {
    check_bowser = false;
    init();
  }

  function has_system_values() {
    if (valtable_obj.MODULEMASK == null) // single UPS ohne module haben IMMER system values
    return true;
    if (valtable_obj.HASSYSTEMVALS == null) // UPS ohne HASSYSTEMVALS variable haben IMMER system values
    return true;
    if (valtable_obj.HASSYSTEMVALS != 0)
    return true;
    return false;
  }

  function first_module() {
    var i, mod = 0;

    if (valtable_obj.MODULEMASK == null)
    return 0;
    for (i = 0; i < 32; i++) {
      if (valtable_obj.MODULEMASK & (1 << i)) {
        mod = i + 1;
        break;
      }
    }
    return mod;
  }

  function create_err_msg_panel() {
    css_obj = { position: 'absolute', top: y_pos, left: x_pos, width: 791, display: 'none', 'z-index': 62 };
    err_div = panes_api.create_err_panel({ id: 'error_div', 'class': 'alert alert-danger' }, css_obj);
  }

  function err_funct(response) {
    jqmodule.refresh(cb, 10 * 1000);
    panes_api.display_err({ response: response, oid: oid, err_div: err_div, err_msg_div_id: 'error_msg_div' });
  }

  function update(data) {
    if (data.ups == null) {
      return;
    }

    set_variable(data);
    jqmodule.refresh(cb, report_obj.refresh * 1000);
    if (err_div)
    err_div.css('display', 'none');

    if (window.menu_api != null)
    menu_api.update(data);

    unms_obj = get_unms_obj(data);
    if (unms_obj != null) {
      info_head_api.update(unms_obj);
    }

    if (!pannes_created) {
      //  wenn keine system values hier das erste modul suchen und ups_id setzen
      if (!has_system_values() && ups_id == 0) {
        ups_id = first_module();
        get_module_values(ups_id);
        return false;
      }
      pannes_created = true;
      create_panes();
      return false;
    }
    set_input_values(); ////Input values
    set_output_values(); //Output values
    set_battery_value(); //Battery value
    set_byp_values(); //Bypass values
    set_iby_ire_ssi_inv_rec_batt(); //SET IBY, IRE, SSI, INVERTER, RECTIFIER and BATTERY pannes
    set_ups_status(); //set UPS status
    set_outpower_loadbars(); //outpower loadbars

    set_model_name();
    create_modules_panes();

    set_sys_mod_status();

    if (plattform !== "ups_view")
    panes_api.set_rom_temp_values(valtable_obj); //set rom temp values
    set_footer();
  }

  function create_footer_pane() {
    if (!oid || !unms_obj)
       return true;
    if (modules_exist_maske == null)
    {
       footer_api.create({ //Attr
           id: 'panel2'
           //'class':'unmsFooter'
         }, { //css
           position: 'absolute',
           top: '540px',
           left: '74px'
         },
         unms_obj
       );
    }
    else
    {
       footer_api.create({ //Attr
           id: 'panel2'
           //'class':'unmsFooter'
         }, { //css
           position: 'absolute',
           top: '640px',
           left: '74px'
         },
         unms_obj
       );
    }
  }

  function set_footer() {
    if (!oid || !unms_obj)
    return true;
    footer_api.set(unms_obj);
  }

  function set_variable(data) {
    valtable_obj = data.ups.valtable;
    report_obj = data.report;

    status = valtable_obj.STATUS;
    tempmax = valtable_obj.TMPMAX;
    tempdeg = valtable_obj.TEMPDEG;
    level = valtable_obj.LEVEL;

    if (plattform === utils_api.plattform_ups_view ||
      plattform === utils_api.plattform_cs121 ||
      plattform === utils_api.plattform_cs141) {
      modules_exist_maske = valtable_obj.MODULEMASK;
      if (modules_exist_maske == 0)
      modules_exist_maske = null;
    } else {
      modules_exist_maske = null;
    }

    if (valtable_obj._upsProtocol == null) {
      ups_protocol = valtable_obj.CONNECT & 0XFFFF;
    } else {
      ups_protocol = valtable_obj._upsProtocol;
    }


    bg_height = modules_exist_maske == null ? 520 : 620;
    bg_width = 818;

    if (valtable_obj.INPHASES != null)
    inphases = valtable_obj.INPHASES;
    else
    inphases = valtable_obj.PHASES;

    if (valtable_obj.OUTPHASES != null)
    outphases = valtable_obj.OUTPHASES;
    else
    outphases = valtable_obj.PHASES;

    if (valtable_obj.EX_BYP_NUMLINES != null && valtable_obj.EX_BYP_NUMLINES != -9999)
    bypphases = valtable_obj.EX_BYP_NUMLINES;
    else
    bypphases = inphases;

    if (valtable_obj.OUTPOWER != null) {
      if (outphases > 2) {
        if ((valtable_obj.OUTPOWER[0] > 100) || (valtable_obj.OUTPOWER[1] > 100) || (valtable_obj.OUTPOWER[2] > 100))
        overload = 1;
        else
        overload = 0;
      } else {
        if (valtable_obj.OUTPOWER[0] > 100)
        overload = 1;
        else
        overload = 0;
      }
    }
  }

  function create_panes() {
    utils_api.setTitle(valtable_obj._platform);
    create_bg(); //background
    create_input_pane(); //Input table
    create_output_pane(); //Outnput   panel
    create_battery_pane(); //Battery panel
    create_iby_ire_ssi_inv_rec_batt(); //create IBY, IRE, SSI, INVERTER, RECTIFIER and BATTERY pannes
    create_byp_pane(); //Bypass panel
    create_status_pane(); //UPS status
    create_model_panes();
    create_modules_panes();
    create_outpower_loadbars(); //outpower loadbars
    if (plattform !== utils_api.plattform_ups_view) {
      create_rom_temp(); //rom temp
    }
    create_footer_pane();
    create_sys_mod_status_panel();
  }

  function create_bg() ///home/generex/prj/cs141/webapp/devices/webif/lib/ups/image/ups/bgn_module.png
  {
    var fragment = document.createDocumentFragment(),
    img_div;
    css_obj = { position: 'absolute', top: y_pos, left: x_pos, width: bg_width, height: bg_height, border: 'solid 1px #808080' };
    img_div = $('<div/>', { 'class': 'img-rounded' }).css(css_obj);
    $('<img />').attr({ 'id': 'bg_img_id', 'class': 'img-rounded', width: 816, 'height': 459, src: './image/ups/bg_ups.png' }).appendTo(img_div);
    fragment.appendChild(img_div[0]);
    document.body.appendChild(fragment);
  }

  function create_model_panes() {
    var model_div, a;
    css_obj = { position: 'absolute', top: y_pos, left: x_pos, align: 'center', width: bg_width, 'z-index': 40 }; // 'background-color': '#3a87ad'
    attr_obj = { id: 'model_div_id', 'class': 'list-group' };
    model_div = panes_api.create_div(attr_obj, css_obj);
    a = $('<a/>', { 'class': 'list-group-item active', id: 'model_name_id' }).appendTo(model_div);
    set_model_name();
  }

  function create_input_pane() //Input table
  {
    var fragment = document.createDocumentFragment(),
    tr, table, tbody, thead, in_div;

    css_obj = { position: 'absolute', top: y_pos + 176, left: x_pos + 8 };

    in_div = $('<div/>', { id: 'input_div_id' }).css(css_obj);
    table = $('<table />', { 'class': 'table table-condensed table-hover', 'id': 'inptable' }).appendTo(in_div); // table-hover
    table.css('display', 'none');
    tbody = $('<tbody />').appendTo(table);
    thead = $('<thead />').appendTo(table);
    tr = $('<tr />').append($('<th />', { colspan: 4 }).append(i18n(lang_Input))).appendTo(thead);

    if (inphases == 1)
    table.css('margin-top', '22px');

    tr = $('<tr />', { 'id': 'inptrl1' }).append($('<td />').append("L1:")).appendTo(tbody);
    $('<td />', { id: 'input_volt1_id' }).appendTo(tr);
    $('<td />', { id: 'input_cur1_id' }).appendTo(tr);
    $('<td />', { id: 'input_freq1_id' }).appendTo(tr);

    tr = $('<tr />', { 'id': 'inptrl2' }).append($('<td />').append("L2:")).appendTo(tbody);
    $('<td />', { id: 'input_volt2_id' }).appendTo(tr);
    $('<td />', { id: 'input_cur2_id' }).appendTo(tr);

    tr = $('<tr />', { 'id': 'inptrl3' }).append($('<td />').append("L3:")).appendTo(tbody);
    $('<td />', { id: 'input_volt3_id' }).appendTo(tr);
    $('<td />', { id: 'input_cur3_id' }).appendTo(tr);

    fragment.appendChild(in_div[0]);
    document.body.appendChild(fragment);
    set_input_values();
  }

  function set_input_values() {
    var input_val = get_ph_1_3_vals(valtable_obj.INVOLT, valtable_obj.INCURR, valtable_obj.INFREQ, inphases);

    if (ups_id == 0) {
      if (input_val.availables[0] == 0 && input_val.availables[1] == 0 && input_val.availables[2] == 0) {
        $('#inptable').css('display', 'none');
        return true;
      }
    }
    $('#inptable').css('display', 'block');

    if (input_val.availables[0] != 0) {
      $('#inptrl1').css('display', 'block');
      $('#input_volt1_id').text(input_val.val_objs[0].values[0]);
      $('#input_cur1_id').text(input_val.val_objs[1].values[0]);
      $('#input_freq1_id').text(input_val.val_objs[2].values[0]);
    } else {
      $('#inptrl1').css('display', 'none');
    }

    if (input_val.availables[1] != 0) {
      $('#inptrl2').css('display', 'block');
      $('#input_volt2_id').text(input_val.val_objs[0].values[1]);
      $('#input_cur2_id').text(input_val.val_objs[1].values[1]);
    } else {
      $('#inptrl2').css('display', 'none');
    }

    if (input_val.availables[2] != 0) {
      $('#inptrl3').css('display', 'block');
      $('#input_volt3_id').text(input_val.val_objs[0].values[2]);
      $('#input_cur3_id').text(input_val.val_objs[1].values[2]);
    } else {
      $('#inptrl3').css('display', 'none');
    }
  }

  function create_output_pane() //Output
  {
    var output_div = null,
    tr, table, tbody, thead,
    fragment = document.createDocumentFragment();

    css_obj = { position: 'absolute', top: y_pos + 176, left: x_pos + 610 };

    output_div = $('<div/>', { id: 'output_div_id' }).css(css_obj);
    table = $('<table />', { 'class': 'table table-condensed table-hover', 'id': 'outtable' }).appendTo(output_div);
    table.css('display', 'none');
    tbody = $('<tbody />').appendTo(table);
    thead = $('<thead />').appendTo(table);
    tr = $('<tr />').append($('<th />', { colspan: 4 }).append(i18n(lang_Output))).appendTo(thead);

    if (outphases == 1)
    table.css('margin-top', '22px');

    tr = $('<tr />', { 'id': 'outtrl1' }).append($('<td />').append("L1:")).appendTo(tbody);
    $('<td />', { 'id': 'out_volt1_id' }).appendTo(tr);
    $('<td />', { id: 'out_cur1_id' }).appendTo(tr);
    $('<td />', { id: 'out_freq1_id' }).appendTo(tr);

    tr = $('<tr />', { 'id': 'outtrl2' }).append($('<td />').append("L2:")).appendTo(tbody);
    $('<td />', { id: 'out_volt2_id' }).appendTo(tr);
    $('<td />', { id: 'out_cur2_id' }).appendTo(tr);

    tr = $('<tr />', { 'id': 'outtrl3' }).append($('<td />').append("L3:")).appendTo(tbody);
    $('<td />', { id: 'out_volt3_id' }).appendTo(tr);
    $('<td />', { id: 'out_cur3_id' }).appendTo(tr);

    fragment.appendChild(output_div[0]);
    document.body.appendChild(fragment);
    set_output_values();
  }

  function set_output_values() //Output values
  {
    var output_val = get_ph_1_3_vals(valtable_obj.OUTPUT_VOLT, valtable_obj.EX_OUT_CURR, valtable_obj.OUTFREQ, outphases);

    if (ups_id == 0) {
      if (output_val.availables[0] == 0 && output_val.availables[1] == 0 && output_val.availables[2] == 0) {
        $('#outtable').css('display', 'none');
        return true;
      }
    }

    $('#outtable').css('display', 'block');
    if (output_val.availables[0] != 0) {
      $('#outtrl1').css('display', 'block');
      $('#out_volt1_id').text(output_val.val_objs[0].values[0]);
      $('#out_cur1_id').text(output_val.val_objs[1].values[0]);
      $('#out_freq1_id').text(output_val.val_objs[2].values[0]);
    } else {
      $('#outtrl1').css('display', 'none');
    }

    if (output_val.availables[1] != 0) {
      $('#outtrl2').css('display', 'block');
      $('#out_volt2_id').text(output_val.val_objs[0].values[1]);
      $('#out_cur2_id').text(output_val.val_objs[1].values[1]);
    } else {
      $('#outtrl2').css('display', 'none');
    }

    if (output_val.availables[2] != 0) {
      $('#outtrl3').css('display', 'block');
      $('#out_volt3_id').text(output_val.val_objs[0].values[2]);
      $('#out_cur3_id').text(output_val.val_objs[1].values[2]);
    } else {
      $('#outtrl3').css('display', 'none');
    }
  }

  function create_byp_pane() //Bypass
  {
    var byp_div, tr, table, tbody, thead,
    fragment = document.createDocumentFragment();

    css_obj = { position: 'absolute', top: y_pos + 50, left: x_pos + 8 };

    byp_div = $('<div/>', { id: 'bypass_div_id' }).css(css_obj);
    table = table = $('<table />', { 'class': 'table table-condensed table-hover', 'id': 'byptable' }).appendTo(byp_div);
    table.css('display', 'none');
    tbody = $('<tbody />').appendTo(table);
    thead = $('<thead />').appendTo(table);
    tr = $('<tr />').append($('<th />', { colspan: 4 }).append(i18n(lang_Bypass))).appendTo(thead);

    if (bypphases == 1)
    table.css('margin-top', '50px');

    tr = $('<tr />', { 'id': 'byptrl1' }).append($('<td />').append("L1:")).appendTo(tbody);
    $('<td />', { 'id': 'byp_volt1_id' }).appendTo(tr);
    $('<td />', { id: 'byp_cur1_id' }).appendTo(tr);
    $('<td />', { id: 'byp_freq1_id' }).appendTo(tr);

    tr = $('<tr />', { 'id': 'byptrl2' }).append($('<td />').append("L2:")).appendTo(tbody);
    $('<td />', { id: 'byp_volt2_id' }).appendTo(tr);
    $('<td />', { id: 'byp_cur2_id' }).appendTo(tr);

    tr = $('<tr />', { 'id': 'byptrl3' }).append($('<td />').append("L3:")).appendTo(tbody);
    $('<td />', { id: 'byp_volt3_id' }).appendTo(tr);
    $('<td />', { id: 'byp_cur3_id' }).appendTo(tr);

    fragment.appendChild(byp_div[0]);
    document.body.appendChild(fragment);
    set_byp_values();
  }

  function set_byp_values() //Bypass values
  {
    var bypass_val = get_ph_1_3_vals(valtable_obj.EX_BYP_VOLT, valtable_obj.EX_BYP_CURR, valtable_obj.EX_BYP_FREQ, bypphases);

    if (ups_id == 0) {
      if (bypass_val.availables[0] == 0 && bypass_val.availables[1] == 0 && bypass_val.availables[2] == 0) {
        $('#byptable').css('display', 'none');
        return true;
      }
    }

    if ((ups_protocol == 0x80F9) && (ups_id != 0)) // $$MBSALIRM Salicru (oem82) UPS "RM" modular
    {
      $('#bypass_div_id').css('display', 'none');
      $('#ssi_img_id').css('display', 'none');
      $('#bg_img_id').attr('src', './image/ups/bgn_module.png');
      return true;
    }

    $('#byptable').css('display', 'block');
    $('#ssi_img_id').css('display', 'block');
    $('#bg_img_id').attr('src', './image/ups/bg_ups.png');
    if (bypass_val.availables[0] != 0) {
      $('#byptrl1').css('display', 'block');
      $('#byp_volt1_id').text(bypass_val.val_objs[0].values[0]);
      $('#byp_cur1_id').text(bypass_val.val_objs[1].values[0]);
      $('#byp_freq1_id').text(bypass_val.val_objs[2].values[0]);
    } else {
      $('#byptrl1').css('display', 'none');
    }

    if (bypass_val.availables[1] != 0) {
      $('#byptrl2').css('display', 'block');
      $('#byp_volt2_id').text(bypass_val.val_objs[0].values[1]);
      $('#byp_cur2_id').text(bypass_val.val_objs[1].values[1]);
    } else {
      $('#byptrl2').css('display', 'none');
    }

    if (bypass_val.availables[2] != 0) {
      $('#byptrl3').css('display', 'block');
      $('#byp_volt3_id').text(bypass_val.val_objs[0].values[2]);
      $('#byp_cur3_id').text(bypass_val.val_objs[1].values[2]);
    } else {
      $('#byptrl3').css('display', 'none');
    }
  }


  //Standardscreen erweitern: Wenn EX_BATT_VOLTPOS und EX_BATT_VOLTNEG vorhanden und ungleich UPS_VAL_NA (-9999) dann beide Werte anstatt BATTVOLT anzeigen.
  function create_battery_pane() //Battery panel
  {
    var val, tr, table, tbody, thead, batt_div,
    fragment = document.createDocumentFragment();

    css_obj = { position: 'absolute', top: y_pos + 280, left: x_pos + 170 }; //, width:130, height:100
    attr_obj = { id: 'battery_div_id' };
    batt_div = $('<div/>').attr(attr_obj).css(css_obj);
    table = table = $('<table />', { 'class': 'table table-condensed table-hover' }).appendTo(batt_div);
    tbody = $('<tbody />').appendTo(table);
    thead = $('<thead />').appendTo(table);
    tr = $('<tr />').append($('<th />', { colspan: 2 }).append(lang_Battery)).appendTo(thead);

    if ((valtable_obj.EX_BATT_VOLTPOS != null && valtable_obj.EX_BATT_VOLTNEG != null) &&
      (valtable_obj.EX_BATT_VOLTPOS != -9999 && valtable_obj.EX_BATT_VOLTNEG != -9999)) {
      tr = $('<tr />').append($('<td />', { align: 'left' }).append('Pos.' + i18n(lang_Voltage) + ":")).appendTo(tbody);
      $('<td />', { id: 'batt_volt_pos_id', align: 'left' }).
      append(get_value({ values: [valtable_obj.EX_BATT_VOLTPOS], index: 0, unit: ' V', precision: 2 })).appendTo(tr);

      tr = $('<tr />').append($('<td />', { align: 'left' }).append('Neg.' + lang_Voltage + ":")).appendTo(tbody);
      $('<td />', { id: 'batt_volt_neg_id', align: 'left' }).
      append(get_value({ values: [valtable_obj.EX_BATT_VOLTNEG], index: 0, unit: ' V', precision: 2 })).appendTo(tr);
    } else {
      tr = $('<tr />').append($('<td />', { align: 'left' }).append(i18n(lang_Voltage) + ":")).appendTo(tbody);
      $('<td />', { id: 'batt_volt_id', align: 'left' }).
      append(get_value({ values: [valtable_obj.BATTVOLT], index: 0, unit: ' V', precision: 2 })).appendTo(tr);
    }

    if (valtable_obj.EX_BATT_CURRPOS != null && valtable_obj.EX_BATT_CURRPOS != -9999) {
      tr = $('<tr />').append($('<td />', { align: 'left' }).append(i18n('Pos. Chrg.') + lang_Current + ":")).appendTo(tbody);
      $('<td />', { id: 'batt_currpos_id', align: 'left' }).
      append(get_value({ values: [valtable_obj.EX_BATT_CURRPOS], index: 0, unit: ' A', precision: 1 })).appendTo(tr);
    }
    if (valtable_obj.EX_BATT_CURRNEG != null && valtable_obj.EX_BATT_CURRNEG != -9999) {
      tr = $('<tr />').append($('<td />', { align: 'left' }).append(i18n('Neg. Chrg.') + lang_Current + ":")).appendTo(tbody);
      $('<td />', { id: 'batt_currneg_id', align: 'left' }).
      append(get_value({ values: [valtable_obj.EX_BATT_CURRNEG], index: 0, unit: ' A', precision: 1 })).appendTo(tr);
    }
    if (valtable_obj.CUBE_DISCHPOS != null && valtable_obj.CUBE_DISCHPOS != -9999) {
      tr = $('<tr />').append($('<td />', { align: 'left' }).append(i18n('Pos. Disch.') + lang_Current + ":")).appendTo(tbody);
      $('<td />', { id: 'batt_currpos_id2', align: 'left' }).
      append(get_value({ values: [valtable_obj.CUBE_DISCHPOS], index: 0, unit: ' A', precision: 1 })).appendTo(tr);
    }
    if (valtable_obj.CUBE_DISCHNEG != null && valtable_obj.CUBE_DISCHNEG != -9999) {
      tr = $('<tr />').append($('<td />', { align: 'left' }).append(i18n('Neg. Disch.') + lang_Current + ":")).appendTo(tbody);
      $('<td />', { id: 'batt_currneg_id2', align: 'left' }).
      append(get_value({ values: [valtable_obj.CUBE_DISCHNEG], index: 0, unit: ' A', precision: 1 })).appendTo(tr);
    }

    tr = $('<tr />').append($('<td />', { align: 'left' }).append(i18n(lang_Time) + ":")).appendTo(tbody);
    $('<td />', { id: 'batt_autotime_id', align: 'left' }).
    append(get_value({ values: [valtable_obj.AUTONOMTIME], index: 0, unit: ' Min', precision: 0 })).appendTo(tr);

    tr = $('<tr />').append($('<td />', { align: 'left' }).append(i18n(lang_Battery_Capacity_short) + ":")).appendTo(tbody);
    $('<td />', { id: 'batt_cap_id', align: 'left' }).
    append(get_value({ values: [valtable_obj.BATTCAP], index: 0, unit: ' %', precision: 0 })).appendTo(tr);

    if (valtable_obj.EX_BATT_SECONBATT != null) {
      tr = $('<tr />').append($('<td />', { align: 'left' }).append(i18n('On Battery') + ":")).appendTo(tbody);
      $('<td />', { id: 'batt_seconds_id', align: 'left' }).
      append(get_value({ values: [valtable_obj.EX_BATT_SECONBATT], index: 0, unit: ' Sec', precision: 0 })).appendTo(tr);
    }
    fragment.appendChild(batt_div[0]);
    document.body.appendChild(fragment);
  }

  function set_battery_value() //set Battery value
  {
    if ((valtable_obj.EX_BATT_VOLTPOS != null && valtable_obj.EX_BATT_VOLTNEG != null) &&
      (valtable_obj.EX_BATT_VOLTPOS != -9999 && valtable_obj.EX_BATT_VOLTNEG != -9999)) {
      $('#batt_volt_pos_id').text(get_value({ values: [valtable_obj.EX_BATT_VOLTPOS], index: 0, unit: ' V', precision: 2 }));
      $('#batt_volt_neg_id').text(get_value({ values: [valtable_obj.EX_BATT_VOLTNEG], index: 0, unit: ' V', precision: 2 }));
    } else
    $('#batt_volt_id').text(get_value({ values: [valtable_obj.BATTVOLT], index: 0, unit: ' V', precision: 2 }));

    if (valtable_obj.EX_BATT_CURRPOS != null && valtable_obj.EX_BATT_CURRPOS != -9999) {
      $('#batt_currpos_id').text(get_value({ values: [valtable_obj.EX_BATT_CURRPOS], index: 0, unit: ' A', precision: 1 }));
    }
    if (valtable_obj.EX_BATT_CURRNEG != null && valtable_obj.EX_BATT_CURRNEG != -9999) {
      $('#batt_currneg_id').text(get_value({ values: [valtable_obj.EX_BATT_CURRNEG], index: 0, unit: ' A', precision: 1 }));
    }
    if (valtable_obj.CUBE_DISCHPOS != null && valtable_obj.CUBE_DISCHPOS != -9999) {
      $('#batt_currpos_id2').text(get_value({ values: [valtable_obj.CUBE_DISCHPOS], index: 0, unit: ' A', precision: 1 }));
    }
    if (valtable_obj.CUBE_DISCHNEG != null && valtable_obj.CUBE_DISCHNEG != -9999) {
      $('#batt_currneg_id2').text(get_value({ values: [valtable_obj.CUBE_DISCHNEG], index: 0, unit: ' A', precision: 1 }));
    }

    $('#batt_autotime_id').text(get_value({ values: [valtable_obj.AUTONOMTIME], index: 0, unit: ' Min', precision: 0 }));
    $('#batt_cap_id').text(get_value({ values: [valtable_obj.BATTCAP], index: 0, unit: ' %', precision: 0 }));

    if (valtable_obj.EX_BATT_SECONBATT != null)
    $('#batt_seconds_id').text(get_value({ values: [valtable_obj.EX_BATT_SECONBATT], index: 0, unit: ' Sec', precision: 0 }));
  }

  function get_ph_1_3_vals(volt, cur, freq, phases) {
    var val_obj, val;
    l_1_3_vals = {
      availables: [0, 0, 0], //L1-L3
      val_objs: [
        { unit: ' V', precision: 0, values: ['', '', ''] },
        { unit: ' A', precision: 2, values: ['', '', ''] },
        { unit: ' Hz', precision: 2, values: ['', '', ''] }
      ]
    };
    volt = volt || [];
    cur = cur || [], freq = freq || [];
    $.each([volt, cur, freq], function (index, values) {
      val_obj = l_1_3_vals.val_objs[index];
      $.each(values, function (i, val) {
        if (i < phases) {
          val = values[i];
          if ((ups_protocol != prot_snmp && val != -9999) || (ups_protocol == prot_snmp && val != -1)) {
            if (values === freq && (i == 1 || i == 2)) {} else
            l_1_3_vals.availables[i] = 1;

            if (Math.round(val) == val)
            val = val.toString();
            else
            val = val.toFixed(val_obj.precision);
            val += val_obj.unit;
            val_obj.values[i] = val;
          }
        }
      });
    });
    return l_1_3_vals;
  }

  function get_value(val_obj) {
    if (!val_obj.values || (val_obj.values.length <= val_obj.index))
    return 'N/A';
    var val = val_obj.values[val_obj.index];
    if (val == null || val_obj.values[val_obj.index] == -9999)
    return 'N/A';
    if (ups_protocol == prot_snmp && val_obj.values[val_obj.index] == -1)
    return 'N/A';

    if (Math.round(val) == val)
    val = val.toString();
    else
    val = val.toFixed(val_obj.precision);
    val += val_obj.unit;
    return val;
  }

  function create_iby_ire_ssi_inv_rec_batt() //create IBY, IRE, SSI, INVERTER, RECTIFIER and BATTERY  pannes
  {
    var div, fragment = document.createDocumentFragment();
    css_obj = { position: 'absolute', top: y_pos + 229, left: x_pos + 548, width: 24, height: 10, overflow: 'hidden' };

    if (valtable_obj.EX_EXTSTATUS != null && (valtable_obj.EX_EXTSTATUS & (0x2000 | 0x400))) // load off
    css_obj['background-color'] = '#DC3632'; //red
    else if ((status & 16384) != 16384 && (status & 4) != 4) //IBY commlost and active
    css_obj['background-color'] = '#DC3632'; //red
    else
    css_obj['background-color'] = 'rgb(40, 134, 59)'; //'#52B245';//green rgb(40, 134, 59)
    div = $('<div/>').attr({ id: 'iby_div_id', 'class': 'img-rounded' }).css(css_obj);
    fragment.appendChild(div[0]);

    css_obj = { position: 'absolute', top: y_pos + 229, left: x_pos + 220, width: 24, height: 10, overflow: 'hidden' };
    if ((status & 8) == 8 || (status & 128) == 128) //IRE powerfail or input-high
    css_obj['background-color'] = '#DC3632'; //red
    else
    css_obj['background-color'] = 'rgb(40, 134, 59)'; //#52B245';//green
    div = $('<div/>').attr({ id: 'ire_div_id', 'class': 'img-rounded' }).css(css_obj);
    fragment.appendChild(div[0]);

    //bypass led
    css_obj = { position: 'absolute', top: y_pos + 150, left: x_pos + 220, width: 24, height: 10, overflow: 'hidden' };
    if (status & 8) //SSI powerfail
    css_obj['background-color'] = '#F1F1F2';
    else if (status & 1) //bypass
    css_obj['background-color'] = '#52B245';
    else
    css_obj['background-color'] = '#FFF500';

    div = $('<div/>').attr({ id: 'by_led_div_id', 'class': 'img-rounded' }).css(css_obj);
    fragment.appendChild(div[0]);

    document.body.appendChild(fragment);

    if (status & 8) //SSI powerfail
    panes_api.create_img_panel('ssi_img_id', './image/ups/ssww.gif', x_pos + 372, y_pos + 130, 50, 50, 0);
    else if (status & 1) //bypass
    panes_api.create_img_panel('ssi_img_id', './image/ups/sswgr.gif', x_pos + 372, y_pos + 130, 50, 50, 0);
    else
    panes_api.create_img_panel('ssi_img_id', './image/ups/sswge.gif', x_pos + 372, y_pos + 130, 50, 50, 0);

    if ((status & 512) == 512 //INVERTER inverterfail
      ||
      (status & 4096) == 4096 //inverterwarn
      ||
      (status & 256) == 256 //outputhigh
      ||
      (status & 32) == 32 //overtemp
      ||
      (overload == 1)) //overload
    panes_api.create_img_panel('inv_img_id', './image/ups/wrr.gif', x_pos + 458, y_pos + 210, 50, 50, 0);

    else if ((level != 0) && ((tempdeg != null) && (tempdeg != -9999) && (tempmax != null)) && (tempdeg > tempmax)) //overtemp1
    panes_api.create_img_panel('inv_img_id', './image/ups/wrr.gif', x_pos + 458, y_pos + 210, 50, 50, 0);

    else if ((status & 1) == 1) //bypass
    panes_api.create_img_panel('inv_img_id', './image/ups/wrge.gif', x_pos + 458, y_pos + 210, 50, 50, 0);
    else
    panes_api.create_img_panel('inv_img_id', './image/ups/wrgr.gif', x_pos + 458, y_pos + 210, 50, 50, 0);

    if ((status & 8) == 8) //RECTIFIER powerfail
    panes_api.create_img_panel('rec_img_id', './image/ups/grw.gif', x_pos + 286, y_pos + 210, 50, 50, 0);
    else
    panes_api.create_img_panel('rec_img_id', './image/ups/grgr.gif', x_pos + 286, y_pos + 210, 50, 50, 0);

    if ((status & 8) == 8) //BATTERY powerfail
    panes_api.create_img_panel('batt_img_id', './image/ups/Batgrv.gif', x_pos + 372, y_pos + 294, 50, 50, 0);
    else if ((status & 16) == 16 || (status & 1024) == 1024) // batterylow batterybad
    panes_api.create_img_panel('batt_img_id', './image/ups/Batrv.gif', x_pos + 372, y_pos + 294, 50, 50, 0);
    else
    panes_api.create_img_panel('batt_img_id', './image/ups/Batgev.gif', x_pos + 372, y_pos + 294, 50, 50, 0);
  }

  function set_iby_ire_ssi_inv_rec_batt() //SET IBY, IRE, SSI, INVERTER, RECTIFIER and BATTERY  pannes
  {
    if (valtable_obj.EX_EXTSTATUS != null && (valtable_obj.EX_EXTSTATUS & (0x2000 | 0x400))) // load off
    $('#iby_div_id').css('background-color', '#DC3632'); //red
    else if ((status & 16384) != 16384 && (status & 4) != 4) //IBY commlost and active
    $('#iby_div_id').css('background-color', '#DC3632'); //red
    else
    $('#iby_div_id').css('background-color', 'rgb(40, 134, 59)'); //green

    if ((status & 8) == 8 || (status & 128) == 128) //IRE powerfail or input-high
    $('#ire_div_id').css('background-color', '#DC3632'); //red
    else
    $('#ire_div_id').css('background-color', 'rgb(40, 134, 59)'); //green

    if ((status & 8) == 8) //SSI powerfail
    $('#ssi_img_id').attr('src', './image/ups/ssww.gif');
    else if ((status & 1) == 1) //bypass
    $('#ssi_img_id').attr('src', './image/ups/sswgr.gif');
    else
    $('#ssi_img_id').attr('src', './image/ups/sswge.gif');

    if ((status & 512) == 512 //INVERTER inverterfail
      ||
      (status & 4096) == 4096 //inverterwarn
      ||
      (status & 256) == 256 //outputhigh
      ||
      (status & 32) == 32 //overtemp
      ||
      (overload == 1)) //overload
    $('#inv_img_id').attr('src', './image/ups/wrr.gif');

    else if ((level != 0) && ((tempdeg != null) && (tempdeg != "-9999") && (tempmax != null)) && (tempdeg > tempmax)) //overtemp1
    $('#inv_img_id').attr('src', './image/ups/wrr.gif');
    else if ((status & 1) == 1) //bypass
    $('#inv_img_id').attr('src', './image/ups/wrge.gif');
    else
    $('#inv_img_id').attr('src', './image/ups/wrgr.gif');

    if ((status & 8) == 8) //RECTIFIER powerfail
    $('#rec_img_id').attr('src', './image/ups/grw.gif');
    else
    $('#rec_img_id').attr('src', './image/ups/grgr.gif');

    if ((status & 8) == 8) //BATTERY powerfail
    $('#batt_img_id').attr('src', './image/ups/Batgrv.gif');
    else if ((status & 16) == 16 || (status & 1024) == 1024) // batterylow batterybad
    $('#batt_img_id').attr('src', './image/ups/Batrv.gif');
    else
    $('#batt_img_id').attr('src', './image/ups/Batgev.gif');

    //bypass led
    if (status & 8) //SSI powerfail
    $('#by_led_div_id').css('background-color', '#F1F1F2');
    else if (status & 1) //bypass
    $('#by_led_div_id').css('background-color', '#52B245');
    else
    $('#by_led_div_id').css('background-color', '#FFF500');

  }

  function create_status_pane() //UPS status
  {
    var status_div = $('<div/>'),
    status_span;
    css_obj = { position: 'absolute', top: y_pos + 50, left: x_pos + 710, align: 'center', overflow: 'hidden' };

    $(status_div).css(css_obj);
    status_span = $('<span />', { id: 'ups_status_id', 'class': get_status_class(), title: i18n('Click to see Status Text') }).append(i18n('Status'));
    status_span.css('cursor', 'pointer');
    status_span.click(function (event) //visibility: hidden
      {
        var sys_mod_status_txt_div = $('#sys_mod_status_txt_div'),
        display = sys_mod_status_txt_div.css('display') == 'none' ? 'block' : 'none';
        sys_mod_status_txt_div.css({ display: display });
      });

    $('<h4 />').append(status_span).appendTo(status_div);
    status_div.appendTo(document.body);
  }

  function set_ups_status() //set UPS status
  {
    $('#ups_status_id').attr('class', get_status_class());
  }

  function get_status_class() {
    if (valtable_obj.RAWSTATEE != null) {
      if (valtable_obj.RAWSTATEE == 0 || valtable_obj.RAWSTATEE == 1 || valtable_obj.RAWSTATEE == 2 ||
        valtable_obj.RAWSTATEE == 9 || valtable_obj.RAWSTATEE == 10)
      return 'label label-danger';
    }
    if (valtable_obj.RAWSTATEC != null && valtable_obj.RAWSTATEC != 0)
    return 'label label-danger';
    if (valtable_obj.RAWSTATED != null && valtable_obj.RAWSTATED != 0)
    return 'label label-danger';
    if (status & 0xF7BA)
    return 'label label-danger';

    if (valtable_obj.RAWSTATEE != null) {
      if (valtable_obj.RAWSTATEE == 3 || valtable_obj.RAWSTATEE == 5 || valtable_obj.RAWSTATEE == 6 ||
        valtable_obj.RAWSTATEE == 7 || valtable_obj.RAWSTATEE == 8)
      return 'label label-warning';
    }
    if (status & 0x41)
    return 'label label-warning';

    return 'label label-success';
  }

  function create_outpower_loadbars() //outpower loadbars
  {
    var fragment = document.createDocumentFragment(),
    div;

    if (valtable_obj.OUTPOWER == null)
    return true;

    outpower_loadbars_created = true;

    panes_api.create_loadbar_pane({ value: valtable_obj.OUTPOWER[0], id: 'loadbar1_id', left: x_pos + 564, top: y_pos + 336, title: 'L1', scaler: 1 });
    if (outphases > 1)
    panes_api.create_loadbar_pane({ value: valtable_obj.OUTPOWER[1], id: 'loadbar2_id', left: x_pos + 564, top: y_pos + 366, title: 'L2', scaler: 0 });
    if (outphases > 2)
    panes_api.create_loadbar_pane({ value: valtable_obj.OUTPOWER[2], id: 'loadbar3_id', left: x_pos + 564, top: y_pos + 396, title: 'L3', scaler: 0 });

    css_obj = { position: 'absolute', top: y_pos + 310, left: x_pos + 540, width: 250, height: 120, 'background-color': $(document.body).css('background-color'), 'z-index': 20 };
    div = $('<div/>').attr({ id: 'loadbars' }).css(css_obj);
    fragment.appendChild(div[0]);

    document.body.appendChild(fragment);
    set_outpower_loadbars();
  }

  function set_outpower_loadbars() //outpower loadbars
  {
    if (valtable_obj.OUTPOWER == null)
    return true;
    if (valtable_obj.OUTPOWER[0] != -9999) {
      panes_api.set_loabar_value(valtable_obj.OUTPOWER[0], 'loadbar1_id');
    }
    if ((outphases > 1) && valtable_obj.OUTPOWER[1] != -9999) {
      panes_api.set_loabar_value(valtable_obj.OUTPOWER[1], 'loadbar2_id');
    }
    if ((outphases > 2) && valtable_obj.OUTPOWER[2] != -9999) {
      panes_api.set_loabar_value(valtable_obj.OUTPOWER[2], 'loadbar3_id');
    }

    if ((valtable_obj.OUTPOWER[0] != -9999) || (valtable_obj.OUTPOWER[1] != -9999) || (valtable_obj.OUTPOWER[2] != -9999)) {
      $('#loadbars').css('display', 'none');
    } else {
      $('#loadbars').css('display', 'block');
    }
  }

  function create_rom_temp() //rom temp
  {
    var fragment = document.createDocumentFragment(),
    rom_temp_div;
    css_obj = { position: 'absolute', top: y_pos + 306, left: x_pos + 8 };
    rom_temp_div = $('<div/>').attr({ id: 'rom_temp_div_id' }).css(css_obj);
    panes_api.create_rom_temp_pane(valtable_obj, { id: 'rom_temp_table_id', 'class': 'table table-condensed table-hover' }).appendTo(rom_temp_div);
    fragment.appendChild(rom_temp_div[0]);
    document.body.appendChild(fragment);
  }

  function create_modules_panes() //Input table ();
  {
    var fragment = document.createDocumentFragment(),
    tr, tr, table, tbody, thead, button, modules_div, system_div,
    module_index, index_val, hassysvals = has_system_values(),
    stitle;

    if (modules_exist_maske == null) {
      return;
    }

    modules_div = $('#modules_div');
    if (modules_div.length != 0)
    modules_div.remove();

    css_obj = { position: 'absolute', top: y_pos + 480, left: x_pos + 8 }; //, width:300, height:26

    modules_div = $('<div/>', { id: 'modules_div' }).css(css_obj);
    table = $('<table />', { 'class': 'table table-condensed' }).appendTo(modules_div); // table-hover
    tbody = $('<tbody />').appendTo(table);
    thead = $('<thead />').appendTo(table);

    if (hassysvals)
    stitle = i18n('System and Modules');
    else
    stitle = i18n('Modules');

    tr = $('<tr />').append($('<th />', { colspan: 32 }).append(stitle)).appendTo(thead);
    tr = $('<tr />', { id: 'tr_sys_id' }).appendTo(tbody);
    td = $('<td />', { 'align': 'left' }).append(i18n('System')).appendTo(tr);
    td.css({ 'vertical-align': 'middle', width: 70 });
    if (!hassysvals)
    td.css('display', 'none');
    td = $('<td />', { 'align': 'left' }).appendTo(tr);

    if (hassysvals) {
      button = $('<button>', { id: 'module_sys', 'class': "btn btn-sm btn-default" }).append('S').appendTo(td);

      if (0 == ups_id) {
        button.css({ 'border': '2px outset black', 'box-shadow': '0 0 2px black' });
      }

      button.click(function (event) {
        get_module_values(0);
        event.preventDefault();
      });
    }

    tr = $('<tr />').appendTo(tbody);
    td = $('<td />', { 'align': 'left' }).append('Modules').appendTo(tr);
    td.css({ 'vertical-align': 'middle', width: 70 });
    if (!hassysvals)
    td.css('display', 'none');

    for (module_index = 0; module_index < 32; ++module_index) {
      index_val = 1 << module_index;
      if ((modules_exist_maske & index_val) != index_val)
      continue;

      td = $('<td />', { 'align': 'left' }).appendTo(tr);
      button = $('<button>', {
        id: 'module_' + (module_index + 1),
        'class': get_module_status_class(module_index)
      }).append(module_index + 1).appendTo(td);

      if (module_index + 1 == ups_id) {
        button.css({ 'border': '2px outset black', 'box-shadow': '0 0 2px black' });
      }
      button.click(function (event) {
        var id = parseInt($(this).text());
        get_module_values(id);
        event.preventDefault();
      });
    }

    fragment.appendChild(modules_div[0]);
    document.body.appendChild(fragment);
  }

  function create_sys_mod_status_panel() {
    var fragment = document.createDocumentFragment(),
    tr, tr, table, tbody, a,
    title = "Status:" + valtable_obj.MODEL,
    status_txt_array = get_sys_mod_status_txt(),
    sys_mod_status_txt_div;

    css_obj = {
      position: 'absolute',
      top: y_pos + 72,
      left: x_pos + 36,
      align: 'center',
      'background-color': '#F1F1F1',
      display: 'none',
      border: 'solid 1px black'
    };

    sys_mod_status_txt_div = $('<div/>', { id: 'sys_mod_status_txt_div', 'class': 'img-rounded' }).css(css_obj);

    if (ups_id != 0)
    title += " [Module " + ups_id + "]";

    a = $('<a/>', { 'class': 'list-group-item active', id: 'status_model_name_id' }).append(title).appendTo(sys_mod_status_txt_div);
    a.css('height', 30);
    table = $('<table />', { 'class': 'table table-condensed', id: 'sys_mod_status_txt_table' }).appendTo(sys_mod_status_txt_div); // table-hover
    table.css({ 'background-color': '#F1F1F1' })
    tbody = $('<tbody />', { id: 'sys_mod_status_txt_table_body' }).appendTo(table);
    $.each(status_txt_array, function (index, status_txt) {
      tr = $('<tr />').appendTo(tbody);
      td = $('<td />', { 'align': 'left' }).append(status_txt).appendTo(tr);
    });

    fragment.appendChild(sys_mod_status_txt_div[0]);
    document.body.appendChild(fragment);
    new utils_api.draggable('sys_mod_status_txt_div');
  }

  function set_sys_mod_status() {
    var tr, td,
    title = "Status:" + valtable_obj.MODEL,
    tbody = $('#sys_mod_status_txt_table_body'),
    status_txt_array = get_sys_mod_status_txt();

    if (ups_id != 0)
    title += " [Module " + ups_id + "]";

    $('#status_model_name_id').text(title);

    tbody.empty();
    $.each(status_txt_array, function (index, status_txt) {
      tr = $('<tr />').appendTo(tbody);
      td = $('<td />', { 'align': 'left' }).append(status_txt).appendTo(tr);
    });
  }


  function get_sys_mod_status_txt() {
    var status_txt_array = [];
    $.each(status_obj, function (index, status_obj) {
      if (status_obj.bit & valtable_obj.STATUS) {
        status_txt_array.push(i18n(status_obj.txt));
      }
    });

    if (valtable_obj.EX_EXTSTATUS != null) {
      $.each(ext_status_obj, function (index, ext_status_obj) {
        if (ext_status_obj.bit & valtable_obj.EX_EXTSTATUS) {
          status_txt_array.push(i18n(ext_status_obj.txt));
        }
      });
    }

    if (ups_protocol == 0x8116)  // KSTAR2
    {
       if (valtable_obj.RAWSTATEC != null) {
         $.each(rawstatec2, function (index, rawstatec2) {
           if (rawstatec2.bit & valtable_obj.RAWSTATEC) {
             status_txt_array.push(i18n(rawstatec2.txt));
           }
         });
       }

       if (valtable_obj.RAWSTATED != null) {
         $.each(rawstated2, function (index, rawstated2) {
           if (rawstated2.bit & valtable_obj.RAWSTATED) {
             status_txt_array.push(i18n(rawstated2.txt));
           }
         });
       }
    }
    else
    {
       if (valtable_obj.RAWSTATEC != null) {
         $.each(rawstatec, function (index, rawstatec) {
           if (rawstatec.bit & valtable_obj.RAWSTATEC) {
             status_txt_array.push(i18n(rawstatec.txt));
           }
         });
       }

       if (valtable_obj.RAWSTATED != null) {
         $.each(rawstated, function (index, rawstated) {
           if (rawstated.bit & valtable_obj.RAWSTATED) {
             status_txt_array.push(i18n(rawstated.txt));
           }
         });
       }
    }
    if (valtable_obj.RAWSTATEE != null) {
      $.each(rawstatee, function (index, rawstatee) {
        if (rawstatee.val == valtable_obj.RAWSTATEE) {
          status_txt_array.push(i18n(rawstatee.txt));
        }
      });
    }
    return status_txt_array;
  }

  function get_module_values(id) {
    jqmodule.unsubscribe(cb);
    ups_id = id;
    req_params.refresh_tm = 0;

    if (plattform == utils_api.plattform_cs141) {
      return get_cs141_module_values(id);
    }

    req_params.url = get_url("data.cgi"),
    req_params.data = { json: "ups", ups_id: ups_id };
    cb = jqmodule.subscribe(this, { update: update, error: err_funct }, req_params);
  }

  function get_cs141_module_values(id) {
    var cs141_cmd_option = {
      data: {},
      data_type: 'text',
      success: cs141_cmd_update,
      error: err_funct
    },

    cs141_cmd_settings = { methode: "POST", xhttp_methode_override: 'PUT', req_payload_json: true },
    ctl_url = get_url("api/devices/ups/control");

    cs141_cmd_option.data = { code: 'actUpsId', params: { upsId: id } };
    jqmodule.ajax(ctl_url, cs141_cmd_option, cs141_cmd_settings);
  }

  function cs141_cmd_update() {
    if (err_div)
    err_div.css('display', 'none');
    cb = jqmodule.subscribe(this, { update: update, error: err_funct }, req_params);
  }


  function get_module_status_class(module_index) {
    var bit_val = 1 << module_index;
    if ((valtable_obj.RAWSTATEG & bit_val) == bit_val)
    return "btn btn-sm btn-danger";
    return "btn btn-sm btn-default";

  }

  function set_model_name() {
    var model_name = valtable_obj.MODEL;
    if (ups_id != 0)
    model_name += " [Module " + ups_id + "]";
    $('#model_name_id').html(model_name);
  }
}(jQuery));