/*
   lineinteractive_panel.js - UPS line interactive screen.
   Copyright (c) 2014, Generex GmbH. All rights reserved.
   Said.
*/

var lineinteractive_api = (function ($) {
  var oid = get_oid_from_url(),
  cb = null,
  pannes_created = false,
  err_div = null,
  check_bowser = true,
  outpower_loadbars_created = false,
  css_obj, attr_obj,
  valtable_obj, report_obj, unms_obj,
  inphases = null,
  outphases = null,
  bypphases = null,
  status, tempmax, tempdeg, overload = 0,
  x_pos = 10,
  y_pos = 10;

  var req_params = null;

  /*public functions*/
  return {
    init: init
  };

  function init() {
    if (check_bowser && !utils_api.check_bowser()) {
      $('#loading_id').css('display', 'none');
      panes_api.create_br_check_panel(this, browser_check_ok);
      return true;
    }
    create_err_msg_panel();

    req_params = utils_api.get_req_params(utils_api.device_ups, oid);
    cb = jqmodule.subscribe(this, { update: update, error: err_funct }, req_params);
  }

  function browser_check_ok() {
    check_bowser = false;
    init();
  }

  function create_err_msg_panel() {
    css_obj = { position: 'absolute', top: y_pos, left: x_pos, width: 791, display: 'none', 'z-index': 22 };
    err_div = panes_api.create_err_panel({ id: 'error_div', 'class': 'alert alert-danger' }, css_obj);
  }

  function err_funct(response) {
    jqmodule.refresh(cb, 10 * 1000);
    panes_api.display_err({ response: response, oid: oid, err_div: err_div, err_msg_div_id: 'error_msg_div' });
  }

  function update(data) {
    if (data.length == 0) {
      return;
    }

    set_variable(data);
    jqmodule.refresh(cb, report_obj.refresh * 1000);
    if (err_div)
    err_div.css('display', 'none');

    if (window.menu_api != null)
    menu_api.update(data);

    unms_obj = get_unms_obj(data);
    if (unms_obj != null) {
      info_head_api.update(unms_obj);
    }

    if (!pannes_created) {
      pannes_created = true;
      create_panes();
      return false;
    }
    set_input_values(); ////Input values
    set_output_values(); //Output values
    set_battery_value(); //Battery value
    set_byp_values(); //Bypass values
    set_iby_ire_ssi_inv_rec_batt(); //SET IBY, IRE, SSI, INVERTER, RECTIFIER and BATTERY	pannes
    set_ups_status(); //set UPS status
    set_outpower_loadbars(); //outpower loadbars
    if (valtable_obj._platform !== "ups_view")
    panes_api.set_rom_temp_values(valtable_obj); //set rom temp values
    set_footer();
  }

  function create_footer_pane() {
    if (!oid || !unms_obj)
    return true;
    footer_api.create({ //Attr
        id: 'panel2'
        //'class':'unmsFooter'
      }, { //css
        position: 'absolute',
        top: '490px',
        left: '74px'
      },
      unms_obj
    );
  }

  function set_footer() {
    if (!oid || !unms_obj)
    return true;
    footer_api.set(unms_obj);
  }

  function set_variable(data) {
    valtable_obj = data.ups.valtable;
    report_obj = data.report;

    status = valtable_obj.STATUS;
    tempmax = valtable_obj.TMPMAX;
    tempdeg = valtable_obj.TEMPDEG;
    level = valtable_obj.LEVEL;

    if (valtable_obj.INPHASES != null)
    inphases = valtable_obj.INPHASES;
    else
    inphases = valtable_obj.PHASES;

    if (valtable_obj.OUTPHASES != null)
    outphases = valtable_obj.OUTPHASES;
    else
    outphases = valtable_obj.PHASES;

    if (valtable_obj.EX_BYP_NUMLINES != null && valtable_obj.EX_BYP_NUMLINES != -9999)
    bypphases = valtable_obj.EX_BYP_NUMLINES;
    else
    bypphases = inphases;

    if (valtable_obj.OUTPOWER != null) {
      if (outphases > 2) {
        if ((valtable_obj.OUTPOWER[0] > 100) || (valtable_obj.OUTPOWER[1] > 100) || (valtable_obj.OUTPOWER[2] > 100))
        overload = 1;
      } else {
        if (valtable_obj.OUTPOWER[0] > 100)
        overload = 1;
      }
    }
  }

  function create_panes() {
    utils_api.setTitle(valtable_obj._platform);
    create_bg(); //background
    create_input_pane(); //Input table
    create_output_pane(); //Outnput	panel
    create_battery_pane(); //Battery panel
    create_byp_pane(); //Bypass panel
    create_iby_ire_ssi_inv_rec_batt(); //create IBY, IRE, SSI, INVERTER, RECTIFIER and BATTERY	pannes
    create_status_pane(); //UPS status
    create_model_panes();
    create_outpower_loadbars(); //outpower loadbars
    if (valtable_obj._platform !== "ups_view")
    create_rom_temp(); //rom temp
    create_footer_pane();
  }

  function create_bg() {
    var fragment = document.createDocumentFragment(),
    img_div;
    css_obj = { position: 'absolute', top: y_pos, left: x_pos, width: 818, height: 460, border: 'solid 1px #808080' };
    img_div = $('<div/>', { 'class': 'img-rounded' }).css(css_obj);
    $('<img />').attr({ 'id': 'bg_img_id', 'class': 'img-rounded', width: 816, 'height': 459, src: './image/ups/line_int.png' }).appendTo(img_div);
    fragment.appendChild(img_div[0]);
    document.body.appendChild(fragment);
  }

  function create_model_panes() {
    var model_div, a;
    css_obj = { position: 'absolute', top: y_pos, left: x_pos, align: 'center', width: 818 }; // 'background-color': '#3a87ad'
    attr_obj = { id: 'model_div_id', 'class': 'list-group' };
    model_div = panes_api.create_div(attr_obj, css_obj);
    a = $('<a/>', { 'class': 'list-group-item active' }).append(valtable_obj.MODEL).appendTo(model_div);
  }

  function create_input_pane() //Input table
  {
    var input_val = get_ph_1_3_vals(valtable_obj.INVOLT, valtable_obj.INCURR, valtable_obj.INFREQ, inphases),
    fragment = document.createDocumentFragment(),
    val, tr, table, tbody, thead, in_div;

    if (input_val.availables[0] == 0 && input_val.availables[1] == 0 && input_val.availables[2] == 0)
    return true;

    css_obj = { position: 'absolute', top: y_pos + 166, left: x_pos + 8 };

    in_div = $('<div/>', { id: 'input_div_id' }).css(css_obj);
    table = $('<table />', { 'class': 'table table-condensed table-hover' }).appendTo(in_div); // table-hover
    tbody = $('<tbody />').appendTo(table);
    thead = $('<thead />').appendTo(table);

    tr = $('<tr />').append($('<th />', { colspan: 4 }).append(i18n(lang_Input))).appendTo(thead);

    /*if(inphases == 1 )
    	table.css('margin-top','22px');*/

    if (input_val.availables[0] != 0) {
      tr = $('<tr />').append($('<td />').append("L1:")).appendTo(tbody);
      if (!input_val.val_objs[0].values[0].empty())
      $('<td />', { id: 'input_volt1_id' }).append(input_val.val_objs[0].values[0]).appendTo(tr);

      if (!input_val.val_objs[1].values[0].empty())
      $('<td />', { id: 'input_cur1_id' }).append(input_val.val_objs[1].values[0]).appendTo(tr);

      if (!input_val.val_objs[2].values[0].empty())
      $('<td />', { id: 'input_freq1_id' }).append(input_val.val_objs[2].values[0]).appendTo(tr);
    }

    if (input_val.availables[1] != 0) {
      tr = $('<tr />').append($('<td />').append("L2:")).appendTo(tbody);
      if (!input_val.val_objs[0].values[1].empty())
      $('<td />', { id: 'input_volt2_id' }).append(input_val.val_objs[0].values[1]).appendTo(tr);

      if (!input_val.val_objs[1].values[1].empty())
      $('<td />', { id: 'input_cur2_id' }).append(input_val.val_objs[1].values[1]).appendTo(tr);
    }
    if (input_val.availables[2] != 0) {
      tr = $('<tr />').append($('<td />').append("L3:")).appendTo(tbody);
      if (!input_val.val_objs[0].values[2].empty())
      $('<td />', { id: 'input_volt3_id' }).append(input_val.val_objs[0].values[2]).appendTo(tr);

      if (!input_val.val_objs[1].values[2].empty())
      $('<td />', { id: 'input_cur3_id' }).append(input_val.val_objs[1].values[2]).appendTo(tr);
    }

    fragment.appendChild(in_div[0]);
    document.body.appendChild(fragment);
  }

  function set_input_values() {
    var input_val = get_ph_1_3_vals(valtable_obj.INVOLT, valtable_obj.INCURR, valtable_obj.INFREQ, inphases);
    if (input_val.availables[0] == 0 && input_val.availables[1] == 0 && input_val.availables[2] == 0)
    return true;

    if (input_val.availables[0] != 0) {
      $('#input_volt1_id').text(input_val.val_objs[0].values[0]);
      $('#input_cur1_id').text(input_val.val_objs[1].values[0]);
      $('#input_freq1_id').text(input_val.val_objs[2].values[0]);
    }
    if (input_val.availables[1] != 0) {
      $('#input_volt2_id').text(input_val.val_objs[0].values[1]);
      $('#input_cur2_id').text(input_val.val_objs[1].values[1]);
    }
    if (input_val.availables[2] != 0) {
      $('#input_volt3_id').text(input_val.val_objs[0].values[2]);
      $('#input_cur3_id').text(input_val.val_objs[1].values[2]);
    }
  }

  function create_output_pane() //Output
  {
    var output_val = get_ph_1_3_vals(valtable_obj.OUTPUT_VOLT, valtable_obj.EX_OUT_CURR, valtable_obj.OUTFREQ, outphases),
    output_div = null,
    val, tr, table, tbody, thead,
    fragment = document.createDocumentFragment();
    if (output_val.availables[0] == 0 && output_val.availables[1] == 0 && output_val.availables[2] == 0)
    return true;

    css_obj = { position: 'absolute', top: y_pos + 166, left: x_pos + 610 };

    output_div = $('<div/>', { id: 'output_div_id' }).css(css_obj);
    table = $('<table />', { 'class': 'table table-condensed table-hover' }).appendTo(output_div);

    tbody = $('<tbody />').appendTo(table);
    thead = $('<thead />').appendTo(table);
    tr = $('<tr />').append($('<th />', { colspan: 4 }).append(i18n(lang_Output))).appendTo(thead);

    /*if(outphases == 1 )
    	table.css('margin-top','22px');*/

    if (output_val.availables[0] != 0) {
      tr = $('<tr />').append($('<td />').append("L1:")).appendTo(tbody);
      if (!output_val.val_objs[0].values[0].empty())
      $('<td />', { 'id': 'out_volt1_id' }).append(output_val.val_objs[0].values[0]).appendTo(tr);

      if (!output_val.val_objs[1].values[0].empty())
      $('<td />', { id: 'out_cur1_id' }).append(output_val.val_objs[1].values[0]).appendTo(tr);

      if (!output_val.val_objs[2].values[0].empty())
      $('<td />', { id: 'out_freq1_id' }).append(output_val.val_objs[2].values[0]).appendTo(tr);
    }
    if (output_val.availables[1] != 0) {
      tr = $('<tr />').append($('<td />').append("L2:")).appendTo(tbody);
      if (!output_val.val_objs[0].values[1].empty())
      $('<td />', { id: 'out_volt2_id' }).append(output_val.val_objs[0].values[1]).appendTo(tr);

      if (!output_val.val_objs[1].values[1].empty())
      $('<td />', { id: 'out_cur2_id' }).append(output_val.val_objs[1].values[1]).appendTo(tr);
    }
    if (output_val.availables[2] != 0) {
      tr = $('<tr />').append($('<td />').append("L3:")).appendTo(tbody);
      if (!output_val.val_objs[0].values[2].empty())
      $('<td />', { id: 'out_volt3_id' }).append(output_val.val_objs[0].values[2]).appendTo(tr);

      if (!output_val.val_objs[1].values[2].empty())
      $('<td />', { id: 'out_cur3_id' }).append(output_val.val_objs[1].values[2]).appendTo(tr);
    }

    fragment.appendChild(output_div[0]);
    document.body.appendChild(fragment);
  }

  function set_output_values() //Output values
  {
    var output_val = get_ph_1_3_vals(valtable_obj.OUTPUT_VOLT, valtable_obj.EX_OUT_CURR, valtable_obj.OUTFREQ, outphases);
    if (output_val.availables[0] == 0 && output_val.availables[1] == 0 && output_val.availables[2] == 0)
    return true;

    if (output_val.availables[0] != 0) {
      $('#out_volt1_id').text(output_val.val_objs[0].values[0]);
      $('#out_cur1_id').text(output_val.val_objs[1].values[0]);
      $('#out_freq1_id').text(output_val.val_objs[2].values[0]);
    }
    if (output_val.availables[1] != 0) {
      $('#out_volt2_id').text(output_val.val_objs[0].values[1]);
      $('#out_cur2_id').text(output_val.val_objs[1].values[1]);
    }
    if (output_val.availables[2] != 0) {
      $('#out_volt3_id').text(output_val.val_objs[0].values[2]);
      $('#out_cur3_id').text(output_val.val_objs[1].values[2]);
    }
  }

  function create_byp_pane() //Bypass
  {
    var bypass_val = get_ph_1_3_vals(valtable_obj.EX_BYP_VOLT, valtable_obj.EX_BYP_CURR, valtable_obj.EX_BYP_FREQ, bypphases),
    byp_div, val, tr, table, tbody, thead,
    fragment = document.createDocumentFragment();

    if (bypass_val.availables[0] == 0 && bypass_val.availables[1] == 0 && bypass_val.availables[2] == 0)
    return true;

    css_obj = { position: 'absolute', top: y_pos + 46, left: x_pos + 330 };
    byp_div = $('<div/>', { id: 'bypass_div_id' }).css(css_obj);
    table = table = $('<table />', { 'class': 'table table-condensed table-hover' }).appendTo(byp_div);
    tbody = $('<tbody />').appendTo(table);
    thead = $('<thead />').appendTo(table);
    tr = $('<tr />').append($('<th />', { colspan: 4 }).append(i18n(lang_Bypass))).appendTo(thead);

    /*if(bypphases == 1)
    	table.css('margin-top','50px');*/

    if (bypass_val.availables[0] != 0) {
      tr = $('<tr />').append($('<td />').append("L1:")).appendTo(tbody);
      if (!bypass_val.val_objs[0].values[0].empty())
      $('<td />', { 'id': 'byp_volt1_id' }).append(bypass_val.val_objs[0].values[0]).appendTo(tr);

      if (!bypass_val.val_objs[1].values[0].empty())
      $('<td />', { id: 'byp_cur1_id' }).append(bypass_val.val_objs[1].values[0]).appendTo(tr);

      if (!bypass_val.val_objs[2].values[0].empty())
      $('<td />', { id: 'byp_freq1_id' }).append(bypass_val.val_objs[2].values[0]).appendTo(tr);
    }

    if (bypass_val.availables[1] != 0) {
      tr = $('<tr />').append($('<td />').append("L2:")).appendTo(tbody);
      if (!bypass_val.val_objs[0].values[1].empty())
      $('<td />', { id: 'byp_volt2_id' }).append(bypass_val.val_objs[0].values[1]).appendTo(tr);

      if (!bypass_val.val_objs[1].values[1].empty())
      $('<td />', { id: 'byp_cur2_id' }).append(bypass_val.val_objs[1].values[1]).appendTo(tr);
    }

    if (bypass_val.availables[2] != 0) {
      tr = $('<tr />').append($('<td />').append("L3:")).appendTo(tbody);
      if (!bypass_val.val_objs[0].values[2].empty())
      $('<td />', { id: 'byp_volt3_id' }).append(bypass_val.val_objs[0].values[2]).appendTo(tr);

      if (!bypass_val.val_objs[1].values[2].empty())
      $('<td />', { id: 'byp_cur3_id' }).append(bypass_val.val_objs[1].values[2]).appendTo(tr);
    }
    fragment.appendChild(byp_div[0]);
    document.body.appendChild(fragment);
  }

  function set_byp_values() //Bypass values
  {
    var bypass_val = get_ph_1_3_vals(valtable_obj.EX_BYP_VOLT, valtable_obj.EX_BYP_CURR, valtable_obj.EX_BYP_FREQ, bypphases);

    if (bypass_val.availables[0] == 0 && bypass_val.availables[1] == 0 && bypass_val.availables[2] == 0)
    return true;

    if (bypass_val.availables[0] != 0) {
      $('#byp_volt1_id').text(bypass_val.val_objs[0].values[0]);
      $('#byp_cur1_id').text(bypass_val.val_objs[1].values[0]);
      $('#byp_freq1_id').text(bypass_val.val_objs[2].values[0]);
    }
    if (bypass_val.availables[1] != 0) {
      $('#byp_volt2_id').text(bypass_val.val_objs[0].values[1]);
      $('#byp_cur2_id').text(bypass_val.val_objs[1].values[1]);
    }
    if (bypass_val.availables[2] != 0) {
      $('#byp_volt3_id').text(bypass_val.val_objs[0].values[2]);
      $('#byp_cur3_id').text(bypass_val.val_objs[1].values[2]);
    }
  }


  //Standardscreen erweitern: Wenn EX_BATT_VOLTPOS und EX_BATT_VOLTNEG vorhanden und ungleich UPS_VAL_NA (-9999) dann beide Werte anstatt BATTVOLT anzeigen.
  function create_battery_pane() //Battery panel
  {
    var val, tr, table, tbody, thead, batt_div,
    fragment = document.createDocumentFragment();

    css_obj = { position: 'absolute', top: y_pos + 286, left: x_pos + 210 }; //, width:130, height:100
    attr_obj = { id: 'battery_div_id' };
    batt_div = $('<div/>').attr(attr_obj).css(css_obj);
    table = table = $('<table />', { 'class': 'table table-condensed table-hover' }).appendTo(batt_div);
    tbody = $('<tbody />').appendTo(table);
    thead = $('<thead />').appendTo(table);
    tr = $('<tr />').append($('<th />', { colspan: 2 }).append(i18n(lang_Battery))).appendTo(thead);

    if ((valtable_obj.EX_BATT_VOLTPOS != null && valtable_obj.EX_BATT_VOLTNEG != null) &&
      (valtable_obj.EX_BATT_VOLTPOS != -9999 && valtable_obj.EX_BATT_VOLTNEG != -9999)) {
      tr = $('<tr />').append($('<td />', { align: 'left' }).append('Pos.' + i18n(lang_Voltage) + ":")).appendTo(tbody);
      $('<td />', { id: 'batt_volt_pos_id' }).
      append(get_value({ values: [valtable_obj.EX_BATT_VOLTPOS], index: 0, unit: ' V', precision: 1 })).appendTo(tr);

      tr = $('<tr />').append($('<td />', { align: 'left' }).append('Neg.' + i18n(lang_Voltage) + ":")).appendTo(tbody);
      $('<td />', { id: 'batt_volt_neg_id' }).
      append(get_value({ values: [valtable_obj.EX_BATT_VOLTNEG], index: 0, unit: ' V', precision: 1 })).appendTo(tr);
    } else {
      tr = $('<tr />').append($('<td />', { align: 'left' }).append(i18n(lang_Voltage) + ":")).appendTo(tbody);
      $('<td />', { id: 'batt_volt_id' }).
      append(get_value({ values: [valtable_obj.BATTVOLT], index: 0, unit: ' V', precision: 0 })).appendTo(tr);
    }

    tr = $('<tr />').append($('<td />', { align: 'left' }).append(i18n(lang_Time) + ":")).appendTo(tbody);
    $('<td />', { id: 'batt_autotime_id' }).
    append(get_value({ values: [valtable_obj.AUTONOMTIME], index: 0, unit: ' Min', precision: 0 })).appendTo(tr);

    tr = $('<tr />').append($('<td />', { align: 'left' }).append(i18n(lang_Battery_Capacity_short) + ":")).appendTo(tbody);
    $('<td />', { id: 'batt_cap_id' }).
    append(get_value({ values: [valtable_obj.BATTCAP], index: 0, unit: ' %', precision: 0 })).appendTo(tr);

    if (valtable_obj.EX_BATT_SECONBATT != null) {
      tr = $('<tr />').append($('<td />', { align: 'left' }).append(i18n('Seconds') + ":")).appendTo(tbody);
      $('<td />', { id: 'batt_seconds_id' }).
      append(get_value({ values: [valtable_obj.EX_BATT_SECONBATT], index: 0, unit: ' Sec', precision: 0 })).appendTo(tr);
    }
    fragment.appendChild(batt_div[0]);
    document.body.appendChild(fragment);
  }

  function set_battery_value() //set Battery value
  {
    if ((valtable_obj.EX_BATT_VOLTPOS != null && valtable_obj.EX_BATT_VOLTNEG != null) &&
      (valtable_obj.EX_BATT_VOLTPOS != -9999 && valtable_obj.EX_BATT_VOLTNEG != -9999)) {
      $('#batt_volt_pos_id').text(get_value({ values: [valtable_obj.EX_BATT_VOLTPOS], index: 0, unit: ' V', precision: 1 }));
      $('#batt_volt_neg_id').text(get_value({ values: [valtable_obj.EX_BATT_VOLTNEG], index: 0, unit: ' V', precision: 1 }));
    } else
    $('#batt_volt_id').text(get_value({ values: [valtable_obj.BATTVOLT], index: 0, unit: ' V', precision: 0 }));

    $('#batt_autotime_id').text(get_value({ values: [valtable_obj.AUTONOMTIME], index: 0, unit: ' Min', precision: 0 }));
    $('#batt_cap_id').text(get_value({ values: [valtable_obj.BATTCAP], index: 0, unit: ' %', precision: 0 }));

    if (valtable_obj.EX_BATT_SECONBATT != null)
    $('#batt_seconds_id').text(get_value({ values: [valtable_obj.EX_BATT_SECONBATT], index: 0, unit: ' Sec', precision: 0 }));
  }

  function get_ph_1_3_vals(volt, cur, freq, phases) {
    var val_obj, val;
    l_1_3_vals = {
      availables: [0, 0, 0], //L1-L3
      val_objs: [
        { unit: ' V', precision: 0, values: ['', '', ''] },
        { unit: ' A', precision: 2, values: ['', '', ''] },
        { unit: ' Hz', precision: 2, values: ['', '', ''] }
      ]
    };
    volt = volt || [];
    cur = cur || [], freq = freq || [];
    $.each([volt, cur, freq], function (index, values) {
      val_obj = l_1_3_vals.val_objs[index];
      $.each(values, function (i, val) {
        if (i < phases) {
          val = values[i];
          if (val != -9999) {
            if (values === freq && (i == 1 || i == 2)) {

            } else
            l_1_3_vals.availables[i] = 1;

            if (Math.round(val) == val)
            val = val.toString();
            else
            val = val.toFixed(val_obj.precision);
            val += val_obj.unit;
            val_obj.values[i] = val;
          }
        }
      });
    });
    return l_1_3_vals;
  }

  function get_value(val_obj) {
    if (!val_obj.values || (val_obj.values.length <= val_obj.index))
    return 'N/A';
    var val = val_obj.values[val_obj.index];
    if (val == null || val_obj.values[val_obj.index] == -9999)
    return 'N/A';

    if (Math.round(val) == val)
    val = val.toString();
    else
    val = val.toFixed(val_obj.precision);
    val += val_obj.unit;
    return val;
  }

  function create_iby_ire_ssi_inv_rec_batt() //create IBY, IRE, SSI, INVERTER, RECTIFIER and BATTERY	pannes
  {
    var div, fragment = document.createDocumentFragment();
    css_obj = { position: 'absolute', top: y_pos + 150, left: x_pos + 588, width: 24, height: 10, overflow: 'hidden' };

    if ((status & 1) == 1) //bypass
    css_obj['background-color'] = 'rgb(40, 134, 59)';
    else if ((status & 16384) != 16384 && (status & 4) != 4) //IBY commlost and active
    css_obj['background-color'] = '#DC3632'; //red
    else
    css_obj['background-color'] = 'rgb(40, 134, 59)'; //'#52B245';
    div = $('<div/>').attr({ id: 'iby_div_id', 'class': 'img-rounded' }).css(css_obj);
    fragment.appendChild(div[0]);

    css_obj = { position: 'absolute', top: y_pos + 150, left: x_pos + 174, width: 24, height: 10, overflow: 'hidden' };

    if ((status & 1) == 1) //bypass
    css_obj['background-color'] = 'rgb(40, 134, 59)';
    else if ((status & 8) == 8 || (status & 128) == 128) //IRE powerfail or input-high
    css_obj['background-color'] = '#DC3632'; //red
    else
    css_obj['background-color'] = 'rgb(40, 134, 59)';
    div = $('<div/>').attr({ id: 'ire_div_id', 'class': 'img-rounded' }).css(css_obj);
    fragment.appendChild(div[0]);

    document.body.appendChild(fragment);

    if ((status & 8) == 8) //SSI powerfail
    panes_api.create_img_panel('ssi_img_id', './image/ups/sswgegr.gif', x_pos + 498, y_pos + 130, 50, 50, 0);
    else if ((status & 1) == 1) //bypass
    panes_api.create_img_panel('ssi_img_id', './image/ups/sswge.gif', x_pos + 498, y_pos + 130, 50, 50, 0);
    else
    panes_api.create_img_panel('ssi_img_id', './image/ups/sswgrge.gif', x_pos + 498, y_pos + 130, 50, 50, 0);

    if ((status & 512) == 512 //INVERTER inverterfail
      ||
      (status & 4096) == 4096 //inverterwarn
      ||
      (status & 256) == 256 //outputhigh
      ||
      (status & 32) == 32 //overtemp
      ||
      (overload == 1)) //overload
    panes_api.create_img_panel('inv_img_id', './image/ups/wrgr.gif', x_pos + 428, y_pos + 210, 50, 50, 0);

    else if ((level != 0) && ((tempdeg != null) && (tempdeg != -9999) && (tempmax != null)) && (tempdeg > tempmax)) //overtemp1
    panes_api.create_img_panel('inv_img_id', './image/ups/wrgr.gif', x_pos + 428, y_pos + 210, 50, 50, 0);

    else if ((status & 1) == 1) //bypass
    panes_api.create_img_panel('inv_img_id', './image/ups/wrge.gif', x_pos + 428, y_pos + 210, 50, 50, 0);
    else if ((status & 8) == 8) //powerfail
    panes_api.create_img_panel('inv_img_id', './image/ups/wrgr.gif', x_pos + 428, y_pos + 210, 50, 50, 0);
    else
    panes_api.create_img_panel('inv_img_id', './image/ups/wrge.gif', x_pos + 428, y_pos + 210, 50, 50, 0);

    if ((status & 1) == 1) //bypass
    panes_api.create_img_panel('rec_img_id', './image/ups/grge.gif', x_pos + 316, y_pos + 210, 50, 50, 0);
    else if ((status & 8) == 8) //RECTIFIER powerfail
    panes_api.create_img_panel('rec_img_id', './image/ups/grge.gif', x_pos + 316, y_pos + 210, 50, 50, 0);
    else
    panes_api.create_img_panel('rec_img_id', './image/ups/grgr.gif', x_pos + 316, y_pos + 210, 50, 50, 0);

    if ((valtable_obj.RAWSTATEA & 2) == 2 || (valtable_obj.RAWSTATEA & 65536) == 65536) //Battery Breaker Open and Battery Fail
    panes_api.create_img_panel('batt_img_id', './image/ups/Batrv.gif', x_pos + 372, y_pos + 294, 50, 50, 0);
    else if ((status & 1) == 1) //bypass
    panes_api.create_img_panel('batt_img_id', './image/ups/Batgev.gif', x_pos + 372, y_pos + 294, 50, 50, 0);
    else if ((status & 8) == 8) //BATTERY powerfail
    panes_api.create_img_panel('batt_img_id', './image/ups/Batgrv.gif', x_pos + 372, y_pos + 294, 50, 50, 0);
    else if ((status & 16) == 16 || (status & 1024) == 1024) // batterylow batterybad
    panes_api.create_img_panel('batt_img_id', './image/ups/Batrv.gif', x_pos + 372, y_pos + 294, 50, 50, 0);
    else
    panes_api.create_img_panel('batt_img_id', './image/ups/Batgev.gif', x_pos + 372, y_pos + 294, 50, 50, 0);
  }

  function set_iby_ire_ssi_inv_rec_batt() //SET IBY, IRE, SSI, INVERTER, RECTIFIER and BATTERY	pannes
  {

    if ((status & 1) == 1) //bypass
    $('#iby_div_id').css('background-color', 'rgb(40, 134, 59)'); //green
    else if ((status & 16384) != 16384 && (status & 4) != 4) //IBY commlost and active
    $('#iby_div_id').css('background-color', '#DC3632'); //red
    else
    $('#iby_div_id').css('background-color', 'rgb(40, 134, 59)'); //green

    if ((status & 1) == 1) //bypass
    $('#ire_div_id').css('background-color', 'rgb(40, 134, 59)'); //green
    else if ((status & 8) == 8 || (status & 128) == 128) //IRE powerfail or input-high
    $('#ire_div_id').css('background-color', '#DC3632'); //red
    else
    $('#ire_div_id').css('background-color', 'rgb(40, 134, 59)'); //green

    if ((status & 8) == 8) //SSI powerfail
    $('#ssi_img_id').attr('src', './image/ups/sswgegr.gif');
    else if ((status & 1) == 1) //bypass
    $('#ssi_img_id').attr('src', './image/ups/sswge.gif');
    else
    $('#ssi_img_id').attr('src', './image/ups/sswgrge.gif');

    if ((status & 512) == 512 //INVERTER inverterfail
      ||
      (status & 4096) == 4096 //inverterwarn
      ||
      (status & 256) == 256 //outputhigh
      ||
      (status & 32) == 32 //overtemp
      ||
      (overload == 1)) //overload
    $('#inv_img_id').attr('src', './image/ups/wrgr.gif');

    else if ((level != 0) && ((tempdeg != null) && (tempdeg != "-9999") && (tempmax != null)) && (tempdeg > tempmax)) //overtemp1
    $('#inv_img_id').attr('src', './image/ups/wrgr.gif');
    else if ((status & 1) == 1) //bypass
    $('#inv_img_id').attr('src', './image/ups/wrge.gif');
    else if ((status & 8) == 8) //powerfail
    $('#inv_img_id').attr('src', './image/ups/wrgr.gif');
    else
    $('#inv_img_id').attr('src', './image/ups/wrge.gif');

    if ((status & 1) == 1) //bypass
    $('#rec_img_id').attr('src', './image/ups/grge.gif');
    else if ((status & 8) == 8) //RECTIFIER powerfail
    $('#rec_img_id').attr('src', './image/ups/grge.gif');
    else
    $('#rec_img_id').attr('src', './image/ups/grgr.gif');

    if ((valtable_obj.RAWSTATEA & 2) == 2 || (valtable_obj.RAWSTATEA & 65536) == 65536) //Battery Breaker Open and Battery Fail
    $('#batt_img_id').attr('src', './image/ups/Batrv.gif');
    else if ((status & 1) == 1) //bypass
    $('#batt_img_id').attr('src', './image/ups/Batgev.gif');
    else if ((status & 8) == 8) //BATTERY powerfail
    $('#batt_img_id').attr('src', './image/ups/Batgrv.gif');
    else if ((status & 16) == 16 || (status & 1024) == 1024) // batterylow batterybad
    $('#batt_img_id').attr('src', './image/ups/Batrv.gif');
    else
    $('#batt_img_id').attr('src', './image/ups/Batgev.gif');
  }

  function create_status_pane() //UPS status
  {
    var status_div = $('<div/>');
    css_obj = { position: 'absolute', top: y_pos + 44, left: x_pos + 20, align: 'center', overflow: 'hidden' };
    $(status_div).css(css_obj);
    $('<h4 />').append($('<span />', { id: 'ups_status_id', 'class': get_status_class() }).append(i18n('Status'))).appendTo(status_div);
    status_div.appendTo(document.body);
  }

  function set_ups_status() //set UPS status
  {
    $('#ups_status_id').attr('class', get_status_class());
  }

  function get_status_class() {
    if ((valtable_obj.RAWSTATEA & 2) == 2) //Battery Breaker Open
    return 'label label-danger';
    if (status == 4)
    return 'label label-success';
    else
    return 'label label-danger';
  }

  function set_outpower_loadbars() //outpower loadbars
  {
    if (valtable_obj.OUTPOWER == null)
    return true;
    if (valtable_obj.OUTPOWER[0] != -9999)
    panes_api.set_loabar_value(valtable_obj.OUTPOWER[0], 'loadbar1_id');
    if ((outphases > 1) && valtable_obj.OUTPOWER[1] != -9999)
    panes_api.set_loabar_value(valtable_obj.OUTPOWER[1], 'loadbar2_id');
    if ((outphases > 2) && valtable_obj.OUTPOWER[2] != -9999)
    panes_api.set_loabar_value(valtable_obj.OUTPOWER[2], 'loadbar3_id');
  }

  function create_outpower_loadbars() //outpower loadbars
  {
    if (valtable_obj.OUTPOWER == null)
    return true;
    outpower_loadbars_created = true;

    if (valtable_obj.OUTPOWER[0] != -9999.00)
    panes_api.create_loadbar_pane({ value: valtable_obj.OUTPOWER[0], id: 'loadbar1_id', left: x_pos + 564, top: y_pos + 336, title: 'L1', scaler: 1 });
    if ((outphases > 1) && valtable_obj.OUTPOWER[1] != -9999.00)
    panes_api.create_loadbar_pane({ value: valtable_obj.OUTPOWER[1], id: 'loadbar2_id', left: x_pos + 564, top: y_pos + 366, title: 'L2', scaler: 0 });
    if ((outphases > 2) && valtable_obj.OUTPOWER[2] != -9999.00)
    panes_api.create_loadbar_pane({ value: valtable_obj.OUTPOWER[2], id: 'loadbar3_id', left: x_pos + 564, top: y_pos + 396, title: 'L3', scaler: 0 });
  }

  function create_rom_temp() //rom temp
  {
    var fragment = document.createDocumentFragment(),
    rom_temp_div;
    css_obj = { position: 'absolute', top: y_pos + 316, left: x_pos + 8 };
    rom_temp_div = $('<div/>').attr({ id: 'rom_temp_div_id' }).css(css_obj);
    panes_api.create_rom_temp_pane(valtable_obj, { id: 'rom_temp_table_id', 'class': 'table table-condensed table-hover' }).appendTo(rom_temp_div);
    fragment.appendChild(rom_temp_div[0]);
    document.body.appendChild(fragment);
  }

}(jQuery));