<?xml version="1.0"?>
<!--
   mega7_panel.xsl - Mitsubishi 1100 A/B screen.
   Copyright (c) 2018, Generex GmbH. All rights reserved.
   uw
-->
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform" version="1.0">
  <!-- Methods -->
  <!--Header-->
  <xsl:template name="mega7_panel-header">
    <xsl:call-template name="app-header">
      <xsl:with-param name="top" select="3"/>
    </xsl:call-template>
  </xsl:template>

  <!--Footer-->
  <xsl:template name="mega7_panel-footer" >
    <xsl:call-template name="app-footer" >
      <xsl:with-param name="left" select="24"/>
      <xsl:with-param name="top" select="510"/>
      <xsl:with-param name="width" select="752"/>
      <xsl:with-param name="height" select="62"/>
    </xsl:call-template>
  </xsl:template>
  <!--Menu-->
  <xsl:template name="mega7_panel-menu">
    <xsl:call-template name="app-menu">
      <xsl:with-param name="top" select="428"/>
      <xsl:with-param name="left" select="0"/>
      <xsl:with-param name="width" select="0"/>
      <xsl:with-param name="height" select="0"/>
      <xsl:with-param name="class" select="'XBold_black'"/>
    </xsl:call-template>
  </xsl:template>

  <!--Body-->
  <xsl:template name="mega7_panel-body">
    <xsl:call-template name="IEpixelBugfixCall"/>
    <xsl:call-template name="js_refresh_call">
      <xsl:with-param    name="timeout" select="1000*//report/@refresh"/>
    </xsl:call-template>
    <xsl:apply-templates />
  </xsl:template>

  <!-- Implementation -->

  <xsl:template match="values">
    <xsl:variable name="phases" select="value[@name='PHASES']/@val"/>
    <!-- Out phases -->
    <xsl:variable name="outphases">
      <xsl:choose>
        <xsl:when test="string-length(value[@name='OUTPHASES']/@val)!=0">
          <xsl:value-of select="value[@name='OUTPHASES']/@val" />
        </xsl:when>
        <xsl:otherwise>
          <xsl:value-of select="$phases" />
        </xsl:otherwise>
      </xsl:choose>
    </xsl:variable>
    <!-- In phases -->
    <xsl:variable name="inphases">
      <xsl:choose>
        <xsl:when test="string-length(value[@name='INPHASES']/@val)!=0">
          <xsl:value-of select="value[@name='INPHASES']/@val" />
        </xsl:when>
        <xsl:otherwise>
          <xsl:value-of select="$phases" />
        </xsl:otherwise>
      </xsl:choose>
    </xsl:variable>

    <!-- STATUS -->
    <xsl:variable name="state" select="value[@name='STATUS']/@val"/>
    <!-- Extended STATUS -->
    <xsl:variable name="ext_state" select="value[@name='EX_EXTSTATUS']/@val"/>

    <!-- Bits masked Status-->
    <xsl:variable name="bypass">
      <xsl:call-template name="bit">
        <xsl:with-param name="val" select="$state"/>
        <xsl:with-param name="bit" select="$bit0"/>
      </xsl:call-template>
    </xsl:variable>
    <xsl:variable name="powerfail">
      <xsl:call-template name="bit">
        <xsl:with-param name="val" select="$state"/>
        <xsl:with-param name="bit" select="$bit3"/>
      </xsl:call-template>
    </xsl:variable>
    <xsl:variable name="testact">
      <xsl:call-template name="bit">
        <xsl:with-param name="val" select="$state"/>
        <xsl:with-param name="bit" select="$bit6"/>
      </xsl:call-template>
    </xsl:variable>

    <!-- Bits masked Ext. Status-->
    <xsl:variable name="over_temperature">
      <xsl:call-template name="bit">
        <xsl:with-param name="val" select="$ext_state"/>
        <xsl:with-param name="bit" select="$bit4"/>
      </xsl:call-template>
    </xsl:variable>
    <xsl:variable name="input_bad">
      <xsl:call-template name="bit">
        <xsl:with-param name="val" select="$ext_state"/>
        <xsl:with-param name="bit" select="$bit5"/>
      </xsl:call-template>
    </xsl:variable>
    <xsl:variable name="output_bad">
      <xsl:call-template name="bit">
        <xsl:with-param name="val" select="$ext_state"/>
        <xsl:with-param name="bit" select="$bit6"/>
      </xsl:call-template>
    </xsl:variable>
    <xsl:variable name="over_load">
      <xsl:call-template name="bit">
        <xsl:with-param name="val" select="$ext_state"/>
        <xsl:with-param name="bit" select="$bit7"/>
      </xsl:call-template>
    </xsl:variable>
    <xsl:variable name="sys_shutdown">
      <xsl:call-template name="bit">
        <xsl:with-param name="val" select="$ext_state"/>
        <xsl:with-param name="bit" select="$bit11"/>
      </xsl:call-template>
    </xsl:variable>
    <xsl:variable name="output_off">
      <xsl:call-template name="bit">
        <xsl:with-param name="val" select="$ext_state"/>
        <xsl:with-param name="bit" select="$bit13"/>
      </xsl:call-template>
    </xsl:variable>
    <xsl:variable name="charger_failure">
      <xsl:call-template name="bit">
        <xsl:with-param name="val" select="$ext_state"/>
        <xsl:with-param name="bit" select="$bit12"/>
      </xsl:call-template>
    </xsl:variable>
    <xsl:variable name="ups_sys_off">
      <xsl:call-template name="bit">
        <xsl:with-param name="val" select="$ext_state"/>
        <xsl:with-param name="bit" select="$bit14"/>
      </xsl:call-template>
    </xsl:variable>
    <xsl:variable name="fuse_failure">
      <xsl:call-template name="bit">
        <xsl:with-param name="val" select="$ext_state"/>
        <xsl:with-param name="bit" select="$bit16"/>
      </xsl:call-template>
    </xsl:variable>
    <xsl:variable name="general_fault">
      <xsl:call-template name="bit">
        <xsl:with-param name="val" select="$ext_state"/>
        <xsl:with-param name="bit" select="$bit17"/>
      </xsl:call-template>
    </xsl:variable>
    <xsl:variable name="shutdown_imminent">
      <xsl:call-template name="bit">
        <xsl:with-param name="val" select="$ext_state"/>
        <xsl:with-param name="bit" select="$bit22"/>
      </xsl:call-template>
    </xsl:variable>
    <xsl:variable name="bypass_bad">
      <xsl:call-template name="bit">
        <xsl:with-param name="val" select="$ext_state"/>
        <xsl:with-param name="bit" select="$bit9"/>
      </xsl:call-template>
    </xsl:variable>
    <xsl:variable name="ups_awaiting_power">
      <xsl:call-template name="bit">
        <xsl:with-param name="val" select="$ext_state"/>
        <xsl:with-param name="bit" select="$bit20"/>
      </xsl:call-template>
    </xsl:variable>

    <xsl:variable name="bg_img">
      <xsl:choose>
        <xsl:when test="$powerfail=1" >
          <xsl:value-of select="'./lib/ups/image/mega/eon_bb.png'" />
        </xsl:when>

        <xsl:when test="$bypass=1" >
          <xsl:value-of select="'./lib/ups/image/mega/eon_byp.png'" />
        </xsl:when>

        <xsl:when test="$testact=1" >
          <xsl:value-of select="'./lib/ups/image/mega/eon_tb.png'" />
        </xsl:when>
        <xsl:when test="$state=4" >
          <xsl:value-of select="'./lib/ups/image/mega/eon_nb.png'" />
        </xsl:when>
        <xsl:otherwise>
          <xsl:value-of select="'./lib/ups/image/mega/eon_3_4.png'" />
        </xsl:otherwise>
      </xsl:choose>
    </xsl:variable>

    <xsl:call-template name="background">
      <xsl:with-param name="imagepath" select="$bg_img"/>
      <xsl:with-param name="x" select="'10'"/>
      <xsl:with-param name="y" select="'10'"/>
      <xsl:with-param name="w" select="'792'"/>
      <xsl:with-param name="h" select="'490'"/>
      <xsl:with-param name="border" select="'1'"/>
      <xsl:with-param name="id" select="'background'"/>
    </xsl:call-template>

    <!-- MODEL -->
    <div class="XXLargeBold" style="position: absolute; left: 270px; top:40px; width: 263px; height: 17px; text-align:center">
      <xsl:value-of select="value[@name='MODEL']/@val" />
    </div>

    <xsl:variable name="outpower" select="value[@name='OUTPOWER']/@val"/>
    <!-- UPS Protocol -->
    <xsl:variable name="upsProtocol">
      <xsl:choose>
        <xsl:when test="string-length(value[@name='_upsProtocol']/@val)!=0">
          <xsl:value-of select="value[@name='_upsProtocol']/@val" />
        </xsl:when>
        <xsl:otherwise>
          <xsl:call-template name="unsigner">
            <xsl:with-param name="val" select="value[@name='CONNECT']/@val"/>
            <xsl:with-param name="bits" select="16"/>
          </xsl:call-template>
          <!--<xsl:value-of select="value[@name='CONNECT']/@val" />-->
        </xsl:otherwise>
      </xsl:choose>
    </xsl:variable>

    <xsl:variable name="overload">
      <xsl:choose>
        <xsl:when test="$outphases=3">
          <xsl:variable name="outpower0" select="value[@name='OUTPOWER']/@val"/>
          <xsl:variable name="outpower1" select="value[@name='OUTPOWER']/@val2"/>
          <xsl:variable name="outpower2" select="value[@name='OUTPOWER']/@val3"/>
          <xsl:choose>
            <xsl:when test="$outpower0>100 or $outpower1>100 or $outpower2>100">1</xsl:when>
            <xsl:otherwise>0</xsl:otherwise>
          </xsl:choose>
        </xsl:when>
        <xsl:otherwise>
          <xsl:choose>
            <xsl:when test="$outpower>100">1</xsl:when>
            <xsl:otherwise>0</xsl:otherwise>
          </xsl:choose>
        </xsl:otherwise>
      </xsl:choose>
    </xsl:variable>

    <xsl:variable name="tempdeg">
      <xsl:choose>
        <xsl:when test="value[@name='LEVEL']/@val = 0 or value[@name='TEMPDEG'] = -1">
          <xsl:value-of select="'N/A'"/>
        </xsl:when>
        <xsl:otherwise>
          <xsl:call-template name="round-decimal">
            <xsl:with-param  name="value" select="value[@name='TEMPDEG']/@val"/>
            <xsl:with-param name="precision" select="1"/>
          </xsl:call-template>
        </xsl:otherwise>
      </xsl:choose>
    </xsl:variable>
    <xsl:variable name="battvolt">
      <xsl:choose>
        <xsl:when test="value[@name='LEVEL']/@val = 0">
          <xsl:value-of select="'N/A'"/>
        </xsl:when>
        <xsl:otherwise>
          <xsl:call-template name="round-decimal">
            <xsl:with-param  name="value" select="value[@name='BATTVOLT']/@val"/>
            <xsl:with-param name="precision" select="2"/>
          </xsl:call-template>
        </xsl:otherwise>
      </xsl:choose>
    </xsl:variable>

    <xsl:variable name="tempdeg-unit">
      <xsl:choose>
        <xsl:when test="value[@name='LEVEL']/@val = 0"></xsl:when>
        <xsl:otherwise>&#176;C </xsl:otherwise>
      </xsl:choose>
    </xsl:variable>

    <xsl:variable name="tempmax" select="value[@name='TMPMAX']/@val"/>
    <xsl:variable name="overtemp1">
      <xsl:choose>
        <xsl:when test="value[@name='LEVEL']/@val != 0 and $tempdeg>$tempmax">1</xsl:when>
        <xsl:otherwise>0</xsl:otherwise>
      </xsl:choose>
    </xsl:variable>

    <!-- COMMON ALARM -->
    <xsl:variable name="alarm_status">
      <xsl:choose>
        <xsl:when test="4!=$state and 1!=$testact">
          <xsl:value-of select="1" />
        </xsl:when>
        <!--Status-->
        <xsl:when test="$over_temperature=1">
          <xsl:value-of select="1" />
        </xsl:when>
        <!--Ext. Status-->
        <xsl:when test="$input_bad=1">
          <xsl:value-of select="1" />
        </xsl:when>
        <xsl:when test="$output_bad=1">
          <xsl:value-of select="1" />
        </xsl:when>
        <xsl:when test="$output_off=1">
          <xsl:value-of select="1" />
        </xsl:when>
        <xsl:when test="$charger_failure=1">
          <xsl:value-of select="1" />
        </xsl:when>
        <xsl:when test="$fuse_failure=1">
          <xsl:value-of select="1" />
        </xsl:when>
        <xsl:when test="$general_fault=1">
          <xsl:value-of select="1" />
        </xsl:when>
        <xsl:when test="$shutdown_imminent=1">
          <xsl:value-of select="1" />
        </xsl:when>
        <xsl:when test="$bypass_bad=1">
          <xsl:value-of select="1" />
        </xsl:when>
        <xsl:when test="$ups_sys_off=1">
          <xsl:value-of select="1" />
        </xsl:when>
        <xsl:when test="$ups_awaiting_power=1">
          <xsl:value-of select="1" />
        </xsl:when>
        <xsl:when test="$sys_shutdown=1">
          <xsl:value-of select="1" />
        </xsl:when>
        <xsl:otherwise>
          <xsl:value-of select="0" />
        </xsl:otherwise>
      </xsl:choose>
    </xsl:variable>
    <xsl:if test="$alarm_status=1">
      <div id="alarm_txt_id" style="position: absolute; left: 48px; top: 400px; width: 33px; height: 12px; color:#3341AA; font-weight:bold; font-size:11pt">
         <script type="text/javascript"> translate_text('alarm_txt_id', 'Alarm');</script>
      </div>
      <div style="position: absolute; background-color:#DD0000; left: 48px; top: 416px; width: 42px; height: 15px">
        <!--<img border="1" src="./lib/ups/image/ups/led33x12red.gif" />
      -->
      </div>
    </xsl:if>

    <!-- Ticket #3380 -->
    <xsl:variable name="wattunit">
      <xsl:value-of select="'kW'"/>
    </xsl:variable>
    <!--
      <xsl:choose>
        <xsl:when test="$upsProtocol = '8106' or $upsProtocol = '33030' or $upsProtocol = '8115' or $upsProtocol = '33045'">
          <xsl:value-of select="'kW'"/>
        </xsl:when>
        <xsl:otherwise>
          <xsl:value-of select="'W'"/>
        </xsl:otherwise>
      </xsl:choose>
    </xsl:variable>
  -->
    <!-- end Ticket #3380 -->

    <!-- Input -->
    <xsl:variable name="inp_volt1">
      <xsl:call-template name="round-decimal">
        <xsl:with-param name="value" select="value[@name='INVOLT']/@val"/>
        <xsl:with-param name="precision" select="0" />
      </xsl:call-template>
    </xsl:variable>
    <xsl:variable name="inp_volt2">
      <xsl:call-template name="round-decimal">
        <xsl:with-param name="value" select="value[@name='INVOLT']/@val2"/>
        <xsl:with-param name="precision" select="0" />
      </xsl:call-template>
    </xsl:variable>
    <xsl:variable name="inp_volt3">
      <xsl:call-template name="round-decimal">
        <xsl:with-param name="value" select="value[@name='INVOLT']/@val3"/>
        <xsl:with-param name="precision" select="0" />
      </xsl:call-template>
    </xsl:variable>

    <xsl:variable name="inp_curr1">
      <xsl:choose>
        <xsl:when test="value[@name='INCURR']/@val &gt; -9999">
          <xsl:call-template name="round-decimal">
            <xsl:with-param name="value" select="value[@name='INCURR']/@val"/>
            <xsl:with-param name="precision" select="0" />
          </xsl:call-template>
        </xsl:when>
        <xsl:otherwise>
          <xsl:value-of select="''" />
        </xsl:otherwise>
      </xsl:choose>
    </xsl:variable>
    <xsl:variable name="inp_curr2">
      <xsl:choose>
        <xsl:when test="value[@name='INCURR']/@val2 &gt; -9999">
          <xsl:call-template name="round-decimal">
            <xsl:with-param name="value" select="value[@name='INCURR']/@val2"/>
            <xsl:with-param name="precision" select="0" />
          </xsl:call-template>
        </xsl:when>
        <xsl:otherwise>
          <xsl:value-of select="''" />
        </xsl:otherwise>
      </xsl:choose>
    </xsl:variable>
    <xsl:variable name="inp_curr3">
      <xsl:choose>
        <xsl:when test="value[@name='INCURR']/@val3 &gt; -9999">
          <xsl:call-template name="round-decimal">
            <xsl:with-param name="value" select="value[@name='INCURR']/@val3"/>
            <xsl:with-param name="precision" select="0" />
          </xsl:call-template>
        </xsl:when>
        <xsl:otherwise>
          <xsl:value-of select="''" />
        </xsl:otherwise>
      </xsl:choose>
    </xsl:variable>

    <xsl:variable name="inp_pow1">
      <xsl:choose>
        <xsl:when test="value[@name='EX_INP_WATT']/@val &gt; -9999">
          <xsl:call-template name="round-decimal">
            <xsl:with-param name="value" select="value[@name='EX_INP_WATT']/@val"/>
            <xsl:with-param name="precision" select="2" />
          </xsl:call-template>
        </xsl:when>
        <xsl:otherwise>
          <xsl:value-of select="''" />
        </xsl:otherwise>
      </xsl:choose>
    </xsl:variable>
    <xsl:variable name="inp_pow2">
      <xsl:choose>
        <xsl:when test="value[@name='EX_INP_WATT']/@val2 &gt; -9999">
          <xsl:call-template name="round-decimal">
            <xsl:with-param name="value" select="value[@name='EX_INP_WATT']/@val2"/>
            <xsl:with-param name="precision" select="2" />
          </xsl:call-template>
        </xsl:when>
        <xsl:otherwise>
          <xsl:value-of select="''" />
        </xsl:otherwise>
      </xsl:choose>
    </xsl:variable>
    <xsl:variable name="inp_pow3">
      <xsl:choose>
        <xsl:when test="value[@name='EX_INP_WATT']/@val3 &gt; -9999">
          <xsl:call-template name="round-decimal">
            <xsl:with-param name="value" select="value[@name='EX_INP_WATT']/@val3"/>
            <xsl:with-param name="precision" select="2" />
          </xsl:call-template>
        </xsl:when>
        <xsl:otherwise>
          <xsl:value-of select="''" />
        </xsl:otherwise>
      </xsl:choose>
    </xsl:variable>

    <xsl:variable name="inp_freq1">
      <xsl:choose>
        <xsl:when test="value[@name='INFREQ']/@val &gt; -9999">
          <xsl:call-template name="round-decimal">
            <xsl:with-param name="value" select="value[@name='INFREQ']/@val"/>
            <xsl:with-param name="precision" select="1" />
          </xsl:call-template>
        </xsl:when>
        <xsl:otherwise>
          <xsl:value-of select="''" />
        </xsl:otherwise>
      </xsl:choose>
    </xsl:variable>
    <xsl:variable name="inp_freq2">
      <xsl:choose>
        <xsl:when test="value[@name='INFREQ']/@val2 &gt; -9999">
          <xsl:call-template name="round-decimal">
            <xsl:with-param name="value" select="value[@name='INFREQ']/@val2"/>
            <xsl:with-param name="precision" select="1" />
          </xsl:call-template>
        </xsl:when>
        <xsl:otherwise>
          <xsl:value-of select="''" />
        </xsl:otherwise>
      </xsl:choose>
    </xsl:variable>
    <xsl:variable name="inp_freq3">
      <xsl:choose>
        <xsl:when test="value[@name='INFREQ']/@val3 &gt; -9999">
          <xsl:call-template name="round-decimal">
            <xsl:with-param name="value" select="value[@name='INFREQ']/@val3"/>
            <xsl:with-param name="precision" select="1" />
          </xsl:call-template>
        </xsl:when>
        <xsl:otherwise>
          <xsl:value-of select="''" />
        </xsl:otherwise>
      </xsl:choose>
    </xsl:variable>


    <xsl:variable name="inp_volta1">
      <xsl:choose>
        <xsl:when test="value[@name='INCURR']/@val &gt; -9999">
          <xsl:call-template name="round-decimal">
            <xsl:with-param name="value" select="(value[@name='INVOLT']/@val * value[@name='INCURR']/@val) div 1000.0"/>
            <xsl:with-param name="precision" select="2" />
          </xsl:call-template>
        </xsl:when>
        <xsl:otherwise>
          <xsl:value-of select="''" />
        </xsl:otherwise>
      </xsl:choose>
    </xsl:variable>
    <xsl:variable name="inp_volta2">
      <xsl:choose>
        <xsl:when test="value[@name='INCURR']/@val2 &gt; -9999">
          <xsl:call-template name="round-decimal">
            <xsl:with-param name="value" select="(value[@name='INVOLT']/@val2 * value[@name='INCURR']/@val2) div 1000.0"/>
            <xsl:with-param name="precision" select="2" />
          </xsl:call-template>
        </xsl:when>
        <xsl:otherwise>
          <xsl:value-of select="''" />
        </xsl:otherwise>
      </xsl:choose>
    </xsl:variable>
    <xsl:variable name="inp_volta3">
      <xsl:choose>
        <xsl:when test="value[@name='INCURR']/@val3 &gt; -9999">
          <xsl:call-template name="round-decimal">
            <xsl:with-param name="value" select="(value[@name='INVOLT']/@val3 * value[@name='INCURR']/@val3) div 1000.0"/>
            <xsl:with-param name="precision" select="2" />
          </xsl:call-template>
        </xsl:when>
        <xsl:otherwise>
          <xsl:value-of select="''" />
        </xsl:otherwise>
      </xsl:choose>
    </xsl:variable>

    <!--Input-->
    <div id="Input_id" style="position:absolute;left:48px;top:256px;width:260px;height:10;align:center;text-align:left;color:#3341AA; font-weight:bold; font-size:11pt">
      <script type="text/javascript"> translate_text('Input_id', 'Input');</script>
    </div>
    <div style="position:absolute;left:40px;top:276px;width:260px;height:76;align:center;">
      <table cellpadding="0" cellspacing="4" style="color:black;font-family: arial;font-weight:bold;font-size: 11px;margin-left:6;border: 1px solid #405080">
        <tr>
          <td id="Input_freq" align="left">
            <script type="text/javascript"> translate_text('Input_freq', ['Frequency', ':']);</script>
          </td>
          <td align = "left">
            <xsl:if test="string-length($inp_freq1)!=0">
              <xsl:value-of select="concat($inp_freq1,'&#160;Hz')" />
            </xsl:if>
          </td>
          <td align = "left">
            <xsl:if test="$upsProtocol != '8106' and $upsProtocol != '33030' and $upsProtocol != '8115' and $upsProtocol != '33045'">
              <xsl:if test="string-length($inp_freq2)!=0">
                <xsl:value-of select="concat($inp_freq2,'&#160;Hz')" />
              </xsl:if>
            </xsl:if>
          </td>
          <td align = "left">
            <xsl:if test="$upsProtocol != '8106' and $upsProtocol != '33030' and $upsProtocol != '8115' and $upsProtocol != '33045'">
              <xsl:if test="string-length($inp_freq3)!=0">
                <xsl:value-of select="concat($inp_freq3,'&#160;Hz')" />
              </xsl:if>
            </xsl:if>
          </td>
        </tr>
        <tr>
          <td id="Input_volt" align="left">
            <script type="text/javascript"> translate_text('Input_volt', ['Voltage', ':']);</script>
          </td>
          <td align = "left">
            <xsl:value-of select="concat($inp_volt1,'&#160;V')" />
          </td>
          <xsl:if test="value[@name='EX_INP_NUMLINES']/@val &gt; 1">
            <td align = "left" >
              <xsl:value-of select="concat($inp_volt2,'&#160;V')" />
            </td>
          </xsl:if>
          <xsl:if test="value[@name='EX_INP_NUMLINES']/@val &gt; 2">
            <td align = "left">
              <xsl:value-of select="concat($inp_volt3,'&#160;V')" />
            </td>
          </xsl:if>
        </tr>

        <xsl:if test="string-length($inp_curr1) != 0 or string-length($inp_curr2) != 0 or string-length($inp_curr3) != 0">
          <tr>
            <td id="Input_curr" align="left">
              <script type="text/javascript"> translate_text('Input_curr', ['Current', ':']);</script>
            </td>
            <td align = "left">
              <xsl:if test="string-length($inp_curr1) != 0">
                <xsl:value-of select="concat($inp_curr1,'&#160;A')" />
              </xsl:if>
            </td>
            <xsl:if test="value[@name='EX_INP_NUMLINES']/@val &gt; 1">
              <td align = "left" >
                <xsl:if test="string-length($inp_curr2) != 0">
                  <xsl:value-of select="concat($inp_curr2,'&#160;A')" />
                </xsl:if>
              </td>
            </xsl:if>
            <xsl:if test="value[@name='EX_INP_NUMLINES']/@val &gt; 2">
              <td align = "left">
                <xsl:if test="string-length($inp_curr2) != 0">
                  <xsl:value-of select="concat($inp_curr3,'&#160;A')" />
                </xsl:if>
              </td>
            </xsl:if>
          </tr>
        </xsl:if>

        <xsl:if test="string-length($inp_pow1) != 0 or string-length($inp_pow2) != 0 or string-length($inp_pow3) != 0">
          <tr>
            <td id="Input_pow" align="left">
              <script type="text/javascript"> translate_text('Input_pow', ['Power', ':']);</script>
            </td>
            <td align = "left">
              <xsl:if test="string-length($inp_pow1) != 0">
                <xsl:value-of select="concat(concat($inp_pow1,'&#160;'), $wattunit)" />
              </xsl:if>
            </td>
            <xsl:if test="value[@name='EX_INP_NUMLINES']/@val &gt; 1">
              <td align = "left" >
                <xsl:if test="string-length($inp_pow2) != 0">
                  <xsl:if test="$upsProtocol != '8106' and $upsProtocol != '33030' and $upsProtocol != '8115' and $upsProtocol != '33045'">
                    <xsl:value-of select="concat(concat($inp_pow2,'&#160;'), $wattunit)" />
                  </xsl:if>
                </xsl:if>
              </td>
            </xsl:if>
            <xsl:if test="value[@name='EX_INP_NUMLINES']/@val &gt; 2">
              <td align = "left">
                <xsl:if test="string-length($inp_pow3) != 0">
                  <xsl:if test="$upsProtocol != '8106' and $upsProtocol != '33030' and $upsProtocol != '8115' and $upsProtocol != '33045'">
                    <xsl:value-of select="concat(concat($inp_pow3,'&#160;'), $wattunit)" />
                  </xsl:if>
                </xsl:if>
              </td>
            </xsl:if>
          </tr>
        </xsl:if>

        <xsl:if test="string-length($inp_volta1) != 0 or string-length($inp_volta2) != 0 or string-length($inp_volta3) != 0">
          <tr>
            <td align="left">
              <xsl:value-of select="'Inp.&#160;kVA:'" />
            </td>
            <td align = "left">
              <xsl:if test="string-length($inp_volta1) != 0">
                <xsl:value-of select="concat($inp_volta1,'&#160;kVA')" />
              </xsl:if>
            </td>
            <xsl:if test="value[@name='EX_INP_NUMLINES']/@val &gt; 1">
              <td align = "left">
                <xsl:if test="string-length($inp_volta2) != 0">
                  <xsl:value-of select="concat($inp_volta2,'&#160;kVA')" />
                </xsl:if>
              </td>
            </xsl:if>
            <xsl:if test="value[@name='EX_INP_NUMLINES']/@val &gt; 2">
              <td align = "left">
                <xsl:if test="string-length($inp_volta3) != 0">
                  <xsl:value-of select="concat($inp_volta3,'&#160;kVA')" />
                </xsl:if>
              </td>
            </xsl:if>
          </tr>
        </xsl:if>
      </table>
    </div>

    <!--Bypass-->
    <xsl:variable name="byp_freq1">
      <xsl:choose>
        <xsl:when test="value[@name='EX_BYP_FREQ']/@val &gt; -9999">
          <xsl:call-template name="round-decimal">
            <xsl:with-param name="value" select="value[@name='EX_BYP_FREQ']/@val"/>
            <xsl:with-param name="precision" select="1" />
          </xsl:call-template>
        </xsl:when>
        <xsl:otherwise>
          <xsl:value-of select="''" />
        </xsl:otherwise>
      </xsl:choose>
    </xsl:variable>

    <xsl:variable name="byp_volt1">
      <xsl:choose>
        <xsl:when test="value[@name='EX_BYP_VOLT']/@val &gt; -9999">
         <xsl:call-template name="round-decimal">
            <xsl:with-param name="value" select="value[@name='EX_BYP_VOLT']/@val"/>
            <xsl:with-param name="precision" select="0" />
         </xsl:call-template>
        </xsl:when>
        <xsl:otherwise>
          <xsl:value-of select="''" />
        </xsl:otherwise>
      </xsl:choose>
    </xsl:variable>
    <xsl:variable name="byp_volt2">
      <xsl:choose>
        <xsl:when test="value[@name='EX_BYP_VOLT']/@val2 &gt; -9999">
         <xsl:call-template name="round-decimal">
            <xsl:with-param name="value" select="value[@name='EX_BYP_VOLT']/@val2"/>
            <xsl:with-param name="precision" select="0" />
         </xsl:call-template>
        </xsl:when>
        <xsl:otherwise>
          <xsl:value-of select="''" />
        </xsl:otherwise>
      </xsl:choose>
    </xsl:variable>
    <xsl:variable name="byp_volt3">
      <xsl:choose>
        <xsl:when test="value[@name='EX_BYP_VOLT']/@val3 &gt; -9999">
         <xsl:call-template name="round-decimal">
           <xsl:with-param name="value" select="value[@name='EX_BYP_VOLT']/@val3"/>
           <xsl:with-param name="precision" select="0" />
         </xsl:call-template>
        </xsl:when>
        <xsl:otherwise>
          <xsl:value-of select="''" />
        </xsl:otherwise>
      </xsl:choose>
    </xsl:variable>

    <xsl:variable name="byp_curr1">
      <xsl:choose>
        <xsl:when test="value[@name='EX_BYP_CURR']/@val &gt; -9999">
          <xsl:call-template name="round-decimal">
            <xsl:with-param name="value" select="value[@name='EX_BYP_CURR']/@val"/>
            <xsl:with-param name="precision" select="0" />
          </xsl:call-template>
        </xsl:when>
        <xsl:otherwise>
          <xsl:value-of select="''" />
        </xsl:otherwise>
      </xsl:choose>
    </xsl:variable>
    <xsl:variable name="byp_curr2">
      <xsl:choose>
        <xsl:when test="value[@name='EX_BYP_CURR']/@val2 &gt; -9999">
          <xsl:call-template name="round-decimal">
            <xsl:with-param name="value" select="value[@name='EX_BYP_CURR']/@val2"/>
            <xsl:with-param name="precision" select="0" />
          </xsl:call-template>
        </xsl:when>
        <xsl:otherwise>
          <xsl:value-of select="''" />
        </xsl:otherwise>
      </xsl:choose>
    </xsl:variable>
    <xsl:variable name="byp_curr3">
      <xsl:choose>
        <xsl:when test="value[@name='EX_BYP_CURR']/@val3 &gt; -9999">
          <xsl:call-template name="round-decimal">
            <xsl:with-param name="value" select="value[@name='EX_BYP_CURR']/@val3"/>
            <xsl:with-param name="precision" select="0" />
          </xsl:call-template>
        </xsl:when>
        <xsl:otherwise>
          <xsl:value-of select="''" />
        </xsl:otherwise>
      </xsl:choose>
    </xsl:variable>

    <xsl:variable name="byp_pow1">
      <xsl:choose>
        <xsl:when test="value[@name='EX_BYP_WATT']/@val &gt; -9999">
          <xsl:call-template name="round-decimal">
            <xsl:with-param name="value" select="(value[@name='EX_BYP_WATT']/@val) div 10.0"/>
            <xsl:with-param name="precision" select="1" />
          </xsl:call-template>
        </xsl:when>
        <xsl:otherwise>
          <xsl:value-of select="''" />
        </xsl:otherwise>
      </xsl:choose>
    </xsl:variable>

    <!--Bypass-->
    <div id="Bypass_id" style="position:absolute;left:528px;top:60px;width:260px;height:10;align:center;text-align:left;color:#3341AA; font-weight:bold; font-size:11pt">
      <script type="text/javascript"> translate_text('Bypass_id', 'Bypass');</script>
    </div>
    <div style="position:absolute;left:520px;top:80px;width:260px;height:76;align:center;">
      <table cellpadding="0" cellspacing="4" style="color:black;font-family: arial;font-weight:bold;font-size: 11px;margin-left:6;border: 1px solid #405080">
        <tr>
          <td id="Bypass_freq" align="left">
            <script type="text/javascript"> translate_text('Bypass_freq', ['Frequency', ':']);</script>
          </td>
          <td align = "left">
            <xsl:if test="string-length($byp_freq1)!=0">
              <xsl:value-of select="concat($inp_freq1,'&#160;Hz')" />
            </xsl:if>
          </td>
        </tr>
        <tr>
          <td id="Bypass_volt" align="left">
            <script type="text/javascript"> translate_text('Bypass_volt', ['Voltage', ':']);</script>
          </td>
          <td align = "left">
            <xsl:if test="string-length($byp_volt1) != 0">
               <xsl:value-of select="concat($byp_volt1,'&#160;V')" />
            </xsl:if>
          </td>
          <xsl:if test="value[@name='EX_BYP_NUMLINES']/@val &gt; 1">
            <td align = "left" >
               <xsl:if test="string-length($byp_volt2) != 0">
                  <xsl:value-of select="concat($byp_volt2,'&#160;V')" />
               </xsl:if>
            </td>
          </xsl:if>
          <xsl:if test="value[@name='EX_BYP_NUMLINES']/@val &gt; 2">
            <td align = "left">
               <xsl:if test="string-length($byp_volt3) != 0">
                  <xsl:value-of select="concat($byp_volt3,'&#160;V')" />
               </xsl:if>
            </td>
          </xsl:if>
        </tr>
        <xsl:if test="string-length($byp_curr1) != 0 or string-length($byp_curr2) != 0 or string-length($byp_curr3) != 0">
          <tr>
            <td id="Bypass_curr" align="left">
              <script type="text/javascript"> translate_text('Bypass_curr', ['Current', ':']);</script>
            </td>
            <td align = "left">
              <xsl:if test="string-length($byp_curr1) != 0">
                <xsl:value-of select="concat($byp_curr1,'&#160;A')" />
              </xsl:if>
            </td>
            <xsl:if test="value[@name='EX_BYP_NUMLINES']/@val &gt; 1">
              <td align = "left" >
                <xsl:if test="string-length($byp_curr2) != 0">
                  <xsl:value-of select="concat($byp_curr2,'&#160;A')" />
                </xsl:if>
              </td>
            </xsl:if>
            <xsl:if test="value[@name='EX_BYP_NUMLINES']/@val &gt; 2">
              <td align = "left">
                <xsl:if test="string-length($byp_curr2) != 0">
                  <xsl:value-of select="concat($byp_curr3,'&#160;A')" />
                </xsl:if>
              </td>
            </xsl:if>
          </tr>
        </xsl:if>
        <xsl:if test="string-length($byp_pow1) != 0">
          <tr>
            <td id="Bypass_pow" align="left">
              <script type="text/javascript"> translate_text('Bypass_pow', ['Power', ':']);</script>
            </td>
            <td align = "left">
              <xsl:if test="string-length($byp_pow1) != 0">
                <xsl:value-of select="concat(concat($byp_pow1,'&#160;'), $wattunit)" />
              </xsl:if>
            </td>
          </tr>
        </xsl:if>
      </table>
    </div>

    <xsl:variable name="title_bar1">
      <xsl:choose>
        <xsl:when test="value[@name='OUTPOWER']/@val &gt; -1">
          <xsl:value-of select="concat(substring-before(value[@name='OUTPOWER']/@val,'.'), '%')" />
        </xsl:when>
        <xsl:otherwise>
          <xsl:value-of select="''" />
        </xsl:otherwise>
      </xsl:choose>
    </xsl:variable>
    <!--
    <xsl:variable name="title_bar2">
      <xsl:choose>
        <xsl:when test="value[@name='OUTPOWER']/@val2 &gt; -1">
          <xsl:value-of select="concat(substring-before(value[@name='OUTPOWER']/@val2,'.'), '%')" />
        </xsl:when>
        <xsl:otherwise>
          <xsl:value-of select="''" />
        </xsl:otherwise>
      </xsl:choose>
    </xsl:variable>
    <xsl:variable name="title_bar3">
      <xsl:choose>
        <xsl:when test="value[@name='OUTPOWER']/@val3 &gt; -1">
          <xsl:value-of select="concat(substring-before(value[@name='OUTPOWER']/@val3,'.'), '%')" />
        </xsl:when>
        <xsl:otherwise>
          <xsl:value-of select="''" />
        </xsl:otherwise>
      </xsl:choose>
    </xsl:variable>
-->

    <!-- Output Power Load BAR 1,2 and 3 -->
    <xsl:call-template name="Loadbar1">
      <xsl:with-param name="load1" select="value[@name='OUTPOWER']/@val"/>
      <xsl:with-param name="x1" select="'724'"/>
      <xsl:with-param name="y" select="410"/>
      <xsl:with-param name="yOffset" select="5"/>
      <xsl:with-param name="border" select="1"/>
      <xsl:with-param name="scale" select="1"/>
      <xsl:with-param name="outphases" select="1"/>
      <xsl:with-param name="textcolor" select="'#3341AA'"/>
      <xsl:with-param  name="title_bar1" select="$title_bar1"/>
      <xsl:with-param  name="output_txt1" select="' '"/>
    </xsl:call-template>

    <!-- OUTPUT-->
    <xsl:variable name="output_volt">
      <xsl:choose>
        <xsl:when test="string-length(value[@name='OUTPUT_VOLT']/@val)!=0">
          <xsl:value-of select="value[@name='OUTPUT_VOLT']" />
        </xsl:when>
        <xsl:otherwise>
          <xsl:value-of select="value[@name='SOLAWROV']" />
        </xsl:otherwise>
      </xsl:choose>
    </xsl:variable>

    <xsl:variable name="outp_freq">
      <xsl:call-template name="round-decimal">
        <xsl:with-param name="value" select="value[@name='OUTFREQ']/@val"/>
        <xsl:with-param name="precision" select="1" />
      </xsl:call-template>
    </xsl:variable>

    <xsl:variable name="outp_volt1">
      <xsl:call-template name="round-decimal">
        <xsl:with-param name="value" select="value[@name='SOLAWROV']/@val"/>
        <xsl:with-param name="precision" select="0" />
      </xsl:call-template>
    </xsl:variable>
    <xsl:variable name="outp_volt2">
      <xsl:call-template name="round-decimal">
        <xsl:with-param name="value" select="value[@name='SOLAWROV']/@val2"/>
        <xsl:with-param name="precision" select="0" />
      </xsl:call-template>
    </xsl:variable>
    <xsl:variable name="outp_volt3">
      <xsl:call-template name="round-decimal">
        <xsl:with-param name="value" select="value[@name='SOLAWROV']/@val3"/>
        <xsl:with-param name="precision" select="0" />
      </xsl:call-template>
    </xsl:variable>

    <xsl:variable name="outp_curr1">
      <xsl:call-template name="round-decimal">
        <xsl:with-param name="value" select="value[@name='EX_OUT_CURR']/@val"/>
        <xsl:with-param name="precision" select="0" />
      </xsl:call-template>
    </xsl:variable>
    <xsl:variable name="outp_curr2">
      <xsl:call-template name="round-decimal">
        <xsl:with-param name="value" select="value[@name='EX_OUT_CURR']/@val2"/>
        <xsl:with-param name="precision" select="0" />
      </xsl:call-template>
    </xsl:variable>
    <xsl:variable name="outp_curr3">
      <xsl:call-template name="round-decimal">
        <xsl:with-param name="value" select="value[@name='EX_OUT_CURR']/@val3"/>
        <xsl:with-param name="precision" select="0" />
      </xsl:call-template>
    </xsl:variable>

    <xsl:variable name="outp_pow1">
      <xsl:call-template name="round-decimal">
        <xsl:with-param name="value" select="(value[@name='EX_OUT_WATT']/@val) div 10.0"/>
        <xsl:with-param name="precision" select="1" />
      </xsl:call-template>
    </xsl:variable>
    <!--
    <xsl:variable name="outp_pow2">
      <xsl:call-template name="round-decimal">
        <xsl:with-param name="value" select="value[@name='EX_OUT_WATT']/@val2"/>
        <xsl:with-param name="precision" select="2" />
      </xsl:call-template>
    </xsl:variable>
    <xsl:variable name="outp_pow3">
      <xsl:call-template name="round-decimal">
        <xsl:with-param name="value" select="value[@name='EX_OUT_WATT']/@val3"/>
        <xsl:with-param name="precision" select="2" />
      </xsl:call-template>
    </xsl:variable>
-->
    <xsl:variable name="outp_volta1">
      <xsl:call-template name="round-decimal">
        <xsl:with-param name="value" select="(value[@name='SOLAWROV']/@val * value[@name='EX_OUT_CURR']/@val) div 1000.0"/>
        <xsl:with-param name="precision" select="2" />
      </xsl:call-template>
    </xsl:variable>
    <xsl:variable name="outp_volta2">
      <xsl:call-template name="round-decimal">
        <xsl:with-param name="value" select="(value[@name='SOLAWROV']/@val2 * value[@name='EX_OUT_CURR']/@val2) div 1000.0"/>
        <xsl:with-param name="precision" select="2" />
      </xsl:call-template>
    </xsl:variable>
    <xsl:variable name="outp_volta3">
      <xsl:call-template name="round-decimal">
        <xsl:with-param name="value" select="(value[@name='SOLAWROV']/@val3 * value[@name='EX_OUT_CURR']/@val3) div 1000.0"/>
        <xsl:with-param name="precision" select="2" />
      </xsl:call-template>
    </xsl:variable>

    <xsl:variable name="output_source">
      <xsl:choose>
        <xsl:when test="value[@name='EX_OUT_SOURCE']/@val=0">
          <xsl:value-of select="'Normal'" />
        </xsl:when>
        <xsl:when test="value[@name='EX_OUT_SOURCE']/@val=1">
          <xsl:value-of select="'On Battery'" />
        </xsl:when>
        <xsl:when test="value[@name='EX_OUT_SOURCE']/@val=2">
          <xsl:value-of select="'Static Bypass'" />
        </xsl:when>
        <xsl:when test="value[@name='EX_OUT_SOURCE']/@val=5">
          <xsl:value-of select="'Manual Bypass'" />
        </xsl:when>
        <xsl:otherwise>
          <xsl:value-of select="''" />
        </xsl:otherwise>
      </xsl:choose>
    </xsl:variable>

    <xsl:variable name="inv_stat">
      <xsl:choose>
        <xsl:when test="value[@name='EX_OUT_WATT']/@val2=0">
          <xsl:value-of select="'OFF'" />
        </xsl:when>
        <xsl:when test="value[@name='EX_OUT_WATT']/@val2=1">
          <xsl:value-of select="'Sync'" />
        </xsl:when>
        <xsl:otherwise>
          <xsl:value-of select="'Async'" />
        </xsl:otherwise>
      </xsl:choose>
    </xsl:variable>

    <div id="output_id" style="position:absolute;left:528px;top:256px;width:260px;height:10;align:center;text-align:left;color:#3341AA; font-weight:bold; font-size:11pt">
      <script type="text/javascript"> translate_text('output_id', 'Output');</script>
    </div>
    <div style="position:absolute;left:520px;top:276px;width:260px;height:76;align:center;">
      <table cellpadding="0" cellspacing="4" style="color:black;font-family: arial;font-weight:bold;font-size: 11px;margin-left:6;border: 1px solid #405080">
        <tr>
          <td id="output_freq" align="left">
            <script type="text/javascript"> translate_text('output_freq', ['Frequency', ':']);</script>
          </td>
          <td align = "left">
            <xsl:value-of select="concat($outp_freq,'&#160;Hz')" />
          </td>
        </tr>
        <tr>
          <td id="output_volt" align="left">
            <script type="text/javascript"> translate_text('output_volt', ['Voltage', ':']);</script>
          </td>
          <td align = "left">
            <xsl:value-of select="concat($outp_volt1,'&#160;V')" />
          </td>
          <xsl:if test="value[@name='EX_OUT_NUMLINES']/@val &gt; 1">
            <td align = "left" >
              <xsl:value-of select="concat($outp_volt2,'&#160;V')" />
            </td>
          </xsl:if>
          <xsl:if test="value[@name='EX_OUT_NUMLINES']/@val &gt; 2">
            <td align = "left">
              <xsl:value-of select="concat($outp_volt3,'&#160;V')" />
            </td>
          </xsl:if>
        </tr>
        <tr>
          <td id="output_curr" align="left">
            <script type="text/javascript"> translate_text('output_curr', ['Current', ':']);</script>
          </td>
          <td align = "left">
            <xsl:value-of select="concat($outp_curr1,'&#160;A')" />
          </td>
          <xsl:if test="value[@name='EX_OUT_NUMLINES']/@val &gt; 1">
            <td align = "left" >
              <xsl:value-of select="concat($outp_curr2,'&#160;A')" />
            </td>
          </xsl:if>
          <xsl:if test="value[@name='EX_OUT_NUMLINES']/@val &gt; 2">
            <td align = "left">
              <xsl:value-of select="concat($outp_curr3,'&#160;A')" />
            </td>
          </xsl:if>
        </tr>
        <tr>
          <td id="output_pow" align="left">
            <script type="text/javascript"> translate_text('output_pow', ['Power', ':']);</script>
          </td>
          <td align = "left">
            <xsl:value-of select="concat(concat($outp_pow1,'&#160;'), $wattunit)" />
          </td>
          <!--
          <xsl:if test="value[@name='EX_OUT_NUMLINES']/@val &gt; 1">
            <td align = "left" >
              <xsl:if test="$upsProtocol != '8106' and $upsProtocol != '33030' and $upsProtocol != '8115' and $upsProtocol != '33045'">
                <xsl:value-of select="concat(concat($outp_pow2,'&#160;'), $wattunit)" />
              </xsl:if>
            </td>
          </xsl:if>
          <xsl:if test="value[@name='EX_OUT_NUMLINES']/@val &gt; 2">
            <td align = "left">
              <xsl:if test="$upsProtocol != '8106' and $upsProtocol != '33030' and $upsProtocol != '8115' and $upsProtocol != '33045'">
                <xsl:value-of select="concat(concat($outp_pow3,'&#160;'), $wattunit)" />
              </xsl:if>
            </td>
          </xsl:if>
        -->
        </tr>
        <tr>
          <td align="left">
            <xsl:value-of select="'Outp.&#160;kVA:'" />
          </td>
          <td align = "left">
            <xsl:value-of select="concat($outp_volta1,'&#160;kVA')" />
          </td>
          <td align = "left">
            <xsl:value-of select="concat($outp_volta2,'&#160;kVA')" />
          </td>
          <td align = "left">
            <xsl:value-of select="concat($outp_volta3,'&#160;kVA')" />
          </td>
        </tr>
        <xsl:if test="string-length($output_source)!=0">
          <tr>
            <td align="left">
              <xsl:value-of select="'Source:'" />
            </td>
            <td align = "left">
              <xsl:value-of select="$output_source" />
            </td>
          </tr>
        </xsl:if>
        <tr>
          <td align="left">
            <xsl:value-of select="'Inv. Status:'" />
          </td>
          <td align = "left">
            <xsl:value-of select="$inv_stat" />
          </td>
        </tr>
      </table>
    </div>

    <!-- BATTERY-->
    <div id="battery_id" style="position:absolute;left:305px;top:340px;width:266px;height:10;align:center;text-align:left;color:#3341AA; font-weight:bold; font-size:11pt">
      <script type="text/javascript"> translate_text('battery_id', 'Battery');</script>
    </div>
    <div style="position:absolute;left:300px;top:360px;width:180px;height:110;text-align:center;">
      <table cellpadding="0" cellspacing="4" style="color:black;font-family: arial;font-weight:bold;font-size: 11px;margin-left:6;border: 1px solid #405080">
        <tr>
          <td id="output_ext_cap" align="left">
            <script type="text/javascript"> translate_text('output_ext_cap', ['Est. Capacity', ':']);</script>
          </td>
          <td align = "left">
            <xsl:value-of select="concat(value[@name='BATTCAP']/@val,'&#160;%')" />
          </td>
        </tr>
        <tr>
          <td id="batt_sec" align="left">
            <script type="text/javascript"> translate_text('batt_sec', ['Seconds', ':']);</script>
          </td>
          <td align = "left">
            <xsl:value-of select="concat(value[@name='EX_BATT_SECONBATT']/@val,'&#160;Sec')" />
          </td>
        </tr>
        <!--
        <tr>
          <td id="output_mins" align="left">
            <script type="text/javascript"> translate_text('output_mins', ['Minutes', ':']);</script>
          </td>
          <td align = "left">
            <xsl:value-of select="concat(value[@name='AUTONOMTIME']/@val,'&#160;Min')" />
          </td>
        </tr>
      -->
        <tr>
          <td id="batt_volt" align="left">
            <script type="text/javascript"> translate_text('batt_volt', ['Voltage', ':']);</script>
          </td>
          <td align = "left">
            <xsl:value-of select="concat($battvolt,'&#160;V')" />
          </td>
        </tr>
        <tr>
          <td id="batt_curr" align="left">
            <script type="text/javascript"> translate_text('batt_curr', ['Current', ':']);</script>
          </td>
          <td align = "left">
            <xsl:value-of select="concat(value[@name='EX_BATT_CURRPOS']/@val,'&#160;A')" />
          </td>
        </tr>
        <tr>
          <td id="charger_status_id" align="left">
            <script type="text/javascript"> translate_text('charger_status_id', ['Charger Status', ':']);</script>
          </td>
          <td id="charger_status_txt" align = "left">
            <xsl:choose>
              <xsl:when test="$output_off=1">
                <script type="text/javascript"> translate_text('charger_status_txt', 'Off');</script>
              </xsl:when>
              <xsl:when test="value[@name='SOLSECHARGE']/@val=0">
                <script type="text/javascript"> translate_text('charger_status_txt', 'Floating');</script>
              </xsl:when>
              <xsl:when test="value[@name='SOLSECHARGE']/@val=1">
                <script type="text/javascript"> translate_text('charger_status_txt', 'Charging');</script>
              </xsl:when>
              <xsl:when test="value[@name='SOLSECHARGE']/@val=3">
                <script type="text/javascript"> translate_text('charger_status_txt', 'Discharging');</script>
              </xsl:when>
              <xsl:otherwise>
                <script type="text/javascript"> translate_text('charger_status_txt', 'N/A'');</script>
              </xsl:otherwise>
            </xsl:choose>
          </td>
        </tr>
      </table>
    </div>

    <!--UPS Identification -->
    <div style="position:absolute;left:48px;top:60px;width:260px;height:10;align:center;text-align:left;color:#3341AA; font-weight:bold; font-size:11pt">
      <xsl:value-of select="'UPS Identification'" />
    </div>
    <div style="position:absolute;left:40px;top:80px;width:290px;height:62;text-align:center;">
      <table cellpadding="0" cellspacing="4" style="color:black;font-family: arial;font-weight: bold;font-size: 11px;margin-left:6;border: 1px solid #405080">
        <tr>
          <td id="model_id" align="left">
            <script type="text/javascript"> translate_text('model_id', ['Model', ':']);</script>
          </td>
          <td align = "left">
            <xsl:value-of select="value[@name='SOLSEMODEL']/@val" />
          </td>
        </tr>
        <tr>
          <td id="ident_id" align="left">
            <script type="text/javascript"> translate_text('ident_id', ['Identification', ':']);</script>
          </td>
          <td align = "left">
            <xsl:value-of select="value[@name='SOLSEIDENT']/@val" />
          </td>
        </tr>
        <tr>
          <td align="left">
            <xsl:value-of select="'Software&#160;Version:'" />
          </td>
          <td align = "left">
            <xsl:value-of select="value[@name='SOLSESOFTWARE']/@val" />
          </td>
        </tr>
      </table>
    </div>

    <!-- ROM TEMP
   <xsl:call-template name="rom_temp">
      <xsl:with-param name="x" select="396" />
      <xsl:with-param name="y" select="380" />
      <xsl:with-param name="temp" select="$tempdeg"/>
      <xsl:with-param name="temp_text" select="$lang-Temperature-short" />
      <xsl:with-param name="temp_unit" select="'&#176;C'"/>
   </xsl:call-template>
-->
  </xsl:template>
  <!-- EOS -->
</xsl:stylesheet>
