/*
   mitsu_panel.js - Mitsubishi SEC-Modelle
   Copyright (c) 2019, Generex GmbH. All rights reserved.
   UW.
*/

var mitsu_api = (function ($) {
  var oid = get_oid_from_url(),
    cb = null,
    pannes_created = false,
    err_div = null,
    inphases = null,
    outphases = null,
    bypphases = null,
    check_bowser = true,
    css_obj, attr_obj,
    valtable_obj = null,
    report_obj = null,
    ups_obj = null,
    unms_obj = null,
    old_alarms = '',
    status, tempmax, tempdeg, overload = 0,
    plattform = get_plattform(),
    ups_id = 0,
    ups_protocol = 0,
    prot_snmp = 0x803A,
    bg_height, bg_width,
    x_pos = 10, y_pos = 10;

    var evt_css_obj = {
      position: 'absolute',
      top: 550,
      'left': 50,
      width: 1000,
      height: 150,
      'background': 'white',
      align: 'center',
      'overflow-x': 'scroll',
      'overflow-y': 'scroll',
      'border': '2 solid #808080'
    },
    evt_attObj = { id: 'log_div_id', 'class': 'tdSmall8_black' };

  var evt1100AB_obj = [
   {id: 2,  code: "UF303", txt: "UPS CONTROL CIRCUIT ERROR", prio: "Major"},
   {id: 4,  code: "UF305", txt: "UPS CONTROL CIRCUIT ERROR", prio: "Major"},
   {id: 6,  code: "UF202", txt: "OUTPUT UNDERVOLTAGE", prio: "Minor"},
   {id: 8,  code: "UF258", txt: "LOAD ABNORMAL", prio: "Minor"},
   {id: 10, code: "UF301", txt: "UPS CONTROL CIRCUIT ERROR", prio: "Major"},
   {id: 12, code: "UF302", txt: "UPS CONTROL CIRCUIT ERROR", prio: "Major"},
   {id: 14, code: "UF306", txt: "CONTROL POWER SUPPLY ABNORMAL", prio: "Major"},
   {id: 16, code: "UF320", txt: "UPS CONTROL CIRCUIT ERROR", prio: "Minor"},
   {id: 18, code: "UF349", txt: "MODULE MAJOR FAULT", prio: "Major"},
   {id: 20, code: "UF340", txt: "MODULE MINOR FAULT", prio: "Minor"},
   {id: 22, code: "UF341", txt: "MODULE 1 MAJOR FAULT", prio: "Minor"},
   {id: 24, code: "UF342", txt: "MODULE 2 MAJOR FAULT", prio: "Minor"},
   {id: 26, code: "UF343", txt: "MODULE 3 MAJOR FAULT", prio: "Minor"},
   {id: 28, code: "UF344", txt: "MODULE 4 MAJOR FAULT", prio: "Minor"},
   {id: 30, code: "UF345", txt: "MODULE 5 MAJOR FAULT", prio: "Minor"},
   {id: 32, code: "UF401", txt: "52S ABNORMAL", prio: "Major"},
   {id: 34, code: "UF402", txt: "52S ABNORMAL", prio: "Major"},
   {id: 36, code: "UF055", txt: "CONVERTER ABNORMAL", prio: "Minor"},
   {id: 38, code: "UF151", txt: "BATTERY VOLTAGE ABNORMAL", prio: "Minor"},
   {id: 40, code: "UF156", txt: "CHG.STOPPED (BATTERY OVERTEMP.)", prio: "Minor"},
   {id: 42, code: "UF157", txt: "BATTERY OVERTEMPERATURE", prio: "Minor"},
   {id: 44, code: "UF158", txt: "BATTERY LIQUID LOW", prio: "Minor"},
   {id: 46, code: "UF161", txt: "CHG.STOPPED (DC VOLT. ABNORMAL)", prio: "Minor"},
   {id: 48, code: "UF162", txt: "BATTERY ABNORMAL", prio: "Minor"},
   {id: 50, code: "UF163", txt: "BATTERY VOLTAGE ABNORMAL", prio: "Minor"},
   {id: 52, code: "UF253", txt: "UPS CONTROL CIRCUIT ERROR", prio: "Minor"},
   {id: 54, code: "UF256", txt: "OUTPUT VOLTAGE ABNORMAL", prio: "Minor"},
   {id: 56, code: "UF351", txt: "CONTROL POWER SUPPLY ABNORMAL", prio: "Minor"},
   {id: 58, code: "UF352", txt: "CONTROL POWER SUPPLY ABNORMAL", prio: "Minor"},
   {id: 60, code: "UF372", txt: "UPS CONTROL CIRCUIT ERROR", prio: "Minor"},
   {id: 62, code: "UF376", txt: "UPS CONTROL CIRCUIT ERROR", prio: "Minor"},
   {id: 64, code: "UF377", txt: "UPS CONTROL CIRCUIT ERROR", prio: "Minor"},
   {id: 68, code: "UF452", txt: "CB3 ABNORMAL", prio: "Minor"},
   {id: 70, code: "UA805", txt: "OVERLOAD", prio: "Alarm"},
   {id: 72, code: "UA806", txt: "INVERTER OVERLOAD", prio: "Alarm"},
   {id: 74, code: "UA807", txt: "INVERTER OVERLOAD", prio: "Alarm"},
   {id: 76, code: "UA808", txt: "OVERLOAD", prio: "Alarm"},
   {id: 80, code: "UA812", txt: "BYPASS VOLTAGE OUT OF RANGE", prio: "Alarm"},
   {id: 82, code: "UA813", txt: "BYPASS PHASE ROTATION ERROR", prio: "Alarm"},
   {id: 84, code: "UA814", txt: "BYPASS FREQUENCY OUT OF RANGE", prio: "Alarm"},
   {id: 86, code: "UA817", txt: "EMERGENCY STOP", prio: "Alarm"},
   {id: 88, code: "UA819", txt: "REMOTE BUTTON ABNORMAL", prio: "Alarm"},
   {id: 90, code: "UA820", txt: "LOCAL BUTTON ABNORMAL", prio: "Alarm"},
   {id: 92, code: "UA821", txt: "TRANSFER INHIBIT", prio: "Alarm"},
   {id: 94, code: "UA822", txt: "TRANSFER INHIBIT", prio: "Alarm"},
   {id: 96, code: "UA830", txt: "INTERLOCK SWITCH ON", prio: "Alarm"},
   {id: 98, code: "UA831", txt: "EMERGENCY BYPASS SWITCH ON", prio: "Alarm"},
   {id: 100,code: "UA834", txt: "BATTERY DEPLETED AC OUT STOPPED", prio: "Alarm"},
   {id: 102,code: "UA835", txt: "TRANSFER INHIBIT", prio: "Alarm"},
   {id: 104,code: "UA861", txt: "MODULE ALARM", prio: "Alarm"},
   {id: 106,code: "UA862", txt: "MODULE MINOR FAULT", prio: "Alarm"},
   {id: 108,code: "UA863", txt: "MODULE INPUT VOLT. OUT OF RANGE", prio: "Alarm"},
   {id: 110,code: "UA864", txt: "MODULE OVERLOAD", prio: "Alarm"},
   {id: 112,code: "UA865", txt: "MODULE BATTERY DEPLETED", prio: "Alarm"},
   {id: 114,code: "UA890", txt: "EXTERNAL ALARM", prio: "Alarm"},
   {id: 116,code: "UF323", txt: "UPS CONTROL CIRCUIT ERROR", prio: "Minor"},
   {id: 118,code: "UF353", txt: "EXTERNAL I/F PCB ABNORMAL", prio: "Minor"},
   {id: 120,code: "UF254", txt: "UPS CONTROL CIRCUIT ERROR", prio: "Minor"},
   {id: 122,code: "UA866", txt: "BATTERY DEPLETED ALARM", prio: "Alarm"},
   {id: 124,code: "UA824", txt: "72B OPEN", prio: "Alarm"},
   {id: 126,code: "UF346", txt: "MODULE 6 MAJOR FAULT", prio: "Minor"},
   {id: 128,code: "UF347", txt: "MODULE 7 MAJOR FAULT", prio: "Minor"},
   {id: 130,code: "UF348", txt: "MODULE 8 MAJOR FAULT", prio: "Minor"},
   {id: 132,code: "UA891", txt: "CHARGE PROHIBITED", prio: "Alarm"},
   {id: 134,code: "UA892", txt: "DISCHARGE PROHIBITED", prio: "Alarm"}
  ];

  var evt7011_obj = [
   {id: 2, code: "UF003", txt: "PRECHARGE FAULT", prio: "Major"},
   {id: 4, code: "UF102", txt: "DC OVERVOLTAGE", prio: "Major"},
   {id: 6, code: "UF103", txt: "DC UNDERVOLTAGE", prio: "Major"},
   {id: 8, code: "UF109", txt: "DC UNBALANCED", prio: "Major"},
   {id: 10, code: "UF112", txt: "DC CIRCUIT ABNORMAL", prio: "Major"},
   {id: 12, code: "UF201", txt: "AC OUTPUT OVERVOLTAGE", prio: "Major"},
   {id: 14, code: "UF202", txt: "AC OUTPUT UNDERVOLTAGE", prio: "Major"},
   {id: 16, code: "UF218", txt: "UPS CONTROL CIRCUIT ERROR", prio: "Major"},
   {id: 18, code: "UF230", txt: "UPS CONTROL CIRCUIT ERROR", prio: "Major"},
   {id: 20, code: "UF301", txt: "UPS CONTROL CIRCUIT ERROR", prio: "Major"},
   {id: 22, code: "UF302", txt: "UPS CONTROL CIRCUIT ERROR", prio: "Major"},
   {id: 24, code: "UF303", txt: "UPS CONTROL CIRCUIT ERROR", prio: "Major"},
   {id: 26, code: "UF305", txt: "UPS CONTROL CIRCUIT ERROR", prio: "Major"},
   {id: 28, code: "UF306", txt: "CONTROL POWER SUPPLY ABNORMAL", prio: "Major"},
   {id: 30, code: "UF323", txt: "UPS CONTROL CIRCUIT ERROR", prio: "Major"},
   {id: 32, code: "UF349", txt: "ALL MODULE FAULT", prio: "Major"},
   {id: 34, code: "UF401", txt: "52S ABNORMAL", prio: "Major"},
   {id: 36, code: "UF402", txt: "52S ABNORMAL", prio: "Major"},
   {id: 38, code: "UF552", txt: "DUMMY FAULT", prio: "Major"},
   {id: 40, code: "UF061", txt: "MOD.1 CONV. ABNORMAL", prio: "Minor"},
   {id: 42, code: "UF062", txt: "MOD.2 CONV. ABNORMAL", prio: "Minor"},
   {id: 44, code: "UF063", txt: "MOD.3 CONV. ABNORMAL", prio: "Minor"},
   {id: 46, code: "UF064", txt: "MOD.4 CONV. ABNORMAL", prio: "Minor"},
   {id: 48, code: "UF104", txt: "DISCHARGE FAULT", prio: "Minor"},
   {id: 50, code: "UF151", txt: "DC VOLTAGE ABNORMAL", prio: "Minor"},
   {id: 54, code: "UF156", txt: "CHG.STOPPED(BATTERY OVERTEMP.)", prio: "Minor"},
   {id: 56, code: "UF157", txt: "BATTERY OVERTEMPERATURE", prio: "Minor"},
   {id: 58, code: "UF161", txt: "CHG.STOPPED(DC VOLT. ABNORMAL)", prio: "Minor"},
   {id: 60, code: "UF162", txt: "BATTERY ABNORMAL", prio: "Minor"},
   {id: 62, code: "UF163", txt: "BATTERY VOLTAGE ABNORMAL", prio: "Minor"},
   {id: 64, code: "", txt: "BATTERY WARNING", prio: "Notice"},
   {id: 66, code: "", txt: "BATTERY UPDATE", prio: "Notice"},
   {id: 68, code: "", txt: "BATTERY CUTOFF", prio: "Notice"},
   {id: 70, code: "UF181", txt: "MOD.1 CHOP. ABNORMAL", prio: "Minor"},
   {id: 72, code: "UF182", txt: "MOD.2 CHOP. ABNORMAL", prio: "Minor"},
   {id: 78, code: "UF191", txt: "MOD.1 DC OVERVOLTAGE", prio: "Minor"},
   {id: 80, code: "UF192", txt: "MOD.2 DC OVERVOLTAGE", prio: "Minor"},
   {id: 86, code: "UF254", txt: "UPS CONTROL CIRCUIT ERROR", prio: "Minor"},
   {id: 88, code: "UF256", txt: "AC OUTPUT VOLTAGE ABNORMAL", prio: "Minor"},
   {id: 90, code: "UF258", txt: "LOAD ABNORMAL", prio: "Minor"},
   {id: 92, code: "UF261", txt: "MOD.1 INVERTER ABNORMAL", prio: "Minor"},
   {id: 94, code: "UF262", txt: "MOD.2 INVERTER ABNORMAL", prio: "Minor"},
   {id: 100, code: "UF271", txt: "MOD.1 INVERTER OVERCURRENT", prio: "Minor"},
   {id: 102, code: "UF272", txt: "MOD.2 INVERTER OVERCURRENT", prio: "Minor"},
   {id: 108, code: "UF281", txt: "MOD.1 OVERTEMPERATURE", prio: "Minor"},
   {id: 110, code: "UF282", txt: "MOD.2 OVERTEMPERATURE", prio: "Minor"},
   {id: 116, code: "", txt: "UPS UPDATE", prio: "Notice"},
   {id: 118, code: "UF352", txt: "CONTROL POWER SUPPLY ABNORMAL", prio: "Minor"},
   {id: 120, code: "UF371", txt: "UPS CONTROL CIRCUIT ERROR", prio: "Minor"},
   {id: 122, code: "UF372", txt: "UPS CONTROL CIRCUIT ERROR", prio: "Minor"},
   {id: 124, code: "UF451", txt: "52S ABNORMAL", prio: "Minor"},
   {id: 126, code: "UF455", txt: "BYPASS CIRCUIT ABNORMAL", prio: "Minor"},
   {id: 128, code: "UA801", txt: "AC INPUT VOLTAGE OUT OF RANGE2", prio: "Alarm"},
   {id: 130, code: "UA802", txt: "AC INPUT FREQUENCY OUT OF RANGE2", prio: "Alarm"},
   {id: 132, code: "UA803", txt: "AC INPUT PHASE ROTATION ERROR", prio: "Alarm"},
   {id: 134, code: "UA806", txt: "OVERLOAD", prio: "Alarm"},
   {id: 136, code: "UA807", txt: "POWER OVERLOAD", prio: "Alarm"},
   {id: 138, code: "UA808", txt: "OVERLOAD ALARM", prio: "Alarm"},
   {id: 140, code: "UA809", txt: "POWER OVERLOAD ALARM", prio: "Alarm"},
   {id: 142, code: "UA810", txt: "INSTANT OVERLOAD", prio: "Alarm"},
   {id: 144, code: "UA812", txt: "AC INPUT VOLTAGE OUT OF RANGE1", prio: "Alarm"},
   {id: 146, code: "UA814", txt: "AC INPUT FREQUENCY OUT OF RANGE1", prio: "Alarm"},
   {id: 148, code: "UA817", txt: "EMERGENCY STOP ACTIVATED", prio: "Alarm"},
   {id: 150, code: "UA831", txt: "EMERGENCY BYPASS SWITCH ON", prio: "Alarm"},
   {id: 152, code: "UA834", txt: "BATTERY DEPLETED", prio: "Alarm"},
   {id: 154, code: "UA835", txt: "BATTERY DEPLETED ALARM", prio: "Alarm"},
   {id: 156, code: "UA840", txt: "SHUTDOWN1", prio: "Alarm"},
   {id: 158, code: "UA841", txt: "SHUTDOWN2", prio: "Alarm"},
   {id: 160, code: "UA860", txt: "REMOTE BUTTON ABNORMAL", prio: "Alarm"},
   {id: 162, code: "UA861", txt: "LOCAL BUTTON ABNORMAL", prio: "Alarm"},
   {id: 164, code: "UA845", txt: "UPS CONTROL CIRCUIT ERROR", prio: "Alarm"},
   {id: 168, code: "UA816", txt: "LOAD ON BYPASS LONG TIME", prio: "Alarm"},
   {id: 170, code: "UF071", txt: "MOD.1 CONV. ABNORMAL", prio: "Minor"},
   {id: 172, code: "UF072", txt: "MOD.2 CONV. ABNORMAL", prio: "Minor"},
   {id: 178, code: "UF307", txt: "CONTROL POWER SUPPLY ABNORMAL", prio: "Major"}
  ];

  var evtABSummit = [
   {id: 180, code: "UA801", txt: "AC INPUT VOLTAGE OUT OF RANGE", prio: "Alarm"},
   {id: 182, code: "UA802", txt: "AC INPUT FREQUENCY OUT OF RANGE", prio: "Alarm"},
   {id: 184, code: "UA803", txt: "AC INPUT PHASE ROTATION ERROR", prio: "Alarm"},
   {id: 186, code: "UA804", txt: "CONVERTER OPE. PROHIBITION", prio: "Alarm"},
   {id: 188, code: "UA805", txt: "INVERTER OVERLOAD", prio: "Alarm"},
   {id: 190, code: "UA806", txt: "INVERTER OVERLOAD", prio: "Alarm"},
   {id: 192, code: "UA807", txt: "INVERTER OVERLOAD", prio: "Alarm"},
   {id: 194, code: "UA808", txt: "INVERTER OVERLOAD", prio: "Alarm"},
   {id: 198, code: "UA810", txt: "INVERTER OVERLOAD", prio: "Alarm"},
   {id: 202, code: "UA812", txt: "BYPASS VOLTAGE OUT OF RANGE", prio: "Alarm"},
   {id: 204, code: "UA813", txt: "BYPASS PHASE ROTATION ERROR", prio: "Alarm"},
   {id: 206, code: "UA814", txt: "BYPASS FREQUENCY OUT OF RANGE", prio: "Alarm"},
   {id: 208, code: "UA815", txt: "TRANSFER PROHIBITION", prio: "Alarm"},
   {id: 212, code: "UA817", txt: "EMERGENCY STOP ACTIVATED", prio: "Alarm"},
   {id: 214, code: "UA821", txt: "TRANSFER PROHIBITION", prio: "Alarm"},
   {id: 216, code: "UA822", txt: "TRANSFER PROHIBITION", prio: "Alarm"},
   {id: 218, code: "UA824", txt: "CB2 OPEN", prio: "Alarm"},
   {id: 220, code: "UA827", txt: "52C OPE. PROHIBITION", prio: "Alarm"},
   {id: 240, code: "UA828", txt: "COOLING FAN ABNORMAL", prio: "Alarm"},
   {id: 222, code: "UA831", txt: "EMERGENCY BYPASS SWITCH ON", prio: "Alarm"},
   {id: 226, code: "UA833", txt: "52L OPEN", prio: "Alarm"},
   {id: 228, code: "UA834", txt: "BATTERY DEPLETED/AC OUT STOPPED", prio: "Alarm"},
   {id: 230, code: "UA835", txt: "TRANSFER PROHIBITION", prio: "Alarm"},
   {id: 232, code: "UA860", txt: "REMOTE BUTTON ABNORMAL", prio: "Alarm"},
   {id: 234, code: "UA861", txt: "LOCAL BUTTON ABNORMAL", prio: "Alarm"},
   {id: 200, code: "UA870", txt: "BALANCER OVERLOAD", prio: "Alarm"},
   {id: 252, code: "UA871", txt: "BYPASS VOLTAGE OUT OF RANGE(2)", prio: "Alarm"},
   {id: 236, code: "UA890", txt: "EXTERNAL ALARM", prio: "Alarm"},
   {id: 2,   code: "UF001", txt: "INPUT CIRCUIT ABNORMAL", prio: "Major"},
   {id: 4,   code: "UF002", txt: "CONVERTER OVERCURRENT", prio: "Major"},
   {id: 6,   code: "UF003", txt: "CONVERTER ABNORMAL", prio: "Major"},
   {id: 92,  code: "UF004", txt: "CONVERTER OVERCURRENT", prio: "Major"},
   {id: 8,   code: "UF006", txt: "CONVERTER ABNORMAL", prio: "Major"},
   {id: 10, code: "UF011", txt: "CB1 ABNORMAL", prio: "Major"},
   {id: 12, code: "UF052", txt: "CB1 ABNORMAL", prio: "Minor"},
   {id: 16, code: "UF056", txt: "CONVERTER OVERCURRENT", prio: "Minor"},
   {id: 18, code: "UF059", txt: "INPUT CIRCUIT ABNORMAL", prio: "Minor"},
   {id: 20, code: "UF102", txt: "DC OVERVOLTAGE", prio: "Major"},
   {id: 22, code: "UF103", txt: "DC UNDERVOLTAGE", prio: "Major"},
   {id: 24, code: "UF108", txt: "CHOPPER OVERCURRENT", prio: "Major"},
   {id: 26, code: "UF109", txt: "DC UNBALANCED", prio: "Major"},
   {id: 28, code: "UF110", txt: "ZERO PHASE OVERCURRENT", prio: "Major"},
   {id: 30, code: "UF111", txt: "UPS CONTROL CIRCUIT ERROR", prio: "Major"},
   {id: 32, code: "UF112", txt: "DC CIRCUIT ABNORMAL", prio: "Major"},
   {id: 36, code: "UF119", txt: "DC GROUND FAULT", prio: "Major"},
   {id: 34, code: "UF128", txt: "CONTROL POWER SUPPLY ABNORMAL", prio: "Major"},
   {id: 42, code: "UF151", txt: "DC VOLTAGE ABNORMAL", prio: "Major"},
   {id: 44, code: "UF152", txt: "DC VOLTAGE ABNORMAL", prio: "Major"},
   {id: 48, code: "UF154", txt: "CB2 ABNORMAL", prio: "Minor"},
   {id: 50, code: "UF156", txt: "CHG.STOPPED(BATTERYOVERTEMP.)", prio: "Minor"},
   {id: 52, code: "UF157", txt: "BATTERY OVERTEMPERATURE", prio: "Minor"},
   {id: 54, code: "UF158", txt: "BATTERY LIQUID LOW", prio: "Minor"},
   {id: 38, code: "UF159", txt: "DC GROUND FAULT", prio: "Minor"},
   {id: 56, code: "UF160", txt: "UPS CONTROL CIRCUIT ERROR", prio: "Minor"},
   {id: 58, code: "UF161", txt: "CHG.STOPPED(DC VOLT. ABNORMAL)", prio: "Minor"},
   {id: 60, code: "UF162", txt: "BATTERY ABNORMAL", prio: "Minor"},
   {id: 62, code: "UF163", txt: "BATTERY VOLTAGE ABNORMAL", prio: "Minor"},
   {id: 64, code: "UF170", txt: "VDB SENSOR ABNORMAL", prio: "Minor"},
   {id: 66, code: "UF171", txt: "CHG. STOPPED(DEVICE STATUS)", prio: "Minor"},
   {id: 68, code: "UF172", txt: "CHG. STOPPED(DEVICE STATUS)", prio: "Minor"},
   {id: 72, code: "UF201", txt: "INVERTER OVERVOLTAGE", prio: "Major"},
   {id: 74, code: "UF202", txt: "INVERTER UNDERVOLTAGE", prio: "Major"},
   {id: 76, code: "UF203", txt: "INVERTER OVERCURRENT", prio: "Major"},
   {id: 78, code: "UF204", txt: "OUTPUT CIRCUIT ABNORMAL", prio: "Major"},
   {id: 80, code: "UF206", txt: "UPS CONTROL CIRCUIT ERROR", prio: "Major"},
   {id: 82, code: "UF207", txt: "ZERO PHASE OVERCURRENT", prio: "Major"},
   {id: 84, code: "UF208", txt: "UPS CONTROL CIRCUIT ERROR", prio: "Major"},
   {id: 86, code: "UF209", txt: "52C ABNORMAL", prio: "Major"},
   {id: 88, code: "UF210", txt: "52C ABNORMAL", prio: "Major"},
   {id: 90, code: "UF211", txt: "52C ABNORMAL", prio: "Major"},
   {id: 100,code: "UF217", txt: "INVERTER OVERVOLTAGE", prio: "Major"},
   {id: 94, code: "UF214", txt: "COOLING FAN ABNORMAL", prio: "Major"},
   {id: 96, code: "UF213", txt: "OVERTEMPERATURE", prio: "Major"},
   {id: 98, code: "UF230", txt: "ZERO PHASE OVERCURRENT", prio: "Major"},
   {id: 102, code: "UF253", txt: "UPS CONTROL CIRCUIT ERROR", prio: "Minor"},
   {id: 104, code: "UF256", txt: "OUTPUT VOLTAGE ABNORMAL", prio: "Minor"},
   {id: 106, code: "UF257", txt: "52C ABNORMAL", prio: "Minor"},
   {id: 108, code: "UF258", txt: "LOAD ABNORMAL", prio: "Minor"},
   {id: 110, code: "UF259", txt: "ANOTHER UPS ABNORMAL", prio: "Minor"},
   {id: 114, code: "UF301", txt: "UPS CONTROL CIRCUIT ERROR", prio: "Major"},
   {id: 116, code: "UF302", txt: "UPS CONTROL CIRCUIT ERROR", prio: "Major"},
   {id: 118, code: "UF303", txt: "UPS CONTROL CIRCUIT ERROR", prio: "Major"},
   {id: 122, code: "UF305", txt: "UPS CONTROL CIRCUIT ERROR", prio: "Major"},
   {id: 124, code: "UF306", txt: "UPS CONTROL CIRCUIT ERROR", prio: "Major"},
   {id: 126, code: "UF309", txt: "INVERTER VOLTAGE ABNORMAL", prio: "Major"},
   {id: 128, code: "UF310", txt: "CONTROL POWER SUPPLY ABNORMAL", prio: "Major"},
   {id: 130, code: "UF320", txt: "UPS CONTROL CIRCUIT ERROR", prio: "Major"},
   {id: 120, code: "UF323", txt: "UPS CONTROL CIRCUIT ERROR", prio: "Major"},
   {id: 132, code: "UF331", txt: "UPS CONTROL CIRCUIT ERROR", prio: "Major"},
   {id: 134, code: "UF332", txt: "UPS CONTROL CIRCUIT ERROR", prio: "Major"},
   {id: 136, code: "UF333", txt: "UPS CONTROL CIRCUIT ERROR", prio: "Major"},
   {id: 138, code: "UF334", txt: "UPS CONTROL CIRCUIT ERROR", prio: "Major"},
   {id: 140, code: "UF352", txt: "CONTROL POWER SUPPLY ABNORMAL", prio: "Minor"},
   {id: 142, code: "UF363", txt: "UPS CONTROL CIRCUIT ERROR", prio: "Minor"},
   {id: 158, code: "UF371", txt: "UPS CONTROL CIRCUIT ERROR", prio: "Minor"},
   {id: 144, code: "UF372", txt: "UPS CONTROL CIRCUIT ERROR", prio: "Minor"},
   {id: 146, code: "UF374", txt: "UPS CONTROL CIRCUIT ERROR", prio: "Minor"},
   {id: 148, code: "UF375", txt: "UPS CONTROL CIRCUIT ERROR", prio: "Minor"},
   {id: 150, code: "UF376", txt: "UPS CONTROL CIRCUIT ERROR", prio: "Minor"},
   {id: 152, code: "UF377", txt: "UPS CONTROL CIRCUIT ERROR", prio: "Minor"},
   {id: 154, code: "UF378", txt: "UPS CONTROL CIRCUIT ERROR", prio: "Minor"},
   {id: 156, code: "UF379", txt: "UPS CONTROL CIRCUIT ERROR", prio: "Minor"},
   {id: 244, code: "UF380", txt: "ESTS CONTROL CIRCUIT ERROR", prio: "Minor"},
   {id: 246, code: "UF381", txt: "ESTS CONTROL CIRCUIT ERROR", prio: "Minor"},
   {id: 160, code: "UF401", txt: "52S ABNORMAL", prio: "Major"},
   {id: 162, code: "UF402", txt: "52S ABNORMAL", prio: "Major"},
   {id: 164, code: "UF420", txt: "52L OPERATION ERROR", prio: "Major"},
   {id: 168, code: "UF451", txt: "52S ABNORMAL", prio: "Minor"},
   {id: 170, code: "UF452", txt: "CB3 ABNORMAL", prio: "Minor"},
   {id: 250, code: "UF461", txt: "OVERTEMPERATURE", prio: "Minor"}
  ];

  var evt9900CXD = [
   {id: 810, code: "UA810", txt: "OVERLOAD", prio: "Alarm"},
   {id: 812, code: "UA812", txt: "BYPS.VOLT.OUT RNG", prio: "Alarm"},
   {id: 813, code: "UA813", txt: "BYPS.PHASE ABNL", prio: "Alarm"},
   {id: 814, code: "UA814", txt: "BYPS.FREQ.OUT RNG", prio: "Alarm"},
   {id: 817, code: "UA817", txt: "EMERG.STOP ACTIV.", prio: "Alarm"},
   {id: 821, code: "UA821", txt: "REM.BUTTON CLOSE", prio: "Alarm"},
   {id: 822, code: "UA822", txt: "LOC.BUTTON ABNL.", prio: "Alarm"},
   {id: 824, code: "UA824", txt: "CB2 OPEN", prio: "Alarm"},
   {id: 830, code: "UA830", txt: "INTERLOCK SWITCH ON", prio: "Alarm"},
   {id: 831, code: "UA831", txt: "EMERG.BYPS.SW.ON", prio: "Alarm"},
   {id: 833, code: "UA833", txt: "52L OPEN", prio: "Alarm"},
   {id: 834, code: "UA834", txt: "BATTERY DEPLETED", prio: "Alarm"},
   {id: 835, code: "UA835", txt: "TRANS.INHIBITED", prio: "Alarm"},
   {id: 861, code: "UA861", txt: "MODULE ALARM", prio: "Alarm"},
   {id: 862, code: "UA862", txt: "MODULE MINOR FAULT", prio: "Alarm"},
   {id: 863, code: "UA863", txt: "MOD.IN.VOLT.", prio: "Alarm"},
   {id: 864, code: "UA864", txt: "MODULE OVERLOAD", prio: "Alarm"},
   {id: 865, code: "UA865", txt: "MOD.BAT.END", prio: "Alarm"},
   {id: 866, code: "UA866", txt: "BATT.END WA", prio: "Alarm"},
   {id: 890, code: "UA890", txt: "EXTERNAL ALARM", prio: "Alarm"},
   {id: 006, code: "UF006", txt: "CONVERTER ABNORMAL", prio: "Major"},
   {id: 055, code: "UF055", txt: "CONVERTER ABNORMAL", prio: "Minor"},
   {id: 151, code: "UF151", txt: "BAT.VOLTAGE ABNL.", prio: "Minor"},
   {id: 156, code: "UF156", txt: "CHG.STOPPED", prio: "Minor"},
   {id: 157, code: "UF157", txt: "BATTERY OVERTEMP.", prio: "Minor"},
   {id: 158, code: "UF158", txt: "BATTERY LIQUID LOW", prio: "Minor"},
   {id: 161, code: "UF161", txt: "CHG.STOPPED", prio: "Minor"},
   {id: 162, code: "UF162", txt: "BATTERY ABNORMAL", prio: "Minor"},
   {id: 163, code: "UF163", txt: "BAT.VOLTAGE ABNL.", prio: "Minor"},
   {id: 172, code: "UF172", txt: "INVERTER UV.", prio: "Minor"},
   {id: 202, code: "UF202", txt: "CTRL.CIRCUIT ERR.", prio: "Major"},
   {id: 253, code: "UF253", txt: "CTRL.CIRCUIT ERR.", prio: "Minor"},
   {id: 254, code: "UF254", txt: "O/P VOLTAGE ABNL.", prio: "Minor"},
   {id: 256, code: "UF256", txt: "LOAD ABNORMAL", prio: "Minor"},
   {id: 258, code: "UF258", txt: "CTRL.CIRCUIT ERR.", prio: "Minor"},
   {id: 301, code: "UF301", txt: "CTRL.CIRCUIT ERR.", prio: "Major"},
   {id: 302, code: "UF302", txt: "CTRL.CIRCUIT ERR.", prio: "Major"},
   {id: 303, code: "UF303", txt: "CTRL.CIRCUIT ERR.", prio: "Major"},
   {id: 305, code: "UF305", txt: "CTRL.CIRCUIT ERR.", prio: "Major"},
   {id: 306, code: "UF306", txt: "CTRL.CIRCUIT ERR.", prio: "Major"},
   {id: 320, code: "UF320", txt: "CTRL.CIRCUIT ERR.", prio: "Minor"},
   {id: 321, code: "UF321", txt: "CTRL.CIRCUIT ERR.", prio: "Major"},
   {id: 322, code: "UF322", txt: "CTRL.CIRCUIT ERR.", prio: "Major"},
   {id: 323, code: "UF323", txt: "CTRL.CIRCUIT ERR.", prio: "Major"},
   {id: 341, code: "UF341", txt: "#1 MAJOR FAULT", prio: "Major"},
   {id: 342, code: "UF342", txt: "#2 MAJOR FAULT", prio: "Major"},
   {id: 343, code: "UF343", txt: "#3 MAJOR FAULT", prio: "Major"},
   {id: 344, code: "UF344", txt: "#4 MAJOR FAULT", prio: "Major"},
   {id: 349, code: "UF349", txt: "MODULE MAJOR FAULT", prio: "Major"},
   {id: 353, code: "UF353", txt: "EXTERNAL I/F PCB ABNORMAL", prio: "Minor"},
   {id: 371, code: "UF371", txt: "CTRL.CIRCUIT ERR.", prio: "Minor"},
   {id: 372, code: "UF372", txt: "CTRL.CIRCUIT ERR.", prio: "Minor"},
   {id: 374, code: "UF374", txt: "CTRL.CIRCUIT ERR.", prio: "Minor"},
   {id: 375, code: "UF375", txt: "CTRL.CIRCUIT ERR.", prio: "Minor"},
   {id: 376, code: "UF376", txt: "CTRL.CIRCUIT ERR.", prio: "Minor"},
   {id: 377, code: "UF377", txt: "CTRL.CIRCUIT ERR.", prio: "Minor"},
   {id: 378, code: "UF378", txt: "CTRL.CIRCUIT ERR.", prio: "Minor"},
   {id: 381, code: "UF381", txt: "CTRL.CIRCUIT ERR.", prio: "Minor"},
   {id: 382, code: "UF382", txt: "CTRL.CIRCUIT ERR.", prio: "Minor"},
   {id: 383, code: "UF383", txt: "CTRL.CIRCUIT ERR.", prio: "Minor"},
   {id: 401, code: "UF401", txt: "52S ABNORMAL", prio: "Major"},
   {id: 402, code: "UF402", txt: "52S ABNORMAL", prio: "Major"},
   {id: 420, code: "UF420", txt: "52L OPERATION ERR.", prio: "Major"},
   {id: 451, code: "UF451", txt: "52S ABNORMAL", prio: "Minor"},
   {id: 452, code: "UF452", txt: "CB3 ABNORMAL", prio: "Minor"}
  ];

  var req_params = null;

  /*public functions*/
  return {
    init: init
  };

  function init() {
    if (check_bowser && !utils_api.check_bowser()) {
      $('#loading_id').css('display', 'none');
      panes_api.create_br_check_panel(this, browser_check_ok);
      return true;
    }
    create_err_msg_panel();

    req_params = utils_api.get_req_params(utils_api.device_ups, oid);
    cb = jqmodule.subscribe(this, { update: update, error: err_funct }, req_params);
  }

  function browser_check_ok() {
    check_bowser = false;
    init();
  }

  function create_err_msg_panel() {
    css_obj = { position: 'absolute', top: y_pos, left: x_pos, width: 791, display: 'none', 'z-index': 62 };
    err_div = panes_api.create_err_panel({ id: 'error_div', 'class': 'alert alert-danger' }, css_obj);
  }

  function err_funct(response) {
    jqmodule.refresh(cb, 10 * 1000);
    panes_api.display_err({ response: response, oid: oid, err_div: err_div, err_msg_div_id: 'error_msg_div' });
  }

  function update(data) {
    if (data.ups == null) {
      return;
    }

    set_variable(data);
    jqmodule.refresh(cb, report_obj.refresh * 1000);
    if (err_div)
      err_div.css('display', 'none');

    if (window.menu_api != null)
      menu_api.update(data);

    unms_obj = get_unms_obj(data);
    if (unms_obj != null) {
      info_head_api.update(unms_obj);
    }

    if (!pannes_created) {
      pannes_created = true;
      create_panes();
      return true;
    }

    set_values();
  }

  function set_values() {
    set_bg();
    set_input_values(); //Input values
    set_bypass_values(); //Bypass values
    set_output_values(); //Output values
    set_battery_values(); //Battery value
    set_ups_status(); //set UPS status
    create_alarm_pane();
    set_footer();
    panes_api.set_autonomy_time_infos(valtable_obj);
  }

  function create_footer_pane() {
    if (!oid || !unms_obj)
      return true;
    footer_api.create({ //Attr
        id: 'panel2'
        //'class':'unmsFooter'
      }, { //css
        position: 'absolute',
        top:  '740px',
        left: '74px'
      },
      unms_obj
    );
  }

  function set_footer() {
    if (!oid || !unms_obj)
      return true;
    footer_api.set(unms_obj);
  }

  function set_variable(data) {
    ups_obj = data.ups;
    report_obj = data.report;
    valtable_obj = ups_obj.valtable;
    status = valtable_obj.STATUS;
    tempmax = valtable_obj.TMPMAX;
    tempdeg = valtable_obj.TEMPDEG;
    level = valtable_obj.LEVEL;

    if (valtable_obj._upsProtocol == null) {
      ups_protocol = valtable_obj.CONNECT & 0XFFFF;
    } else {
      ups_protocol = valtable_obj._upsProtocol;
    }

    if (valtable_obj.INPHASES != null)
      inphases = valtable_obj.INPHASES;
    else
      inphases = valtable_obj.PHASES;

    if (valtable_obj.OUTPHASES != null)
      outphases = valtable_obj.OUTPHASES;
    else
      outphases = valtable_obj.PHASES;

    if (valtable_obj.EX_BYP_NUMLINES != null && valtable_obj.EX_BYP_NUMLINES != -9999)
      bypphases = valtable_obj.EX_BYP_NUMLINES;
    else
      bypphases = inphases;

    bg_height = 700;
    bg_width = 1100;

    if (valtable_obj.OUTPOWER != null)
    {
      if (valtable_obj.OUTPOWER[0] > 100)
        overload = 1;
      else
        overload = 0;
    }
  }

  function create_panes() {
    utils_api.setTitle(valtable_obj._platform);
    create_status_panes(); //UPS status
    create_bg(); //background
    create_input_pane(); //Input table
    create_bypass_pane(); //Input table
    create_output_pane(); //Outnput   panel
    create_battery_pane(); //Battery panel
    create_alarm_pane();
    create_footer_pane();
    panes_api.create_autonomy_time_infos_panel('autonomy_time_label', valtable_obj);
  }

  function get_img_prefix()
  {
    if (valtable_obj.MODEL_NO == 4 || valtable_obj.MODEL_NO == 6) // 1100A, 7011B
      return './image/mitsu/single/';
    return './image/mitsu/dual/';
  }

  function create_bg()
  {
    var fragment = document.createDocumentFragment(),
      bg_img, ups_img, ups_div, img_div, bg_div, ups_img_src;

   css_obj = { position: 'absolute', top: y_pos, left: x_pos, width: bg_width, height: bg_height, border: 'solid 1px #808080' };
   bg_div = $('<div/>', { 'class': 'img-rounded' }).css(css_obj);

   var isSingle = (valtable_obj.MODEL_NO == 4 || valtable_obj.MODEL_NO == 6); // 1100A, 7011B

   if (isSingle)
   {
//    css_obj = { position: 'absolute', top: y_pos + 60, left: x_pos + 130, width: 800, height: 350/*, border: 'solid 1px #808080'*/ };
  //  img_div = $('<div/>').css(css_obj);
  //  bg_img = $('<img />').attr({ 'id': 'bg_img_id', 'class': 'img-rounded' }).appendTo(img_div);
    //  fragment.appendChild(img_div[0]);

      // bypass line
      css_obj = { position: 'absolute', top: y_pos + 82, left: x_pos + 227, width: 639, height: 95};
      img_div = $('<div/>').css(css_obj);
      bg_img = $('<img />').attr({ 'id': 'byp_img_id', 'class': 'img-rounded' }).appendTo(img_div);
      fragment.appendChild(img_div[0]);

      // input line
      css_obj = { position: 'absolute', top: y_pos + 175, left: x_pos + 214, width: 128, height: 42};
      img_div = $('<div/>').css(css_obj);
      bg_img = $('<img />').attr({ 'id': 'inp_img_id', 'class': 'img-rounded' }).appendTo(img_div);
      fragment.appendChild(img_div[0]);

      // gr
      css_obj = { position: 'absolute', top: y_pos + 156, left: x_pos + 354, width: 95, height: 79};
      img_div = $('<div/>').css(css_obj);
      bg_img = $('<img />').attr({ 'id': 'gr_img_id', 'class': 'img-rounded' }).appendTo(img_div);
      fragment.appendChild(img_div[0]);

      // batterie
      css_obj = { position: 'absolute', top: y_pos + 175, left: x_pos + 462, width: 150, height: 160};
      img_div = $('<div/>').css(css_obj);
      bg_img = $('<img />').attr({ 'id': 'bat_img_id', 'class': 'img-rounded' }).appendTo(img_div);
      fragment.appendChild(img_div[0]);

      // wr
      css_obj = { position: 'absolute', top: y_pos + 156, left: x_pos + 625, width: 97, height: 79};
      img_div = $('<div/>').css(css_obj);
      bg_img = $('<img />').attr({ 'id': 'wr_img_id', 'class': 'img-rounded' }).appendTo(img_div);
      fragment.appendChild(img_div[0]);

      // output line
      css_obj = { position: 'absolute', top: y_pos + 175, left: x_pos + 735, width: 184, height: 42};
      img_div = $('<div/>').css(css_obj);
      bg_img = $('<img />').attr({ 'id': 'out_img_id', 'class': 'img-rounded' }).appendTo(img_div);
      fragment.appendChild(img_div[0]);
   }
   else
   {
      // bypass line
      css_obj = { position: 'absolute', top: y_pos + 83, left: x_pos + 165, width: 713, height: 107};
      img_div = $('<div/>').css(css_obj);
      bg_img = $('<img />').attr({ 'id': 'byp_img_id', 'class': 'img-rounded' }).appendTo(img_div);
      fragment.appendChild(img_div[0]);

      // input line
      css_obj = { position: 'absolute', top: y_pos + 189, left: x_pos + 165, width: 128, height: 46};
      img_div = $('<div/>').css(css_obj);
      bg_img = $('<img />').attr({ 'id': 'inp_img_id', 'class': 'img-rounded' }).appendTo(img_div);
      fragment.appendChild(img_div[0]);

      // gr
      css_obj = { position: 'absolute', top: y_pos + 168, left: x_pos + 307, width: 107, height: 88};
      img_div = $('<div/>').css(css_obj);
      bg_img = $('<img />').attr({ 'id': 'gr_img_id', 'class': 'img-rounded' }).appendTo(img_div);
      fragment.appendChild(img_div[0]);

      // batterie
      css_obj = { position: 'absolute', top: y_pos + 190, left: x_pos + 428, width: 166, height: 179};
      img_div = $('<div/>').css(css_obj);
      bg_img = $('<img />').attr({ 'id': 'bat_img_id', 'class': 'img-rounded' }).appendTo(img_div);
      fragment.appendChild(img_div[0]);

      // wr
      css_obj = { position: 'absolute', top: y_pos + 168, left: x_pos + 611, width: 107, height: 87};
      img_div = $('<div/>').css(css_obj);
      bg_img = $('<img />').attr({ 'id': 'wr_img_id', 'class': 'img-rounded' }).appendTo(img_div);
      fragment.appendChild(img_div[0]);

      // output line
      css_obj = { position: 'absolute', top: y_pos + 189, left: x_pos + 731, width: 204, height: 45};
      img_div = $('<div/>').css(css_obj);
      bg_img = $('<img />').attr({ 'id': 'out_img_id', 'class': 'img-rounded' }).appendTo(img_div);
      fragment.appendChild(img_div[0]);
   }

    css_obj = { 'line-height': '200px', position: 'absolute', top: y_pos + 310, left: x_pos + 10, width: 450, height: 200/*, border: 'solid 1px #808080'*/};
    ups_div = $('<div/>').css(css_obj);

    // image of UPS
    ups_img_src = './image/mitsu/';
    if (valtable_obj.MODEL_NO == 4) //1100A
      ups_img_src += '1100A.jpg';
    else if (valtable_obj.MODEL_NO == 5) //1100B
      ups_img_src += '1100B.jpg';
    else if (valtable_obj.MODEL_NO == 6) //7011
      ups_img_src += '7011B.jpg';
    else if (valtable_obj.MODEL_NO == 7) //9900AEGIS
      ups_img_src += '9900AEGIS.jpg';
    else if (valtable_obj.MODEL_NO == 8) //9900B
      ups_img_src += '9900B.jpg';
    else if (valtable_obj.MODEL_NO == 9) //9900C
      ups_img_src += '9900C.jpg';
    else if (valtable_obj.MODEL_NO == 10) //9900CX
      ups_img_src += '9900CX.jpg';
    else if (valtable_obj.MODEL_NO == 11) //9900D
      ups_img_src += '9900C.jpg';
    else if (valtable_obj.MODEL_NO == 12) //Summit
      ups_img_src += 'Summit.jpg';
    ups_img = $('<img />').attr({ /*'align':'left', */'margin-bottom': '0.25em', 'vertical-align': 'middle', 'id': 'ups_img_id'/*, 'class': 'img-rounded'*/, src: ups_img_src }).appendTo(ups_div);

    fragment.appendChild(bg_div[0]);
    fragment.appendChild(img_div[0]);
    fragment.appendChild(ups_div[0]);
    document.body.appendChild(fragment);
    set_bg();
  }

  function set_bg()
  {
      var img_name, bg_img_src = get_img_prefix();
      var is7011B = valtable_obj.MODEL_NO == 6;
      var is1100A = valtable_obj.MODEL_NO == 4;
      var is1100B = valtable_obj.MODEL_NO == 5;

      if (is7011B)
      {
//         img_name = bg_img_src + 'SingleInputBlank.png';
  //       $('#bg_img_id').attr('src', img_name);
         // bypass status
         img_name = bg_img_src + 'byp02.png';
         if (valtable_obj.INVOLT[0] > 0)
         {
            if (valtable_obj.RAWSTATEB & (1 << 4))
               img_name = bg_img_src + 'byp03.png';
            else
               img_name = bg_img_src + 'byp01.png';
         }
         $('#byp_img_id').attr('src', img_name);

         // input status
         img_name = bg_img_src + 'inp02.png';
         if (valtable_obj.INVOLT[0] > 0)
         {
            if (valtable_obj.RAWSTATEB & 0x1)
               img_name = bg_img_src + 'inp03.png';
            else
               img_name = bg_img_src + 'inp01.png';
         }
         $('#inp_img_id').attr('src', img_name);

         // gr status
         if (valtable_obj.RAWSTATEB & (1 << 6))
            img_name = bg_img_src + 'gr01.png';
         else
            img_name = bg_img_src + 'gr02.png';
         $('#gr_img_id').attr('src', img_name);

         // battery status
         img_name = bg_img_src + 'bat04.png';
         if (valtable_obj.RAWSTATEB & (1 << 6))
         {
            if (!(valtable_obj.RAWSTATEB & (1 << 1)))
            {
               img_name = bg_img_src + 'bat01.png';
            }
         }
         else if (valtable_obj.RAWSTATEB & (1 << 1))
         {
            img_name = bg_img_src + 'bat03.png';
         }
         else
         {
            img_name = bg_img_src + 'bat02.png';
         }
         $('#bat_img_id').attr('src', img_name);

         // wr status
         if (valtable_obj.RAWSTATEB & (1 << 7))
            img_name = bg_img_src + 'wr01.png';
         else
            img_name = bg_img_src + 'wr02.png';
         $('#wr_img_id').attr('src', img_name);

         // output status
         img_name = bg_img_src + 'out01.png';
         if ((valtable_obj.RAWSTATEB & (1 << 4)) || (valtable_obj.RAWSTATEB & (1 << 3))) // bypass oder line ok
         {
            if (valtable_obj.RAWSTATEB & (1 << 3))
               img_name = bg_img_src + 'out02.png';
            else if (valtable_obj.RAWSTATEB & (1 << 7))
               img_name = bg_img_src + 'out03.png';
            else
               img_name = bg_img_src + 'out04.png';
         }
         $('#out_img_id').attr('src', img_name);
      }
      else
      {
         var volt;

         // single input UPS 1100A uses the same command MB1 for status as 3phase input UPS
         // but bypass line mimic uses input voltage instead of bypass voltage
         if (is1100A)
            volt = valtable_obj.INVOLT[0];
         else
            volt = valtable_obj.EX_BYP_VOLT[0];

         // bypass status
         img_name = bg_img_src + 'byp02.png';
         if (volt > 0)
         {
            if (valtable_obj.RAWSTATEA & (1 << 4))
               img_name = bg_img_src + 'byp03.png';
            else
               img_name = bg_img_src + 'byp01.png';
         }
         $('#byp_img_id').attr('src', img_name);

         // input status
         img_name = bg_img_src + 'inp02.png';
         if (valtable_obj.INVOLT[0] > 0)
         {
            var bit = 0;
            if (is1100A || is1100B)
            {
               bit = 18;
            }
            if (valtable_obj.RAWSTATEA & (1 << bit))
               img_name = bg_img_src + 'inp03.png';
            else
               img_name = bg_img_src + 'inp01.png';
         }
         $('#inp_img_id').attr('src', img_name);

         // gr status
         if (valtable_obj.RAWSTATEA & (1 << 18))
            img_name = bg_img_src + 'gr01.png';
         else
            img_name = bg_img_src + 'gr02.png';
         $('#gr_img_id').attr('src', img_name);

         // battery status
         img_name = bg_img_src + 'bat04.png';
         if (valtable_obj.RAWSTATEA & (1 << 18))
         {
            if (!(valtable_obj.RAWSTATEA & (1 << 1)))
            {
               img_name = bg_img_src + 'bat01.png';
            }
         }
         else if (valtable_obj.RAWSTATEA & (1 << 1))
         {
            img_name = bg_img_src + 'bat03.png';
         }
         else
         {
            img_name = bg_img_src + 'bat02.png';
         }
         $('#bat_img_id').attr('src', img_name);

         // wr status
         if (valtable_obj.RAWSTATEA & (1 << 5))
            img_name = bg_img_src + 'wr01.png';
         else
            img_name = bg_img_src + 'wr02.png';
         $('#wr_img_id').attr('src', img_name);

         // output status
         img_name = bg_img_src + 'out01.png';
         if ((valtable_obj.RAWSTATEA & (1 << 4)) || (valtable_obj.RAWSTATEA & (1 << 3))) // bypass oder line ok
         {
            if (valtable_obj.RAWSTATEA & (1 << 3))
               img_name = bg_img_src + 'out02.png';
            else if (valtable_obj.RAWSTATEA & (1 << 5))
               img_name = bg_img_src + 'out03.png';
            else
               img_name = bg_img_src + 'out04.png';
         }
         $('#out_img_id').attr('src', img_name);
      }
  }

  function get_evt_table()
  {
    if (valtable_obj.MODEL_NO == 4 || valtable_obj.MODEL_NO == 5) //1100A/B
      return evt1100AB_obj;
    else if (valtable_obj.MODEL_NO == 6) //7011
      return evt7011_obj;
    else if (valtable_obj.MODEL_NO == 7 || valtable_obj.MODEL_NO == 8 || valtable_obj.MODEL_NO == 12) //9900AEGIS/B/Summit
      return evtABSummit;
    else if (valtable_obj.MODEL_NO == 9 || valtable_obj.MODEL_NO == 10 || valtable_obj.MODEL_NO == 11) //9900C/CX/D
      return evt9900CXD;
    return 0;
  }

  function get_event(evttable, evt_no)
  {
     var i;
     for (i = 0; i < evttable.length; i++)
     {
        if (evttable[i].id == evt_no)
           return evttable[i];
     }
     return 0;
  }

  function create_alarm_pane()
  {
    var log_div, th, tr, td, table, tbody, date;

    if (old_alarms == valtable_obj.SECALARMS)
      return;
    old_alarms = valtable_obj.SECALARMS;

    // headerzeile der tabelle, die nicht mitscrollt
    var fragment = document.createDocumentFragment();
    var css_obj = { position: 'absolute', top: 530, left: 50,  width: 1000, height: 20};//, border: 'solid 2px #808080'};
    var t_div = $('<div/>').css(css_obj);
    var t_table = $('<table />', { width: 1000, height: 20 }).appendTo(t_div);
    var t_tbody = $('<tbody />').appendTo(t_table);
    var t_thead = $('<thead />').appendTo(t_table);

    var t_tr = $('<tr />', { height: 20, width: '100%' }).css({ 'font-weight': 'bold', 'background-color': '#DDDDDD', 'cellspacing': 0});
//    td = $('<td />', { align: 'left', width: '25%' }).append('Time Stamp');
//    td.appendTo(tr);
    var t_td = $('<td />', { align: 'left', width: '18%' }).append('Fault Code');
    t_td.appendTo(t_tr);
    t_td = $('<td />', { align: 'left', width: '55%' }).append('&nbsp;Description');
    t_td.appendTo(t_tr);
    t_td = $('<td />', { align: 'left', width: '20%' }).append('Priority');
    t_td.appendTo(t_tr);
    t_tr.appendTo(t_tbody);
    t_tbody.appendTo(t_table);
    t_table.appendTo(t_div);

    fragment.appendChild(t_div[0]);
    document.body.appendChild(fragment);

    // tabelleninhalt leeren
    log_div = $('#' + evt_attObj['id']);
    if (log_div.length > 0)
      log_div.remove();

    // scrollende tabelle in einem div kreiren
    log_div = panes_api.create_div(evt_attObj, evt_css_obj);
    thead = $('<thead />');
    tbody = $('<tbody />');
    table = $('<table/>', {
      id: 'table_' + evt_attObj['id'],
      'class': evt_attObj['class'],
      'align': 'center',
      cellpadding: 2,
      cellspacing: 2,
      width: '100%',
      'text-align': 'center',
      'margin-left': '10px'
    });

    /*
    tr = $('<tr />', { height: 20, width: '100%' }).css({ 'font-weight': 'bold', 'background-color': '#DDDDDD', 'cellspacing': 0});
//    td = $('<td />', { align: 'left', width: '25%' }).append('Time Stamp');
//    td.appendTo(tr);
    td = $('<td />', { align: 'left', width: '20%' }).append('Fault Code');
    td.appendTo(tr);
    td = $('<td />', { align: 'left', width: '60%' }).append('Description');
    td.appendTo(tr);
    td = $('<td />', { align: 'left', width: '20%' }).append('Priority');
    td.appendTo(tr);
*/
//    tr.appendTo(tbody);
    tbody.appendTo(table);
    table.appendTo(log_div);

    //log body
    var toks = valtable_obj.SECALARMS.split(',');
    var i, numalr = Number(toks[0]);
    var evttable = get_evt_table();
    //var date = new Date(Date.now());
    if (evttable != 0)
    {
       for (i = 1; i <= numalr; i++)
       {
         var alr = get_event(evttable, toks[i]);
         if (alr != 0)
         {
           tr = $('<tr />', { align: 'left' });
           //tr.append($('<td />', { 'nowrap': true }).append(utils_api.formatDate(date, "dd.mm.yyyy") + '&nbsp;&nbsp;&nbsp' + utils_api.formatDate(date, "hh:nn:ss")));
           tr.append($('<td />', { nowrap: "yes", align: "left", width: '20%'}).append(alr.code));
           tr.append($('<td />', { nowrap: "yes", align: "left", width: '60%' }).append(alr.txt));
           tr.append($('<td />', { nowrap: "yes", align: "left", width: '20%' }).append(alr.prio));

           tr.appendTo(tbody);
         }
       }
    }
  }

  function create_input_pane() //Input table
  {
    var fragment = document.createDocumentFragment(),
        tr, table, tbody, thead, in_div;//, top_div;
    var is7011B = valtable_obj.MODEL_NO == 6;

//    css_obj = { position: 'absolute', top: y_pos + 220, left: x_pos + 50, border: 'solid 2px #808080'};
//    top_div = $('<div/>', { id: 'input_div_id'/*,'class': 'img-rounded'*/ }).css(css_obj);
//    top_div.text('Input');

    //var status_div = $('<div/>',{ id: xid ,'class': 'img-rounded'}),

    css_obj = { position: 'absolute', top: y_pos + 170, left: x_pos + 40};//, border: 'solid 2px #808080'};
    in_div = $('<div/>', { id: 'input_div_id','class': 'img-rounded' }).css(css_obj);
    table = $('<table />', { 'class': 'table table-condensed table-hover', 'id': 'inptable' }).appendTo(in_div); // table-hover
    tbody = $('<tbody />').appendTo(table);

    thead = $('<thead />').appendTo(table);
    tr = $('<tr />').append($('<th />', { colspan: 2 }).append(i18n(lang_Input))).appendTo(thead);

    if (inphases == 2)
    {
       if (is7011B)
       {
         tr = $('<tr />', { 'id': 'inptrl1', 'align': 'left' }).append($('<td />').append("A-N:")).appendTo(tbody);
         $('<td />', { id: 'input_volt1_id', 'align': 'left'}).appendTo(tr);
         tr = $('<tr />', { 'id': 'inptrl2', 'align': 'left' }).append($('<td />').append("B-N:")).appendTo(tbody);
         $('<td />', { id: 'input_volt2_id', 'align': 'left'}).appendTo(tr);
       }
       else
       {
         tr = $('<tr />', { 'id': 'inptrl1', 'align': 'left' }).append($('<td />').append("A-B:")).appendTo(tbody);
         $('<td />', { id: 'input_volt1_id', 'align': 'left'}).appendTo(tr);
         tr = $('<tr />', { 'id': 'inptrl2', 'align': 'left' }).append($('<td />').append("B-A:")).appendTo(tbody);
         $('<td />', { id: 'input_volt2_id', 'align': 'left'}).appendTo(tr);
       }
       tr = $('<tr />', { 'id': 'inptrl3', 'align': 'left' }).append($('<td />').append("")).appendTo(tbody);
       $('<td />', { id: 'input_volt3_id', 'align': 'left'}).appendTo(tr);
       tr = $('<tr />', { 'id': 'inptrl4', 'align': 'left' }).append($('<td />').append("Freq:")).appendTo(tbody);
       $('<td />', { id: 'input_freq1_id', 'align': 'left'}).appendTo(tr);
    }
    else
    {
       if (is7011B)
       {
         tr = $('<tr />', { 'id': 'inptrl1', 'align': 'left' }).append($('<td />').append("A-N:")).appendTo(tbody);
         $('<td />', { id: 'input_volt1_id', 'align': 'left'}).appendTo(tr);
         tr = $('<tr />', { 'id': 'inptrl2', 'align': 'left' }).append($('<td />').append("B-N:")).appendTo(tbody);
         $('<td />', { id: 'input_volt2_id', 'align': 'left'}).appendTo(tr);
         tr = $('<tr />', { 'id': 'inptrl3', 'align': 'left' }).append($('<td />').append("C-N:")).appendTo(tbody);
         $('<td />', { id: 'input_volt3_id', 'align': 'left'}).appendTo(tr);
       }
       else
       {
         tr = $('<tr />', { 'id': 'inptrl1', 'align': 'left' }).append($('<td />').append("A-B:")).appendTo(tbody);
         $('<td />', { id: 'input_volt1_id', 'align': 'left'}).appendTo(tr);
         tr = $('<tr />', { 'id': 'inptrl2', 'align': 'left' }).append($('<td />').append("B-C:")).appendTo(tbody);
         $('<td />', { id: 'input_volt2_id', 'align': 'left'}).appendTo(tr);
         tr = $('<tr />', { 'id': 'inptrl3', 'align': 'left' }).append($('<td />').append("C-A:")).appendTo(tbody);
         $('<td />', { id: 'input_volt3_id', 'align': 'left'}).appendTo(tr);
       }
       tr = $('<tr />', { 'id': 'inptrl4', 'align': 'left' }).append($('<td />').append("Freq:")).appendTo(tbody);
       $('<td />', { id: 'input_freq1_id', 'align': 'left'}).appendTo(tr);
    }

//    fragment.appendChild(top_div[0]);
    fragment.appendChild(in_div[0]);
    document.body.appendChild(fragment);
    set_input_values();
  }

  function set_input_values()
  {
    var input_val = get_ph_1_3_vals(valtable_obj.INVOLT, valtable_obj.INCURR, valtable_obj.INFREQ, inphases);

   $('#input_volt1_id').text(input_val.val_objs[0].values[0]);
   $('#input_volt2_id').text(input_val.val_objs[0].values[1]);
   $('#input_volt3_id').text(input_val.val_objs[0].values[2]);
   $('#input_freq1_id').text(input_val.val_objs[2].values[0]);
  }

  function create_bypass_pane() //Input table
  {
    var fragment = document.createDocumentFragment(),
        tr, table, tbody, thead, in_div;
    var is7011B = valtable_obj.MODEL_NO == 6;

    css_obj = { position: 'absolute', top: y_pos + 30, left: x_pos + 40 };

    byp_div = $('<div/>', { id: 'bypass_div_id' }).css(css_obj);
    table = $('<table />', { 'class': 'table table-condensed table-hover', 'id': 'byptable' }).appendTo(byp_div); // table-hover
    tbody = $('<tbody />').appendTo(table);
    thead = $('<thead />').appendTo(table);
    tr = $('<tr />').append($('<th />', { colspan: 2 }).append(i18n(lang_Bypass))).appendTo(thead);

    if (is7011B)
    {
       tr = $('<tr />', { 'id': 'byptrl1', 'align': 'left' }).append($('<td />').append("A-N:")).appendTo(tbody);
       $('<td />', { id: 'byp_volt1_id', 'align': 'left'}).appendTo(tr);
       tr = $('<tr />', { 'id': 'byptrl2', 'align': 'left' }).append($('<td />').append("B-N:")).appendTo(tbody);
       $('<td />', { id: 'byp_volt2_id', 'align': 'left'}).appendTo(tr);
       tr = $('<tr />', { 'id': 'byptrl3', 'align': 'left' }).append($('<td />').append("C-N:")).appendTo(tbody);
       $('<td />', { id: 'byp_volt3_id', 'align': 'left'}).appendTo(tr);
    }
    else
    {
       tr = $('<tr />', { 'id': 'byptrl1', 'align': 'left' }).append($('<td />').append("A-B:")).appendTo(tbody);
       $('<td />', { id: 'byp_volt1_id', 'align': 'left'}).appendTo(tr);
       tr = $('<tr />', { 'id': 'byptrl2', 'align': 'left' }).append($('<td />').append("B-C:")).appendTo(tbody);
       $('<td />', { id: 'byp_volt2_id', 'align': 'left'}).appendTo(tr);
       tr = $('<tr />', { 'id': 'byptrl3', 'align': 'left' }).append($('<td />').append("C-A:")).appendTo(tbody);
       $('<td />', { id: 'byp_volt3_id', 'align': 'left'}).appendTo(tr);
    }
    tr = $('<tr />', { 'id': 'byptrl4', 'align': 'left' }).append($('<td />').append("Freq:")).appendTo(tbody);
    $('<td />', { id: 'byp_freq1_id', 'align': 'left'}).appendTo(tr);

    fragment.appendChild(byp_div[0]);
    document.body.appendChild(fragment);
    set_bypass_values();
  }

  function set_bypass_values()
  {
    var val = get_ph_1_3_vals(valtable_obj.EX_BYP_VOLT, valtable_obj.EX_BYP_CURR, valtable_obj.EX_BYP_FREQ, bypphases);

   $('#byp_volt1_id').text(val.val_objs[0].values[0]);
   $('#byp_volt2_id').text(val.val_objs[0].values[1]);
   $('#byp_volt3_id').text(val.val_objs[0].values[2]);
   $('#byp_freq1_id').text(val.val_objs[2].values[0]);
  }

  function create_output_pane() //Output
  {
    var output_div = null,
      tr, table, tbody, thead,
      fragment = document.createDocumentFragment();
    var is7011B = valtable_obj.MODEL_NO == 6;

    css_obj = { position: 'absolute', top: y_pos + 100, left: x_pos + 950 };
    output_div = $('<div/>', { id: 'output_div_id' }).css(css_obj);
    table = $('<table />', { 'class': 'table table-condensed table-hover', 'id': 'outtable' }).appendTo(output_div);
    tbody = $('<tbody />').appendTo(table);
    thead = $('<thead />').appendTo(table);
    tr = $('<tr />').append($('<th />', { colspan: 2 }).append(i18n(lang_Output))).appendTo(thead);

    if (is7011B)
    {
       tr = $('<tr />', { 'id': 'outtrl1', 'align': 'left' }).append($('<td />').append("A-N:")).appendTo(tbody);
       $('<td />', { id: 'out_volt1_id', 'align': 'left' }).appendTo(tr);
       tr = $('<tr />', { 'id': 'outtrl2', 'align': 'left' }).append($('<td />').append("B-N:")).appendTo(tbody);
       $('<td />', { id: 'out_volt2_id', 'align': 'left' }).appendTo(tr);
       tr = $('<tr />', { 'id': 'outtrl3', 'align': 'left' }).append($('<td />').append("C-N:")).appendTo(tbody);
       $('<td />', { id: 'out_volt3_id', 'align': 'left' }).appendTo(tr);
    }
    else
    {
       tr = $('<tr />', { 'id': 'outtrl1', 'align': 'left' }).append($('<td />').append("A-B:")).appendTo(tbody);
       $('<td />', { id: 'out_volt1_id', 'align': 'left' }).appendTo(tr);
       tr = $('<tr />', { 'id': 'outtrl2', 'align': 'left' }).append($('<td />').append("B-C:")).appendTo(tbody);
       $('<td />', { id: 'out_volt2_id', 'align': 'left' }).appendTo(tr);
       tr = $('<tr />', { 'id': 'outtrl3', 'align': 'left' }).append($('<td />').append("C-A:")).appendTo(tbody);
       $('<td />', { id: 'out_volt3_id', 'align': 'left' }).appendTo(tr);
    }

    tr = $('<tr />', { 'id': 'outtrl4', 'align': 'left' }).append($('<td />').append("A:")).appendTo(tbody);
    $('<td />', { id: 'out_curr1_id', 'align': 'left' }).appendTo(tr);
    tr = $('<tr />', { 'id': 'outtrl6', 'align': 'left' }).append($('<td />').append("B:")).appendTo(tbody);
    $('<td />', { id: 'out_curr2_id', 'align': 'left' }).appendTo(tr);
    tr = $('<tr />', { 'id': 'outtrl6', 'align': 'left' }).append($('<td />').append("C:")).appendTo(tbody);
    $('<td />', { id: 'out_curr3_id', 'align': 'left' }).appendTo(tr);

    tr = $('<tr />', { 'id': 'outtrl7', 'align': 'left' }).append($('<td />').append("Power:")).appendTo(tbody);
    $('<td />', { id: 'out_pwr1_id', 'align': 'left' }).appendTo(tr);

    tr = $('<tr />', { 'id': 'outtrl8', 'align': 'left' }).append($('<td />').append("Freq:")).appendTo(tbody);
    $('<td />', { id: 'out_freq1_id', 'align': 'left' }).appendTo(tr);

    fragment.appendChild(output_div[0]);
    document.body.appendChild(fragment);

    set_output_values();
  }

  function set_output_values() //Output values
  {
    var val = get_ph_1_3_vals(valtable_obj.SOLAWROV, valtable_obj.EX_OUT_CURR, valtable_obj.OUTFREQ, outphases);

   $('#out_volt1_id').text(val.val_objs[0].values[0]);
   $('#out_volt2_id').text(val.val_objs[0].values[1]);
   $('#out_volt3_id').text(val.val_objs[0].values[2]);

   $('#out_curr1_id').text(val.val_objs[1].values[0]);
   $('#out_curr2_id').text(val.val_objs[1].values[1]);
   $('#out_curr3_id').text(val.val_objs[1].values[2]);

   $('#out_pwr1_id').html(get_value({ values: [valtable_obj.EX_OUT_WATT[0]], index: 0, unit: ' kW', precision: 1 }));
   $('#out_freq1_id').text(val.val_objs[2].values[0]);
  }

  function create_battery_pane() //Battery panel
  {
    var val, tr, table, tbody, thead, batt_div, cap_div,
      fragment = document.createDocumentFragment();

    css_obj = { position: 'absolute', top: y_pos + 280, left: x_pos + 600 }; //, width:130, height:100
    attr_obj = { id: 'battery_div_id' };
    batt_div = $('<div/>').attr(attr_obj).css(css_obj);
    table = table = $('<table />', { 'class': 'table table-condensed table-hover' }).appendTo(batt_div);
    tbody = $('<tbody />').appendTo(table);
    thead = $('<thead />').appendTo(table);
    tr = $('<tr />').append($('<th />', { colspan: 2 }).append(i18n(lang_Battery))).appendTo(thead);

    tr = $('<tr />').append($('<td />', { align: 'left' }).append(i18n('Battery Status') + ":")).appendTo(tbody);
    $('<td />', { id: 'batt_stat_id', align: 'left' }).appendTo(tr);

    tr = $('<tr />').append($('<td />', { align: 'left' }).append(i18n('Battery Charge') + ":")).appendTo(tbody);
    $('<td />', { id: 'batt_chrg_id', align: 'left' }).appendTo(tr);

    tr = $('<tr />').append($('<td />', { align: 'left' }).append(i18n(lang_Voltage) + ":")).appendTo(tbody);
    $('<td />', { id: 'batt_voltpos_id', align: 'left' }).appendTo(tr);

    tr = $('<tr />').append($('<td />', { align: 'left' }).append(i18n(lang_Current) + ":")).appendTo(tbody);
    $('<td />', { id: 'batt_currpos_id', align: 'left' }).appendTo(tr);

    tr = $('<tr />').append($('<td />', { align: 'left' }).append(i18n('Seconds on Battery') + ":")).appendTo(tbody);
    $('<td />', { id: 'batt_seconds_id', align: 'left' }).appendTo(tr);

    tr = $('<tr />').append($('<td />', { align: 'left' }).append(i18n('Percent Remaining') + ":")).appendTo(tbody);
    $('<td />', { id: 'batt_cap_id', align: 'left' }).appendTo(tr);

    fragment.appendChild(batt_div[0]);
    document.body.appendChild(fragment);

    set_battery_values();
  }

  function set_battery_values() //set Battery value
  {
    var battstat = 'N/A', battchrg = 'N/A';
    if (valtable_obj.BATTERY_STATE == 0)
      battstat = 'Ok';
    else if (valtable_obj.BATTERY_STATE == 1)
      battstat = 'Low';
    else if (valtable_obj.BATTERY_STATE == 2)
      battstat = 'Depletet';

    if (valtable_obj.SOLSECHARGE == 0)
      battchrg = 'Floating';
    else if (valtable_obj.SOLSECHARGE == 1)
      battchrg = 'Charging';
    else if (valtable_obj.SOLSECHARGE == 2)
      battchrg = 'Resting';
    else if (valtable_obj.SOLSECHARGE == 3)
      battchrg = 'Discharging';

    $('#batt_stat_id').text(battstat);
    $('#batt_chrg_id').text(battchrg);
    $('#batt_seconds_id').text(get_value({ values: [valtable_obj.EX_BATT_SECONBATT], index: 0, unit: ' Sec', precision: 0 }));
    $('#batt_voltpos_id').text(get_value({ values: [valtable_obj.BATTVOLT], index: 0, unit: ' VDC', precision: 1 }));
    $('#batt_currpos_id').text(get_value({ values: [valtable_obj.EX_BATT_CURRPOS], index: 0, unit: ' A', precision: 1 }));
    $('#batt_cap_id').text(get_value({ values: [valtable_obj.BATTCAP], index: 0, unit: ' %', precision: 0 }));
  }

  function get_value(val_obj, na_string) {
    var val = null;
    var a = null;

    if (!val_obj.values || (val_obj.values.length <= val_obj.index))
      return na_string == null ? 'N/A' : na_string;

    val = val_obj.values[val_obj.index];
    if (val == null || val == -9999)
      return na_string == null ? 'N/A' : na_string;
    if (ups_protocol == prot_snmp && val == -1)
      return na_string == null ? 'N/A' : na_string;

    val = val.toFixed(val_obj.precision);
    val += val_obj.unit;

    return val;
  }

  function get_ph_1_3_vals(volt, cur, freq, phases) {
    var val_obj, val;
    l_1_3_vals = {
      availables: [0, 0, 0], //L1-L3
      val_objs: [
        { unit: ' VAC', precision: 1, values: ['', '', ''] },
        { unit: ' A', precision: 1, values: ['', '', ''] },
        { unit: ' Hz', precision: 1, values: ['', '', ''] }
      ]
    };
    volt = volt || [];
    cur = cur || [], freq = freq || [];
    $.each([volt, cur, freq], function (index, values) {
      val_obj = l_1_3_vals.val_objs[index];
      $.each(values, function (i, val) {
        if (i < phases) {
          val = values[i];
          if ((ups_protocol != prot_snmp && val != -9999) || (ups_protocol == prot_snmp && val != -1)) {
            if (values === freq && (i == 1 || i == 2)) {} else
              l_1_3_vals.availables[i] = 1;

            val = val.toFixed(val_obj.precision);
            val += val_obj.unit;
            val_obj.values[i] = val;
          }
        }
      });
    });
    return l_1_3_vals;
  }

  function create_status_panes()
  {
      create_status_pane(200, 'loi_id', 'Load on Inverter');
      create_status_pane(350, 'bo_id', 'Battery Operation');
      create_status_pane(500, 'lob_id', 'Load on Bypass');
      create_status_pane(650, 'ol_id', 'Overload');
      create_status_pane(800, 'uf_id', 'UPS Fault');
  }

  function create_status_pane(xoff, xid, xtxt)
  {
    var status_div = $('<div/>',{id: xid , 'class': 'img-rounded'});

    css_obj = {'padding-top': '6px', position: 'absolute', top: y_pos + 30, left: x_pos + xoff, width: 120, height: 30, 'text-align': 'center', overflow: 'hidden', border: 'solid 2px #808080', 'font-weight': 'bold', 'font-size': '13px' };
    $(status_div).css(css_obj).append(i18n(xtxt));;
    status_div.appendTo(document.body);
    set_ups_status();
  }

  function set_ups_status() //set UPS status
  {
   if (valtable_obj.EX_OUT_SOURCE == 0)   // normal
      $('#loi_id').css('background-color', '#52B245');
   else
      $('#loi_id').css('background-color', '#DDDDDD');

   if (valtable_obj.EX_OUT_SOURCE == 1)   // on battery
      $('#bo_id').css('background-color', 'orange');
   else
      $('#bo_id').css('background-color', '#DDDDDD');

   if (valtable_obj.EX_OUT_SOURCE == 2)   // on bypass
      $('#lob_id').css('background-color', 'orange');
   else
      $('#lob_id').css('background-color', '#DDDDDD');

   if (valtable_obj.STATUS & 0x100)   // overload
      $('#ol_id').css('background-color', 'orange');
   else
      $('#ol_id').css('background-color', '#DDDDDD');

   if (valtable_obj.STATUS & 0x2000)   // general fault
      $('#uf_id').css('background-color', 'red');
   else
      $('#uf_id').css('background-color', '#DDDDDD');
  }

  function set_model_name() {
    var model_name = valtable_obj.MODEL;
    $('#model_name_id').html(model_name);
  }

}(jQuery));
