/*
   Copyright (c) 2020, Generex GmbH. All rights reserved.
   UW.
*/

var narada8_api = (function ($) {
  var oid = get_oid_from_url(),
    cb = null,
    pannes_created = false,
    err_div = null,
    check_bowser = true,
    css_obj, attr_obj,
    valtable_obj = null,
    report_obj = null,
    ups_obj = null,
    unms_obj = null,
    inphases = null,
    ups_protocol = 0,
    prot_snmp = 0x803A,
    bg_height, bg_width;
    x_pos = 10, y_pos = 10;

  var req_params = null;

  /*public functions*/
  return {
    init: init
  };

  function init() {
    if (check_bowser && !utils_api.check_bowser()) {
      $('#loading_id').css('display', 'none');
      panes_api.create_br_check_panel(this, browser_check_ok);
      return true;
    }
    create_err_msg_panel();

    req_params = utils_api.get_req_params(utils_api.device_ups, oid);
    cb = jqmodule.subscribe(this, { update: update, error: err_funct }, req_params);
  }

  function browser_check_ok() {
    check_bowser = false;
    init();
  }

  function create_err_msg_panel() {
    css_obj = { position: 'absolute', top: y_pos, left: x_pos, width: 791, display: 'none', 'z-index': 62 };
    err_div = panes_api.create_err_panel({ id: 'error_div', 'class': 'alert alert-danger' }, css_obj);
  }

  function err_funct(response) {
    jqmodule.refresh(cb, 10 * 1000);
    panes_api.display_err({ response: response, oid: oid, err_div: err_div, err_msg_div_id: 'error_msg_div' });
  }

  function update(data) {
    if (data.ups == null) {
      return;
    }

    set_variable(data);
    jqmodule.refresh(cb, report_obj.refresh * 1000);
    if (err_div)
      err_div.css('display', 'none');

    if (window.menu_api != null)
      menu_api.update(data);

    unms_obj = get_unms_obj(data);
    if (unms_obj != null)
    {
      info_head_api.update(unms_obj);
    }

    if (!pannes_created)
    {
      pannes_created = true;
      create_panes();

      return true;
    }

    set_values();
  }

  function set_values() {
    set_all_tabs();
    set_model_name();
    set_footer();
  }

  function create_footer_pane() {
    if (!oid || !unms_obj)
      return true;
    footer_api.create({ //Attr
        id: 'panel2'
        //'class':'unmsFooter'
      }, { //css
        position: 'absolute',
        top: '540px',
        left: '74px'
      },
      unms_obj
    );
  }

  function set_footer() {
    if (!oid || !unms_obj)
      return true;
    footer_api.set(unms_obj);
  }

  function set_variable(data)
  {
    ups_obj = data.ups;
    report_obj = data.report;
    valtable_obj = ups_obj.valtable;

    if (valtable_obj._upsProtocol == null)
    {
      ups_protocol = valtable_obj.CONNECT & 0XFFFF;
    }
    else
    {
      ups_protocol = valtable_obj._upsProtocol;
    }
  }

  function create_panes() {
    utils_api.setTitle(valtable_obj._platform);
    create_all_tabs();
    create_bg();
    create_model_panes();
    create_footer_pane();
    set_all_tabs();
  }

  function create_bg() {
    var fragment = document.createDocumentFragment(),
      img_div;

    css_obj = { position: 'absolute', top: y_pos, left: x_pos, width: bg_width, height: bg_height, 'z-index': -1, border: 'solid 1px #808080' };
    img_div = $('<div/>', { 'class': 'img-rounded' }).css(css_obj);
    fragment.appendChild(img_div[0]);
    document.body.appendChild(fragment);
  }

  function create_model_panes() {
    var model_div, a;
    css_obj = { position: 'absolute', top: y_pos, left: x_pos, align: 'center', width: bg_width, 'z-index': 40 }; // 'background-color': '#3a87ad'
    attr_obj = { id: 'model_div_id', 'class': 'list-group' };
    model_div = panes_api.create_div(attr_obj, css_obj);
    a = $('<a/>', { 'class': 'list-group-item active', id: 'model_name_id' }).appendTo(model_div);
    set_model_name();
  }

  /////////////////////////////////////////////////

  function create_tab(name, idx, x, y)
  {
    var div, css_obj;
    css_obj = { align: 'center', position: 'absolute', top: y + 32 , left: x , width: 266, height: 170 };
    div = $('<div/>', {id: 'tab' + name + idx, 'class': "tabcontent"}).css(css_obj);//.append(name + idx);
    return div;
  }

  function create_data_tab(fragment, idx, x, y)
  {
    var div = create_tab('Data', idx, x, y);
    var tr, table, tbody;

    table = $('<table />', { id: 'id_data' + idx, 'class': 'table table-condensed table-hover' }).appendTo(div);
    tbody = $('<tbody />').appendTo(table);

    tr = $('<tr />').append($('<td />', { align: 'left' }).append(i18n('Voltage') + ":")).appendTo(tbody);
    $('<td />', { id: 'volt_id' + idx, align: 'left' }).appendTo(tr);
    tr = $('<tr />').append($('<td />', { align: 'left' }).append(i18n('Current') + ":")).appendTo(tbody);
    $('<td />', { id: 'curr_id' + idx, align: 'left' }).appendTo(tr);
    tr = $('<tr />').append($('<td />', { align: 'left' }).append(i18n('Remaining Capacity') + ":")).appendTo(tbody);
    $('<td />', { id: 'cap_id' + idx, align: 'left' }).appendTo(tr);
    tr = $('<tr />').append($('<td />', { align: 'left' }).append(i18n('Avg. Cell Temp.') + ":")).appendTo(tbody);
    $('<td />', { id: 'celltemp_id' + idx, align: 'left' }).appendTo(tr);
    tr = $('<tr />').append($('<td />', { align: 'left' }).append(i18n('Ambient Temp.') + ":")).appendTo(tbody);
    $('<td />', { id: 'ambtemp_id' + idx, align: 'left' }).appendTo(tr);
    tr = $('<tr />').append($('<td />', { align: 'left' }).append(i18n('State of Charge') + ":")).appendTo(tbody);
    $('<td />', { id: 'soc_id' + idx, align: 'left' }).appendTo(tr);

    fragment.appendChild(div[0]);
  }

  function create_status_tab(fragment, idx, x, y)
  {
    var div = create_tab('Status', idx, x, y);
    fragment.appendChild(div[0]);
  }

  function create_alarms_tab(fragment, idx, x, y)
  {
    var div = create_tab('Alarms', idx, x, y);
    fragment.appendChild(div[0]);
  }

  function create_info_tab(fragment, idx, x, y)
  {
    var div = create_tab('Info', idx, x, y);
    var tr, table, tbody;

    table = $('<table />', { id: 'id_info' + idx, 'class': 'table table-condensed table-hover' }).appendTo(div);
    tbody = $('<tbody />').appendTo(table);

    tr = $('<tr />').append($('<td />', { align: 'left' }).append(i18n('SN') + ":")).appendTo(tbody);
    $('<td />', { id: 'sn_id' + idx, align: 'left' }).appendTo(tr);
    tr = $('<tr />').append($('<td />', { align: 'left' }).append(i18n('PN') + ":")).appendTo(tbody);
    $('<td />', { id: 'pn_id' + idx, align: 'left' }).appendTo(tr);
    tr = $('<tr />').append($('<td />', { align: 'left' }).append(i18n('SW Version') + ":")).appendTo(tbody);
    $('<td />', { id: 'swver_id' + idx, align: 'left' }).appendTo(tr);
    tr = $('<tr />').append($('<td />', { align: 'left' }).append(i18n('HW Version') + ":")).appendTo(tbody);
    $('<td />', { id: 'hwver_id' + idx, align: 'left' }).appendTo(tr);

    fragment.appendChild(div[0]);
  }

  function create_all_tabs()
  {
    var div, button, fragment = document.createDocumentFragment();

    var i, packs = valtable_obj.NARADA_BATTPACKS, x = x_pos + 20, y = y_pos + 90;

   // packs = 6;
    for (i = 0; i < packs; i++)
    {
       if (i > 3) // zweite reihe
       {
         y = y_pos + 340;
         x = x_pos + 20 + ((i - 4) * 280);
       }
       else
       {
         x = x_pos + 20 + (i * 280);
       }

       css_obj = { position: 'absolute', top: y - 34, left: x, align: 'center', width: 266, 'z-index': -2 }; // 'background-color': '#3a87ad'
       attr_obj = { id: 'head_div_id' + i, 'class': 'list-group' };
       var head_div = $('<div/>').attr(attr_obj).css(css_obj);
       fragment.appendChild(head_div[0]);
       var a = $('<a/>', { 'class': 'list-group-item active', id: 'head_name_id' + i }).append('Pack ' + (i + 1)).appendTo(head_div);

       css_obj = { position: 'absolute', top: y, left: x };
       div = $('<div/>', {'class': "tab"}).css(css_obj);

       button = $('<button>', { id: 'Data' + i, 'class': "tablinks" }).append('Data').appendTo(div);
       button.click(function (event) {
           var id = 'tab' + $(this).attr('id');
           openTab(event, id);
           event.preventDefault();
         });
       button = $('<button>', { id: 'Status' + i, 'class': "tablinks" }).append('Status').appendTo(div);
       button.click(function (event) {
           var id = 'tab' + $(this).attr('id');
           openTab(event, id);
           event.preventDefault();
         });
       button = $('<button>', { id: 'Alarms' + i, 'class': "tablinks" }).append('Alarms').appendTo(div);
       button.click(function (event) {
           var id = 'tab' + $(this).attr('id');
           openTab(event, id);
           event.preventDefault();
         });
       button = $('<button>', { id: 'Info' + i, 'class': "tablinks" }).append('Info').appendTo(div);
       button.click(function (event) {
           var id = 'tab' + $(this).attr('id');
           openTab(event, id);
           event.preventDefault();
         });
       fragment.appendChild(div[0]);

       create_data_tab(fragment, i, x, y);
       create_status_tab(fragment, i, x, y);
       create_alarms_tab(fragment, i, x, y);
       create_info_tab(fragment, i, x, y);
    }
    document.body.appendChild(fragment);

    bg_width = x_pos + (packs > 4 ? 4 * 285 : packs * 290);
    bg_height = y + 220;

    // open all data tabs
    for (i = 0; i < packs; i++)
    {
      document.getElementById('Data' + i).click();
    }
  }

  /////////////////////////////////////////////////

  function set_data_tab(idx)
  {
    $('#volt_id' + idx).text(get_value({ values: valtable_obj.NARADA_PACKVOLT, index: idx, unit: ' V', precision: 1 }));
    $('#curr_id' + idx).text(get_value({ values: valtable_obj.NARADA_CURRENT, index: idx, unit: ' A', precision: 1 }));
    $('#cap_id' + idx).text(get_value({ values: valtable_obj.NARADA_REMCAPAH, index: idx, unit: ' Ah', precision: 1 }));
    $('#celltemp_id' + idx).html(get_value({ values: valtable_obj.NARADA_AVGCELLT, index: idx, unit: ' &deg;C', precision: 1 }));
    $('#ambtemp_id' + idx).html(get_value({ values: valtable_obj.NARADA_ENVT, index: idx, unit: ' &deg;C', precision: 1 }));
    $('#soc_id' + idx).text(get_value({ values: valtable_obj.NARADA_SOC, index: idx, unit: ' %', precision: 1 }));
  }

  function set_status_tab(idx)
  {
    var div = document.getElementById('tabStatus' + idx);
    var tr, table, tbody;

    $('#id_status' + idx).remove();

    table = $('<table />', { id: 'id_status' + idx, 'class': 'table table-condensed table-hover' }).appendTo(div);
    tbody = $('<tbody />').appendTo(table);

    var faultstate = valtable_obj.NARADA_FAULTSTATE[idx];

    if (faultstate & 0x100)
    {
      tr = $('<tr />').append($('<td />').append(i18n('Charging'))).appendTo(tbody);
    }
    if (faultstate & 0x200)
    {
      tr = $('<tr />').append($('<td />').append(i18n('Discharging'))).appendTo(tbody);
    }
    if (faultstate & 0x400)
    {
      tr = $('<tr />').append($('<td />').append(i18n('Chg MOS Connect'))).appendTo(tbody);
    }
    else
    {
      tr = $('<tr />').append($('<td />').append(i18n('Chg MOS Disconnect'))).appendTo(tbody);
    }
    if (faultstate & 0x800)
    {
      tr = $('<tr />').append($('<td />').append(i18n('Disg MOS Connect'))).appendTo(tbody);
    }
    else
    {
      tr = $('<tr />').append($('<td />').append(i18n('Disg MOS Disconnect'))).appendTo(tbody);
    }
    if (faultstate & 0x1000)
    {
      tr = $('<tr />').append($('<td />').append(i18n('Limit Current Enable'))).appendTo(tbody);
    }
    else
    {
      tr = $('<tr />').append($('<td />').append(i18n('Limit Current Disable'))).appendTo(tbody);
    }
  }

  function set_alarms_tab(idx)
  {
    var div = document.getElementById('tabAlarms' + idx);
    var tr, table, tbody;

    $('#id_alarms' + idx).remove();

    table = $('<table />', { id: 'id_alarms' + idx, 'class': 'table table-condensed table-hover' }).appendTo(div);
    tbody = $('<tbody />').appendTo(table);

    var faultstate = valtable_obj.NARADA_FAULTSTATE[idx];

    if ((faultstate & (0x1 | 0x2 | 0x4)) == 0)
    {
      tr = $('<tr />').append($('<td />').append(i18n('NONE'))).appendTo(tbody);
    }
    else
    {
       if (faultstate & 0x1)
       {
         tr = $('<tr />').append($('<td />').append(i18n('Front-end Sample Error'))).appendTo(tbody);
       }
       if (faultstate & 0x2)
       {
         tr = $('<tr />').append($('<td />').append(i18n('Temp Sense Disconnect'))).appendTo(tbody);
       }
       if (faultstate & 0x4)
       {
         tr = $('<tr />').append($('<td />').append(i18n('Inversed Graft Error'))).appendTo(tbody);
       }
    }
  }

  function set_info_tab(idx)
  {
    var sn, pn, sw, hw;

    if (idx == 0)
    {
      sn = valtable_obj.NARADA1_SERIAL;
      pn = valtable_obj.NARADA1_PRODSTYLE;
      sw = valtable_obj.NARADA1_SWVER;
      hw = valtable_obj.NARADA1_HWVER;
    }
    else if (idx == 1)
    {
      sn = valtable_obj.NARADA2_SERIAL;
      pn = valtable_obj.NARADA2_PRODSTYLE;
      sw = valtable_obj.NARADA2_SWVER;
      hw = valtable_obj.NARADA2_HWVER;
    }
    else if (idx == 2)
    {
      sn = valtable_obj.NARADA3_SERIAL;
      pn = valtable_obj.NARADA3_PRODSTYLE;
      sw = valtable_obj.NARADA3_SWVER;
      hw = valtable_obj.NARADA3_HWVER;
    }
    else if (idx == 3)
    {
      sn = valtable_obj.NARADA4_SERIAL;
      pn = valtable_obj.NARADA4_PRODSTYLE;
      sw = valtable_obj.NARADA4_SWVER;
      hw = valtable_obj.NARADA4_HWVER;
    }
    else if (idx == 4)
    {
      sn = valtable_obj.NARADA5_SERIAL;
      pn = valtable_obj.NARADA5_PRODSTYLE;
      sw = valtable_obj.NARADA5_SWVER;
      hw = valtable_obj.NARADA5_HWVER;
    }
    else if (idx == 5)
    {
      sn = valtable_obj.NARADA6_SERIAL;
      pn = valtable_obj.NARADA6_PRODSTYLE;
      sw = valtable_obj.NARADA6_SWVER;
      hw = valtable_obj.NARADA6_HWVER;
    }
    else if (idx == 6)
    {
      sn = valtable_obj.NARADA7_SERIAL;
      pn = valtable_obj.NARADA7_PRODSTYLE;
      sw = valtable_obj.NARADA7_SWVER;
      hw = valtable_obj.NARADA7_HWVER;
    }
    else if (idx == 7)
    {
      sn = valtable_obj.NARADA8_SERIAL;
      pn = valtable_obj.NARADA8_PRODSTYLE;
      sw = valtable_obj.NARADA8_SWVER;
      hw = valtable_obj.NARADA8_HWVER;
    }
    $('#sn_id' + idx).text(sn);
    $('#pn_id' + idx).text(pn);
    $('#swver_id' + idx).text(sw);
    $('#hwver_id' + idx).html(hw);
  }

  function set_all_tabs()
  {
    var i, packs = valtable_obj.NARADA_BATTPACKS;

    //packs = 6;
    for (i = 0; i < packs; i++)
    {
       set_data_tab(i);
       set_status_tab(i);
       set_alarms_tab(i);
       set_info_tab(i);
    }
  }

  function openTab(evt, tabname)
  {
     // Declare all variables
     var i, tabcontent, tablinks, idx;

     idx = tabname[tabname.length - 1];
     // Get all elements with class="tabcontent" and right id and hide them
     tabcontent = document.getElementsByClassName("tabcontent");
     for (i = 0; i < tabcontent.length; i++)
     {
       var id = tabcontent[i].id;
       if (id[id.length - 1] == idx)
         tabcontent[i].style.display = "none";
     }

     // Get all elements with class="tablinks" and right id and remove the class "active"
     tablinks = document.getElementsByClassName("tablinks");
     for (i = 0; i < tablinks.length; i++)
     {
       var id = tablinks[i].id;
       if (id[id.length - 1] == idx)
          tablinks[i].className = tablinks[i].className.replace(" active", "");
     }

     // Show the current tab, and add an "active" class to the button that opened the tab
     document.getElementById(tabname).style.display = "block";
     evt.currentTarget.className += " active";
  }

  /////////////////////////////////////////////////

  function get_value(val_obj, na_string)
  {
    var val = null;

    if (!val_obj.values || (val_obj.values.length <= val_obj.index))
      return na_string == null ? 'N/A' : na_string;

    val = val_obj.values[val_obj.index];
    if (val == null || val == -9999)
      return na_string == null ? 'N/A' : na_string;
    if (ups_protocol == prot_snmp && val == -1)
      return na_string == null ? 'N/A' : na_string;

    val = val.toFixed(val_obj.precision);
    val += val_obj.unit;
    return val;
  }

  function set_model_name() {
    var model_name = valtable_obj.MODEL;
    $('#model_name_id').html(model_name);
  }
}(jQuery));

