/*
   Copyright (c) 2019, Generex GmbH. All rights reserved.
   UW.
*/

var narada_api = (function ($) {
  var oid = get_oid_from_url(),
    cb = null,
    pannes_created = false,
    err_div = null,
    check_bowser = true,
    css_obj, attr_obj,
    valtable_obj = null,
    report_obj = null,
    ups_obj = null,
    unms_obj = null,
    inphases = null,
    ups_protocol = 0,
    prot_snmp = 0x803A,
    bg_height, bg_width;
    x_pos = 10, y_pos = 10;

  var req_params = null;

  /*public functions*/
  return {
    init: init
  };

  function init() {
    if (check_bowser && !utils_api.check_bowser()) {
      $('#loading_id').css('display', 'none');
      panes_api.create_br_check_panel(this, browser_check_ok);
      return true;
    }
    create_err_msg_panel();

    req_params = utils_api.get_req_params(utils_api.device_ups, oid);
    cb = jqmodule.subscribe(this, { update: update, error: err_funct }, req_params);
  }

  function browser_check_ok() {
    check_bowser = false;
    init();
  }

  function create_err_msg_panel() {
    css_obj = { position: 'absolute', top: y_pos, left: x_pos, width: 791, display: 'none', 'z-index': 62 };
    err_div = panes_api.create_err_panel({ id: 'error_div', 'class': 'alert alert-danger' }, css_obj);
  }

  function err_funct(response) {
    jqmodule.refresh(cb, 10 * 1000);
    panes_api.display_err({ response: response, oid: oid, err_div: err_div, err_msg_div_id: 'error_msg_div' });
  }

  function update(data) {
    if (data.ups == null) {
      return;
    }

    set_variable(data);
    jqmodule.refresh(cb, report_obj.refresh * 1000);
    if (err_div)
      err_div.css('display', 'none');

    if (window.menu_api != null)
      menu_api.update(data);

    unms_obj = get_unms_obj(data);
    if (unms_obj != null)
    {
      info_head_api.update(unms_obj);
    }

    if (!pannes_created)
    {
      pannes_created = true;
      create_panes();
      return true;
    }

    set_values();
  }

  function set_values() {
    set_all_values();
    create_status_panes();
    create_protection_panes();
    set_model_name();
    set_footer();
  }

  function create_footer_pane() {
    if (!oid || !unms_obj)
      return true;
    footer_api.create({ //Attr
        id: 'panel2'
        //'class':'unmsFooter'
      }, { //css
        position: 'absolute',
        top: '540px',
        left: '74px'
      },
      unms_obj
    );
  }

  function set_footer() {
    if (!oid || !unms_obj)
      return true;
    footer_api.set(unms_obj);
  }

  function set_variable(data)
  {
    ups_obj = data.ups;
    report_obj = data.report;
    valtable_obj = ups_obj.valtable;

    if (valtable_obj._upsProtocol == null)
    {
      ups_protocol = valtable_obj.CONNECT & 0XFFFF;
    }
    else
    {
      ups_protocol = valtable_obj._upsProtocol;
    }

    bg_height = 500;
    bg_width = 800;
  }

  function create_panes() {
    utils_api.setTitle(valtable_obj._platform);
    create_bg();
    create_values_panes();
    create_status_panes();
    create_protection_panes();
    create_model_panes();
    create_footer_pane();
  }

  function create_bg() {
    var fragment = document.createDocumentFragment(),
      img_div;

    css_obj = { position: 'absolute', top: y_pos, left: x_pos, width: bg_width, height: bg_height, border: 'solid 1px #808080' };
    img_div = $('<div/>', { 'class': 'img-rounded' }).css(css_obj);
    fragment.appendChild(img_div[0]);
    document.body.appendChild(fragment);
  }

  function create_model_panes() {
    var model_div, a;
    css_obj = { position: 'absolute', top: y_pos, left: x_pos, align: 'center', width: bg_width, 'z-index': 40 }; // 'background-color': '#3a87ad'
    attr_obj = { id: 'model_div_id', 'class': 'list-group' };
    model_div = panes_api.create_div(attr_obj, css_obj);
    a = $('<a/>', { 'class': 'list-group-item active', id: 'model_name_id' }).appendTo(model_div);
    set_model_name();
  }

  /////////////////////////////////////////////////

  function create_protection_panes()
  {
    var val, tr, table, tbody, thead, div,
      fragment = document.createDocumentFragment();

    $('#id_protectiontable').remove();

    css_obj = { position: 'absolute', top: y_pos + 100, left: x_pos + 580 };
    div = $('<div/>').css(css_obj);
    table = $('<table />', { id: 'id_protectiontable', 'class': 'table table-condensed table-hover table-bordered' }).appendTo(div);
    tbody = $('<tbody />').appendTo(table);
    thead = $('<thead />').appendTo(table);
    tr = $('<tr />').append($('<th />').css('text-align', 'center').append(i18n('Protection Flag'))).appendTo(thead);

    if (valtable_obj.ProtectionFlag & 0x10)
    {
      tr = $('<tr />').append($('<td />').append(i18n('SC Protect'))).appendTo(tbody);
    }
    if (valtable_obj.ProtectionFlag & 0x20)
    {
      tr = $('<tr />').append($('<td />').append(i18n('OC Protect'))).appendTo(tbody);
    }
    if (valtable_obj.ProtectionFlag & 0x40)
    {
      tr = $('<tr />').append($('<td />').append(i18n('Chg OT Protect'))).appendTo(tbody);
    }
    if (valtable_obj.ProtectionFlag & 0x80)
    {
      tr = $('<tr />').append($('<td />').append(i18n('Chg UT Protect'))).appendTo(tbody);
    }
    if (valtable_obj.ProtectionFlag & 0x100)
    {
      tr = $('<tr />').append($('<td />').append(i18n('Disg OT Protect'))).appendTo(tbody);
    }
    if (valtable_obj.ProtectionFlag & 0x200)
    {
      tr = $('<tr />').append($('<td />').append(i18n('Disg UT Protect'))).appendTo(tbody);
    }

    fragment.appendChild(div[0]);
    document.body.appendChild(fragment);
  }

  /////////////////////////////////////////////////

  function create_status_panes()
  {
    var val, tr, table, tbody, thead, div,
      fragment = document.createDocumentFragment();

    $('#id_statustable').remove();

    css_obj = { position: 'absolute', top: y_pos + 100, left: x_pos + 350 };
    div = $('<div/>').css(css_obj);
    table = $('<table />', { id: 'id_statustable', 'class': 'table table-condensed table-hover table-bordered' }).appendTo(div);
    tbody = $('<tbody />').appendTo(table);
    thead = $('<thead />').appendTo(table);
    tr = $('<tr />').append($('<th />').css('text-align', 'center').append(i18n('Faults/Status'))).appendTo(thead);

    if (valtable_obj.STATUS & 0x1)
    {
      tr = $('<tr />').append($('<td />').append(i18n('Front-end Sample Error'))).appendTo(tbody);
    }
    if (valtable_obj.STATUS & 0x2)
    {
      tr = $('<tr />').append($('<td />').append(i18n('Temp Sense Disconnect'))).appendTo(tbody);
    }
    if (valtable_obj.STATUS & 0x4)
    {
      tr = $('<tr />').append($('<td />').append(i18n('Inversed Graft Error'))).appendTo(tbody);
    }
    if (valtable_obj.STATUS & 0x100)
    {
      tr = $('<tr />').append($('<td />').append(i18n('Charging'))).appendTo(tbody);
    }
    if (valtable_obj.STATUS & 0x200)
    {
      tr = $('<tr />').append($('<td />').append(i18n('Discharging'))).appendTo(tbody);
    }
    if (valtable_obj.STATUS & 0x400)
    {
      tr = $('<tr />').append($('<td />').append(i18n('Chg MOS Connect'))).appendTo(tbody);
    }
    else
    {
      tr = $('<tr />').append($('<td />').append(i18n('Chg MOS Disconnect'))).appendTo(tbody);
    }
    if (valtable_obj.STATUS & 0x800)
    {
      tr = $('<tr />').append($('<td />').append(i18n('Disg MOS Connect'))).appendTo(tbody);
    }
    else
    {
      tr = $('<tr />').append($('<td />').append(i18n('Disg MOS Disconnect'))).appendTo(tbody);
    }
    if (valtable_obj.STATUS & 0x1000)
    {
      tr = $('<tr />').append($('<td />').append(i18n('Limit Current Enable'))).appendTo(tbody);
    }
    else
    {
      tr = $('<tr />').append($('<td />').append(i18n('Limit Current Disable'))).appendTo(tbody);
    }
    fragment.appendChild(div[0]);
    document.body.appendChild(fragment);
  }

  /////////////////////////////////////////////////

  function create_values_panes()
  {
    var val, tr, table, tbody, thead, div,
      fragment = document.createDocumentFragment();

    css_obj = { position: 'absolute', top: y_pos + 100, left: x_pos + 50 };
    div = $('<div/>').css(css_obj);
    table = $('<table />', { 'class': 'table table-condensed table-hover table-bordered' }).appendTo(div);
    tbody = $('<tbody />').appendTo(table);
    thead = $('<thead />').appendTo(table);
    tr = $('<tr />').append($('<th />', {colspan: 2 }).css('text-align', 'center').append(i18n('Present Values'))).appendTo(thead);

    tr = $('<tr />').append($('<td />').append(i18n('Pack Volt:'))).appendTo(tbody);
    $('<td />', { id: 'id_packvolt', align: 'right', width:100 }).appendTo(tr);
    tr = $('<tr />').append($('<td />').append(i18n('Current:'))).appendTo(tbody);
    $('<td />', { id: 'id_current', align: 'right', width:100 }).appendTo(tr);
    tr = $('<tr />').append($('<td />').append(i18n('Remain Capacity:'))).appendTo(tbody);
    $('<td />', { id: 'id_remcap', align: 'right', width:100 }).appendTo(tr);
    tr = $('<tr />').append($('<td />').append(i18n('Avg Cell Temp:'))).appendTo(tbody);
    $('<td />', { id: 'id_celltemp', align: 'right', width:100 }).appendTo(tr);
    tr = $('<tr />').append($('<td />').append(i18n('Env Temp:'))).appendTo(tbody);
    $('<td />', { id: 'id_envtemp', align: 'right', width:100 }).appendTo(tr);
    tr = $('<tr />').append($('<td />').append(i18n('State of charge:'))).appendTo(tbody);
    $('<td />', { id: 'id_soc', align: 'right', width:100 }).appendTo(tr);

    fragment.appendChild(div[0]);
    document.body.appendChild(fragment);

    set_all_values();
  }

  function set_all_values()
  {
    $('#id_packvolt').text(get_value({ values: [valtable_obj.Voltage / 100.0], index: 0, unit: ' V', precision: 1 }));
    $('#id_current').text(get_value({ values: [(valtable_obj.Current - 10000) / 10.0], index: 0, unit: ' A', precision: 1 }));
    $('#id_remcap').text(get_value({ values: [(valtable_obj.CapacityAh) / 10.0], index: 0, unit: ' Ah', precision: 1 }));
    $('#id_celltemp').html(get_value({ values: [(valtable_obj.AverageCellTemp - 400) / 10.0], index: 0, unit: ' &deg;C', precision: 1 }));
    $('#id_envtemp').html(get_value({ values: [(valtable_obj.EnvTemp - 400) / 10.0], index: 0, unit: ' &deg;C', precision: 1 }));
    $('#id_soc').text(get_value({ values: [(valtable_obj.SOC) / 100.0], index: 0, unit: ' %', precision: 1 }));
  }

  function get_value(val_obj, na_string)
  {
    var val = null;

    if (!val_obj.values || (val_obj.values.length <= val_obj.index))
      return na_string == null ? 'N/A' : na_string;

    val = val_obj.values[val_obj.index];
    if (val == null || val == -9999)
      return na_string == null ? 'N/A' : na_string;
    if (ups_protocol == prot_snmp && val == -1)
      return na_string == null ? 'N/A' : na_string;

    val = val.toFixed(val_obj.precision);
    val += val_obj.unit;
    return val;
  }

  function set_model_name() {
    var model_name = valtable_obj.MODEL;
    $('#model_name_id').html(model_name);
  }
}(jQuery));

