/*
   nextys_panel.js - Nextys DCW20 screen.
   Copyright (c) 2019, Generex GmbH. All rights reserved.
   Said/UW.
*/

var nextys_api = (function ($) {
  var oid = get_oid_from_url(),
    cb = null,
    pannes_created = false,
    err_div = null,
    check_bowser = true,
    css_obj, attr_obj,
    valtable_obj = null,
    report_obj = null,
    ups_obj = null,
    unms_obj = null,
    status, tempmax, tempdeg, overload = 0,
    plattform = get_plattform(),
    ups_id = 0,
    ups_protocol = 0,
    prot_snmp = 0x803A,
    bg_height, bg_width;
    x_pos = 10, y_pos = 10;

  var status_obj = [

      { bit: 0x0008, txt: "The UPS is drawing power from the batteries" },
      { bit: 0x0010, txt: "The remaining battery run-time is low" },
      { bit: 0x0400, txt: "One or more batteries have been determined to require replacement" },
      { bit: 0x0020, txt: "A temperature is out of tolerance" },
      { bit: 0x0080, txt: "Input high" },
      { bit: 0x0100, txt: "The output load exceeds the UPS output capacity" },
      { bit: 0x2000, txt: "A general fault in the UPS has been detected" },
      { bit: 0x4000, txt: "A problem has been encountered in the communications between the agent and the UPS" },
      { bit: 0x0200, txt: "Inverter error" },
      { bit: 0x1000, txt: "Inverter warning" },
      { bit: 0x0002, txt: "UPS is in shutdown mode" },
      { bit: 0x0001, txt: "The Bypass is presently engaged on the UPS" },
      { bit: 0x0040, txt: "A test is in progress" },
      { bit: 0x0004, txt: "UPS Output active" },
      { bit: 0x0800, txt: "UPS in ECO mode" }
    ],
    ext_status_obj = [
      { bit: 0x00000008, txt: "The UPS will be unable to sustain the present load when and if the utility power is lost" },
      { bit: 0x00000040, txt: "An output condition (other than OutputOverload) is out of tolerance" },
      { bit: 0x00000200, txt: "The Bypass is out of tolerance" },
      { bit: 0x00000400, txt: "The UPS has shutdown as requested, i.e., the output is off" },
      { bit: 0x00000800, txt: "The entire UPS has shutdown as commanded" },
      { bit: 0x00001000, txt: "An uncorrected problem has been detected within the UPS charger subsystem" },
      { bit: 0x00002000, txt: "The output of the UPS is in the off state" },
      { bit: 0x00004000, txt: "The UPS system is in the off state" },
      { bit: 0x00008000, txt: "The failure of one or more fans in the UPS has been detected" },
      { bit: 0x00010000, txt: "The failure of one or more fuses has been detected" },
      { bit: 0x00040000, txt: "The result of the last diagnostic test indicates a failure" },
      { bit: 0x00100000, txt: "The UPS output is off and the UPS is awaiting the return of input power" },
      { bit: 0x00200000, txt: "An UPS Shutdown After Delay countdown is underway" },
      { bit: 0x00400000, txt: "The UPS will turn off power to the load immediatly" }
    ];

  var req_params = null;

  /*public functions*/
  return {
    init: init
  };

  function init() {
    if (check_bowser && !utils_api.check_bowser()) {
      $('#loading_id').css('display', 'none');
      panes_api.create_br_check_panel(this, browser_check_ok);
      return true;
    }
    create_err_msg_panel();

    req_params = utils_api.get_req_params(utils_api.device_ups, oid);
    cb = jqmodule.subscribe(this, { update: update, error: err_funct }, req_params);
  }

  function browser_check_ok() {
    check_bowser = false;
    init();
  }

  function create_err_msg_panel() {
    css_obj = { position: 'absolute', top: y_pos, left: x_pos, width: 791, display: 'none', 'z-index': 62 };
    err_div = panes_api.create_err_panel({ id: 'error_div', 'class': 'alert alert-danger' }, css_obj);
  }

  function err_funct(response) {
    jqmodule.refresh(cb, 10 * 1000);
    panes_api.display_err({ response: response, oid: oid, err_div: err_div, err_msg_div_id: 'error_msg_div' });
  }

  function update(data) {
    if (data.ups == null) {
      return;
    }

    set_variable(data);
    jqmodule.refresh(cb, report_obj.refresh * 1000);
    if (err_div)
      err_div.css('display', 'none');

    if (window.menu_api != null)
      menu_api.update(data);

    unms_obj = get_unms_obj(data);
    if (unms_obj != null) {
      info_head_api.update(unms_obj);
    }

    if (!pannes_created) {
      pannes_created = true;
      create_panes();
      return true;
    }

    set_values();
  }

  function set_values() {
    set_input_values(); ////Input values
    set_output_values(); //Output values
    set_battery_value(); //Battery value
    set_iby_ire_ssi_inv_rec_batt(); //SET IBY, IRE, SSI, INVERTER, RECTIFIER and BATTERY pannes
    set_ups_status(); //set UPS status
    set_outpower_loadbars(); //outpower loadbars
    set_model_name();
    set_sys_mod_status();
    if (plattform !== utils_api.plattform_ups_view)
      panes_api.set_rom_temp_values(valtable_obj); //set rom temp values
    set_footer();
    panes_api.set_autonomy_time_infos(valtable_obj);
  }

  function create_footer_pane() {
    if (!oid || !unms_obj)
      return true;
    footer_api.create({ //Attr
        id: 'panel2'
        //'class':'unmsFooter'
      }, { //css
        position: 'absolute',
        top:  '540px',
        left: '74px'
      },
      unms_obj
    );
  }

  function set_footer() {
    if (!oid || !unms_obj)
      return true;
    footer_api.set(unms_obj);
  }

  function set_variable(data) {
    ups_obj = data.ups;
    report_obj = data.report;
    valtable_obj = ups_obj.valtable;
    status = valtable_obj.STATUS;
    tempmax = valtable_obj.TMPMAX;
    tempdeg = valtable_obj.TEMPDEG;
    level = valtable_obj.LEVEL;

    if (valtable_obj._upsProtocol == null) {
      ups_protocol = valtable_obj.CONNECT & 0XFFFF;
    } else {
      ups_protocol = valtable_obj._upsProtocol;
    }

    bg_height = 500;
    bg_width = 818;

    if (valtable_obj.OUTPOWER != null)
    {
      if (valtable_obj.OUTPOWER[0] > 100)
        overload = 1;
      else
        overload = 0;
    }
  }

  function create_panes() {
    utils_api.setTitle(valtable_obj._platform);
    create_bg(); //background
    create_input_pane(); //Input table
    create_output_pane(); //Outnput   panel
    create_battery_pane(); //Battery panel
    create_iby_ire_ssi_inv_rec_batt(); //create IBY, IRE, SSI, INVERTER, RECTIFIER and BATTERY pannes
    create_status_pane(); //UPS status
    create_model_panes();
    create_outpower_loadbars(); //outpower loadbars
    if (plattform !== utils_api.plattform_ups_view) {
      create_rom_temp(); //rom temp
    }
    create_footer_pane();
    create_sys_mod_status_panel();
    panes_api.create_autonomy_time_infos_panel('autonomy_time_label', valtable_obj);
  }

  function create_bg() {
    var fragment = document.createDocumentFragment(),
      img_div, bg_img, bg_img_src = './image/ups/bgn_module.png';

    css_obj = { position: 'absolute', top: y_pos, left: x_pos, width: bg_width, height: bg_height, border: 'solid 1px #808080' };
    img_div = $('<div/>', { 'class': 'img-rounded' }).css(css_obj);
    bg_img = $('<img />').attr({ 'id': 'bg_img_id', 'class': 'img-rounded', width: 816, 'height': 459, src: bg_img_src }).appendTo(img_div);
    fragment.appendChild(img_div[0]);
    document.body.appendChild(fragment);
  }

  function create_model_panes() {
    var model_div, a;
    css_obj = { position: 'absolute', top: y_pos, left: x_pos, align: 'center', width: bg_width, 'z-index': 40 }; // 'background-color': '#3a87ad'
    attr_obj = { id: 'model_div_id', 'class': 'list-group' };
    model_div = panes_api.create_div(attr_obj, css_obj);
    a = $('<a/>', { 'class': 'list-group-item active', id: 'model_name_id' }).appendTo(model_div);
    set_model_name();
  }

  function create_input_pane() //Input table
  {
    var fragment = document.createDocumentFragment(),
        tr, table, tbody, thead, in_div;

    css_obj = { position: 'absolute', top: y_pos + 160, left: x_pos + 30 };

    in_div = $('<div/>', { id: 'input_div_id' }).css(css_obj);
    table = $('<table />', { 'class': 'table table-condensed table-hover', 'id': 'inptable' }).appendTo(in_div); // table-hover
    tbody = $('<tbody />').appendTo(table);
    thead = $('<thead />').appendTo(table);
    tr = $('<tr />').append($('<th />', { colspan: 4 }).append(i18n(lang_Input))).appendTo(thead);
    table.css('margin-top', '40px');

    tr = $('<tr />', { 'id': 'inptrl1' }).appendTo(tbody);
    $('<td />', { id: 'input_volt1_id' }).appendTo(tr);
    $('<td />', { id: 'input_cur1_id' }).appendTo(tr);

    fragment.appendChild(in_div[0]);
    document.body.appendChild(fragment);
    set_input_values();
  }

  function set_input_values()
  {
    $('#input_volt1_id').text(get_value({ values: [valtable_obj.INVOLT[0]], index: 0, unit: ' V', precision: 1 }));
    $('#input_cur1_id').text(get_value({ values: [valtable_obj.INCURR[0]], index: 0, unit: ' A', precision: 1 }));
  }

  function create_output_pane() //Output
  {
    var output_div = null,
      tr, table, tbody, thead,
      fragment = document.createDocumentFragment();

    css_obj = { position: 'absolute', top: y_pos + 160, left: x_pos + 650 };
    output_div = $('<div/>', { id: 'output_div_id' }).css(css_obj);
    table = $('<table />', { 'class': 'table table-condensed table-hover', 'id': 'outtable' }).appendTo(output_div);
    tbody = $('<tbody />').appendTo(table);
    thead = $('<thead />').appendTo(table);
    tr = $('<tr />').append($('<th />', { colspan: 4 }).append(i18n(lang_Output))).appendTo(thead);
    table.css('margin-top', '40px');

    tr = $('<tr />', { 'id': 'outtrl1' }).appendTo(tbody);
    $('<td />', { 'id': 'out_volt1_id' }).appendTo(tr);
    $('<td />', { id: 'out_cur1_id' }).appendTo(tr);

    fragment.appendChild(output_div[0]);
    document.body.appendChild(fragment);
    set_output_values();
  }

  function set_output_values() //Output values
  {
    $('#out_volt1_id').text(get_value({ values: [valtable_obj.OUTPUT_VOLT[0]], index: 0, unit: ' V', precision: 1 }));
    $('#out_cur1_id').text(get_value({ values: [valtable_obj.EX_OUT_CURR[0]], index: 0, unit: ' A', precision: 1 }));
  }

  //Standardscreen erweitern: Wenn EX_BATT_VOLTPOS und EX_BATT_VOLTNEG vorhanden und ungleich UPS_VAL_NA (-9999) dann beide Werte anstatt BATTVOLT anzeigen.
  function create_battery_pane() //Battery panel
  {
    var val, tr, table, tbody, thead, batt_div,
      fragment = document.createDocumentFragment();

    css_obj = { position: 'absolute', top: y_pos + 306, left: x_pos + 198 }; //, width:130, height:100
    attr_obj = { id: 'battery_div_id' };
    batt_div = $('<div/>').attr(attr_obj).css(css_obj);
    table = table = $('<table />', { 'class': 'table table-condensed table-hover' }).appendTo(batt_div);
    tbody = $('<tbody />').appendTo(table);
    thead = $('<thead />').appendTo(table);
    tr = $('<tr />').append($('<th />', { colspan: 2 }).append(i18n(lang_Battery))).appendTo(thead);

    tr = $('<tr />').append($('<td />', { align: 'left' }).append(i18n(lang_Voltage) + ":")).appendTo(tbody);
    $('<td />', { id: 'batt_volt_id', align: 'left' }).
    append(get_value({ values: [valtable_obj.BATTVOLT], index: 0, unit: ' V', precision: 2 })).appendTo(tr);

    tr = $('<tr />').append($('<td />', { align: 'left' }).append(i18n(lang_Current) + ":")).appendTo(tbody);
    $('<td />', { id: 'batt_currpos_id', align: 'left' }).
    append(get_value({ values: [valtable_obj.EX_BATT_CURRPOS], index: 0, unit: ' A', precision: 1 })).appendTo(tr);

    tr = $('<tr />').append($('<td />', { align: 'left', id: 'autonomy_time_label' }).append(i18n(lang_Time) + ":")).appendTo(tbody);
    $('<td />', { id: 'batt_autotime_id', align: 'left' }).
    append(get_value({ values: [valtable_obj.AUTONOMTIME], index: 0, unit: ' Min', precision: 0 })).appendTo(tr);

    tr = $('<tr />').append($('<td />', { align: 'left' }).append(i18n(lang_Battery_Capacity_short) + ":")).appendTo(tbody);
    $('<td />', { id: 'batt_cap_id', align: 'left' }).
    append(get_value({ values: [valtable_obj.BATTCAP], index: 0, unit: ' %', precision: 0 })).appendTo(tr);

    if (valtable_obj.EX_BATT_SECONBATT != null) {
      tr = $('<tr />').append($('<td />', { align: 'left' }).append(i18n('On Battery') + ":")).appendTo(tbody);
      $('<td />', { id: 'batt_seconds_id', align: 'left' }).
      append(get_value({ values: [valtable_obj.EX_BATT_SECONBATT], index: 0, unit: ' Sec', precision: 0 })).appendTo(tr);
    }

    fragment.appendChild(batt_div[0]);
    document.body.appendChild(fragment);
  }

  function set_battery_value() //set Battery value
  {
    $('#batt_volt_id').text(get_value({ values: [valtable_obj.BATTVOLT], index: 0, unit: ' V', precision: 2 }));
    $('#batt_currpos_id').text(get_value({ values: [valtable_obj.EX_BATT_CURRPOS], index: 0, unit: ' A', precision: 1 }));
    $('#batt_autotime_id').text(get_value({ values: [valtable_obj.AUTONOMTIME], index: 0, unit: ' Min', precision: 0 }));
    $('#batt_cap_id').text(get_value({ values: [valtable_obj.BATTCAP], index: 0, unit: ' %', precision: 0 }));
    if (valtable_obj.EX_BATT_SECONBATT != null)
      $('#batt_seconds_id').text(get_value({ values: [valtable_obj.EX_BATT_SECONBATT], index: 0, unit: ' Sec', precision: 0 }));
  }

  function get_value(val_obj, na_string) {
    var val = null;
    var a = null;

    if (!val_obj.values || (val_obj.values.length <= val_obj.index))
      return na_string == null ? 'N/A' : na_string;

    val = val_obj.values[val_obj.index];
    if (val == null || val == -9999)
      return na_string == null ? 'N/A' : na_string;
    if (ups_protocol == prot_snmp && val == -1)
      return na_string == null ? 'N/A' : na_string;

    val = val.toFixed(val_obj.precision);
    val += val_obj.unit;

    return val;
  }

  function create_iby_ire_ssi_inv_rec_batt() //create IBY, IRE, SSI, INVERTER, RECTIFIER and BATTERY  pannes
  {
    var div, fragment = document.createDocumentFragment();
    css_obj = { position: 'absolute', top: y_pos + 229, left: x_pos + 548, width: 24, height: 10, overflow: 'hidden' };

    div = $('<div/>').attr({ id: 'iby_div_id', 'class': 'img-rounded' }).css(css_obj); // load off
    fragment.appendChild(div[0]);


    css_obj = { position: 'absolute', top: y_pos + 229, left: x_pos + 220, width: 24, height: 10, overflow: 'hidden' };
    div = $('<div/>').attr({ id: 'ire_div_id', 'class': 'img-rounded' }).css(css_obj);
    fragment.appendChild(div[0]);
    panes_api.create_img_panel('rec_img_id', './image/ups/dcgr.png', x_pos + 286, y_pos + 210, 50, 50, 0); //RECTIFIER powerfail

    document.body.appendChild(fragment);

    panes_api.create_img_panel('inv_img_id', './image/ups/dcgr.png', x_pos + 458, y_pos + 210, 50, 50, 0); ////INVERTER
    panes_api.create_img_panel('batt_img_id', './image/ups/Batgrv.gif', x_pos + 372, y_pos + 294, 50, 50, 0); //BATTERY powerfail

    set_iby_ire_ssi_inv_rec_batt() //SET IBY, IRE, SSI, INVERTER, RECTIFIER and BATTERY  pannes
  }

  function set_iby_ire_ssi_inv_rec_batt() //SET IBY, IRE, SSI, INVERTER, RECTIFIER and BATTERY  pannes
  {
    if (valtable_obj.EX_EXTSTATUS != null && (valtable_obj.EX_EXTSTATUS & (0x2000 | 0x400))) // load off
    {
      $('#iby_div_id').css('background-color', '#DC3632'); //red
    } else if ((status & 16384) != 16384 && (status & 4) != 4) //IBY commlost and active
    {
      $('#iby_div_id').css('background-color', '#DC3632'); //red
    } else {
      $('#iby_div_id').css('background-color', 'rgb(40, 134, 59)'); //green
    }

    if ((status & 8) == 8 || (status & 128) == 128) //IRE powerfail or input-high
    {
      $('#ire_div_id').css('background-color', '#DC3632'); //red
    } else {
      $('#ire_div_id').css('background-color', 'rgb(40, 134, 59)'); //green
    }

    if ((status & 512) == 512 //INVERTER inverterfail
      ||
      (status & 4096) == 4096 //inverterwarn
      ||
      (status & 256) == 256 //outputhigh
      ||
      (status & 32) == 32 //overtemp
      ||
      (overload == 1)) //overload
    {
      $('#inv_img_id').attr('src', './image/ups/dcr.png');
    } else if ((level != 0) && ((tempdeg != null) && (tempdeg != "-9999") && (tempmax != null)) && (tempdeg > tempmax)) //overtemp1
    {
      $('#inv_img_id').attr('src', './image/ups/dcr.png');
    } else if ((status & 1) == 1) //bypass
    {
      $('#inv_img_id').attr('src', './image/ups/dcy.png');
    } else {
      $('#inv_img_id').attr('src', './image/ups/dcgr.png');
    }

    if ((status & 8) == 8) //RECTIFIER powerfail
    {
      $('#rec_img_id').attr('src', './image/ups/dcr.png');
    } else {
      $('#rec_img_id').attr('src', './image/ups/dcgr.png');
    }

    if ((status & 8) == 8) //BATTERY powerfail
    {
      $('#batt_img_id').attr('src', './image/ups/Batgrv.gif');
    } else if ((status & 16) == 16 || (status & 1024) == 1024) // batterylow batterybad
    {
      $('#batt_img_id').attr('src', './image/ups/Batrv.gif');
    } else {
      $('#batt_img_id').attr('src', './image/ups/Batgev.gif');
    }
  }

  function create_status_pane() //UPS status
  {
    var status_div = $('<div/>'),
      status_span;
    css_obj = { position: 'absolute', top: y_pos + 50, left: x_pos + 710, align: 'center', overflow: 'hidden' };

    $(status_div).css(css_obj);
    status_span = $('<span />', { id: 'ups_status_id', 'class': get_status_class(), title: i18n('Click to see Status Text') }).append(i18n('Status'));
    status_span.css('cursor', 'pointer');
    status_span.click(function (event) //visibility: hidden
      {
        var sys_mod_status_txt_div = $('#sys_mod_status_txt_div'),
          display = sys_mod_status_txt_div.css('display') == 'none' ? 'block' : 'none';
        sys_mod_status_txt_div.css({ display: display });
      });

    $('<h4 />').append(status_span).appendTo(status_div);
    status_div.appendTo(document.body);
  }

  function set_ups_status() //set UPS status
  {
    $('#ups_status_id').attr('class', get_status_class());
  }

  function get_status_class() {
    if (status & 0xF7BA)
      return 'label label-danger';

    if (status & 0x41)
      return 'label label-warning';

    return 'label label-success';
  }

  function create_outpower_loadbars() //outpower loadbars
  {
    if (valtable_obj.OUTPOWER == null)
      return true;

    panes_api.create_loadbar_pane({ value: valtable_obj.OUTPOWER[0], id: 'loadbar1_id', left: x_pos + 524, top: y_pos + 370, title: 'L1', scaler: 1 });
    set_outpower_loadbars();
  }

  function set_outpower_loadbars() //outpower loadbars
  {
    if (valtable_obj.OUTPOWER == null)
      return true;
    if (valtable_obj.OUTPOWER[0] != -9999) {
      panes_api.set_loabar_value(valtable_obj.OUTPOWER[0], 'loadbar1_id');
    }
  }

  function create_rom_temp() //rom temp
  {
    var fragment = document.createDocumentFragment(),
      rom_temp_div;

    css_obj = { position: 'absolute', top: y_pos + 306, left: x_pos + 8 };
    rom_temp_div = $('<div/>').attr({ id: 'rom_temp_div_id' }).css(css_obj);
    panes_api.create_rom_temp_pane(valtable_obj, { id: 'rom_temp_table_id', 'class': 'table table-condensed table-hover' }).appendTo(rom_temp_div);
    fragment.appendChild(rom_temp_div[0]);
    document.body.appendChild(fragment);
  }

  function create_sys_mod_status_panel() {
    var fragment = document.createDocumentFragment(),
      tr, tr, table, tbody, a,
      title = i18n("Status") + ":" + valtable_obj.MODEL,
      status_txt_array = get_sys_mod_status_txt(),
      sys_mod_status_txt_div;

    css_obj = {
      position: 'absolute',
      top: y_pos + 72,
      left: x_pos + 36,
      align: 'center',
      'background-color': '#F1F1F1',
      display: 'none',
      border: 'solid 1px black'
    };

    sys_mod_status_txt_div = $('<div/>', { id: 'sys_mod_status_txt_div', 'class': 'img-rounded' }).css(css_obj);

    a = $('<a/>', { 'class': 'list-group-item active', id: 'status_model_name_id' }).append(title).appendTo(sys_mod_status_txt_div);
    a.css('height', 30);
    table = $('<table />', { 'class': 'table table-condensed', id: 'sys_mod_status_txt_table' }).appendTo(sys_mod_status_txt_div); // table-hover
    table.css({ 'background-color': '#F1F1F1' })
    tbody = $('<tbody />', { id: 'sys_mod_status_txt_table_body' }).appendTo(table);
    $.each(status_txt_array, function (index, status_txt) {
      tr = $('<tr />').appendTo(tbody);
      td = $('<td />', { 'align': 'left' }).append(status_txt).appendTo(tr);
    });

    fragment.appendChild(sys_mod_status_txt_div[0]);
    document.body.appendChild(fragment);
    new utils_api.draggable('sys_mod_status_txt_div');
  }

  function set_sys_mod_status() {
    var tr, td,
      title = i18n("Status") + ":" + valtable_obj.MODEL,
      tbody = $('#sys_mod_status_txt_table_body'),
      status_txt_array = get_sys_mod_status_txt();

    if (ups_id != 0)
      title += " [" + i18n("Module") + " " + ups_id + "]";

    $('#status_model_name_id').text(title);

    tbody.empty();
    $.each(status_txt_array, function (index, status_txt) {
      tr = $('<tr />').appendTo(tbody);
      td = $('<td />', { 'align': 'left' }).append(status_txt).appendTo(tr);
    });
  }


  function get_sys_mod_status_txt() {
    var status_txt_array = [];
    $.each(status_obj, function (index, status_obj) {
      if (status_obj.bit & valtable_obj.STATUS) {
        status_txt_array.push(i18n(status_obj.txt));
      }
    });

    if (valtable_obj.EX_EXTSTATUS != null) {
      $.each(ext_status_obj, function (index, status_obj) {
        if (status_obj.bit & valtable_obj.EX_EXTSTATUS) {
          status_txt_array.push(i18n(status_obj.txt));
        }
      });
    }

    return status_txt_array;
  }

  function set_model_name() {
    var model_name = valtable_obj.MODEL;
    $('#model_name_id').html(model_name);
  }
}(jQuery));
