/*
   Woodward_dctl_panel.js - Woodward Diesel Control screen.
   Copyright (c) 2011, Generex GmbH. All rights reserved.
   Said.
*/

var pildiesel_api = (function ($) {
  var oid = get_oid_from_url(),
  cb = null,
  err_div = null,
  valtable_obj,
  report_obj, unms_obj,
  pannes_created = false;
  check_bowser = true,
  req_params = null;

  return {
    init: init
  };

  function init() {
    if (check_bowser && !utils_api.check_bowser()) {
      $('#loading_id').css('display', 'none');
      panes_api.create_br_check_panel(this, browser_check_ok);
      return true;
    }
    create_err_msg_panel();

    req_params = utils_api.get_req_params(utils_api.device_ups, oid);
    cb = jqmodule.subscribe(this, { update: update, error: err_funct }, req_params);
  }

  function browser_check_ok() {
    check_bowser = false;
    init();
  }

  function create_err_msg_panel() {
    css_obj = { position: 'absolute', top: 10, left: 10, width: 810, display: 'none', 'z-index': 10 };
    err_div = panes_api.create_err_panel({ id: 'error_div', 'class': 'alert alert-danger' }, css_obj);
  }

  function err_funct(response) {
    jqmodule.refresh(cb, 10 * 1000);
    panes_api.display_err({ response: response, oid: oid, err_div: err_div, err_msg_div_id: 'error_msg_div' });
  }

  function update(data) {
    if (data.length == 0) {
      return;
    }

    valtable_obj = data.ups.valtable;
    report_obj = data.report;

    jqmodule.refresh(cb, report_obj.refresh * 1000);
    if (err_div)
    err_div.css('display', 'none');

    if (window.menu_api != null)
    menu_api.update(data);

    unms_obj = get_unms_obj(data);
    if (unms_obj != null) {
      info_head_api.update(unms_obj);
    }

    if (!pannes_created) {
      create_panel(valtable_obj); //create Woodward Diesel Control screen.
      create_footer_pane();
      pannes_created = true;
      return;
    }
    set_input_1_8_values(valtable_obj); //set input 1-8 values
    set_ext_input_1_8_values(valtable_obj); //set ext input 1-8 values
    set_ext_input_1_16_values(valtable_obj); //set ext input 1-16 values
    set_footer();
  }

  function create_panel(valtable_obj) //create Woodward Diesel Control screen.
  {
    var tr, td, table, tbody, cssObj, location;

    location = valtable_obj.LOCATION;
    if (location == null || location == "")
    location = "Unknown"

    dctl_div = $('<div/>', { id: 'div_dctl_id', 'class': 'main', 'align': 'center' }).css('position', 'relative').css('top', '20px');

    //Title and location
    tbody = $('<tbody />');
    table = $('<table/>', { id: 'dctl_title_id', 'align': 'center', cellpadding: '3', cellspacing: '3' });
    cssObj = { 'border-spacing': '6', 'text-align': 'left' };
    $(table).css(cssObj);

    tr = $('<tr />');
    td = $('<td />', { 'class': 'XBold_black_16' }).append(i18n(lang_Woodward_diesel_control));
    td.appendTo(tr);
    tr.appendTo(tbody);
    tr = $('<tr />');
    td = $('<td />', { id: 'dctl_location_id', 'class': 'XLarge_black' }).append(i18n(lang_UPSMAN_Location) + ":" + location);
    td.appendTo(tr);
    tr.appendTo(tbody);
    tbody.appendTo(table);
    table.appendTo(dctl_div);
    dctl_div.appendTo(document.body);

    //Values table
    tbody = $('<tbody />');
    table = $('<table/>', { id: 'dctl_values_id', 'class': 'XBold_black', 'align': 'center', cellpadding: '3', cellspacing: '3' });
    cssObj = { 'border-spacing': '6', 'text-align': 'left' };
    $(table).css(cssObj);

    tr = $('<tr />').append($('<td />').append(create_input_1_8_table(valtable_obj))); //create input 1-8 table
    tr.append($('<td />').append(create_ext_input_1_8_table(valtable_obj))); //create ext input 1-8 table
    tr.append($('<td />').append(create_ext_input_9_16_table(valtable_obj))); //create ext input 9-16 table

    tr.appendTo(tbody);
    tbody.appendTo(table);
    table.appendTo(dctl_div);
    dctl_div.append('<button onclick="location.href=\'/panelcfg.shtml\'">' + i18n('Edit LED Labels') + '</button>');
    dctl_div.appendTo(document.body);
  }

  function create_input_1_8_table(valtable_obj) //create input 1-8 table
  {
    var tr, td, table, tbody, cssObj;

    tbody = $('<tbody />');
    table = $('<table/>', { id: 'dctl_input_1_8_id', 'align': 'center', cellpadding: '3', cellspacing: '3' });
    cssObj = { 'border-spacing': '6', 'text-align': 'left', 'table-layout': 'fixed' };
    $(table).css(cssObj);

    tr = $('<tr />');
    td = $('<td />').css('width', '24'); //Input 1
    $('<img />').attr({ id: 'dctl_input_1_img_id', 'width': 22, 'height': 22, 'src': get_input_img(32768, valtable_obj.PILDIESEL_STATEA), 'border': 0 }).appendTo(td);
    td.appendTo(tr);
    tr.append($('<td/>', { id: 'dctl_input_1_id' }).css('width', '300').append(i18n(valtable_obj.PILDIESEL_01LBL)));
    tr.appendTo(tbody);

    tr = $('<tr />');
    td = $('<td />').css('width', '24'); //Input 2
    $('<img />').attr({ id: 'dctl_input_2_img_id', 'width': 22, 'height': 22, 'src': get_input_img(16384, valtable_obj.PILDIESEL_STATEA), 'border': 0 }).appendTo(td);
    td.appendTo(tr);
    tr.append($('<td />', { id: 'dctl_input_2_id' }).css('width', '300').append(i18n(valtable_obj.PILDIESEL_02LBL)));
    tr.appendTo(tbody);

    tr = $('<tr />');
    td = $('<td />').css('width', '24'); //Input 3
    $('<img />').attr({ id: 'dctl_input_3_img_id', 'width': 22, 'height': 22, 'src': get_input_img(8192, valtable_obj.PILDIESEL_STATEA), 'border': 0 }).appendTo(td);
    td.appendTo(tr);
    tr.append($('<td />', { id: 'dctl_input_3_id' }).css('width', '300').append(i18n(valtable_obj.PILDIESEL_03LBL)));
    tr.appendTo(tbody);

    tr = $('<tr />');
    td = $('<td />').css('width', '24'); //Input 4
    $('<img />').attr({ id: 'dctl_input_4_img_id', 'width': 22, 'height': 22, 'src': get_input_img(4096, valtable_obj.PILDIESEL_STATEA), 'border': 0 }).appendTo(td);
    td.appendTo(tr);
    tr.append($('<td />', { id: 'dctl_input_4_id' }).css('width', '300').append(i18n(valtable_obj.PILDIESEL_04LBL)));
    tr.appendTo(tbody);

    tr = $('<tr />');
    td = $('<td />').css('width', '24'); //Input 5
    $('<img />').attr({ id: 'dctl_input_5_img_id', 'width': 22, 'height': 22, 'src': get_input_img(2048, valtable_obj.PILDIESEL_STATEA), 'border': 0 }).appendTo(td);
    td.appendTo(tr);
    tr.append($('<td />', { id: 'dctl_input_5_id' }).css('width', '300').append(i18n(valtable_obj.PILDIESEL_05LBL)));
    tr.appendTo(tbody);

    tr = $('<tr />');
    td = $('<td />').css('width', '24'); //Input 6
    $('<img />').attr({ id: 'dctl_input_6_img_id', 'width': 22, 'height': 22, 'src': get_input_img(1024, valtable_obj.PILDIESEL_STATEA), 'border': 0 }).appendTo(td);
    td.appendTo(tr);
    tr.append($('<td />', { id: 'dctl_input_6_id' }).css('width', '300').append(i18n(valtable_obj.PILDIESEL_06LBL)));
    tr.appendTo(tbody);

    tr = $('<tr />');
    td = $('<td />').css('width', '24'); //Input 7
    $('<img />').attr({ id: 'dctl_input_7_img_id', 'width': 22, 'height': 22, 'src': get_input_img(512, valtable_obj.PILDIESEL_STATEA), 'border': 0 }).appendTo(td);
    td.appendTo(tr);
    tr.append($('<td />', { id: 'dctl_input_7_id' }).css('width', '300').append(i18n(valtable_obj.PILDIESEL_07LBL)));
    tr.appendTo(tbody);

    tr = $('<tr />');
    td = $('<td />').css('width', '24'); //Input 8
    $('<img />').attr({ id: 'dctl_input_8_img_id', 'width': 22, 'height': 22, 'src': get_input_img(256, valtable_obj.PILDIESEL_STATEA), 'border': 0 }).appendTo(td);
    td.appendTo(tr);
    tr.append($('<td />', { id: 'dctl_input_8_id' }).css('width', '300').append(i18n(valtable_obj.PILDIESEL_08LBL)));
    tr.appendTo(tbody);


    tbody.appendTo(table);
    return table;
  }

  function create_ext_input_1_8_table(valtable_obj) //create ext input 1-8 table
  {
    var tr, td, table, tbody, cssObj;

    tbody = $('<tbody />');
    table = $('<table/>', { id: 'dctl_ext_input_1_8_id', 'align': 'center', cellpadding: '3', cellspacing: '3' });
    cssObj = { 'border-spacing': '6', 'text-align': 'left', 'table-layout': 'fixed' };
    $(table).css(cssObj);

    tr = $('<tr />');
    td = $('<td />').css('width', '24'); //ex Input 1
    $('<img />').attr({ id: 'dctl_ext_input_1_img_id', 'width': 22, 'height': 22, 'src': get_input_img(1, valtable_obj.PILDIESEL_STATEB), 'border': 0 }).appendTo(td);
    td.appendTo(tr);
    tr.append($('<td />', { id: 'dctl_ext_input_1_id' }).css('width', '300').append(i18n(valtable_obj.PILDIESEL_09LBL)));
    tr.appendTo(tbody);

    tr = $('<tr />');
    td = $('<td />').css('width', '24'); //ex Input 2
    $('<img />').attr({ id: 'dctl_ext_input_2_img_id', 'width': 22, 'height': 22, 'src': get_input_img(2, valtable_obj.PILDIESEL_STATEB), 'border': 0 }).appendTo(td);
    td.appendTo(tr);
    tr.append($('<td />', { id: 'dctl_ext_input_2_id' }).css('width', '300').append(i18n(valtable_obj.PILDIESEL_10LBL)));
    tr.appendTo(tbody);

    tr = $('<tr />');
    td = $('<td />').css('width', '24'); //ex Input 3
    $('<img />').attr({ id: 'dctl_ext_input_3_img_id', 'width': 22, 'height': 22, 'src': get_input_img(4, valtable_obj.PILDIESEL_STATEB), 'border': 0 }).appendTo(td);
    td.appendTo(tr);
    tr.append($('<td />', { id: 'dctl_ext_input_3_id' }).css('width', '300').append(i18n(valtable_obj.PILDIESEL_11LBL)));
    tr.appendTo(tbody);

    tr = $('<tr />');
    td = $('<td />').css('width', '24'); //ex Input 4
    $('<img />').attr({ id: 'dctl_ext_input_4_img_id', 'width': 22, 'height': 22, 'src': get_input_img(8, valtable_obj.PILDIESEL_STATEB), 'border': 0 }).appendTo(td);
    td.appendTo(tr);
    tr.append($('<td />', { id: 'dctl_ext_input_4_id' }).css('width', '300').append(i18n(valtable_obj.PILDIESEL_12LBL)));
    tr.appendTo(tbody);

    tr = $('<tr />');
    td = $('<td />').css('width', '24'); //ex Input 5
    $('<img />').attr({ id: 'dctl_ext_input_5_img_id', 'width': 22, 'height': 22, 'src': get_input_img(16, valtable_obj.PILDIESEL_STATEB), 'border': 0 }).appendTo(td);
    td.appendTo(tr);
    tr.append($('<td />', { id: 'dctl_ext_input_5_id' }).css('width', '300').append(i18n(valtable_obj.PILDIESEL_13LBL)));
    tr.appendTo(tbody);

    tr = $('<tr />');
    td = $('<td />').css('width', '24'); //ex Input 6
    $('<img />').attr({ id: 'dctl_ext_input_6_img_id', 'width': 22, 'height': 22, 'src': get_input_img(32, valtable_obj.PILDIESEL_STATEB), 'border': 0 }).appendTo(td);
    td.appendTo(tr);
    tr.append($('<td />', { id: 'dctl_ext_input_6_id' }).css('width', '300').append(i18n(valtable_obj.PILDIESEL_14LBL)));
    tr.appendTo(tbody);

    tr = $('<tr />');
    td = $('<td />').css('width', '24'); //ex Input 7
    $('<img />').attr({ id: 'dctl_ext_input_7_img_id', 'width': 22, 'height': 22, 'src': get_input_img(64, valtable_obj.PILDIESEL_STATEB), 'border': 0 }).appendTo(td);
    td.appendTo(tr);
    tr.append($('<td />', { id: 'dctl_ext_input_7_id' }).css('width', '300').append(i18n(valtable_obj.PILDIESEL_15LBL)));
    tr.appendTo(tbody);

    tr = $('<tr />');
    td = $('<td />').css('width', '24'); //ex Input 8
    $('<img />').attr({ id: 'dctl_ext_input_8_img_id', 'width': 22, 'height': 22, 'src': get_input_img(128, valtable_obj.PILDIESEL_STATEB), 'border': 0 }).appendTo(td);
    td.appendTo(tr);
    tr.append($('<td />', { id: 'dctl_ext_input_8_id' }).css('width', '300').append(i18n(valtable_obj.PILDIESEL_16LBL)));
    tr.appendTo(tbody);

    tbody.appendTo(table);
    return table;
  }

  function create_ext_input_9_16_table(valtable_obj) //create ext input 1-16 table
  {
    var tr, td, table, tbody, cssObj;

    tbody = $('<tbody />');
    table = $('<table/>', { id: 'dctl_ext_input_9_16_id', 'align': 'center', cellpadding: '3', cellspacing: '3' });
    cssObj = { 'border-spacing': '6', 'text-align': 'left', 'table-layout': 'fixed' };
    $(table).css(cssObj);

    tr = $('<tr />');
    td = $('<td />').css('width', '24'); //ex Input 9
    $('<img />').attr({ id: 'dctl_ext_input_9_img_id', 'width': 22, 'height': 22, 'src': get_input_img(256, valtable_obj.PILDIESEL_STATEB), 'border': 0 }).appendTo(td);
    td.appendTo(tr);
    tr.append($('<td />', { id: 'dctl_ext_input_9_id' }).css('width', '300').append(i18n(valtable_obj.PILDIESEL_17LBL)));
    tr.appendTo(tbody);

    tr = $('<tr />');
    td = $('<td />').css('width', '24'); //ex Input 10
    $('<img />').attr({ id: 'dctl_ext_input_10_img_id', 'width': 22, 'height': 22, 'src': get_input_img(512, valtable_obj.PILDIESEL_STATEB), 'border': 0 }).appendTo(td);
    td.appendTo(tr);
    tr.append($('<td />', { id: 'dctl_ext_input_10_id' }).css('width', '300').append(i18n(valtable_obj.PILDIESEL_18LBL)));
    tr.appendTo(tbody);

    tr = $('<tr />');
    td = $('<td />').css('width', '24'); //ex Input 11
    $('<img />').attr({ id: 'dctl_ext_input_11_img_id', 'width': 22, 'height': 22, 'src': get_input_img(1024, valtable_obj.PILDIESEL_STATEB), 'border': 0 }).appendTo(td);
    td.appendTo(tr);
    tr.append($('<td />', { id: 'dctl_ext_input_11_id' }).css('width', '300').append(i18n(valtable_obj.PILDIESEL_19LBL)));
    tr.appendTo(tbody);

    tr = $('<tr />');
    td = $('<td />').css('width', '24'); //ex Input 12
    $('<img />').attr({ id: 'dctl_ext_input_12_img_id', 'width': 22, 'height': 22, 'src': get_input_img(2048, valtable_obj.PILDIESEL_STATEB), 'border': 0 }).appendTo(td);
    td.appendTo(tr);
    tr.append($('<td />', { id: 'dctl_ext_input_12_id' }).css('width', '300').append(i18n(valtable_obj.PILDIESEL_20LBL)));
    tr.appendTo(tbody);

    tr = $('<tr />');
    td = $('<td />').css('width', '24'); //ex Input 13
    $('<img />').attr({ id: 'dctl_ext_input_13_img_id', 'width': 22, 'height': 22, 'src': get_input_img(4096, valtable_obj.PILDIESEL_STATEB), 'border': 0 }).appendTo(td);
    td.appendTo(tr);
    tr.append($('<td />', { id: 'dctl_ext_input_13_id' }).css('width', '300').append(i18n(valtable_obj.PILDIESEL_21LBL)));
    tr.appendTo(tbody);

    tr = $('<tr />');
    td = $('<td />').css('width', '24'); //ex Input 14
    $('<img />').attr({ id: 'dctl_ext_input_14_img_id', 'width': 22, 'height': 22, 'src': get_input_img(8192, valtable_obj.PILDIESEL_STATEB), 'border': 0 }).appendTo(td);
    td.appendTo(tr);
    tr.append($('<td />', { id: 'dctl_ext_input_14_id' }).css('width', '300').append(i18n(valtable_obj.PILDIESEL_22LBL)));
    tr.appendTo(tbody);

    tr = $('<tr />');
    td = $('<td />').css('width', '24'); //ex Input 15
    $('<img />').attr({ id: 'dctl_ext_input_15_img_id', 'width': 22, 'height': 22, 'src': get_input_img(16384, valtable_obj.PILDIESEL_STATEB), 'border': 0 }).appendTo(td);
    td.appendTo(tr);
    tr.append($('<td />', { id: 'dctl_ext_input_15_id' }).css('width', '300').append(i18n(valtable_obj.PILDIESEL_23LBL)));
    tr.appendTo(tbody);

    tr = $('<tr />');
    td = $('<td />').css('width', '24'); //ex Input 16
    $('<img />').attr({ id: 'dctl_ext_input_16_img_id', 'width': 22, 'height': 22, 'src': get_input_img(32768, valtable_obj.PILDIESEL_STATEB), 'border': 0 }).appendTo(td);
    td.appendTo(tr);
    tr.append($('<td />', { id: 'dctl_ext_input_16_id' }).css('width', '300').append(i18n(valtable_obj.PILDIESEL_24LBL)));
    tr.appendTo(tbody);

    tbody.appendTo(table);
    return table;
  }

  function set_input_1_8_values(valtable_obj) //set input 1-8 values
  {
    $('#' + 'dctl_input_1_img_id').attr('src', get_input_img(32768, valtable_obj.PILDIESEL_STATEA));
    $('#' + 'dctl_input_1_id').text(i18n(valtable_obj.PILDIESEL_01LBL));

    $('#' + 'dctl_input_2_img_id').attr('src', get_input_img(16384, valtable_obj.PILDIESEL_STATEA));
    $('#' + 'dctl_input_2_id').text(i18n(valtable_obj.PILDIESEL_02LBL));

    $('#' + 'dctl_input_3_img_id').attr('src', get_input_img(8192, valtable_obj.PILDIESEL_STATEA));
    $('#' + 'dctl_input_3_id').text(i18n(valtable_obj.PILDIESEL_03LBL));

    $('#' + 'dctl_input_4_img_id').attr('src', get_input_img(4096, valtable_obj.PILDIESEL_STATEA));
    $('#' + 'dctl_input_4_id').text(valtable_obj.PILDIESEL_04LBL);

    $('#' + 'dctl_input_5_img_id').attr('src', get_input_img(2048, valtable_obj.PILDIESEL_STATEA));
    $('#' + 'dctl_input_5_id').text(i18n(valtable_obj.PILDIESEL_05LBL));

    $('#' + 'dctl_input_6_img_id').attr('src', get_input_img(1024, valtable_obj.PILDIESEL_STATEA));
    $('#' + 'dctl_input_6_id').text(i18n(valtable_obj.PILDIESEL_06LBL));

    $('#' + 'dctl_input_7_img_id').attr('src', get_input_img(512, valtable_obj.PILDIESEL_STATEA));
    $('#' + 'dctl_input_7_id').text(i18n(valtable_obj.PILDIESEL_07LBL));

    $('#' + 'dctl_input_8_img_id').attr('src', get_input_img(256, valtable_obj.PILDIESEL_STATEA));
    $('#' + 'dctl_input_8_id').text(i18n(valtable_obj.PILDIESEL_08LBL));
  }

  function set_ext_input_1_8_values(valtable_obj) //set ext input 1-8 values
  {
    $('#' + 'dctl_ext_input_1_img_id').attr('src', get_input_img(1, valtable_obj.PILDIESEL_STATEB));
    $('#' + 'dctl_ext_input_1_id').text(i18n(valtable_obj.PILDIESEL_09LBL));

    $('#' + 'dctl_ext_input_2_img_id').attr('src', get_input_img(2, valtable_obj.PILDIESEL_STATEB));
    $('#' + 'dctl_ext_input_2_id').text(i18n(valtable_obj.PILDIESEL_10LBL));

    $('#' + 'dctl_ext_input_3_img_id').attr('src', get_input_img(4, valtable_obj.PILDIESEL_STATEB));
    $('#' + 'dctl_ext_input_3_id').text(i18n(valtable_obj.PILDIESEL_11LBL));

    $('#' + 'dctl_ext_input_4_img_id').attr('src', get_input_img(8, valtable_obj.PILDIESEL_STATEB));
    $('#' + 'dctl_ext_input_4_id').text(i18n(valtable_obj.PILDIESEL_12LBL));

    $('#' + 'dctl_ext_input_5_img_id').attr('src', get_input_img(16, valtable_obj.PILDIESEL_STATEB));
    $('#' + 'dctl_ext_input_5_id').text(i18n(valtable_obj.PILDIESEL_13LBL));

    $('#' + 'dctl_ext_input_6_img_id').attr('src', get_input_img(32, valtable_obj.PILDIESEL_STATEB));
    $('#' + 'dctl_ext_input_6_id').text(i18n(valtable_obj.PILDIESEL_14LBL));

    $('#' + 'dctl_ext_input_7_img_id').attr('src', get_input_img(64, valtable_obj.PILDIESEL_STATEB));
    $('#' + 'dctl_ext_input_7_id').text(i18n(valtable_obj.PILDIESEL_15LBL));

    $('#' + 'dctl_ext_input_8_img_id').attr('src', get_input_img(128, valtable_obj.PILDIESEL_STATEB));
    $('#' + 'dctl_ext_input_8_id').text(i18n(valtable_obj.PILDIESEL_16LBL));
  }

  function set_ext_input_1_16_values(valtable_obj) //set ext input 1-16 values
  {
    $('#' + 'dctl_ext_input_9_img_id').attr('src', get_input_img(256, valtable_obj.PILDIESEL_STATEB));
    $('#' + 'dctl_ext_input_9_id').text(i18n(valtable_obj.PILDIESEL_17LBL));

    $('#' + 'dctl_ext_input_10_img_id').attr('src', get_input_img(512, valtable_obj.PILDIESEL_STATEB));
    $('#' + 'dctl_ext_input_10_id').text(i18n(valtable_obj.PILDIESEL_18LBL));

    $('#' + 'dctl_ext_input_11_img_id').attr('src', get_input_img(1024, valtable_obj.PILDIESEL_STATEB));
    $('#' + 'dctl_ext_input_11_id').text(i18n(valtable_obj.PILDIESEL_19LBL));

    $('#' + 'dctl_ext_input_12_img_id').attr('src', get_input_img(2048, valtable_obj.PILDIESEL_STATEB));
    $('#' + 'dctl_ext_input_12_id').text(i18n(valtable_obj.PILDIESEL_20LBL));

    $('#' + 'dctl_ext_input_13_img_id').attr('src', get_input_img(4096, valtable_obj.PILDIESEL_STATEB));
    $('#' + 'dctl_ext_input_13_id').text(i18n(valtable_obj.PILDIESEL_21LBL));

    $('#' + 'dctl_ext_input_14_img_id').attr('src', get_input_img(8192, valtable_obj.PILDIESEL_STATEB));
    $('#' + 'dctl_ext_input_14_id').text(i18n(valtable_obj.PILDIESEL_22LBL));

    $('#' + 'dctl_ext_input_15_img_id').attr('src', get_input_img(16384, valtable_obj.PILDIESEL_STATEB));
    $('#' + 'dctl_ext_input_15_id').text(i18n(valtable_obj.PILDIESEL_23LBL));

    $('#' + 'dctl_ext_input_16_img_id').attr('src', get_input_img(32768, valtable_obj.PILDIESEL_STATEB));
    $('#' + 'dctl_ext_input_16_id').text(i18n(valtable_obj.PILDIESEL_24LBL));
  }

  function get_input_img(value, variable) {
    if ((variable & value) == value)
    return "./image/dctl/red.gif";
    return "./image/dctl/dark_red.gif";
  }

  function create_footer_pane() {
    if (!oid || !unms_obj)
    return true;
    footer_api.create({ //Attr
        id: 'panel2'
      }, { //css
        position: 'absolute',
        top: '450px'
      },
      unms_obj
    );
  }

  function set_footer() {
    if (!oid || !unms_obj)
    return true;
    footer_api.set(unms_obj);
  }

}(jQuery));