/*
   sec_panel.js
   Copyright (c) 2020, Generex GmbH. All rights reserved.
   UW.
*/

var sec_api = (function ($) {
  var oid = get_oid_from_url(),
  cb = null,
  pannes_created = false,
  err_div = null,
  check_bowser = true,
  outpower_loadbars_created = false,
  css_obj, attr_obj,
  log_entries,
  valtable_obj, report_obj, unms_obj,
  inphases = null,
  outphases = null,
  bypphases = null,
  status, tempmax, tempdeg, overload = 0,
  plattform = get_plattform(),
  ups_id = 0,
  ups_protocol = 0,
  bg_height, bg_width;
  x_pos = 10, y_pos = 10,

 //log_event_pane attr/css
 evt_css_obj = {
   position: 'absolute',
   top: 65,
   'left': 270,
   width: 640,
   height: 136,
   'background': 'white',
   align: 'center',
   'overflow-x': 'scroll',
   'overflow-y': 'scroll',
   'border': '1 solid #405080'
 },
 evt_attObj = { id: 'log_div_id', 'class': 'tdSmall8_black' };

  var status_obj = [

    { bit: 0x0008, txt: "The UPS is drawing power from the batteries" },
    { bit: 0x0010, txt: "The remaining battery run-time is low" },
    { bit: 0x0400, txt: "One or more batteries have been determined to require replacement" },
    { bit: 0x0020, txt: "A temperature is out of tolerance" },
    { bit: 0x0080, txt: "Input high" },
    { bit: 0x0100, txt: "The output load exceeds the UPS output capacity" },
    { bit: 0x2000, txt: "A general fault in the UPS has been detected" },
    { bit: 0x4000, txt: "A problem has been encountered in the communications between the agent and the UPS" },
    { bit: 0x0200, txt: "Inverter error" },
    { bit: 0x1000, txt: "Inverter warning" },
    { bit: 0x0002, txt: "UPS is in shutdown mode" },
    { bit: 0x0001, txt: "The Bypass is presently engaged on the UPS" },
    { bit: 0x0040, txt: "A test is in progress" },
    { bit: 0x0004, txt: "UPS Output active" },
    { bit: 0x0800, txt: "UPS in ECO mode" }
  ],
  ext_status_obj = [
    { bit: 0x00000008, txt: "The UPS will be unable to sustain the present load when and if the utility power is lost" },
    { bit: 0x00000040, txt: "An output condition (other than OutputOverload) is out of tolerance" },
    { bit: 0x00000200, txt: "The Bypass is out of tolerance" },
    { bit: 0x00000400, txt: "The UPS has shutdown as requested, i.e., the output is off" },
    { bit: 0x00000800, txt: "The entire UPS has shutdown as commanded" },
    { bit: 0x00001000, txt: "An uncorrected problem has been detected within the UPS charger subsystem" },
    { bit: 0x00002000, txt: "The output of the UPS is in the off state" },
    { bit: 0x00004000, txt: "The UPS system is in the off state" },
    { bit: 0x00008000, txt: "The failure of one or more fans in the UPS has been detected" },
    { bit: 0x00010000, txt: "The failure of one or more fuses has been detected" },
    { bit: 0x00040000, txt: "The result of the last diagnostic test indicates a failure" },
    { bit: 0x00100000, txt: "The UPS output is off and the UPS is awaiting the return of input power" },
    { bit: 0x00200000, txt: "An UPS Shutdown After Delay countdown is underway" },
    { bit: 0x00400000, txt: "The UPS will turn off power to the load immediatly" }
  ],
  alarms_obj = [
    { bit: 0x00000001, txt: "Redundancy lost" },
    { bit: 0x00000008, txt: "SBP error" },
    { bit: 0x00000040, txt: "Redundancy + 1 lost" },
    { bit: 0x00000080, txt: "Off manual" },
    { bit: 0x00000100, txt: "Battery error" },
    { bit: 0x00000200, txt: "Battery not connected" },
    { bit: 0x00000400, txt: "Replace fan" },
  ];

  var req_params = null;

  /*public functions*/
  return {
    init: init
  };

  function init() {
    if (check_bowser && !utils_api.check_bowser()) {
      $('#loading_id').css('display', 'none');
      panes_api.create_br_check_panel(this, browser_check_ok);
      return true;
    }
    create_err_msg_panel();

    req_params = utils_api.get_req_params(utils_api.device_ups, oid);
    cb = jqmodule.subscribe(this, { update: update, error: err_funct }, req_params);
  }

  function browser_check_ok() {
    check_bowser = false;
    init();
  }

  function create_err_msg_panel() {
    css_obj = { position: 'absolute', top: y_pos, left: x_pos, width: 791, display: 'none', 'z-index': 62 };
    err_div = panes_api.create_err_panel({ id: 'error_div', 'class': 'alert alert-danger' }, css_obj);
  }

  function err_funct(response) {
    jqmodule.refresh(cb, 10 * 1000);
    panes_api.display_err({ response: response, oid: oid, err_div: err_div, err_msg_div_id: 'error_msg_div' });
  }

  function update(data) {
    if (data.ups == null) {
      return;
    }

    set_variable(data);
    jqmodule.refresh(cb, report_obj.refresh * 1000);
    if (err_div)
      err_div.css('display', 'none');

    log_entries = data.ups.log_entries;
    if (log_entries != null)
      log_entries = log_entries.reverse();

    if (window.menu_api != null)
      menu_api.update(data);

    unms_obj = get_unms_obj(data);
    if (unms_obj != null) {
      info_head_api.update(unms_obj);
    }

    if (!pannes_created) {
      pannes_created = true;
      create_panes();
      return false;
    }
    set_input_values(); ////Input values
    set_output_values(); //Output values
    set_battery_value(); //Battery value
    set_symbols();
    set_ups_status(); //set UPS status
    set_outpower_loadbars(); //outpower loadbars
    set_model_name();
    set_sys_mod_status();

    set_footer();
    if (log_entries != null)
      panes_api.create_log_event_pane(log_entries, evt_attObj, evt_css_obj); //create log events pane.
  }

  function create_footer_pane() {
    if (!oid || !unms_obj)
      return true;
    footer_api.create({ //Attr
        id: 'panel2'
      }, { //css
        position: 'absolute',
        top: '690px',
        left: '140px'
      },
      unms_obj
    );
  }

  function set_footer() {
    if (!oid || !unms_obj)
    return true;
    footer_api.set(unms_obj);
  }

  function set_variable(data) {
    valtable_obj = data.ups.valtable;
    report_obj = data.report;

    status = valtable_obj.STATUS;
    tempmax = valtable_obj.TMPMAX;
    tempdeg = valtable_obj.TEMPDEG;
    level = valtable_obj.LEVEL;

    bg_height = 670;
    bg_width = 917;

    if (valtable_obj.INPHASES != null)
    inphases = valtable_obj.INPHASES;
    else
    inphases = valtable_obj.PHASES;

    if (valtable_obj.OUTPHASES != null)
    outphases = valtable_obj.OUTPHASES;
    else
    outphases = valtable_obj.PHASES;

    if (valtable_obj.EX_BYP_NUMLINES != null && valtable_obj.EX_BYP_NUMLINES != -9999)
    bypphases = valtable_obj.EX_BYP_NUMLINES;
    else
    bypphases = inphases;

    if (valtable_obj._upsProtocol == null) {
      ups_protocol = valtable_obj.CONNECT & 0XFFFF;
    } else {
      ups_protocol = valtable_obj._upsProtocol;
    }

    if (valtable_obj.OUTPOWER != null) {
      if (outphases > 2) {
        if (((valtable_obj.OUTPOWER[0] > 100) && (valtable_obj.OUTPOWER[0] < 500)) ||
          ((valtable_obj.OUTPOWER[1] > 100) && (valtable_obj.OUTPOWER[1] < 500)) ||
          ((valtable_obj.OUTPOWER[2] > 100) && (valtable_obj.OUTPOWER[2] < 500)))
        overload = 1;
        else
        overload = 0;
      } else {
        if (valtable_obj.OUTPOWER[0] > 100 && valtable_obj.OUTPOWER[0] < 500)
        overload = 1;
        else
        overload = 0;
      }
    }
  }

  function create_panes() {
    utils_api.setTitle(valtable_obj._platform);
    create_bg(); //background
    create_info_pane();
    create_input_pane(); //Input table
    create_output_pane(); //Outnput   panel
    create_battery_pane(); //Battery panel
    create_symbols(); //create symbols for bypass, input, output, battery
    create_status_pane(); //UPS status
    create_model_panes();
    create_outpower_loadbars(); //outpower loadbars
    create_footer_pane();
    create_sys_mod_status_panel();
    if (log_entries != null)
      panes_api.create_log_event_pane(log_entries, evt_attObj, evt_css_obj); //create log events pane.
  }

  function create_bg() {
    var fragment = document.createDocumentFragment(), div;

    css_obj = { position: 'absolute', top: y_pos, left: x_pos, align: 'center', width: bg_width, height: bg_height, border: 'solid 1px #808080' };
    div = $('<div/>', { 'class': 'img-rounded' }).css(css_obj);
    fragment.appendChild(div[0]);

    css_obj = { position: 'absolute', top: y_pos + 100, left: x_pos + 28, align: 'center', width: 816, height: 459 };
    div = $('<div/>', { 'class': 'img-rounded' }).css(css_obj);
    $('<img />').attr({ 'id': 'bg_img_id', 'class': 'img-rounded', width: 816, 'height': 459, src: './image/ups/bg_mpx.png' }).appendTo(div);
    fragment.appendChild(div[0]);

    document.body.appendChild(fragment);
  }

  function create_model_panes() {
    var model_div, a;
    css_obj = { position: 'absolute', top: y_pos, left: x_pos, align: 'center', width: bg_width, 'z-index': 40 }; // 'background-color': '#3a87ad'
    attr_obj = { id: 'model_div_id', 'class': 'list-group' };
    model_div = panes_api.create_div(attr_obj, css_obj);
    a = $('<a/>', { 'class': 'list-group-item active', id: 'model_name_id' }).appendTo(model_div);
    set_model_name();
  }

  function create_info_pane()
  {
    var fragment = document.createDocumentFragment(),
    tr, table, tbody, thead, in_div;

    css_obj = { position: 'absolute', top: y_pos + 100, left: x_pos + 30 };

    in_div = $('<div/>', { id: 'input_div_id' }).css(css_obj);
    table = $('<table />', { 'class': 'table table-condensed table-hover', 'id': 'inptable' }).appendTo(in_div); // table-hover
    tbody = $('<tbody />').appendTo(table);
    thead = $('<thead />').appendTo(table);
    tr = $('<tr />').append($('<th />', { colspan: 2 }).append(i18n("UPS Identification"))).appendTo(thead);

    tr = $('<tr />', { 'id': 'model', align: 'left' }).append($('<td />').append(i18n("Model") + ":")).appendTo(tbody);
    $('<td />', { id: 'model_id' }).appendTo(tr);
    tr = $('<tr />', { 'id': 'ident', align: 'left'  }).append($('<td />').append(i18n("Identification") + ":")).appendTo(tbody);
    $('<td />', { id: 'ident_id' }).appendTo(tr);
    tr = $('<tr />', { 'id': 'manu', align: 'left'  }).append($('<td />').append(i18n("Manufacturer") + ":")).appendTo(tbody);
    $('<td />', { id: 'manu_id' }).appendTo(tr);
    tr = $('<tr />', { 'id': 'swv', align: 'left'  }).append($('<td />').append(i18n("Software Version") + ":")).appendTo(tbody);
    $('<td />', { id: 'swv_id' }).appendTo(tr);

    fragment.appendChild(in_div[0]);
    document.body.appendChild(fragment);

    set_info_pane();
  }

  function set_info_pane()
  {
    $('#model_id').text(valtable_obj.SOLSEMODEL);
    $('#ident_id').text(valtable_obj.SOLSEIDENT);
    $('#manu_id').text(valtable_obj.SOLSEMANUFACT);
    $('#swv_id').text(valtable_obj.SOLSESOFTWARE);
  }

  function create_input_pane() //Input table
  {
    var fragment = document.createDocumentFragment(),
    tr, table, tbody, thead, in_div;

    css_obj = { position: 'absolute', top: y_pos + 480, left: x_pos + 30 };

    in_div = $('<div/>', { id: 'input_div_id' }).css(css_obj);
    table = $('<table />', { 'class': 'table table-condensed table-hover', 'id': 'inptable' }).appendTo(in_div); // table-hover
    tbody = $('<tbody />').appendTo(table);
    thead = $('<thead />').appendTo(table);
    tr = $('<tr />').append($('<th />', { colspan: 4 }).append(i18n(lang_Input))).appendTo(thead);

    tr = $('<tr />', { 'id': 'input_volt', align: 'left'  }).append($('<td />').append(i18n("Voltage") + ":")).appendTo(tbody);
    $('<td />', { id: 'input_volt1_id', align: 'left'  }).appendTo(tr);
    $('<td />', { id: 'input_volt2_id', align: 'left'  }).appendTo(tr);
    $('<td />', { id: 'input_volt3_id', align: 'left'  }).appendTo(tr);

    tr = $('<tr />', { 'id': 'input_curr', align: 'left'  }).append($('<td />').append(i18n("Current") + ":")).appendTo(tbody);
    $('<td />', { id: 'input_cur1_id', align: 'left'  }).appendTo(tr);
    $('<td />', { id: 'input_cur2_id', align: 'left'  }).appendTo(tr);
    $('<td />', { id: 'input_cur3_id', align: 'left'  }).appendTo(tr);

    tr = $('<tr />', { 'id': 'input_pwr', align: 'left'  }).append($('<td />').append(i18n("Power") + ":")).appendTo(tbody);
    $('<td />', { id: 'input_watt1_id', align: 'left' }).appendTo(tr);
    $('<td />', { id: 'input_watt2_id', align: 'left' }).appendTo(tr);
    $('<td />', { id: 'input_watt3_id', align: 'left' }).appendTo(tr);

    tr = $('<tr />', { 'id': 'input_kva', align: 'left'  }).append($('<td />').append(i18n("kVA") + ":")).appendTo(tbody);
    $('<td />', { id: 'input_va1_id', align: 'left' }).appendTo(tr);
    $('<td />', { id: 'input_va2_id', align: 'left' }).appendTo(tr);
    $('<td />', { id: 'input_va3_id', align: 'left' }).appendTo(tr);

    tr = $('<tr />', { 'id': 'input_freq', align: 'left'  }).append($('<td />').append(i18n("Frequency") + ":")).appendTo(tbody);
    $('<td />', { id: 'input_freq1_id', align: 'left'  }).appendTo(tr);
    $('<td />', { id: 'input_freq2_id', align: 'left'  }).appendTo(tr);
    $('<td />', { id: 'input_freq3_id', align: 'left'  }).appendTo(tr);

    fragment.appendChild(in_div[0]);
    document.body.appendChild(fragment);
    set_input_values();
  }

  function set_input_values()
  {
    var input_val = get_ph_1_3_vals(valtable_obj.INVOLT, valtable_obj.INCURR, valtable_obj.INFREQ, inphases, valtable_obj.EX_INP_WATT);

    if (input_val.availables[0] != 0) {
      $('#input_volt1_id').text(input_val.val_objs[0].values[0]);
      $('#input_cur1_id').text(input_val.val_objs[1].values[0]);
      $('#input_watt1_id').text(input_val.val_objs[3].values[0]);
      $('#input_freq1_id').text(input_val.val_objs[2].values[0]);

      if (valtable_obj.INCURR[0] > -9999)
      {
         var val = (valtable_obj.INCURR[0] * valtable_obj.INVOLT[0]) / 1000;
         val = val.toFixed(2);
         val += " kVA";
      }
      else
      {
         val = "n/a";
      }
      $('#input_va1_id').text(val);
   }

    if (input_val.availables[1] != 0) {
      $('#input_volt2_id').text(input_val.val_objs[0].values[1]);
      $('#input_cur2_id').text(input_val.val_objs[1].values[1]);
      $('#input_watt2_id').text(input_val.val_objs[3].values[1]);
//      $('#input_freq2_id').text(input_val.val_objs[2].values[1]);

      if (valtable_obj.INCURR[1] > -9999)
      {
         var val = (valtable_obj.INCURR[1] * valtable_obj.INVOLT[1]) / 1000;
         val = val.toFixed(2);
         val += " kVA";
      }
      else
      {
         val = "n/a";
      }
      $('#input_va2_id').text(val);
   }

    if (input_val.availables[2] != 0) {
      $('#input_volt3_id').text(input_val.val_objs[0].values[2]);
      $('#input_cur3_id').text(input_val.val_objs[1].values[2]);
      $('#input_watt3_id').text(input_val.val_objs[3].values[2]);
//      $('#input_freq3_id').text(input_val.val_objs[2].values[2]);

      if (valtable_obj.INCURR[2] > -9999)
      {
         var val = (valtable_obj.INCURR[2] * valtable_obj.INVOLT[2]) / 1000;
         val = val.toFixed(2);
         val += " kVA";
      }
      else
      {
         val = "n/a";
      }
      $('#input_va3_id').text(val);
    }
  }

  function create_output_pane() //Output
  {
    var output_div = null,
    tr, table, tbody, thead,
    fragment = document.createDocumentFragment();

    css_obj = { position: 'absolute', top: y_pos + 480, left: x_pos + 600 };

    output_div = $('<div/>', { id: 'output_div_id' }).css(css_obj);
    table = $('<table />', { 'class': 'table table-condensed table-hover', id: 'outtable' }).appendTo(output_div);
    tbody = $('<tbody />').appendTo(table);
    thead = $('<thead />').appendTo(table);
    tr = $('<tr />').append($('<th />', { colspan: 4 }).append(i18n(lang_Output))).appendTo(thead);

    tr = $('<tr />', { 'id': 'out_volt', align: 'left'  }).append($('<td />').append(i18n("Voltage") + ":")).appendTo(tbody);
    $('<td />', { id: 'out_volt1_id', align: 'left'  }).appendTo(tr);
    $('<td />', { id: 'out_volt2_id', align: 'left'  }).appendTo(tr);
    $('<td />', { id: 'out_volt3_id', align: 'left'  }).appendTo(tr);

    tr = $('<tr />', { 'id': 'out_curr', align: 'left'  }).append($('<td />').append(i18n("Current") + ":")).appendTo(tbody);
    $('<td />', { id: 'out_cur1_id', align: 'left'  }).appendTo(tr);
    $('<td />', { id: 'out_cur2_id', align: 'left'  }).appendTo(tr);
    $('<td />', { id: 'out_cur3_id', align: 'left'  }).appendTo(tr);

    tr = $('<tr />', { 'id': 'out_pwr', align: 'left'  }).append($('<td />').append(i18n("Power") + ":")).appendTo(tbody);
    $('<td />', { id: 'out_watt1_id', align: 'left' }).appendTo(tr);
    $('<td />', { id: 'out_watt2_id', align: 'left' }).appendTo(tr);
    $('<td />', { id: 'out_watt3_id', align: 'left' }).appendTo(tr);

    tr = $('<tr />', { 'id': 'out_kva', align: 'left'  }).append($('<td />').append(i18n("kVA") + ":")).appendTo(tbody);
    $('<td />', { id: 'out_va1_id', align: 'left' }).appendTo(tr);
    $('<td />', { id: 'out_va2_id', align: 'left' }).appendTo(tr);
    $('<td />', { id: 'out_va3_id', align: 'left' }).appendTo(tr);

    tr = $('<tr />', { 'id': 'out_freq', align: 'left'  }).append($('<td />').append(i18n("Frequency") + ":")).appendTo(tbody);
    $('<td />', { id: 'out_freq1_id', align: 'left'  }).appendTo(tr);
    $('<td />', { id: 'out_freq2_id', align: 'left'  }).appendTo(tr);
    $('<td />', { id: 'out_freq3_id', align: 'left'  }).appendTo(tr);

    fragment.appendChild(output_div[0]);
    document.body.appendChild(fragment);
    set_output_values();
  }

  function set_output_values() //Output values
  {
    var output_val = get_ph_1_3_vals(valtable_obj.SOLAWROV, valtable_obj.EX_OUT_CURR, valtable_obj.OUTFREQ, outphases, valtable_obj.EX_OUT_WATT);

    if (output_val.availables[0] != 0) {
      $('#out_volt1_id').text(output_val.val_objs[0].values[0]);
      $('#out_cur1_id').text(output_val.val_objs[1].values[0]);
      $('#out_watt1_id').text(output_val.val_objs[3].values[0]);
      $('#out_freq1_id').text(output_val.val_objs[2].values[0]);

      if (valtable_obj.EX_OUT_CURR[0] > -9999)
      {
         var val = (valtable_obj.EX_OUT_CURR[0] * valtable_obj.SOLAWROV[0]) / 1000;
         val = val.toFixed(2);
         val += " kVA";
      }
      else
      {
         val = "n/a";
      }
      $('#out_va1_id').text(val);
    }

    if (output_val.availables[1] != 0) {
      $('#out_volt2_id').text(output_val.val_objs[0].values[1]);
      $('#out_cur2_id').text(output_val.val_objs[1].values[1]);
      $('#out_watt2_id').text(output_val.val_objs[3].values[1]);

      if (valtable_obj.EX_OUT_CURR[1] > -9999)
      {
         var val = (valtable_obj.EX_OUT_CURR[1] * valtable_obj.SOLAWROV[1]) / 1000;
         val = val.toFixed(2);
         val += " kVA";
      }
      else
      {
         val = "n/a";
      }
      $('#out_va2_id').text(val);
    }

    if (output_val.availables[2] != 0) {
      $('#out_volt3_id').text(output_val.val_objs[0].values[2]);
      $('#out_cur3_id').text(output_val.val_objs[1].values[2]);
      $('#out_watt3_id').text(output_val.val_objs[3].values[2]);

      if (valtable_obj.EX_OUT_CURR[2] > -9999)
      {
         var val = (valtable_obj.EX_OUT_CURR[2] * valtable_obj.SOLAWROV[2]) / 1000;
         val = val.toFixed(2);
         val += " kVA";
      }
      else
      {
         val = "n/a";
      }
      $('#out_va3_id').text(val);
    }
  }

  function create_battery_pane() //Battery panel
  {
    var val, tr, table, tbody, thead, batt_div, battpow,
    fragment = document.createDocumentFragment();

    css_obj = { position: 'absolute', top: y_pos + 480, left: x_pos + 360 };
    attr_obj = { id: 'battery_div_id' };
    batt_div = $('<div/>').attr(attr_obj).css(css_obj);
    table = table = $('<table />', { 'class': 'table table-condensed table-hover' }).appendTo(batt_div);
    tbody = $('<tbody />').appendTo(table);
    thead = $('<thead />').appendTo(table);
    tr = $('<tr />').append($('<th />', { colspan: 2 }).append(i18n(lang_Battery))).appendTo(thead);

    tr = $('<tr />').append($('<td />', { align: 'left' }).append(i18n(lang_Voltage) + ":")).appendTo(tbody);
    $('<td />', { id: 'batt_volt_id', align: 'left' }).appendTo(tr);

    tr = $('<tr />').append($('<td />', { align: 'left' }).append(i18n(lang_Current) + ":")).appendTo(tbody);
    $('<td />', { id: 'batt_currpos_id', align: 'left' }).appendTo(tr);

    tr = $('<tr />').append($('<td />', { align: 'left' }).append(i18n("Charger status") + ":")).appendTo(tbody);
    $('<td />', { id: 'batt_chrgstatus_id', align: 'left' }).appendTo(tr);

    tr = $('<tr />').append($('<td />', { align: 'left' }).append(i18n("Minutes") + ":")).appendTo(tbody);
    $('<td />', { id: 'batt_autotime_id', align: 'left' }).appendTo(tr);

    fragment.appendChild(batt_div[0]);
    document.body.appendChild(fragment);
    set_battery_value();
  }

  function set_battery_value() //set Battery value
  {
    var chrg_status = "n/a";

    $('#batt_volt_id').text(get_value({ values: [valtable_obj.BATTVOLT], index: 0, unit: ' V', precision: 0 }));
    $('#batt_currpos_id').text(get_value({ values: [valtable_obj.EX_BATT_CURRPOS], index: 0, unit: ' A', precision: 1 }));
    $('#batt_autotime_id').text(get_value({ values: [valtable_obj.AUTONOMTIME], index: 0, unit: ' min', precision: 0 }));

    if (valtable_obj.SOLSECHARGE == 0)
      chrg_status = "floating";
    else if (valtable_obj.SOLSECHARGE == 1)
      chrg_status = "charging";
    else if (valtable_obj.SOLSECHARGE == 2)
      chrg_status = "discharging";
    $('#batt_chrgstatus_id').text(i18n(chrg_status));
  }

  function get_ph_1_3_vals(volt, cur, freq, phases, watt) {
    var val_obj, val;
    l_1_3_vals = {
      availables: [0, 0, 0], //L1-L3
      val_objs: [
        { unit: ' V', precision: 0, values: ['', '', ''] },
        { unit: ' A', precision: 1, values: ['', '', ''] },
        { unit: ' Hz', precision: 1, values: ['', '', ''] },
        { unit: ' kW', precision: 2, values: ['', '', ''] },
      ]
    };
    volt = volt || [];
    cur = cur || [], freq = freq || [];
    watt = watt || [];
    $.each([volt, cur, freq, watt], function (index, values) {
      val_obj = l_1_3_vals.val_objs[index];
      $.each(values, function (i, val) {
        if (i < phases) {
          val = values[i];
          if (val != -9999) {
            // do some nice output:
            // if watt or va are bigger than 1000 display
            // them as kW
            if (values === watt) {
              if (val >= 1000) {
                val_obj.unit = ' kW';
                val_obj.precision = 1;
                val = val / 1000.0;
              } else {
                val_obj.unit = ' W';
                val_obj.precision = 0;
              }
            }
            l_1_3_vals.availables[i] = 1;
            val = val.toFixed(val_obj.precision);
            val += val_obj.unit;
            val_obj.values[i] = val;
          }
          else
          {
            val_obj.values[i] = "n/a";
          }
        }
      });
    });
    return l_1_3_vals;
  }

  function get_value(val_obj) {
    if (!val_obj.values || (val_obj.values.length <= val_obj.index))
    return 'n/a';
    var val = val_obj.values[val_obj.index];
    if (val == null || val_obj.values[val_obj.index] == -9999)
    return 'n/a';

    val = val.toFixed(val_obj.precision);
    val += val_obj.unit;
    return val;
  }

  function create_symbols()
  {
    var div, fragment = document.createDocumentFragment();

   // output led
    css_obj = { position: 'absolute', top: y_pos + 229 + 101, left: x_pos + 528, width: 94, height: 6, overflow: 'hidden' };
    div = $('<div/>').attr({ id: 'output_led_id' }).css(css_obj);
    fragment.appendChild(div[0]);

    // input leds
    css_obj = { position: 'absolute', top: y_pos + 229 + 101, left: x_pos + 225, width: 98, height: 6, overflow: 'hidden' };
    div = $('<div/>').attr({ id: 'input_led_id' }).css(css_obj);
    fragment.appendChild(div[0]);

    css_obj = { position: 'absolute', top: y_pos + 229 + 101, left: x_pos + 372, width: 50, height: 6, overflow: 'hidden' };
    div = $('<div/>').attr({ id: 'input_led2_id' }).css(css_obj);
    fragment.appendChild(div[0]);

    //bypass leds
    css_obj = { position: 'absolute', top: y_pos + 150 + 101, left: x_pos + 225, width: 173, height: 6, overflow: 'hidden' };
    div = $('<div/>').attr({ id: 'bypass_led_id' }).css(css_obj);
    fragment.appendChild(div[0]);

    css_obj = { position: 'absolute', top: y_pos + 150 + 101, left: x_pos + 448, width: 104, height: 6, overflow: 'hidden' };
    div = $('<div/>').attr({ id: 'bypass_outled_id' }).css(css_obj);
    fragment.appendChild(div[0]);

    css_obj = { position: 'absolute', top: y_pos + 150 + 101, left: x_pos + 546, width: 6, height: 79, overflow: 'hidden' };
    div = $('<div/>').attr({ id: 'bypass_out2led_id' }).css(css_obj);
    fragment.appendChild(div[0]);

    // battery leds
    css_obj = { position: 'absolute', top: y_pos + 229 + 101, left: x_pos + 421, width: 6, height: 64, overflow: 'hidden' };
    div = $('<div/>').attr({ id: 'batt_led_id' }).css(css_obj);
    fragment.appendChild(div[0]);

    css_obj = { position: 'absolute', top: y_pos + 229 + 101, left: x_pos + 421, width: 58, height: 6, overflow: 'hidden' };
    div = $('<div/>').attr({ id: 'batt_led2_id' }).css(css_obj);
    fragment.appendChild(div[0]);

    document.body.appendChild(fragment);

    panes_api.create_img_panel('input_img_id', './image/ups/grw.gif', x_pos + 322, y_pos + 210 + 100, 50, 50, 0); //RECTIFIER powerfail
    panes_api.create_img_panel('bypass_img_id', './image/ups/ssww.gif', x_pos + 398, y_pos + 130 + 100, 50, 50, 0); //SSI powerfail
    panes_api.create_img_panel('output_img_id', './image/ups/wrr.gif', x_pos + 478, y_pos + 210 + 100, 50, 50, 0); ////INVERTER
    panes_api.create_img_panel('batt_img_id', './image/ups/Batgrv.gif', x_pos + 398, y_pos + 294 + 100, 50, 50, 0); //BATTERY powerfail

    set_symbols();
  }

  function set_symbols()
  {
    var loadoff = (valtable_obj.STATUS & 0x2) || (valtable_obj.EX_EXTSTATUS != null && (valtable_obj.EX_EXTSTATUS & (0x2000 | 0x400)));
    var bypassbad = (valtable_obj.EX_EXTSTATUS != null && (valtable_obj.EX_EXTSTATUS & 0x200));
    var inputbad = (valtable_obj.INVOLT[0] < 20) || (valtable_obj.BATTVOLT < 20) || (valtable_obj.EX_EXTSTATUS != null && (valtable_obj.EX_EXTSTATUS & 0x20));
    var status = valtable_obj.STATUS;
    var overload;

    if (valtable_obj.OUTPOWER != null) {
      if (outphases > 2) {
        if ((valtable_obj.OUTPOWER[0] > 100) || (valtable_obj.OUTPOWER[1] > 100) || (valtable_obj.OUTPOWER[2] > 100))
          overload = 1;
        else
          overload = 0;
      } else {
        if (valtable_obj.OUTPOWER[0] > 100)
          overload = 1;
        else
          overload = 0;
      }
    }

    // input leds
    if ((status & 0x88) || inputbad) // powerfail or input-high or input bad
    {
      $('#input_led_id').css('background-color', '#DC3632'); //red
      $('#input_led2_id').css('background-color', '#DC3632'); //red
    } else {
      $('#input_led_id').css('background-color', '#52B245'); //green
      $('#input_led2_id').css('background-color', '#52B245'); //green
    }
    if (loadoff)
    {
      $('#input_led2_id').css('background-color', '#DC3632'); //red
    }

    // input img
    if (status & 0x8) // powerfail
    {
      $('#input_img_id').attr('src', './image/ups/grr.gif');
    } else if (inputbad || loadoff) {
      $('#input_img_id').attr('src', './image/ups/grr.gif');
    } else {
      $('#input_img_id').attr('src', './image/ups/grgr.gif');
    }

    // bypass img
    if (status & 0x8) // powerfail
    {
      if (status & 0x1) // and bypass
        $('#bypass_img_id').attr('src', './image/ups/sswgr.gif');
      else
        $('#bypass_img_id').attr('src', './image/ups/ssww.gif');
      if (bypassbad) // and bypass bad
        $('#bypass_img_id').attr('src', './image/ups/sswr.gif');
    }
    else if (status & 0x1) // only bypass
    {
      $('#bypass_img_id').attr('src', './image/ups/sswge.gif');
    }
    else if (bypassbad) // UPS_AB_BYPASSBAD The Bypass is out of tolerance
    {
      $('#bypass_img_id').attr('src', './image/ups/sswr.gif');
    }
    else if ((status & 0x4) || loadoff)  // online mode or output off
    {
      $('#bypass_img_id').attr('src', './image/ups/ssww.gif');
    }
    else
    {
      $('#bypass_img_id').attr('src', './image/ups/sswge.gif');
    }

    // output led
    if (loadoff) // load off
    {
      $('#output_led_id').css('background-color', '#DC3632'); // red
    }
    else if ((status & 0x4000) != 0x4000 && (status & 0x4) != 0x4) // commlost and active
    {
      $('#output_led_id').css('background-color', '#DC3632'); //red
    }
    else if (status & 0x1) // bypass
    {
      $('#output_led_id').css('background-color', '#FFF500'); // yellow
    }
    else
    {
      $('#output_led_id').css('background-color', '#52B245'); //green
    }

    // output img
    if ((status & 0x200) // inverterfailure
      ||
      (status & 0x1000) // inverterwarn
      ||
      (status & 0x100) // output high
      ||
      (status & 0x20) // overtemp
      ||
      (status & 0x1) // bypass
      ||
      (overload == 1)) // overload
    {
      $('#output_img_id').attr('src', './image/ups/wrr.gif');
    }
    else if (loadoff) // load off
    {
      $('#output_img_id').attr('src', './image/ups/wrr.gif');
    }
    else
    {
      $('#output_img_id').attr('src', './image/ups/wrgr.gif');
    }

    // battery leds
    if (loadoff)
    {
      $('#batt_led_id').css('background-color', '#FFF500'); // yellow
      $('#batt_led2_id').css('background-color', '#DC3632'); //red
    }
    else if (status & 0xC) // powerfail or online mode
    {
      $('#batt_led_id').css('background-color', '#52B245'); // green
      $('#batt_led2_id').css('background-color', '#52B245'); // green
    }

    // battery img
    var battimg = './image/ups/Batgrv.gif';  // good

    if (loadoff)
    {
      battimg = './image/ups/Batgev.gif';
    }
    else if (status & 0xC) // powerfail or online mode
    {
      battimg = './image/ups/Batgrv.gif';
    }
    else if ((status & 0x10) || (status & 0x400)) // battery low, battery bad
    {
      battimg = './image/ups/Batrv.gif';
    }
    else
    {
      battimg = './image/ups/Batgev.gif';
    }

    // MPX industrial ups special
    var battcond = valtable_obj.BATTCONDITION;
    if (battcond == 1)  // weak
    {
      battimg = './image/ups/Batgev.gif';
    }
    else if (battcond == 2)  // replace
    {
      battimg = './image/ups/Batrv.gif';
    }

    $('#batt_img_id').attr('src', battimg);

    //bypass led
    if (status & 0x8) // powerfail
    {
      if (status & 0x1) // and bypass
      {
        $('#bypass_led_id').css('background-color', '#52B245'); //green
        $('#bypass_outled_id').css('background-color', '#FFF500'); // yellow
        $('#bypass_out2led_id').css('background-color', '#FFF500'); // yellow
      }
      else
      {
        $('#bypass_led_id').css('background-color', '#DC3632'); //red
        $('#bypass_outled_id').css('background-color', 'lightgray'); //gray #F1F1F2
        $('#bypass_out2led_id').css('background-color', 'lightgray'); //gray #F1F1F2
      }
      if (bypassbad)
      {
        $('#bypass_led_id').css('background-color', '#DC3632'); //red
        $('#bypass_outled_id').css('background-color', '#DC3632'); //red
        $('#bypass_out2led_id').css('background-color', '#DC3632'); //red
      }
    }
    else if (status & 0x1) // only bypass
    {
      $('#bypass_led_id').css('background-color', '#52B245'); //green
      $('#bypass_outled_id').css('background-color', '#FFF500');  // yellow
      $('#bypass_out2led_id').css('background-color', '#FFF500');  // yellow
    }
    else if (bypassbad) // UPS_AB_BYPASSBAD The Bypass is out of tolerance
    {
      $('#bypass_led_id').css('background-color', '#DC3632'); //red
      $('#bypass_outled_id').css('background-color', '#DC3632'); //red
      $('#bypass_out2led_id').css('background-color', '#DC3632'); //red
    }
    else if (loadoff)
    {
      $('#bypass_led_id').css('background-color', '#DC3632'); //red
      $('#bypass_outled_id').css('background-color', 'lightgray');
      $('#bypass_out2led_id').css('background-color', 'lightgray');
    }
    else if (status & 0x4) // online mode
    {
      $('#bypass_led_id').css('background-color', '#52B245'); //green
      $('#bypass_outled_id').css('background-color', 'lightgray');
      $('#bypass_out2led_id').css('background-color', 'lightgray');
    }
    else
    {
      $('#bypass_led_id').css('background-color', '#FFF500'); //gelb
      $('#bypass_outled_id').css('background-color', '#FFF500'); //gelb
      $('#bypass_out2led_id').css('background-color', '#FFF500'); //gelb
    }
  }

  function create_status_pane() //UPS status
  {
    var status_div = $('<div/>'),
    status_span;
    css_obj = { position: 'absolute', top: y_pos + 44, left: x_pos + 30, align: 'center', overflow: 'hidden' };
    $(status_div).css(css_obj);
    status_span = $('<span />', { id: 'ups_status_id', 'class': get_status_class(), title: i18n('Click to see Status Text') }).append(i18n('Status'));
    status_span.css('cursor', 'pointer');
    status_span.click(function (event) //visibility: hidden
      {
        var sys_mod_status_txt_div = $('#sys_mod_status_txt_div'),
        display = sys_mod_status_txt_div.css('display') == 'none' ? 'block' : 'none';
        sys_mod_status_txt_div.css({ display: display });
      });

    $('<h4 />').append(status_span).appendTo(status_div);
    status_div.appendTo(document.body);
  }

  function set_ups_status() //set UPS status
  {
    $('#ups_status_id').attr('class', get_status_class());
  }

  function get_status_class() {
    if (status & 0xF7BA)
    return 'label label-danger';

    if (status & 0x41)
    return 'label label-warning';

    if (valtable_obj.EX_EXTSTATUS != null) {
      if (valtable_obj.EX_EXTSTATUS & 0x75FE48) {
        return 'label label-danger';
      }
    }

    if (valtable_obj.RAWSTATEA != null) {
      if (valtable_obj.RAWSTATEA & 0x7C9) {
        return 'label label-danger';
      }
    }

    return 'label label-success';
  }

  function create_outpower_loadbars() //outpower loadbars
  {
    var fragment = document.createDocumentFragment(),
    div;

    if (valtable_obj.OUTPOWER == null)
    return true;

    outpower_loadbars_created = true;

    panes_api.create_loadbar_pane({ value: valtable_obj.OUTPOWER[0], id: 'loadbar1_id', left: x_pos + 684, top: y_pos + 290, title: 'L1', scaler: 1 });
    if (outphases > 1)
    panes_api.create_loadbar_pane({ value: valtable_obj.OUTPOWER[1], id: 'loadbar2_id', left: x_pos + 684, top: y_pos + 320, title: 'L2', scaler: 0 });
    if (outphases > 2)
    panes_api.create_loadbar_pane({ value: valtable_obj.OUTPOWER[2], id: 'loadbar3_id', left: x_pos + 684, top: y_pos + 350, title: 'L3', scaler: 0 });

    css_obj = { position: 'absolute', top: y_pos + 264, left: x_pos + 660, width: 250, height: 120, 'background-color': $(document.body).css('background-color'), 'z-index': 20 };
    div = $('<div/>').attr({ id: 'loadbars' }).css(css_obj);
    fragment.appendChild(div[0]);

    document.body.appendChild(fragment);
    set_outpower_loadbars();
  }

  function set_outpower_loadbars() //outpower loadbars
  {
    if (valtable_obj.OUTPOWER == null)
    return true;
    if (valtable_obj.OUTPOWER[0] != -9999) {
      panes_api.set_loabar_value(valtable_obj.OUTPOWER[0], 'loadbar1_id');
    }
    if ((outphases > 1) && valtable_obj.OUTPOWER[1] != -9999) {
      panes_api.set_loabar_value(valtable_obj.OUTPOWER[1], 'loadbar2_id');
    }
    if ((outphases > 2) && valtable_obj.OUTPOWER[2] != -9999) {
      panes_api.set_loabar_value(valtable_obj.OUTPOWER[2], 'loadbar3_id');
    }

    if ((valtable_obj.OUTPOWER[0] != -9999) || (valtable_obj.OUTPOWER[1] != -9999) || (valtable_obj.OUTPOWER[2] != -9999)) {
      $('#loadbars').css('display', 'none');
    } else {
      $('#loadbars').css('display', 'block');
    }
  }
/*
  function create_rom_temp() //rom temp
  {
    var fragment = document.createDocumentFragment(),
    rom_temp_div;
    css_obj = { position: 'absolute', top: y_pos + 290, left: x_pos + 10 };
    rom_temp_div = $('<div/>').attr({ id: 'rom_temp_div_id' }).css(css_obj);
    panes_api.create_rom_temp_pane(valtable_obj, { id: 'rom_temp_table_id', 'class': 'table table-condensed table-hover' }, "Battery Temp", "Module Temp").appendTo(rom_temp_div);
    fragment.appendChild(rom_temp_div[0]);
    document.body.appendChild(fragment);
  }
*/
  function create_sys_mod_status_panel() {
    var fragment = document.createDocumentFragment(),
    tr, tr, table, tbody, a,
    title = i18n("Status") + ":" + valtable_obj.MODEL,
    status_txt_array = get_sys_mod_status_txt(),
    sys_mod_status_txt_div;

    css_obj = {
      position: 'absolute',
      top: y_pos + 80,
      left: x_pos + 60,
      align: 'center',
      'background-color': '#F1F1F1',
      display: 'none',
      border: 'solid 1px black',
      'z-index': 70
    },

    sys_mod_status_txt_div = $('<div/>', { id: 'sys_mod_status_txt_div', 'class': 'img-rounded' }).css(css_obj);

    if (ups_id != 0)
    title += " [" + i18n("Module") + " " + ups_id + "]";
    a = $('<a/>', { 'class': 'list-group-item active', id: 'status_model_name_id' }).append(title).appendTo(sys_mod_status_txt_div);
    a.css('height', 30);
    table = $('<table />', { 'class': 'table table-condensed', id: 'sys_mod_status_txt_table' }).appendTo(sys_mod_status_txt_div); // table-hover
    table.css({ 'background-color': '#F1F1F1' })
    tbody = $('<tbody />', { id: 'sys_mod_status_txt_table_body' }).appendTo(table);
    $.each(status_txt_array, function (index, status_txt) {
      tr = $('<tr />').appendTo(tbody);
      td = $('<td />', { 'align': 'left' }).append(status_txt).appendTo(tr);
    });

    fragment.appendChild(sys_mod_status_txt_div[0]);
    document.body.appendChild(fragment);
  }

  function set_sys_mod_status() {
    var tr, td,
    title = "Status:" + valtable_obj.MODEL,
    tbody = $('#sys_mod_status_txt_table_body'),
    status_txt_array = get_sys_mod_status_txt();

    if (ups_id != 0)
    title += " [" + i18n("Module") + " " + ups_id + "]";

    $('#status_model_name_id').text(title);

    tbody.empty();
    $.each(status_txt_array, function (index, status_txt) {
      tr = $('<tr />').appendTo(tbody);
      td = $('<td />', { 'align': 'left' }).append(status_txt).appendTo(tr);
    });
  }


  function get_sys_mod_status_txt() {
    var status_txt_array = [];
    $.each(status_obj, function (index, status_obj) {
      if (status_obj.bit & valtable_obj.STATUS) {
        status_txt_array.push(i18n(status_obj.txt));
      }
    });

    if (valtable_obj.EX_EXTSTATUS != null) {
      $.each(ext_status_obj, function (index, status_obj) {
        if (status_obj.bit & valtable_obj.EX_EXTSTATUS) {
          status_txt_array.push(i18n(status_obj.txt));
        }
      });
    }
    if (valtable_obj.RAWSTATEA != null) {
      $.each(alarms_obj, function (index, status_obj) {
        if (status_obj.bit & valtable_obj.RAWSTATEA) {
          status_txt_array.push(i18n(status_obj.txt));
        }
      });
    }

    return status_txt_array;
  }

  function get_module_values(id) {
    jqmodule.unsubscribe(cb);
    ups_id = id;
    req_params.refresh_tm = 0;

    if (plattform == utils_api.plattform_cs141) {
      return get_cs141_module_values(id);
    }

    req_params.url = get_url("data.cgi"),
    req_params.data = { json: "ups", ups_id: ups_id };
    cb = jqmodule.subscribe(this, { update: update, error: err_funct }, req_params);
  }

  function get_cs141_module_values(id) {
    var cs141_cmd_option = {
      data: {},
      data_type: 'text',
      success: cs141_cmd_update,
      error: err_funct
    },

    cs141_cmd_settings = { methode: "POST", xhttp_methode_override: 'PUT', req_payload_json: true },
    ctl_url = get_url("api/devices/ups/control");

    cs141_cmd_option.data = { code: 'actUpsId', params: { upsId: id } };
    jqmodule.ajax(ctl_url, cs141_cmd_option, cs141_cmd_settings);
  }

  function cs141_cmd_update() {
    if (err_div)
    err_div.css('display', 'none');
    cb = jqmodule.subscribe(this, { update: update, error: err_funct }, req_params);
  }


  function get_module_status_class(module_index) {
    var bit_val = 1 << module_index;
    if ((valtable_obj.RAWSTATEG & bit_val) == bit_val)
    return "btn btn-sm btn-danger";
    return "btn btn-sm btn-default";
  }

  function set_model_name() {
    var model_name = valtable_obj.MODEL;
    if (ups_id != 0)
    model_name += " [" + i18n("Module") + " " + ups_id + "]";
    $('#model_name_id').html(model_name);
  }
}(jQuery));