/*
   sensor_panel.js - Sensor SM_T_H_COM and SM_T_COM screen.
   Copyright (c) 2016, Generex GmbH. All rights reserved.
   Said.
*/

var sensor_api = (function ($) {
  var oid = get_oid_from_url(),
    cb = null,
    pannes_created = false,
    err_div = null,
    check_bowser = true,
    css_obj,
    model_div,
    sensor_obj, report_obj, unms_obj,
    x_pos = 5,
    y_pos = 5,
    sm_t_com_id = 'SM_T_COM',
    sm_t_h_com_id = 'SM_T_H_COM',
    energy_counter_id = 'EnergyCounter',
    c_options = {
      series: {
        lines: { show: true },
        bars: {
          show: true,
          barWidth: 0.6,
          align: 'center',
          lineWidth: 0,
          fill: true,
          fillColor: { colors: [{ opacity: 1 }, { opacity: 1 }, { opacity: 1 }, { opacity: 1 }] }
        },
        color: '#52B245'
      },
      xaxis: { ticks: [], tickLength: 0 },
      yaxis: { min: 0, max: 0 },
      legend: {
        show: false,
        noColumns: 1,
        labelBoxBorderColor: "white",
        backgroundColor: "white",
        position: "nw"
      },
      grid: { backgroundColor: 'white', borderWidth: 1, tickColor: 'black' }
    };


  var req_params = null,

    sensor_cmd_settings = { methode: "POST", xhttp_methode_override: 'PUT', req_payload_json: true },
    ctl_url = get_url("api/devices/sensor/control");

  /*public functions*/
  return {
    init: init
  };

  function init() {
    if (check_bowser && !utils_api.check_bowser()) {
      $('#loading_id').css('display', 'none');
      panes_api.create_br_check_panel(this, browser_check_ok);
      return true;
    }
    create_err_msg_panel();

    if (oid != null) {
      req_params = utils_api.get_req_params(utils_api.device_ups, oid);
    } else {
      req_params = utils_api.get_req_params(utils_api.device_sensorman, oid);
    }

    cb = jqmodule.subscribe(this, { update: update, error: err_funct }, req_params);
  }

  function browser_check_ok() {
    check_bowser = false;
    init();
  }

  function create_err_msg_panel() {
    css_obj = { position: 'absolute', top: y_pos, left: x_pos, width: 791, display: 'none', 'z-index': 22 };
    err_div = panes_api.create_err_panel({ id: 'error_div', 'class': 'alert alert-danger' }, css_obj);
  }

  function err_funct(response) {
    jqmodule.refresh(cb, 10 * 1000);
    panes_api.display_err({ response: response, oid: oid, err_div: err_div, err_msg_div_id: 'error_msg_div' });
  }

  function update(data) {
    if (err_div)
      err_div.css('display', 'none');

    set_variable(data);
    jqmodule.refresh(cb, 10 * 1000);
    if (err_div)
      err_div.css('display', 'none');

    if (window.menu_api != null)
      menu_api.update(data);

    unms_obj = get_unms_obj(data, sensor_obj.deviceType);
    if (unms_obj != null) {
      info_head_api.update(unms_obj);
    }

    if (!pannes_created) {
      pannes_created = true;
      create_panes();
      return false;
    }

    if (sensor_obj.deviceType == sm_t_h_com_id) {
      set_sm_t_h_com_values();
    } else if (sensor_obj.deviceType !== energy_counter_id) {
      set_sm_t_com_value();
    } else {
      set_energy_counter_values();
    }

    set_footer();
  }

  function create_panes() {
    create_bg();
    create_model_panes();

    if (sensor_obj.deviceType == sm_t_h_com_id) {
      create_sm_t_h_com_values();
    } else if (sensor_obj.deviceType == 'EnergyCounter') {
      create_energy_counter();
    } else {
      create_sm_t_com_value();
    }

    create_footer_pane();
  }

  function create_bg() {
    var bg_height = 459;
    if (sensor_obj.deviceType == "EnergyCounter") {
      bg_height = 140
    }

    var fragment = document.createDocumentFragment(),
      bg_div;
    css_obj = { position: 'absolute', top: y_pos, left: x_pos, width: 480, height: bg_height, border: 'solid 1px #808080' };
    bg_div = $('<div/>', { 'class': 'img-rounded' }).css(css_obj);

    fragment.appendChild(bg_div[0]);
    document.body.appendChild(fragment);
  }

  function create_model_panes() {
    var a;
    css_obj = { position: 'absolute', top: y_pos, left: x_pos, align: 'center', width: 480, 'z-index': 10 }; // 'background-color': '#3a87ad'
    attr_obj = { id: 'model_div_id', 'class': 'list-group' };
    model_div = panes_api.create_div(attr_obj, css_obj);
    a = $('<a/>', { 'class': 'list-group-item active', id: 'model_name_id' }).appendTo(model_div);
    if (sensor_obj.deviceType != 'EnergyCounter') {
      a.append('Sensor ' + sensor_obj.deviceType);
    } else {
      a.append(sensor_obj.deviceType);
    }
  }

  function create_sm_t_h_com_values() {
    var left = 60;
    for (var index = 0; index < 2; ++index) {
      create_sm_t_h_com_value(left, index);
      left += 280;
    }
  }

  function create_sm_t_h_com_value(left, index) {
    var fragment = document.createDocumentFragment(),
      data = new Array(),
      sensor_div = null,
      title_div;

    css_obj = {
      position: 'absolute',
      top: 70,
      left: left,
      width: 110,
      height: 45,
      'background-color': 'white',
      align: 'left',
      'text-align': 'left',
      'font-weight': 'bold'
    }; //TEMP1LOCATION
    title_div = $('<div/>', { id: 'sensor_name_' + index }).css(css_obj).append(sensor_obj['TEMP' + (index + 1) + 'LOCATION']);
    fragment.appendChild(title_div[0]);

    css_obj = {
      position: 'absolute',
      'top': 120,
      left: left,
      width: 80,
      height: 300,
      'background-color': 'white',
      'align': 'center'
    };
    sensor_div = $('<div/>').attr({ id: 'sensor_div_' + index }).css(css_obj);
    fragment.appendChild(sensor_div[0]);

    document.body.appendChild(fragment);
    set_sm_t_h_com_value(index);
  }

  function set_sm_t_h_com_values() {
    for (var index = 0; index < 2; ++index) {
      $('#TEMP1HI_THRES' + '_' + index).remove();
      $('#TEMP1LO_THRES' + '_' + index).remove();
      $('#TEMP2HI_THRES' + '_' + index).remove();
      $('#TEMP2LO_THRES' + '_' + index).remove();
      $('#SM2_ANALOG_HI_PREALARMVAL' + '_' + index).remove();
      $('#SM2_ANALOG_LO_PREALARMVAL' + '_' + index).remove();
      set_sm_t_h_com_value(index);
    }
  }

  function set_sm_t_h_com_value(index) {
    var data = new Array();
    var left = index == 0 ? 60 : 341;

    if (index == 0) //Temp
    {
      c_options.yaxis = { min: -30, max: 130 };
      if (sensor_obj.TEMP1UNIT === '°F') {
        c_options.yaxis = {
          min: (-30 * 1.8) + 32.0,
          max: (130 * 1.8) + 32.0
        };
      }
    } else {
      c_options.yaxis = { min: 0, max: 100 };
    }
    c_options.xaxis.ticks.length = 0;
    c_options.series.color = get_sm_t_h_com_color(index);
    data.push({
      label: 'sensor_' + index,
      data: [
        [0, sensor_obj['TEMP' + (index + 1)].toFixed(0)]
      ]
    });
    c_options.xaxis.ticks.push([0, sensor_obj['TEMP' + (index + 1)].toFixed(0) + ' ' + sensor_obj['TEMP' + (index + 1) + 'UNIT']]);

    $.plot('#sensor_div_' + index, data, c_options);

    $('#sensor_name_' + index).text(sensor_obj['TEMP' + (index + 1) + 'LOCATION']);

    //Alarm marker
    if (sensor_obj.SM2_USE_ANALOG_ALARM_HI & (1 << index))
      create_alarm_marker('TEMP' + (index + 1) + 'HI_THRES', index, left, 'Hight Alarm Threshold', '#FF0000');

    if (sensor_obj.SM2_USE_ANALOG_ALARM_LO & (1 << index))
      create_alarm_marker('TEMP' + (index + 1) + 'LO_THRES', index, left, 'Low Alarm Threshold', '#FFFF00');

    if (sensor_obj.SM2_USE_ANALOG_PREALARM_HI & (1 << index))
      create_alarm_marker('TEMP' + (index + 1) + 'PREHI_THRES', index, left, i18n('High PreAlarm Threshold'), '#ee8800');

    if (sensor_obj.SM2_USE_ANALOG_PREALARM_LO & (1 << index))
      create_alarm_marker('TEMP' + (index + 1) + 'PRELO_THRES', index, left, i18n('Low PreAlarm Threshold'), '#ee8800');
  }

  function set_energy_counter_values() {
    for (var index = 1; index < 3; ++index) {
      set_energy_counter_value(index);
    }
  }

  function set_energy_counter_value(index) {
    $('#energy_counter' + index + '_value').text(replacePointWithComma(sensor_obj['EC' + index]) + " " + sensor_obj['EnergyCounterUnit' + (index - 1)]);
    $('#energy_reserved' + index + '_value').text(replacePointWithComma(sensor_obj['EC' + (index + 2)]) + " " + sensor_obj['EnergyCounterUnit' + (index + 1)]);
  }

  function get_sm_t_h_com_color1(index) {
    var val = sensor_obj['TEMP' + (index + 1)],
      lo_thes = sensor_obj['TEMP' + (index + 1) + 'LO_THRES'],
      hi_thes = sensor_obj['TEMP' + (index + 1) + 'HI_THRES'];

    if (val >= hi_thes) {
      return '#ee0000';
    }
    if (val < lo_thes) {
      return '#eeee00';
    }
    return '#52B245';
  }

  function get_sm_t_h_com_color(index) {
    var maske = 1 << index;

    if (((sensor_obj.SM2_ANALOG_HI_ALARM & maske) && (sensor_obj.SM2_USE_ANALOG_ALARM_HI & maske)) ||
      ((sensor_obj.SM2_ANALOG_LO_ALARM & maske) && (sensor_obj.SM2_USE_ANALOG_ALARM_LO & maske))) {
      return '#ee0000'; //rot
    } else {
      if ((sensor_obj.SM2_USE_ANALOG_PREALARM_HI & maske) && (sensor_obj.SM2_ANALOG_HI_PREALARM & maske)) //analogPreAlarmHigh
      {
        return '#ee8800'; //orange /*'#eeee00';//Gelb*/
      } else if ((sensor_obj.SM2_USE_ANALOG_PREALARM_LO & maske) && (sensor_obj.SM2_ANALOG_LO_PREALARM & maske)) //analogPreAlarmLow
      {
        return '#ee8800'; //orange /*'#eeee00';//Gelb*/
      }

      return '#52B245';
    }
  }

  function replacePointWithComma(stringToBeProcessed) {
    var a = stringToBeProcessed.toFixed(1)
    a = a.replace('.', ',');
    return a;
  }

  function create_energy_counter() {

    var fragment = document.createDocumentFragment(),
      data = new Array(),
      energy_counter_div = null,
      title_div;

    energy_counter_div = $('<div/>', { id: 'energy_counter_div_id', position: 'relative' });
    table = $('<table />', { 'class': 'table table-condensed table-hover', 'id': 'inptable', 'text-align': 'left' });

    css_obj = { 'text-align': 'left', 'margin-top': '6px', "margin-left": '90px' };

    table.css(css_obj).appendTo(energy_counter_div);

    tbody = $('<tbody />').appendTo(table);



    tr = $('<tr />', { 'id': 'energy_counter1' }).append($('<td />').append(sensor_obj.EnergyCounterName0)).appendTo(tbody);

    $('<td />', { id: 'energy_counter1_value', }).append(replacePointWithComma(sensor_obj.EC1)).append(" ").append(sensor_obj.EnergyCounterUnit0).appendTo(tr);

    tr = $('<tr />', { 'id': 'energy_counter2' }).append($('<td />').append(sensor_obj.EnergyCounterName1)).appendTo(tbody);
    $('<td />', { id: 'energy_counter2_value' }).append(replacePointWithComma(sensor_obj.EC2)).append(" ").append(sensor_obj.EnergyCounterUnit1).appendTo(tr);


    tr = $('<tr />', { 'id': 'energy_reserved1' }).append($('<td />').append(sensor_obj.EnergyCounterName2)).appendTo(tbody);
    $('<td />', { id: 'energy_reserved1_value' }).append(replacePointWithComma(sensor_obj.EC3)).append(" ").append(sensor_obj.EnergyCounterUnit2).appendTo(tr);

    tr = $('<tr />', { 'id': 'energy_reserved2' }).append($('<td />').append(sensor_obj.EnergyCounterName3)).appendTo(tbody);
    $('<td />', { id: 'energy_reserved2_value' }).append(replacePointWithComma(sensor_obj.EC4)).append(" ").append(sensor_obj.EnergyCounterUnit3).appendTo(tr);

    energy_counter_div.appendTo(model_div);

    fragment.appendChild(model_div[0]);
    document.body.appendChild(fragment);
  }

  function create_sm_t_com_value() {
    var fragment = document.createDocumentFragment(),
      data = new Array(),
      sensor_div = null,
      title_div;

    css_obj = {
      position: 'absolute',
      top: 70,
      left: 200,
      width: 110,
      height: 45,
      'background-color': 'white',
      align: 'left',
      'text-align': 'left',
      'font-weight': 'bold'
    };
    title_div = $('<div/>', { id: 'sensor_name_0' }).css(css_obj).append(sensor_obj.TEMP1LOCATION);
    fragment.appendChild(title_div[0]);

    css_obj = {
      position: 'absolute',
      'top': 120,
      left: 190,
      width: 80,
      height: 300,
      'background-color': 'white',
      'align': 'center'
    };
    sensor_div = $('<div/>').attr({ id: 'sensor_div_0' }).css(css_obj);
    fragment.appendChild(sensor_div[0]);

    document.body.appendChild(fragment);
    set_sm_t_com_value();
  }

  function set_sm_t_com_value() {
    var data = new Array();

    c_options.yaxis = { min: -30, max: 130 };
    if (sensor_obj.TEMP1UNIT === '°F') {
      c_options.yaxis = {
        min: (-30 * 1.8) + 32.0,
        max: (130 * 1.8) + 32.0
      };
    }

    c_options.xaxis.ticks.length = 0;
    c_options.series.color = get_sm_t_h_com_color(0);
    data.push({
      label: 'sensor_0',
      data: [
        [0, sensor_obj.TEMP1.toFixed(0)]
      ]
    });
    c_options.xaxis.ticks.push([0, sensor_obj.TEMP1.toFixed(0) + ' ' + sensor_obj.TEMP1UNIT]);

    $.plot('#sensor_div_0', data, c_options);

    $('#sensor_name_0').text(sensor_obj.TEMP1LOCATION);

    $('#TEMP1HI_THRES_0').remove();
    $('#TEMP1LO_THRES_0').remove();
    $('#SM2_ANALOG_HI_PREALARMVAL_0').remove();
    $('#SM2_ANALOG_LO_PREALARMVAL_0').remove();

    //Alarm marker
    if (sensor_obj.SM2_USE_ANALOG_ALARM_HI & 1)
      create_alarm_marker('TEMP1HI_THRES', 0, 190, 'Hight Alarm Threshold', '#FF0000');

    if (sensor_obj.SM2_USE_ANALOG_ALARM_LO & 1)
      create_alarm_marker('TEMP1LO_THRES', 0, 190, 'Low Alarm Threshold', '#FFFF00');

    if (sensor_obj.SM2_USE_ANALOG_PREALARM_HI & 1)
      create_alarm_marker('TEMP1PREHI_THRES', 0, 190, i18n('High PreAlarm Threshold'), '#ee8800');

    if (sensor_obj.SM2_USE_ANALOG_PREALARM_LO & 1)
      create_alarm_marker('TEMP1PRELO_THRES', 0, 190, i18n('Low PreAlarm Threshold'), '#ee8800');
  }

  function set_variable(data) {
    if (oid != null) //UNMS?
    {
      sensor_obj = data.ups.valtable;
    } else {
      sensor_obj = data.sensor;
    }

    report_obj = data.report;

    if (sensor_obj.TEMP1 != null && sensor_obj.TEMP1UNIT === '°F') {
      sensor_obj.TEMP1 = (sensor_obj.TEMP1 * 1.8) + 32.0;
      sensor_obj.TEMP1HI_THRES = (sensor_obj.TEMP1HI_THRES * 1.8) + 32.0;
      sensor_obj.TEMP1LO_THRES = (sensor_obj.TEMP1LO_THRES * 1.8) + 32.0;
    }

    if (sensor_obj.deviceType == sm_t_h_com_id) {
      sensor_obj.TEMP2UNIT = '% rel H';
    }
  }

  function create_alarm_marker(val_str, index, left, title, color) {
    var fragment = document.createDocumentFragment(),
      div, val, top;

    if (index == 0) //Temp (Range -30 to 100)
    {
      if (sensor_obj.TEMP1UNIT === '°F') {
        val = sensor_obj[val_str] + 22;
        top = (val * 280) / 288;
        top = (400 - top);

      } else {
        val = sensor_obj[val_str] + 30;
        top = (val * 280) / (160);
        top = (400 - top);
      }
    } //
    else //H (Range 0 to 100)
    {
      val = sensor_obj[val_str];
      top = (val * 280) / (100);
      top = (400 - top);
    }

    css_obj = {
      position: 'absolute',
      top: top,
      left: left + 24,
      width: 54,
      height: 4,
      'background-color': color,
      'z-index': 20,
      border: '1px solid black'
    };
    if (index != 0)
      css_obj.left -= 1;

    div = $('<div/>', { id: val_str + '_' + index, title: title }).css(css_obj);
    fragment.appendChild(div[0]);

    document.body.appendChild(fragment);
  }

  function create_footer_pane() {
    if (!oid || !unms_obj)
      return true;
    footer_api.create({ //Attr
        id: 'panel2'
        //'class':'unmsFooter'
      }, { //css
        position: 'absolute',
        top: '490px',
        left: '74px'
      },
      unms_obj
    );
  }

  function set_footer() {
    if (!oid || !unms_obj)
      return true;
    footer_api.set(unms_obj);
  }

}(jQuery));