/*
   siel_panel.js - SIEL screen.
   Copyright (c) 2013-2018, Generex GmbH. All rights reserved.
   Said.
*/

var siel_api = (function ($) {
  var oid = get_oid_from_url(),
  cb = null, //Callback Obj
  pannes_created = false,
  err_div = null,
  check_bowser = true,
  css_obj,
  valtable_obj, report_obj, unms_obj,
  inphases = null,
  outphases = null,
  bypphases = null,
  x_pos = 5,
  y_pos = 5;

  var req_params = null;

  /*public functions*/
  return {
    init: init
  };

  function init() {
    if (check_bowser && !utils_api.check_bowser()) {
      $('#loading_id').css('display', 'none');
      panes_api.create_br_check_panel(this, browser_check_ok);
      return true;
    }
    create_err_msg_panel();

    req_params = utils_api.get_req_params(utils_api.device_ups, oid);
    cb = jqmodule.subscribe(this, { update: update, error: err_funct }, req_params);
  }

  function browser_check_ok() {
    check_bowser = false;
    init();
  }

  function create_err_msg_panel() {
    css_obj = { position: 'absolute', top: y_pos, left: x_pos, width: 791, display: 'none', 'z-index': 22 };
    err_div = panes_api.create_err_panel({ id: 'error_div', 'class': 'alert alert-danger' }, css_obj);
  }

  function err_funct(response) {
    jqmodule.refresh(cb, 10 * 1000);
    panes_api.display_err({ response: response, oid: oid, err_div: err_div, err_msg_div_id: 'error_msg_div' });
  }

  function update(data) {
    if (data.ups == null) {
      return;
    }

    set_variable(data);
    jqmodule.refresh(cb, report_obj.refresh * 1000);
    if (err_div)
    err_div.css('display', 'none');

    if (window.menu_api != null)
    menu_api.update(data);

    unms_obj = get_unms_obj(data);
    if (unms_obj != null) {
      info_head_api.update(unms_obj);
    }

    if (!pannes_created) {
      pannes_created = true;
      create_panes();
      return false;
    }

    set_input_values();
    set_output_values();
    set_byp_values(); //Bypass values
    set_battery_values(); //set Battery value
    set_outpower_loadbars(); //outpower loadbars
    create_aux_alarm_pane();
    create_ext_alarm_pane();
    set_leds();

    set_footer();
    //set .....
  }

  function create_footer_pane() {
    if (!oid || !unms_obj)
    return true;
    footer_api.create({ //Attr
        id: 'panel2'
        //'class':'unmsFooter'
      }, { //css
        position: 'absolute',
        top: '690px',
        left: '74px'
      },
      unms_obj
    );
  }

  function set_footer() {
    if (!oid || !unms_obj)
    return true;
    footer_api.set(unms_obj);
  }

  function set_variable(data) {
    valtable_obj = data.ups.valtable;
    report_obj = data.report;

    if (valtable_obj.INPHASES != null)
    inphases = valtable_obj.INPHASES;
    else
    inphases = valtable_obj.PHASES;

    if (valtable_obj.OUTPHASES != null)
    outphases = valtable_obj.OUTPHASES;
    else
    outphases = valtable_obj.PHASES;

    if (valtable_obj.EX_BYP_NUMLINES != null && valtable_obj.EX_BYP_NUMLINES != -9999)
    bypphases = valtable_obj.EX_BYP_NUMLINES;
    else
    bypphases = inphases;

    valtable_obj._tempUnit = valtable_obj._tempUnit || 0; //0 = Celsius
  }

  function create_panes() {
    utils_api.setTitle(valtable_obj._platform);
    create_bg(); //background
    create_model_panes();
    create_input_pane(); //Input
    create_output_pane(); //Output
    create_byp_pane(); //Bypass panel
    create_battery_pane(); //Battery panel
    create_outpower_loadbars(); //outpower loadbars
    create_aux_alarm_pane();
    create_ext_alarm_pane();
    create_led_panes();

    create_footer_pane();
    //.....
  }

  function create_bg() {
    var fragment = document.createDocumentFragment(),
    img_div;
    css_obj = { position: 'absolute', top: y_pos, left: x_pos, width: 962, height: 522, border: 'solid 1px #808080', align: 'center', 'text-align': 'center' };
    img_div = $('<div/>', { 'id': 'bg_div_id', 'class': 'img-rounded' }).css(css_obj);
    $('<img />').attr({ 'id': 'bg_img_id', 'class': 'img-rounded', src: './image/siel/bg.gif', width: 960, height: 480 }).
    css({ 'margin-top': 20 }).appendTo(img_div);

    fragment.appendChild(img_div[0]);
    document.body.appendChild(fragment);
  }

  function create_led_panes() {
    var fragment = document.createDocumentFragment(),
    img_div;
    css_obj = { position: 'absolute', top: y_pos + 244, left: x_pos + 313, width: 15, height: 8, align: 'center', 'text-align': 'center', 'z-index': 8 };

    img_div = $('<div/>').css(css_obj);
    $('<img />').attr({ 'id': 'leda_img_id', src: get_leda_img(), width: 15, height: 8 }).appendTo(img_div);
    fragment.appendChild(img_div[0]);

    css_obj = { position: 'absolute', top: y_pos + 266, left: x_pos + 313, width: 15, height: 8, align: 'center', 'text-align': 'center', 'z-index': 8 };
    img_div = $('<div/>').css(css_obj);
    $('<img />').attr({ 'id': 'ledb_img_id', src: get_ledb_img(), width: 15, height: 8 }).appendTo(img_div);
    fragment.appendChild(img_div[0]);

    css_obj = { position: 'absolute', top: y_pos + 266, left: x_pos + 401, width: 15, height: 8, align: 'center', 'text-align': 'center', 'z-index': 8 };
    img_div = $('<div/>').css(css_obj);
    $('<img />').attr({ 'id': 'ledc_img_id', src: get_ledc_img(), width: 15, height: 8 }).appendTo(img_div);
    fragment.appendChild(img_div[0]);

    css_obj = { position: 'absolute', top: y_pos + 265, left: x_pos + 487, width: 15, height: 8, align: 'center', 'text-align': 'center', 'z-index': 8 };
    img_div = $('<div/>').css(css_obj);
    $('<img />').attr({ 'id': 'ledd_img_id', src: get_ledd_img(), width: 15, height: 8 }).appendTo(img_div);
    fragment.appendChild(img_div[0]); //

    css_obj = { position: 'absolute', top: y_pos + 243, left: x_pos + 575, width: 15, height: 8, align: 'center', 'text-align': 'center', 'z-index': 8 };
    img_div = $('<div/>').css(css_obj);
    $('<img />').attr({ 'id': 'ledf_img_id', src: get_ledf_img(), width: 15, height: 8 }).appendTo(img_div);
    fragment.appendChild(img_div[0]);

    css_obj = { position: 'absolute', top: y_pos + 266, left: x_pos + 575, width: 15, height: 8, align: 'center', 'text-align': 'center', 'z-index': 8 };
    img_div = $('<div/>').css(css_obj);
    $('<img />').attr({ 'id': 'ledg_img_id', src: get_ledg_img(), width: 15, height: 8 }).appendTo(img_div);
    fragment.appendChild(img_div[0]);

    css_obj = { position: 'absolute', top: y_pos + 188, left: x_pos + 575, width: 15, height: 8, align: 'center', 'text-align': 'center', 'z-index': 8 };
    img_div = $('<div/>').css(css_obj);
    $('<img />').attr({ 'id': 'lede_img_id', src: get_lede_img(), width: 15, height: 8 }).appendTo(img_div);
    fragment.appendChild(img_div[0]);

    document.body.appendChild(fragment);
  }

  function set_leds() {
    $('#leda_img_id').attr('src', get_leda_img());
    $('#ledb_img_id').attr('src', get_ledb_img());
    $('#ledc_img_id').attr('src', get_ledc_img());
    $('#ledd_img_id').attr('src', get_ledd_img());
    $('#ledf_img_id').attr('src', get_ledf_img());
    $('#ledg_img_id').attr('src', get_ledg_img());
    $('#lede_img_id').attr('src', get_lede_img());
  }

  function create_model_panes() {
    var model_div, a;
    css_obj = { position: 'absolute', top: y_pos, left: x_pos, align: 'center', width: 962, 'z-index': 9 }; // 'background-color': '#3a87ad'
    attr_obj = { id: 'model_div_id', 'class': 'list-group' };
    model_div = panes_api.create_div(attr_obj, css_obj);
    a = $('<a/>', { 'class': 'list-group-item active', id: 'model_name_id' }).append(valtable_obj.MODEL).appendTo(model_div);
  }

  function create_input_pane() //Input
  {
    var fragment = document.createDocumentFragment(),
    tr, table, tbody, thead, in_div;

    css_obj = { position: 'absolute', top: y_pos + 186, left: x_pos + 8 };

    in_div = $('<div/>', { id: 'input_div_id' }).css(css_obj);
    table = $('<table />', { 'class': 'table table-condensed table-hover', 'id': 'inptable' }).appendTo(in_div); // table-hover
    table.css('display', 'none');
    tbody = $('<tbody />').appendTo(table);
    thead = $('<thead />').appendTo(table);
    tr = $('<tr />').append($('<th />', { colspan: 4 }).append(i18n(lang_Input))).appendTo(thead);

    if (inphases == 1)
    table.css('margin-top', '22px');

    tr = $('<tr />', { 'id': 'inptrl1' }).append($('<td />').append("L1:")).appendTo(tbody);
    $('<td />', { id: 'input_volt1_id' }).appendTo(tr);
    $('<td />', { id: 'input_cur1_id' }).appendTo(tr);
    $('<td />', { id: 'input_freq1_id' }).appendTo(tr);

    tr = $('<tr />', { 'id': 'inptrl2' }).append($('<td />').append("L2:")).appendTo(tbody);
    $('<td />', { id: 'input_volt2_id' }).appendTo(tr);
    $('<td />', { id: 'input_cur2_id' }).appendTo(tr);

    tr = $('<tr />', { 'id': 'inptrl3' }).append($('<td />').append("L3:")).appendTo(tbody);
    $('<td />', { id: 'input_volt3_id' }).appendTo(tr);
    $('<td />', { id: 'input_cur3_id' }).appendTo(tr);

    fragment.appendChild(in_div[0]);
    document.body.appendChild(fragment);
    set_input_values();
  }

  function set_input_values() {
    var input_val = get_ph_1_3_vals(valtable_obj.INVOLT, valtable_obj.INCURR, valtable_obj.INFREQ, inphases);

    $('#inptable').css('display', 'block');

    if (input_val.availables[0] != 0) {
      $('#inptrl1').css('display', 'block');
      $('#input_volt1_id').text(input_val.val_objs[0].values[0]);
      $('#input_cur1_id').text(input_val.val_objs[1].values[0]);
      $('#input_freq1_id').text(input_val.val_objs[2].values[0]);
    } else {
      $('#inptrl1').css('display', 'none');
    }

    if (input_val.availables[1] != 0) {
      $('#inptrl2').css('display', 'block');
      $('#input_volt2_id').text(input_val.val_objs[0].values[1]);
      $('#input_cur2_id').text(input_val.val_objs[1].values[1]);
    } else {
      $('#inptrl2').css('display', 'none');
    }

    if (input_val.availables[2] != 0) {
      $('#inptrl3').css('display', 'block');
      $('#input_volt3_id').text(input_val.val_objs[0].values[2]);
      $('#input_cur3_id').text(input_val.val_objs[1].values[2]);
    } else {
      $('#inptrl3').css('display', 'none');
    }
  }

  function create_output_pane() //Output
  {
    var output_div = null,
    tr, table, tbody, thead,
    fragment = document.createDocumentFragment();

    css_obj = { position: 'absolute', top: y_pos + 186, left: x_pos + 780 };

    output_div = $('<div/>', { id: 'output_div_id' }).css(css_obj);
    table = $('<table />', { 'class': 'table table-condensed table-hover', 'id': 'outtable' }).appendTo(output_div);
    table.css('display', 'none');
    tbody = $('<tbody />').appendTo(table);
    thead = $('<thead />').appendTo(table);
    tr = $('<tr />').append($('<th />', { colspan: 4 }).append(i18n(lang_Output))).appendTo(thead);

    if (outphases == 1)
    table.css('margin-top', '22px');

    tr = $('<tr />', { 'id': 'outtrl1' }).append($('<td />').append("L1:")).appendTo(tbody);
    $('<td />', { 'id': 'out_volt1_id' }).appendTo(tr);
    $('<td />', { id: 'out_cur1_id' }).appendTo(tr);
    $('<td />', { id: 'out_freq1_id' }).appendTo(tr);

    tr = $('<tr />', { 'id': 'outtrl2' }).append($('<td />').append("L2:")).appendTo(tbody);
    $('<td />', { id: 'out_volt2_id' }).appendTo(tr);
    $('<td />', { id: 'out_cur2_id' }).appendTo(tr);

    tr = $('<tr />', { 'id': 'outtrl3' }).append($('<td />').append("L3:")).appendTo(tbody);
    $('<td />', { id: 'out_volt3_id' }).appendTo(tr);
    $('<td />', { id: 'out_cur3_id' }).appendTo(tr);

    fragment.appendChild(output_div[0]);
    document.body.appendChild(fragment);
    set_output_values();
  }

  function set_output_values() //Output values
  {
    var output_val = get_ph_1_3_vals(valtable_obj.OUTPUT_VOLT, valtable_obj.OUTPUTCURRENT, valtable_obj.OUTFREQ, outphases);

    $('#outtable').css('display', 'block');
    if (output_val.availables[0] != 0) {
      $('#outtrl1').css('display', 'block');
      $('#out_volt1_id').text(output_val.val_objs[0].values[0]);
      $('#out_cur1_id').text(output_val.val_objs[1].values[0]);
      $('#out_freq1_id').text(output_val.val_objs[2].values[0]);
    } else {
      $('#outtrl1').css('display', 'none');
    }

    if (output_val.availables[1] != 0) {
      $('#outtrl2').css('display', 'block');
      $('#out_volt2_id').text(output_val.val_objs[0].values[1]);
      $('#out_cur2_id').text(output_val.val_objs[1].values[1]);
    } else {
      $('#outtrl2').css('display', 'none');
    }

    if (output_val.availables[2] != 0) {
      $('#outtrl3').css('display', 'block');
      $('#out_volt3_id').text(output_val.val_objs[0].values[2]);
      $('#out_cur3_id').text(output_val.val_objs[1].values[2]);
    } else {
      $('#outtrl3').css('display', 'none');
    }
  }

  function create_byp_pane() //Bypass
  {
    var byp_div, tr, table, tbody, thead,
    fragment = document.createDocumentFragment();

    css_obj = { position: 'absolute', top: y_pos + 40, left: x_pos + 426 };
    byp_div = $('<div/>', { id: 'bypass_div_id' }).css(css_obj);
    table = table = $('<table />', { 'class': 'table table-condensed table-hover', 'id': 'byptable' }).appendTo(byp_div);
    table.css('display', 'none');
    tbody = $('<tbody />').appendTo(table);
    thead = $('<thead />').appendTo(table);
    tr = $('<tr />').append($('<th />', { colspan: 4 }).append(i18n(lang_Bypass))).appendTo(thead);

    if (bypphases == 1)
    table.css('margin-top', '50px');

    tr = $('<tr />', { 'id': 'byptrl1' }).append($('<td />').append("L1:")).appendTo(tbody);
    $('<td />', { 'id': 'byp_volt1_id' }).appendTo(tr);
    $('<td />', { id: 'byp_cur1_id' }).appendTo(tr);
    $('<td />', { id: 'byp_freq1_id' }).appendTo(tr);

    tr = $('<tr />', { 'id': 'byptrl2' }).append($('<td />').append("L2:")).appendTo(tbody);
    $('<td />', { id: 'byp_volt2_id' }).appendTo(tr);
    $('<td />', { id: 'byp_cur2_id' }).appendTo(tr);

    tr = $('<tr />', { 'id': 'byptrl3' }).append($('<td />').append("L3:")).appendTo(tbody);
    $('<td />', { id: 'byp_volt3_id' }).appendTo(tr);
    $('<td />', { id: 'byp_cur3_id' }).appendTo(tr);

    fragment.appendChild(byp_div[0]);
    document.body.appendChild(fragment);
    set_byp_values();
  }

  function set_byp_values() //Bypass values
  {
    var bypass_val = get_ph_1_3_vals(valtable_obj.BP_VOLT, valtable_obj.EX_BYP_CURR, valtable_obj.EX_BYP_FREQ, bypphases);

    $('#byptable').css('display', 'block');
    if (bypass_val.availables[0] != 0) {
      $('#byptrl1').css('display', 'block');
      $('#byp_volt1_id').text(bypass_val.val_objs[0].values[0]);
      $('#byp_cur1_id').text(bypass_val.val_objs[1].values[0]);
      $('#byp_freq1_id').text(bypass_val.val_objs[2].values[0]);
    } else {
      $('#byptrl1').css('display', 'none');
    }

    if (bypass_val.availables[1] != 0) {
      $('#byptrl2').css('display', 'block');
      $('#byp_volt2_id').text(bypass_val.val_objs[0].values[1]);
      $('#byp_cur2_id').text(bypass_val.val_objs[1].values[1]);
    } else {
      $('#byptrl2').css('display', 'none');
    }

    if (bypass_val.availables[2] != 0) {
      $('#byptrl3').css('display', 'block');
      $('#byp_volt3_id').text(bypass_val.val_objs[0].values[2]);
      $('#byp_cur3_id').text(bypass_val.val_objs[1].values[2]);
    } else {
      $('#byptrl3').css('display', 'none');
    }
  }

  function create_battery_pane() //Battery panel
  {
    var val, tr, table, tbody, thead, batt_div,
    fragment = document.createDocumentFragment();

    css_obj = { position: 'absolute', top: y_pos + 366, left: x_pos + 360 }; //, width:130, height:100
    attr_obj = { id: 'battery_div_id' };
    batt_div = $('<div/>').attr(attr_obj).css(css_obj);
    table = table = $('<table />', { 'class': 'table table-condensed table-hover' }).appendTo(batt_div);
    tbody = $('<tbody />').appendTo(table);
    thead = $('<thead />').appendTo(table);
    tr = $('<tr />').append($('<th />', { colspan: 2 }).append(i18n(lang_Battery))).appendTo(thead);

    if ((valtable_obj.EX_BATT_VOLTPOS != null && valtable_obj.EX_BATT_VOLTNEG != null) &&
      (valtable_obj.EX_BATT_VOLTPOS != -9999 && valtable_obj.EX_BATT_VOLTNEG != -9999)) {
      tr = $('<tr />').append($('<td />', { align: 'left' }).append('Pos.' + i18n(lang_Voltage) + ":")).appendTo(tbody);
      $('<td />', { id: 'batt_volt_pos_id', align: 'left' }).
      append(get_value({ values: [valtable_obj.EX_BATT_VOLTPOS], index: 0, unit: ' V', precision: 2 })).appendTo(tr);

      tr = $('<tr />').append($('<td />', { align: 'left' }).append('Neg.' + i18n(lang_Voltage) + ":")).appendTo(tbody);
      $('<td />', { id: 'batt_volt_neg_id', align: 'left' }).
      append(get_value({ values: [valtable_obj.EX_BATT_VOLTNEG], index: 0, unit: ' V', precision: 2 })).appendTo(tr);
    } else {
      tr = $('<tr />').append($('<td />', { align: 'left' }).append(i18n(lang_Voltage) + ":")).appendTo(tbody);
      $('<td />', { id: 'batt_volt_id', align: 'left' }).
      append(get_value({ values: [valtable_obj.BATTVOLT], index: 0, unit: ' V', precision: 2 })).appendTo(tr);
    }

    if (valtable_obj.EX_BATT_CURRPOS != null && valtable_obj.EX_BATT_CURRPOS != -9999) {
      tr = $('<tr />').append($('<td />', { align: 'left' }).append(i18n(lang_Current) + ":")).appendTo(tbody);
      $('<td />', { id: 'batt_currpos_id', align: 'left' }).
      append(get_value({ values: [valtable_obj.EX_BATT_CURRPOS], index: 0, unit: ' A', precision: 1 })).appendTo(tr);
    }

    tr = $('<tr />').append($('<td />', { align: 'left' }).append(i18n(lang_Time) + ":")).appendTo(tbody);
    $('<td />', { id: 'batt_autotime_id', align: 'left' }).
    append(get_value({ values: [valtable_obj.AUTONOMTIME], index: 0, unit: ' Min', precision: 0 })).appendTo(tr);

    tr = $('<tr />').append($('<td />', { align: 'left' }).append(i18n(lang_Battery_Capacity_short) + ":")).appendTo(tbody);
    $('<td />', { id: 'batt_cap_id', align: 'left' }).
    append(get_value({ values: [valtable_obj.BATTCAP], index: 0, unit: ' %', precision: 0 })).appendTo(tr);

    tr = $('<tr />').append($('<td />', { align: 'left' }).append(i18n("Temperature:"))).appendTo(tbody);
    $('<td />', { id: 'batt_temp_id', align: 'left' }).
    append(get_value({ values: [valtable_obj.TEMPDEG], index: 0, unit: ' &deg;C', precision: 1 })).appendTo(tr);

    if (valtable_obj.SECSONBATT != null) {
      tr = $('<tr />').append($('<td />', { align: 'left' }).append(i18n('On Battery') + ":")).appendTo(tbody);
      $('<td />', { id: 'batt_seconds_id', align: 'left' }).
      append(get_value({ values: [valtable_obj.SECSONBATT], index: 0, unit: ' Sec', precision: 0 })).appendTo(tr);
    }
    fragment.appendChild(batt_div[0]);
    document.body.appendChild(fragment);
  }

  function set_battery_values() //set Battery value
  {
    if ((valtable_obj.EX_BATT_VOLTPOS != null && valtable_obj.EX_BATT_VOLTNEG != null) &&
      (valtable_obj.EX_BATT_VOLTPOS != -9999 && valtable_obj.EX_BATT_VOLTNEG != -9999)) {
      $('#batt_volt_pos_id').text(get_value({ values: [valtable_obj.EX_BATT_VOLTPOS], index: 0, unit: ' V', precision: 2 }));
      $('#batt_volt_neg_id').text(get_value({ values: [valtable_obj.EX_BATT_VOLTNEG], index: 0, unit: ' V', precision: 2 }));
    } else
    $('#batt_volt_id').text(get_value({ values: [valtable_obj.BATTVOLT], index: 0, unit: ' V', precision: 2 }));

    if (valtable_obj.EX_BATT_CURRPOS != null && valtable_obj.EX_BATT_CURRPOS != -9999) {
      $('#batt_currpos_id').text(get_value({ values: [valtable_obj.EX_BATT_CURRPOS], index: 0, unit: ' A', precision: 1 }));
    }

    $('#batt_autotime_id').text(get_value({ values: [valtable_obj.AUTONOMTIME], index: 0, unit: ' Min', precision: 0 }));
    $('#batt_cap_id').text(get_value({ values: [valtable_obj.BATTCAP], index: 0, unit: ' %', precision: 0 }));
    $('#batt_temp_id').html(get_value({ values: [valtable_obj.TEMPDEG], index: 0, unit: ' &deg;C', precision: 1 }));

    if (valtable_obj.SECSONBATT != null)
    $('#batt_seconds_id').text(get_value({ values: [valtable_obj.SECSONBATT], index: 0, unit: ' Sec', precision: 0 }));
  }

  function create_outpower_loadbars() //outpower loadbars
  {
    var fragment = document.createDocumentFragment(),
    div;

    if (valtable_obj.OUTPOWER == null)
    return true;

    outpower_loadbars_created = true;

    panes_api.create_loadbar_pane({ value: valtable_obj.OUTPOWER[0], id: 'loadbar1_id', left: x_pos + 720, top: y_pos + 406, title: 'L1', scaler: 1 });
    if (outphases > 1)
    panes_api.create_loadbar_pane({ value: valtable_obj.OUTPOWER[1], id: 'loadbar2_id', left: x_pos + 720, top: y_pos + 436, title: 'L2', scaler: 0 });
    if (outphases > 2)
    panes_api.create_loadbar_pane({ value: valtable_obj.OUTPOWER[2], id: 'loadbar3_id', left: x_pos + 720, top: y_pos + 466, title: 'L3', scaler: 0 });

    css_obj = { position: 'absolute', top: y_pos + 310, left: x_pos + 540, width: 250, height: 120, 'background-color': $(document.body).css('background-color'), 'z-index': 20 };
    div = $('<div/>').attr({ id: 'loadbars' }).css(css_obj);
    fragment.appendChild(div[0]);

    document.body.appendChild(fragment);
    set_outpower_loadbars();
  }

  function set_outpower_loadbars() //outpower loadbars
  {
    if (valtable_obj.OUTPOWER == null)
    return true;
    if (valtable_obj.OUTPOWER[0] != -9999) {
      panes_api.set_loabar_value(valtable_obj.OUTPOWER[0], 'loadbar1_id');
    }
    if ((outphases > 1) && valtable_obj.OUTPOWER[1] != -9999) {
      panes_api.set_loabar_value(valtable_obj.OUTPOWER[1], 'loadbar2_id');
    }
    if ((outphases > 2) && valtable_obj.OUTPOWER[2] != -9999) {
      panes_api.set_loabar_value(valtable_obj.OUTPOWER[2], 'loadbar3_id');
    }

    if ((valtable_obj.OUTPOWER[0] != -9999) || (valtable_obj.OUTPOWER[1] != -9999) || (valtable_obj.OUTPOWER[2] != -9999)) {
      $('#loadbars').css('display', 'none');
    } else {
      $('#loadbars').css('display', 'block');
    }
  }

  function create_aux_alarm_pane() {
    var fragment = document.createDocumentFragment(),
    tr, table, tbody, thead, div, index = 0;
    alarms = get_aux_alarms();

    css_obj = { position: 'absolute', top: y_pos + 520, left: x_pos + 8, width: 300 };

    $('#aux_alarm_div_id').remove();

    div = $('<div/>', { id: 'aux_alarm_div_id' }).css(css_obj);
    table = $('<table />', { 'class': 'table table-condensed table-hover', 'id': 'aux_alarm_table' }).appendTo(div);

    tbody = $('<tbody />').appendTo(table);
    thead = $('<thead />').appendTo(table);
    tr = $('<tr />').append($('<th />').append(i18n('Aux Alarms'))).appendTo(thead);

    for (index = 0; index < alarms.length; ++index) {
      tr = $('<tr />').appendTo(tbody);
      $('<td />', { align: 'left' }).append(alarms[index]).appendTo(tr);
    }

    fragment.appendChild(div[0]);
    document.body.appendChild(fragment);
  }

  function create_ext_alarm_pane() {
    var fragment = document.createDocumentFragment(),
    tr, table, tbody, thead, div, index = 0;
    alarms = get_ext_alarms();

    css_obj = { position: 'absolute', top: y_pos + 520, left: x_pos + 308, width: 300 };

    $('#ext_alarm_div_id').remove();

    div = $('<div/>', { id: 'ext_alarm_div_id' }).css(css_obj);
    table = $('<table />', { 'class': 'table table-condensed table-hover', 'id': 'aux_alarm_table' }).appendTo(div);

    tbody = $('<tbody />').appendTo(table);
    thead = $('<thead />').appendTo(table);
    tr = $('<tr />').append($('<th />').append(i18n('Ext. Alarms'))).appendTo(thead);

    for (index = 0; index < alarms.length; ++index) {
      tr = $('<tr />').appendTo(tbody);
      $('<td />', { align: 'left' }).append(alarms[index]).appendTo(tr);
    }

    fragment.appendChild(div[0]);
    document.body.appendChild(fragment);

    $('#bg_div_id').css('height', $('#aux_alarm_div_id').height() > $('#ext_alarm_div_id').height() ? $('#aux_alarm_div_id').height() + 522 :
      $('#ext_alarm_div_id').height() + 522);

  }

  function get_ext_alarms() {
    var alarms = [],
    index = 0;

    if ((valtable_obj.SIEL_STATUSBYTES[8] & 0x01) == 0)
    alarms[index++] = i18n("Temperature Probe KO");
    if ((valtable_obj.SIEL_STATUSBYTES[8] & 0x02) == 0)
    alarms[index++] = i18n("Battery Temperature High") + " &gt;30&deg;C";
    if ((valtable_obj.SIEL_STATUSBYTES[8] & 0x04) == 0)
    alarms[index++] = i18n("Air Temperature High") + " &gt;30&deg;C";
    if ((valtable_obj.SIEL_STATUSBYTES[8] & 0x08) == 0)
    alarms[index++] = i18n("Preventive Maintenace Required");
    if ((valtable_obj.SIEL_STATUSBYTES[8] & 0x10) == 0)
    alarms[index++] = i18n("Maintenace Required");

    return alarms;
  }

  function get_aux_alarms() {
    var alarms = [],
    index = 0;

    if ((valtable_obj.SIEL_STATUSBYTES[4] & 0x01) == 0)
    alarms[index++] = i18n("Int. Input Switch");
    if ((valtable_obj.SIEL_STATUSBYTES[4] & 0x02) == 0)
    alarms[index++] = i18n("Int. Emergency Mains Switch");
    if ((valtable_obj.SIEL_STATUSBYTES[4] & 0x04) == 0)
    alarms[index++] = i18n("Int. Output Switch");
    if ((valtable_obj.SIEL_STATUSBYTES[4] & 0x10) == 0)
    alarms[index++] = i18n("Bad Rectifier Mains");
    if ((valtable_obj.SIEL_STATUSBYTES[4] & 0x40) == 0)
    alarms[index++] = i18n("Int. Rectifier Electric Protections");
    if ((valtable_obj.SIEL_STATUSBYTES[4] & 0x80) == 0)
    alarms[index++] = i18n("Operator Present");
    if ((valtable_obj.SIEL_STATUSBYTES[5] & 0x01) == 0)
    alarms[index++] = i18n("Int. Inverter Electric Protection");
    if ((valtable_obj.SIEL_STATUSBYTES[5] & 0x04) == 0)
    alarms[index++] = i18n("Int. Static Switch Electric Protections");
    if ((valtable_obj.SIEL_STATUSBYTES[5] & 0x08) == 0)
    alarms[index++] = i18n("Int. Static Switch Overtemperature");

    if (valtable_obj._upsProtocol != 0x80a0)
    if ((valtable_obj.SIEL_STATUSBYTES[5] & 0x10) == 0)
    alarms[index++] = i18n("Int. Energy Recovery Relay");

    if ((valtable_obj.SIEL_STATUSBYTES[5] & 0x40) == 0)
    alarms[index++] = i18n("UPS Ventilation Anomaly");
    if ((valtable_obj.SIEL_STATUSBYTES[5] & 0x80) == 0)
    alarms[index++] = i18n("Battery Disabled");

    if (valtable_obj._upsProtocol != 0x80a0)
    if ((valtable_obj.SIEL_STATUSBYTES[6] & 0x01) == 0)
    alarms[index++] = i18n("Batteries Or Output Not Insulated");

    if ((valtable_obj.SIEL_STATUSBYTES[6] & 0x02) == 0)
    alarms[index++] = i18n("Battery Low");
    if ((valtable_obj.SIEL_STATUSBYTES[6] & 0x04) == 0)
    alarms[index++] = i18n("External Limitation In Progress");
    if ((valtable_obj.SIEL_STATUSBYTES[6] & 0x10) == 0)
    alarms[index++] = i18n("General System Anomaly");

    if ((valtable_obj.SIEL_STATUSBYTES[6] & 0x20) == 0)
    alarms[index++] = i18n("Battery Current Outgoing");
    else
    alarms[index++] = i18n("Battery Current Incoming");

    if ((valtable_obj.SIEL_STATUSBYTES[6] & 0x40) == 0)
    alarms[index++] = i18n("Rectifier Overload");
    if ((valtable_obj.SIEL_STATUSBYTES[6] & 0x80) == 0)
    alarms[index++] = i18n("Signaling Not Working");

    if (valtable_obj._upsProtocol != 0x8095) {
      //if((valtable_obj.SIEL_STATUSBYTES[7] & 0x01) == 0) alarms[index++] = "Executable Battery Test";
      //if((valtable_obj.SIEL_STATUSBYTES[7] & 0x02) == 0) alarms[index++] = "Internal Synchronization";
      if ((valtable_obj.SIEL_STATUSBYTES[7] & 0x08) == 0)
      alarms[index++] = "Battery Load KO";
      if ((valtable_obj.SIEL_STATUSBYTES[7] & 0x20) == 0)
      alarms[index++] = i18n("Active Power Incoming Into Inverter");
    }

    return alarms;
  }

  function get_ph_1_3_vals(volt, cur, freq, phases) {
    var val_obj, val;
    l_1_3_vals = {
      availables: [0, 0, 0], //L1-L3
      val_objs: [
        { unit: ' V', precision: 0, values: ['', '', ''] },
        { unit: ' A', precision: 2, values: ['', '', ''] },
        { unit: ' Hz', precision: 2, values: ['', '', ''] }
      ]
    };
    volt = volt || [];
    cur = cur || [], freq = freq || [];
    $.each([volt, cur, freq], function (index, values) {
      val_obj = l_1_3_vals.val_objs[index];
      $.each(values, function (i, val) {
        if (i < phases) {
          val = values[i];
          if (val != -9999) {
            if (values === freq && (i == 1 || i == 2)) {} else
            l_1_3_vals.availables[i] = 1;

            if (Math.round(val) == val)
            val = val.toString();
            else
            val = val.toFixed(val_obj.precision);
            val += val_obj.unit;
            val_obj.values[i] = val;
          }
        }
      });
    });
    return l_1_3_vals;
  }

  function get_value(val_obj) {
    if (!val_obj.values || (val_obj.values.length <= val_obj.index))
    return 'N/A';
    var val = val_obj.values[val_obj.index];
    if (val == null || val_obj.values[val_obj.index] == -9999)
    return 'N/A';

    if (val_obj.unit == ' &deg;C') //Temperature?
    {
      return panes_api.get_temperature(val, valtable_obj._tempUnit, val_obj.precision);
    }

    if (Math.round(val) == val)
    val = val.toString();
    else
    val = val.toFixed(val_obj.precision);
    val += val_obj.unit;
    return val;
  }

  function get_leda_img() {
    if (valtable_obj.SIEL_STATUSBYTES[2] & 0x10)
    return './image/siel/ledr.gif';
    return './image/siel/ledg.gif';
  }

  function get_ledb_img() {
    if (valtable_obj.SIEL_STATUSBYTES[3] & 0x01)
    return './image/siel/ledr.gif';
    return './image/siel/ledg.gif';
  }

  function get_ledc_img() {
    if ((valtable_obj.SIEL_STATUSBYTES[3] & 0x08) == 0)
    return './image/siel/ledg.gif';
    if ((valtable_obj.SIEL_STATUSBYTES[1] & 0x08) == 0)
    return './image/siel/ledo.gif';
    if ((valtable_obj.SIEL_STATUSBYTES[1] & 0x04) == 0)
    return './image/siel/ledr.gif';

    return './image/siel/led.gif';
  }

  function get_ledd_img() {
    if (valtable_obj.SIEL_STATUSBYTES[3] & 0x40)
    return './image/siel/led.gif';
    return './image/siel/ledg.gif';
  }

  function get_lede_img() {
    if (valtable_obj.SIEL_STATUSBYTES[2] & 0x08)
    return './image/siel/led.gif';
    return './image/siel/ledr.gif';
  }

  function get_ledf_img() {
    if (valtable_obj.SIEL_STATUSBYTES[2] & 0x02)
    return './image/siel/led.gif';
    return './image/siel/ledg.gif';
  }

  function get_ledg_img() {
    if (valtable_obj.SIEL_STATUSBYTES[3] & 0x10)
    return './image/siel/led.gif';
    return './image/siel/ledg.gif';
  }

}(jQuery));