/*
   slcube3_panel.js - Salicrus Cube 3 screen.
   Copyright (c) 2013-2016, Generex GmbH. All rights reserved.
   Said.
*/

var slcube3_api = (function ($) {
  var oid = get_oid_from_url(),
  cb = null,
  pannes_created = false,
  err_div = null,
  check_bowser = true,
  outpower_loadbars_created = false,
  css_obj, attr_obj,
  valtable_obj, report_obj, unms_obj,
  inphases = null,
  outphases = null,
  bypphases = null,
  status, tempmax, tempdeg, overload = 0,
  x_pos = 10,
  y_pos = 10;

  var status_obj = [

    { bit: 0x0008, txt: "The UPS is drawing power from the batteries" },
    { bit: 0x0010, txt: "The remaining battery run-time is low" },
    { bit: 0x0400, txt: "One or more batteries have been determined to require replacement" },
    { bit: 0x0020, txt: "A temperature is out of tolerance" },
    { bit: 0x0080, txt: "Input high" },
    { bit: 0x0100, txt: "The output load exceeds the UPS output capacity" },
    { bit: 0x2000, txt: "A general fault in the UPS has been detected" },
    { bit: 0x4000, txt: "A problem has been encountered in the communications between the agent and the UPS" },
    { bit: 0x0200, txt: "Inverter error" },
    { bit: 0x1000, txt: "Inverter warning" },
    { bit: 0x0002, txt: "UPS is in shutdown mode" },
    { bit: 0x0001, txt: "The Bypass is presently engaged on the UPS" },
    { bit: 0x0040, txt: "A test is in progress" },
    { bit: 0x0004, txt: "UPS Output active" },
    { bit: 0x0800, txt: "UPS in ECO mode" }
  ],
  ext_status_obj = [
    { bit: 0x00000008, txt: "The UPS will be unable to sustain the present load when and if the utility power is lost" },
    { bit: 0x00000020, txt: "An input condition is out of tolerance" },
    { bit: 0x00000040, txt: "An output condition (other than OutputOverload) is out of tolerance" },
    { bit: 0x00000200, txt: "The Bypass is out of tolerance" },
    { bit: 0x00000400, txt: "The UPS has shutdown as requested, i.e., the output is off" },
    { bit: 0x00000800, txt: "The entire UPS has shutdown as commanded" },
    { bit: 0x00001000, txt: "An uncorrected problem has been detected within the UPS charger subsystem" },
    { bit: 0x00002000, txt: "The output of the UPS is in the off state" },
    { bit: 0x00004000, txt: "The UPS system is in the off state" },
    { bit: 0x00008000, txt: "The failure of one or more fans in the UPS has been detected" },
    { bit: 0x00010000, txt: "The failure of one or more fuses has been detected" },
    { bit: 0x00040000, txt: "The result of the last diagnostic test indicates a failure" },
    { bit: 0x00100000, txt: "The UPS output is off and the UPS is awaiting the return of input power" },
    { bit: 0x00200000, txt: "An UPS Shutdown After Delay countdown is underway" },
    { bit: 0x00400000, txt: "The UPS will turn off power to the load immediatly" }
  ];

  var req_params = null;

  /*public functions*/
  return {
    init: init
  };

  function init() {
    if (check_bowser && !utils_api.check_bowser()) {
      $('#loading_id').css('display', 'none');
      panes_api.create_br_check_panel(this, browser_check_ok);
      return true;
    }
    create_err_msg_panel();

    req_params = utils_api.get_req_params(utils_api.device_ups, oid);
    cb = jqmodule.subscribe(this, { update: update, error: err_funct }, req_params);
  }

  function browser_check_ok() {
    check_bowser = false;
    init();
  }

  function create_err_msg_panel() {
    css_obj = { position: 'absolute', top: y_pos, left: x_pos, width: 791, display: 'none', 'z-index': 22 };
    err_div = panes_api.create_err_panel({ id: 'error_div', 'class': 'alert alert-danger' }, css_obj);
  }

  function err_funct(response) {
    panes_api.display_err({ response: response, oid: oid, err_div: err_div, err_msg_div_id: 'error_msg_div' });
  }

  function update(data) {
    set_variable(data);
    jqmodule.refresh(cb, report_obj.refresh * 1000);
    if (err_div)
    err_div.css('display', 'none');

    if (window.menu_api != null)
    menu_api.update(data);
    unms_obj = get_unms_obj(data);
    if (unms_obj != null) {
      info_head_api.update(unms_obj);
    }

    if (!pannes_created) {
      pannes_created = true;
      create_panes();
      return false;
    }
    set_input_values(); ////Input values
    set_output_values(); //Output values
    set_battery_value(); //Battery value
    set_byp_values(); //Bypass values
    set_iby_ire_ssi_inv_rec_batt(); //SET IBY, IRE, SSI, INVERTER, RECTIFIER and BATTERY pannes
    set_ups_status(); //set UPS status
    set_outpower_loadbars(); //outpower loadbars
    if (valtable_obj._platform !== "ups_view")
    panes_api.set_rom_temp_values(valtable_obj); //set rom temp values
    set_footer();
    set_sys_mod_status();
  }

  function create_footer_pane() {
    if (!oid || !unms_obj)
    return true;
    footer_api.create({ //Attr
        id: 'panel2'
        //'class':'unmsFooter'
      }, { //css
        position: 'absolute',
        top: '490px',
        left: '74px'
      },
      unms_obj
    );
  }

  function set_footer() {
    if (!oid || !unms_obj)
    return true;
    footer_api.set(unms_obj);
  }

  function set_variable(data) {
    valtable_obj = data.ups.valtable;
    report_obj = data.report;

    status = valtable_obj.STATUS;
    tempmax = valtable_obj.TMPMAX;
    tempdeg = valtable_obj.TEMPDEG;
    level = valtable_obj.LEVEL;

    if (valtable_obj.INPHASES != null)
    inphases = valtable_obj.INPHASES;
    else
    inphases = valtable_obj.PHASES;

    if (valtable_obj.OUTPHASES != null)
    outphases = valtable_obj.OUTPHASES;
    else
    outphases = valtable_obj.PHASES;

    if (valtable_obj.EX_BYP_NUMLINES != null && valtable_obj.EX_BYP_NUMLINES != -9999)
    bypphases = valtable_obj.EX_BYP_NUMLINES;
    else
    bypphases = inphases;

    if (valtable_obj.OUTPOWER != null) {
      if (outphases > 2) {
        if ((valtable_obj.OUTPOWER[0] > 100) || (valtable_obj.OUTPOWER[1] > 100) || (valtable_obj.OUTPOWER[2] > 100))
        overload = 1;
      } else {
        if (valtable_obj.OUTPOWER[0] > 100)
        overload = 1;
      }
    }
  }

  function create_panes() {
    utils_api.setTitle(valtable_obj._platform);
    create_bg(); //background
    create_input_pane(); //Input table
    create_output_pane(); //Outnput   panel
    create_battery_pane(); //Battery panel
    create_byp_pane(); //Bypass panel
    create_iby_ire_ssi_inv_rec_batt(); //create IBY, IRE, SSI, INVERTER, RECTIFIER and BATTERY pannes
    create_status_pane(); //UPS status
    create_model_panes();
    create_outpower_loadbars(); //outpower loadbars
    if (valtable_obj._platform !== "ups_view")
    create_rom_temp(); //rom temp
    create_footer_pane();
    create_sys_mod_status_panel();
  }

  function create_bg() {
    var fragment = document.createDocumentFragment(),
    img_div;
    css_obj = { position: 'absolute', 'text-align': 'center', 'display': 'table-cell', 'virtical-align': 'middle', top: y_pos, left: x_pos, width: 850, height: 460, border: 'solid 1px #808080' };
    img_div = $('<div/>', { 'class': 'img-rounded' }).css(css_obj);
    $('<img />').attr({ id: 'bg_img_id', 'class': 'img-rounded', width: 816, 'height': 459, src: './image/ups/bgn.png' }).appendTo(img_div);
    fragment.appendChild(img_div[0]);
    document.body.appendChild(fragment);
  }

  function create_model_panes() {
    var model_div, a;
    css_obj = { position: 'absolute', top: y_pos, left: x_pos, align: 'center', width: 850 };
    attr_obj = { id: 'model_div_id', 'class': 'list-group' };
    model_div = panes_api.create_div(attr_obj, css_obj);
    a = $('<a/>', { 'class': 'list-group-item active' }).append(valtable_obj.MODEL).appendTo(model_div);
  }

  function create_input_pane() //Input table
  {
    var input_val = get_ph_1_3_vals(valtable_obj.INVOLT, valtable_obj.CUBE_IPCURR, valtable_obj.INFREQ, inphases),
    fragment = document.createDocumentFragment(),
    val, tr, table, tbody, thead, in_div;

    if (input_val.availables[0] == 0 && input_val.availables[1] == 0 && input_val.availables[2] == 0)
    return true;

    css_obj = { position: 'absolute', top: y_pos + 176, left: x_pos + 8 };

    in_div = $('<div/>', { id: 'input_div_id' }).css(css_obj);
    table = $('<table />', { 'class': 'table table-condensed table-hover' }).appendTo(in_div); // table-hover
    tbody = $('<tbody />').appendTo(table);
    thead = $('<thead />').appendTo(table);

    tr = $('<tr />').append($('<th />', { colspan: 4 }).append(i18n(lang_Input))).appendTo(thead);

    if (inphases == 1)
    table.css('margin-top', '22px');

    if (input_val.availables[0] != 0) {
      tr = $('<tr />').append($('<td />').append("L1:")).appendTo(tbody);
      if (!input_val.val_objs[0].values[0].empty())
      $('<td />', { id: 'input_volt1_id', title: 'L1-2 ' + i18n('Input Voltage') }).append(input_val.val_objs[0].values[0]).appendTo(tr);

      if (!input_val.val_objs[1].values[0].empty())
      $('<td />', { id: 'input_cur1_id', title: 'L1 ' + i18n('Input Current') }).append(input_val.val_objs[1].values[0]).appendTo(tr);

      if (!input_val.val_objs[2].values[0].empty())
      $('<td />', { id: 'input_freq1_id', title: 'L1 ' + i18n('Input Frequency') }).append(input_val.val_objs[2].values[0]).appendTo(tr);
    }

    if (input_val.availables[1] != 0) {
      tr = $('<tr />').append($('<td />').append("L2:")).appendTo(tbody);
      if (!input_val.val_objs[0].values[1].empty())
      $('<td />', { id: 'input_volt2_id', title: 'L2-3 ' + i18n('Input Voltage') }).append(input_val.val_objs[0].values[1]).appendTo(tr);

      if (!input_val.val_objs[1].values[1].empty())
      $('<td />', { id: 'input_cur2_id', title: 'L2 ' + i18n('Input Current') }).append(input_val.val_objs[1].values[1]).appendTo(tr);
    }
    if (input_val.availables[2] != 0) {
      tr = $('<tr />').append($('<td />').append("L3:")).appendTo(tbody);
      if (!input_val.val_objs[0].values[2].empty())
      $('<td />', { id: 'input_volt3_id', title: 'L3-1 ' + i18n('Input Voltage') }).append(input_val.val_objs[0].values[2]).appendTo(tr);

      if (!input_val.val_objs[1].values[2].empty())
      $('<td />', { id: 'input_cur3_id', title: 'L3 ' + i18n('Input Current') }).append(input_val.val_objs[1].values[2]).appendTo(tr);
    }

    fragment.appendChild(in_div[0]);
    document.body.appendChild(fragment);
  }

  function set_input_values() {
    var input_val = get_ph_1_3_vals(valtable_obj.INVOLT, valtable_obj.CUBE_IPCURR, valtable_obj.INFREQ, inphases);
    if (input_val.availables[0] == 0 && input_val.availables[1] == 0 && input_val.availables[2] == 0)
    return true;

    if (input_val.availables[0] != 0) {
      $('#input_volt1_id').text(input_val.val_objs[0].values[0]);
      $('#input_cur1_id').text(input_val.val_objs[1].values[0]);
      $('#input_freq1_id').text(input_val.val_objs[2].values[0]);
    }
    if (input_val.availables[1] != 0) {
      $('#input_volt2_id').text(input_val.val_objs[0].values[1]);
      $('#input_cur2_id').text(input_val.val_objs[1].values[1]);
    }
    if (input_val.availables[2] != 0) {
      $('#input_volt3_id').text(input_val.val_objs[0].values[2]);
      $('#input_cur3_id').text(input_val.val_objs[1].values[2]);
    }
  }

  function create_output_pane() //Output
  {
    var output_val = get_ph_1_3_vals(valtable_obj.OUTPUT_VOLT, valtable_obj.CUBE_OPCURR, [valtable_obj.OUTPUTFREQ], outphases),
    output_div = null,
    val, tr, table, tbody, thead,
    fragment = document.createDocumentFragment();
    if (output_val.availables[0] == 0 && output_val.availables[1] == 0 && output_val.availables[2] == 0)
    return true;

    css_obj = { position: 'absolute', top: y_pos + 176, left: x_pos + 620 };

    output_div = $('<div/>', { id: 'output_div_id' }).css(css_obj);
    table = $('<table />', { 'class': 'table table-condensed table-hover' }).appendTo(output_div);

    tbody = $('<tbody />').appendTo(table);
    thead = $('<thead />').appendTo(table);
    tr = $('<tr />').append($('<th />', { colspan: 4 }).append(i18n(lang_Output))).appendTo(thead);

    if (outphases == 1)
    table.css('margin-top', '22px');

    if (output_val.availables[0] != 0) {
      tr = $('<tr />').append($('<td />').append("L1:")).appendTo(tbody);
      if (!output_val.val_objs[0].values[0].empty())
      $('<td />', { 'id': 'out_volt1_id', title: 'L1-N ' + i18n('Output Voltage') }).append(output_val.val_objs[0].values[0]).appendTo(tr);

      if (!output_val.val_objs[1].values[0].empty())
      $('<td />', { id: 'out_cur1_id', title: 'L1 ' + i18n('Output Current') }).append(output_val.val_objs[1].values[0]).appendTo(tr);

      val = get_value({ values: valtable_obj.CUBE_OPKW, index: 0, unit: ' kW', precision: 1 });
      if (val != 'N/A')
      $('<td />', { id: 'out_act_apparent1_id', title: 'L1 ' + i18n('Output Active Power') }).append(val).appendTo(tr);

      if (!output_val.val_objs[2].values[0].empty())
      $('<td />', { id: 'out_freq1_id', title: i18n('Output Frequency') }).append(output_val.val_objs[2].values[0]).appendTo(tr);
    }
    if (output_val.availables[1] != 0) {
      tr = $('<tr />').append($('<td />').append("L2:")).appendTo(tbody);
      if (!output_val.val_objs[0].values[1].empty())
      $('<td />', { id: 'out_volt2_id', title: 'L2-N ' + i18n('Output Voltage') }).append(output_val.val_objs[0].values[1]).appendTo(tr);

      if (!output_val.val_objs[1].values[1].empty())
      $('<td />', { id: 'out_cur2_id', title: 'L2 ' + i18n('Output Current') }).append(output_val.val_objs[1].values[1]).appendTo(tr);

      val = get_value({ values: valtable_obj.CUBE_OPKW, index: 1, unit: ' kW', precision: 1 });
      if (val != 'N/A')
      $('<td />', { id: 'out_act_apparent2_id', title: 'L2 ' + i18n('Output Active Power') }).append(val).appendTo(tr);

    }
    if (output_val.availables[2] != 0) {
      tr = $('<tr />').append($('<td />').append("L3:")).appendTo(tbody);
      if (!output_val.val_objs[0].values[2].empty())
      $('<td />', { id: 'out_volt3_id', title: 'L3-N ' + i18n('Output Voltage') }).append(output_val.val_objs[0].values[2]).appendTo(tr);

      if (!output_val.val_objs[1].values[2].empty())
      $('<td />', { id: 'out_cur3_id', title: 'L3 ' + i18n('Output Current') }).append(output_val.val_objs[1].values[2]).appendTo(tr);

      val = get_value({ values: valtable_obj.CUBE_OPKW, index: 2, unit: ' kW', precision: 1 });
      if (val != 'N/A')
      $('<td />', { id: 'out_act_apparent3_id', title: 'L2 ' + i18n('Output Active Power') }).append(val).appendTo(tr);
    }

    fragment.appendChild(output_div[0]);
    document.body.appendChild(fragment);
  }

  function set_output_values() //Output values
  {
    var val,
    output_val = get_ph_1_3_vals(valtable_obj.OUTPUT_VOLT, valtable_obj.CUBE_OPCURR, [valtable_obj.OUTPUTFREQ], outphases);
    if (output_val.availables[0] == 0 && output_val.availables[1] == 0 && output_val.availables[2] == 0)
    return true;

    if (output_val.availables[0] != 0) {
      $('#out_volt1_id').text(output_val.val_objs[0].values[0]);
      $('#out_cur1_id').text(output_val.val_objs[1].values[0]);
      $('#out_freq1_id').text(output_val.val_objs[2].values[0]);
    }
    if (output_val.availables[1] != 0) {
      $('#out_volt2_id').text(output_val.val_objs[0].values[1]);
      $('#out_cur2_id').text(output_val.val_objs[1].values[1]);
    }
    if (output_val.availables[2] != 0) {
      $('#out_volt3_id').text(output_val.val_objs[0].values[2]);
      $('#out_cur3_id').text(output_val.val_objs[1].values[2]);
    }

    val = get_value({ values: valtable_obj.CUBE_OPKW, index: 0, unit: ' kW', precision: 1 });
    if (val != 'N/A')
    $('#out_act_apparent1_id').text(val);

    val = get_value({ values: valtable_obj.CUBE_OPKW, index: 1, unit: ' kW', precision: 1 });
    if (val != 'N/A')
    $('#out_act_apparent2_id').text(val);

    val = get_value({ values: valtable_obj.CUBE_OPKW, index: 2, unit: ' kW', precision: 1 });
    if (val != 'N/A')
    $('#out_act_apparent3_id').text(val);

  }

  function create_byp_pane() //Bypass
  {
    var bypass_val = get_ph_1_3_vals(valtable_obj.CUBE_BYPVOLT, valtable_obj.CUBE_BYPCURR, [valtable_obj.CUBE_BYPFREQ], bypphases),
    byp_div, val, tr, table, tbody, thead,
    fragment = document.createDocumentFragment();

    if (bypass_val.availables[0] == 0 && bypass_val.availables[1] == 0 && bypass_val.availables[2] == 0)
    return true;

    css_obj = { position: 'absolute', top: y_pos + 50, left: x_pos + 436 };
    byp_div = $('<div/>', { id: 'bypass_div_id' }).css(css_obj);
    table = table = $('<table />', { 'class': 'table table-condensed table-hover' }).appendTo(byp_div);
    tbody = $('<tbody />').appendTo(table);
    thead = $('<thead />').appendTo(table);
    tr = $('<tr />').append($('<th />', { colspan: 4 }).append(i18n(lang_Bypass))).appendTo(thead);

    if (bypphases == 1)
    table.css('margin-top', '50px');

    if (bypass_val.availables[0] != 0) {
      tr = $('<tr />').append($('<td />').append("L1:")).appendTo(tbody);
      if (!bypass_val.val_objs[0].values[0].empty())
      $('<td />', { 'id': 'byp_volt1_id' }).append(bypass_val.val_objs[0].values[0]).appendTo(tr);

      if (!bypass_val.val_objs[1].values[0].empty())
      $('<td />', { id: 'byp_cur1_id' }).append(bypass_val.val_objs[1].values[0]).appendTo(tr);

      if (!bypass_val.val_objs[2].values[0].empty())
      $('<td />', { id: 'byp_freq1_id' }).append(bypass_val.val_objs[2].values[0]).appendTo(tr);
    }

    if (bypass_val.availables[1] != 0) {
      tr = $('<tr />').append($('<td />').append("L2:")).appendTo(tbody);
      if (!bypass_val.val_objs[0].values[1].empty())
      $('<td />', { id: 'byp_volt2_id' }).append(bypass_val.val_objs[0].values[1]).appendTo(tr);

      if (!bypass_val.val_objs[1].values[1].empty())
      $('<td />', { id: 'byp_cur2_id' }).append(bypass_val.val_objs[1].values[1]).appendTo(tr);
    }

    if (bypass_val.availables[2] != 0) {
      tr = $('<tr />').append($('<td />').append("L3:")).appendTo(tbody);
      if (!bypass_val.val_objs[0].values[2].empty())
      $('<td />', { id: 'byp_volt3_id' }).append(bypass_val.val_objs[0].values[2]).appendTo(tr);

      if (!bypass_val.val_objs[1].values[2].empty())
      $('<td />', { id: 'byp_cur3_id' }).append(bypass_val.val_objs[1].values[2]).appendTo(tr);
    }
    fragment.appendChild(byp_div[0]);
    document.body.appendChild(fragment);
  }

  function set_byp_values() //Bypass values
  {
    var bypass_val = get_ph_1_3_vals(valtable_obj.CUBE_BYPVOLT, valtable_obj.CUBE_BYPCURR, [valtable_obj.CUBE_BYPFREQ], bypphases);

    if (bypass_val.availables[0] == 0 && bypass_val.availables[1] == 0 && bypass_val.availables[2] == 0)
    return true;

    if (bypass_val.availables[0] != 0) {
      $('#byp_volt1_id').text(bypass_val.val_objs[0].values[0]);
      $('#byp_cur1_id').text(bypass_val.val_objs[1].values[0]);
      $('#byp_freq1_id').text(bypass_val.val_objs[2].values[0]);
    }
    if (bypass_val.availables[1] != 0) {
      $('#byp_volt2_id').text(bypass_val.val_objs[0].values[1]);
      $('#byp_cur2_id').text(bypass_val.val_objs[1].values[1]);
    }
    if (bypass_val.availables[2] != 0) {
      $('#byp_volt3_id').text(bypass_val.val_objs[0].values[2]);
      $('#byp_cur3_id').text(bypass_val.val_objs[1].values[2]);
    }
  }


  //Standardscreen erweitern: Wenn EX_BATT_VOLTPOS und EX_BATT_VOLTNEG vorhanden und ungleich UPS_VAL_NA (-9999) dann beide Werte anstatt BATTVOLT anzeigen.
  function create_battery_pane() //Battery panel
  {
    var val, tr, table, tbody, thead, batt_div,
    fragment = document.createDocumentFragment();

    css_obj = { position: 'absolute', top: y_pos + 306, left: x_pos + 210 }; //, width:130, height:100
    attr_obj = { id: 'battery_div_id' };
    batt_div = $('<div/>').attr(attr_obj).css(css_obj);
    table = table = $('<table />', { 'class': 'table table-condensed table-hover' }).appendTo(batt_div);
    tbody = $('<tbody />').appendTo(table);
    thead = $('<thead />').appendTo(table);
    tr = $('<tr />').append($('<th />', { colspan: 2 }).append(i18n(lang_Battery))).appendTo(thead);

    if ((valtable_obj.CUBE_BATTVOLTPOS != null && valtable_obj.CUBE_BATTVOLTNEG != null) &&
      (valtable_obj.CUBE_BATTVOLTPOS != -9999 && valtable_obj.CUBE_BATTVOLTNEG != -9999)) {
      tr = $('<tr />').append($('<td />', { align: 'left' }).append('Pos.' + i18n(lang_Voltage) + ":")).appendTo(tbody);
      $('<td />', { id: 'batt_volt_pos_id', align: 'left' }).
      append(get_value({ values: [valtable_obj.CUBE_BATTVOLTPOS], index: 0, unit: ' V', precision: 2 })).appendTo(tr);

      tr = $('<tr />').append($('<td />', { align: 'left' }).append('Neg.' + i18n(lang_Voltage) + ":")).appendTo(tbody);
      $('<td />', { id: 'batt_volt_neg_id', align: 'left' }).
      append(get_value({ values: [valtable_obj.CUBE_BATTVOLTNEG], index: 0, unit: ' V', precision: 2 })).appendTo(tr);
    } else {
      tr = $('<tr />').append($('<td />', { align: 'left' }).append(i18n(lang_Voltage) + ":")).appendTo(tbody);
      $('<td />', { id: 'batt_volt_id', align: 'left' }).
      append(get_value({ values: [valtable_obj.BATTVOLT], index: 0, unit: ' V', precision: 0 })).appendTo(tr);
    }

    tr = $('<tr />').append($('<td />', { align: 'left' }).append(i18n(lang_Time) + ":")).appendTo(tbody);
    $('<td />', { id: 'batt_autotime_id', align: 'left' }).
    append(get_value({ values: [valtable_obj.AUTONOMTIME], index: 0, unit: ' Min', precision: 0 })).appendTo(tr);

    tr = $('<tr />').append($('<td />', { align: 'left' }).append(i18n(lang_Battery_Capacity_short) + ":")).appendTo(tbody);
    $('<td />', { id: 'batt_cap_id', align: 'left' }).
    append(get_value({ values: [valtable_obj.BATTCAP], index: 0, unit: ' %', precision: 0 })).appendTo(tr);

    if (valtable_obj.EX_BATT_SECONBATT != null) {
      tr = $('<tr />').append($('<td />', { align: 'left' }).append(i18n('Seconds') + ":")).appendTo(tbody);
      $('<td />', { id: 'batt_seconds_id', align: 'left' }).
      append(get_value({ values: [valtable_obj.EX_BATT_SECONBATT], index: 0, unit: ' Sec', precision: 0 })).appendTo(tr);
    }
    fragment.appendChild(batt_div[0]);
    document.body.appendChild(fragment);
  }

  function set_battery_value() //set Battery value
  {
    if ((valtable_obj.CUBE_BATTVOLTPOS != null && valtable_obj.CUBE_BATTVOLTNEG != null) &&
      (valtable_obj.CUBE_BATTVOLTPOS != -9999 && valtable_obj.CUBE_BATTVOLTNEG != -9999)) {
      $('#batt_volt_pos_id').text(get_value({ values: [valtable_obj.CUBE_BATTVOLTPOS], index: 0, unit: ' V', precision: 2 }));
      $('#batt_volt_neg_id').text(get_value({ values: [valtable_obj.CUBE_BATTVOLTNEG], index: 0, unit: ' V', precision: 2 }));
    } else
    $('#batt_volt_id').text(get_value({ values: [valtable_obj.BATTVOLT], index: 0, unit: ' V', precision: 0 }));

    $('#batt_autotime_id').text(get_value({ values: [valtable_obj.AUTONOMTIME], index: 0, unit: ' Min', precision: 0 }));
    $('#batt_cap_id').text(get_value({ values: [valtable_obj.BATTCAP], index: 0, unit: ' %', precision: 0 }));

    if (valtable_obj.EX_BATT_SECONBATT != null)
    $('#batt_seconds_id').text(get_value({ values: [valtable_obj.EX_BATT_SECONBATT], index: 0, unit: ' Sec', precision: 0 }));
  }

  function get_ph_1_3_vals(volt, cur, freq, phases) {
    var val_obj, val;
    l_1_3_vals = {
      availables: [0, 0, 0], //L1-L3
      val_objs: [
        { unit: ' V', precision: 0, values: ['', '', ''] },
        { unit: ' A', precision: 1, values: ['', '', ''] },
        { unit: ' Hz', precision: 1, values: ['', '', ''] }
      ]
    };
    volt = volt || [];
    cur = cur || [], freq = freq || [];
    $.each([volt, cur, freq], function (index, values) {
      val_obj = l_1_3_vals.val_objs[index];
      $.each(values, function (i, val) {
        if (i < phases) {
          val = values[i];
          if (val != -9999) {
            if (values === freq && (i == 1 || i == 2)) {

            } else
            l_1_3_vals.availables[i] = 1;

            if (Math.round(val) == val)
            val = val.toString();
            else
            val = val.toFixed(val_obj.precision);
            val += val_obj.unit;
            val_obj.values[i] = val;
          }
        }
      });
    });
    return l_1_3_vals;
  }

  function get_value(val_obj) {
    if (!val_obj.values || (val_obj.values.length <= val_obj.index))
    return 'N/A';
    var val = val_obj.values[val_obj.index];
    if (val == null || val_obj.values[val_obj.index] == -9999)
    return 'N/A';

    if (Math.round(val) == val)
    val = val.toString();
    else
    val = val.toFixed(val_obj.precision);
    val += val_obj.unit;
    return val;
  }

  function create_iby_ire_ssi_inv_rec_batt() //create IBY, IRE, SSI, INVERTER, RECTIFIER and BATTERY  pannes
  {
    var div, fragment = document.createDocumentFragment();
    css_obj = { position: 'absolute', top: y_pos + 229, left: x_pos + 572, width: 24, height: 10, overflow: 'hidden' };

    if ((status & 16384) != 16384 && (status & 4) != 4) //IBY commlost and active
    css_obj['background-color'] = '#DC3632'; //red
    else
    css_obj['background-color'] = 'rgb(40, 134, 59)'; //'#52B245';//green rgb(40, 134, 59)
    div = $('<div/>').attr({ id: 'iby_div_id', 'class': 'img-rounded' }).css(css_obj);
    fragment.appendChild(div[0]);

    css_obj = { position: 'absolute', top: y_pos + 229, left: x_pos + 230, width: 24, height: 10, overflow: 'hidden' };
    if ((status & 8) == 8 || (status & 128) == 128) //IRE powerfail or input-high
    css_obj['background-color'] = '#DC3632'; //red
    else
    css_obj['background-color'] = 'rgb(40, 134, 59)'; //#52B245';//green
    div = $('<div/>').attr({ id: 'ire_div_id', 'class': 'img-rounded' }).css(css_obj);
    fragment.appendChild(div[0]);

    document.body.appendChild(fragment);

    if ((status & 8) == 8) //SSI powerfail
    panes_api.create_img_panel('ssi_img_id', './image/ups/ssww.gif', x_pos + 388, y_pos + 130, 50, 50, 0);
    else if ((status & 1) == 1) //bypass
    panes_api.create_img_panel('ssi_img_id', './image/ups/sswgr.gif', x_pos + 388, y_pos + 130, 50, 50, 0);
    else
    panes_api.create_img_panel('ssi_img_id', './image/ups/sswge.gif', x_pos + 388, y_pos + 130, 50, 50, 0);

    if ((status & 512) == 512 //INVERTER inverterfail
      ||
      (status & 4096) == 4096 //inverterwarn
      ||
      (status & 256) == 256 //outputhigh
      ||
      (status & 32) == 32 //overtemp
      ||
      (overload == 1)) //overload
    panes_api.create_img_panel('inv_img_id', './image/ups/wrr.gif', x_pos + 458, y_pos + 210, 50, 50, 0);

    else if ((level != 0) && ((tempdeg != null) && (tempdeg != -9999) && (tempmax != null)) && (tempdeg > tempmax)) //overtemp1
    panes_api.create_img_panel('inv_img_id', './image/ups/wrr.gif', x_pos + 458, y_pos + 210, 50, 50, 0);

    else if ((status & 1) == 1) //bypass
    panes_api.create_img_panel('inv_img_id', './image/ups/wrge.gif', x_pos + 458, y_pos + 210, 50, 50, 0);
    else
    panes_api.create_img_panel('inv_img_id', './image/ups/wrgr.gif', x_pos + 458, y_pos + 210, 50, 50, 0);

    if ((status & 8) == 8) //RECTIFIER powerfail
    panes_api.create_img_panel('rec_img_id', './image/ups/grw.gif', x_pos + 320, y_pos + 210, 50, 50, 0);
    else
    panes_api.create_img_panel('rec_img_id', './image/ups/grgr.gif', x_pos + 320, y_pos + 210, 50, 50, 0);

    if ((status & 8) == 8) //BATTERY powerfail
    panes_api.create_img_panel('batt_img_id', './image/ups/Batgrv.gif', x_pos + 388, y_pos + 294, 50, 50, 0);
    else if ((status & 16) == 16 || (status & 1024) == 1024) // batterylow batterybad
    panes_api.create_img_panel('batt_img_id', './image/ups/Batrv.gif', x_pos + 388, y_pos + 294, 50, 50, 0);
    else
    panes_api.create_img_panel('batt_img_id', './image/ups/Batgev.gif', x_pos + 388, y_pos + 294, 50, 50, 0);
  }

  function set_iby_ire_ssi_inv_rec_batt() //SET IBY, IRE, SSI, INVERTER, RECTIFIER and BATTERY  pannes
  {

    if ((status & 16384) != 16384 && (status & 4) != 4) //IBY commlost and active
    $('#iby_div_id').css('background-color', '#DC3632'); //red
    else
    $('#iby_div_id').css('background-color', 'rgb(40, 134, 59)'); //green


    if ((status & 8) == 8 || (status & 128) == 128) //IRE powerfail or input-high
    $('#ire_div_id').css('background-color', '#DC3632'); //red
    else
    $('#ire_div_id').css('background-color', 'rgb(40, 134, 59)'); //green

    if ((status & 8) == 8) //SSI powerfail
    $('#ssi_img_id').attr('src', './image/ups/ssww.gif');
    else if ((status & 1) == 1) //bypass
    $('#ssi_img_id').attr('src', './image/ups/sswgr.gif');
    else
    $('#ssi_img_id').attr('src', './image/ups/sswge.gif');

    if ((status & 512) == 512 //INVERTER inverterfail
      ||
      (status & 4096) == 4096 //inverterwarn
      ||
      (status & 256) == 256 //outputhigh
      ||
      (status & 32) == 32 //overtemp
      ||
      (overload == 1)) //overload
    $('#inv_img_id').attr('src', './image/ups/wrr.gif');

    else if ((level != 0) && ((tempdeg != null) && (tempdeg != "-9999") && (tempmax != null)) && (tempdeg > tempmax)) //overtemp1
    $('#inv_img_id').attr('src', './image/ups/wrr.gif');
    else if ((status & 1) == 1) //bypass
    $('#inv_img_id').attr('src', './image/ups/wrge.gif');
    else
    $('#inv_img_id').attr('src', './image/ups/wrgr.gif');

    if ((status & 8) == 8) //RECTIFIER powerfail
    $('#rec_img_id').attr('src', './image/ups/grw.gif');
    else
    $('#rec_img_id').attr('src', './image/ups/grgr.gif');

    if ((status & 8) == 8) //BATTERY powerfail
    $('#batt_img_id').attr('src', './image/ups/Batgrv.gif');
    else if ((status & 16) == 16 || (status & 1024) == 1024) // batterylow batterybad
    $('#batt_img_id').attr('src', './image/ups/Batrv.gif');
    else
    $('#batt_img_id').attr('src', './image/ups/Batgev.gif');
  }

  function create_status_pane() //UPS status
  {
    var status_div = $('<div/>'),
    status_span;
    css_obj = { position: 'absolute', top: y_pos + 44, left: x_pos + 20, align: 'center', overflow: 'hidden' };
    $(status_div).css(css_obj);
    status_span = $('<span />', { id: 'ups_status_id', 'class': get_status_class(), title: i18n('Click to see Status Text') }).append(i18n('Status'));
    status_span.css('cursor', 'pointer');
    status_span.click(function (event) //visibility: hidden
      {
        var sys_mod_status_txt_div = $('#sys_mod_status_txt_div'),
        display = sys_mod_status_txt_div.css('display') == 'none' ? 'block' : 'none';
        sys_mod_status_txt_div.css({ display: display });
      });

    $('<h4 />').append(status_span).appendTo(status_div);
    status_div.appendTo(document.body);
  }

  function set_ups_status() //set UPS status
  {
    $('#ups_status_id').attr('class', get_status_class());
  }

  function get_status_class() {
    var status = valtable_obj.STATUS;
    if (status & 0xF7BA)
      return 'label label-danger';

    if (status & 0x41)
      return 'label label-warning';

    if (valtable_obj.EX_EXTSTATUS != null) {
      if (valtable_obj.EX_EXTSTATUS & 0x75FE68) {
        return 'label label-danger';
      }
    }
    return 'label label-success';
  }

  function create_sys_mod_status_panel() {
    var fragment = document.createDocumentFragment(),
    tr, tr, table, tbody, a,
    title = "Status:" + valtable_obj.MODEL,
    status_txt_array = get_sys_mod_status_txt(),
    sys_mod_status_txt_div;

    css_obj = {
      position: 'absolute',
      top: y_pos + 72,
      left: x_pos + 78,
      align: 'center',
      'background-color': '#F1F1F1',
      display: 'none',
      border: 'solid 1px black'
    },

    sys_mod_status_txt_div = $('<div/>', { id: 'sys_mod_status_txt_div', 'class': 'img-rounded' }).css(css_obj);

    a = $('<a/>', { 'class': 'list-group-item active', id: 'status_model_name_id' }).append(title).appendTo(sys_mod_status_txt_div);
    a.css('height', 30);
    table = $('<table />', { 'class': 'table table-condensed', id: 'sys_mod_status_txt_table' }).appendTo(sys_mod_status_txt_div); // table-hover
    table.css({ 'background-color': '#F1F1F1' })
    tbody = $('<tbody />', { id: 'sys_mod_status_txt_table_body' }).appendTo(table);
    $.each(status_txt_array, function (index, status_txt) {
      tr = $('<tr />').appendTo(tbody);
      td = $('<td />', { 'align': 'left' }).append(status_txt).appendTo(tr);
    });

    fragment.appendChild(sys_mod_status_txt_div[0]);
    document.body.appendChild(fragment);
  }

  function set_sys_mod_status() {
    var tr, td,
    title = i18n("Status") + ":" + valtable_obj.MODEL,
    tbody = $('#sys_mod_status_txt_table_body'),
    status_txt_array = get_sys_mod_status_txt();

    $('#status_model_name_id').text(title);

    tbody.empty();
    $.each(status_txt_array, function (index, status_txt) {
      tr = $('<tr />').appendTo(tbody);
      td = $('<td />', { 'align': 'left' }).append(status_txt).appendTo(tr);
    });
  }

  function get_sys_mod_status_txt() {
    var status_txt_array = [];
    $.each(status_obj, function (index, status_obj) {
      if (status_obj.bit & valtable_obj.STATUS) {
        status_txt_array.push(i18n(status_obj.txt));
      }
    });

    if (valtable_obj.EX_EXTSTATUS != null) {
      $.each(ext_status_obj, function (index, status_obj) {
        if (status_obj.bit & valtable_obj.EX_EXTSTATUS) {
          status_txt_array.push(i18n(status_obj.txt));
        }
      });
    }

    return status_txt_array;
  }

  function set_outpower_loadbars() //outpower loadbars
  {
    if (valtable_obj.OUTPOWER == null)
    return true;
    if (valtable_obj.OUTPOWER[0] != -9999)
    panes_api.set_loabar_value(valtable_obj.OUTPOWER[0], 'loadbar1_id');
    if ((outphases > 1) && valtable_obj.OUTPOWER[1] != -9999)
    panes_api.set_loabar_value(valtable_obj.OUTPOWER[1], 'loadbar2_id');
    if ((outphases > 2) && valtable_obj.OUTPOWER[2] != -9999)
    panes_api.set_loabar_value(valtable_obj.OUTPOWER[2], 'loadbar3_id');
  }

  function create_outpower_loadbars() //outpower loadbars
  {
    if (valtable_obj.OUTPOWER == null)
    return true;
    outpower_loadbars_created = true;

    if (valtable_obj.OUTPOWER[0] != -9999.00)
    panes_api.create_loadbar_pane({ value: valtable_obj.OUTPOWER[0], id: 'loadbar1_id', left: x_pos + 618, top: y_pos + 336, title: 'L1', scaler: 1 });
    if ((outphases > 1) && valtable_obj.OUTPOWER[1] != -9999.00)
    panes_api.create_loadbar_pane({ value: valtable_obj.OUTPOWER[1], id: 'loadbar2_id', left: x_pos + 618, top: y_pos + 366, title: 'L2', scaler: 0 });
    if ((outphases > 2) && valtable_obj.OUTPOWER[2] != -9999.00)
    panes_api.create_loadbar_pane({ value: valtable_obj.OUTPOWER[2], id: 'loadbar3_id', left: x_pos + 618, top: y_pos + 396, title: 'L3', scaler: 0 });
  }

  function create_rom_temp() //rom temp
  {
    var fragment = document.createDocumentFragment(),
    rom_temp_div;
    css_obj = { position: 'absolute', top: y_pos + 306, left: x_pos + 8 };
    rom_temp_div = $('<div/>').attr({ id: 'rom_temp_div_id' }).css(css_obj);
    panes_api.create_rom_temp_pane(valtable_obj, { id: 'rom_temp_table_id', 'class': 'table table-condensed table-hover' }, 'Battery Temp.').appendTo(rom_temp_div);
    fragment.appendChild(rom_temp_div[0]);
    document.body.appendChild(fragment);
  }

}(jQuery));