/*
   tense_panel.js - Tense EM 07 screen.
   Copyright (c) 2019, Generex GmbH. All rights reserved.
   UW.
*/

var tense_api = (function ($) {
  var oid = get_oid_from_url(),
    cb = null,
    pannes_created = false,
    err_div = null,
    check_bowser = true,
    css_obj, attr_obj,
    valtable_obj = null,
    report_obj = null,
    ups_obj = null,
    unms_obj = null,
    inphases = null,
    ups_protocol = 0,
    prot_snmp = 0x803A,
    bg_height, bg_width;
    x_pos = 10, y_pos = 10;

  var req_params = null;

  /*public functions*/
  return {
    init: init
  };

  function init() {
    if (check_bowser && !utils_api.check_bowser()) {
      $('#loading_id').css('display', 'none');
      panes_api.create_br_check_panel(this, browser_check_ok);
      return true;
    }
    create_err_msg_panel();

    req_params = utils_api.get_req_params(utils_api.device_ups, oid);
    cb = jqmodule.subscribe(this, { update: update, error: err_funct }, req_params);
  }

  function browser_check_ok() {
    check_bowser = false;
    init();
  }

  function create_err_msg_panel() {
    css_obj = { position: 'absolute', top: y_pos, left: x_pos, width: 791, display: 'none', 'z-index': 62 };
    err_div = panes_api.create_err_panel({ id: 'error_div', 'class': 'alert alert-danger' }, css_obj);
  }

  function err_funct(response) {
    jqmodule.refresh(cb, 10 * 1000);
    panes_api.display_err({ response: response, oid: oid, err_div: err_div, err_msg_div_id: 'error_msg_div' });
  }

  function update(data) {
    if (data.ups == null) {
      return;
    }

    set_variable(data);
    jqmodule.refresh(cb, report_obj.refresh * 1000);
    if (err_div)
      err_div.css('display', 'none');

    if (window.menu_api != null)
      menu_api.update(data);

    unms_obj = get_unms_obj(data);
    if (unms_obj != null)
    {
      info_head_api.update(unms_obj);
    }

    if (!pannes_created)
    {
      pannes_created = true;
      create_panes();
      return true;
    }

    set_values();
  }

  function set_values() {
    set_all_values();
    set_model_name();
    set_footer();
  }

  function create_footer_pane() {
    if (!oid || !unms_obj)
      return true;
    footer_api.create({ //Attr
        id: 'panel2'
        //'class':'unmsFooter'
      }, { //css
        position: 'absolute',
        top: '540px',
        left: '74px'
      },
      unms_obj
    );
  }

  function set_footer() {
    if (!oid || !unms_obj)
      return true;
    footer_api.set(unms_obj);
  }

  function set_variable(data)
  {
    ups_obj = data.ups;
    report_obj = data.report;
    valtable_obj = ups_obj.valtable;

    if (valtable_obj._upsProtocol == null)
    {
      ups_protocol = valtable_obj.CONNECT & 0XFFFF;
    }
    else
    {
      ups_protocol = valtable_obj._upsProtocol;
    }

    bg_height = 700;
    bg_width = 800;
  }

  function create_panes() {
    utils_api.setTitle(valtable_obj._platform);
    create_bg();
    create_values_panes();
    create_model_panes();
    create_footer_pane();
  }

  function create_bg() {
    var fragment = document.createDocumentFragment(),
      img_div;

    css_obj = { position: 'absolute', top: y_pos, left: x_pos, width: bg_width, height: bg_height, border: 'solid 1px #808080' };
    img_div = $('<div/>', { 'class': 'img-rounded' }).css(css_obj);
    fragment.appendChild(img_div[0]);
    document.body.appendChild(fragment);
  }

  function create_model_panes() {
    var model_div, a;
    css_obj = { position: 'absolute', top: y_pos, left: x_pos, align: 'center', width: bg_width, 'z-index': 40 }; // 'background-color': '#3a87ad'
    attr_obj = { id: 'model_div_id', 'class': 'list-group' };
    model_div = panes_api.create_div(attr_obj, css_obj);
    a = $('<a/>', { 'class': 'list-group-item active', id: 'model_name_id' }).appendTo(model_div);
    set_model_name();
  }

  /////////////////////////////////////////////////

  function create_vll_pane()
  {
    var val, tr, table, tbody, thead, div,
      fragment = document.createDocumentFragment();

    css_obj = { position: 'absolute', top: y_pos + 70, left: x_pos + 185};
    div = $('<div/>').css(css_obj);
    table = $('<table/>', {'class': 'table table-condensed table-hover table-bordered'}).appendTo(div);
    tbody = $('<tbody />').appendTo(table);
    thead = $('<thead />').appendTo(table);

    tr = $('<tr />').appendTo(thead);
    $('<th />').css('text-align','center').append(i18n('VLL1-2')).appendTo(tr);
    $('<th />').css('text-align','center').append(i18n('VLL2-3')).appendTo(tr);
    $('<th />').css('text-align','center').append(i18n('VLL3-1')).appendTo(tr);

    tr = $('<tr />').appendTo(tbody);
    $('<td />', { id: 'vll1_id', align: 'center', width:140 }).appendTo(tr);
    $('<td />', { id: 'vll2_id', align: 'center', width:140 }).appendTo(tr);
    $('<td />', { id: 'vll3_id', align: 'center', width:140 }).appendTo(tr);

    fragment.appendChild(div[0]);
    document.body.appendChild(fragment);
  }

  function create_table_pane(x, y, label, prefix, postfix)
  {
    var val, tr, table, tbody, thead, div,
      fragment = document.createDocumentFragment();

    css_obj = { position: 'absolute', top: y_pos + y, left: x_pos + x };
    div = $('<div/>').css(css_obj);
    table = $('<table />', { 'class': 'table table-condensed table-hover table-bordered' }).appendTo(div);
    tbody = $('<tbody />').appendTo(table);
    thead = $('<thead />').appendTo(table);
    tr = $('<tr />').append($('<th />')).appendTo(thead);
    $('<th />').css('text-align','center').append(i18n('Present')).appendTo(tr);
    $('<th />').css('text-align','center').append(i18n('Max')).appendTo(tr);
    $('<th />').css('text-align','center').append(i18n('Min')).appendTo(tr);
    $('<th />').css('text-align','center').append(i18n('Average')).appendTo(tr);

    for (var i = 1; i <= 3; i++)
    {
      tr = $('<tr />').append($('<td />', { align: 'left', width:80 }).append(i18n(prefix + 'L' + i + postfix))).appendTo(tbody);
      $('<td />', { id: label + 'l' + i + '_curr_id', align: 'center', width:140 }).appendTo(tr);
      $('<td />', { id: label + 'l' + i + '_max_id', align: 'center', width:140 }).appendTo(tr);
      $('<td />', { id: label + 'l' + i + '_min_id', align: 'center', width:140 }).appendTo(tr);
      $('<td />', { id: label + 'l' + i + '_avg_id', align: 'center', width:140 }).appendTo(tr);
    }

    fragment.appendChild(div[0]);
    document.body.appendChild(fragment);
  }

  function create_values_panes()
  {
    create_vll_pane();
    create_table_pane(80, 150, 'v', 'V ', '-N');
    create_table_pane(80, 280, 'curr', 'Current ', '');
    create_table_pane(80, 410, 'pwr', 'APower ', '');
    create_table_pane(80, 540, 'freq', 'Freq ', '');

    set_all_values();
  }

  function set_all_values()
  {
    var i;

    for (i = 1; i <= 3; i++)
    {
      $('#vll'+ i + '_id').text(get_value({ values: [valtable_obj.METER_VOLT_LL[i - 1]], index: 0, unit: ' V', precision: 1 }));

      $('#vl' + i + '_curr_id').text(get_value({ values: [valtable_obj.METER_VOLT_LN[i - 1]], index: 0, unit: ' V', precision: 2 }));
      $('#vl' + i + '_max_id').text(get_value({ values: [valtable_obj.METER_VOLT_LN_MAX[i - 1]], index: 0, unit: ' V', precision: 2 }));
      $('#vl' + i + '_min_id').text(get_value({ values: [valtable_obj.METER_VOLT_LN_MIN[i - 1]], index: 0, unit: ' V', precision: 2 }));
      $('#vl' + i + '_avg_id').text(get_value({ values: [valtable_obj.METER_VOLT_LN_AVG[i - 1]], index: 0, unit: ' V', precision: 2 }));

      $('#currl' + i + '_curr_id').text(get_value({ values: [valtable_obj.METER_CURR[i - 1]], index: 0, unit: ' A', precision: 2 }));
      $('#currl' + i + '_max_id').text(get_value({ values: [valtable_obj.METER_CURR_MAX[i - 1]], index: 0, unit: ' A', precision: 2 }));
      $('#currl' + i + '_min_id').text(get_value({ values: [valtable_obj.METER_CURR_MIN[i - 1]], index: 0, unit: ' A', precision: 2 }));
      $('#currl' + i + '_avg_id').text(get_value({ values: [valtable_obj.METER_CURR_AVG[i - 1]], index: 0, unit: ' A', precision: 2 }));

      $('#pwrl' + i + '_curr_id').text(get_value({ values: [valtable_obj.METER_APPPWR[i - 1]], index: 0, unit: ' VA', precision: 2 }));
      $('#pwrl' + i + '_max_id').text(get_value({ values: [valtable_obj.METER_APPPWR_MAX[i - 1]], index: 0, unit: ' VA', precision: 2 }));
      $('#pwrl' + i + '_min_id').text(get_value({ values: [valtable_obj.METER_APPPWR_MIN[i - 1]], index: 0, unit: ' VA', precision: 2 }));
      $('#pwrl' + i + '_avg_id').text(get_value({ values: [valtable_obj.METER_APPPWR_AVG[i - 1]], index: 0, unit: ' VA', precision: 2 }));

      $('#freql' + i + '_curr_id').text(get_value({ values: [valtable_obj.METER_FREQ[i - 1]], index: 0, unit: ' Hz', precision: 2 }));
      $('#freql' + i + '_max_id').text(get_value({ values: [valtable_obj.METER_FREQ_MAX[i - 1]], index: 0, unit: ' Hz', precision: 2 }));
      $('#freql' + i + '_min_id').text(get_value({ values: [valtable_obj.METER_FREQ_MIN[i - 1]], index: 0, unit: ' Hz', precision: 2 }));
      $('#freql' + i + '_avg_id').text(get_value({ values: [valtable_obj.METER_FREQ_AVG[i - 1]], index: 0, unit: ' Hz', precision: 2 }));
    }
  }

  function get_value(val_obj, na_string)
  {
    var val = null;

    if (!val_obj.values || (val_obj.values.length <= val_obj.index))
      return na_string == null ? 'N/A' : na_string;

    val = val_obj.values[val_obj.index];
    if (val == null || val == -9999)
      return na_string == null ? 'N/A' : na_string;
    if (ups_protocol == prot_snmp && val == -1)
      return na_string == null ? 'N/A' : na_string;

    val = val.toFixed(val_obj.precision);
    val += val_obj.unit;
    return val;
  }

  function set_model_name() {
    var model_name = valtable_obj.MODEL;
    $('#model_name_id').html(model_name);
  }
}(jQuery));

