/*
 * Common definitions for all TwinSource screens.
 */
const STATUS_BIT_POSITION = {
    sourcesOutOfSync: 0,
    switchAFail: 1,
    switchBFail: 2,
    switchAOn: 3,
    switchBOn: 4,
    bypassAOn: 5,
    bypassBOn: 6,
    sourceAFail: 7,
    sourceBFail: 8,
    generalFault: 9,
    redundancyLost: 10,
    outputHigh: 11,
    outputFail: 12,
    sourceAPreferred: 13,
    sourceBPreferred: 14,
    autoReturnOn: 15,
    inputSelectedRelayOn: 16,
    buzzerOn: 17
  };
  const ALARM_BIT_POSITION = {
    batteryBad: 0,
    onBattery: 1,
    lowBattery: 2,
    depletedBattery: 3,
    tempBad: 4,
    inputBad: 5,
    outputBad: 6,
    outputOverload: 7,
    onBypass: 8,
    outputOffAsReq: 9,
    upsOffAsReq: 10,
    chargerFail: 11,
    upsOutputOff: 12,
    upsSystemOff: 13,
    fanFailure: 14,
    fuseFailure: 15,
    generalFault: 16,
    diagTestFailed: 17,
    communicationLost: 18,
    awaitingPower: 19,
    shutdownPending: 20,
    shutdownimminent: 21,
    testingProgess: 22,
    receptacleOff: 23,
    onExternBypass: 24,
    outputOvervolt: 25
  };
  const FUNCTION_ID = {
    switchSource: 3000,
    acknowledgeUnit: 3001,
    acknowledgeAll: 3002,
    switchConnection: 3003,
    setTime: 3004,
    setName: 3005,
    saveSettings: 3006
  };
  const BUTTON_CATEGORY = {
    deviceMenu: 'deviceMenu',
    adjustTime: 'adjustTime',
    submitNames: 'submitNames',
    changePreferredSource: 'changePreferredSource',
    acknowledgeAll: 'acknowledgeAll',
    isConnected: 'isConnected'
  };

  const DIAGRAM_LED = {
    source1: 'source1',
    source2: 'source2',
    load1: 'load1',
    load2: 'load2'
  };
  const MIN_VAC = 10;
  const BIT_MASK = 1;

  function STSUnitLEDs(unitID, ledNumber) {
    this.id = unitID;
    this.ledNumber = ledNumber;
    this.status = LED_OFF;
    this.color = 'grau';
  }

  function STSDiagramLEDs(unitID, ledType) {
    this.id = unitID;
    this.ledType = ledType;
    this.status = LED_OFF;
    this.color = 'grau';
  }

  const LED_OFF = 0;
  const LED_ON = 1;
  const LED_LOW = 2;
  const LED_MED = 3;
  const LED_HI = 4;

  function getLEDSourceFile(color, state) {
    var led = "./image/twinsource/led";
    if (null == color || null == state) return led + "grau10x10.gif";
    switch (state) {
      case LED_OFF:
        led += "grau10x10.gif";
        break;
      case LED_ON:
        led += color + "10x10.gif";
        break;
      case LED_LOW:
        led += color + "10x10_flash_l.gif";
        break;
      case LED_MED:
        led += color + "10x10_flash_m.gif";
        break;
      case LED_HI:
        led += color + "10x10_flash_h.gif";
        break;
      default:
        led += "grau10x10.gif";
    }
    return led;
  }

  function Point(x ,y) {
    this.x = x;
    this.y = y;
  }

  function getDiagramLEDIDName(type) {
    let baseName = 'diagram_led_';
    if (type===DIAGRAM_LED.source1)
      return baseName + DIAGRAM_LED.source1;
    if (type===DIAGRAM_LED.source2)
      return baseName + DIAGRAM_LED.source2;
    if (type===DIAGRAM_LED.load1)
      return baseName + DIAGRAM_LED.load1;
    if (type===DIAGRAM_LED.load2)
      return baseName + DIAGRAM_LED.load2;
    return null;
  }

  function getUnitName(valTable, unitID) {
    let name;
    switch (unitID) {
      case 0:
        name = valTable.TWINSOURCE_NAME_00;
        break;
      case 1:
        name = valTable.TWINSOURCE_NAME_01;
        break;
      case 2:
        name = valTable.TWINSOURCE_NAME_02;
        break;
      case 3:
        name = valTable.TWINSOURCE_NAME_03;
        break;
      case 4:
        name = valTable.TWINSOURCE_NAME_04;
        break;
      case 5:
        name = valTable.TWINSOURCE_NAME_05;
        break;
      case 6:
        name = valTable.TWINSOURCE_NAME_06;
        break;
      case 7:
        name = valTable.TWINSOURCE_NAME_07;
        break;
      case 8:
        name = valTable.TWINSOURCE_NAME_08;
        break;
      case 9:
        name = valTable.TWINSOURCE_NAME_09;
        break;
      case 10:
        name = valTable.TWINSOURCE_NAME_10;
        break;
      case 11:
        name = valTable.TWINSOURCE_NAME_11;
        break;
      case 12:
        name = valTable.TWINSOURCE_NAME_12;
        break;
      case 13:
        name = valTable.TWINSOURCE_NAME_13;
        break;
      case 14:
        name = valTable.TWINSOURCE_NAME_14;
        break;
      case 15:
        name = valTable.TWINSOURCE_NAME_15;
        break;
      default:
        name = "Unknown";
    }
    return name;
  }

  const MAX_LENGTH_INPUT = 15;

  const canvasWidth = 300;
  const canvasHeight = 150;
  const drawMargin = 10;
  const drawMarginHalf = drawMargin/2;
  const SelectA = new Point(drawMargin,drawMargin);
  const SelectB = new Point(SelectA.x,canvasHeight-drawMargin);
  const BendA = new Point(canvasWidth/2,drawMargin);
  const BendB = new Point(canvasWidth/2,canvasHeight-drawMargin);
  const Intersection = new Point(canvasWidth/2 + (BendB.y-BendA.y)/2,(BendB.y-BendA.y)/2 + drawMargin);
  const Load = new Point(canvasWidth-drawMargin,(BendB.y-BendA.y)/2 + drawMargin);
  const LoadA = new Point(BendA.x + (Intersection.x - BendA.x)/2,BendA.y + (Intersection.y - BendA.y)/2);
  const LoadB = new Point(BendB.x + (Intersection.x - BendB.x)/2,BendB.y + (Intersection.y - BendB.y)/2);
