/*
 * Specific definitions for TwinSource model DCC-I monitor screen.
 */
var twinsourceI_api = (function ($) {
  var oid = get_oid_from_url(),
  cb = null, //Callback Obj
  pannes_created = false,
  err_div = null,
  check_bowser = true,
  css_obj,
  valtable_obj, report_obj, unms_obj,
  inphases = null,
  outphases = null,
  x_pos = 5,
  y_pos = 5,
  frameHeight = 800,
  frameWidth = 1000;

  var req_params = null;

  var ups_cmd_option =
  {
     data: {},
     data_type: 'json',
     success: sts_cmd_update,
     error: err_funct
  },
  ups_cmd_settings = { methode: "POST", xhttp_methode_override: 'PUT', req_payload_json: true },
  ctl_url = get_url("api/devices/ups/control");

  const STS_UNIT = 'ATS Unit';
  const LOAD_ON_SOURCE_1 = 'Load On Source 1';
  const LOAD_ON_SOURCE_2 = 'Load On Source 2';
  const PREFERRED_SOURCE = 'Preferred Source';
  const SOURCE_1_STATUS = 'Source 1 Status';
  const SOURCE_2_STATUS = 'Source 2 Status';
  const LOAD_ON_ALTERNATE = 'Load On Alternate Source'
  const SOURCES_OUT_OF_SYNC = 'Sources Out of Sync';
  const SWITCH_OFF_NORMAL_POSITION = 'Switch Off Normal Position';
  const SYSTEM_OVERLOAD = 'System Overload';
  const SUMMARY_ALARM = 'Summary Alarm';
  const SERVICE_ALARM = 'Service Alarm';
  const SERVICE_ALARM_S1 = 'Service Alarm S1';
  const SERVICE_ALARM_S2 = 'Service Alarm S2';
  const COMMUNICATIONS_STATUS = 'Communications Status';
  const columnHeaders = [ STS_UNIT,
                        LOAD_ON_SOURCE_1,
                        LOAD_ON_SOURCE_2,
                        PREFERRED_SOURCE,
                        SOURCE_1_STATUS,
                        SOURCE_2_STATUS,
                        '',
                        LOAD_ON_ALTERNATE,
                        SOURCES_OUT_OF_SYNC,
                        '',
                        SUMMARY_ALARM,
                        SERVICE_ALARM,
                        SERVICE_ALARM_S1,
                        SERVICE_ALARM_S2,
                        '',
                        COMMUNICATIONS_STATUS,
                      ];
  const MIN_VAC = 10;
  const BIT_MASK = 1;

  /*public functions*/
  return {
    init: init
  };

  function init() {
    if (check_bowser && !utils_api.check_bowser()) {
      $('#loading_id').css('display', 'none');
      panes_api.create_br_check_panel(this, browser_check_ok);
      return true;
    }
    create_err_msg_panel();

    req_params = utils_api.get_req_params(utils_api.device_ups, oid);
    cb = jqmodule.subscribe(this, { update: update, error: err_funct }, req_params);
  }

  function browser_check_ok() {
    check_bowser = false;
    init();
  }

  function create_err_msg_panel() {
    css_obj = { position: 'absolute', top: y_pos, left: x_pos, width: 791, display: 'none', 'z-index': 22 };
    err_div = panes_api.create_err_panel({ id: 'error_div', 'class': 'alert alert-danger' }, css_obj);
  }

  function err_funct(response) {
    jqmodule.refresh(cb, 10 * 1000);
    panes_api.display_err({ response: response, oid: oid, err_div: err_div, err_msg_div_id: 'error_msg_div' });
  }

  function update(data) {
    if (data.ups == null) {
      return;
    }

    set_variable(data);
    jqmodule.refresh(cb, report_obj.refresh * 1000);
    if (err_div)
      err_div.css('display', 'none');

    if (window.menu_api != null)
      menu_api.update(data);

    unms_obj = get_unms_obj(data);
    if (unms_obj != null)
    {
      info_head_api.update(unms_obj);
    }

    if (!pannes_created)
    {
      pannes_created = true;
      create_panes();
      return false;
    }
    setLEDs();
    set_footer();
    //set .....
  }

  function set_variable(data) {
    valtable_obj = data.ups.valtable;
    report_obj = data.report;

    valtable_obj._tempUnit = valtable_obj._tempUnit || 0; //0 = Celsius

    if (valtable_obj.INPHASES != null)
    inphases = valtable_obj.INPHASES;
    else
    inphases = valtable_obj.PHASES;

    if (valtable_obj.OUTPHASES != null)
    outphases = valtable_obj.OUTPHASES;
    else
    outphases = valtable_obj.PHASES;
  }

  /*
   * Update html elements
   */
  function setLEDs() {
    for(let unitID = 1; unitID <= valtable_obj.TWINSOURCE_CONNECTED.length; unitID++) {
      for(let col = 0; col < columnHeaders.length; col++) {
        if(columnIsEmpty(col)) continue;
        unitLED = new STSUnitLEDs(unitID, col);
        setDeviceMenuButton(unitLED.id);
        setLoadOnSource(unitLED);
        setPreferredSource(unitLED);
        setSourcesStatus(unitLED);
        setLoadOnAltrnateSource(unitLED);
        setSourcesOutOfSync(unitLED);
        setServiceAlarm(unitLED);
        setSourceServiceAlarm(unitLED);
        setSummaryAlarm(unitLED);
        setCommunicationsLEDIfColumnIsComunications(unitLED);
      }
    }
  }

  function columnIsEmpty(columnNumber) {
    return ''===columnHeaders[columnNumber];
  }

  function setDeviceMenuButton(unitID) {
    if(!unitIsConnected(unitID)) {
        $('#' + getButtonName(unitID, BUTTON_CATEGORY.deviceMenu)).prop('disabled', true);
    } else {
      $('#' + getButtonName(unitID, BUTTON_CATEGORY.deviceMenu)).prop('disabled', false);
    }
  }

  function columnIsStsUnitDeviceMenuButton(columnNumber) {
    return columnHeaders[columnNumber]===STS_UNIT;
  }

  function setLoadOnSource(unitLED) {
    if(columnIsLoadOnSource1OrSource2(unitLED.ledNumber)) setLEDOfLoadOnSource(unitLED);
  }

  function columnIsLoadOnSource1OrSource2(columnNumber) {
    return columnHeaders[columnNumber]===LOAD_ON_SOURCE_1 || columnHeaders[columnNumber]===LOAD_ON_SOURCE_2;
  }

  function setLEDOfLoadOnSource(unitLED) {
    setLEDWithCallback(unitLED, getColorOfLoadOnSourceLED);
  }

  function getColorOfLoadOnSourceLED(unitLED) {
    if(loadIsOnSource(unitLED))
      return 'gruen';
    return 'grau';
  }

  function loadIsOnSource(unitLED) {
    loadIsOnSourceBitFlag = ((valtable_obj.TWINSOURCE_ALLFLAGS[unitLED.id-1]) >> (4-unitLED.ledNumber)) & BIT_MASK;
    return loadIsOnSourceBitFlag;
  }

  function getColorOfPreferredOrAlternateLoadOnSource(unitLED) {
    if(loadIsOnPreferredSource(unitLED))
      return 'gruen';
    return 'gelb';
  }

  function loadIsOnPreferredSource(unitLED) {
    return (unitLED.ledNumber-1)===valtable_obj.TWINSOURCE_PREFERRED_SOURCE[unitLED.id-1]
  }

  function setPreferredSource(unitLED) {
    if(columnIsPreferredSource(unitLED.ledNumber)) setTextOfPreferredSource(unitLED);
  }

  function columnIsPreferredSource(columnNumber) {
    return columnHeaders[columnNumber]===PREFERRED_SOURCE;
  }

  function setTextOfPreferredSource(unitLED) {
    let text = "";
    if(unitIsConnected(unitLED.id)) {
      text = getPreferredSourceText(unitLED);
    }
    $('#' + getTableDataName(unitLED.id,unitLED.ledNumber)).text(text);
  }

  function getPreferredSourceText(unitLED) {
    if(oneSourceIsPreferred(unitLED))
      return "S" + (parseInt(valtable_obj.TWINSOURCE_PREFERRED_SOURCE[unitLED.id-1]) + 1);
    return "N/A";
  }

  function oneSourceIsPreferred(unitLED) {
    let source1Preferred = (valtable_obj.TWINSOURCE_ALLFLAGS[unitLED.id-1] >> ALL_FLAGS_BIT_POSITION.source1preferred) & BIT_MASK;
    let source2Preferred = (valtable_obj.TWINSOURCE_ALLFLAGS[unitLED.id-1] >> ALL_FLAGS_BIT_POSITION.source2preferred) & BIT_MASK;
    return source1Preferred ^ source2Preferred;
  }

  function setSourcesStatus(unitLED) {
    if(columnIsSource1OrSource2Status(unitLED.ledNumber)) setLEDOfSourceStatus(unitLED);
  }

  function columnIsSource1OrSource2Status(columnNumber) {
    return columnHeaders[columnNumber]===SOURCE_1_STATUS || columnHeaders[columnNumber]===SOURCE_2_STATUS;
  }

  function setLEDOfSourceStatus(unitLED) {
    setLEDWithCallback(unitLED, getColorOfSourceStatusLED);
  }

  function getColorOfSourceStatusLED(unitLED) {
    if(sourceIsInDesiredPowerQualityZone(unitLED))
      return 'gruen';
    if(sourceIsInMarginalPowerQualityZone(unitLED))
      return 'gelb';
    if(sourceIsIUnacceptablePowerQualityZone(unitLED))
      return 'rot';
    return 'rot';
  }

  function sourceIsInDesiredPowerQualityZone(unitLED) {
    values = getSourceStatusValues(unitLED);
    return !(values.voltageInUPQZone || values.voltageInMPQZone || values.dcSupplyFailure || values.frequencyOutOfRange);
  }

  function sourceIsInMarginalPowerQualityZone(unitLED) {
    values = getSourceStatusValues(unitLED);
    return values.voltageInMPQZone && !(values.dcSupplyFailure || values.frequencyOutOfRange);
  }

  function sourceIsIUnacceptablePowerQualityZone(unitLED) {
    values = getSourceStatusValues(unitLED);
    return (values.voltageInUPQZone || values.dcSupplyFailure || values.frequencyOutOfRange);
  }

  function getSourceStatusValues(unitLED) {
    let voltageInUPQZone, voltageInMPQZone, dcSupplyFailure, alternateSourcePreferred, frequencyOutOfRange;
    if(columnHeaders[unitLED.ledNumber]===SOURCE_1_STATUS) {
      voltageInUPQZone =  ((valtable_obj.TWINSOURCE_ALLFLAGS[unitLED.id-1]) >> ALL_FLAGS_BIT_POSITION.source1VoltageInUPQZone) & BIT_MASK;
      voltageInMPQZone =  ((valtable_obj.TWINSOURCE_ALLFLAGS[unitLED.id-1]) >> ALL_FLAGS_BIT_POSITION.source1VoltageInMPQZone) & BIT_MASK;
      dcSupplyFailure =  ((valtable_obj.TWINSOURCE_ALLFLAGS[unitLED.id-1]) >> ALL_FLAGS_BIT_POSITION.source1DCSupplyFailure) & BIT_MASK;
      alternateSourcePreferred =  ((valtable_obj.TWINSOURCE_ALLFLAGS[unitLED.id-1]) >> ALL_FLAGS_BIT_POSITION.source2preferred) & BIT_MASK;
      frequencyOutOfRange =  ((valtable_obj.TWINSOURCE_ALLFLAGS[unitLED.id-1]) >> ALL_FLAGS_BIT_POSITION.source1FreqOutOfRange) & BIT_MASK;
    } else {
      voltageInUPQZone =  ((valtable_obj.TWINSOURCE_ALLFLAGS[unitLED.id-1]) >> ALL_FLAGS_BIT_POSITION.source2VoltageInUPQZone) & BIT_MASK;
      voltageInMPQZone =  ((valtable_obj.TWINSOURCE_ALLFLAGS[unitLED.id-1]) >> ALL_FLAGS_BIT_POSITION.source2VoltageInMPQZone) & BIT_MASK;
      dcSupplyFailure =  ((valtable_obj.TWINSOURCE_ALLFLAGS[unitLED.id-1]) >> ALL_FLAGS_BIT_POSITION.source2DCSupplyFailure) & BIT_MASK;
      alternateSourcePreferred =  ((valtable_obj.TWINSOURCE_ALLFLAGS[unitLED.id-1]) >> ALL_FLAGS_BIT_POSITION.source1preferred) & BIT_MASK;
      frequencyOutOfRange =  ((valtable_obj.TWINSOURCE_ALLFLAGS[unitLED.id-1]) >> ALL_FLAGS_BIT_POSITION.source2FreqOutOfRange) & BIT_MASK;
    }
    sourceStatusValues = {
      voltageInUPQZone: voltageInUPQZone,
      voltageInMPQZone: voltageInMPQZone,
      dcSupplyFailure: dcSupplyFailure,
      alternateSourcePreferred: alternateSourcePreferred,
      frequencyOutOfRange: frequencyOutOfRange,
    }
    return sourceStatusValues;
  }

  function setLoadOnAltrnateSource(unitLED) {
    if(columnHeaders[unitLED.ledNumber]===LOAD_ON_ALTERNATE) setLoadOnAltrnateSourceLED(unitLED);
  }

  function setLoadOnAltrnateSourceLED(unitLED) {
    setLEDWithCallback(unitLED, getColorOfLoadOnAltrnateSourceLED);
  }

  function getColorOfLoadOnAltrnateSourceLED(unitLED) {
    if(loadOnAltrnateSource(unitLED))
      return 'rot';
    return 'grau';
  }

  function loadOnAltrnateSource(unitLED) {
    return ((valtable_obj.TWINSOURCE_ALLFLAGS[unitLED.id-1]) >> ALL_FLAGS_BIT_POSITION.loadOnAlternate) & BIT_MASK;
  }

  function setSourcesOutOfSync(unitLED) {
    if(columnHeaders[unitLED.ledNumber]===SOURCES_OUT_OF_SYNC) setSourcesOutOfSyncLED(unitLED);
  }

  function setSourcesOutOfSyncLED(unitLED) {
    setLEDWithCallback(unitLED, getColorOfSourcesOutOfSyncLED);
  }

  function getColorOfSourcesOutOfSyncLED(unitLED) {
    if(sourcesAreOutOfSync(unitLED))
      return 'rot';
    return 'grau';
  }

  function sourcesAreOutOfSync(unitLED) {
    return ((valtable_obj.TWINSOURCE_ALLFLAGS[unitLED.id-1]) >> ALL_FLAGS_BIT_POSITION.source1AndSource2OutOfSync) & BIT_MASK;
  }

  function setServiceAlarm(unitLED) {
    if(columnIsServiceAlarm(unitLED.ledNumber)) setServiceAlarmLED(unitLED);
  }

  function columnIsServiceAlarm(columnNumber) {
    return columnHeaders[columnNumber]===SERVICE_ALARM;
  }

  function setServiceAlarmLED(unitLED) {
    setLEDWithCallback(unitLED, getColorOfServiceAlarmLED);
  }

  function getColorOfServiceAlarmLED(unitLED) {
    if(serviceAlarm(unitLED))
      return 'rot';
    return 'grau';
  }

  function serviceAlarm(unitLED) {
    let serviceAlarmBitFlagMask = (1 << ALL_FLAGS_BIT_POSITION.source1DCSupplyFailure)
      | (1 << ALL_FLAGS_BIT_POSITION.source2DCSupplyFailure);
    return (valtable_obj.TWINSOURCE_ALLFLAGS[unitLED.id-1]) & serviceAlarmBitFlagMask;
  }

  function setSourceServiceAlarm(unitLED) {
    if(columnIsSource1OrSource2ServiceAlarm(unitLED.ledNumber)) setSourceServiceAlarmLED(unitLED);
  }

  function columnIsSource1OrSource2ServiceAlarm(columnNumber) {
    return columnHeaders[columnNumber]===SERVICE_ALARM_S1 || columnHeaders[columnNumber]===SERVICE_ALARM_S2;
  }

  function setSourceServiceAlarmLED(unitLED) {
    setLEDWithCallback(unitLED, getColorOfSourceServiceAlarmLED);
  }

  function getColorOfSourceServiceAlarmLED(unitLED) {
    if(sourceServiceAlarm(unitLED))
      return 'rot';
    return 'grau';
  }

  function sourceServiceAlarm(unitLED) {
    if(columnHeaders[unitLED.ledNumber]===SERVICE_ALARM_S1) {
      return ((valtable_obj.TWINSOURCE_ALLFLAGS[unitLED.id-1]) >> ALL_FLAGS_BIT_POSITION.source1DCSupplyFailure) & BIT_MASK;
    }
    return ((valtable_obj.TWINSOURCE_ALLFLAGS[unitLED.id-1]) >> ALL_FLAGS_BIT_POSITION.source2DCSupplyFailure) & BIT_MASK;
  }

  function setSwitchOFFNormalPosition(unitLED) {
    if(columnIsSwitchOffNormalPosition(unitLED.ledNumber)) setSwitchOffNormalPositionLED(unitLED);
  }

  function columnIsSwitchOffNormalPosition(columnNumber) {
    return columnHeaders[columnNumber]===SWITCH_OFF_NORMAL_POSITION;
  }

  function setSwitchOffNormalPositionLED(unitLED) {
    setLEDWithCallback(unitLED, getColorOfSwitchOffNormalPositionLED);
  }

  function getColorOfSwitchOffNormalPositionLED(unitLED) {
    if(switchOffNormalPosition(unitLED))
      return 'rot';
    return 'grau';
  }

  function switchOffNormalPosition(unitLED) {
    return ((valtable_obj.TWINSOURCE_ALLFLAGS[unitLED.id-1]) >> ALL_FLAGS_BIT_POSITION.offDefaultPreferred) & BIT_MASK;
  }

  function setSystemOverload(unitLED) {
    if(columnIsSystemOverload(unitLED.ledNumber)) setSystemOverloadLED(unitLED);
  }

  function columnIsSystemOverload(columnNumber) {
    return columnHeaders[columnNumber]===SYSTEM_OVERLOAD;
  }

  function setSystemOverloadLED(unitLED) {
    setLEDWithCallback(unitLED, getColorOfSystemOverloadLED);
  }

  function getColorOfSystemOverloadLED(unitLED) {
    if(systemOverload(unitLED))
      return 'rot';
    return 'grau';
  }

  function systemOverload(unitLED) {
    return ((valtable_obj.TWINSOURCE_ALLFLAGS[unitLED.id-1]) >> ALL_FLAGS_BIT_POSITION.systemOverLoad) & BIT_MASK;
  }

  function setSummaryAlarm(unitLED) {
    if(columnIsSummaryAlarm(unitLED.ledNumber)) setSummaryAlarmLED(unitLED);
  }

  function columnIsSummaryAlarm(columnNumber) {
    return columnHeaders[columnNumber]===SUMMARY_ALARM;
  }

  function setSummaryAlarmLED(unitLED) {
    setLEDWithCallback(unitLED, getColorOfSummaryAlarmLED);
  }

  function getColorOfSummaryAlarmLED(unitLED) {
    if(summaryAlarm(unitLED))
      return 'rot';
    return 'grau';
  }

  function summaryAlarm(unitLED) {
    let summaryAlarmBitFlagMask = (1 << ALL_FLAGS_BIT_POSITION.source1VoltageInMPQZone)
      | (1 << ALL_FLAGS_BIT_POSITION.source1DCSupplyFailure)
      | (1 << ALL_FLAGS_BIT_POSITION.source2VoltageInMPQZone)
      | (1 << ALL_FLAGS_BIT_POSITION.source2DCSupplyFailure)
      | (1 << ALL_FLAGS_BIT_POSITION.source1AndSource2OutOfSync)
      | (1 << ALL_FLAGS_BIT_POSITION.loadOnAlternate)
      | (1 << ALL_FLAGS_BIT_POSITION.systemOverLoad)
      | (1 << ALL_FLAGS_BIT_POSITION.systemOverCurrent)
      | (1 << ALL_FLAGS_BIT_POSITION.setupConfigurationBad)
      | (1 << ALL_FLAGS_BIT_POSITION.offDefaultPreferred)
      | (1 << ALL_FLAGS_BIT_POSITION.source1VoltageInUPQZone)
      | (1 << ALL_FLAGS_BIT_POSITION.source1FreqOutOfRange)
      | (1 << ALL_FLAGS_BIT_POSITION.source2VoltageInUPQZone)
      | (1 << ALL_FLAGS_BIT_POSITION.source2FreqOutOfRange);
    return (valtable_obj.TWINSOURCE_ALLFLAGS[unitLED.id-1]) & summaryAlarmBitFlagMask;
  }

  function setCommunicationsLEDIfColumnIsComunications(unitLED) {
    if(columnHeaders[unitLED.ledNumber]===COMMUNICATIONS_STATUS) setLEDStatusOfComunications(unitLED);
  }

  function setLEDStatusOfComunications(unitLED) {
    unitLED.status = LED_ON;
    if(unitIsConnected(unitLED.id)) {
      unitLED.color = getColorOfCommunicationStatus(unitLED.id);
    } else {
      unitLED.color = 'grau';
    }
    setLED(unitLED);
  }

  function unitIsConnected(unitID) {
    return valtable_obj.TWINSOURCE_CONNECTED[unitID-1];
  }

  function getColorOfCommunicationStatus(unitID) {
    if(!communicationLost(unitID))
      return 'gruen';
    return 'rot';
  }

  function communicationLost(unitID) {
    return ((valtable_obj.TWINSOURCE_ALLFLAGS[unitID-1]) >> ALL_FLAGS_BIT_POSITION.communicationLost) & BIT_MASK;
  }

  function setLED(unitLED) {
    var ledSourceFile = getLEDSourceFile(unitLED.color, unitLED.status);
    $('#' + getLEDName(unitLED.id,unitLED.ledNumber)).attr('src', ledSourceFile);
  }

  function setLEDWithCallback(unitLED, callbackFunction) {
    if(unitIsConnected(unitLED.id)) {
      unitLED.status = LED_ON;
      unitLED.color = callbackFunction(unitLED);
    } else {
      unitLED.color = 'grau';
    }
    setLED(unitLED);
  }

  function set_footer() {
    if (!oid || !unms_obj)
    return true;
    footer_api.set(unms_obj);
  }

  /*
   * Create html elements.
   */
  function create_panes() {
    utils_api.setTitle(valtable_obj._platform);
    createFrame();
    createModelTitleBar();
    create_footer_pane();
    createOverviewTable();
    setLEDs();
  }

  function createFrame() {
    let fragment = document.createDocumentFragment(),
      img_div;
    css_obj = { position: 'absolute', top: y_pos, left: x_pos, width: frameWidth, height: frameHeight, border: 'solid 1px #808080' };
    img_div = $('<div/>', { 'class': 'img-rounded' }).css(css_obj);
    fragment.appendChild(img_div[0]);
    document.body.appendChild(fragment);
  }

  function createModelTitleBar() {
    let model_div, a;
    css_obj = { position: 'absolute', top: y_pos, left: x_pos, align: 'center', width: frameWidth, 'z-index': 40 };
    attr_obj = { id: 'model_div_id', 'class': 'list-group' };
    model_div = panes_api.create_div(attr_obj, css_obj);
    a = $('<a/>', { 'class': 'list-group-item active', id: 'model_name_id' }).appendTo(model_div);
    setModelName();
  }

  function setModelName() {
    var model_name = valtable_obj.MODEL;
    $('#model_name_id').html(model_name);
  }

  function create_footer_pane() {
    if (!oid || !unms_obj)
        return true;
    footer_api.create({ //Attr
        id: 'panel2'
        //'class':'unmsFooter'
    }, { //css
         position: 'absolute',
        top: '490px',
        left: '74px'
    },
       unms_obj
    );
  }

  function createOverviewTable() {
    var val, tr, td, th, table, tbody, thead, img, div
      fragment = document.createDocumentFragment();

    css_obj = { position: 'absolute', top: y_pos + 100, left: x_pos + 50 };
    div = $('<div/>').css(css_obj);
    table = $('<table />', { 'class': 'table table-condensed table-hover' }).appendTo(div);
    tbody = $('<tbody />').appendTo(table);
    thead = $('<thead />').appendTo(table);

    createColumnHeaders(thead);
    createRowsinTable(tbody);

    fragment.appendChild(div[0]);
    document.body.appendChild(fragment);
  }

  function createColumnHeaders(thead) {
    let tr = $('<tr />', {colspan: columnHeaders.length }).appendTo(thead);
    $('<th/>').append('Unit ID').appendTo(tr)
    for(let col = 0; col < columnHeaders.length; col++) {
      createDynamicTableHeader(col, tr);
    }
  }

  function createDynamicTableHeader(columnNumber, tableRow) {
    if(columnIsStsUnitDeviceMenuButton(columnNumber)) {
      return $('<th />', {scope: 'col', class: 'deviceMenu'}).append(columnHeaders[columnNumber]).appendTo(tableRow);
    } else {
      let th = $('<th />', {scope: 'col', class:'rotate' }).appendTo(tableRow);
      let div = $('<div />').appendTo(th);
      $('<span />').append(columnHeaders[columnNumber]).appendTo(div);
    }
  }

  function createRowsinTable(tableBody) {
    for(let unitID = 1; unitID <= valtable_obj.TWINSOURCE_CONNECTED.length; unitID++) {
	    let row = createRowInTable(tableBody, unitID);
      createPlaceholderForItemsForAllColumnsInRow(unitID, row);
    }
  }

  function createRowInTable(tableBody, rowNumber) {
    return createRowWithHeader(rowNumber).appendTo(tableBody);
  }

  function createRowWithHeader(rowNumber) {
    let rowName = 'STS Unit ' + rowNumber + ':';
    let deviceMenuButton = createButtonContainer(rowNumber, BUTTON_CATEGORY.deviceMenu).append(rowName);
    return $('<tr />').append($('<th />', {scope: 'row'}).append($('<td />').append(rowNumber)));
  }

  function createPlaceholderForItemsForAllColumnsInRow(unitID, row) {
    for(let col = 0; col < columnHeaders.length; col++) {
      let td = $('<td />', { id: getTableDataName(unitID, col), align: 'center', width:20 }).appendTo(row);
      createImageOrButtonContainer(unitID, col).appendTo(td);
    }
  }

  function createImageOrButtonContainer(unitID, columnNumber) {
    if(columnIsStsUnitDeviceMenuButton(columnNumber)) {
      let rowName = getUnitName(valtable_obj, unitID-1) + ':';
      return createButtonContainer(unitID, BUTTON_CATEGORY.deviceMenu).append(rowName);
    }
    return createImageContainer(unitID, columnNumber);
  }

  function createImageContainer(unitID, col) {
    return $('<img />', { id: getLEDName(unitID, col), 'src': '' });
  }

  function createButtonContainer(unitID, category) {
    let button = $('<button />', { id: getButtonName(unitID, category), 'src': '', 'class': category });
    button = setOnClickForButtonDependingOnCategoryAndReturn(button, category, unitID);
    return button;
  }

  function setOnClickForButtonDependingOnCategoryAndReturn(button, category, unitID) {
    if (category===BUTTON_CATEGORY.changePreferredSource) {
      button = setChangePreferredSourceOnClickForButton(button, unitID - 1);
    }
    if (category===BUTTON_CATEGORY.deviceMenu) {
      button = setLinkToUnitScreenOnClickForButton(button, unitID - 1);
    }
    return button;
  }

  function setChangePreferredSourceOnClickForButton(button, unitID) {
    return button.on("click",function (event) {
      if (report_obj._platform == "cs141") {
         send_cs141_ups_cmd({ "code": "upsExeCommand", params: { cmdId: FUNCTION_ID.switchSource, params: unitID } });
      } else {
         send_ups_cmd({ type: "ups_cmd", cmd: 'UPS_EXE_COMMAND', p1: FUNCTION_ID.switchSource, p2: unitID });
      }
      event.preventDefault();
   });
  }

  function send_ups_cmd(ups_cmd_parameters)//send UPS command
  {
     var url;
     if (err_div)
        err_div.css('display', 'none');

     url = get_url("data.cgi");
     ups_cmd_option.data = ups_cmd_parameters;
     document.getElementById('loading_id').style['display'] = 'block';

     document.body.style.cursor = 'wait';

     jqmodule.ajax(url, ups_cmd_option);
  }

  function sts_cmd_update(data)//STS command response
  {
     if (err_div)
        err_div.css('display', 'none');
  }

  function send_cs141_ups_cmd(ups_cmd_parameters) {
     ups_cmd_option.data = ups_cmd_parameters;
     jqmodule.ajax(ctl_url, ups_cmd_option, ups_cmd_settings);
  }

  function setLinkToUnitScreenOnClickForButton(button, unitID) {
    return button.on("click",function () {
      let id = unitID + 1;
      window.location.href='twinsourcedcci_unit.html?id='+id;
      return false;
   });
  }

  function getLEDName(unitID, columnNumber) {
    return 'unit_' + unitID + '_led_' + columnNumber;
  }

  function getButtonName(unitID, category) {
    return 'unit_' + unitID + '_button_' + category;
  }

  function getTableDataName(unitID, columnNumber) {
    return 'sts_' + unitID + '_item_' + columnNumber;
  }
}(jQuery));