/*
 * Specific definitions for TwinSource model DCC-I detailed unit screen.
 */
var twinsourceI_unit_api = (function ($) {
    var oid = get_oid_from_url(),
    cb = null, //Callback Obj
    pannes_created = false,
    err_div = null,
    check_bowser = true,
    css_obj,
    valtable_obj, report_obj, unms_obj,
    inphases = null,
    outphases = null,
    x_pos = 5,
    y_pos = 5,
    frameWidth = 650,
    frameHeight = 850,
    xAlignPosition = x_pos + 50,
    headerYPosition = y_pos + 100,
    nameHeaderYPosition = headerYPosition + 50,
    infoHeaderYPosition = nameHeaderYPosition + 50,
    diagramCanvasYPosition = infoHeaderYPosition + 100,
    diagramCanvasXPosition = xAlignPosition + 100,
    diagramCanvasWidth = 300,
    diagramCanvasHeigth = 150,
    groupInfoLinkButtonYPosition = diagramCanvasYPosition + diagramCanvasHeigth + 50,
    ledTableYPosition = groupInfoLinkButtonYPosition + 50,
    timer, now;

    var req_params = null;

    const LOAD_ON_SOURCE_1 = 'Load On Source 1';
    const LOAD_ON_SOURCE_2 = 'Load On Source 2';
    const SOURCE_1_PREFERRED = 'Source 1 Preferred';
    const SOURCE_2_PREFERRED = 'Source 2 Preferred';
    const SOURCE_1_VOLTAGE_IN_MPQ_ZONE = 'Source 1 Voltage in MPQ Zone';
    const SOURCE_2_VOLTAGE_IN_MPQ_ZONE = 'Source 2 Voltage in MPQ Zone';
    const SOURCE_1_VOLTAGE_IN_UPQ_ZONE = 'Source 1 Voltage in UPQ Zone';
    const SOURCE_2_VOLTAGE_IN_UPQ_ZONE = 'Source 2 Voltage in UPQ Zone';
    const SOURCE_1_FREQUENCY_OUT_OF_RANGE = 'Source 1 Frequency out of range';
    const SOURCE_2_FREQUENCY_OUT_OF_RANGE = 'Source 2 Frequency out of range';
    const SOURCE_1_DC_SUPPLY_FAILURE = 'Source 1 DC Supply Failure';
    const SOURCE_2_DC_SUPPLY_FAILURE = 'Source 2 DC Supply Failure';
    const LOAD_ON_ALTERNATE = 'Load On Alternate';
    const SOURCES_OUT_OF_SYNC = 'Source 1 & Source 2 out of Sync';
    const TIME_WAS_CHANGED = 'Time was changed';
    const OFF_DEFAULT_PREFERRED = 'Off Default Preferred';
    const SETUP_CONFIGURATION_BAD = 'Setup Configuration Bad';
    const SYSTEM_OVER_CURRENT = 'System Over Current';
    const SYSTEM_OVER_LOAD = 'System Over Load';
    const SUMMARY_ALARM = 'Summary Alarm';
    const tableEntries = [ LOAD_ON_SOURCE_1,
                           LOAD_ON_SOURCE_2,
                           SOURCE_1_PREFERRED,
                           SOURCE_2_PREFERRED,
                           SOURCE_1_VOLTAGE_IN_MPQ_ZONE,
                           SOURCE_2_VOLTAGE_IN_MPQ_ZONE,
                           SOURCE_1_VOLTAGE_IN_UPQ_ZONE,
                           SOURCE_2_VOLTAGE_IN_UPQ_ZONE,
                           SOURCE_1_FREQUENCY_OUT_OF_RANGE,
                           SOURCE_2_FREQUENCY_OUT_OF_RANGE,
                           SOURCE_1_DC_SUPPLY_FAILURE,
                           SOURCE_2_DC_SUPPLY_FAILURE,
                           '',
                           '',
                           LOAD_ON_ALTERNATE,
                           SOURCES_OUT_OF_SYNC,
                           '',
                           TIME_WAS_CHANGED,
                           OFF_DEFAULT_PREFERRED,
                           SETUP_CONFIGURATION_BAD,
                           SYSTEM_OVER_CURRENT,
                           SYSTEM_OVER_LOAD,
                           SUMMARY_ALARM,
                         ];
    const NUM_OF_COLUMNS = 2;

    /*public functions*/
    return {
      init: init
    };

    function init() {
      if (check_bowser && !utils_api.check_bowser()) {
        $('#loading_id').css('display', 'none');
        panes_api.create_br_check_panel(this, browser_check_ok);
        return true;
      }
      create_err_msg_panel();

      req_params = utils_api.get_req_params(utils_api.device_ups, oid);
      cb = jqmodule.subscribe(this, { update: update, error: err_funct }, req_params);
    }

    function browser_check_ok() {
      check_bowser = false;
      init();
    }

    function create_err_msg_panel() {
      css_obj = { position: 'absolute', top: y_pos, left: x_pos, width: 791, display: 'none', 'z-index': 22 };
      err_div = panes_api.create_err_panel({ id: 'error_div', 'class': 'alert alert-danger' }, css_obj);
    }

    function err_funct(response) {
      jqmodule.refresh(cb, 10 * 1000);
      panes_api.display_err({ response: response, oid: oid, err_div: err_div, err_msg_div_id: 'error_msg_div' });
    }

    function update(data) {
      if (data.ups == null) {
        return;
      }

      set_variable(data);
      jqmodule.refresh(cb, report_obj.refresh * 1000);
      if (err_div)
        err_div.css('display', 'none');

      if (window.menu_api != null)
        menu_api.update(data);

      unms_obj = get_unms_obj(data);
      if (unms_obj != null)
      {
        info_head_api.update(unms_obj);
      }

      if (!pannes_created)
      {
        pannes_created = true;
        create_panes();
        return false;
      }
      setLEDs();
      //drawDiagramOnCanvas();
      setDiagramLEDs();
      setMetrics();
      setTime();
      set_footer();
      //set .....
    }

    function set_variable(data) {
      valtable_obj = data.ups.valtable;
      report_obj = data.report;

      valtable_obj._tempUnit = valtable_obj._tempUnit || 0; //0 = Celsius

      if (valtable_obj.INPHASES != null)
      inphases = valtable_obj.INPHASES;
      else
      inphases = valtable_obj.PHASES;

      if (valtable_obj.OUTPHASES != null)
      outphases = valtable_obj.OUTPHASES;
      else
      outphases = valtable_obj.PHASES;
    }

    /*
     * Update html elements
     */
    function setLEDs() {
        let unitID = getUnitID();
        for(let ledNumber = 0; ledNumber < tableEntries.length; ledNumber++) {
            let infoLED = new STSUnitLEDs(unitID, ledNumber);
            setLoadOnSource(infoLED);
            setPreferredSource(infoLED);
            setVoltageInMPQ(infoLED);
            setVoltageInUPQ(infoLED);
            setFrequencyOutOfRange(infoLED);
            setDCSupplyFailure(infoLED);
            setLoadOnAlternate(infoLED);
            setSourcesOutOfSync(infoLED);
            setTimeWasChanged(infoLED);
            setOffDefaultPreferred(infoLED);
            setSetupConfigurationBad(infoLED);
            setSystemOverCurrent(infoLED);
            setSystemOverLoad(infoLED);
            setSummaryAlarm(infoLED);
        }
    }

    function setLoadOnSource(infoLED) {
        if (infoLED instanceof STSUnitLEDs)
            if(entryIsLoadOnSource1OrSource2(infoLED.ledNumber)) setLEDOfLoadOnSource(infoLED);
        if (infoLED instanceof STSDiagramLEDs)
            if(diagramLEDTypeIsLoadOnSource1OrSource2(infoLED.ledType)) setLEDOfLoadOnSource(infoLED);
    }

    function entryIsLoadOnSource1OrSource2(entryNumber) {
        return tableEntries[entryNumber]===LOAD_ON_SOURCE_1 || tableEntries[entryNumber]===LOAD_ON_SOURCE_2;
    }

    function diagramLEDTypeIsLoadOnSource1OrSource2(ledType) {
        return ledType===DIAGRAM_LED.load1 || ledType===DIAGRAM_LED.load2;
    }

    function setLEDOfLoadOnSource(unitLED) {
        setLEDWithCallback(unitLED, getColorOfLoadOnSourceLED);
    }

    function getColorOfLoadOnSourceLED(unitLED) {
        if(loadOnSource(unitLED)) {
          return 'gruen';
        }
      }

    function loadOnSource(unitLED) {
        let allFlagsBits = valtable_obj.TWINSOURCE_ALLFLAGS[unitLED.id];
        if(ledIsLoadOnSource1(unitLED))
            return (allFlagsBits >> ALL_FLAGS_BIT_POSITION.loadOnSource1) & BIT_MASK;
        return (allFlagsBits >> ALL_FLAGS_BIT_POSITION.loadOnSource2) & BIT_MASK;
    }

    function ledIsLoadOnSource1(unitLED) {
        if (unitLED instanceof STSUnitLEDs)
           return tableEntries[unitLED.ledNumber]===LOAD_ON_SOURCE_1;
        if (unitLED instanceof STSDiagramLEDs)
            return unitLED.ledType===DIAGRAM_LED.load1;
    }

    function setPreferredSource(infoLED) {
        if(entryIsPreferredSource1OrSource2(infoLED.ledNumber)) setLEDOfPreferredSource(infoLED);
    }

    function entryIsPreferredSource1OrSource2(entryNumber) {
        return tableEntries[entryNumber]===SOURCE_1_PREFERRED || tableEntries[entryNumber]===SOURCE_2_PREFERRED;
    }

    function setLEDOfPreferredSource(unitLED) {
        setLEDWithCallback(unitLED, getColorOfPreferredSourceLED);
    }

    function getColorOfPreferredSourceLED(unitLED) {
        if(sourceIsPreferred(unitLED))
          return 'gruen';
      }

    function sourceIsPreferred(unitLED) {
        let allFlagsBits = valtable_obj.TWINSOURCE_ALLFLAGS[unitLED.id];
        if(tableEntries[unitLED.ledNumber]===SOURCE_1_PREFERRED)
            return (allFlagsBits >> ALL_FLAGS_BIT_POSITION.source1preferred) & BIT_MASK;
        return (allFlagsBits >> ALL_FLAGS_BIT_POSITION.source2preferred) & BIT_MASK;
    }

    function setVoltageInMPQ(infoLED) {
        if(entryIsVoltageInMPQForSource1OrSource2(infoLED.ledNumber)) setLEDOfVoltageInMPQ(infoLED);
    }

    function entryIsVoltageInMPQForSource1OrSource2(entryNumber) {
        let logicalExpression = tableEntries[entryNumber]===SOURCE_1_VOLTAGE_IN_MPQ_ZONE
            || tableEntries[entryNumber]===SOURCE_2_VOLTAGE_IN_MPQ_ZONE;
        return logicalExpression;
    }

    function setLEDOfVoltageInMPQ(unitLED) {
        setLEDWithCallback(unitLED, getColorOfVoltageInMPQLED);
    }

    function getColorOfVoltageInMPQLED(unitLED) {
        if(voltageIsInMPQ(unitLED))
          return 'gelb';
      }

    function voltageIsInMPQ(unitLED) {
        let allFlagsBits = valtable_obj.TWINSOURCE_ALLFLAGS[unitLED.id];
        if(tableEntries[unitLED.ledNumber]===SOURCE_1_VOLTAGE_IN_MPQ_ZONE)
            return (allFlagsBits >> ALL_FLAGS_BIT_POSITION.source1VoltageInMPQZone) & BIT_MASK;
        return (allFlagsBits >> ALL_FLAGS_BIT_POSITION.source2VoltageInMPQZone) & BIT_MASK;
    }

    function setVoltageInUPQ(infoLED) {
        if(entryIsVoltageInUPQForSource1OrSource2(infoLED.ledNumber)) setLEDOfVoltageInUPQ(infoLED);
    }

    function entryIsVoltageInUPQForSource1OrSource2(entryNumber) {
        let logicalExpression = tableEntries[entryNumber]===SOURCE_1_VOLTAGE_IN_UPQ_ZONE
            || tableEntries[entryNumber]===SOURCE_2_VOLTAGE_IN_UPQ_ZONE;
        return logicalExpression;
    }

    function setLEDOfVoltageInUPQ(unitLED) {
        setLEDWithCallback(unitLED, getColorOfVoltageInUPQLED);
    }

    function getColorOfVoltageInUPQLED(unitLED) {
        if(voltageIsInUPQ(unitLED))
            return 'rot';
    }

    function voltageIsInUPQ(unitLED) {
        let allFlagsBits = valtable_obj.TWINSOURCE_ALLFLAGS[unitLED.id];
        if(tableEntries[unitLED.ledNumber]===SOURCE_1_VOLTAGE_IN_UPQ_ZONE)
            return (allFlagsBits >> ALL_FLAGS_BIT_POSITION.source1VoltageInUPQZone) & BIT_MASK;
        return (allFlagsBits >> ALL_FLAGS_BIT_POSITION.source2VoltageInUPQZone) & BIT_MASK;
    }

    function setFrequencyOutOfRange(infoLED) {
        if(entryIsFrequencyOutOfRangeForSource1OrSource2(infoLED.ledNumber)) setLEDOfFrequencyOutOfRange(infoLED);
    }

    function entryIsFrequencyOutOfRangeForSource1OrSource2(entryNumber) {
        let logicalExpression = tableEntries[entryNumber]===SOURCE_1_FREQUENCY_OUT_OF_RANGE
            || tableEntries[entryNumber]===SOURCE_2_FREQUENCY_OUT_OF_RANGE;
        return logicalExpression;
    }

    function setLEDOfFrequencyOutOfRange(unitLED) {
        setLEDWithCallback(unitLED, getColorOfFrequencyOutOfRangeLED);
    }

    function getColorOfFrequencyOutOfRangeLED(unitLED) {
        if(frequencyIsOutOfRange(unitLED))
          return 'rot';
    }

    function frequencyIsOutOfRange(unitLED) {
        let allFlagsBits = valtable_obj.TWINSOURCE_ALLFLAGS[unitLED.id];
        if(tableEntries[unitLED.ledNumber]===SOURCE_1_FREQUENCY_OUT_OF_RANGE)
            return (allFlagsBits >> ALL_FLAGS_BIT_POSITION.source1FreqOutOfRange) & BIT_MASK;
        return (allFlagsBits >> ALL_FLAGS_BIT_POSITION.source2FreqOutOfRange) & BIT_MASK;
    }

    function setDCSupplyFailure(infoLED) {
        if(entryIsDCSupplyFailureForSource1OrSource2(infoLED.ledNumber)) setLEDOfDCSupplyFailure(infoLED);
    }

    function entryIsDCSupplyFailureForSource1OrSource2(entryNumber) {
        let logicalExpression =  tableEntries[entryNumber]===SOURCE_1_DC_SUPPLY_FAILURE
            || tableEntries[entryNumber]===SOURCE_2_DC_SUPPLY_FAILURE;
        return logicalExpression;
    }

    function setLEDOfDCSupplyFailure(unitLED) {
        setLEDWithCallback(unitLED, getColorOfDCSupplyFailureLED);
    }

    function getColorOfDCSupplyFailureLED(unitLED) {
        if(dcSupplyHasFailure(unitLED))
          return 'rot';
      }

    function dcSupplyHasFailure(unitLED) {
        let allFlagsBits = valtable_obj.TWINSOURCE_ALLFLAGS[unitLED.id];
        if(tableEntries[unitLED.ledNumber]===SOURCE_1_DC_SUPPLY_FAILURE)
            return (allFlagsBits >> ALL_FLAGS_BIT_POSITION.source1DCSupplyFailure) & BIT_MASK;
        return (allFlagsBits >> ALL_FLAGS_BIT_POSITION.source2DCSupplyFailure) & BIT_MASK;
    }

    function setLoadOnAlternate(infoLED) {
        if(entryIsLoadOnAlternate(infoLED.ledNumber)) setLEDOfLoadOnAlternate(infoLED);
    }

    function entryIsLoadOnAlternate(entryNumber) {
        return tableEntries[entryNumber]===LOAD_ON_ALTERNATE;
    }

    function setLEDOfLoadOnAlternate(unitLED) {
        setLEDWithCallback(unitLED, getColorOfLoadOnAlternate);
    }

    function getColorOfLoadOnAlternate(unitLED) {
        if(loadIsOnAlternate(unitLED))
          return 'rot';
      }

    function loadIsOnAlternate(unitLED) {
        let allFlagsBits = valtable_obj.TWINSOURCE_ALLFLAGS[unitLED.id];
        return (allFlagsBits >> ALL_FLAGS_BIT_POSITION.loadOnAlternate) & BIT_MASK;
    }

    function setSourcesOutOfSync(infoLED) {
        if(entryIsSourcesOutOfSync(infoLED.ledNumber)) setLEDOfSourcesOutOfSync(infoLED);
    }

    function entryIsSourcesOutOfSync(entryNumber) {
        return tableEntries[entryNumber]===SOURCES_OUT_OF_SYNC;
    }

    function setLEDOfSourcesOutOfSync(unitLED) {
        setLEDWithCallback(unitLED, getColorOfSourcesOutOfSync);
    }

    function getColorOfSourcesOutOfSync(unitLED) {
        if(sourcesOutOfSync(unitLED))
          return 'rot';
      }

    function sourcesOutOfSync(unitLED) {
        let allFlagsBits = valtable_obj.TWINSOURCE_ALLFLAGS[unitLED.id];
        return (allFlagsBits >> ALL_FLAGS_BIT_POSITION.source1AndSource2OutOfSync) & BIT_MASK;
    }

    function setTimeWasChanged(infoLED) {
        if(entryIsTimeWasChanged(infoLED.ledNumber)) setLEDOfTimeWasChanged(infoLED);
    }

    function entryIsTimeWasChanged(entryNumber) {
        return tableEntries[entryNumber]===TIME_WAS_CHANGED;
    }

    function setLEDOfTimeWasChanged(unitLED) {
        setLEDWithCallback(unitLED, getColorOfTimeWasChanged);
    }

    function getColorOfTimeWasChanged(unitLED) {
        if(timeWasChanged(unitLED))
          return 'gruen';
    }

    function timeWasChanged(unitLED) {
        let allFlagsBits = valtable_obj.TWINSOURCE_ALLFLAGS[unitLED.id];
        return (allFlagsBits >> ALL_FLAGS_BIT_POSITION.timeWasChanged) & BIT_MASK;
    }

    function setOffDefaultPreferred(infoLED) {
        if(entryIsOffDefaultPreferred(infoLED.ledNumber)) setLEDOfOffDefaultPreferred(infoLED);
    }

    function entryIsOffDefaultPreferred(entryNumber) {
        return tableEntries[entryNumber]===OFF_DEFAULT_PREFERRED;
    }

    function setLEDOfOffDefaultPreferred(unitLED) {
        setLEDWithCallback(unitLED, getColorOfOffDefaultPreferred);
    }

    function getColorOfOffDefaultPreferred(unitLED) {
        if(offDefaultPreferred(unitLED))
          return 'gelb';
    }

    function offDefaultPreferred(unitLED) {
        let allFlagsBits = valtable_obj.TWINSOURCE_ALLFLAGS[unitLED.id];
        return (allFlagsBits >> ALL_FLAGS_BIT_POSITION.offDefaultPreferred) & BIT_MASK;
    }

    function setSetupConfigurationBad(infoLED) {
        if(entryIsSetupConfigurationBad(infoLED.ledNumber)) setLEDOfSetupConfigurationBad(infoLED);
    }

    function entryIsSetupConfigurationBad(entryNumber) {
        return tableEntries[entryNumber]===SETUP_CONFIGURATION_BAD;
    }

    function setLEDOfSetupConfigurationBad(unitLED) {
        setLEDWithCallback(unitLED, getColorOfSetupConfigurationBad);
    }

    function getColorOfSetupConfigurationBad(unitLED) {
        if(setupConfigurationIsBad(unitLED))
          return 'rot';
    }

    function setupConfigurationIsBad(unitLED) {
        let allFlagsBits = valtable_obj.TWINSOURCE_ALLFLAGS[unitLED.id];
        return (allFlagsBits >> ALL_FLAGS_BIT_POSITION.setupConfigurationBad) & BIT_MASK;
    }

    function setSystemOverCurrent(infoLED) {
        if(entryIsSystemOverCurrent(infoLED.ledNumber)) setLEDOfSystemOverCurrent(infoLED);
    }

    function entryIsSystemOverCurrent(entryNumber) {
        return tableEntries[entryNumber]===SYSTEM_OVER_CURRENT;
    }

    function setLEDOfSystemOverCurrent(unitLED) {
        setLEDWithCallback(unitLED, getColorOfSystemOverCurrent);
    }

    function getColorOfSystemOverCurrent(unitLED) {
        if(systemHasOverCurrent(unitLED))
          return 'rot';
    }

    function systemHasOverCurrent(unitLED) {
        let allFlagsBits = valtable_obj.TWINSOURCE_ALLFLAGS[unitLED.id];
        return (allFlagsBits >> ALL_FLAGS_BIT_POSITION.systemOverCurrent) & BIT_MASK;
    }

    function setSystemOverLoad(infoLED) {
        if(entryIsSystemOverLoad(infoLED.ledNumber)) setLEDOfSystemOverLoad(infoLED);
    }

    function entryIsSystemOverLoad(entryNumber) {
        return tableEntries[entryNumber]===SYSTEM_OVER_LOAD;
    }

    function setLEDOfSystemOverLoad(unitLED) {
        setLEDWithCallback(unitLED, getColorOfSystemOverLoad);
    }

    function getColorOfSystemOverLoad(unitLED) {
        if(systemHasOverLoad(unitLED))
          return 'rot';
    }

    function systemHasOverLoad(unitLED) {
        let allFlagsBits = valtable_obj.TWINSOURCE_ALLFLAGS[unitLED.id];
        return (allFlagsBits >> ALL_FLAGS_BIT_POSITION.systemOverLoad) & BIT_MASK;
    }

    function setSummaryAlarm(infoLED) {
        if(entryIsSummaryAlarm(infoLED.ledNumber)) setLEDOfSummaryAlarm(infoLED);
    }

    function entryIsSummaryAlarm(entryNumber) {
        return tableEntries[entryNumber]===SUMMARY_ALARM;
    }

    function setLEDOfSummaryAlarm(unitLED) {
        setLEDWithCallback(unitLED, getColorOfSummaryAlarm);
    }

    function getColorOfSummaryAlarm(unitLED) {
        if(summaryAlarm(unitLED))
          return 'rot';
      }

      function summaryAlarm(unitLED) {
        let summaryAlarmBitFlagMask = (1 << ALL_FLAGS_BIT_POSITION.source1VoltageInMPQZone)
          | (1 << ALL_FLAGS_BIT_POSITION.source1DCSupplyFailure)
          | (1 << ALL_FLAGS_BIT_POSITION.source2VoltageInMPQZone)
          | (1 << ALL_FLAGS_BIT_POSITION.source2DCSupplyFailure)
          | (1 << ALL_FLAGS_BIT_POSITION.source1AndSource2OutOfSync)
          | (1 << ALL_FLAGS_BIT_POSITION.loadOnAlternate)
          | (1 << ALL_FLAGS_BIT_POSITION.systemOverLoad)
          | (1 << ALL_FLAGS_BIT_POSITION.systemOverCurrent)
          | (1 << ALL_FLAGS_BIT_POSITION.setupConfigurationBad)
          | (1 << ALL_FLAGS_BIT_POSITION.offDefaultPreferred)
          | (1 << ALL_FLAGS_BIT_POSITION.source1VoltageInUPQZone)
          | (1 << ALL_FLAGS_BIT_POSITION.source1FreqOutOfRange)
          | (1 << ALL_FLAGS_BIT_POSITION.source2VoltageInUPQZone)
          | (1 << ALL_FLAGS_BIT_POSITION.source2FreqOutOfRange);
        return (valtable_obj.TWINSOURCE_ALLFLAGS[unitLED.id]) & summaryAlarmBitFlagMask;
      }

    function setLEDWithCallback(unitLED, callbackFunction) {
        unitLED.status = LED_ON;
        unitLED.color = callbackFunction(unitLED);
        if (unitLED instanceof STSUnitLEDs)
            setLED(unitLED);
        if (unitLED instanceof STSDiagramLEDs)
            setDiagramLED(unitLED);
    }

    function setLED(unitLED) {
        var ledSourceFile = getLEDSourceFile(unitLED.color, unitLED.status);
        $('#' + getLEDName(unitLED.ledNumber)).attr('src', ledSourceFile);
    }

    function setDiagramLEDs() {
        unitID = getUnitID();
        setSourceStatusDiagramLEDs(unitID);
        setLoadOnSourceDiagramLEDs(unitID);
    }

    function setSourceStatusDiagramLEDs(unitID) {
        source1LED = new STSDiagramLEDs(unitID, DIAGRAM_LED.source1);
        setSourcesStatus(source1LED);
        source2LED = new STSDiagramLEDs(unitID, DIAGRAM_LED.source2);
        setSourcesStatus(source2LED);
    }

    function setLoadOnSourceDiagramLEDs(unitID) {
        load1LED = new STSDiagramLEDs(unitID, DIAGRAM_LED.load1);
        setLoadOnSource(load1LED);
        load2LED = new STSDiagramLEDs(unitID, DIAGRAM_LED.load2);
        setLoadOnSource(load2LED);
    }

    function setSourcesStatus(unitLED) {
        if(typeIsSource1OrSource2Status(unitLED.ledType)) setLEDOfSourceStatus(unitLED);
    }

    function typeIsSource1OrSource2Status(type) {
        return type === DIAGRAM_LED.source1 || type === DIAGRAM_LED.source2;
    }

    function setLEDOfSourceStatus(unitLED) {
        setLEDWithCallback(unitLED, getColorOfSourceStatusLED);
    }

    function getColorOfSourceStatusLED(unitLED) {
        if(sourceIsInDesiredPowerQualityZone(unitLED))
            return 'gruen';
        if(sourceIsInMarginalPowerQualityZone(unitLED))
            return 'gelb';
        if(sourceIsIUnacceptablePowerQualityZone(unitLED))
            return 'rot';
        return 'rot';
    }

    function sourceIsInDesiredPowerQualityZone(unitLED) {
        values = getSourceStatusValues(unitLED);
        return !(values.voltageInUPQZone || values.voltageInMPQZone || values.dcSupplyFailure || values.frequencyOutOfRange);
    }

    function sourceIsInMarginalPowerQualityZone(unitLED) {
        values = getSourceStatusValues(unitLED);
        return values.voltageInMPQZone && !(values.dcSupplyFailure || values.frequencyOutOfRange);
    }

    function sourceIsIUnacceptablePowerQualityZone(unitLED) {
        values = getSourceStatusValues(unitLED);
        return (values.voltageInUPQZone || values.dcSupplyFailure || values.frequencyOutOfRange);
    }

    function getSourceStatusValues(unitLED) {
        let voltageInUPQZone, voltageInMPQZone, dcSupplyFailure, alternateSourcePreferred, frequencyOutOfRange;
        if(isSource1LEDType(unitLED)) {
            voltageInUPQZone =  ((valtable_obj.TWINSOURCE_ALLFLAGS[unitLED.id]) >> ALL_FLAGS_BIT_POSITION.source1VoltageInUPQZone) & BIT_MASK;
            voltageInMPQZone =  ((valtable_obj.TWINSOURCE_ALLFLAGS[unitLED.id]) >> ALL_FLAGS_BIT_POSITION.source1VoltageInMPQZone) & BIT_MASK;
            dcSupplyFailure =  ((valtable_obj.TWINSOURCE_ALLFLAGS[unitLED.id]) >> ALL_FLAGS_BIT_POSITION.source1DCSupplyFailure) & BIT_MASK;
            alternateSourcePreferred =  ((valtable_obj.TWINSOURCE_ALLFLAGS[unitLED.id]) >> ALL_FLAGS_BIT_POSITION.source2preferred) & BIT_MASK;
            frequencyOutOfRange =  ((valtable_obj.TWINSOURCE_ALLFLAGS[unitLED.id]) >> ALL_FLAGS_BIT_POSITION.source1FreqOutOfRange) & BIT_MASK;
        } else {
            voltageInUPQZone =  ((valtable_obj.TWINSOURCE_ALLFLAGS[unitLED.id]) >> ALL_FLAGS_BIT_POSITION.source2VoltageInUPQZone) & BIT_MASK;
            voltageInMPQZone =  ((valtable_obj.TWINSOURCE_ALLFLAGS[unitLED.id]) >> ALL_FLAGS_BIT_POSITION.source2VoltageInMPQZone) & BIT_MASK;
            dcSupplyFailure =  ((valtable_obj.TWINSOURCE_ALLFLAGS[unitLED.id]) >> ALL_FLAGS_BIT_POSITION.source2DCSupplyFailure) & BIT_MASK;
            alternateSourcePreferred =  ((valtable_obj.TWINSOURCE_ALLFLAGS[unitLED.id]) >> ALL_FLAGS_BIT_POSITION.source1preferred) & BIT_MASK;
            frequencyOutOfRange =  ((valtable_obj.TWINSOURCE_ALLFLAGS[unitLED.id]) >> ALL_FLAGS_BIT_POSITION.source2FreqOutOfRange) & BIT_MASK;
        }
        sourceStatusValues = {
            voltageInUPQZone: voltageInUPQZone,
            voltageInMPQZone: voltageInMPQZone,
            dcSupplyFailure: dcSupplyFailure,
            alternateSourcePreferred: alternateSourcePreferred,
            frequencyOutOfRange: frequencyOutOfRange,
        }
        return sourceStatusValues;
    }

    function isSource1LEDType(unitLED) {
        if (unitLED instanceof STSUnitLEDs)
            return columnHeaders[unitLED.ledNumber]===SOURCE_1_STATUS;
        if (unitLED instanceof STSDiagramLEDs)
            return unitLED.ledType === DIAGRAM_LED.source1;
    }

    function setDiagramLED(unitLED) {
        var ledSourceFile = getLEDSourceFile(unitLED.color, unitLED.status);
        $('#' + getDiagramLEDIDName(unitLED.ledType)).attr('src', ledSourceFile);
    }

    function setMetrics() {
        let unitID = getUnitID();
        setSourceLabels();
        setLoadLabel();
        setVoltages(unitID);
        setFrequencies(unitID);
        setOutputCurrent(unitID);
        setRealPower(unitID);
        setApparentPower(unitID);
        setPowerFactor(unitID);
    }

    function setSourceLabels() {
        setLabel(METRICS.source1);
        setLabel(METRICS.source2);
    }

    function setLoadLabel() {
        setLabel(METRICS.load);
    }

    function setLabel(type) {
        getMetric(type).text(type);
    }

    function setVoltages(unitID) {
        setVoltage(METRICS.source1Voltage, unitID);
        setVoltage(METRICS.source2Voltage, unitID);
    }

    function setVoltage(type, unitID) {
        let sourceVoltage = getSourceVoltage(type, unitID);
        getMetric(type).text(sourceVoltage);
    }

    function getSourceVoltage(type, unitID) {
        if (METRICS.source1Voltage === type)
            return inUnit(valtable_obj.TWINSOURCE_SOURCE1_VAC[unitID], 'V');
        return inUnit(valtable_obj.TWINSOURCE_SOURCE2_VAC[unitID], 'V');
    }

    function setFrequencies(unitID) {
        setFrequency(METRICS.source1Frequency, unitID);
        setFrequency(METRICS.source2Frequency, unitID);
    }

    function setFrequency(type, unitID) {
        let sourceFrequency = getSourceFrequency(type, unitID);
        getMetric(type).text(sourceFrequency);
    }

    function getSourceFrequency(type, unitID) {
        if (METRICS.source1Frequency === type)
            return inUnit(valtable_obj.TWINSOURCE_SOURCE1_FREQ[unitID], 'Hz');
        return inUnit(valtable_obj.TWINSOURCE_SOURCE2_FREQ[unitID], 'Hz');
    }

    function setRealPower(unitID) {
        setPowerInWatts(METRICS.powerInWatts, unitID);
    }

    function setPowerInWatts(type, unitID) {
        let powerInWatts = getPowerInWatts(unitID);
        getMetric(type).text(powerInWatts);
    }

    function getPowerInWatts(unitID) {
        return inUnit(valtable_obj.TWINSOURCE_POWER_IN_WATTS[unitID], 'W');
    }

    function setOutputCurrent(unitID) {
        setLoadInAmpere(METRICS.loadInAmpere, unitID);
    }

    function setLoadInAmpere(type, unitID) {
        let loadInAmpere = getLoadInAmpere(unitID);
        getMetric(type).text(loadInAmpere);
    }

    function getLoadInAmpere(unitID) {
        return inUnit(valtable_obj.TWINSOURCE_LOAD_IN_AMPERE[unitID], 'A');
    }

    function setApparentPower(unitID) {
        setVoltAmpere(METRICS.loadVoltAmpere, unitID);
    }

    function setVoltAmpere(type, unitID) {
        let voltAmpere = getVoltAmpere(unitID);
        getMetric(type).text(voltAmpere);
    }

    function getVoltAmpere(unitID) {
        return inUnit(valtable_obj.TWINSOURCE_VOLTAMPERE[unitID], 'VA');
    }

    function setPowerFactor(unitID) {
        let powerFactor = getPowerFactor(unitID);
        getMetric(METRICS.powerFactor).text(powerFactor);
    }

    function getPowerFactor(unitID) {
        return inUnit(valtable_obj.TWINSOURCE_POWER_FACTOR[unitID], 'PF');
    }

    function setLabel(type) {
        getMetric(type).text(type);
    }

    function getMetric(type) {
        return $('#' + getMetricName(type));
    }

    function inUnit(value, unit) {
        return value + unit;
    }

    function set_footer() {
      if (!oid || !unms_obj)
        return true;
      footer_api.set(unms_obj);
    }

    function create_panes() {
      utils_api.setTitle(valtable_obj._platform);
      createFrame();
      createModelTitleBar();
      //create_sts_led();
      create_footer_pane();
      createDiagram();
      createDiagramLEDs();
      createMetrics();
      createUnitDetailsPage();
      setLEDs();
      setDiagramLEDs();
      setMetrics();
      setTime();
    }

    /*
     * Create html elements
     */
    function createFrame() {
      let fragment = document.createDocumentFragment(),
        img_div;
      css_obj = { position: 'absolute', top: y_pos, left: x_pos,
                  width: frameWidth, height: frameHeight, border: 'solid 1px #808080' };
      img_div = $('<div/>', { 'class': 'img-rounded' }).css(css_obj);
      fragment.appendChild(img_div[0]);
      document.body.appendChild(fragment);
    }

    function createModelTitleBar() {
      let model_div, a;
      css_obj = { position: 'absolute', top: y_pos, left: x_pos, align: 'center', width: frameWidth, 'z-index': 40 };
      attr_obj = { id: 'model_div_id', 'class': 'list-group' };
      model_div = panes_api.create_div(attr_obj, css_obj);
      a = $('<a/>', { 'class': 'list-group-item active', id: 'model_name_id' }).appendTo(model_div);
      setModelName();
    }

    function setModelName() {
      var model_name = valtable_obj.MODEL;
      $('#model_name_id').html(model_name);
    }

    function createDiagram() {
        fragment = document.createDocumentFragment();
        let canvas = createCanvas();
        fragment.appendChild(canvas[0]);
        document.body.appendChild(fragment);
        drawDiagramOnCanvas();
    }

    function createCanvas() {
        css_obj = { position: 'absolute', top: diagramCanvasYPosition, left: diagramCanvasXPosition };
        return $('<canvas />', { id: 'stsDiagramCanvas', width: diagramCanvasWidth, height: diagramCanvasHeigth }).css(css_obj);
    }

    function drawDiagramOnCanvas() {
        window.onload = init;
        var canvas = document.getElementById('stsDiagramCanvas');
        var context = canvas.getContext('2d');
        context.beginPath();
        context.moveTo(SelectA.x,SelectA.y);
        context.lineTo(BendA.x,BendA.y);
        context.lineTo(Intersection.x,Intersection.y);
        context.moveTo(SelectB.x,SelectB.y);
        context.lineTo(BendB.x,BendB.y);
        context.lineTo(Intersection.x,Intersection.y);
        context.lineTo(Load.x,Load.y);
        context.lineWidth = 3;
        context.stroke();

    }

    function createDiagramLEDs() {
        fragment = document.createDocumentFragment();
        let source1LEDPosition = new Point(diagramCanvasXPosition + SelectA.x - drawMarginHalf, diagramCanvasYPosition + SelectA.y - yDiagramOffset);
        let source1LED = createDiagramLED(DIAGRAM_LED.source1, source1LEDPosition);
        let source2LEDPosition = new Point(diagramCanvasXPosition + SelectB.x - drawMarginHalf, diagramCanvasYPosition + SelectB.y - yDiagramOffset);
        let source2LED = createDiagramLED(DIAGRAM_LED.source2, source2LEDPosition);
        let load1LEDPosition = new Point(diagramCanvasXPosition + LoadA.x - drawMarginHalf, diagramCanvasYPosition + LoadA.y - yDiagramOffset);
        let load1LED = createDiagramLED(DIAGRAM_LED.load1, load1LEDPosition);
        let load2LEDPosition = new Point(diagramCanvasXPosition + LoadB.x - drawMarginHalf, diagramCanvasYPosition + LoadB.y - yDiagramOffset);
        let load2LED = createDiagramLED(DIAGRAM_LED.load2, load2LEDPosition);
        fragment.appendChild(source1LED[0]);
        fragment.appendChild(source2LED[0]);
        fragment.appendChild(load1LED[0]);
        fragment.appendChild(load2LED[0]);
        document.body.appendChild(fragment);
    }

    function createDiagramLED(type, ledPosition) {
        let div;
        css_obj = { position: 'absolute', top: ledPosition.y, left: ledPosition.x };
        div = $('<div/>').css(css_obj);
        let led = getLEDSourceFile('grau', LED_OFF);
        let ledImage = $('<img/>', { id: getDiagramLEDIDName(type), 'src': led });
        div.append(ledImage);
        return div;
    }

    function createMetrics() {
        fragment = document.createDocumentFragment();
        let source1LabelPosition = new Point(diagramCanvasXPosition - 2*drawMargin, diagramCanvasYPosition + SelectA.y - yDiagramOffset);
        let source1Label = createMetricLabel(METRICS.source1, source1LabelPosition);
        let source2LabelPosition = new Point(diagramCanvasXPosition - 2*drawMargin, diagramCanvasYPosition + SelectB.y - yDiagramOffset);
        let source2Label = createMetricLabel(METRICS.source2, source2LabelPosition);
        let loadLabelPosition = new Point(diagramCanvasXPosition + Load.x + drawMargin, diagramCanvasYPosition + Load.y - yDiagramOffset);
        let loadLabel = createMetricLabel(METRICS.load, loadLabelPosition);
        let source1VoltagePosition = new Point(diagramCanvasXPosition + drawMargin, diagramCanvasYPosition - drawMargin);
        let source1Voltage = createMetricLabel(METRICS.source1Voltage, source1VoltagePosition);
        let source2VoltagePosition = new Point(diagramCanvasXPosition + drawMargin, diagramCanvasYPosition + diagramCanvasHeigth);
        let source2Voltage = createMetricLabel(METRICS.source2Voltage, source2VoltagePosition);
        let source1FrequencyPosition = new Point(diagramCanvasXPosition + 5*drawMargin, diagramCanvasYPosition - drawMargin);
        let source1Frequency = createMetricLabel(METRICS.source1Frequency, source1FrequencyPosition);
        let source2FrequencyPosition = new Point(diagramCanvasXPosition + 5*drawMargin, diagramCanvasYPosition + diagramCanvasHeigth);
        let source2Frequency = createMetricLabel(METRICS.source2Frequency, source2FrequencyPosition);
        let loadOutputCurrentPosition = new Point(loadLabelPosition.x + 7*drawMargin, loadLabelPosition.y - 3*drawMargin);
        let loadOutputCurrent = createMetricLabel(METRICS.loadInAmpere, loadOutputCurrentPosition);
        let loadRealPowerPosition = new Point(loadLabelPosition.x + 7*drawMargin, loadLabelPosition.y - drawMargin);
        let loadRealPower = createMetricLabel(METRICS.powerInWatts, loadRealPowerPosition);
        let loadApparentPowerPosition = new Point(loadLabelPosition.x + 7*drawMargin, loadLabelPosition.y + drawMargin);
        let loadApparentPower = createMetricLabel(METRICS.loadVoltAmpere, loadApparentPowerPosition);
        let loadPowerFactorPosition = new Point(loadLabelPosition.x + 7*drawMargin, loadLabelPosition.y + 3*drawMargin);
        let loadPowerFactor = createMetricLabel(METRICS.powerFactor, loadPowerFactorPosition);
        fragment.appendChild(source1Label[0]);
        fragment.appendChild(source2Label[0]);
        fragment.appendChild(loadLabel[0]);
        fragment.appendChild(source1Voltage[0]);
        fragment.appendChild(source2Voltage[0]);
        fragment.appendChild(source1Frequency[0]);
        fragment.appendChild(source2Frequency[0]);
        fragment.appendChild(loadOutputCurrent[0]);
        fragment.appendChild(loadRealPower[0]);
        fragment.appendChild(loadApparentPower[0]);
        fragment.appendChild(loadPowerFactor[0]);
        document.body.appendChild(fragment);
        drawDiagramOnCanvas();
    }

    function createMetricLabel(type, metricPosition) {
        let div;
        css_obj = { position: 'absolute', top: metricPosition.y, left: metricPosition.x };
        div = $('<div/>').css(css_obj);
        let metric = $('<span/>', { id: getMetricName(type) });
        div.append(metric);
        return div;
    }

    function createUnitDetailsPage() {
        fragment = document.createDocumentFragment();
        let unitHeader = createHeader();
        let unitHeaderName = createHeaderName();
        let infoHeader = createInfoHeader();
        let groupInfoButton = createGroupInfoLinkButton();
        let unitTimeOutput = createUnitTimeOutput();
        let ledTable = createLEDTable();
        fragment.appendChild(unitHeader[0]);
        fragment.appendChild(unitHeaderName[0]);
        fragment.appendChild(infoHeader[0]);
        fragment.appendChild(groupInfoButton[0]);
        fragment.appendChild(unitTimeOutput[0]);
        fragment.appendChild(ledTable[0]);
        document.body.appendChild(fragment);
    }

    function createHeader() {
        css_obj = { position: 'absolute', top: headerYPosition, left: xAlignPosition };
        return $('<h1/>').css(css_obj).append(`ATS Unit Status`);
    }

    function createHeaderName() {
        css_obj = { position: 'absolute', top: nameHeaderYPosition, left: xAlignPosition };
        let id = getUnitID();
        return $('<h1/>').css(css_obj).append(`Name: ` + getUnitName(valtable_obj, id));
    }

    function createInfoHeader() {
        css_obj = { position: 'absolute', top: infoHeaderYPosition, left: xAlignPosition };
        let id = getLabelID();
        return $('<h2/>').css(css_obj).append(`Unit ID:\t${id}`);
    }

    function createGroupInfoLinkButton() {
        css_obj = { position: 'absolute', top: groupInfoLinkButtonYPosition, left: xAlignPosition };
        let button = $('<button />').css(css_obj).append('Group Info');
        return button.on("click",function () {
            window.location.href='twinsourcedcci.html';
            return false;
        });
    }

    function createUnitTimeOutput() {
        css_obj = { position: 'absolute', top: groupInfoLinkButtonYPosition, left:  + 300 };
        unitTime = getUnitTime();
        let timeOutput = $('<div />', { id: 'time'}).css(css_obj).append(unitTime);
        return timeOutput;
    }

    function getUnitTime() {
        now = getTimeFromSTS();
        timer = setInterval(updateTime, 1000);
        return now.toLocaleString('en-US');
    }

    function updateTime() {
        now.setSeconds( now.getSeconds() + 1);
        $('#time').text(now.toLocaleString('en-US'));
    }

    function setTime() {
        clearInterval(timer);
        getUnitTime();
    }

    function getTimeFromSTS() {
        let id = getUnitID();
        let year = valtable_obj.TWINSOURCE_YEAR_FROM_2000[id] + 2000;
        let month = valtable_obj.TWINSOURCE_MONTH[id];
        let day = valtable_obj.TWINSOURCE_DAY_OF_MONTH[id];
        let hours = valtable_obj.TWINSOURCE_HOURS[id];
        let minutes = valtable_obj.TWINSOURCE_MINUTES[id];
        let seconds = valtable_obj.TWINSOURCE_SECONDS[id];
        return new Date(year, month, day, hours, minutes, seconds);
    }

    function createLEDTable() {
        var table, tbody, div;
        css_obj = { position: 'absolute', top: ledTableYPosition, left: xAlignPosition, 'text-align': 'left' };
        div = $('<div/>').css(css_obj);
        table = $('<table />', { 'class': 'table table-condensed table-hover' }).appendTo(div);
        tbody = $('<tbody />').appendTo(table);
        createRowsinTable(tbody);
        return div;
    }

    function createRowsinTable(tableBody) {
        let row;
        for (let entry = 0; entry < tableEntries.length; entry++) {
            row =  createNewRow(entry, row, tableBody);
            let tableLED = createTableLED(entry);
            let tableEntry = createTableEntry(entry);
            tableLED.appendTo(row);
            tableEntry.appendTo(row);
        }
    }

    function createNewRow(entry, row, tableBody) {
        if (entry % NUM_OF_COLUMNS === 0) {
            return $('<tr />').appendTo(tableBody);
        }
        return row;
    }

    function createTableLED(entry) {
        let led = getLEDSourceFile('grau', LED_OFF);
        let ledImage = $('<img/>', { id: getLEDName(entry), 'src': led })
        let tableLED = $('<td/>', { 'style' : 'width:25px' });
        if(!entryIsEmpty(entry)){
            tableLED.append(ledImage);
        }
        return tableLED;
    }

    function createTableEntry(entry) {
        let tableEntry = $('<td/>', { 'style' : 'width:250px' });
        if(!entryIsEmpty(entry)){
            tableEntry.append(tableEntries[entry]);
        }
        return tableEntry;
    }

    function entryIsEmpty(entry) {
        return ''===tableEntries[entry];
    }

    function getLabelID() {
        return (new URL(location.href)).searchParams.get('id');
    }

    function getUnitID() {
        return getLabelID() - 1;
    }

    function getLEDName(ledID) {
        return 'led_' + ledID;
    }

    function getMetricName(type) {
        return 'metric_' + type;
    }

    function create_footer_pane() {
      if (!oid || !unms_obj)
          return true;
      footer_api.create({ //Attr
          id: 'panel2'
          //'class':'unmsFooter'
      }, { //css
           position: 'absolute',
          top: '490px',
          left: '74px'
      },
         unms_obj
      );
    }

}(jQuery));