/*
 * Specific definitions for TwinSource models DCC-II and DCC-III detailed unit screen.
 */
var twinsourceI_unit_api = (function ($) {
    var oid = get_oid_from_url(),
    cb = null, //Callback Obj
    pannes_created = false,
    err_div = null,
    check_bowser = true,
    css_obj,
    valtable_obj, report_obj, unms_obj,
    inphases = null,
    outphases = null,
    x_pos = 5,
    y_pos = 5,
    frameWidth = 650,
    frameHeight = 1000,
    xAlignPosition = x_pos + 50,
    headerYPosition = y_pos + 100,
    nameHeaderYPosition = headerYPosition + 50,
    infoHeaderYPosition = nameHeaderYPosition + 50,
    diagramCanvasYPosition = infoHeaderYPosition + 100,
    diagramCanvasXPosition = xAlignPosition + 100,
    diagramCanvasWidth = 300,
    diagramCanvasHeigth = 150,
    groupInfoLinkButtonYPosition = diagramCanvasYPosition + diagramCanvasHeigth + 50,
    ledTableYPosition = groupInfoLinkButtonYPosition + 50;

    var req_params = null;

    const LOAD_ON_SOURCE_1 = 'Load On Source 1';
    const LOAD_ON_SOURCE_2 = 'Load On Source 2';
    const SOURCE_1_PREFERRED = 'Source 1 Preferred';
    const SOURCE_2_PREFERRED = 'Source 2 Preferred';
    const SOURCE_1_VOLTAGE_IN_MPQ_ZONE = 'Source 1 Voltage in MPQ Zone';
    const SOURCE_2_VOLTAGE_IN_MPQ_ZONE = 'Source 2 Voltage in MPQ Zone';
    const SOURCE_1_VOLTAGE_IN_UPQ_ZONE = 'Source 1 Voltage in UPQ Zone';
    const SOURCE_2_VOLTAGE_IN_UPQ_ZONE = 'Source 2 Voltage in UPQ Zone';
    const SOURCE_1_FREQUENCY_OUT_OF_RANGE = 'Source 1 Frequency out of range';
    const SOURCE_2_FREQUENCY_OUT_OF_RANGE = 'Source 2 Frequency out of range';
    const SOURCE_1_OPEN_SCR = 'Source 1 Open SCR';
    const SOURCE_2_OPEN_SCR = 'Source 2 Open SCR';
    const SOURCE_1_SHORTED_SCR = 'Source 1 Shorted SCR';
    const SOURCE_2_SHORTED_SCR = 'Source 2 Shorted SCR';
    const SOURCE_1_LOGIC_POWER_SUPPLY_FAILURE = 'Source 1 Logic Power Supply Failure';
    const SOURCE_2_LOGIC_POWER_SUPPLY_FAILURE = 'Source 2 Logic Power Supply Failure';
    const SOURCE_1_GATE_DRIVER_POWER_SUPPLY_FAILURE = 'Source 1 Gate Driver Power Supply Failure';
    const SOURCE_2_GATE_DRIVER_POWER_SUPPLY_FAILURE = 'Source 2 Gate Driver Power Supply Failure';
    const LOAD_ON_ALTERNATE = 'Load On Alternate';
    const SOURCES_OUT_OF_SYNC = 'Source 1 & Source 2 out of Sync';
    const FAN_FAILURE = 'Fan Failure';
    const SYSTEM_OVERLOADED = 'System Overloaded';
    const SELECTOR_SWITCH_OFF_NORMAL = 'Selector Switch off Normal';
    const SYSTEM_OUTPUT_FAULT_CURRENT = 'System Output Fault Current';
    const SELECTOR_SWITCH_DO_NOT_BYPASS = "Selector Switch Don't Bypass";
    const SYSTEM_OVER_TEMPERATURE = 'System Over Temperature (internal)';
    const SUMMARY_ALARM = 'Summary Alarm';
    const SERVICE_EM = 'Service EM (Electronic Module)';
    const tableEntries = [ LOAD_ON_SOURCE_1,
                           LOAD_ON_SOURCE_2,
                           SOURCE_1_PREFERRED,
                           SOURCE_2_PREFERRED,
                           SOURCE_1_VOLTAGE_IN_MPQ_ZONE,
                           SOURCE_2_VOLTAGE_IN_MPQ_ZONE,
                           SOURCE_1_VOLTAGE_IN_UPQ_ZONE,
                           SOURCE_2_VOLTAGE_IN_UPQ_ZONE,
                           SOURCE_1_FREQUENCY_OUT_OF_RANGE,
                           SOURCE_2_FREQUENCY_OUT_OF_RANGE,
                           SOURCE_1_OPEN_SCR,
                           SOURCE_2_OPEN_SCR,
                           SOURCE_1_SHORTED_SCR,
                           SOURCE_2_SHORTED_SCR,
                           SOURCE_1_LOGIC_POWER_SUPPLY_FAILURE,
                           SOURCE_2_LOGIC_POWER_SUPPLY_FAILURE,
                           SOURCE_1_GATE_DRIVER_POWER_SUPPLY_FAILURE,
                           SOURCE_2_GATE_DRIVER_POWER_SUPPLY_FAILURE,
                           '',
                           '',
                           LOAD_ON_ALTERNATE,
                           '',
                           SOURCES_OUT_OF_SYNC,
                           FAN_FAILURE,
                           SYSTEM_OVERLOADED,
                           SELECTOR_SWITCH_OFF_NORMAL,
                           SYSTEM_OUTPUT_FAULT_CURRENT,
                           SELECTOR_SWITCH_DO_NOT_BYPASS,
                           SYSTEM_OVER_TEMPERATURE,
                           '',
                           '',
                           '',
                           SUMMARY_ALARM,
                           SERVICE_EM,
                         ];
    const NUM_OF_COLUMNS = 2;

    /*public functions*/
    return {
      init: init
    };

    function init() {
      if (check_bowser && !utils_api.check_bowser()) {
        $('#loading_id').css('display', 'none');
        panes_api.create_br_check_panel(this, browser_check_ok);
        return true;
      }
      create_err_msg_panel();

      req_params = utils_api.get_req_params(utils_api.device_ups, oid);
      cb = jqmodule.subscribe(this, { update: update, error: err_funct }, req_params);
    }

    function browser_check_ok() {
      check_bowser = false;
      init();
    }

    function create_err_msg_panel() {
      css_obj = { position: 'absolute', top: y_pos, left: x_pos, width: 791, display: 'none', 'z-index': 22 };
      err_div = panes_api.create_err_panel({ id: 'error_div', 'class': 'alert alert-danger' }, css_obj);
    }

    function err_funct(response) {
      jqmodule.refresh(cb, 10 * 1000);
      panes_api.display_err({ response: response, oid: oid, err_div: err_div, err_msg_div_id: 'error_msg_div' });
    }

    function update(data) {
      if (data.ups == null) {
        return;
      }

      set_variable(data);
      jqmodule.refresh(cb, report_obj.refresh * 1000);
      if (err_div)
        err_div.css('display', 'none');

      if (window.menu_api != null)
        menu_api.update(data);

      unms_obj = get_unms_obj(data);
      if (unms_obj != null)
      {
        info_head_api.update(unms_obj);
      }

      if (!pannes_created)
      {
        pannes_created = true;
        create_panes();
        return false;
      }
      setLEDs();
      setDiagramLEDs();
      setMetrics();
      set_footer();
      //set .....
    }

    function set_variable(data) {
      valtable_obj = data.ups.valtable;
      report_obj = data.report;

      valtable_obj._tempUnit = valtable_obj._tempUnit || 0; //0 = Celsius

      if (valtable_obj.INPHASES != null)
      inphases = valtable_obj.INPHASES;
      else
      inphases = valtable_obj.PHASES;

      if (valtable_obj.OUTPHASES != null)
      outphases = valtable_obj.OUTPHASES;
      else
      outphases = valtable_obj.PHASES;
    }

    /*
     * Update html elements
     */
    function setLEDs() {
        let unitID = getUnitID();
        for(let ledNumber = 0; ledNumber < tableEntries.length; ledNumber++) {
            let infoLED = new STSUnitLEDs(unitID, ledNumber);
            setLoadOnSource(infoLED);
            setPreferredSource(infoLED);
            setVoltageInMPQ(infoLED);
            setVoltageInUPQ(infoLED);
            setFrequencyOutOfRange(infoLED);
            setOpenSCR(infoLED);
            setShortedSCR(infoLED);
            setLogicPowerSupplyFailure(infoLED);
            setGateDriverPowerSupplyFailure(infoLED);
            setLoadOnAlternate(infoLED);
            setSourcesOutOfSync(infoLED);
            setFanFailure(infoLED);
            setSystemOverloaded(infoLED);
            setSelectorSwitchOffNormal(infoLED);
            setSystemOutputFaultCurrent(infoLED);
            setSelecotrSwitchDoNotBypass(infoLED);
            setSystemOverTemperature(infoLED);
            setSummaryAlarm(infoLED);
            setServiceEM(infoLED);
        }
    }

    function setLoadOnSource(infoLED) {
        if (infoLED instanceof STSUnitLEDs)
            if(entryIsLoadOnSource1OrSource2(infoLED.ledNumber)) setLEDOfLoadOnSource(infoLED);
        if (infoLED instanceof STSDiagramLEDs)
            if(diagramLEDTypeIsLoadOnSource1OrSource2(infoLED.ledType)) setLEDOfLoadOnSource(infoLED);
    }

    function entryIsLoadOnSource1OrSource2(entryNumber) {
        return tableEntries[entryNumber]===LOAD_ON_SOURCE_1 || tableEntries[entryNumber]===LOAD_ON_SOURCE_2;
    }

    function diagramLEDTypeIsLoadOnSource1OrSource2(ledType) {
        return ledType===DIAGRAM_LED.load1 || ledType===DIAGRAM_LED.load2;
    }

    function setLEDOfLoadOnSource(unitLED) {
        setLEDWithCallback(unitLED, getColorOfLoadOnSourceLED);
    }

    function getColorOfLoadOnSourceLED(unitLED) {
        if(loadOnSource(unitLED)) {
          return 'gruen';
        }
      }

    function loadOnSource(unitLED) {
        let alarmStatusFlagsBits = valtable_obj.TWINSOURCE_ALARMSTATUSFLAGS[unitLED.id];
        if(ledIsLoadOnSource1(unitLED))
            return (alarmStatusFlagsBits >> ALARM_STATUS_FLAGS_BIT_POSITION.loadOnSource1) & BIT_MASK;
        return (alarmStatusFlagsBits >> ALARM_STATUS_FLAGS_BIT_POSITION.loadOnSource2) & BIT_MASK;
    }

    function ledIsLoadOnSource1(unitLED) {
        if (unitLED instanceof STSUnitLEDs)
           return tableEntries[unitLED.ledNumber]===LOAD_ON_SOURCE_1;
        if (unitLED instanceof STSDiagramLEDs)
            return unitLED.ledType===DIAGRAM_LED.load1;
    }

    function setPreferredSource(infoLED) {
        if(entryIsPreferredSource1OrSource2(infoLED.ledNumber)) setLEDOfPreferredSource(infoLED);
    }

    function entryIsPreferredSource1OrSource2(entryNumber) {
        return tableEntries[entryNumber]===SOURCE_1_PREFERRED || tableEntries[entryNumber]===SOURCE_2_PREFERRED;
    }

    function setLEDOfPreferredSource(unitLED) {
        setLEDWithCallback(unitLED, getColorOfPreferredSourceLED);
    }

    function getColorOfPreferredSourceLED(unitLED) {
        if(sourceIsPreferred(unitLED))
          return 'gruen';
      }

    function sourceIsPreferred(unitLED) {
        let alarmStatusFlagsBits = valtable_obj.TWINSOURCE_ALARMSTATUSFLAGS[unitLED.id];
        if(tableEntries[unitLED.ledNumber]===SOURCE_1_PREFERRED)
            return (alarmStatusFlagsBits >> ALARM_STATUS_FLAGS_BIT_POSITION.source1Preferred) & BIT_MASK;
        return (alarmStatusFlagsBits >> ALARM_STATUS_FLAGS_BIT_POSITION.source2Preferred) & BIT_MASK;
    }

    function setVoltageInMPQ(infoLED) {
        if(entryIsVoltageInMPQForSource1OrSource2(infoLED.ledNumber)) setLEDOfVoltageInMPQ(infoLED);
    }

    function entryIsVoltageInMPQForSource1OrSource2(entryNumber) {
        let logicalExpression = tableEntries[entryNumber]===SOURCE_1_VOLTAGE_IN_MPQ_ZONE
            || tableEntries[entryNumber]===SOURCE_2_VOLTAGE_IN_MPQ_ZONE;
        return logicalExpression;
    }

    function setLEDOfVoltageInMPQ(unitLED) {
        setLEDWithCallback(unitLED, getColorOfVoltageInMPQLED);
    }

    function getColorOfVoltageInMPQLED(unitLED) {
        if(voltageIsInMPQ(unitLED))
          return 'gelb';
      }

    function voltageIsInMPQ(unitLED) {
        let alarmStatusFlagsBits = valtable_obj.TWINSOURCE_ALARMSTATUSFLAGS[unitLED.id];
        if(tableEntries[unitLED.ledNumber]===SOURCE_1_VOLTAGE_IN_MPQ_ZONE)
            return (alarmStatusFlagsBits >> ALARM_STATUS_FLAGS_BIT_POSITION.source1VoltageInMPQZone) & BIT_MASK;
        return (alarmStatusFlagsBits >> ALARM_STATUS_FLAGS_BIT_POSITION.source2VoltageInMPQZone) & BIT_MASK;
    }

    function setVoltageInUPQ(infoLED) {
        if(entryIsVoltageInUPQForSource1OrSource2(infoLED.ledNumber)) setLEDOfVoltageInUPQ(infoLED);
    }

    function entryIsVoltageInUPQForSource1OrSource2(entryNumber) {
        let logicalExpression = tableEntries[entryNumber]===SOURCE_1_VOLTAGE_IN_UPQ_ZONE
            || tableEntries[entryNumber]===SOURCE_2_VOLTAGE_IN_UPQ_ZONE;
        return logicalExpression;
    }

    function setLEDOfVoltageInUPQ(unitLED) {
        setLEDWithCallback(unitLED, getColorOfVoltageInUPQLED);
    }

    function getColorOfVoltageInUPQLED(unitLED) {
        if(voltageIsInUPQ(unitLED))
            return 'rot';
    }

    function voltageIsInUPQ(unitLED) {
        let alarmStatusFlagsBits = valtable_obj.TWINSOURCE_ALARMSTATUSFLAGS[unitLED.id];
        if(tableEntries[unitLED.ledNumber]===SOURCE_1_VOLTAGE_IN_UPQ_ZONE)
            return (alarmStatusFlagsBits >> ALARM_STATUS_FLAGS_BIT_POSITION.source1VoltageInUPQZone) & BIT_MASK;
        return (alarmStatusFlagsBits >> ALARM_STATUS_FLAGS_BIT_POSITION.source2VoltageInUPQZone) & BIT_MASK;
    }

    function setFrequencyOutOfRange(infoLED) {
        if(entryIsFrequencyOutOfRangeForSource1OrSource2(infoLED.ledNumber)) setLEDOfFrequencyOutOfRange(infoLED);
    }

    function entryIsFrequencyOutOfRangeForSource1OrSource2(entryNumber) {
        let logicalExpression = tableEntries[entryNumber]===SOURCE_1_FREQUENCY_OUT_OF_RANGE
            || tableEntries[entryNumber]===SOURCE_2_FREQUENCY_OUT_OF_RANGE;
        return logicalExpression;
    }

    function setLEDOfFrequencyOutOfRange(unitLED) {
        setLEDWithCallback(unitLED, getColorOfFrequencyOutOfRangeLED);
    }

    function getColorOfFrequencyOutOfRangeLED(unitLED) {
        if(frequencyIsOutOfRange(unitLED))
          return 'rot';
    }

    function frequencyIsOutOfRange(unitLED) {
        let alarmStatusFlagsBits = valtable_obj.TWINSOURCE_ALARMSTATUSFLAGS[unitLED.id];
        if(tableEntries[unitLED.ledNumber]===SOURCE_1_FREQUENCY_OUT_OF_RANGE)
            return (alarmStatusFlagsBits >> ALARM_STATUS_FLAGS_BIT_POSITION.source1FreqOutOfRange) & BIT_MASK;
        return (alarmStatusFlagsBits >> ALARM_STATUS_FLAGS_BIT_POSITION.source2FreqOutOfRange) & BIT_MASK;
    }

    function setOpenSCR(infoLED) {
        if(entryIsOpenSCRForSource1OrSource2(infoLED.ledNumber)) setLEDOfOpenSCR(infoLED);
    }

    function entryIsOpenSCRForSource1OrSource2(entryNumber) {
        let logicalExpression =  tableEntries[entryNumber]===SOURCE_1_OPEN_SCR
            || tableEntries[entryNumber]===SOURCE_2_OPEN_SCR;
        return logicalExpression;
    }

    function setLEDOfOpenSCR(unitLED) {
        setLEDWithCallback(unitLED, getColorOfOpenSCRLED);
    }

    function getColorOfOpenSCRLED(unitLED) {
        if(openSCRFailure(unitLED))
          return 'rot';
      }

    function openSCRFailure(unitLED) {
        let alarmStatusFlagsBits = valtable_obj.TWINSOURCE_ALARMSTATUSFLAGS[unitLED.id];
        if(tableEntries[unitLED.ledNumber]===SOURCE_1_OPEN_SCR)
            return (alarmStatusFlagsBits >> ALARM_STATUS_FLAGS_BIT_POSITION.source1OpenSCR) & BIT_MASK;
        return (alarmStatusFlagsBits >> ALARM_STATUS_FLAGS_BIT_POSITION.source2OpenSCR) & BIT_MASK;
    }

    function setShortedSCR(infoLED) {
        if(entryIsShortedSCRForSource1OrSource2(infoLED.ledNumber)) setLEDOfShortedSCR(infoLED);
    }

    function entryIsShortedSCRForSource1OrSource2(entryNumber) {
        let logicalExpression =  tableEntries[entryNumber]===SOURCE_1_SHORTED_SCR
            || tableEntries[entryNumber]===SOURCE_2_SHORTED_SCR;
        return logicalExpression;
    }

    function setLEDOfShortedSCR(unitLED) {
        setLEDWithCallback(unitLED, getColorOfShortedSCRLED);
    }

    function getColorOfShortedSCRLED(unitLED) {
        if(shortedSCRFailure(unitLED))
          return 'rot';
      }

    function shortedSCRFailure(unitLED) {
        let alarmStatusFlagsBits = valtable_obj.TWINSOURCE_ALARMSTATUSFLAGS[unitLED.id];
        if(tableEntries[unitLED.ledNumber]===SOURCE_1_SHORTED_SCR)
            return (alarmStatusFlagsBits >> ALARM_STATUS_FLAGS_BIT_POSITION.source1ShortedSCR) & BIT_MASK;
        return (alarmStatusFlagsBits >> ALARM_STATUS_FLAGS_BIT_POSITION.source2ShortedSCR) & BIT_MASK;
    }

    function setLogicPowerSupplyFailure(infoLED) {
        if(entryIsLogicPowerSupplyFailureForSource1OrSource2(infoLED.ledNumber)) setLEDOfLogicPowerSupplyFailure(infoLED);
    }

    function entryIsLogicPowerSupplyFailureForSource1OrSource2(entryNumber) {
        let logicalExpression =  tableEntries[entryNumber]===SOURCE_1_LOGIC_POWER_SUPPLY_FAILURE
            || tableEntries[entryNumber]===SOURCE_2_LOGIC_POWER_SUPPLY_FAILURE;
        return logicalExpression;
    }

    function setLEDOfLogicPowerSupplyFailure(unitLED) {
        setLEDWithCallback(unitLED, getColorOfLogicPowerSupplyFailureLED);
    }

    function getColorOfLogicPowerSupplyFailureLED(unitLED) {
        if(logicPowerSupplyHasFailure(unitLED))
          return 'rot';
      }

    function logicPowerSupplyHasFailure(unitLED) {
        let alarmStatusFlagsBits = valtable_obj.TWINSOURCE_ALARMSTATUSFLAGS[unitLED.id];
        if(tableEntries[unitLED.ledNumber]===SOURCE_1_LOGIC_POWER_SUPPLY_FAILURE)
            return (alarmStatusFlagsBits >> ALARM_STATUS_FLAGS_BIT_POSITION.source1LogicPowerSupplyFailure) & BIT_MASK;
        return (alarmStatusFlagsBits >> ALARM_STATUS_FLAGS_BIT_POSITION.source2LogicPowerSupplyFailure) & BIT_MASK;
    }

    function setGateDriverPowerSupplyFailure(infoLED) {
        if(entryIsGateDriverPowerSupplyFailureForSource1OrSource2(infoLED.ledNumber)) setLEDOfGateDriverPowerSupplyFailure(infoLED);
    }

    function entryIsGateDriverPowerSupplyFailureForSource1OrSource2(entryNumber) {
        let logicalExpression =  tableEntries[entryNumber]===SOURCE_1_GATE_DRIVER_POWER_SUPPLY_FAILURE
            || tableEntries[entryNumber]===SOURCE_2_GATE_DRIVER_POWER_SUPPLY_FAILURE;
        return logicalExpression;
    }

    function setLEDOfGateDriverPowerSupplyFailure(unitLED) {
        setLEDWithCallback(unitLED, getColorOfGateDriverPowerSupplyFailureLED);
    }

    function getColorOfGateDriverPowerSupplyFailureLED(unitLED) {
        if(gateDriverPowerSupplyHasFailure(unitLED))
          return 'rot';
      }

    function gateDriverPowerSupplyHasFailure(unitLED) {
        let alarmStatusFlagsBits = valtable_obj.TWINSOURCE_ALARMSTATUSFLAGS[unitLED.id];
        if(tableEntries[unitLED.ledNumber]===SOURCE_1_GATE_DRIVER_POWER_SUPPLY_FAILURE)
            return (alarmStatusFlagsBits >> ALARM_STATUS_FLAGS_BIT_POSITION.source1GateDriverPowerSupplyFailure) & BIT_MASK;
        return (alarmStatusFlagsBits >> ALARM_STATUS_FLAGS_BIT_POSITION.source2GateDriverPowerSupplyFailure) & BIT_MASK;
    }

    function setLoadOnAlternate(infoLED) {
        if(entryIsLoadOnAlternate(infoLED.ledNumber)) setLEDOfLoadOnAlternate(infoLED);
    }

    function entryIsLoadOnAlternate(entryNumber) {
        return tableEntries[entryNumber]===LOAD_ON_ALTERNATE;
    }

    function setLEDOfLoadOnAlternate(unitLED) {
        setLEDWithCallback(unitLED, getColorOfLoadOnAlternate);
    }

    function getColorOfLoadOnAlternate(unitLED) {
        if(loadIsOnAlternate(unitLED))
          return 'gelb';
      }

    function loadIsOnAlternate(unitLED) {
        let alarmStatusFlagsBits = valtable_obj.TWINSOURCE_ALARMSTATUSFLAGS[unitLED.id];
        return (alarmStatusFlagsBits >> ALARM_STATUS_FLAGS_BIT_POSITION.loadOnAlternate) & BIT_MASK;
    }

    function setSourcesOutOfSync(infoLED) {
        if(entryIsSourcesOutOfSync(infoLED.ledNumber)) setLEDOfSourcesOutOfSync(infoLED);
    }

    function entryIsSourcesOutOfSync(entryNumber) {
        return tableEntries[entryNumber]===SOURCES_OUT_OF_SYNC;
    }

    function setLEDOfSourcesOutOfSync(unitLED) {
        setLEDWithCallback(unitLED, getColorOfSourcesOutOfSync);
    }

    function getColorOfSourcesOutOfSync(unitLED) {
        if(sourcesOutOfSync(unitLED))
          return 'rot';
      }

    function sourcesOutOfSync(unitLED) {
        let alarmStatusFlagsBits = valtable_obj.TWINSOURCE_ALARMSTATUSFLAGS[unitLED.id];
        return (alarmStatusFlagsBits >> ALARM_STATUS_FLAGS_BIT_POSITION.source1AndSource2OutOfSync) & BIT_MASK;
    }

    function setFanFailure(infoLED) {
        if(entryIsFanFailure(infoLED.ledNumber)) setLEDOfFanFailure(infoLED);
    }

    function entryIsFanFailure(entryNumber) {
        return tableEntries[entryNumber]===FAN_FAILURE;
    }

    function setLEDOfFanFailure(unitLED) {
        setLEDWithCallback(unitLED, getColorOfFanFailure);
    }

    function getColorOfFanFailure(unitLED) {
        if(fanFailure(unitLED))
          return 'rot';
    }

    function fanFailure(unitLED) {
        let alarmStatusFlagsBits = valtable_obj.TWINSOURCE_ALARMSTATUSFLAGS[unitLED.id];
        return (alarmStatusFlagsBits >> ALARM_STATUS_FLAGS_BIT_POSITION.fanFailure) & BIT_MASK;
    }

    function setSelectorSwitchOffNormal(infoLED) {
        if(entryIsSelectorSwitchOffNormal(infoLED.ledNumber)) setLEDOfSelectorSwitchOffNormal(infoLED);
    }

    function entryIsSelectorSwitchOffNormal(entryNumber) {
        return tableEntries[entryNumber]===SELECTOR_SWITCH_OFF_NORMAL;
    }

    function setLEDOfSelectorSwitchOffNormal(unitLED) {
        setLEDWithCallback(unitLED, getColorOfSelectorSwitchOffNormal);
    }

    function getColorOfSelectorSwitchOffNormal(unitLED) {
        if(selectorSwitchOffNormal(unitLED))
          return 'gelb';
    }

    function selectorSwitchOffNormal(unitLED) {
        let alarmStatusFlagsBits = valtable_obj.TWINSOURCE_ALARMSTATUSFLAGS[unitLED.id];
        return (alarmStatusFlagsBits >> ALARM_STATUS_FLAGS_BIT_POSITION.selectorSwitchOffNormal) & BIT_MASK;
    }

    function setSystemOutputFaultCurrent(infoLED) {
        if(entryIsSystemOutputFaultCurrent(infoLED.ledNumber)) setLEDOfSystemOutputFaultCurrent(infoLED);
    }

    function entryIsSystemOutputFaultCurrent(entryNumber) {
        return tableEntries[entryNumber]===SYSTEM_OUTPUT_FAULT_CURRENT;
    }

    function setLEDOfSystemOutputFaultCurrent(unitLED) {
        setLEDWithCallback(unitLED, getColorOfSystemOutputFaultCurrent);
    }

    function getColorOfSystemOutputFaultCurrent(unitLED) {
        if(systemOutputFaultCurrent(unitLED))
          return 'rot';
    }

    function systemOutputFaultCurrent(unitLED) {
        let alarmStatusFlagsBits = valtable_obj.TWINSOURCE_ALARMSTATUSFLAGS[unitLED.id];
        return (alarmStatusFlagsBits >> ALARM_STATUS_FLAGS_BIT_POSITION.systemOutputFaultCurrent) & BIT_MASK;
    }

    function setSelecotrSwitchDoNotBypass(infoLED) {
        if(entryIsSelecotrSwitchDoNotBypass(infoLED.ledNumber)) setLEDOfSelecotrSwitchDoNotBypass(infoLED);
    }

    function entryIsSelecotrSwitchDoNotBypass(entryNumber) {
        return tableEntries[entryNumber]===SELECTOR_SWITCH_DO_NOT_BYPASS;
    }

    function setLEDOfSelecotrSwitchDoNotBypass(unitLED) {
        setLEDWithCallback(unitLED, getColorOfSelecotrSwitchDoNotBypass);
    }

    function getColorOfSelecotrSwitchDoNotBypass(unitLED) {
        if(selecotrSwitchDoNotBypass(unitLED))
          return 'rot';
    }

    function selecotrSwitchDoNotBypass(unitLED) {
        let alarmStatusFlagsBits = valtable_obj.TWINSOURCE_ALARMSTATUSFLAGS[unitLED.id];
        return (alarmStatusFlagsBits >> ALARM_STATUS_FLAGS_BIT_POSITION.selectorSwitchDoNotBypass) & BIT_MASK;
    }

    function setSystemOverTemperature(infoLED) {
        if(entryIsSystemOverTemperature(infoLED.ledNumber)) setLEDOfSystemOverTemperature(infoLED);
    }

    function entryIsSystemOverTemperature(entryNumber) {
        return tableEntries[entryNumber]===SYSTEM_OVER_TEMPERATURE;
    }

    function setLEDOfSystemOverTemperature(unitLED) {
        setLEDWithCallback(unitLED, getColorOfSystemOverTemperature);
    }

    function getColorOfSystemOverTemperature(unitLED) {
        if(systemHasOverTemperature(unitLED))
          return 'rot';
    }

    function systemHasOverTemperature(unitLED) {
        let alarmStatusFlagsBits = valtable_obj.TWINSOURCE_ALARMSTATUSFLAGS[unitLED.id];
        return (alarmStatusFlagsBits >> ALARM_STATUS_FLAGS_BIT_POSITION.systemOverTemperature) & BIT_MASK;
    }

    function setSystemOverloaded(infoLED) {
        if(entryIsSystemOverloaded(infoLED.ledNumber)) setLEDOfSystemOverloaded(infoLED);
    }

    function entryIsSystemOverloaded(entryNumber) {
        return tableEntries[entryNumber]===SYSTEM_OVERLOADED;
    }

    function setLEDOfSystemOverloaded(unitLED) {
        setLEDWithCallback(unitLED, getColorOfSystemOverloaded);
    }

    function getColorOfSystemOverloaded(unitLED) {
        if(systemHasOverloaded(unitLED))
          return 'rot';
    }

    function systemHasOverloaded(unitLED) {
        let alarmStatusFlagsBits = valtable_obj.TWINSOURCE_ALARMSTATUSFLAGS[unitLED.id];
        return (alarmStatusFlagsBits >> ALARM_STATUS_FLAGS_BIT_POSITION.systemOverloaded) & BIT_MASK;
    }

    function setSummaryAlarm(infoLED) {
        if(entryIsSummaryAlarm(infoLED.ledNumber)) setLEDOfSummaryAlarm(infoLED);
    }

    function entryIsSummaryAlarm(entryNumber) {
        return tableEntries[entryNumber]===SUMMARY_ALARM;
    }

    function setLEDOfSummaryAlarm(unitLED) {
        setLEDWithCallback(unitLED, getColorOfSummaryAlarm);
    }

    function getColorOfSummaryAlarm(unitLED) {
        if(summaryAlarm(unitLED))
          return 'rot';
      }

    function summaryAlarm(unitLED) {
        let alarmStatusFlagsBits = valtable_obj.TWINSOURCE_ALARMSTATUSFLAGS[unitLED.id];
        return (alarmStatusFlagsBits >> ALARM_STATUS_FLAGS_BIT_POSITION.summaryAlarm) & BIT_MASK;
    }

    function setServiceEM(infoLED) {
        if(entryIsServiceEM(infoLED.ledNumber)) setLEDOfServiceEM(infoLED);
    }

    function entryIsServiceEM(entryNumber) {
        return tableEntries[entryNumber]===SERVICE_EM;
    }

    function setLEDOfServiceEM(unitLED) {
        setLEDWithCallback(unitLED, getColorOfServiceEM);
    }

    function getColorOfServiceEM(unitLED) {
        if(serviceEM(unitLED))
          return 'rot';
      }

    function serviceEM(unitLED) {
        let alarmStatusFlagsBits = valtable_obj.TWINSOURCE_ALARMSTATUSFLAGS[unitLED.id];
        return (alarmStatusFlagsBits >> ALARM_STATUS_FLAGS_BIT_POSITION.serviceEM) & BIT_MASK;
    }

    function setLEDWithCallback(unitLED, callbackFunction) {
        unitLED.status = LED_ON;
        unitLED.color = callbackFunction(unitLED);
        if (unitLED instanceof STSUnitLEDs)
            setLED(unitLED);
        if (unitLED instanceof STSDiagramLEDs)
            setDiagramLED(unitLED);
    }

    function setLED(unitLED) {
        var ledSourceFile = getLEDSourceFile(unitLED.color, unitLED.status);
        $('#' + getLEDName(unitLED.ledNumber)).attr('src', ledSourceFile);
    }

    function setDiagramLEDs() {
        unitID = getUnitID();
        setSourceStatusDiagramLEDs(unitID);
        setLoadOnSourceDiagramLEDs(unitID);
    }

    function setSourceStatusDiagramLEDs(unitID) {
        source1LED = new STSDiagramLEDs(unitID, DIAGRAM_LED.source1);
        setSourcesStatus(source1LED);
        source2LED = new STSDiagramLEDs(unitID, DIAGRAM_LED.source2);
        setSourcesStatus(source2LED);
    }

    function setLoadOnSourceDiagramLEDs(unitID) {
        load1LED = new STSDiagramLEDs(unitID, DIAGRAM_LED.load1);
        setLoadOnSource(load1LED);
        load2LED = new STSDiagramLEDs(unitID, DIAGRAM_LED.load2);
        setLoadOnSource(load2LED);
    }

    function setSourcesStatus(unitLED) {
        if(typeIsSource1OrSource2Status(unitLED.ledType)) setLEDOfSourceStatus(unitLED);
    }

    function typeIsSource1OrSource2Status(type) {
        return type === DIAGRAM_LED.source1 || type === DIAGRAM_LED.source2;
    }

    function setLEDOfSourceStatus(unitLED) {
        setLEDWithCallback(unitLED, getColorOfSourceStatusLED);
    }

    function getColorOfSourceStatusLED(unitLED) {
        if(sourceIsInDesiredPowerQualityZone(unitLED))
            return 'gruen';
        if(sourceIsInMarginalPowerQualityZone(unitLED))
            return 'gelb';
        if(sourceIsIUnacceptablePowerQualityZone(unitLED))
            return 'rot';
        return 'rot';
    }

    function sourceIsInDesiredPowerQualityZone(unitLED) {
        values = getSourceStatusValues(unitLED);
        return !(values.voltageInMPQZone || values.voltageInUPQZone || values.frequencyOutOfRange
            || values.openSCR || values.logicPowerSupplyFailure || values.gateDriverPowerSupplyFailure
            || values.oppositeSourceShortedSCR);
    }

    function sourceIsInMarginalPowerQualityZone(unitLED) {
        values = getSourceStatusValues(unitLED);
        return values.voltageInMPQZone && !(values.voltageInUPQZone || values.frequencyOutOfRange
            || values.openSCR || values.logicPowerSupplyFailure || values.gateDriverPowerSupplyFailure
            || values.oppositeSourceShortedSCR);
    }

    function sourceIsIUnacceptablePowerQualityZone(unitLED) {
        values = getSourceStatusValues(unitLED);
        return (values.voltageInUPQZone || values.frequencyOutOfRange || values.openSCR
            || values.logicPowerSupplyFailure || values.gateDriverPowerSupplyFailure
            || values.oppositeSourceShortedSCR);
    }

    function getSourceStatusValues(unitLED) {
        let voltageInMPQZone, voltageInUPQZone, frequencyOutOfRange, openSCR, logicPowerSupplyFailure, gateDriverPowerSupplyFailure, oppositeSourceShortedSCR;
        let alarmStatusFlagsBits = (valtable_obj.TWINSOURCE_ALARMSTATUSFLAGS[unitLED.id]);
        if(isSource1LEDType(unitLED)) {
            voltageInMPQZone =  (alarmStatusFlagsBits >> ALARM_STATUS_FLAGS_BIT_POSITION.source1VoltageInMPQZone) & BIT_MASK;
            voltageInUPQZone =  (alarmStatusFlagsBits >> ALARM_STATUS_FLAGS_BIT_POSITION.source1VoltageInUPQZone) & BIT_MASK;
            frequencyOutOfRange =  (alarmStatusFlagsBits >> ALARM_STATUS_FLAGS_BIT_POSITION.source1FreqOutOfRange) & BIT_MASK;
            openSCR =  (alarmStatusFlagsBits >> ALARM_STATUS_FLAGS_BIT_POSITION.source1OpenSCR) & BIT_MASK;
            logicPowerSupplyFailure =  (alarmStatusFlagsBits >> ALARM_STATUS_FLAGS_BIT_POSITION.source1LogicPowerSupplyFailure) & BIT_MASK;
            gateDriverPowerSupplyFailure =  (alarmStatusFlagsBits >> ALARM_STATUS_FLAGS_BIT_POSITION.source1GateDriverPowerSupplyFailure) & BIT_MASK;
            oppositeSourceShortedSCR =  (alarmStatusFlagsBits >> ALARM_STATUS_FLAGS_BIT_POSITION.source2ShortedSCR) & BIT_MASK;
        } else {
            voltageInMPQZone =  (alarmStatusFlagsBits >> ALARM_STATUS_FLAGS_BIT_POSITION.source2VoltageInMPQZone) & BIT_MASK;
            voltageInUPQZone =  (alarmStatusFlagsBits >> ALARM_STATUS_FLAGS_BIT_POSITION.source2VoltageInUPQZone) & BIT_MASK;
            frequencyOutOfRange =  (alarmStatusFlagsBits >> ALARM_STATUS_FLAGS_BIT_POSITION.source2FreqOutOfRange) & BIT_MASK;
            openSCR =  (alarmStatusFlagsBits >> ALARM_STATUS_FLAGS_BIT_POSITION.source2OpenSCR) & BIT_MASK;
            logicPowerSupplyFailure =  (alarmStatusFlagsBits >> ALARM_STATUS_FLAGS_BIT_POSITION.source2LogicPowerSupplyFailure) & BIT_MASK;
            gateDriverPowerSupplyFailure =  (alarmStatusFlagsBits >> ALARM_STATUS_FLAGS_BIT_POSITION.source2GateDriverPowerSupplyFailure) & BIT_MASK;
            oppositeSourceShortedSCR =  (alarmStatusFlagsBits >> ALARM_STATUS_FLAGS_BIT_POSITION.source1ShortedSCR) & BIT_MASK;
        }
        sourceStatusValues = {
            voltageInMPQZone: voltageInMPQZone,
            voltageInUPQZone: voltageInUPQZone,
            frequencyOutOfRange: frequencyOutOfRange,
            openSCR: openSCR,
            logicPowerSupplyFailure: logicPowerSupplyFailure,
            gateDriverPowerSupplyFailure: gateDriverPowerSupplyFailure,
            oppositeSourceShortedSCR: oppositeSourceShortedSCR,
        }
        return sourceStatusValues;
    }

    function isSource1LEDType(unitLED) {
        if (unitLED instanceof STSUnitLEDs)
            return columnHeaders[unitLED.ledNumber]===SOURCE_1_STATUS;
        if (unitLED instanceof STSDiagramLEDs)
            return unitLED.ledType === DIAGRAM_LED.source1;
    }

    function setDiagramLED(unitLED) {
        var ledSourceFile = getLEDSourceFile(unitLED.color, unitLED.status);
        $('#' + getDiagramLEDIDName(unitLED.ledType)).attr('src', ledSourceFile);
    }

    function setMetrics() {
        setSourceLabels();
        setLoadLabel();
    }

    function setSourceLabels() {
        setLabel(METRICS.source1);
        setLabel(METRICS.source2);
    }

    function setLoadLabel() {
        setLabel(METRICS.load);
    }

    function setLabel(type) {
        getMetric(type).text(type);
    }

    function setLED(unitLED) {
        var ledSourceFile = getLEDSourceFile(unitLED.color, unitLED.status);
        $('#' + getLEDName(unitLED.ledNumber)).attr('src', ledSourceFile);
    }

    function setLabel(type) {
        getMetric(type).text(type);
    }

    function getMetric(type) {
        return $('#' + getMetricName(type));
    }

    function inUnit(value, unit) {
        return value + unit;
    }

    function set_footer() {
      if (!oid || !unms_obj)
        return true;
      footer_api.set(unms_obj);
    }

    /*
     * Create html elements
     */
    function create_panes() {
      utils_api.setTitle(valtable_obj._platform);
      createFrame();
      createModelTitleBar();
      create_footer_pane();
      createDiagram();
      createDiagramLEDs();
      createMetrics();
      createUnitDetailsPage();
      setLEDs();
      setDiagramLEDs();
      setMetrics();
    }

    function createFrame() {
      let fragment = document.createDocumentFragment(),
        img_div;
      css_obj = { position: 'absolute', top: y_pos, left: x_pos,
                  width: frameWidth, height: frameHeight, border: 'solid 1px #808080' };
      img_div = $('<div/>', { 'class': 'img-rounded' }).css(css_obj);
      fragment.appendChild(img_div[0]);
      document.body.appendChild(fragment);
    }

    function createModelTitleBar() {
      let model_div, a;
      css_obj = { position: 'absolute', top: y_pos, left: x_pos, align: 'center', width: frameWidth, 'z-index': 40 };
      attr_obj = { id: 'model_div_id', 'class': 'list-group' };
      model_div = panes_api.create_div(attr_obj, css_obj);
      a = $('<a/>', { 'class': 'list-group-item active', id: 'model_name_id' }).appendTo(model_div);
      setModelName();
    }

    function setModelName() {
      var model_name = valtable_obj.MODEL;
      $('#model_name_id').html(model_name);
    }

    function createDiagram() {
        fragment = document.createDocumentFragment();
        let canvas = createCanvas();
        fragment.appendChild(canvas[0]);
        document.body.appendChild(fragment);
        drawDiagramOnCanvas();
    }

    function createCanvas() {
        css_obj = { position: 'absolute', top: diagramCanvasYPosition, left: diagramCanvasXPosition };
        return $('<canvas />', { id: 'stsDiagramCanvas', width: diagramCanvasWidth, height: diagramCanvasHeigth }).css(css_obj);
    }

    function drawDiagramOnCanvas() {
        window.onload = init;
        var canvas = document.getElementById('stsDiagramCanvas');
        var context = canvas.getContext('2d');
        context.beginPath();
        context.moveTo(SelectA.x,SelectA.y);
        context.lineTo(BendA.x,BendA.y);
        context.lineTo(Intersection.x,Intersection.y);
        context.moveTo(SelectB.x,SelectB.y);
        context.lineTo(BendB.x,BendB.y);
        context.lineTo(Intersection.x,Intersection.y);
        context.lineTo(Load.x,Load.y);
        context.lineWidth = 3;
        context.stroke();

    }

    function createDiagramLEDs() {
        fragment = document.createDocumentFragment();
        let source1LEDPosition = new Point(diagramCanvasXPosition + SelectA.x - drawMarginHalf, diagramCanvasYPosition + SelectA.y - yDiagramOffset);
        let source1LED = createDiagramLED(DIAGRAM_LED.source1, source1LEDPosition);
        let source2LEDPosition = new Point(diagramCanvasXPosition + SelectB.x - drawMarginHalf, diagramCanvasYPosition + SelectB.y - yDiagramOffset);
        let source2LED = createDiagramLED(DIAGRAM_LED.source2, source2LEDPosition);
        let load1LEDPosition = new Point(diagramCanvasXPosition + LoadA.x - drawMarginHalf, diagramCanvasYPosition + LoadA.y - yDiagramOffset);
        let load1LED = createDiagramLED(DIAGRAM_LED.load1, load1LEDPosition);
        let load2LEDPosition = new Point(diagramCanvasXPosition + LoadB.x - drawMarginHalf, diagramCanvasYPosition + LoadB.y - yDiagramOffset);
        let load2LED = createDiagramLED(DIAGRAM_LED.load2, load2LEDPosition);
        fragment.appendChild(source1LED[0]);
        fragment.appendChild(source2LED[0]);
        fragment.appendChild(load1LED[0]);
        fragment.appendChild(load2LED[0]);
        document.body.appendChild(fragment);
    }

    function createDiagramLED(type, ledPosition) {
        let div;
        css_obj = { position: 'absolute', top: ledPosition.y, left: ledPosition.x };
        div = $('<div/>').css(css_obj);
        let led = getLEDSourceFile('grau', LED_OFF);
        let ledImage = $('<img/>', { id: getDiagramLEDIDName(type), 'src': led });
        div.append(ledImage);
        return div;
    }

    function createMetrics() {
        fragment = document.createDocumentFragment();
        let source1LabelPosition = new Point(diagramCanvasXPosition - 2*drawMargin, diagramCanvasYPosition + SelectA.y - yDiagramOffset);
        let source1Label = createMetricLabel(METRICS.source1, source1LabelPosition);
        let source2LabelPosition = new Point(diagramCanvasXPosition - 2*drawMargin, diagramCanvasYPosition + SelectB.y - yDiagramOffset);
        let source2Label = createMetricLabel(METRICS.source2, source2LabelPosition);
        let loadLabelPosition = new Point(diagramCanvasXPosition + Load.x + drawMargin, diagramCanvasYPosition + Load.y - yDiagramOffset);
        let loadLabel = createMetricLabel(METRICS.load, loadLabelPosition);
        let source1VoltagePosition = new Point(diagramCanvasXPosition + drawMargin, diagramCanvasYPosition - drawMargin);
        let source1Voltage = createMetricLabel(METRICS.source1Voltage, source1VoltagePosition);
        let source2VoltagePosition = new Point(diagramCanvasXPosition + drawMargin, diagramCanvasYPosition + diagramCanvasHeigth);
        let source2Voltage = createMetricLabel(METRICS.source2Voltage, source2VoltagePosition);
        let source1FrequencyPosition = new Point(diagramCanvasXPosition + 5*drawMargin, diagramCanvasYPosition - drawMargin);
        let source1Frequency = createMetricLabel(METRICS.source1Frequency, source1FrequencyPosition);
        let source2FrequencyPosition = new Point(diagramCanvasXPosition + 5*drawMargin, diagramCanvasYPosition + diagramCanvasHeigth);
        let source2Frequency = createMetricLabel(METRICS.source2Frequency, source2FrequencyPosition);
        let loadOutputCurrentPosition = new Point(loadLabelPosition.x + 7*drawMargin, loadLabelPosition.y - 3*drawMargin);
        let loadOutputCurrent = createMetricLabel(METRICS.loadInAmpere, loadOutputCurrentPosition);
        let loadRealPowerPosition = new Point(loadLabelPosition.x + 7*drawMargin, loadLabelPosition.y - drawMargin);
        let loadRealPower = createMetricLabel(METRICS.powerInWatts, loadRealPowerPosition);
        let loadApparentPowerPosition = new Point(loadLabelPosition.x + 7*drawMargin, loadLabelPosition.y + drawMargin);
        let loadApparentPower = createMetricLabel(METRICS.loadVoltAmpere, loadApparentPowerPosition);
        let loadPowerFactorPosition = new Point(loadLabelPosition.x + 7*drawMargin, loadLabelPosition.y + 3*drawMargin);
        let loadPowerFactor = createMetricLabel(METRICS.powerFactor, loadPowerFactorPosition);
        fragment.appendChild(source1Label[0]);
        fragment.appendChild(source2Label[0]);
        fragment.appendChild(loadLabel[0]);
        fragment.appendChild(source1Voltage[0]);
        fragment.appendChild(source2Voltage[0]);
        fragment.appendChild(source1Frequency[0]);
        fragment.appendChild(source2Frequency[0]);
        fragment.appendChild(loadOutputCurrent[0]);
        fragment.appendChild(loadRealPower[0]);
        fragment.appendChild(loadApparentPower[0]);
        fragment.appendChild(loadPowerFactor[0]);
        document.body.appendChild(fragment);
        drawDiagramOnCanvas();
    }

    function createMetricLabel(type, metricPosition) {
        let div;
        css_obj = { position: 'absolute', top: metricPosition.y, left: metricPosition.x };
        div = $('<div/>').css(css_obj);
        let metric = $('<span/>', { id: getMetricName(type) });
        div.append(metric);
        return div;
    }

    function createUnitDetailsPage() {
        fragment = document.createDocumentFragment();
        let unitHeader = createHeader();
        let unitHeaderName = createHeaderName();
        let infoHeader = createInfoHeader();
        let groupInfoButton = createGroupInfoLinkButton();
        let ledTable = createLEDTable();
        fragment.appendChild(unitHeader[0]);
        fragment.appendChild(unitHeaderName[0]);
        fragment.appendChild(infoHeader[0]);
        fragment.appendChild(groupInfoButton[0]);
        fragment.appendChild(ledTable[0]);
        document.body.appendChild(fragment);
    }

    function createHeader() {
        css_obj = { position: 'absolute', top: headerYPosition, left: xAlignPosition };
        return $('<h1/>').css(css_obj).append(`STS Unit Status`);
    }

    function createHeaderName() {
        css_obj = { position: 'absolute', top: nameHeaderYPosition, left: xAlignPosition };
        let id = getLabelID();
        return $('<h1/>').css(css_obj).append(`Name: ` + getUnitName(valtable_obj, id-1));
    }

    function createInfoHeader() {
        css_obj = { position: 'absolute', top: infoHeaderYPosition, left: xAlignPosition };
        let id = getLabelID();
        return $('<h2/>').css(css_obj).append(`Unit ID:\t${id}`);
    }

    function createGroupInfoLinkButton() {
        css_obj = { position: 'absolute', top: groupInfoLinkButtonYPosition, left: xAlignPosition };
        let button = $('<button />').css(css_obj).append('Group Info');
        return button.on("click",function () {
            window.location.href='twinsourcedccii-dcciii.html';
            return false;
         });
    }

    function createLEDTable() {
        var table, tbody, div;
        css_obj = { position: 'absolute', top: ledTableYPosition, left: xAlignPosition, 'text-align': 'left' };
        div = $('<div/>').css(css_obj);
        table = $('<table />', { 'class': 'table table-condensed table-hover' }).appendTo(div);
        tbody = $('<tbody />').appendTo(table);
        createRowsinTable(tbody);
        return div;
    }

    function createRowsinTable(tableBody) {
        let row;
        for (let entry = 0; entry < tableEntries.length; entry++) {
            row =  createNewRow(entry, row, tableBody);
            let tableLED = createTableLED(entry);
            let tableEntry = createTableEntry(entry);
            tableLED.appendTo(row);
            tableEntry.appendTo(row);
        }
    }

    function createNewRow(entry, row, tableBody) {
        if (entry % NUM_OF_COLUMNS === 0) {
            return $('<tr />').appendTo(tableBody);
        }
        return row;
    }

    function createTableLED(entry) {
        let led = getLEDSourceFile('grau', LED_OFF);
        let ledImage = $('<img/>', { id: getLEDName(entry), 'src': led })
        let tableLED = $('<td/>', { 'style' : 'width:25px' });
        if(!entryIsEmpty(entry)){
            tableLED.append(ledImage);
        }
        return tableLED;
    }

    function createTableEntry(entry) {
        let tableEntry = $('<td/>', { 'style' : 'width:250px' });
        if(!entryIsEmpty(entry)){
            tableEntry.append(tableEntries[entry]);
        }
        return tableEntry;
    }

    function entryIsEmpty(entry) {
        return ''===tableEntries[entry];
    }

    function getLabelID() {
        return (new URL(location.href)).searchParams.get('id');
    }

    function getUnitID() {
        return getLabelID() - 1;
    }

    function getLEDName(ledID) {
        return 'led_' + ledID;
    }

    function getMetricName(type) {
        return 'metric_' + type;
    }

    function create_footer_pane() {
      if (!oid || !unms_obj)
          return true;
      footer_api.create({ //Attr
          id: 'panel2'
          //'class':'unmsFooter'
      }, { //css
           position: 'absolute',
          top: '490px',
          left: '74px'
      },
         unms_obj
      );
    }

}(jQuery));
