<?xml version="1.0"?>
<!--
   web_common.xsl - XSL-template for diverse common elements UNMS-II.
   Copyright (c) 2005, Generex GmbH. All rights reserved.
   klaus
-->
<xsl:stylesheet version="1.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform">
  <xsl:output method="html" indent="yes" omit-xml-declaration="yes" encoding="iso-8859-1"/>

  <!-- window-title -->
  <xsl:variable name="service_type" select="//service/@type"/>
  <xsl:template match="text()"/>
  <xsl:template name="window-title">
    <xsl:variable name="title">
      <xsl:choose>
        <xsl:when test="$service_type='upsman'" >
          <xsl:value-of select="'UPS View Web client'"/>
        </xsl:when>
        <xsl:otherwise>
          <xsl:value-of select="'NMS Web Client'"/>
        </xsl:otherwise>
      </xsl:choose>
    </xsl:variable>
    <script type="text/javascript">
      window.document.title = '<xsl:value-of select="$title"/>';
    </script>
  </xsl:template>

  <!-- background -->
  <xsl:template name="background">
    <xsl:param name="imagepath" select="0"/>
    <xsl:param name="x" select="0"/>
    <xsl:param name="y" select="0"/>
    <xsl:param name="w" select="0"/>
    <xsl:param name="h" select="0"/>
    <xsl:param name="border" select="0"/>
    <xsl:param name="id" select="'background'"/>
    <xsl:element name="div">
      <xsl:attribute name="id">
        <xsl:value-of select="$id"/>
      </xsl:attribute>
      <xsl:attribute name="style">
        position: absolute; left:<xsl:value-of select="$x"/>px; top:<xsl:value-of select="$y"/>px; width:<xsl:value-of select="$w"/>px; height:<xsl:value-of select="$h"/>px;
      </xsl:attribute>
      <xsl:element name="img">
        <xsl:attribute name="src">
          <xsl:value-of select="$imagepath"/>
        </xsl:attribute>
        <xsl:attribute name="border">
          <xsl:value-of select="$border"/>
        </xsl:attribute>
        <xsl:attribute name="width">
          <xsl:value-of select="$w"/>
        </xsl:attribute>
        <xsl:attribute name="height">
          <xsl:value-of select="$h"/>
        </xsl:attribute>
      </xsl:element>
    </xsl:element>
  </xsl:template>

  <!--headline -->
  <xsl:variable name="img_path">
    <xsl:value-of select="'./lib/ups/image/Panel_III.png'"/>
  </xsl:variable>

  <xsl:template name="headline">
    <xsl:param name="top" select="3"/>
    <xsl:param name="left" select="0"/>
    <xsl:param name="width" select="788"/>
    <xsl:param name="img-path" select="$img_path"/>
    <xsl:param name="zonename" select="''"/>
    <xsl:param name="servicename" select="0"/>
    <xsl:param name="ip-address" select="''"/>
    <xsl:param name="class" select="'XLargeBold'"/>

    <xsl:variable name="service-tag">
      <xsl:choose>
        <!--
			<xsl:when test="string-length($ip-address)!=0" >
        	<xsl:value-of select="concat($servicename, concat($lang-at, $ip-address))"/>
		  </xsl:when>
		  -->
        <xsl:when test="string-length($servicename)!=0" >
          <xsl:value-of select="$servicename"/>
        </xsl:when>
        <xsl:otherwise>
          <xsl:value-of select="'Unknown'"/>
        </xsl:otherwise>
      </xsl:choose>
    </xsl:variable>
    <xsl:variable name="text">
      <xsl:choose>
        <xsl:when test="string-length($zonename) !=0" >
          <xsl:value-of select="concat($zonename, concat(': ', $service-tag))"/>
        </xsl:when>
        <xsl:otherwise>
          <xsl:value-of select="$service-tag"/>
        </xsl:otherwise>
      </xsl:choose>

    </xsl:variable>
    <div id="panel3" class="{$class}" style="position:absolute; top:{$top}px;left:{$left + ($width - 650) div 2}px;width:650px;height:35px;">
      <xsl:if test="$img-path != ''">
        <img border="0" src="{$img-path}" width="650" height="35"/>
      </xsl:if>
    </div>
    <div class="{$class}" id="panel4" style="position:absolute; top:{$top+5}px;left:{$left}px;width:{$width}px;text-align:center;">
      <xsl:value-of select="$text"/>
    </div>
  </xsl:template>


  <!-- generic-led creates an img-tag for led's in a highly general way:                                             -->
  <!-- $path, $prefix and $extension are to determine the base for building  the image source string                 -->
  <!-- $width and $height control the according properties of the img-tag                                            -->
  <!-- From $value $count-bits starting from $start-bit (0-based!) are taken to get a numeric value                  -->
  <!-- This value is match in the $color-scheme list, which must be a comma-separated list of strings without blanks -->
  <!-- i.e. take color-scheme 'grey,red,green,yellow' and a value of 7, and startbit = 0 and count-bits=2            -->
  <!-- then the 0th and 1st bit give a value of 2 + 1 = 3, and a color string  of 'yellow"                           -->
  <!-- together with the defaults this produces and image source of "./lib/imgstatus_yellow.gif"                     -->
  <xsl:template name="generic-led">
    <xsl:param name="path" select="'../image/'"/>
    <xsl:param name="prefix" select="'status_'"/>
    <xsl:param name="extension" select="'.gif'"/>
    <xsl:param name="color-scheme" select="''"/>
    <xsl:param name="state" select="0"/>
    <xsl:param name="start-bit" select="1"/>
    <xsl:param name="count-bits" select="1"/>
    <xsl:param name="width" select="12"/>
    <xsl:param name="height" select="12"/>

    <xsl:variable name="bit-value">
      <xsl:call-template name="getbits">
        <xsl:with-param name="val" select="$state"/>
        <xsl:with-param name="start-bit" select="$start-bit"/>
        <xsl:with-param name="count-bits" select="$count-bits"/>
      </xsl:call-template>
    </xsl:variable>
    <xsl:variable name="color">
      <xsl:call-template name="tokenize">
        <xsl:with-param name="token-string" select="$color-scheme"/>
        <xsl:with-param name="which" select="$bit-value + 1"/>
        <xsl:with-param name="delim" select="','"/>
      </xsl:call-template>
    </xsl:variable>
    <xsl:variable name="image-source" select="concat($path, concat($prefix, concat($color, $extension)))"/>
    <img src="{$image-source}" width="{$width}" height="{$height}" border=""/>
  </xsl:template>

</xsl:stylesheet>

