<?xml version="1.0"?>
<!--
   cet_common.xsl - XSL-template for CE&T specific elements.
   Copyright (c) 2010, Generex GmbH. All rights reserved.
   Said Raqui
-->
<xsl:stylesheet version="1.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform">
<xsl:output method="html" indent="yes" omit-xml-declaration="yes" encoding="iso-8859-1"/>

<!-- Temperature Value -->
<xsl:template name="get-CET-temperature"><!--unit = 0 Celsius , unit = 1 Fahrenheit-->
  <xsl:param name="value" select="0" />
  <xsl:param name="temp-unit" select="value[@name='_tempUnit']/@val" />
  <xsl:param name="precision" select="0" />

      <xsl:choose>
         <xsl:when test="$temp-unit=1">
            <xsl:call-template name="get-value">
               <xsl:with-param name="value" select="($value*1.8)+32.0" />
               <xsl:with-param name= "unit" select="''"/>
               <xsl:with-param name="precision" select="$precision"/>
            </xsl:call-template>
         </xsl:when>
         <xsl:otherwise>
            <xsl:call-template name="get-value">
               <xsl:with-param name="value" select="$value"/>
               <xsl:with-param name= "unit" select="''"/>
               <xsl:with-param name="precision" select="$precision"/>
            </xsl:call-template>        
         </xsl:otherwise>
      </xsl:choose>
</xsl:template>

	<!-- Site and location pane-->
	<xsl:template name="site-location-pane">
		<xsl:param name="x" select="26" />
	  <xsl:param name="y" select="40" />
	  <xsl:param name="w" select="360" />
	  <xsl:param name="h" select="90" />
	  <xsl:param name="align" select="'center'" />
	 	<xsl:param name="visibility" select="'visible'" />
	 	<xsl:param name="class" select="'XBold_black'" />
		<xsl:param name="id" select="'site_location_div'" />
		<xsl:param name="site_name" select="'AGIL&#160;CE&#43;T'" />
		<xsl:param name="txt1" select="'Site&#160;name'" />
		<xsl:param name="txt2" select="$lang-Location" />

		<xsl:variable name="location">
			<xsl:choose>
        <xsl:when test="string-length(//values/value[@name='LOCATION']/@val)!=0" >
        	<xsl:value-of select="//values/value[@name='LOCATION']/@val"/>
				</xsl:when>
			<xsl:otherwise>
				<xsl:value-of select="'Unknown'"/>
			</xsl:otherwise>
			</xsl:choose>
		</xsl:variable>

  	<div id="{$id}" class="{$class}" align="left" style="position:absolute; left:{$x}px; top:{$y}px; width:{$w}px; height:{$h}px;text-align:left;">
  		<table cellspacing="1" cellpadding="1" border="0"  class="{$class}" style="background-color:#C0C0C0;">
  			<tr>
					<td><xsl:value-of select="concat($txt1,':')"/></td>
					<td style="text-align:left;"><xsl:value-of select="$site_name"/></td>
				</tr>
  			<tr>
					<td><xsl:value-of select="concat($lang-Location,':')"/></td>
					<td style="text-align:left;"><xsl:value-of select="$location"/></td>
				</tr>
			</table>
		</div>
	</xsl:template>

	<!-- output power level and battery autonomy pane-->
	<xsl:template name="output-power-level-batt-autonomy-pane">
		<xsl:param name="x" select="26" />
	  <xsl:param name="y" select="540" />
	  <xsl:param name="w" select="360" />
	  <xsl:param name="h" select="90" />
	  <xsl:param name="align" select="'center'" />
	 	<xsl:param name="visibility" select="'visible'" />
	 	<xsl:param name="class" select="'XBold_black'" />
		<xsl:param name="id" select="'output_power_level_batt_autonomy_div'" />
		<xsl:param name="txt1" select="'System&#160;Loaded&#160;at'" />
		<xsl:param name="txt2" select="'BAT'" />
		<xsl:param name="ac_out_available_pow" select="'0'" />
		<xsl:param name="ac_out_batt_autonomy" select="'0'" />
		<xsl:param name="ac_out_conf" select="'0'" />
		<xsl:param name="dc_batt_conf" select="'0'" />


		<xsl:variable name="location">
			<xsl:choose>
        <xsl:when test="string-length(//values/value[@name='LOCATION']/@val)!=0" >
        	<xsl:value-of select="//values/value[@name='LOCATION']/@val"/>
				</xsl:when>
			<xsl:otherwise>
				<xsl:value-of select="'Unknown'"/>
			</xsl:otherwise>
			</xsl:choose>
		</xsl:variable>

		<xsl:variable name="batt1_autonomy">
			<xsl:call-template name="get-hh-tt-from-string">
				<xsl:with-param name="time" select="$ac_out_batt_autonomy/@val"/>
			</xsl:call-template>
		</xsl:variable>

		<xsl:variable name="batt2_autonomy">
			<xsl:call-template name="get-hh-tt-from-string">
				<xsl:with-param name="time" select="$ac_out_batt_autonomy/@val2"/>
			</xsl:call-template>
		</xsl:variable>

  	<div id="{$id}" class="{$class}" align="left" style="position:absolute; left:{$x}px; top:{$y}px; width:{$w}px; height:{$h}px;text-align:left;">
  		<table cellspacing="3" cellpadding="2" border="0"  class="{$class}" style="background-color:#C0C0C0;">
  			<tr>
					<td><xsl:value-of select="concat($txt1,':')"/></td>
					<xsl:if test="$ac_out_conf = 2">
						<td style="text-align:left;">
							<xsl:value-of select="concat(concat(concat($ac_out_available_pow/@val,'%&#160;&#160;'),
																																		concat($ac_out_available_pow/@val2,'%&#160;&#160;')),concat($ac_out_available_pow/@val3,'%'))"/>
						</td>
					</xsl:if>
					<xsl:if test="$ac_out_conf = 1">
						<td style="text-align:left;">
							<xsl:value-of select="concat($ac_out_available_pow/@val,'%&#160;&#160;')"/>
						</td>
					</xsl:if>

				</tr>
  			<tr>
					<td>
						<xsl:if test="$dc_batt_conf > 0">
							<xsl:value-of select="concat(concat($txt2,'&#160;1&#160;&#160;'),$batt1_autonomy)"/>
						</xsl:if>
					</td>
					<td>
						<xsl:if test="$dc_batt_conf > 1">
							<xsl:value-of select="concat(concat($txt2,'&#160;2&#160;&#160;'),$batt2_autonomy)"/>
						</xsl:if>
					</td>
				</tr>
			</table>
		</div>
	</xsl:template>

	<!-- Alarm and in/out text pane-->
	<xsl:template name="alarm-in-out-txt-pane">
		<xsl:param name="x" select="650" />
	  <xsl:param name="y" select="500" />
	  <xsl:param name="w" select="360" />
	  <xsl:param name="h" select="200" />
	  <xsl:param name="align" select="'center'" />
	 	<xsl:param name="visibility" select="'visible'" />
	 	<xsl:param name="class" select="'XBold_blue'" />
		<xsl:param name="id" select="'alarm_in_out_txt_div'" />
		<xsl:param name="major_alarm" select="'0'" />
		<xsl:param name="minor_alarm" select="'0'" />
		<xsl:param name="major_alarm_num" select="'0'" />
		<xsl:param name="minor_alarm_num" select="'0'" />
		<xsl:param name="in_outputs" select="'0'" />
		<xsl:param name="serial_nr" select="'0'" />
		<xsl:param name="soft_nr" select="'0'" />
		<xsl:param name="softsub_nr" select="'0'" />
		<xsl:param name="txt1" select="'Major&#160;Alarm'" />
		<xsl:param name="txt2" select="'Minor&#160;Alarm'" />
		<xsl:param name="txt3" select="'Inputs/Outputs'" />

		<xsl:param name="txt4" select="'T3S:&#160;S/N '" />
		<xsl:param name="txt5" select="'Vs'" />

      <script type="text/javascript">
         function get_cet_alarms()
         {
            return get_alarm_page_from_ups({unitid:'', priority:'', sortmode:'', outputType:2, inputType:1, lines:''});
         }
      </script>

		<xsl:variable name="major_alarm_led">
			<xsl:call-template name="get-alarm-led-color">
				<xsl:with-param name="alarm_val" select="$major_alarm" />
			</xsl:call-template>
		</xsl:variable>
		<xsl:variable name="minor_alarm_led">
			<xsl:call-template name="get-alarm-led-color">
				<xsl:with-param name="alarm_val" select="$minor_alarm" />
			</xsl:call-template>
		</xsl:variable>

		<xsl:variable name="inoutput_alarm_led">
			<xsl:call-template name="get-alarm-led-color">
				<xsl:with-param name="alarm_val" select="$in_outputs" />
			</xsl:call-template>
		</xsl:variable>

  	<div id="{$id}" class="{$class}" align="left" style="position:absolute; left:{$x}px; top:{$y}px; width:{$w}px; height:{$h}px;text-align:left;">
  		<table cellspacing="1" cellpadding="1" border="0"  class="{$class}" style="background-color:#C0C0C0;">
  			<tr>
					<td><img border="0" src="{$major_alarm_led}" width="10" height="10"/></td>
					<td style="text-align:left;">
                <xsl:choose>
                  <xsl:when test="//service/@type='CS141'">
                     <a href="javascript:get_cet_alarms()" title="Click to display alarm page" style="text-decoration : none;" class ="MediumLink">
                        <xsl:value-of select="concat($txt1,concat('&#160;(',concat($major_alarm_num,')')))"/>
                     </a>
                  </xsl:when>

                  <xsl:otherwise>
                     <a href="javascript:get_alarm_page_from_ups(830,600,'2')" title="Click to display alarm page" style="text-decoration : none;" class ="MediumLink">
                        <xsl:value-of select="concat($txt1,concat('&#160;(',concat($major_alarm_num,')')))"/>
                     </a>
                  </xsl:otherwise>
                </xsl:choose>
               </td>
				</tr>
  			<tr>
					<td><img border="0" src="{$minor_alarm_led}" width="10" height="10"/></td>
					<td style="text-align:left;">
                <xsl:choose>
                  <xsl:when test="//service/@type='CS141'">
                     <a href="javascript:get_cet_alarms()" title="Click to display alarm page" style="text-decoration : none;" class ="MediumLink">
                        <xsl:value-of select="concat($txt2,concat('&#160;(',concat($minor_alarm_num,')')))"/>
                     </a>
                  </xsl:when>

                  <xsl:otherwise>
                     <a href="javascript:get_alarm_page_from_ups(830,600,'2')" title="Click to display alarm page" style="text-decoration : none;" class ="MediumLink">
                        <xsl:value-of select="concat($txt2,concat('&#160;(',concat($minor_alarm_num,')')))"/>
                     </a>
                  </xsl:otherwise>
                </xsl:choose>
					</td>
				</tr>
  			<tr>
					<td><img border="0" src="{$inoutput_alarm_led}" width="10" height="10"/></td>
					<td style="text-align:left;">
						<a href="javascript:show('digital_inputs_div');show('digital_outputs_div');" title="Click to display inputs and outputs states" style="text-decoration : none;" class ="MediumLink">
							<xsl:value-of select="$txt3"/>
						</a>
					</td>
				</tr>
				</table>
				<table cellspacing="1" cellpadding="1" border="0" class="tdSmall9_White">
  			<tr>
 					<td>
						<xsl:value-of select="'&#160;&#160;&#160;'"/>
					</td>
				</tr>
				<tr>
					<td>
						<xsl:value-of select="concat($txt4,concat('&#160;',concat($serial_nr,concat('&#160;',concat(concat(concat('&#160;&#160;&#160;',$txt5),concat('&#160;',$soft_nr)),concat('.',$softsub_nr))))))"/>
					</td>
				</tr>
			</table>
		</div>
	</xsl:template>

	<!-- CET Alarm Leds Color-->
	<xsl:template name="get-alarm-led-color">
		<xsl:param name="alarm_val" select="'0'" />
		<xsl:param name="alarm_com_lost" select="'0'" />
		<!--Alarm led-->
    <xsl:choose>
    	<xsl:when test="$alarm_com_lost = 1">
    		<xsl:value-of select="'./lib/ups/image/ce_t/led_gray.gif'"/>
    	</xsl:when>
    	<xsl:when test="$alarm_val = 0">
    		<xsl:value-of select="'./lib/ups/image/ce_t/led_gray.gif'"/>
    	</xsl:when>
    	<xsl:when test="$alarm_val = 1">
    		<xsl:value-of select="'./lib/ups/image/ce_t/led_green.gif'"/>
    	</xsl:when>
    	<xsl:when test="$alarm_val = 2">
    		<xsl:value-of select="'./lib/ups/image/ce_t/led_yellow.gif'"/>
    	</xsl:when>
    	<xsl:when test="$alarm_val = 3">
    		<xsl:value-of select="'./lib/ups/image/ce_t/led_orange.gif'"/>
    	</xsl:when>
    	<xsl:when test="$alarm_val = 4">
    		<xsl:value-of select="'./lib/ups/image/ce_t/led_red.gif'"/>
    	</xsl:when>
    	<xsl:when test="$alarm_val = 5">
    		<xsl:value-of select="'./lib/ups/image/ce_t/led_blgreen.gif'"/>
    	</xsl:when>
    	<xsl:when test="$alarm_val = 6">
    		<xsl:value-of select="'./lib/ups/image/ce_t/led_blyellow.gif'"/>
    	</xsl:when>
    	<xsl:when test="$alarm_val = 7">
    		<xsl:value-of select="'./lib/ups/image/ce_t/led_blorange.gif'"/>
    	</xsl:when>
    	<xsl:when test="$alarm_val = 8">
    		<xsl:value-of select="'./lib/ups/image/ce_t/led_blred.gif'"/>
    	</xsl:when>
      <xsl:otherwise></xsl:otherwise>
    </xsl:choose>
	</xsl:template>

	<!-- CET led pane-->
	<xsl:template name="cet-led-pane">
		<xsl:param name="x" select="670" />
	  <xsl:param name="y" select="450" />
	  <xsl:param name="w" select="14" />
	  <xsl:param name="h" select="14" />
	  <xsl:param name="align" select="'center'" />
	 	<xsl:param name="visibility" select="'visible'" />
		<xsl:param name="id" select="'cet_led_div'" />
		<xsl:param name="alarm_val" select="'0'" />
		<xsl:param name="alarm_com_lost" select="'0'" />

		<xsl:variable name="alarm_led">
			<xsl:call-template name="get-alarm-led-color">
				<xsl:with-param name="alarm_val" select="$alarm_val" />
				<xsl:with-param name="alarm_com_lost" select="$alarm_com_lost" />
			</xsl:call-template>
		</xsl:variable>

  	<div id="{$id}" align="left" style="position:absolute; left:{$x}px; top:{$y}px; width:{$w}px; height:{$h}px;">
			<img border="0" src="{$alarm_led}" width="10" height="10"/>
		</div>
	</xsl:template>

	<!-- CET Arrow img-->
	<xsl:template name="get-arrow-img">
		<xsl:param name="arrow_val" select="'0'" />
		<xsl:param name="direct" select="'hor'" />
		<xsl:param name="img_path" select="'./lib/ups/image/ce_t/'" />
		<xsl:param name="alarm_com_lost" select="'0'"/>
		<!--Arrow img-->
    <xsl:choose>
    	<xsl:when test="($arrow_val=1 or $arrow_val=2 or $arrow_val=3 or $arrow_val=4) and $alarm_com_lost = 1">
    		<xsl:value-of select="concat(concat(concat(concat($img_path, 'arrow_'), $direct),'_in_gray'),'.gif')"/>
    	</xsl:when>
    	<xsl:when test="$arrow_val = 1">
    		<xsl:value-of select="concat(concat(concat(concat($img_path, 'arrow_'), $direct),'_in_gray'),'.gif')"/>
    	</xsl:when>
    	<xsl:when test="$arrow_val = 2">
    		<xsl:value-of select="concat(concat(concat(concat($img_path, 'arrow_'), $direct),'_in_green'),'.gif')"/>
    	</xsl:when>
    	<xsl:when test="$arrow_val = 3">
    		<xsl:value-of select="concat(concat(concat(concat($img_path, 'arrow_'), $direct),'_in_orange'),'.gif')"/>
    	</xsl:when>
    	<xsl:when test="$arrow_val = 4">
    		<xsl:value-of select="concat(concat(concat(concat($img_path, 'arrow_'), $direct),'_in_red'),'.gif')"/>
    	</xsl:when>
    	<xsl:when test="($arrow_val=5 or $arrow_val=6 or $arrow_val=7) and $alarm_com_lost = 1">
    		<xsl:value-of select="concat(concat(concat(concat($img_path, 'arrow_'), $direct),'_out_gray'),'.gif')"/>
    	</xsl:when>
    	<xsl:when test="$arrow_val = 5">
    		<xsl:value-of select="concat(concat(concat(concat($img_path, 'arrow_'), $direct),'_out_green'),'.gif')"/>
    	</xsl:when>
    	<xsl:when test="$arrow_val = 6">
    		<xsl:value-of select="concat(concat(concat(concat($img_path, 'arrow_'), $direct),'_out_orange'),'.gif')"/>
    	</xsl:when>
    	<xsl:when test="$arrow_val = 7">
    		<xsl:value-of select="concat(concat(concat(concat($img_path, 'arrow_'), $direct),'_out_red'),'.gif')"/>
    	</xsl:when>
      <xsl:otherwise><xsl:value-of select="concat($img_path, 'arrow_empty.png')"/></xsl:otherwise>
    </xsl:choose>
	</xsl:template>

	<!-- CET arrow pane-->
	<xsl:template name="cet-arrow-pane">
		<xsl:param name="x" select="670" />
	  <xsl:param name="y" select="450" />
	  <xsl:param name="w" select="32" />
	  <xsl:param name="h" select="32" />
	  <xsl:param name="align" select="'center'" />
	 	<xsl:param name="visibility" select="'visible'" />
		<xsl:param name="id" select="'cet_arrow_div'" />
		<xsl:param name="arrow_val" select="'0'" />
		<xsl:param name="direct" select="'hor'" />
		<xsl:param name="alarm_com_lost" select="'0'"/>

		<xsl:variable name="arrow_img">
			<xsl:call-template name="get-arrow-img">
				<xsl:with-param name="arrow_val" select="$arrow_val" />
				<xsl:with-param name="direct" select="$direct" />
				<xsl:with-param name="alarm_com_lost" select="$alarm_com_lost" />
			</xsl:call-template>
		</xsl:variable>

  	<div id="{$id}" align="left" style="position:absolute; left:{$x}px; top:{$y}px; width:{$w}px; height:{$h}px;">
			<img border="0" src="{$arrow_img}" width="{$w}" height="{$h}"/>
		</div>
	</xsl:template>

	<!-- CET Status of the MBP-->
	<xsl:template name="cet-mbp-pane">
		<xsl:param name="x" select="373" />
	  <xsl:param name="y" select="110" />
	  <xsl:param name="w" select="72" />
	  <xsl:param name="h" select="42" />
	  <xsl:param name="align" select="'center'" />
	 	<xsl:param name="visibility" select="'visible'" />
		<xsl:param name="id" select="'cet_mbp_div'" />
		<xsl:param name="mbp_val" select="'0'" />
		<xsl:param name="img_path" select="'./lib/ups/image/ce_t/'" />
		<xsl:param name="alarm_com_lost" select="'0'"/>

		<xsl:variable name="mbp_img">
    	<xsl:choose>
    		<xsl:when test="$alarm_com_lost = 1">
    			<xsl:value-of select="concat($img_path, 'mbp.png')"/>
    		</xsl:when>
    		<xsl:when test="$mbp_val = 1">
    			<xsl:value-of select="concat($img_path, 'mbp0.jpg')"/>
    		</xsl:when>
    		<xsl:when test="$mbp_val = 2">
    			<xsl:value-of select="concat($img_path, 'mbp1.jpg')"/>
    		</xsl:when>
      <xsl:otherwise><xsl:value-of select="concat($img_path, 'mbp.png')"/></xsl:otherwise>
    </xsl:choose>
		</xsl:variable>

  	<div id="{$id}" align="left" style="position:absolute; left:{$x}px; top:{$y}px; width:{$w}px; height:{$h}px;">
			<img border="0" src="{$mbp_img}" width="{$w}" height="{$h}"/>
		</div>
	</xsl:template>



	<!-- CET digital inputs pane-->
	<xsl:template name="digital-inputs-pane">
		<xsl:param name="x" select="40" />
	  <xsl:param name="y" select="80" />
	  <xsl:param name="w" select="300" />
	  <xsl:param name="h" select="100" />
	  <xsl:param name="align" select="'center'" />
	 	<xsl:param name="visibility" select="'hidden'" />
	 	<xsl:param name="class" select="'tdSmall9'" />
		<xsl:param name="id" select="'digital_inputs_div'" />
		<xsl:param name="title" select="'Digital&#160;Inputs'" />
		<xsl:param name="txt1" select="'Reference'" />
		<xsl:param name="txt2" select="'Label'" />
		<xsl:param name="txt3" select="'State'" />
		<xsl:param name="num_tsio" select="'0'" />
		<xsl:param name="t3s_state" select="'0'" />
		<xsl:param name="tsio_state" select="'0'" />
		<xsl:param name="tsio_com_err" select="'0'" />

		<!--T3S Input1 state-->
		<xsl:variable name="t3s_input_1_state">
	    <xsl:choose>
	    	<xsl:when test="($t3s_state mod ($bit0 * 2)) - ($t3s_state mod $bit0)"><xsl:value-of select="'TRUE'"/></xsl:when>
	      <xsl:otherwise><xsl:value-of select="'FALSE'"/></xsl:otherwise>
	    </xsl:choose>
		</xsl:variable>
		<!--T3S Input2 state-->
		<xsl:variable name="t3s_input_2_state">
	    <xsl:choose>
	    	<xsl:when test="($t3s_state mod ($bit1 * 2)) - ($t3s_state mod $bit1)"><xsl:value-of select="'TRUE'"/></xsl:when>
	      <xsl:otherwise><xsl:value-of select="'FALSE'"/></xsl:otherwise>
	    </xsl:choose>
		</xsl:variable>

		<!--TSIO1 Input1 state-->
		<xsl:variable name="tsio1_input1_state">
	    <xsl:choose>
	    	<xsl:when test="($tsio_com_err mod ($bit0 * 2)) - ($tsio_com_err mod $bit0)= 0"><xsl:value-of select="'????'"/></xsl:when>
	    	<xsl:when test="($tsio_state/@val mod ($bit0 * 2)) - ($tsio_state/@val mod $bit0)"><xsl:value-of select="'TRUE'"/></xsl:when>
	      <xsl:otherwise><xsl:value-of select="'FALSE'"/></xsl:otherwise>
	    </xsl:choose>
		</xsl:variable>
		<!--TSIO1 Input2 state-->
		<xsl:variable name="tsio1_input2_state">
	    <xsl:choose>
	    	<xsl:when test="($tsio_com_err mod ($bit0 * 2)) - ($tsio_com_err mod $bit0)= 0"><xsl:value-of select="'????'"/></xsl:when>
	    	<xsl:when test="($tsio_state/@val mod ($bit1 * 2)) - ($tsio_state/@val mod $bit1)"><xsl:value-of select="'TRUE'"/></xsl:when>
	      <xsl:otherwise><xsl:value-of select="'FALSE'"/></xsl:otherwise>
	    </xsl:choose>
		</xsl:variable>
		<!--TSIO1 Input3 state-->
		<xsl:variable name="tsio1_input3_state">
	    <xsl:choose>
	    	<xsl:when test="($tsio_com_err mod ($bit0 * 2)) - ($tsio_com_err mod $bit0)= 0"><xsl:value-of select="'????'"/></xsl:when>
	    	<xsl:when test="($tsio_state/@val mod ($bit2 * 2)) - ($tsio_state/@val mod $bit2)"><xsl:value-of select="'TRUE'"/></xsl:when>
	      <xsl:otherwise><xsl:value-of select="'FALSE'"/></xsl:otherwise>
	    </xsl:choose>
		</xsl:variable>
		<!--TSIO1 Input4 state-->
		<xsl:variable name="tsio1_input4_state">
	    <xsl:choose>
	    	<xsl:when test="($tsio_com_err mod ($bit0 * 2)) - ($tsio_com_err mod $bit0)= 0"><xsl:value-of select="'????'"/></xsl:when>
	    	<xsl:when test="($tsio_state/@val mod ($bit3 * 2)) - ($tsio_state/@val mod $bit3)"><xsl:value-of select="'TRUE'"/></xsl:when>
	      <xsl:otherwise><xsl:value-of select="'FALSE'"/></xsl:otherwise>
	    </xsl:choose>
		</xsl:variable>

		<!--TSIO2 Input1 state-->
		<xsl:variable name="tsio2_input1_state">
	    <xsl:choose>
	    	<xsl:when test="($tsio_com_err mod ($bit1 * 2)) - ($tsio_com_err mod $bit1)= 0"><xsl:value-of select="'????'"/></xsl:when>
	    	<xsl:when test="($tsio_state/@val2 mod ($bit0 * 2)) - ($tsio_state/@val2 mod $bit0)"><xsl:value-of select="'TRUE'"/></xsl:when>
	      <xsl:otherwise><xsl:value-of select="'FALSE'"/></xsl:otherwise>
	    </xsl:choose>
		</xsl:variable>
		<!--TSIO2 Input2 state-->
		<xsl:variable name="tsio2_input2_state">
	    <xsl:choose>
	    	<xsl:when test="($tsio_com_err mod ($bit1 * 2)) - ($tsio_com_err mod $bit1)= 0"><xsl:value-of select="'????'"/></xsl:when>
	    	<xsl:when test="($tsio_state/@val2 mod ($bit1 * 2)) - ($tsio_state/@val2 mod $bit1)"><xsl:value-of select="'TRUE'"/></xsl:when>
	      <xsl:otherwise><xsl:value-of select="'FALSE'"/></xsl:otherwise>
	    </xsl:choose>
		</xsl:variable>
		<!--TSIO2 Input3 state-->
		<xsl:variable name="tsio2_input3_state">
	    <xsl:choose>
	    	<xsl:when test="($tsio_com_err mod ($bit1 * 2)) - ($tsio_com_err mod $bit1)= 0"><xsl:value-of select="'????'"/></xsl:when>
	    	<xsl:when test="($tsio_state/@val2 mod ($bit2 * 2)) - ($tsio_state/@val2 mod $bit2)"><xsl:value-of select="'TRUE'"/></xsl:when>
	      <xsl:otherwise><xsl:value-of select="'FALSE'"/></xsl:otherwise>
	    </xsl:choose>
		</xsl:variable>
		<!--TSIO2 Input4 state-->
		<xsl:variable name="tsio2_input4_state">
	    <xsl:choose>
	    	<xsl:when test="($tsio_com_err mod ($bit1 * 2)) - ($tsio_com_err mod $bit1)= 0"><xsl:value-of select="'????'"/></xsl:when>
	    	<xsl:when test="($tsio_state/@val2 mod ($bit3 * 2)) - ($tsio_state/@val2 mod $bit3)"><xsl:value-of select="'TRUE'"/></xsl:when>
	      <xsl:otherwise><xsl:value-of select="'FALSE'"/></xsl:otherwise>
	    </xsl:choose>
		</xsl:variable>

		<!--TSIO3 Input1 state-->
		<xsl:variable name="tsio3_input1_state">
	    <xsl:choose>
	    	<xsl:when test="($tsio_com_err mod ($bit2 * 2)) - ($tsio_com_err mod $bit2)= 0"><xsl:value-of select="'????'"/></xsl:when>
	    	<xsl:when test="($tsio_state/@val3 mod ($bit0 * 2)) - ($tsio_state/@val3 mod $bit0)"><xsl:value-of select="'TRUE'"/></xsl:when>
	      <xsl:otherwise><xsl:value-of select="'FALSE'"/></xsl:otherwise>
	    </xsl:choose>
		</xsl:variable>
		<!--TSIO3 Input2 state-->
		<xsl:variable name="tsio3_input2_state">
	    <xsl:choose>
	    	<xsl:when test="($tsio_com_err mod ($bit2 * 2)) - ($tsio_com_err mod $bit2)= 0"><xsl:value-of select="'????'"/></xsl:when>
	    	<xsl:when test="($tsio_state/@val3 mod ($bit1 * 2)) - ($tsio_state/@val3 mod $bit1)"><xsl:value-of select="'TRUE'"/></xsl:when>
	      <xsl:otherwise><xsl:value-of select="'FALSE'"/></xsl:otherwise>
	    </xsl:choose>
		</xsl:variable>
		<!--TSIO3 Input3 state-->
		<xsl:variable name="tsio3_input3_state">
	    <xsl:choose>
	    	<xsl:when test="($tsio_com_err mod ($bit2 * 2)) - ($tsio_com_err mod $bit2)= 0"><xsl:value-of select="'????'"/></xsl:when>
	    	<xsl:when test="($tsio_state/@val3 mod ($bit2 * 2)) - ($tsio_state/@val3 mod $bit2)"><xsl:value-of select="'TRUE'"/></xsl:when>
	      <xsl:otherwise><xsl:value-of select="'FALSE'"/></xsl:otherwise>
	    </xsl:choose>
		</xsl:variable>
		<!--TSIO3 Input4 state-->
		<xsl:variable name="tsio3_input4_state">
	    <xsl:choose>
	    	<xsl:when test="($tsio_com_err mod ($bit2 * 2)) - ($tsio_com_err mod $bit2)= 0"><xsl:value-of select="'????'"/></xsl:when>
	    	<xsl:when test="($tsio_state/@val3 mod ($bit3 * 2)) - ($tsio_state/@val3 mod $bit3)"><xsl:value-of select="'TRUE'"/></xsl:when>
	      <xsl:otherwise><xsl:value-of select="'FALSE'"/></xsl:otherwise>
	    </xsl:choose>
		</xsl:variable>

		<!--TSIO4 Input1 state-->
		<xsl:variable name="tsio4_input1_state">
	    <xsl:choose>
	    	<xsl:when test="($tsio_com_err mod ($bit3 * 2)) - ($tsio_com_err mod $bit3)= 0"><xsl:value-of select="'????'"/></xsl:when>
	    	<xsl:when test="($tsio_state/@val4 mod ($bit0 * 2)) - ($tsio_state/@val4 mod $bit0)"><xsl:value-of select="'TRUE'"/></xsl:when>
	      <xsl:otherwise><xsl:value-of select="'FALSE'"/></xsl:otherwise>
	    </xsl:choose>
		</xsl:variable>
		<!--TSIO4 Input2 state-->
		<xsl:variable name="tsio4_input2_state">
	    <xsl:choose>
	    	<xsl:when test="($tsio_com_err mod ($bit3 * 2)) - ($tsio_com_err mod $bit3)= 0"><xsl:value-of select="'????'"/></xsl:when>
	    	<xsl:when test="($tsio_state/@val4 mod ($bit1 * 2)) - ($tsio_state/@val4 mod $bit1)"><xsl:value-of select="'TRUE'"/></xsl:when>
	      <xsl:otherwise><xsl:value-of select="'FALSE'"/></xsl:otherwise>
	    </xsl:choose>
		</xsl:variable>
		<!--TSIO4 Input3 state-->
		<xsl:variable name="tsio4_input3_state">
	    <xsl:choose>
	    	<xsl:when test="($tsio_com_err mod ($bit3 * 2)) - ($tsio_com_err mod $bit3)= 0"><xsl:value-of select="'????'"/></xsl:when>
	    	<xsl:when test="($tsio_state/@val4 mod ($bit2 * 2)) - ($tsio_state/@val4 mod $bit2)"><xsl:value-of select="'TRUE'"/></xsl:when>
	      <xsl:otherwise><xsl:value-of select="'FALSE'"/></xsl:otherwise>
	    </xsl:choose>
		</xsl:variable>
		<!--TSIO4 Input4 state-->
		<xsl:variable name="tsio4_input4_state">
	    <xsl:choose>
	    	<xsl:when test="($tsio_com_err mod ($bit3 * 2)) - ($tsio_com_err mod $bit3)= 0"><xsl:value-of select="'????'"/></xsl:when>
	    	<xsl:when test="($tsio_state/@val4 mod ($bit3 * 2)) - ($tsio_state/@val4 mod $bit3)"><xsl:value-of select="'TRUE'"/></xsl:when>
	      <xsl:otherwise><xsl:value-of select="'FALSE'"/></xsl:otherwise>
	    </xsl:choose>
		</xsl:variable>

		<!--TSIO5 Input1 state-->
		<xsl:variable name="tsio5_input1_state">
	    <xsl:choose>
	    	<xsl:when test="($tsio_com_err mod ($bit4 * 2)) - ($tsio_com_err mod $bit4)= 0"><xsl:value-of select="'????'"/></xsl:when>
	    	<xsl:when test="($tsio_state/@val5 mod ($bit0 * 2)) - ($tsio_state/@val5 mod $bit0)"><xsl:value-of select="'TRUE'"/></xsl:when>
	      <xsl:otherwise><xsl:value-of select="'FALSE'"/></xsl:otherwise>
	    </xsl:choose>
		</xsl:variable>
		<!--TSIO5 Input2 state-->
		<xsl:variable name="tsio5_input2_state">
	    <xsl:choose>
	    	<xsl:when test="($tsio_com_err mod ($bit4 * 2)) - ($tsio_com_err mod $bit4)= 0"><xsl:value-of select="'????'"/></xsl:when>
	    	<xsl:when test="($tsio_state/@val5 mod ($bit1 * 2)) - ($tsio_state/@val5 mod $bit1)"><xsl:value-of select="'TRUE'"/></xsl:when>
	      <xsl:otherwise><xsl:value-of select="'FALSE'"/></xsl:otherwise>
	    </xsl:choose>
		</xsl:variable>
		<!--TSIO5 Input3 state-->
		<xsl:variable name="tsio5_input3_state">
	    <xsl:choose>
	    	<xsl:when test="($tsio_com_err mod ($bit4 * 2)) - ($tsio_com_err mod $bit4)= 0"><xsl:value-of select="'????'"/></xsl:when>
	    	<xsl:when test="($tsio_state/@val5 mod ($bit2 * 2)) - ($tsio_state/@val5 mod $bit2)"><xsl:value-of select="'TRUE'"/></xsl:when>
	      <xsl:otherwise><xsl:value-of select="'FALSE'"/></xsl:otherwise>
	    </xsl:choose>
		</xsl:variable>

		<!--TSIO6 Input1 state-->
		<xsl:variable name="tsio6_input1_state">
	    <xsl:choose>
	    	<xsl:when test="($tsio_com_err mod ($bit5 * 2)) - ($tsio_com_err mod $bit5)= 0"><xsl:value-of select="'????'"/></xsl:when>
	    	<xsl:when test="($tsio_state/@val6 mod ($bit0 * 2)) - ($tsio_state/@val6 mod $bit0)"><xsl:value-of select="'TRUE'"/></xsl:when>
	      <xsl:otherwise><xsl:value-of select="'FALSE'"/></xsl:otherwise>
	    </xsl:choose>
		</xsl:variable>
		<!--TSIO6 Input2 state-->
		<xsl:variable name="tsio6_input2_state">
	    <xsl:choose>
	    	<xsl:when test="($tsio_com_err mod ($bit5 * 2)) - ($tsio_com_err mod $bit5)= 0"><xsl:value-of select="'????'"/></xsl:when>
	    	<xsl:when test="($tsio_state/@val6 mod ($bit1 * 2)) - ($tsio_state/@val6 mod $bit1)"><xsl:value-of select="'TRUE'"/></xsl:when>
	      <xsl:otherwise><xsl:value-of select="'FALSE'"/></xsl:otherwise>
	    </xsl:choose>
		</xsl:variable>
		<!--TSIO6 Input3 state-->
		<xsl:variable name="tsio6_input3_state">
	    <xsl:choose>
	    	<xsl:when test="($tsio_com_err mod ($bit5 * 2)) - ($tsio_com_err mod $bit5)= 0"><xsl:value-of select="'????'"/></xsl:when>
	    	<xsl:when test="($tsio_state/@val6 mod ($bit2 * 2)) - ($tsio_state/@val6 mod $bit2)"><xsl:value-of select="'TRUE'"/></xsl:when>
	      <xsl:otherwise><xsl:value-of select="'FALSE'"/></xsl:otherwise>
	    </xsl:choose>
		</xsl:variable>

  	<div id="{$id}" onmousedown="dragstart(this)" onclick="bring_to_front('{$id}')" class="XBold_black" align="left" style="position:absolute; left:{$x}px; top:{$y}px; width:{$w}px; height:{$h}px;text-align:left;visibility:{$visibility};z-index:1;">
  		<table cellspacing="0" cellpadding="0" border="0"  class="XBold_black" style="position:absolute;width:{$w}px; height:20px;color:white;background-color:#800000;">
  			<thead>
    			<tr>
      			<th align="left"><xsl:value-of select="concat('&#160;&#160;',$title)"/></th>
      			<th><xsl:value-of select="'&#160;'"/></th>
      			<th align="right">
							<a href="javascript:show('{$id}');" class="btn">
								<span><span  style="color:white; background-color:#800000;"><xsl:value-of select="'X'"/></span></span>
							</a>
						</th>
    			</tr>
  			</thead>
			</table><!--border 1px solid black;border-collapse:collapse;-->

  		<table border="1" cellspacing="0" cellpadding="0" class="{$class}" style="position:absolute;top:{$y -60}px; width:{$w}px;color:black;background-color:white;text-align:center;">
			<tbody>
  			<tr>
					<td class="XBold_black"><xsl:value-of select="$txt1"/></td>
					<td class="XBold_black"><xsl:value-of select="$txt2"/></td>
					<td class="XBold_black"><xsl:value-of select="$txt3"/></td>
				</tr>
  			<tr>
					<td><xsl:value-of select="'T3S&#160;Input1'"/></td>
					<td><xsl:value-of select="value[@name='CET2_LBLT3SINP1']/@val"/></td>
					<td><xsl:value-of select="$t3s_input_1_state"/></td>
				</tr>
  			<tr>
					<td><xsl:value-of select="'T3S&#160;Input2'"/></td>
					<td><xsl:value-of select="value[@name='CET2_LBLT3SINP2']/@val"/></td>
					<td><xsl:value-of select="$t3s_input_2_state"/></td>
				</tr>

				<xsl:if test="$num_tsio > 0">
	  			<tr>
						<td><xsl:value-of select="'TSIO1&#160;Input1'"/></td>
						<td><xsl:value-of select="value[@name='CET2_LBLTSIOI11']/@val"/></td>
						<td><xsl:value-of select="$tsio1_input1_state"/></td>
					</tr>
	  			<tr>
						<td><xsl:value-of select="'TSIO1&#160;Input2'"/></td>
						<td><xsl:value-of select="value[@name='CET2_LBLTSIOI12']/@val"/></td>
						<td><xsl:value-of select="$tsio1_input2_state"/></td>
					</tr>
	  			<tr>
						<td><xsl:value-of select="'TSIO1&#160;Input3'"/></td>
						<td><xsl:value-of select="value[@name='CET2_LBLTSIOI13']/@val"/></td>
						<td><xsl:value-of select="$tsio1_input3_state"/></td>
					</tr>
	  			<tr>
						<td><xsl:value-of select="'TSIO1&#160;Input4'"/></td>
						<td><xsl:value-of select="value[@name='CET2_LBLTSIOI14']/@val"/></td>
						<td><xsl:value-of select="$tsio1_input4_state"/></td>
					</tr>
				</xsl:if>

				<xsl:if test="$num_tsio > 1">
	  			<tr>
						<td><xsl:value-of select="'TSIO2&#160;Input1'"/></td>
						<td><xsl:value-of select="value[@name='CET2_LBLTSIOI21']/@val"/></td>
						<td><xsl:value-of select="$tsio2_input1_state"/></td>
					</tr>
	  			<tr>
						<td><xsl:value-of select="'TSIO2&#160;Input2'"/></td>
						<td><xsl:value-of select="value[@name='CET2_LBLTSIOI22']/@val"/></td>
						<td><xsl:value-of select="$tsio2_input2_state"/></td>
					</tr>
	  			<tr>
						<td><xsl:value-of select="'TSIO2&#160;Input3'"/></td>
						<td><xsl:value-of select="value[@name='CET2_LBLTSIOI23']/@val"/></td>
						<td><xsl:value-of select="$tsio2_input3_state"/></td>
					</tr>
	  			<tr>
						<td><xsl:value-of select="'TSIO2&#160;Input4'"/></td>
						<td><xsl:value-of select="value[@name='CET2_LBLTSIOI24']/@val"/></td>
						<td><xsl:value-of select="$tsio2_input4_state"/></td>
					</tr>
				</xsl:if>
			</tbody>
			</table>
		</div>
	</xsl:template>

	<!-- CET digital outputs pane-->
	<xsl:template name="digital-outputs-pane">
		<xsl:param name="x" select="200" />
	  <xsl:param name="y" select="80" />
	  <xsl:param name="w" select="300" />
	  <xsl:param name="h" select="100" />
	  <xsl:param name="align" select="'center'" />
	 	<xsl:param name="visibility" select="'hidden'" />
	 	<xsl:param name="class" select="'tdSmall9'" />
		<xsl:param name="id" select="'digital_outputs_div'" />

		<xsl:param name="title" select="'Digital&#160;Outputs'" />
		<xsl:param name="txt1" select="'Reference'" />
		<xsl:param name="txt2" select="'Label'" />
		<xsl:param name="txt3" select="'State'" />
		<xsl:param name="num_tsio" select="'0'" />
		<xsl:param name="t3s_state" select="'0'" />
		<xsl:param name="tsio_state" select="'0'" />
		<xsl:param name="tsio_com_err" select="'0'" />

		<!--T3S Output1 state-->
		<xsl:variable name="t3s_output1_state">
	    <xsl:choose>
	    	<xsl:when test="($t3s_state mod ($bit8 * 2)) - ($t3s_state mod $bit8)"><xsl:value-of select="'TRUE'"/></xsl:when>
	      <xsl:otherwise><xsl:value-of select="'FALSE'"/></xsl:otherwise>
	    </xsl:choose>
		</xsl:variable>
		<!--T3S Output2 state-->
		<xsl:variable name="t3s_output2_state">
	    <xsl:choose>
	    	<xsl:when test="($t3s_state mod ($bit9 * 2)) - ($t3s_state mod $bit9)"><xsl:value-of select="'TRUE'"/></xsl:when>
	      <xsl:otherwise><xsl:value-of select="'FALSE'"/></xsl:otherwise>
	    </xsl:choose>
		</xsl:variable>
		<!--T3S Output4 state-->
		<xsl:variable name="t3s_output3_state">
	    <xsl:choose>
	    	<xsl:when test="($t3s_state mod ($bit10 * 2)) - ($t3s_state mod $bit10)"><xsl:value-of select="'TRUE'"/></xsl:when>
	      <xsl:otherwise><xsl:value-of select="'FALSE'"/></xsl:otherwise>
	    </xsl:choose>
		</xsl:variable>

		<!--TSIO1 Output1 state-->
		<xsl:variable name="tsio1_output1_state">
	    <xsl:choose>
	    	<xsl:when test="($tsio_com_err mod ($bit0 * 2)) - ($tsio_com_err mod $bit0)= 0"><xsl:value-of select="'????'"/></xsl:when>
	    	<xsl:when test="($tsio_state/@val mod ($bit8 * 2)) - ($tsio_state/@val mod $bit8)"><xsl:value-of select="'TRUE'"/></xsl:when>
	      <xsl:otherwise><xsl:value-of select="'FALSE'"/></xsl:otherwise>
	    </xsl:choose>
		</xsl:variable>
		<!--TSIO1 Output2 state-->
		<xsl:variable name="tsio1_output2_state">
	    <xsl:choose>
	    	<xsl:when test="($tsio_com_err mod ($bit0 * 2)) - ($tsio_com_err mod $bit0)= 0"><xsl:value-of select="'????'"/></xsl:when>
	    	<xsl:when test="($tsio_state/@val mod ($bit9 * 2)) - ($tsio_state/@val mod $bit9)"><xsl:value-of select="'TRUE'"/></xsl:when>
	      <xsl:otherwise><xsl:value-of select="'FALSE'"/></xsl:otherwise>
	    </xsl:choose>
		</xsl:variable>
		<!--TSIO1 Output3 state-->
		<xsl:variable name="tsio1_output3_state">
	    <xsl:choose>
	    	<xsl:when test="($tsio_com_err mod ($bit0 * 2)) - ($tsio_com_err mod $bit0)= 0"><xsl:value-of select="'????'"/></xsl:when>
	    	<xsl:when test="($tsio_state/@val mod ($bit10 * 2)) - ($tsio_state/@val mod $bit10)"><xsl:value-of select="'TRUE'"/></xsl:when>
	      <xsl:otherwise><xsl:value-of select="'FALSE'"/></xsl:otherwise>
	    </xsl:choose>
		</xsl:variable>
		<!--TSIO1 Output4 state-->
		<xsl:variable name="tsio1_output4_state">
	    <xsl:choose>
	    	<xsl:when test="($tsio_com_err mod ($bit0 * 2)) - ($tsio_com_err mod $bit0)= 0"><xsl:value-of select="'????'"/></xsl:when>
	    	<xsl:when test="($tsio_state/@val mod ($bit11 * 2)) - ($tsio_state/@val mod $bit11)"><xsl:value-of select="'TRUE'"/></xsl:when>
	      <xsl:otherwise><xsl:value-of select="'FALSE'"/></xsl:otherwise>
	    </xsl:choose>
		</xsl:variable>

		<!--TSIO2 Output1 state-->
		<xsl:variable name="tsio2_output1_state">
	    <xsl:choose>
	    	<xsl:when test="($tsio_com_err mod ($bit1 * 2)) - ($tsio_com_err mod $bit1)= 0"><xsl:value-of select="'????'"/></xsl:when>
	    	<xsl:when test="($tsio_state/@val2 mod ($bit8 * 2)) - ($tsio_state/@val2 mod $bit8)"><xsl:value-of select="'TRUE'"/></xsl:when>
	      <xsl:otherwise><xsl:value-of select="'FALSE'"/></xsl:otherwise>
	    </xsl:choose>
		</xsl:variable>
		<!--TSIO2 Output2 state-->
		<xsl:variable name="tsio2_output2_state">
	    <xsl:choose>
	    	<xsl:when test="($tsio_com_err mod ($bit1 * 2)) - ($tsio_com_err mod $bit1)= 0"><xsl:value-of select="'????'"/></xsl:when>
	    	<xsl:when test="($tsio_state/@val2 mod ($bit9 * 2)) - ($tsio_state/@val2 mod $bit9)"><xsl:value-of select="'TRUE'"/></xsl:when>
	      <xsl:otherwise><xsl:value-of select="'FALSE'"/></xsl:otherwise>
	    </xsl:choose>
		</xsl:variable>
		<!--TSIO2 Output3 state-->
		<xsl:variable name="tsio2_output3_state">
	    <xsl:choose>
	    	<xsl:when test="($tsio_com_err mod ($bit1 * 2)) - ($tsio_com_err mod $bit1)= 0"><xsl:value-of select="'????'"/></xsl:when>
	    	<xsl:when test="($tsio_state/@val2 mod ($bit10 * 2)) - ($tsio_state/@val2 mod $bit10)"><xsl:value-of select="'TRUE'"/></xsl:when>
	      <xsl:otherwise><xsl:value-of select="'FALSE'"/></xsl:otherwise>
	    </xsl:choose>
		</xsl:variable>
		<!--TSIO2 Output4 state-->
		<xsl:variable name="tsio2_output4_state">
	    <xsl:choose>
	    	<xsl:when test="($tsio_com_err mod ($bit1 * 2)) - ($tsio_com_err mod $bit1)= 0"><xsl:value-of select="'????'"/></xsl:when>
	    	<xsl:when test="($tsio_state/@val2 mod ($bit11 * 2)) - ($tsio_state/@val2 mod $bit11)"><xsl:value-of select="'TRUE'"/></xsl:when>
	      <xsl:otherwise><xsl:value-of select="'FALSE'"/></xsl:otherwise>
	    </xsl:choose>
		</xsl:variable>

		<!--TSIO3 Output1 state-->
		<xsl:variable name="tsio3_output1_state">
	    <xsl:choose>
	    	<xsl:when test="($tsio_com_err mod ($bit2 * 2)) - ($tsio_com_err mod $bit2)= 0"><xsl:value-of select="'????'"/></xsl:when>
	    	<xsl:when test="($tsio_state/@val3 mod ($bit8 * 2)) - ($tsio_state/@val3 mod $bit8)"><xsl:value-of select="'TRUE'"/></xsl:when>
	      <xsl:otherwise><xsl:value-of select="'FALSE'"/></xsl:otherwise>
	    </xsl:choose>
		</xsl:variable>
		<!--TSIO3 Output2 state-->
		<xsl:variable name="tsio3_output2_state">
	    <xsl:choose>
	    	<xsl:when test="($tsio_com_err mod ($bit2 * 2)) - ($tsio_com_err mod $bit2)= 0"><xsl:value-of select="'????'"/></xsl:when>
	    	<xsl:when test="($tsio_state/@val3 mod ($bit9 * 2)) - ($tsio_state/@val3 mod $bit9)"><xsl:value-of select="'TRUE'"/></xsl:when>
	      <xsl:otherwise><xsl:value-of select="'FALSE'"/></xsl:otherwise>
	    </xsl:choose>
		</xsl:variable>
		<!--TSIO3 Output3 state-->
		<xsl:variable name="tsio3_output3_state">
	    <xsl:choose>
	    	<xsl:when test="($tsio_com_err mod ($bit2 * 2)) - ($tsio_com_err mod $bit2)= 0"><xsl:value-of select="'????'"/></xsl:when>
	    	<xsl:when test="($tsio_state/@val3 mod ($bit10 * 2)) - ($tsio_state/@val3 mod $bit10)"><xsl:value-of select="'TRUE'"/></xsl:when>
	      <xsl:otherwise><xsl:value-of select="'FALSE'"/></xsl:otherwise>
	    </xsl:choose>
		</xsl:variable>
		<!--TSIO3 Output4 state-->
		<xsl:variable name="tsio3_output4_state">
	    <xsl:choose>
	    	<xsl:when test="($tsio_com_err mod ($bit2 * 2)) - ($tsio_com_err mod $bit2)= 0"><xsl:value-of select="'????'"/></xsl:when>
	    	<xsl:when test="($tsio_state/@val3 mod ($bit11 * 2)) - ($tsio_state/@val3 mod $bit11)"><xsl:value-of select="'TRUE'"/></xsl:when>
	      <xsl:otherwise><xsl:value-of select="'FALSE'"/></xsl:otherwise>
	    </xsl:choose>
		</xsl:variable>

		<!--TSIO4 Output1 state-->
		<xsl:variable name="tsio4_output1_state">
	    <xsl:choose>
	    	<xsl:when test="($tsio_com_err mod ($bit3 * 2)) - ($tsio_com_err mod $bit3)= 0"><xsl:value-of select="'????'"/></xsl:when>
	    	<xsl:when test="($tsio_state/@val4 mod ($bit8* 2)) - ($tsio_state/@val4 mod $bit8)"><xsl:value-of select="'TRUE'"/></xsl:when>
	      <xsl:otherwise><xsl:value-of select="'FALSE'"/></xsl:otherwise>
	    </xsl:choose>
		</xsl:variable>
		<!--TSIO4 Output2 state-->
		<xsl:variable name="tsio4_output2_state">
	    <xsl:choose>
	    	<xsl:when test="($tsio_com_err mod ($bit3 * 2)) - ($tsio_com_err mod $bit3)= 0"><xsl:value-of select="'????'"/></xsl:when>
	    	<xsl:when test="($tsio_state/@val4 mod ($bit9 * 2)) - ($tsio_state/@val4 mod $bit9)"><xsl:value-of select="'TRUE'"/></xsl:when>
	      <xsl:otherwise><xsl:value-of select="'FALSE'"/></xsl:otherwise>
	    </xsl:choose>
		</xsl:variable>
		<!--TSIO4 Output3 state-->
		<xsl:variable name="tsio4_output3_state">
	    <xsl:choose>
	    	<xsl:when test="($tsio_com_err mod ($bit3 * 2)) - ($tsio_com_err mod $bit3)= 0"><xsl:value-of select="'????'"/></xsl:when>
	    	<xsl:when test="($tsio_state/@val4 mod ($bit10 * 2)) - ($tsio_state/@val4 mod $bit10)"><xsl:value-of select="'TRUE'"/></xsl:when>
	      <xsl:otherwise><xsl:value-of select="'FALSE'"/></xsl:otherwise>
	    </xsl:choose>
		</xsl:variable>
		<!--TSIO4 Output4 state-->
		<xsl:variable name="tsio4_output4_state">
	    <xsl:choose>
	    	<xsl:when test="($tsio_com_err mod ($bit3 * 2)) - ($tsio_com_err mod $bit3)= 0"><xsl:value-of select="'????'"/></xsl:when>
	    	<xsl:when test="($tsio_state/@val4 mod ($bit11 * 2)) - ($tsio_state/@val4 mod $bit11)"><xsl:value-of select="'TRUE'"/></xsl:when>
	      <xsl:otherwise><xsl:value-of select="'FALSE'"/></xsl:otherwise>
	    </xsl:choose>
		</xsl:variable>

		<!--TSIO5 Output1 state-->
		<xsl:variable name="tsio5_output1_state">
	    <xsl:choose>
	    	<xsl:when test="($tsio_com_err mod ($bit4 * 2)) - ($tsio_com_err mod $bit4)= 0"><xsl:value-of select="'????'"/></xsl:when>
	    	<xsl:when test="($tsio_state/@val5 mod ($bit8 * 2)) - ($tsio_state/@val5 mod $bit8)"><xsl:value-of select="'TRUE'"/></xsl:when>
	      <xsl:otherwise><xsl:value-of select="'FALSE'"/></xsl:otherwise>
	    </xsl:choose>
		</xsl:variable>
		<!--TSIO5 Output2 state-->
		<xsl:variable name="tsio5_output2_state">
	    <xsl:choose>
	    	<xsl:when test="($tsio_com_err mod ($bit4 * 2)) - ($tsio_com_err mod $bit4)= 0"><xsl:value-of select="'????'"/></xsl:when>
	    	<xsl:when test="($tsio_state/@val5 mod ($bit9 * 2)) - ($tsio_state/@val5 mod $bit9)"><xsl:value-of select="'TRUE'"/></xsl:when>
	      <xsl:otherwise><xsl:value-of select="'FALSE'"/></xsl:otherwise>
	    </xsl:choose>
		</xsl:variable>


		<!--TSIO6 Output1 state-->
		<xsl:variable name="tsio6_output1_state">
	    <xsl:choose>
	    	<xsl:when test="($tsio_com_err mod ($bit5 * 2)) - ($tsio_com_err mod $bit5)= 0"><xsl:value-of select="'????'"/></xsl:when>
	    	<xsl:when test="($tsio_state/@val6 mod ($bit8 * 2)) - ($tsio_state/@val6 mod $bit8)"><xsl:value-of select="'TRUE'"/></xsl:when>
	      <xsl:otherwise><xsl:value-of select="'FALSE'"/></xsl:otherwise>
	    </xsl:choose>
		</xsl:variable>
		<!--TSIO6 Output2 state-->
		<xsl:variable name="tsio6_output2_state">
	    <xsl:choose>
	    	<xsl:when test="($tsio_com_err mod ($bit5 * 2)) - ($tsio_com_err mod $bit5)= 0"><xsl:value-of select="'????'"/></xsl:when>
	    	<xsl:when test="($tsio_state/@val6 mod ($bit9 * 2)) - ($tsio_state/@val6 mod $bit9)"><xsl:value-of select="'TRUE'"/></xsl:when>
	      <xsl:otherwise><xsl:value-of select="'FALSE'"/></xsl:otherwise>
	    </xsl:choose>
		</xsl:variable>

  	<div id="{$id}" onmousedown="dragstart(this)" onclick="bring_to_front('{$id}')" class="XBold_black" align="left" style="position:absolute; left:{$x}px; top:{$y}px; width:{$w}px; height:{$h}px;text-align:left;visibility:{$visibility};z-index:1;">
  		<table cellspacing="0" cellpadding="0" border="0"  class="XBold_black" style="position:absolute; width:{$w}px; height:20px;color:white;background-color:#800000;">
  			<thead>
    			<tr>
      			<th align="left"><xsl:value-of select="concat('&#160;&#160;',$title)"/></th>
      			<th><xsl:value-of select="'&#160;'"/></th>
      			<th align="right">
							<a href="javascript:show('{$id}');" class="btn">
								<span><span  style="color:white; background-color:#800000;"><xsl:value-of select="'X'"/></span></span>
							</a>
						</th>
    			</tr>
  			</thead>
			</table><!--border 1px solid black;border-collapse:collapse;-->

  		<table border="1" cellspacing="0" cellpadding="0" class="{$class}" style="position:absolute; top:{$y - 60}px; width:{$w}px;color:black;background-color:white;text-align:center;">
				<tbody>
  			<tr>
					<td class="XBold_black"><xsl:value-of select="$txt1"/></td>
					<td class="XBold_black"><xsl:value-of select="$txt2"/></td>
					<td class="XBold_black"><xsl:value-of select="$txt3"/></td>
				</tr>
  			<tr>
					<td><xsl:value-of select="'T3S&#160;Output1'"/></td>
					<td><xsl:value-of select="value[@name='CET2_LBLT3SOUT1']/@val"/></td>
					<td><xsl:value-of select="$t3s_output1_state"/></td>
				</tr>
  			<tr>
					<td><xsl:value-of select="'T3S&#160;Output2'"/></td>
					<td><xsl:value-of select="value[@name='CET2_LBLT3SOUT2']/@val"/></td>
					<td><xsl:value-of select="$t3s_output2_state"/></td>
				</tr>
  			<tr>
					<td><xsl:value-of select="'T3S&#160;Output3'"/></td>
					<td><xsl:value-of select="value[@name='CET2_LBLT3SOUT3']/@val"/></td>
					<td><xsl:value-of select="$t3s_output3_state"/></td>
				</tr>


				<xsl:if test="$num_tsio > 0">
	  			<tr>
						<td><xsl:value-of select="'TSIO1&#160;Output1'"/></td>
						<td><xsl:value-of select="value[@name='CET2_LBLTSIOO11']/@val"/></td>
						<td><xsl:value-of select="$tsio1_output1_state"/></td>
					</tr>
	  			<tr>
						<td><xsl:value-of select="'TSIO1&#160;Output2'"/></td>
						<td><xsl:value-of select="value[@name='CET2_LBLTSIOO12']/@val"/></td>
						<td><xsl:value-of select="$tsio1_output2_state"/></td>
					</tr>
	  			<tr>
						<td><xsl:value-of select="'TSIO1&#160;Output3'"/></td>
						<td><xsl:value-of select="value[@name='CET2_LBLTSIOO13']/@val"/></td>
						<td><xsl:value-of select="$tsio1_output3_state"/></td>
					</tr>
	  			<tr>
						<td><xsl:value-of select="'TSIO1&#160;Output4'"/></td>
						<td><xsl:value-of select="value[@name='CET2_LBLTSIOO14']/@val"/></td>
						<td><xsl:value-of select="$tsio1_output4_state"/></td>
					</tr>
				</xsl:if>

				<xsl:if test="$num_tsio > 1">
	  			<tr>
						<td><xsl:value-of select="'TSIO2&#160;Output1'"/></td>
						<td><xsl:value-of select="value[@name='CET2_LBLTSIOO21']/@val"/></td>
						<td><xsl:value-of select="$tsio2_output1_state"/></td>
					</tr>
	  			<tr>
						<td><xsl:value-of select="'TSIO2&#160;Output2'"/></td>
						<td><xsl:value-of select="value[@name='CET2_LBLTSIOO22']/@val"/></td>
						<td><xsl:value-of select="$tsio2_output2_state"/></td>
					</tr>
	  			<tr>
						<td><xsl:value-of select="'TSIO2&#160;Output3'"/></td>
						<td><xsl:value-of select="value[@name='CET2_LBLTSIOO23']/@val"/></td>
						<td><xsl:value-of select="$tsio2_output3_state"/></td>
					</tr>
	  			<tr>
						<td><xsl:value-of select="'TSIO2&#160;Output4'"/></td>
						<td><xsl:value-of select="value[@name='CET2_LBLTSIOO24']/@val"/></td>
						<td><xsl:value-of select="$tsio2_output4_state"/></td>
					</tr>
				</xsl:if>
			</tbody>
			</table>
		</div>
	</xsl:template>


	<!-- Commercial network  button-->
	<xsl:template name="commercial-network-button">
		<xsl:param name="x" select="86" />
	  <xsl:param name="y" select="258" />
	  <xsl:param name="w" select="158" />
	  <xsl:param name="h" select="104" />
	  <xsl:param name="align" select="'center'" />
	 	<xsl:param name="visibility" select="'visible'" />
	 	<xsl:param name="class" select="'tdSmall9'" />
		<xsl:param name="id" select="'commercial_network_button_div'" />

		<div id="{$id}" align="left" style="position:absolute; left:{$x}px; top:{$y}px; width:{$w}px; height:{$h}px;text-align:left;visibility:{$visibility};z-index:0;">
			<input onclick="javascript:show('commercial_network_div');" alt="Click to display Commercial network box" width="{$w}" height="{$h}" type="image" title="Click to display Commercial network box" src="./lib/ups/image/ce_t/arrow_empty.png" name="Bildbutton"/>
		</div>
	</xsl:template>

	<!-- commercial-network-pane-->
	<xsl:template name="commercial-network-pane">
		<xsl:param name="x" select="40" />
	  <xsl:param name="y" select="80" />
	  <xsl:param name="w" select="320" />
	  <xsl:param name="h" select="60" />
	  <xsl:param name="align" select="'center'" />
	 	<xsl:param name="visibility" select="'hidden'" />
	 	<xsl:param name="class" select="'XBold_black'" />
		<xsl:param name="id" select="'commercial_network_div'" />
		<xsl:param name="title1" select="'AC&#160;input&#160;A'" />
		<xsl:param name="title2" select="'AC&#160;input&#160;B'" />

		<xsl:param name="ac_input_conf" select="'0'" />
		<xsl:param name="ac_input_watt" select="'0'" />
		<xsl:param name="in_watt_precision" select="'1'" />
		<xsl:param name="ac_input_volt" select="'0'" />
		<xsl:param name="in_volt_precision" select="'0'" />
		<xsl:param name="ac_input_curr" select="'0'" />
		<xsl:param name="in_curr_precision" select="'1'" />
		<xsl:param name="ac_input_freq" select="'0'" />
		<xsl:param name="in_freq_precision" select="'1'" />

		<!--AC Input A variables Vac(V)-->
		<xsl:variable name="ac_input_a_vac_1">
			<xsl:call-template name="get-value">
				<xsl:with-param name="value" select="$ac_input_volt/@val"/>
				<xsl:with-param name="precision" 	select="$in_volt_precision"/>
			</xsl:call-template>
		</xsl:variable>
		<xsl:variable name="ac_input_a_vac_2">
			<xsl:call-template name="get-value">
				<xsl:with-param name="value" select="$ac_input_volt/@val2"/>
				<xsl:with-param name="precision" 	select="$in_volt_precision"/>
			</xsl:call-template>
		</xsl:variable>
		<xsl:variable name="ac_input_a_vac_3">
			<xsl:call-template name="get-value">
				<xsl:with-param name="value" select="$ac_input_volt/@val3"/>
				<xsl:with-param name="precision" 	select="$in_volt_precision"/>
			</xsl:call-template>
		</xsl:variable>
		<!--AC Input A variables P(KW)-->
		<xsl:variable name="ac_input_a_pkw_1">
			<xsl:call-template name="get-value">
				<xsl:with-param name="value" select="$ac_input_watt/@val div 1000"/>
				<xsl:with-param name="precision" 	select="$in_watt_precision"/>
			</xsl:call-template>
		</xsl:variable>
		<xsl:variable name="ac_input_a_pkw_2">
			<xsl:call-template name="get-value">
				<xsl:with-param name="value" select="$ac_input_watt/@val2 div 1000"/>
				<xsl:with-param name="precision" 	select="$in_watt_precision"/>
			</xsl:call-template>
		</xsl:variable>
		<xsl:variable name="ac_input_a_pkw_3">
			<xsl:call-template name="get-value">
				<xsl:with-param name="value" select="$ac_input_watt/@val3 div 1000"/>
				<xsl:with-param name="precision" 	select="$in_watt_precision"/>
			</xsl:call-template>
		</xsl:variable>
		<!--AC Input A variables I(A)-->
		<xsl:variable name="ac_input_a_ia_1">
			<xsl:call-template name="get-value">
				<xsl:with-param name="value" select="$ac_input_curr/@val"/>
				<xsl:with-param name="precision" 	select="$in_curr_precision"/>
			</xsl:call-template>
		</xsl:variable>
		<xsl:variable name="ac_input_a_ia_2">
			<xsl:call-template name="get-value">
				<xsl:with-param name="value" select="$ac_input_curr/@val2"/>
				<xsl:with-param name="precision" 	select="$in_curr_precision"/>
			</xsl:call-template>
		</xsl:variable>
		<xsl:variable name="ac_input_a_ia_3">
			<xsl:call-template name="get-value">
				<xsl:with-param name="value" select="$ac_input_curr/@val3"/>
				<xsl:with-param name="precision" 	select="$in_curr_precision"/>
			</xsl:call-template>
		</xsl:variable>
		<!--AC Input A variables F(Hz)-->
		<xsl:variable name="ac_input_a_fhz_1">
			<xsl:call-template name="get-value">
				<xsl:with-param name="value" select="$ac_input_freq/@val"/>
				<xsl:with-param name="precision" 	select="$in_freq_precision"/>
			</xsl:call-template>
		</xsl:variable>
		<xsl:variable name="ac_input_a_fhz_2">
			<xsl:call-template name="get-value">
				<xsl:with-param name="value" select="$ac_input_freq/@val2"/>
				<xsl:with-param name="precision" 	select="$in_freq_precision"/>
			</xsl:call-template>
		</xsl:variable>
		<xsl:variable name="ac_input_a_fhz_3">
			<xsl:call-template name="get-value">
				<xsl:with-param name="value" select="$ac_input_freq/@val3"/>
				<xsl:with-param name="precision" 	select="$in_freq_precision"/>
			</xsl:call-template>
		</xsl:variable>


		<!--AC Input B variables Vac(V)-->
		<xsl:variable name="ac_input_b_vac_1">
			<xsl:call-template name="get-value">
				<xsl:with-param name="value" select="$ac_input_volt/@val4"/>
				<xsl:with-param name="precision" 	select="$in_volt_precision"/>
			</xsl:call-template>
		</xsl:variable>
		<xsl:variable name="ac_input_b_vac_2">
			<xsl:call-template name="get-value">
				<xsl:with-param name="value" select="$ac_input_volt/@val5"/>
				<xsl:with-param name="precision" 	select="$in_volt_precision"/>
			</xsl:call-template>
		</xsl:variable>
		<xsl:variable name="ac_input_b_vac_3">
			<xsl:call-template name="get-value">
				<xsl:with-param name="value" select="$ac_input_volt/@val6"/>
				<xsl:with-param name="precision" 	select="$in_volt_precision"/>
			</xsl:call-template>
		</xsl:variable>
		<!--AC Input B variables P(KW)-->
		<xsl:variable name="ac_input_b_pkw_1">
			<xsl:call-template name="get-value">
				<xsl:with-param name="value" select="$ac_input_watt/@val4 div 1000"/>
				<xsl:with-param name="precision" 	select="$in_watt_precision"/>
			</xsl:call-template>
		</xsl:variable>
		<xsl:variable name="ac_input_b_pkw_2">
			<xsl:call-template name="get-value">
				<xsl:with-param name="value" select="$ac_input_watt/@val5 div 1000"/>
				<xsl:with-param name="precision" 	select="$in_watt_precision"/>
			</xsl:call-template>
		</xsl:variable>
		<xsl:variable name="ac_input_b_pkw_3">
			<xsl:call-template name="get-value">
				<xsl:with-param name="value" select="$ac_input_watt/@val6 div 1000"/>
				<xsl:with-param name="precision" 	select="$in_watt_precision"/>
			</xsl:call-template>
		</xsl:variable>
		<!--AC Input B variables I(A)-->
		<xsl:variable name="ac_input_b_ia_1">
			<xsl:call-template name="get-value">
				<xsl:with-param name="value" select="$ac_input_curr/@val4"/>
				<xsl:with-param name="precision" 	select="$in_curr_precision"/>
			</xsl:call-template>
		</xsl:variable>
		<xsl:variable name="ac_input_b_ia_2">
			<xsl:call-template name="get-value">
				<xsl:with-param name="value" select="$ac_input_curr/@val5"/>
				<xsl:with-param name="precision" 	select="$in_curr_precision"/>
			</xsl:call-template>
		</xsl:variable>
		<xsl:variable name="ac_input_b_ia_3">
			<xsl:call-template name="get-value">
				<xsl:with-param name="value" select="$ac_input_curr/@val6"/>
				<xsl:with-param name="precision" 	select="$in_curr_precision"/>
			</xsl:call-template>
		</xsl:variable>
		<!--AC Input B variables F(Hz)-->
		<xsl:variable name="ac_input_b_fhz_1">
			<xsl:call-template name="get-value">
				<xsl:with-param name="value" select="$ac_input_freq/@val4"/>
				<xsl:with-param name="precision" 	select="$in_freq_precision"/>
			</xsl:call-template>
		</xsl:variable>
		<xsl:variable name="ac_input_b_fhz_2">
			<xsl:call-template name="get-value">
				<xsl:with-param name="value" select="$ac_input_freq/@val5"/>
				<xsl:with-param name="precision" 	select="$in_freq_precision"/>
			</xsl:call-template>
		</xsl:variable>
		<xsl:variable name="ac_input_b_fhz_3">
			<xsl:call-template name="get-value">
				<xsl:with-param name="value" select="$ac_input_freq/@val6"/>
				<xsl:with-param name="precision" 	select="$in_freq_precision"/>
			</xsl:call-template>
		</xsl:variable>

  	<div id="{$id}" onmousedown="dragstart(this)" onclick="bring_to_front('{$id}')" class="XBold_black" align="left" style="position:absolute; left:{$x}px; top:{$y}px; width:{$w}px; height:{$h}px;text-align:left;visibility:{$visibility};z-index:1;">
	  	<xsl:if test="$ac_input_conf > 0">
				<table cellspacing="0" cellpadding="0" border="0"  class="XBold_black" style="position:absolute; width:{$w}px; height:20px;color:white;background-color:#800000;">
		  		<thead>
		    			<tr>
		      			<th align="left"><xsl:value-of select="concat('&#160;&#160;',$title1)"/></th>
		      			<th><xsl:value-of select="'&#160;'"/></th>
		      			<th align="right">
									<a href="javascript:show('{$id}');" class="btn">
										<span><span  style="color:white; background-color:#800000;"><xsl:value-of select="'X'"/></span></span>
									</a>
								</th>
		    			</tr>
		  			</thead>
				</table>

		  	<table border="1" cellspacing="0" cellpadding="0" class="{$class}" style="position:absolute; top:{$y - 60}px; width:{$w}px;color:black;background-color:white;text-align:center;">
					<tbody>
			  		<tr>
							<td><xsl:value-of select="'Phase'"/></td>
							<td><xsl:value-of select="'Vac(V)'"/></td>
							<td><xsl:value-of select="'P(kW)'"/></td>
							<td><xsl:value-of select="'I(A)'"/></td>
							<td><xsl:value-of select="'F(Hz)'"/></td>
							</tr>
							<tr>
								<td><xsl:value-of select="'R&#160;(L1)'"/></td>
								<td><xsl:value-of select="$ac_input_a_vac_1"/></td>
								<td><xsl:value-of select="$ac_input_a_pkw_1"/></td>
								<td><xsl:value-of select="$ac_input_a_ia_1"/></td>
								<td><xsl:value-of select="$ac_input_a_fhz_1"/></td>
							</tr>
							<xsl:if test="$ac_input_conf=3 or $ac_input_conf=4">
								<tr>
									<td><xsl:value-of select="'S&#160;(L2)'"/></td>
									<td><xsl:value-of select="$ac_input_a_vac_2"/></td>
									<td><xsl:value-of select="$ac_input_a_pkw_2"/></td>
									<td><xsl:value-of select="$ac_input_a_ia_2"/></td>
									<td><xsl:value-of select="$ac_input_a_fhz_2"/></td>
								</tr>
								<tr>
									<td><xsl:value-of select="'T&#160;(L3)'"/></td>
										<td><xsl:value-of select="$ac_input_a_vac_3"/></td>
										<td><xsl:value-of select="$ac_input_a_pkw_3"/></td>
										<td><xsl:value-of select="$ac_input_a_ia_3"/></td>
										<td><xsl:value-of select="$ac_input_a_fhz_3"/></td>
								</tr>
							</xsl:if>
						</tbody>
					</table>
		  		<xsl:if test="$ac_input_conf=4 or $ac_input_conf=2">
						<table cellspacing="0" cellpadding="0" border="0"  class="XBold_black" style="position:absolute; left:{$x - 40}px; top:{$y + 6}px; width:{$w}px; height:20px;color:white;background-color:#800000;">
				  		<thead>
				    			<tr>
				      			<th align="left"><xsl:value-of select="concat('&#160;&#160;',$title2)"/></th>
				      			<th><xsl:value-of select="'&#160;'"/></th>
				    			</tr>
				  			</thead>
							</table>
					  	<table border="1" cellspacing="0" cellpadding="0" class="{$class}" style="position:absolute; left:{$x - 40}px; top:{$y + 22}px; width:{$w}px;color:black;background-color:white;text-align:center;">
								<tbody>
						  		<tr>
										<td><xsl:value-of select="'Phase'"/></td>
										<td><xsl:value-of select="'Vac(V)'"/></td>
										<td><xsl:value-of select="'P(kW)'"/></td>
										<td><xsl:value-of select="'I(A)'"/></td>
										<td><xsl:value-of select="'F(Hz)'"/></td>
										</tr>
										<tr>
											<td><xsl:value-of select="'R&#160;(L1)'"/></td>
											<td><xsl:value-of select="$ac_input_b_vac_1"/></td>
											<td><xsl:value-of select="$ac_input_b_pkw_1"/></td>
											<td><xsl:value-of select="$ac_input_b_ia_1"/></td>
											<td><xsl:value-of select="$ac_input_b_fhz_1"/></td>
										</tr>
										<xsl:if test="$ac_input_conf=4">
											<tr>
												<td><xsl:value-of select="'S&#160;(L2)'"/></td>
												<td><xsl:value-of select="$ac_input_b_vac_2"/></td>
												<td><xsl:value-of select="$ac_input_b_pkw_2"/></td>
												<td><xsl:value-of select="$ac_input_b_ia_2"/></td>
												<td><xsl:value-of select="$ac_input_b_fhz_2"/></td>
											</tr>
											<tr>
												<td><xsl:value-of select="'T&#160;(L3)'"/></td>
												<td><xsl:value-of select="$ac_input_b_vac_3"/></td>
												<td><xsl:value-of select="$ac_input_b_pkw_3"/></td>
												<td><xsl:value-of select="$ac_input_b_ia_3"/></td>
												<td><xsl:value-of select="$ac_input_b_fhz_3"/></td>
											</tr>
										</xsl:if>
									</tbody>
								</table>
							</xsl:if>
						</xsl:if>
					</div>
			</xsl:template>

	<!-- Battery  button-->
	<xsl:template name="battery-button">
		<xsl:param name="x" select="328" />
	  <xsl:param name="y" select="460" />
	  <xsl:param name="w" select="160" />
	  <xsl:param name="h" select="104" />
	  <xsl:param name="align" select="'center'" />
	 	<xsl:param name="visibility" select="'visible'" />
	 	<xsl:param name="class" select="'tdSmall9'" />
		<xsl:param name="id" select="'battery_button_div'" />

		<div id="{$id}" align="left" style="position:absolute; left:{$x}px; top:{$y}px; width:{$w}px; height:{$h}px;text-align:left;visibility:{$visibility};z-index:0;">
			<input onclick="javascript:show('battery_div');" alt="Click to display Battery box" title="Click to display Battery box" width="{$w}" height="{$h}" type="image" src="./lib/ups/image/ce_t/arrow_empty.png" name="Bildbutton"/>
		</div>
	</xsl:template>


	<!-- CET Battery mode-->
	<xsl:template name="get-batt-mode">
		<xsl:param name="dc_out_mod" select="'0'" />
	    <xsl:choose>
	    	<xsl:when test="$dc_out_mod=1"><xsl:value-of select="'Charge&#160;mode&#160;is&#160;Off'"/></xsl:when>
	    	<xsl:when test="$dc_out_mod=2"><xsl:value-of select="'Charge&#160;mode&#160;is&#160;Safe'"/></xsl:when>
	    	<xsl:when test="$dc_out_mod=3"><xsl:value-of select="'Charge&#160;mode&#160;is&#160;Float'"/></xsl:when>
	    	<xsl:when test="$dc_out_mod=4"><xsl:value-of select="'Charge&#160;mode&#160;is&#160;Boost'"/></xsl:when>
	    	<xsl:when test="$dc_out_mod=5"><xsl:value-of select="'Charge&#160;mode&#160;is&#160;Discharge'"/></xsl:when>
	    	<xsl:when test="$dc_out_mod=6"><xsl:value-of select="'Charge&#160;mode&#160;is&#160;Test'"/></xsl:when>
	      <xsl:otherwise>
					<xsl:value-of select="'Charge&#160;mode&#160;is&#160;not&#160;known'"/>
	      </xsl:otherwise>
	    </xsl:choose>
	</xsl:template>

	<!-- CET Battery pane-->
	<xsl:template name="battery-pane">
		<xsl:param name="x" select="20" />
	  <xsl:param name="y" select="80" />
	  <xsl:param name="w" select="460" />
	  <xsl:param name="h" select="100" />
	  <xsl:param name="align" select="'center'" />
	 	<xsl:param name="visibility" select="'hidden'" />
	 	<xsl:param name="class" select="'XBold_black'" />
		<xsl:param name="id" select="'battery_div'" />
		<xsl:param name="title1" select="'Battery1'" />
		<xsl:param name="title2" select="'Battery2'" />
		<xsl:param name="dc_batt_conf" select="'0'" />
		<xsl:param name="dc_in_pow" select="'0'" />
		<xsl:param name="dc_in_volt" select="'0'" />
		<xsl:param name="dc_in_curr" select="'0'" />
		<xsl:param name="dc_in_temp" select="'0'" />
		<xsl:param name="dc_in_cap" select="'0'" />
		<xsl:param name="dc_out_mod" select="'0'" />

		<xsl:param name="dc_in_pow_precision" select="'1'" />
		<xsl:param name="dc_in_volt_precision" select="'0'" />
		<xsl:param name="dc_in_curr_precision" select="'1'" />
		<xsl:param name="dc_in_temp_precision" select="'1'" />
		<xsl:param name="dc_in_cap_precision" select="'0'" />

		<!--Battery 1 variables-->
		<xsl:variable name="batt_1_dc_in_volt">
			<xsl:call-template name="get-value">
				<xsl:with-param name="value" select="$dc_in_volt/@val"/>
				<xsl:with-param name="precision" 	select="$dc_in_volt_precision"/>
			</xsl:call-template>
		</xsl:variable>
		<xsl:variable name="batt_1_dc_in_curr">
			<xsl:call-template name="get-value">
				<xsl:with-param name="value" select="$dc_in_curr/@val"/>
				<xsl:with-param name="precision" 	select="$dc_in_curr_precision"/>
			</xsl:call-template>
		</xsl:variable>
		<xsl:variable name="batt_1_dc_in_pow">
			<xsl:call-template name="get-value">
				<xsl:with-param name="value" select="$dc_in_pow/@val div 1000"/>
				<xsl:with-param name="precision" 	select="$dc_in_pow_precision"/>
			</xsl:call-template>
		</xsl:variable>
		<xsl:variable name="batt_1_dc_in_cap">
			<xsl:call-template name="get-value">
				<xsl:with-param name="value" select="$dc_in_cap/@val"/>
				<xsl:with-param name="precision" 	select="$dc_in_cap_precision"/>
			</xsl:call-template>
		</xsl:variable>

		<!--Battery 2 variables-->
		<xsl:variable name="batt_2_dc_in_volt">
			<xsl:call-template name="get-value">
				<xsl:with-param name="value" select="$dc_in_volt/@val2"/>
				<xsl:with-param name="precision" 	select="$dc_in_volt_precision"/>
			</xsl:call-template>
		</xsl:variable>
		<xsl:variable name="batt_2_dc_in_curr">
			<xsl:call-template name="get-value">
				<xsl:with-param name="value" select="$dc_in_curr/@val2"/>
				<xsl:with-param name="precision" 	select="$dc_in_curr_precision"/>
			</xsl:call-template>
		</xsl:variable>
		<xsl:variable name="batt_2_dc_in_pow">
			<xsl:call-template name="get-value">
				<xsl:with-param name="value" select="$dc_in_pow/@val2 div 1000"/>
				<xsl:with-param name="precision" 	select="$dc_in_pow_precision"/>
			</xsl:call-template>
		</xsl:variable>
		<xsl:variable name="batt_2_dc_in_cap">
			<xsl:call-template name="get-value">
				<xsl:with-param name="value" select="$dc_in_cap/@val2"/>
				<xsl:with-param name="precision" 	select="$dc_in_cap_precision"/>
			</xsl:call-template>
		</xsl:variable>


		<!--Battery 1 mode variable-->
		<xsl:variable name="batt_1_mode">
			<xsl:call-template name="get-batt-mode">
				<xsl:with-param name="dc_out_mod" select="$dc_out_mod/@val"/>
			</xsl:call-template>
		</xsl:variable>
		<!--Battery 2 mode variable-->
		<xsl:variable name="batt_2_mode">
			<xsl:call-template name="get-batt-mode">
				<xsl:with-param name="dc_out_mod" select="$dc_out_mod/@val2"/>
			</xsl:call-template>
		</xsl:variable>

  	<div id="{$id}" onmousedown="dragstart(this)" onclick="bring_to_front('{$id}')" class="XBold_black" align="left" style="position:absolute; left:{$x}px; top:{$y}px; width:{$w}px; height:{$h}px;text-align:left;visibility:{$visibility};z-index:1;">
	  	<xsl:if test="$dc_batt_conf > 0">
	  		<table cellspacing="0" cellpadding="0" border="0"  class="XBold_black" style="position:absolute;width:{$w}px; height:20px;color:white;background-color:#800000;">
	  			<thead>
	    			<tr>
	      			<th align="left"><xsl:value-of select="concat('&#160;&#160;',$title1)"/></th>
	      			<th align="right">
								<a href="javascript:show('{$id}');" class="btn">
									<span><span  style="color:white; background-color:#800000;"><xsl:value-of select="'X'"/></span></span>
								</a>
							</th>
	    			</tr>
	  			</thead>
				</table>
		  	<table border="1" cellspacing="2" cellpadding="2" class="{$class}" style="position:absolute;top:{$y -60}px;width:{$w}px;color:black;background-color:white;text-align:center;">
					<tbody>
			  		<tr>
							<td><xsl:value-of select="'Vdc(V)'"/></td>
							<td><xsl:value-of select="'P(kW)'"/></td>
							<td><xsl:value-of select="'I(A)'"/></td>
                     <xsl:if test="value[@name='_tempUnit']/@val=1">
							   <td><xsl:value-of select="'T(&#176;F)'"/></td>
                     </xsl:if>
                     <xsl:if test="value[@name='_tempUnit']/@val=0 or string-length(value[@name='_tempUnit']/@val)=0">
							   <td><xsl:value-of select="'T(&#176;C)'"/></td>
                     </xsl:if>
							<td><xsl:value-of select="'Capacity(%)'"/></td>
							<td class="XBold_black"><xsl:value-of select="'Mode'"/></td>
						</tr>
			  		<tr>
							<td><xsl:value-of select="$batt_1_dc_in_volt"/></td>
							<td><xsl:value-of select="$batt_1_dc_in_pow"/></td>
							<td><xsl:value-of select="$batt_1_dc_in_curr"/></td>
							<td>
					  		<xsl:choose>
					  		  <xsl:when test="$dc_in_temp/@val &lt; -200">
					  		  	<xsl:value-of select="'---'"/>
									</xsl:when>
									<xsl:otherwise>
                              <xsl:call-template name="get-CET-temperature">
                                 <xsl:with-param name="value" select="$dc_in_temp/@val" />
                                 <xsl:with-param name="precision" select="$dc_in_temp_precision"/>
                              </xsl:call-template>
						    	</xsl:otherwise>
						    </xsl:choose>
							</td>
							<td>
					  		<xsl:choose>
					  		  <xsl:when test="$batt_1_dc_in_cap = 0">
					  		  	<xsl:value-of select="'---'"/>
									</xsl:when>
									<xsl:otherwise>
										<xsl:value-of select="$batt_1_dc_in_cap"/>
						    	</xsl:otherwise>
						    </xsl:choose>
							</td>
							<td><xsl:value-of select="$batt_1_mode"/></td>
						</tr>
					</tbody>
				</table>
				<xsl:if test="$dc_batt_conf > 1">
		  		<table cellspacing="0" cellpadding="0" border="0"  class="{$class}" style="position:absolute;top:{$y - 6}px; width:{$w}px; height:20px;color:white;background-color:#800000;">
		  			<thead>
		    			<tr>
		      			<th align="left"><xsl:value-of select="concat('&#160;&#160;',$title2)"/></th>
		    			</tr>
		  			</thead>
					</table>
			  	<table border="1" cellspacing="2" cellpadding="2" class="{$class}" style="position:absolute;top:{$y + 14 }px; width:{$w}px;color:black;background-color:white;text-align:center;">
						<tbody>
				  		<tr>
								<td><xsl:value-of select="'Vdc(V)'"/></td>
								<td><xsl:value-of select="'P(kW)'"/></td>
								<td><xsl:value-of select="'I(A)'"/></td>
                     <xsl:if test="value[@name='_tempUnit']/@val=1">
							   <td><xsl:value-of select="'T(&#176;F)'"/></td>
                     </xsl:if>
                     <xsl:if test="value[@name='_tempUnit']/@val=0 or string-length(value[@name='_tempUnit']/@val)=0">
							   <td><xsl:value-of select="'T(&#176;C)'"/></td>
                     </xsl:if>
								<td><xsl:value-of select="'Capacity(%)'"/></td>
								<td><xsl:value-of select="'Mode'"/></td>
							</tr>
				  		<tr>
								<td><xsl:value-of select="$batt_2_dc_in_volt"/></td>
								<td><xsl:value-of select="$batt_2_dc_in_pow"/></td>
								<td><xsl:value-of select="$batt_2_dc_in_curr"/></td>
							<td>
					  		<xsl:choose>
					  		  <xsl:when test="$dc_in_temp/@val2 &lt; -200">
					  		  	   <xsl:value-of select="'---'"/>
								</xsl:when>
                           <xsl:otherwise>
                              <xsl:call-template name="get-CET-temperature">
                                 <xsl:with-param name="value" select="$dc_in_temp/@val2" />
                                 <xsl:with-param name="precision" select="$dc_in_temp_precision"/>
                              </xsl:call-template>
                           </xsl:otherwise>
						    </xsl:choose>
							</td>
							<td>
					  		<xsl:choose>
					  		  <xsl:when test="$batt_2_dc_in_cap = 0">
					  		  	<xsl:value-of select="'---'"/>
									</xsl:when>
									<xsl:otherwise>
										<xsl:value-of select="$batt_2_dc_in_cap"/>
						    	</xsl:otherwise>
						    </xsl:choose>
							</td>
							<td><xsl:value-of select="$batt_2_mode"/></td>
						</tr>
					</tbody>
				</table>
			</xsl:if>
		</xsl:if>
	</div>
	</xsl:template>

	<!-- Load  button-->
	<xsl:template name="load-button">
		<xsl:param name="x" select="574" />
	  <xsl:param name="y" select="256" />
	  <xsl:param name="w" select="160" />
	  <xsl:param name="h" select="104" />
	  <xsl:param name="align" select="'center'" />
	 	<xsl:param name="visibility" select="'visible'" />
	 	<xsl:param name="class" select="'XBold_black'" />
		<xsl:param name="id" select="'load_button_div'" />

		<div id="{$id}" align="left" style="position:absolute; left:{$x}px; top:{$y}px; width:{$w}px; height:{$h}px;text-align:left;visibility:{$visibility};z-index:0;">
			<input onclick="javascript:show('load_div')" alt="Click to display Load box" title="Click to display Load box" width="{$w}" height="{$h}" type="image" src="./lib/ups/image/ce_t/arrow_empty.png" name="Bildbutton"/>
		</div>
	</xsl:template>

	<!-- CET Load pane-->
	<xsl:template name="load-pane">
		<xsl:param name="x" select="20" />
	  <xsl:param name="y" select="80" />
	  <xsl:param name="w" select="360" />
	  <xsl:param name="h" select="200" />
	  <xsl:param name="align" select="'center'" />
	 	<xsl:param name="visibility" select="'hidden'" />
	 	<xsl:param name="class" select="'XBold_black'" />
		<xsl:param name="id" select="'load_div'" />
		<xsl:param name="ac_out_conf" select="'0'" />
		<xsl:param name="ac_out_volt" select="'0'" />
		<xsl:param name="ac_out_pow" select="'0'" />
		<xsl:param name="ac_out_curr" select="'0'" />
		<xsl:param name="ac_out_freq" select="'0'" />
		<xsl:param name="ac_out_pf" select="'0'" />
		<xsl:param name="ac_out_volt_precision" select="'0'" />
		<xsl:param name="ac_out_pow_precision" select="'1'" />
		<xsl:param name="ac_out_curr_precision" select="'1'" />
		<xsl:param name="ac_out_freq_precision" select="'1'" />
		<xsl:param name="ac_out_pf_precision" select="'2'" />


		<!--AC output volt variables Vac(V)-->
		<xsl:variable name="ac_out_volt_1">
			<xsl:call-template name="get-value">
				<xsl:with-param name="value" select="$ac_out_volt/@val"/>
				<xsl:with-param name="precision" 	select="$ac_out_volt_precision"/>
			</xsl:call-template>
		</xsl:variable>
		<xsl:variable name="ac_out_volt_2">
			<xsl:call-template name="get-value">
				<xsl:with-param name="value" select="$ac_out_volt/@val2"/>
				<xsl:with-param name="precision" 	select="$ac_out_volt_precision"/>
			</xsl:call-template>
		</xsl:variable>
		<xsl:variable name="ac_out_volt_3">
			<xsl:call-template name="get-value">
				<xsl:with-param name="value" select="$ac_out_volt/@val3"/>
				<xsl:with-param name="precision" 	select="$ac_out_volt_precision"/>
			</xsl:call-template>
		</xsl:variable>

		<!--AC output power variables P(KW)-->
		<xsl:variable name="ac_out_pow_1">
			<xsl:call-template name="get-value">
				<xsl:with-param name="value" select="$ac_out_pow/@val div 1000"/>
				<xsl:with-param name="precision" 	select="$ac_out_pow_precision"/>
			</xsl:call-template>
		</xsl:variable>
		<xsl:variable name="ac_out_pow_2">
			<xsl:call-template name="get-value">
				<xsl:with-param name="value" select="$ac_out_pow/@val2 div 1000"/>
				<xsl:with-param name="precision" 	select="$ac_out_pow_precision"/>
			</xsl:call-template>
		</xsl:variable>
		<xsl:variable name="ac_out_pow_3">
			<xsl:call-template name="get-value">
				<xsl:with-param name="value" select="$ac_out_pow/@val3 div 1000"/>
				<xsl:with-param name="precision" 	select="$ac_out_pow_precision"/>
			</xsl:call-template>
		</xsl:variable>

		<!--AC output power variables P(KVA)-->
		<xsl:variable name="ac_out_pow_kva_1">
			<xsl:call-template name="get-value">
				<xsl:with-param name="value" select="$ac_out_volt/@val * $ac_out_curr/@val div 1000"/>
				<xsl:with-param name="precision" 	select="$ac_out_pow_precision"/>
			</xsl:call-template>
		</xsl:variable>
		<xsl:variable name="ac_out_pow_kva_2">
			<xsl:call-template name="get-value">
				<xsl:with-param name="value" select="$ac_out_volt/@val2 * $ac_out_curr/@val2 div 1000"/>
				<xsl:with-param name="precision" 	select="$ac_out_pow_precision"/>
			</xsl:call-template>
		</xsl:variable>
		<xsl:variable name="ac_out_pow_kva_3">
			<xsl:call-template name="get-value">
				<xsl:with-param name="value" select="$ac_out_volt/@val3 * $ac_out_curr/@val3 div 1000"/>
				<xsl:with-param name="precision" 	select="$ac_out_pow_precision"/>
			</xsl:call-template>
		</xsl:variable>

		<!--AC output current variables I(A)-->
		<xsl:variable name="ac_out_curr_1">
			<xsl:call-template name="get-value">
				<xsl:with-param name="value" select="$ac_out_curr/@val"/>
				<xsl:with-param name="precision" 	select="$ac_out_curr_precision"/>
			</xsl:call-template>
		</xsl:variable>
		<xsl:variable name="ac_out_curr_2">
			<xsl:call-template name="get-value">
				<xsl:with-param name="value" select="$ac_out_curr/@val2"/>
				<xsl:with-param name="precision" 	select="$ac_out_curr_precision"/>
			</xsl:call-template>
		</xsl:variable>
		<xsl:variable name="ac_out_curr_3">
			<xsl:call-template name="get-value">
				<xsl:with-param name="value" select="$ac_out_curr/@val3"/>
				<xsl:with-param name="precision" 	select="$ac_out_curr_precision"/>
			</xsl:call-template>
		</xsl:variable>

		<!--AC output freq variables F(Hz)-->
		<xsl:variable name="ac_out_freq_1">
			<xsl:call-template name="get-value">
				<xsl:with-param name="value" select="$ac_out_freq/@val"/>
				<xsl:with-param name="precision" 	select="$ac_out_freq_precision"/>
			</xsl:call-template>
		</xsl:variable>
		<xsl:variable name="ac_out_freq_2">
			<xsl:call-template name="get-value">
				<xsl:with-param name="value" select="$ac_out_freq/@val2"/>
				<xsl:with-param name="precision" 	select="$ac_out_freq_precision"/>
			</xsl:call-template>
		</xsl:variable>
		<xsl:variable name="ac_out_freq_3">
			<xsl:call-template name="get-value">
				<xsl:with-param name="value" select="$ac_out_freq/@val3"/>
				<xsl:with-param name="precision" 	select="$ac_out_freq_precision"/>
			</xsl:call-template>
		</xsl:variable>

		<!--AC output pf variables-->
		<xsl:variable name="ac_out_pf_1">
			<xsl:call-template name="get-value">
				<xsl:with-param name="value" select="$ac_out_pf/@val"/>
				<xsl:with-param name="precision" 	select="$ac_out_pf_precision"/>
			</xsl:call-template>
		</xsl:variable>
		<xsl:variable name="ac_out_pf_2">
			<xsl:call-template name="get-value">
				<xsl:with-param name="value" select="$ac_out_pf/@val2"/>
				<xsl:with-param name="precision" 	select="$ac_out_pf_precision"/>
			</xsl:call-template>
		</xsl:variable>
		<xsl:variable name="ac_out_pf_3">
			<xsl:call-template name="get-value">
				<xsl:with-param name="value" select="$ac_out_pf/@val3"/>
				<xsl:with-param name="precision" 	select="$ac_out_pf_precision"/>
			</xsl:call-template>
		</xsl:variable>
		<xsl:if test="$ac_out_conf > 0">
	  	<div id="{$id}" onmousedown="dragstart(this)" onclick="bring_to_front('{$id}')" class="{$class}" align="left" style="position:absolute; left:{$x}px; top:{$y}px; width:{$w}px; height:{$h}px;text-align:left;visibility:{$visibility};z-index:1;">
				<table cellspacing="0" cellpadding="0" border="0"  class="{$class}" style="position:absolute; left:{$x}px; top:{$y}px; width:{$w}px; height:20px;color:white;background-color:#800000;">
					<thead>
		  			<tr>
		    			<th align="left"><xsl:value-of select="concat('&#160;&#160;','Load')"/></th>
		    			<th align="right">
								<a href="javascript:show('{$id}');" class="btn">
									<span><span  style="color:white; background-color:#800000;"><xsl:value-of select="'X'"/></span></span>
								</a>
							</th>
		  			</tr>
					</thead>
				</table>
		  	<table border="1" cellspacing="2" cellpadding="2" class="{$class}" style="position:absolute; left:{$x}px; top:{$y + 20}px; width:{$w}px;color:black;background-color:white;text-align:center;">
					<tbody>
			  		<tr>
							<td><xsl:value-of select="'Vac(V)'"/></td>
							<td><xsl:value-of select="'P(kW)'"/></td>
							<td><xsl:value-of select="'P(KVA)'"/></td>
							<td><xsl:value-of select="'I(A)'"/></td>
							<td><xsl:value-of select="'F(Hz)'"/></td>
							<td><xsl:value-of select="'PF'"/></td>
						</tr>
			  		<tr>
			  			<td><xsl:value-of select="$ac_out_volt_1"/></td>
							<td><xsl:value-of select="$ac_out_pow_1"/></td>
							<td><xsl:value-of select="$ac_out_pow_kva_1"/></td>
							<td><xsl:value-of select="$ac_out_curr_1"/></td>
							<td><xsl:value-of select="$ac_out_freq_1"/></td>
							<td><xsl:value-of select="$ac_out_pf_1"/></td>
						</tr>
						<xsl:if test="$ac_out_conf > 1">
				  		<tr>
				  			<td><xsl:value-of select="$ac_out_volt_2"/></td>
								<td><xsl:value-of select="$ac_out_pow_2"/></td>
								<td><xsl:value-of select="$ac_out_pow_kva_2"/></td>
								<td><xsl:value-of select="$ac_out_curr_2"/></td>
								<td><xsl:value-of select="$ac_out_freq_2"/></td>
								<td><xsl:value-of select="$ac_out_pf_2"/></td>
							</tr>
				  		<tr>
				  			<td><xsl:value-of select="$ac_out_volt_3"/></td>
								<td><xsl:value-of select="$ac_out_pow_3"/></td>
								<td><xsl:value-of select="$ac_out_pow_kva_3"/></td>
								<td><xsl:value-of select="$ac_out_curr_3"/></td>
								<td><xsl:value-of select="$ac_out_freq_3"/></td>
								<td><xsl:value-of select="$ac_out_pf_3"/></td>
							</tr>
					</xsl:if>
					</tbody>
				</table>
			</div>
		</xsl:if>
	</xsl:template>

	<!-- CET ACin connection string-->
	<xsl:template name="get-acin-connection-string">
		<xsl:param name="add_2509_val" select="'0'" />
		<xsl:param name="add_2510_val" select="'0'" />
		<xsl:param name="add_2511_val" select="'0'" />

    <xsl:choose>
    	<xsl:when test="$add_2509_val=1 and $add_2510_val=2 and $add_2511_val=3">
    		<xsl:value-of select="'A-3P'"/>
    	</xsl:when>
    	<xsl:when test="$add_2509_val=4 and $add_2510_val=5 and $add_2511_val=6">
    		<xsl:value-of select="'B-3P'"/>
    	</xsl:when>
    	<xsl:when test="$add_2509_val=1 and $add_2510_val=1 and $add_2511_val=1">
    		<xsl:value-of select="'A-1P'"/>
    	</xsl:when>
    	<xsl:when test="$add_2509_val=4 and $add_2510_val=4 and $add_2511_val=4">
    		<xsl:value-of select="'B-1P'"/>
    	</xsl:when>
      <xsl:otherwise><xsl:value-of select="'???'"/></xsl:otherwise>
    </xsl:choose>
	</xsl:template>

	<!-- CET ACout connection string-->
	<xsl:template name="get-acout-connection-string">
		<xsl:param name="add_2513_val" select="'0'" />
		<xsl:param name="add_2514_val" select="'0'" />
		<xsl:param name="add_2515_val" select="'0'" />

    <xsl:choose>
    	<xsl:when test="$add_2513_val=1 and $add_2514_val=2 and $add_2515_val=3">
    		<xsl:value-of select="'3P'"/>
    	</xsl:when>
    	<xsl:when test="$add_2513_val=1 and $add_2514_val=1 and $add_2515_val=1">
    		<xsl:value-of select="'1P'"/>
    	</xsl:when>
      <xsl:otherwise><xsl:value-of select="'???'"/></xsl:otherwise>
    </xsl:choose>
	</xsl:template>


	<!-- CET module status string-->
	<xsl:template name="get-mod-status-string">
		<xsl:param name="mod_status" select="'0'" />

    <xsl:choose>
    	<xsl:when test="$mod_status=1">
    		<xsl:value-of select="'OK'"/>
    	</xsl:when>
    	<xsl:when test="$mod_status=2">
    		<xsl:value-of select="'ALARM'"/>
    	</xsl:when>
    	<xsl:when test="$mod_status=3">
    		<xsl:value-of select="'ERROR'"/>
    	</xsl:when>
    	<xsl:when test="$mod_status=4">
    		<xsl:value-of select="'OFF'"/>
    	</xsl:when>
      <xsl:otherwise><xsl:value-of select="'UNKNOWN'"/></xsl:otherwise>
    </xsl:choose>
	</xsl:template>

	<!-- CET module status color-->
	<xsl:template name="get-mod-status-color">
		<xsl:param name="mod_status" select="'0'" />

    <xsl:choose>
    	<xsl:when test="$mod_status=1">
    		<xsl:value-of select="'#00FF00'"/>
    	</xsl:when>
    	<xsl:when test="$mod_status=2">
    		<xsl:value-of select="'#FFFF00'"/>
    	</xsl:when>
    	<xsl:when test="$mod_status=3">
    		<xsl:value-of select="'#FF0000'"/>
    	</xsl:when>
    	<xsl:when test="$mod_status=4">
    		<xsl:value-of select="'#D9D9D9'"/>
    	</xsl:when>
      <xsl:otherwise><xsl:value-of select="'#FFFFFF'"/></xsl:otherwise>
    </xsl:choose>
	</xsl:template>

	<!-- Modules  button-->
	<xsl:template name="modules-button">
		<xsl:param name="x" select="314" />
	  <xsl:param name="y" select="228" />
	  <xsl:param name="w" select="190" />
	  <xsl:param name="h" select="162" />
	  <xsl:param name="align" select="'center'" />
	 	<xsl:param name="visibility" select="'visible'" />
	 	<xsl:param name="class" select="'tdSmall9'" />
		<xsl:param name="id" select="'modules_button_div'" />

		<div id="{$id}" align="left" style="position:absolute; left:{$x}px; top:{$y}px; width:{$w}px; height:{$h}px;text-align:left;visibility:{$visibility};z-index:0;">
			<input onclick="javascript:show('modules_div');" alt="Click to display Modules box" title="Click to display Modules box" width="{$w}" height="{$h}" type="image" src="./lib/ups/image/ce_t/arrow_empty.png" name="Bildbutton"/>
		</div>
	</xsl:template>

	<!-- CET Modules pane-->
	<xsl:template name="modules-pane">
		<xsl:param name="x" select="10" />
	  <xsl:param name="y" select="10" />
	  <xsl:param name="w" select="700" />
	  <xsl:param name="h" select="700" />
	  <xsl:param name="align" select="'center'" />
	 	<xsl:param name="visibility" select="'hidden'" />
	 	<xsl:param name="class" select="'XBold_black'" />
		<xsl:param name="id" select="'modules_div'" />
		<xsl:param name="mod_present" select="'0'" />
		<xsl:param name="serial_nm" select="'0'" />
		<xsl:param name="modinfo_gr_ac_ina" select="'0'" />
		<xsl:param name="modinfo_gr_ac_inb" select="'0'" />
		<xsl:param name="modinfo_gr_ac_inc" select="'0'" />
		<xsl:param name="modinfo_gr_ac_outa" select="'0'" />
		<xsl:param name="modinfo_gr_ac_outb" select="'0'" />
		<xsl:param name="modinfo_gr_ac_outc" select="'0'" />
		<xsl:param name="modinfo_pow_ac_ina" select="'0'" />
		<xsl:param name="modinfo_pow_ac_inb" select="'0'" />
		<xsl:param name="modinfo_pow_ac_inc" select="'0'" />
		<xsl:param name="modinfo_pow_ac_outa" select="'0'" />
		<xsl:param name="modinfo_pow_ac_outb" select="'0'" />
		<xsl:param name="modinfo_pow_ac_outc" select="'0'" />
		<xsl:param name="modinfo_pow_va_ac_outa" select="'0'" />
		<xsl:param name="modinfo_pow_va_ac_outb" select="'0'" />
		<xsl:param name="modinfo_pow_va_ac_outc" select="'0'" />
		<xsl:param name="mod_power_dc" select="'0'" />
		<xsl:param name="mod_power_temp" select="'0'" />
		<xsl:param name="mod_status" select="'0'" />
		<xsl:param name="modinfo_gr_dc" select="'0'" />



		<!--acin connection string variables-->
		<xsl:variable name="ac_in_con_str_1">
			<xsl:call-template name="get-acin-connection-string">
				<xsl:with-param name="add_2509_val" select="$modinfo_gr_ac_ina/@val"/>
				<xsl:with-param name="add_2510_val" select="$modinfo_gr_ac_inb/@val"/>
				<xsl:with-param name="add_2511_val" select="$modinfo_gr_ac_inc/@val"/>
			</xsl:call-template>
		</xsl:variable>
		<xsl:variable name="ac_in_con_str_2">
			<xsl:call-template name="get-acin-connection-string">
				<xsl:with-param name="add_2509_val" select="$modinfo_gr_ac_ina/@val2"/>
				<xsl:with-param name="add_2510_val" select="$modinfo_gr_ac_inb/@val2"/>
				<xsl:with-param name="add_2511_val" select="$modinfo_gr_ac_inc/@val2"/>
			</xsl:call-template>
		</xsl:variable>
		<xsl:variable name="ac_in_con_str_3">
			<xsl:call-template name="get-acin-connection-string">
				<xsl:with-param name="add_2509_val" select="$modinfo_gr_ac_ina/@val3"/>
				<xsl:with-param name="add_2510_val" select="$modinfo_gr_ac_inb/@val3"/>
				<xsl:with-param name="add_2511_val" select="$modinfo_gr_ac_inc/@val3"/>
			</xsl:call-template>
		</xsl:variable>
		<xsl:variable name="ac_in_con_str_4">
			<xsl:call-template name="get-acin-connection-string">
				<xsl:with-param name="add_2509_val" select="$modinfo_gr_ac_ina/@val4"/>
				<xsl:with-param name="add_2510_val" select="$modinfo_gr_ac_inb/@val4"/>
				<xsl:with-param name="add_2511_val" select="$modinfo_gr_ac_inc/@val4"/>
			</xsl:call-template>
		</xsl:variable>

		<xsl:variable name="ac_in_con_str_5">
			<xsl:call-template name="get-acin-connection-string">
				<xsl:with-param name="add_2509_val" select="$modinfo_gr_ac_ina/@val5"/>
				<xsl:with-param name="add_2510_val" select="$modinfo_gr_ac_inb/@val5"/>
				<xsl:with-param name="add_2511_val" select="$modinfo_gr_ac_inc/@val5"/>
			</xsl:call-template>
		</xsl:variable>
		<xsl:variable name="ac_in_con_str_6">
			<xsl:call-template name="get-acin-connection-string">
				<xsl:with-param name="add_2509_val" select="$modinfo_gr_ac_ina/@val6"/>
				<xsl:with-param name="add_2510_val" select="$modinfo_gr_ac_inb/@val6"/>
				<xsl:with-param name="add_2511_val" select="$modinfo_gr_ac_inc/@val6"/>
			</xsl:call-template>
		</xsl:variable>
		<xsl:variable name="ac_in_con_str_7">
			<xsl:call-template name="get-acin-connection-string">
				<xsl:with-param name="add_2509_val" select="$modinfo_gr_ac_ina/@val7"/>
				<xsl:with-param name="add_2510_val" select="$modinfo_gr_ac_inb/@val7"/>
				<xsl:with-param name="add_2511_val" select="$modinfo_gr_ac_inc/@val7"/>
			</xsl:call-template>
		</xsl:variable>
		<xsl:variable name="ac_in_con_str_8">
			<xsl:call-template name="get-acin-connection-string">
				<xsl:with-param name="add_2509_val" select="$modinfo_gr_ac_ina/@val8"/>
				<xsl:with-param name="add_2510_val" select="$modinfo_gr_ac_inb/@val8"/>
				<xsl:with-param name="add_2511_val" select="$modinfo_gr_ac_inc/@val8"/>
			</xsl:call-template>
		</xsl:variable>
		<xsl:variable name="ac_in_con_str_9">
			<xsl:call-template name="get-acin-connection-string">
				<xsl:with-param name="add_2509_val" select="$modinfo_gr_ac_ina/@val9"/>
				<xsl:with-param name="add_2510_val" select="$modinfo_gr_ac_inb/@val9"/>
				<xsl:with-param name="add_2511_val" select="$modinfo_gr_ac_inc/@val9"/>
			</xsl:call-template>
		</xsl:variable>
		<xsl:variable name="ac_in_con_str_10">
			<xsl:call-template name="get-acin-connection-string">
				<xsl:with-param name="add_2509_val" select="$modinfo_gr_ac_ina/@val10"/>
				<xsl:with-param name="add_2510_val" select="$modinfo_gr_ac_inb/@val10"/>
				<xsl:with-param name="add_2511_val" select="$modinfo_gr_ac_inc/@val10"/>
			</xsl:call-template>
		</xsl:variable>
		<xsl:variable name="ac_in_con_str_11">
			<xsl:call-template name="get-acin-connection-string">
				<xsl:with-param name="add_2509_val" select="$modinfo_gr_ac_ina/@val11"/>
				<xsl:with-param name="add_2510_val" select="$modinfo_gr_ac_inb/@val11"/>
				<xsl:with-param name="add_2511_val" select="$modinfo_gr_ac_inc/@val11"/>
			</xsl:call-template>
		</xsl:variable>
		<xsl:variable name="ac_in_con_str_12">
			<xsl:call-template name="get-acin-connection-string">
				<xsl:with-param name="add_2509_val" select="$modinfo_gr_ac_ina/@val12"/>
				<xsl:with-param name="add_2510_val" select="$modinfo_gr_ac_inb/@val12"/>
				<xsl:with-param name="add_2511_val" select="$modinfo_gr_ac_inc/@val12"/>
			</xsl:call-template>
		</xsl:variable>
		<xsl:variable name="ac_in_con_str_13">
			<xsl:call-template name="get-acin-connection-string">
				<xsl:with-param name="add_2509_val" select="$modinfo_gr_ac_ina/@val13"/>
				<xsl:with-param name="add_2510_val" select="$modinfo_gr_ac_inb/@val13"/>
				<xsl:with-param name="add_2511_val" select="$modinfo_gr_ac_inc/@val13"/>
			</xsl:call-template>
		</xsl:variable>
		<xsl:variable name="ac_in_con_str_14">
			<xsl:call-template name="get-acin-connection-string">
				<xsl:with-param name="add_2509_val" select="$modinfo_gr_ac_ina/@val14"/>
				<xsl:with-param name="add_2510_val" select="$modinfo_gr_ac_inb/@val14"/>
				<xsl:with-param name="add_2511_val" select="$modinfo_gr_ac_inc/@val14"/>
			</xsl:call-template>
		</xsl:variable>
		<xsl:variable name="ac_in_con_str_15">
			<xsl:call-template name="get-acin-connection-string">
				<xsl:with-param name="add_2509_val" select="$modinfo_gr_ac_ina/@val15"/>
				<xsl:with-param name="add_2510_val" select="$modinfo_gr_ac_inb/@val15"/>
				<xsl:with-param name="add_2511_val" select="$modinfo_gr_ac_inc/@val15"/>
			</xsl:call-template>
		</xsl:variable>
		<xsl:variable name="ac_in_con_str_16">
			<xsl:call-template name="get-acin-connection-string">
				<xsl:with-param name="add_2509_val" select="$modinfo_gr_ac_ina/@val16"/>
				<xsl:with-param name="add_2510_val" select="$modinfo_gr_ac_inb/@val16"/>
				<xsl:with-param name="add_2511_val" select="$modinfo_gr_ac_inc/@val16"/>
			</xsl:call-template>
		</xsl:variable>
		<xsl:variable name="ac_in_con_str_17">
			<xsl:call-template name="get-acin-connection-string">
				<xsl:with-param name="add_2509_val" select="$modinfo_gr_ac_ina/@val17"/>
				<xsl:with-param name="add_2510_val" select="$modinfo_gr_ac_inb/@val17"/>
				<xsl:with-param name="add_2511_val" select="$modinfo_gr_ac_inc/@val17"/>
			</xsl:call-template>
		</xsl:variable>
		<xsl:variable name="ac_in_con_str_18">
			<xsl:call-template name="get-acin-connection-string">
				<xsl:with-param name="add_2509_val" select="$modinfo_gr_ac_ina/@val18"/>
				<xsl:with-param name="add_2510_val" select="$modinfo_gr_ac_inb/@val18"/>
				<xsl:with-param name="add_2511_val" select="$modinfo_gr_ac_inc/@val18"/>
			</xsl:call-template>
		</xsl:variable>
		<xsl:variable name="ac_in_con_str_19">
			<xsl:call-template name="get-acin-connection-string">
				<xsl:with-param name="add_2509_val" select="$modinfo_gr_ac_ina/@val19"/>
				<xsl:with-param name="add_2510_val" select="$modinfo_gr_ac_inb/@val19"/>
				<xsl:with-param name="add_2511_val" select="$modinfo_gr_ac_inc/@val19"/>
			</xsl:call-template>
		</xsl:variable>
		<xsl:variable name="ac_in_con_str_20">
			<xsl:call-template name="get-acin-connection-string">
				<xsl:with-param name="add_2509_val" select="$modinfo_gr_ac_ina/@val20"/>
				<xsl:with-param name="add_2510_val" select="$modinfo_gr_ac_inb/@val20"/>
				<xsl:with-param name="add_2511_val" select="$modinfo_gr_ac_inc/@val20"/>
			</xsl:call-template>
		</xsl:variable>
		<xsl:variable name="ac_in_con_str_21">
			<xsl:call-template name="get-acin-connection-string">
				<xsl:with-param name="add_2509_val" select="$modinfo_gr_ac_ina/@val21"/>
				<xsl:with-param name="add_2510_val" select="$modinfo_gr_ac_inb/@val21"/>
				<xsl:with-param name="add_2511_val" select="$modinfo_gr_ac_inc/@val21"/>
			</xsl:call-template>
		</xsl:variable>
		<xsl:variable name="ac_in_con_str_22">
			<xsl:call-template name="get-acin-connection-string">
				<xsl:with-param name="add_2509_val" select="$modinfo_gr_ac_ina/@val22"/>
				<xsl:with-param name="add_2510_val" select="$modinfo_gr_ac_inb/@val22"/>
				<xsl:with-param name="add_2511_val" select="$modinfo_gr_ac_inc/@val22"/>
			</xsl:call-template>
		</xsl:variable>
		<xsl:variable name="ac_in_con_str_23">
			<xsl:call-template name="get-acin-connection-string">
				<xsl:with-param name="add_2509_val" select="$modinfo_gr_ac_ina/@val23"/>
				<xsl:with-param name="add_2510_val" select="$modinfo_gr_ac_inb/@val23"/>
				<xsl:with-param name="add_2511_val" select="$modinfo_gr_ac_inc/@val23"/>
			</xsl:call-template>
		</xsl:variable>
		<xsl:variable name="ac_in_con_str_24">
			<xsl:call-template name="get-acin-connection-string">
				<xsl:with-param name="add_2509_val" select="$modinfo_gr_ac_ina/@val24"/>
				<xsl:with-param name="add_2510_val" select="$modinfo_gr_ac_inb/@val24"/>
				<xsl:with-param name="add_2511_val" select="$modinfo_gr_ac_inc/@val24"/>
			</xsl:call-template>
		</xsl:variable>
		<xsl:variable name="ac_in_con_str_25">
			<xsl:call-template name="get-acin-connection-string">
				<xsl:with-param name="add_2509_val" select="$modinfo_gr_ac_ina/@val25"/>
				<xsl:with-param name="add_2510_val" select="$modinfo_gr_ac_inb/@val25"/>
				<xsl:with-param name="add_2511_val" select="$modinfo_gr_ac_inc/@val25"/>
			</xsl:call-template>
		</xsl:variable>
		<xsl:variable name="ac_in_con_str_26">
			<xsl:call-template name="get-acin-connection-string">
				<xsl:with-param name="add_2509_val" select="$modinfo_gr_ac_ina/@val26"/>
				<xsl:with-param name="add_2510_val" select="$modinfo_gr_ac_inb/@val26"/>
				<xsl:with-param name="add_2511_val" select="$modinfo_gr_ac_inc/@val26"/>
			</xsl:call-template>
		</xsl:variable>
		<xsl:variable name="ac_in_con_str_27">
			<xsl:call-template name="get-acin-connection-string">
				<xsl:with-param name="add_2509_val" select="$modinfo_gr_ac_ina/@val27"/>
				<xsl:with-param name="add_2510_val" select="$modinfo_gr_ac_inb/@val27"/>
				<xsl:with-param name="add_2511_val" select="$modinfo_gr_ac_inc/@val27"/>
			</xsl:call-template>
		</xsl:variable>
		<xsl:variable name="ac_in_con_str_28">
			<xsl:call-template name="get-acin-connection-string">
				<xsl:with-param name="add_2509_val" select="$modinfo_gr_ac_ina/@val28"/>
				<xsl:with-param name="add_2510_val" select="$modinfo_gr_ac_inb/@val28"/>
				<xsl:with-param name="add_2511_val" select="$modinfo_gr_ac_inc/@val28"/>
			</xsl:call-template>
		</xsl:variable>
		<xsl:variable name="ac_in_con_str_29">
			<xsl:call-template name="get-acin-connection-string">
				<xsl:with-param name="add_2509_val" select="$modinfo_gr_ac_ina/@val29"/>
				<xsl:with-param name="add_2510_val" select="$modinfo_gr_ac_inb/@val29"/>
				<xsl:with-param name="add_2511_val" select="$modinfo_gr_ac_inc/@val29"/>
			</xsl:call-template>
		</xsl:variable>
		<xsl:variable name="ac_in_con_str_30">
			<xsl:call-template name="get-acin-connection-string">
				<xsl:with-param name="add_2509_val" select="$modinfo_gr_ac_ina/@val30"/>
				<xsl:with-param name="add_2510_val" select="$modinfo_gr_ac_inb/@val30"/>
				<xsl:with-param name="add_2511_val" select="$modinfo_gr_ac_inc/@val30"/>
			</xsl:call-template>
		</xsl:variable>
		<xsl:variable name="ac_in_con_str_31">
			<xsl:call-template name="get-acin-connection-string">
				<xsl:with-param name="add_2509_val" select="$modinfo_gr_ac_ina/@val31"/>
				<xsl:with-param name="add_2510_val" select="$modinfo_gr_ac_inb/@val31"/>
				<xsl:with-param name="add_2511_val" select="$modinfo_gr_ac_inc/@val31"/>
			</xsl:call-template>
		</xsl:variable>
		<xsl:variable name="ac_in_con_str_32">
			<xsl:call-template name="get-acin-connection-string">
				<xsl:with-param name="add_2509_val" select="$modinfo_gr_ac_ina/@val32"/>
				<xsl:with-param name="add_2510_val" select="$modinfo_gr_ac_inb/@val32"/>
				<xsl:with-param name="add_2511_val" select="$modinfo_gr_ac_inc/@val32"/>
			</xsl:call-template>
		</xsl:variable>

		<!--acout connection string variables-->
		<xsl:variable name="ac_out_con_str_1">
			<xsl:call-template name="get-acout-connection-string">
				<xsl:with-param name="add_2513_val" select="$modinfo_gr_ac_outa/@val"/>
				<xsl:with-param name="add_2514_val" select="$modinfo_gr_ac_outb/@val"/>
				<xsl:with-param name="add_2515_val" select="$modinfo_gr_ac_outc/@val"/>
			</xsl:call-template>
		</xsl:variable>
		<xsl:variable name="ac_out_con_str_2">
			<xsl:call-template name="get-acout-connection-string">
				<xsl:with-param name="add_2513_val" select="$modinfo_gr_ac_outa/@val2"/>
				<xsl:with-param name="add_2514_val" select="$modinfo_gr_ac_outb/@val2"/>
				<xsl:with-param name="add_2515_val" select="$modinfo_gr_ac_outc/@val2"/>
			</xsl:call-template>
		</xsl:variable>
		<xsl:variable name="ac_out_con_str_3">
			<xsl:call-template name="get-acout-connection-string">
				<xsl:with-param name="add_2513_val" select="$modinfo_gr_ac_outa/@val3"/>
				<xsl:with-param name="add_2514_val" select="$modinfo_gr_ac_outb/@val3"/>
				<xsl:with-param name="add_2515_val" select="$modinfo_gr_ac_outc/@val3"/>
			</xsl:call-template>
		</xsl:variable>
		<xsl:variable name="ac_out_con_str_4">
			<xsl:call-template name="get-acout-connection-string">
				<xsl:with-param name="add_2513_val" select="$modinfo_gr_ac_outa/@val4"/>
				<xsl:with-param name="add_2514_val" select="$modinfo_gr_ac_outb/@val4"/>
				<xsl:with-param name="add_2515_val" select="$modinfo_gr_ac_outc/@val4"/>
			</xsl:call-template>
		</xsl:variable>
		<xsl:variable name="ac_out_con_str_5">
			<xsl:call-template name="get-acout-connection-string">
				<xsl:with-param name="add_2513_val" select="$modinfo_gr_ac_outa/@val5"/>
				<xsl:with-param name="add_2514_val" select="$modinfo_gr_ac_outb/@val5"/>
				<xsl:with-param name="add_2515_val" select="$modinfo_gr_ac_outc/@val5"/>
			</xsl:call-template>
		</xsl:variable>
		<xsl:variable name="ac_out_con_str_6">
			<xsl:call-template name="get-acout-connection-string">
				<xsl:with-param name="add_2513_val" select="$modinfo_gr_ac_outa/@val6"/>
				<xsl:with-param name="add_2514_val" select="$modinfo_gr_ac_outb/@val6"/>
				<xsl:with-param name="add_2515_val" select="$modinfo_gr_ac_outc/@val6"/>
			</xsl:call-template>
		</xsl:variable>
		<xsl:variable name="ac_out_con_str_7">
			<xsl:call-template name="get-acout-connection-string">
				<xsl:with-param name="add_2513_val" select="$modinfo_gr_ac_outa/@val7"/>
				<xsl:with-param name="add_2514_val" select="$modinfo_gr_ac_outb/@val7"/>
				<xsl:with-param name="add_2515_val" select="$modinfo_gr_ac_outc/@val7"/>
			</xsl:call-template>
		</xsl:variable>
		<xsl:variable name="ac_out_con_str_8">
			<xsl:call-template name="get-acout-connection-string">
				<xsl:with-param name="add_2513_val" select="$modinfo_gr_ac_outa/@val8"/>
				<xsl:with-param name="add_2514_val" select="$modinfo_gr_ac_outb/@val8"/>
				<xsl:with-param name="add_2515_val" select="$modinfo_gr_ac_outc/@val8"/>
			</xsl:call-template>
		</xsl:variable>
		<xsl:variable name="ac_out_con_str_9">
			<xsl:call-template name="get-acout-connection-string">
				<xsl:with-param name="add_2513_val" select="$modinfo_gr_ac_outa/@val9"/>
				<xsl:with-param name="add_2514_val" select="$modinfo_gr_ac_outb/@val9"/>
				<xsl:with-param name="add_2515_val" select="$modinfo_gr_ac_outc/@val9"/>
			</xsl:call-template>
		</xsl:variable>
		<xsl:variable name="ac_out_con_str_10">
			<xsl:call-template name="get-acout-connection-string">
				<xsl:with-param name="add_2513_val" select="$modinfo_gr_ac_outa/@val10"/>
				<xsl:with-param name="add_2514_val" select="$modinfo_gr_ac_outb/@val10"/>
				<xsl:with-param name="add_2515_val" select="$modinfo_gr_ac_outc/@val10"/>
			</xsl:call-template>
		</xsl:variable>
		<xsl:variable name="ac_out_con_str_11">
			<xsl:call-template name="get-acout-connection-string">
				<xsl:with-param name="add_2513_val" select="$modinfo_gr_ac_outa/@val11"/>
				<xsl:with-param name="add_2514_val" select="$modinfo_gr_ac_outb/@val11"/>
				<xsl:with-param name="add_2515_val" select="$modinfo_gr_ac_outc/@val11"/>
			</xsl:call-template>
		</xsl:variable>
		<xsl:variable name="ac_out_con_str_12">
			<xsl:call-template name="get-acout-connection-string">
				<xsl:with-param name="add_2513_val" select="$modinfo_gr_ac_outa/@val12"/>
				<xsl:with-param name="add_2514_val" select="$modinfo_gr_ac_outb/@val12"/>
				<xsl:with-param name="add_2515_val" select="$modinfo_gr_ac_outc/@val12"/>
			</xsl:call-template>
		</xsl:variable>
		<xsl:variable name="ac_out_con_str_13">
			<xsl:call-template name="get-acout-connection-string">
				<xsl:with-param name="add_2513_val" select="$modinfo_gr_ac_outa/@val13"/>
				<xsl:with-param name="add_2514_val" select="$modinfo_gr_ac_outb/@val13"/>
				<xsl:with-param name="add_2515_val" select="$modinfo_gr_ac_outc/@val13"/>
			</xsl:call-template>
		</xsl:variable>
		<xsl:variable name="ac_out_con_str_14">
			<xsl:call-template name="get-acout-connection-string">
				<xsl:with-param name="add_2513_val" select="$modinfo_gr_ac_outa/@val14"/>
				<xsl:with-param name="add_2514_val" select="$modinfo_gr_ac_outb/@val14"/>
				<xsl:with-param name="add_2515_val" select="$modinfo_gr_ac_outc/@val14"/>
			</xsl:call-template>
		</xsl:variable>
		<xsl:variable name="ac_out_con_str_15">
			<xsl:call-template name="get-acout-connection-string">
				<xsl:with-param name="add_2513_val" select="$modinfo_gr_ac_outa/@val15"/>
				<xsl:with-param name="add_2514_val" select="$modinfo_gr_ac_outb/@val15"/>
				<xsl:with-param name="add_2515_val" select="$modinfo_gr_ac_outc/@val15"/>
			</xsl:call-template>
		</xsl:variable>
		<xsl:variable name="ac_out_con_str_16">
			<xsl:call-template name="get-acout-connection-string">
				<xsl:with-param name="add_2513_val" select="$modinfo_gr_ac_outa/@val16"/>
				<xsl:with-param name="add_2514_val" select="$modinfo_gr_ac_outb/@val16"/>
				<xsl:with-param name="add_2515_val" select="$modinfo_gr_ac_outc/@val16"/>
			</xsl:call-template>
		</xsl:variable>
		<xsl:variable name="ac_out_con_str_17">
			<xsl:call-template name="get-acout-connection-string">
				<xsl:with-param name="add_2513_val" select="$modinfo_gr_ac_outa/@val17"/>
				<xsl:with-param name="add_2514_val" select="$modinfo_gr_ac_outb/@val17"/>
				<xsl:with-param name="add_2515_val" select="$modinfo_gr_ac_outc/@val17"/>
			</xsl:call-template>
		</xsl:variable>
		<xsl:variable name="ac_out_con_str_18">
			<xsl:call-template name="get-acout-connection-string">
				<xsl:with-param name="add_2513_val" select="$modinfo_gr_ac_outa/@val18"/>
				<xsl:with-param name="add_2514_val" select="$modinfo_gr_ac_outb/@val18"/>
				<xsl:with-param name="add_2515_val" select="$modinfo_gr_ac_outc/@val18"/>
			</xsl:call-template>
		</xsl:variable>
		<xsl:variable name="ac_out_con_str_19">
			<xsl:call-template name="get-acout-connection-string">
				<xsl:with-param name="add_2513_val" select="$modinfo_gr_ac_outa/@val19"/>
				<xsl:with-param name="add_2514_val" select="$modinfo_gr_ac_outb/@val19"/>
				<xsl:with-param name="add_2515_val" select="$modinfo_gr_ac_outc/@val19"/>
			</xsl:call-template>
		</xsl:variable>
		<xsl:variable name="ac_out_con_str_20">
			<xsl:call-template name="get-acout-connection-string">
				<xsl:with-param name="add_2513_val" select="$modinfo_gr_ac_outa/@val20"/>
				<xsl:with-param name="add_2514_val" select="$modinfo_gr_ac_outb/@val20"/>
				<xsl:with-param name="add_2515_val" select="$modinfo_gr_ac_outc/@val20"/>
			</xsl:call-template>
		</xsl:variable>
		<xsl:variable name="ac_out_con_str_21">
			<xsl:call-template name="get-acout-connection-string">
				<xsl:with-param name="add_2513_val" select="$modinfo_gr_ac_outa/@val21"/>
				<xsl:with-param name="add_2514_val" select="$modinfo_gr_ac_outb/@val21"/>
				<xsl:with-param name="add_2515_val" select="$modinfo_gr_ac_outc/@val21"/>
			</xsl:call-template>
		</xsl:variable>
		<xsl:variable name="ac_out_con_str_22">
			<xsl:call-template name="get-acout-connection-string">
				<xsl:with-param name="add_2513_val" select="$modinfo_gr_ac_outa/@val22"/>
				<xsl:with-param name="add_2514_val" select="$modinfo_gr_ac_outb/@val22"/>
				<xsl:with-param name="add_2515_val" select="$modinfo_gr_ac_outc/@val22"/>
			</xsl:call-template>
		</xsl:variable>
		<xsl:variable name="ac_out_con_str_23">
			<xsl:call-template name="get-acout-connection-string">
				<xsl:with-param name="add_2513_val" select="$modinfo_gr_ac_outa/@val23"/>
				<xsl:with-param name="add_2514_val" select="$modinfo_gr_ac_outb/@val23"/>
				<xsl:with-param name="add_2515_val" select="$modinfo_gr_ac_outc/@val23"/>
			</xsl:call-template>
		</xsl:variable>
		<xsl:variable name="ac_out_con_str_24">
			<xsl:call-template name="get-acout-connection-string">
				<xsl:with-param name="add_2513_val" select="$modinfo_gr_ac_outa/@val24"/>
				<xsl:with-param name="add_2514_val" select="$modinfo_gr_ac_outb/@val24"/>
				<xsl:with-param name="add_2515_val" select="$modinfo_gr_ac_outc/@val24"/>
			</xsl:call-template>
		</xsl:variable>
		<xsl:variable name="ac_out_con_str_25">
			<xsl:call-template name="get-acout-connection-string">
				<xsl:with-param name="add_2513_val" select="$modinfo_gr_ac_outa/@val25"/>
				<xsl:with-param name="add_2514_val" select="$modinfo_gr_ac_outb/@val25"/>
				<xsl:with-param name="add_2515_val" select="$modinfo_gr_ac_outc/@val25"/>
			</xsl:call-template>
		</xsl:variable>
		<xsl:variable name="ac_out_con_str_26">
			<xsl:call-template name="get-acout-connection-string">
				<xsl:with-param name="add_2513_val" select="$modinfo_gr_ac_outa/@val26"/>
				<xsl:with-param name="add_2514_val" select="$modinfo_gr_ac_outb/@val26"/>
				<xsl:with-param name="add_2515_val" select="$modinfo_gr_ac_outc/@val26"/>
			</xsl:call-template>
		</xsl:variable>
		<xsl:variable name="ac_out_con_str_27">
			<xsl:call-template name="get-acout-connection-string">
				<xsl:with-param name="add_2513_val" select="$modinfo_gr_ac_outa/@val27"/>
				<xsl:with-param name="add_2514_val" select="$modinfo_gr_ac_outb/@val27"/>
				<xsl:with-param name="add_2515_val" select="$modinfo_gr_ac_outc/@val27"/>
			</xsl:call-template>
		</xsl:variable>
		<xsl:variable name="ac_out_con_str_28">
			<xsl:call-template name="get-acout-connection-string">
				<xsl:with-param name="add_2513_val" select="$modinfo_gr_ac_outa/@val28"/>
				<xsl:with-param name="add_2514_val" select="$modinfo_gr_ac_outb/@val28"/>
				<xsl:with-param name="add_2515_val" select="$modinfo_gr_ac_outc/@val28"/>
			</xsl:call-template>
		</xsl:variable>
		<xsl:variable name="ac_out_con_str_29">
			<xsl:call-template name="get-acout-connection-string">
				<xsl:with-param name="add_2513_val" select="$modinfo_gr_ac_outa/@val29"/>
				<xsl:with-param name="add_2514_val" select="$modinfo_gr_ac_outb/@val29"/>
				<xsl:with-param name="add_2515_val" select="$modinfo_gr_ac_outc/@val29"/>
			</xsl:call-template>
		</xsl:variable>
		<xsl:variable name="ac_out_con_str_30">
			<xsl:call-template name="get-acout-connection-string">
				<xsl:with-param name="add_2513_val" select="$modinfo_gr_ac_outa/@val30"/>
				<xsl:with-param name="add_2514_val" select="$modinfo_gr_ac_outb/@val30"/>
				<xsl:with-param name="add_2515_val" select="$modinfo_gr_ac_outc/@val30"/>
			</xsl:call-template>
		</xsl:variable>
		<xsl:variable name="ac_out_con_str_31">
			<xsl:call-template name="get-acout-connection-string">
				<xsl:with-param name="add_2513_val" select="$modinfo_gr_ac_outa/@val31"/>
				<xsl:with-param name="add_2514_val" select="$modinfo_gr_ac_outb/@val31"/>
				<xsl:with-param name="add_2515_val" select="$modinfo_gr_ac_outc/@val31"/>
			</xsl:call-template>
		</xsl:variable>
		<xsl:variable name="ac_out_con_str_32">
			<xsl:call-template name="get-acout-connection-string">
				<xsl:with-param name="add_2513_val" select="$modinfo_gr_ac_outa/@val32"/>
				<xsl:with-param name="add_2514_val" select="$modinfo_gr_ac_outb/@val32"/>
				<xsl:with-param name="add_2515_val" select="$modinfo_gr_ac_outc/@val32"/>
			</xsl:call-template>
		</xsl:variable>

		<!--Ac In power consumption variables -->
		<xsl:variable name="ac_in_power_consumption_1">
	    <xsl:choose>
	    	<xsl:when test="string($ac_in_con_str_1)='A-3P' or string($ac_in_con_str_1)='B-3P'">
	    		<xsl:value-of select="concat(concat(concat($modinfo_pow_ac_ina/@val,'&#160;'),
	    													concat($modinfo_pow_ac_inb/@val,'&#160;')),concat($modinfo_pow_ac_inc/@val,'&#160;'))"/>
	    	</xsl:when>
	    	<xsl:when test="string($ac_in_con_str_1)='A-1P' or string($ac_in_con_str_1)='B-1P'">
	    		<xsl:value-of select="$modinfo_pow_ac_ina/@val + $modinfo_pow_ac_inb/@val + $modinfo_pow_ac_inc/@val"/>
	    	</xsl:when>
	      <xsl:otherwise><xsl:value-of select="'???'"/></xsl:otherwise>
	    </xsl:choose>
		</xsl:variable>
		<xsl:variable name="ac_in_power_consumption_2">
	    <xsl:choose>
	    	<xsl:when test="string($ac_in_con_str_2)='A-3P' or string($ac_in_con_str_2)='B-3P'">
	    		<xsl:value-of select="concat(concat(concat($modinfo_pow_ac_ina/@val2,'&#160;'),
	    													concat($modinfo_pow_ac_inb/@val2,'&#160;')),concat($modinfo_pow_ac_inc/@val2,'&#160;'))"/>
	    	</xsl:when>
	    	<xsl:when test="string($ac_in_con_str_2)='A-1P' or string($ac_in_con_str_2)='B-1P'">
	    		<xsl:value-of select="$modinfo_pow_ac_ina/@val2 + $modinfo_pow_ac_inb/@val2 + $modinfo_pow_ac_inc/@val2"/>
	    	</xsl:when>
	      <xsl:otherwise><xsl:value-of select="'???'"/></xsl:otherwise>
	    </xsl:choose>
		</xsl:variable>
		<xsl:variable name="ac_in_power_consumption_3">
	    <xsl:choose>
	    	<xsl:when test="string($ac_in_con_str_3)='A-3P' or string($ac_in_con_str_3)='B-3P'">
	    		<xsl:value-of select="concat(concat(concat($modinfo_pow_ac_ina/@val3,'&#160;'),
	    													concat($modinfo_pow_ac_inb/@val3,'&#160;')),concat($modinfo_pow_ac_inc/@val3,'&#160;'))"/>
	    	</xsl:when>
	    	<xsl:when test="string($ac_in_con_str_3)='A-1P' or string($ac_in_con_str_3)='B-1P'">
	    		<xsl:value-of select="$modinfo_pow_ac_ina/@val3 + $modinfo_pow_ac_inb/@val3 + $modinfo_pow_ac_inc/@val3"/>
	    	</xsl:when>
	      <xsl:otherwise><xsl:value-of select="'???'"/></xsl:otherwise>
	    </xsl:choose>
		</xsl:variable>
		<xsl:variable name="ac_in_power_consumption_4">
	    <xsl:choose>
	    	<xsl:when test="string($ac_in_con_str_4)='A-3P' or string($ac_in_con_str_4)='B-3P'">
	    		<xsl:value-of select="concat(concat(concat($modinfo_pow_ac_ina/@val4,'&#160;'),
	    													concat($modinfo_pow_ac_inb/@val4,'&#160;')),concat($modinfo_pow_ac_inc/@val4,'&#160;'))"/>
	    	</xsl:when>
	    	<xsl:when test="string($ac_in_con_str_4)='A-1P' or string($ac_in_con_str_4)='B-1P'">
	    		<xsl:value-of select="$modinfo_pow_ac_ina/@val4 + $modinfo_pow_ac_inb/@val4 + $modinfo_pow_ac_inc/@val4"/>
	    	</xsl:when>
	      <xsl:otherwise><xsl:value-of select="'???'"/></xsl:otherwise>
	    </xsl:choose>
		</xsl:variable>
		<xsl:variable name="ac_in_power_consumption_5">
	    <xsl:choose>
	    	<xsl:when test="string($ac_in_con_str_5)='A-3P' or string($ac_in_con_str_5)='B-3P'">
	    		<xsl:value-of select="concat(concat(concat($modinfo_pow_ac_ina/@val5,'&#160;'),
	    													concat($modinfo_pow_ac_inb/@val5,'&#160;')),concat($modinfo_pow_ac_inc/@val5,'&#160;'))"/>
	    	</xsl:when>
	    	<xsl:when test="string($ac_in_con_str_5)='A-1P' or string($ac_in_con_str_5)='B-1P'">
	    		<xsl:value-of select="$modinfo_pow_ac_ina/@val5 + $modinfo_pow_ac_inb/@val5 + $modinfo_pow_ac_inc/@val5"/>
	    	</xsl:when>
	      <xsl:otherwise><xsl:value-of select="'???'"/></xsl:otherwise>
	    </xsl:choose>
		</xsl:variable>
		<xsl:variable name="ac_in_power_consumption_6">
	    <xsl:choose>
	    	<xsl:when test="string($ac_in_con_str_6)='A-3P' or string($ac_in_con_str_6)='B-3P'">
	    		<xsl:value-of select="concat(concat(concat($modinfo_pow_ac_ina/@val6,'&#160;'),
	    													concat($modinfo_pow_ac_inb/@val6,'&#160;')),concat($modinfo_pow_ac_inc/@val6,'&#160;'))"/>
	    	</xsl:when>
	    	<xsl:when test="string($ac_in_con_str_6)='A-1P' or string($ac_in_con_str_6)='B-1P'">
	    		<xsl:value-of select="$modinfo_pow_ac_ina/@val6 + $modinfo_pow_ac_inb/@val6 + $modinfo_pow_ac_inc/@val6"/>
	    	</xsl:when>
	      <xsl:otherwise><xsl:value-of select="'???'"/></xsl:otherwise>
	    </xsl:choose>
		</xsl:variable>
		<xsl:variable name="ac_in_power_consumption_7">
	    <xsl:choose>
	    	<xsl:when test="string($ac_in_con_str_7)='A-3P' or string($ac_in_con_str_7)='B-3P'">
	    		<xsl:value-of select="concat(concat(concat($modinfo_pow_ac_ina/@val7,'&#160;'),
	    													concat($modinfo_pow_ac_inb/@val7,'&#160;')),concat($modinfo_pow_ac_inc/@val7,'&#160;'))"/>
	    	</xsl:when>
	    	<xsl:when test="string($ac_in_con_str_7)='A-1P' or string($ac_in_con_str_7)='B-1P'">
	    		<xsl:value-of select="$modinfo_pow_ac_ina/@val7 + $modinfo_pow_ac_inb/@val7 + $modinfo_pow_ac_inc/@val7"/>
	    	</xsl:when>
	      <xsl:otherwise><xsl:value-of select="'???'"/></xsl:otherwise>
	    </xsl:choose>
		</xsl:variable>
		<xsl:variable name="ac_in_power_consumption_8">
	    <xsl:choose>
	    	<xsl:when test="string($ac_in_con_str_8)='A-3P' or string($ac_in_con_str_8)='B-3P'">
	    		<xsl:value-of select="concat(concat(concat($modinfo_pow_ac_ina/@val8,'&#160;'),
	    													concat($modinfo_pow_ac_inb/@val8,'&#160;')),concat($modinfo_pow_ac_inc/@val8,'&#160;'))"/>
	    	</xsl:when>
	    	<xsl:when test="string($ac_in_con_str_8)='A-1P' or string($ac_in_con_str_8)='B-1P'">
	    		<xsl:value-of select="$modinfo_pow_ac_ina/@val8 + $modinfo_pow_ac_inb/@val8 + $modinfo_pow_ac_inc/@val8"/>
	    	</xsl:when>
	      <xsl:otherwise><xsl:value-of select="'???'"/></xsl:otherwise>
	    </xsl:choose>
		</xsl:variable>
		<xsl:variable name="ac_in_power_consumption_9">
	    <xsl:choose>
	    	<xsl:when test="string($ac_in_con_str_9)='A-3P' or string($ac_in_con_str_9)='B-3P'">
	    		<xsl:value-of select="concat(concat(concat($modinfo_pow_ac_ina/@val9,'&#160;'),
	    													concat($modinfo_pow_ac_inb/@val9,'&#160;')),concat($modinfo_pow_ac_inc/@val9,'&#160;'))"/>
	    	</xsl:when>
	    	<xsl:when test="string($ac_in_con_str_9)='A-1P' or string($ac_in_con_str_9)='B-1P'">
	    		<xsl:value-of select="$modinfo_pow_ac_ina/@val9 + $modinfo_pow_ac_inb/@val9 + $modinfo_pow_ac_inc/@val9"/>
	    	</xsl:when>
	      <xsl:otherwise><xsl:value-of select="'???'"/></xsl:otherwise>
	    </xsl:choose>
		</xsl:variable>
		<xsl:variable name="ac_in_power_consumption_10">
	    <xsl:choose>
	    	<xsl:when test="string($ac_in_con_str_10)='A-3P' or string($ac_in_con_str_10)='B-3P'">
	    		<xsl:value-of select="concat(concat(concat($modinfo_pow_ac_ina/@val10,'&#160;'),
	    													concat($modinfo_pow_ac_inb/@val10,'&#160;')),concat($modinfo_pow_ac_inc/@val10,'&#160;'))"/>
	    	</xsl:when>
	    	<xsl:when test="string($ac_in_con_str_10)='A-1P' or string($ac_in_con_str_10)='B-1P'">
	    		<xsl:value-of select="$modinfo_pow_ac_ina/@val10 + $modinfo_pow_ac_inb/@val10 + $modinfo_pow_ac_inc/@val10"/>
	    	</xsl:when>
	      <xsl:otherwise><xsl:value-of select="'???'"/></xsl:otherwise>
	    </xsl:choose>
		</xsl:variable>
		<xsl:variable name="ac_in_power_consumption_11">
	    <xsl:choose>
	    	<xsl:when test="string($ac_in_con_str_11)='A-3P' or string($ac_in_con_str_11)='B-3P'">
	    		<xsl:value-of select="concat(concat(concat($modinfo_pow_ac_ina/@val11,'&#160;'),
	    													concat($modinfo_pow_ac_inb/@val11,'&#160;')),concat($modinfo_pow_ac_inc/@val11,'&#160;'))"/>
	    	</xsl:when>
	    	<xsl:when test="string($ac_in_con_str_11)='A-1P' or string($ac_in_con_str_11)='B-1P'">
	    		<xsl:value-of select="$modinfo_pow_ac_ina/@val11 + $modinfo_pow_ac_inb/@val11 + $modinfo_pow_ac_inc/@val11"/>
	    	</xsl:when>
	      <xsl:otherwise><xsl:value-of select="'???'"/></xsl:otherwise>
	    </xsl:choose>
		</xsl:variable>
		<xsl:variable name="ac_in_power_consumption_12">
	    <xsl:choose>
	    	<xsl:when test="string($ac_in_con_str_12)='A-3P' or string($ac_in_con_str_12)='B-3P'">
	    		<xsl:value-of select="concat(concat(concat($modinfo_pow_ac_ina/@val12,'&#160;'),
	    													concat($modinfo_pow_ac_inb/@val12,'&#160;')),concat($modinfo_pow_ac_inc/@val12,'&#160;'))"/>
	    	</xsl:when>
	    	<xsl:when test="string($ac_in_con_str_12)='A-1P' or string($ac_in_con_str_12)='B-1P'">
	    		<xsl:value-of select="$modinfo_pow_ac_ina/@val12 + $modinfo_pow_ac_inb/@val12 + $modinfo_pow_ac_inc/@val12"/>
	    	</xsl:when>
	      <xsl:otherwise><xsl:value-of select="'???'"/></xsl:otherwise>
	    </xsl:choose>
		</xsl:variable>
		<xsl:variable name="ac_in_power_consumption_13">
	    <xsl:choose>
	    	<xsl:when test="string($ac_in_con_str_13)='A-3P' or string($ac_in_con_str_13)='B-3P'">
	    		<xsl:value-of select="concat(concat(concat($modinfo_pow_ac_ina/@val13,'&#160;'),
	    													concat($modinfo_pow_ac_inb/@val13,'&#160;')),concat($modinfo_pow_ac_inc/@val13,'&#160;'))"/>
	    	</xsl:when>
	    	<xsl:when test="string($ac_in_con_str_13)='A-1P' or string($ac_in_con_str_13)='B-1P'">
	    		<xsl:value-of select="$modinfo_pow_ac_ina/@val13 + $modinfo_pow_ac_inb/@val13 + $modinfo_pow_ac_inc/@val13"/>
	    	</xsl:when>
	      <xsl:otherwise><xsl:value-of select="'???'"/></xsl:otherwise>
	    </xsl:choose>
		</xsl:variable>
		<xsl:variable name="ac_in_power_consumption_14">
	    <xsl:choose>
	    	<xsl:when test="string($ac_in_con_str_14)='A-3P' or string($ac_in_con_str_14)='B-3P'">
	    		<xsl:value-of select="concat(concat(concat($modinfo_pow_ac_ina/@val14,'&#160;'),
	    													concat($modinfo_pow_ac_inb/@val14,'&#160;')),concat($modinfo_pow_ac_inc/@val14,'&#160;'))"/>
	    	</xsl:when>
	    	<xsl:when test="string($ac_in_con_str_14)='A-1P' or string($ac_in_con_str_14)='B-1P'">
	    		<xsl:value-of select="$modinfo_pow_ac_ina/@val14 + $modinfo_pow_ac_inb/@val14 + $modinfo_pow_ac_inc/@val14"/>
	    	</xsl:when>
	      <xsl:otherwise><xsl:value-of select="'???'"/></xsl:otherwise>
	    </xsl:choose>
		</xsl:variable>
		<xsl:variable name="ac_in_power_consumption_15">
	    <xsl:choose>
	    	<xsl:when test="string($ac_in_con_str_15)='A-3P' or string($ac_in_con_str_15)='B-3P'">
	    		<xsl:value-of select="concat(concat(concat($modinfo_pow_ac_ina/@val15,'&#160;'),
	    													concat($modinfo_pow_ac_inb/@val15,'&#160;')),concat($modinfo_pow_ac_inc/@val15,'&#160;'))"/>
	    	</xsl:when>
	    	<xsl:when test="string($ac_in_con_str_15)='A-1P' or string($ac_in_con_str_15)='B-1P'">
	    		<xsl:value-of select="$modinfo_pow_ac_ina/@val15 + $modinfo_pow_ac_inb/@val15 + $modinfo_pow_ac_inc/@val15"/>
	    	</xsl:when>
	      <xsl:otherwise><xsl:value-of select="'???'"/></xsl:otherwise>
	    </xsl:choose>
		</xsl:variable>
		<xsl:variable name="ac_in_power_consumption_16">
	    <xsl:choose>
	    	<xsl:when test="string($ac_in_con_str_16)='A-3P' or string($ac_in_con_str_16)='B-3P'">
	    		<xsl:value-of select="concat(concat(concat($modinfo_pow_ac_ina/@val16,'&#160;'),
	    													concat($modinfo_pow_ac_inb/@val16,'&#160;')),concat($modinfo_pow_ac_inc/@val16,'&#160;'))"/>
	    	</xsl:when>
	    	<xsl:when test="string($ac_in_con_str_16)='A-1P' or string($ac_in_con_str_16)='B-1P'">
	    		<xsl:value-of select="$modinfo_pow_ac_ina/@val16 + $modinfo_pow_ac_inb/@val16 + $modinfo_pow_ac_inc/@val16"/>
	    	</xsl:when>
	      <xsl:otherwise><xsl:value-of select="'???'"/></xsl:otherwise>
	    </xsl:choose>
		</xsl:variable>
		<xsl:variable name="ac_in_power_consumption_17">
	    <xsl:choose>
	    	<xsl:when test="string($ac_in_con_str_17)='A-3P' or string($ac_in_con_str_17)='B-3P'">
	    		<xsl:value-of select="concat(concat(concat($modinfo_pow_ac_ina/@val17,'&#160;'),
	    													concat($modinfo_pow_ac_inb/@val17,'&#160;')),concat($modinfo_pow_ac_inc/@val17,'&#160;'))"/>
	    	</xsl:when>
	    	<xsl:when test="string($ac_in_con_str_17)='A-1P' or string($ac_in_con_str_17)='B-1P'">
	    		<xsl:value-of select="$modinfo_pow_ac_ina/@val17 + $modinfo_pow_ac_inb/@val17 + $modinfo_pow_ac_inc/@val17"/>
	    	</xsl:when>
	      <xsl:otherwise><xsl:value-of select="'???'"/></xsl:otherwise>
	    </xsl:choose>
		</xsl:variable>
		<xsl:variable name="ac_in_power_consumption_18">
	    <xsl:choose>
	    	<xsl:when test="string($ac_in_con_str_18)='A-3P' or string($ac_in_con_str_18)='B-3P'">
	    		<xsl:value-of select="concat(concat(concat($modinfo_pow_ac_ina/@val18,'&#160;'),
	    													concat($modinfo_pow_ac_inb/@val18,'&#160;')),concat($modinfo_pow_ac_inc/@val18,'&#160;'))"/>
	    	</xsl:when>
	    	<xsl:when test="string($ac_in_con_str_18)='A-1P' or string($ac_in_con_str_18)='B-1P'">
	    		<xsl:value-of select="$modinfo_pow_ac_ina/@val18 + $modinfo_pow_ac_inb/@val18 + $modinfo_pow_ac_inc/@val18"/>
	    	</xsl:when>
	      <xsl:otherwise><xsl:value-of select="'???'"/></xsl:otherwise>
	    </xsl:choose>
		</xsl:variable>
		<xsl:variable name="ac_in_power_consumption_19">
	    <xsl:choose>
	    	<xsl:when test="string($ac_in_con_str_19)='A-3P' or string($ac_in_con_str_19)='B-3P'">
	    		<xsl:value-of select="concat(concat(concat($modinfo_pow_ac_ina/@val19,'&#160;'),
	    													concat($modinfo_pow_ac_inb/@val19,'&#160;')),concat($modinfo_pow_ac_inc/@val19,'&#160;'))"/>
	    	</xsl:when>
	    	<xsl:when test="string($ac_in_con_str_19)='A-1P' or string($ac_in_con_str_19)='B-1P'">
	    		<xsl:value-of select="$modinfo_pow_ac_ina/@val19 + $modinfo_pow_ac_inb/@val19 + $modinfo_pow_ac_inc/@val19"/>
	    	</xsl:when>
	      <xsl:otherwise><xsl:value-of select="'???'"/></xsl:otherwise>
	    </xsl:choose>
		</xsl:variable>
		<xsl:variable name="ac_in_power_consumption_20">
	    <xsl:choose>
	    	<xsl:when test="string($ac_in_con_str_20)='A-3P' or string($ac_in_con_str_20)='B-3P'">
	    		<xsl:value-of select="concat(concat(concat($modinfo_pow_ac_ina/@val20,'&#160;'),
	    													concat($modinfo_pow_ac_inb/@val20,'&#160;')),concat($modinfo_pow_ac_inc/@val20,'&#160;'))"/>
	    	</xsl:when>
	    	<xsl:when test="string($ac_in_con_str_20)='A-1P' or string($ac_in_con_str_20)='B-1P'">
	    		<xsl:value-of select="$modinfo_pow_ac_ina/@val20 + $modinfo_pow_ac_inb/@val20 + $modinfo_pow_ac_inc/@val20"/>
	    	</xsl:when>
	      <xsl:otherwise><xsl:value-of select="'???'"/></xsl:otherwise>
	    </xsl:choose>
		</xsl:variable>
		<xsl:variable name="ac_in_power_consumption_21">
	    <xsl:choose>
	    	<xsl:when test="string($ac_in_con_str_21)='A-3P' or string($ac_in_con_str_21)='B-3P'">
	    		<xsl:value-of select="concat(concat(concat($modinfo_pow_ac_ina/@val,'&#160;'),
	    													concat($modinfo_pow_ac_inb/@val21,'&#160;')),concat($modinfo_pow_ac_inc/@val21,'&#160;'))"/>
	    	</xsl:when>
	    	<xsl:when test="string($ac_in_con_str_21)='A-1P' or string($ac_in_con_str_21)='B-1P'">
	    		<xsl:value-of select="$modinfo_pow_ac_ina/@val21 + $modinfo_pow_ac_inb/@val21 + $modinfo_pow_ac_inc/@val21"/>
	    	</xsl:when>
	      <xsl:otherwise><xsl:value-of select="'???'"/></xsl:otherwise>
	    </xsl:choose>
		</xsl:variable>
		<xsl:variable name="ac_in_power_consumption_22">
	    <xsl:choose>
	    	<xsl:when test="string($ac_in_con_str_22)='A-3P' or string($ac_in_con_str_22)='B-3P'">
	    		<xsl:value-of select="concat(concat(concat($modinfo_pow_ac_ina/@val22,'&#160;'),
	    													concat($modinfo_pow_ac_inb/@val22,'&#160;')),concat($modinfo_pow_ac_inc/@val22,'&#160;'))"/>
	    	</xsl:when>
	    	<xsl:when test="string($ac_in_con_str_22)='A-1P' or string($ac_in_con_str_22)='B-1P'">
	    		<xsl:value-of select="$modinfo_pow_ac_ina/@val22 + $modinfo_pow_ac_inb/@val22 + $modinfo_pow_ac_inc/@val22"/>
	    	</xsl:when>
	      <xsl:otherwise><xsl:value-of select="'???'"/></xsl:otherwise>
	    </xsl:choose>
		</xsl:variable>
		<xsl:variable name="ac_in_power_consumption_23">
	    <xsl:choose>
	    	<xsl:when test="string($ac_in_con_str_23)='A-3P' or string($ac_in_con_str_23)='B-3P'">
	    		<xsl:value-of select="concat(concat(concat($modinfo_pow_ac_ina/@valv,'&#160;'),
	    													concat($modinfo_pow_ac_inb/@val23,'&#160;')),concat($modinfo_pow_ac_inc/@val23,'&#160;'))"/>
	    	</xsl:when>
	    	<xsl:when test="string($ac_in_con_str_23)='A-1P' or string($ac_in_con_str_23)='B-1P'">
	    		<xsl:value-of select="$modinfo_pow_ac_ina/@val23 + $modinfo_pow_ac_inb/@val23 + $modinfo_pow_ac_inc/@val23"/>
	    	</xsl:when>
	      <xsl:otherwise><xsl:value-of select="'???'"/></xsl:otherwise>
	    </xsl:choose>
		</xsl:variable>
		<xsl:variable name="ac_in_power_consumption_24">
	    <xsl:choose>
	    	<xsl:when test="string($ac_in_con_str_24)='A-3P' or string($ac_in_con_str_24)='B-3P'">
	    		<xsl:value-of select="concat(concat(concat($modinfo_pow_ac_ina/@val24,'&#160;'),
	    													concat($modinfo_pow_ac_inb/@val24,'&#160;')),concat($modinfo_pow_ac_inc/@val24,'&#160;'))"/>
	    	</xsl:when>
	    	<xsl:when test="string($ac_in_con_str_24)='A-1P' or string($ac_in_con_str_24)='B-1P'">
	    		<xsl:value-of select="$modinfo_pow_ac_ina/@val24 + $modinfo_pow_ac_inb/@val24 + $modinfo_pow_ac_inc/@val24"/>
	    	</xsl:when>
	      <xsl:otherwise><xsl:value-of select="'???'"/></xsl:otherwise>
	    </xsl:choose>
		</xsl:variable>
		<xsl:variable name="ac_in_power_consumption_25">
	    <xsl:choose>
	    	<xsl:when test="string($ac_in_con_str_25)='A-3P' or string($ac_in_con_str_25)='B-3P'">
	    		<xsl:value-of select="concat(concat(concat($modinfo_pow_ac_ina/@val25,'&#160;'),
	    													concat($modinfo_pow_ac_inb/@val25,'&#160;')),concat($modinfo_pow_ac_inc/@val25,'&#160;'))"/>
	    	</xsl:when>
	    	<xsl:when test="string($ac_in_con_str_25)='A-1P' or string($ac_in_con_str_25)='B-1P'">
	    		<xsl:value-of select="$modinfo_pow_ac_ina/@val25 + $modinfo_pow_ac_inb/@val25 + $modinfo_pow_ac_inc/@val25"/>
	    	</xsl:when>
	      <xsl:otherwise><xsl:value-of select="'???'"/></xsl:otherwise>
	    </xsl:choose>
		</xsl:variable>
		<xsl:variable name="ac_in_power_consumption_26">
	    <xsl:choose>
	    	<xsl:when test="string($ac_in_con_str_26)='A-3P' or string($ac_in_con_str_26)='B-3P'">
	    		<xsl:value-of select="concat(concat(concat($modinfo_pow_ac_ina/@val26,'&#160;'),
	    													concat($modinfo_pow_ac_inb/@val26,'&#160;')),concat($modinfo_pow_ac_inc/@val26,'&#160;'))"/>
	    	</xsl:when>
	    	<xsl:when test="string($ac_in_con_str_26)='A-1P' or string($ac_in_con_str_26)='B-1P'">
	    		<xsl:value-of select="$modinfo_pow_ac_ina/@val26 + $modinfo_pow_ac_inb/@val26 + $modinfo_pow_ac_inc/@val26"/>
	    	</xsl:when>
	      <xsl:otherwise><xsl:value-of select="'???'"/></xsl:otherwise>
	    </xsl:choose>
		</xsl:variable>
		<xsl:variable name="ac_in_power_consumption_27">
	    <xsl:choose>
	    	<xsl:when test="string($ac_in_con_str_27)='A-3P' or string($ac_in_con_str_27)='B-3P'">
	    		<xsl:value-of select="concat(concat(concat($modinfo_pow_ac_ina/@val27,'&#160;'),
	    													concat($modinfo_pow_ac_inb/@val27,'&#160;')),concat($modinfo_pow_ac_inc/@val27,'&#160;'))"/>
	    	</xsl:when>
	    	<xsl:when test="string($ac_in_con_str_27)='A-1P' or string($ac_in_con_str_27)='B-1P'">
	    		<xsl:value-of select="$modinfo_pow_ac_ina/@val27 + $modinfo_pow_ac_inb/@val27 + $modinfo_pow_ac_inc/@val27"/>
	    	</xsl:when>
	      <xsl:otherwise><xsl:value-of select="'???'"/></xsl:otherwise>
	    </xsl:choose>
		</xsl:variable>
		<xsl:variable name="ac_in_power_consumption_28">
	    <xsl:choose>
	    	<xsl:when test="string($ac_in_con_str_28)='A-3P' or string($ac_in_con_str_28)='B-3P'">
	    		<xsl:value-of select="concat(concat(concat($modinfo_pow_ac_ina/@val28,'&#160;'),
	    													concat($modinfo_pow_ac_inb/@val28,'&#160;')),concat($modinfo_pow_ac_inc/@val28,'&#160;'))"/>
	    	</xsl:when>
	    	<xsl:when test="string($ac_in_con_str_28)='A-1P' or string($ac_in_con_str_28)='B-1P'">
	    		<xsl:value-of select="$modinfo_pow_ac_ina/@val28 + $modinfo_pow_ac_inb/@val28 + $modinfo_pow_ac_inc/@val28"/>
	    	</xsl:when>
	      <xsl:otherwise><xsl:value-of select="'???'"/></xsl:otherwise>
	    </xsl:choose>
		</xsl:variable>
		<xsl:variable name="ac_in_power_consumption_29">
	    <xsl:choose>
	    	<xsl:when test="string($ac_in_con_str_29)='A-3P' or string($ac_in_con_str_29)='B-3P'">
	    		<xsl:value-of select="concat(concat(concat($modinfo_pow_ac_ina/@val29,'&#160;'),
	    													concat($modinfo_pow_ac_inb/@val29,'&#160;')),concat($modinfo_pow_ac_inc/@val29,'&#160;'))"/>
	    	</xsl:when>
	    	<xsl:when test="string($ac_in_con_str_29)='A-1P' or string($ac_in_con_str_29)='B-1P'">
	    		<xsl:value-of select="$modinfo_pow_ac_ina/@val29 + $modinfo_pow_ac_inb/@val29 + $modinfo_pow_ac_inc/@val29"/>
	    	</xsl:when>
	      <xsl:otherwise><xsl:value-of select="'???'"/></xsl:otherwise>
	    </xsl:choose>
		</xsl:variable>
		<xsl:variable name="ac_in_power_consumption_30">
	    <xsl:choose>
	    	<xsl:when test="string($ac_in_con_str_30)='A-3P' or string($ac_in_con_str_30)='B-3P'">
	    		<xsl:value-of select="concat(concat(concat($modinfo_pow_ac_ina/@val30,'&#160;'),
	    													concat($modinfo_pow_ac_inb/@val30,'&#160;')),concat($modinfo_pow_ac_inc/@val30,'&#160;'))"/>
	    	</xsl:when>
	    	<xsl:when test="string($ac_in_con_str_30)='A-1P' or string($ac_in_con_str_30)='B-1P'">
	    		<xsl:value-of select="$modinfo_pow_ac_ina/@val30 + $modinfo_pow_ac_inb/@val30 + $modinfo_pow_ac_inc/@val30"/>
	    	</xsl:when>
	      <xsl:otherwise><xsl:value-of select="'???'"/></xsl:otherwise>
	    </xsl:choose>
		</xsl:variable>
		<xsl:variable name="ac_in_power_consumption_31">
	    <xsl:choose>
	    	<xsl:when test="string($ac_in_con_str_31)='A-3P' or string($ac_in_con_str_31)='B-3P'">
	    		<xsl:value-of select="concat(concat(concat($modinfo_pow_ac_ina/@val31,'&#160;'),
	    													concat($modinfo_pow_ac_inb/@val31,'&#160;')),concat($modinfo_pow_ac_inc/@val31,'&#160;'))"/>
	    	</xsl:when>
	    	<xsl:when test="string($ac_in_con_str_31)='A-1P' or string($ac_in_con_str_31)='B-1P'">
	    		<xsl:value-of select="$modinfo_pow_ac_ina/@val31 + $modinfo_pow_ac_inb/@val31 + $modinfo_pow_ac_inc/@val31"/>
	    	</xsl:when>
	      <xsl:otherwise><xsl:value-of select="'???'"/></xsl:otherwise>
	    </xsl:choose>
		</xsl:variable>
		<xsl:variable name="ac_in_power_consumption_32">
	    <xsl:choose>
	    	<xsl:when test="string($ac_in_con_str_32)='A-3P' or string($ac_in_con_str_32)='B-3P'">
	    		<xsl:value-of select="concat(concat(concat($modinfo_pow_ac_ina/@val32,'&#160;'),
	    													concat($modinfo_pow_ac_inb/@val32,'&#160;')),concat($modinfo_pow_ac_inc/@val32,'&#160;'))"/>
	    	</xsl:when>
	    	<xsl:when test="string($ac_in_con_str_32)='A-1P' or string($ac_in_con_str_32)='B-1P'">
	    		<xsl:value-of select="$modinfo_pow_ac_ina/@val32 + $modinfo_pow_ac_inb/@val32 + $modinfo_pow_ac_inc/@val32"/>
	    	</xsl:when>
	      <xsl:otherwise><xsl:value-of select="'???'"/></xsl:otherwise>
	    </xsl:choose>
		</xsl:variable>

		<!--Ac Out power consumption variables -->
		<xsl:variable name="ac_out_power_consumption_1">
	    <xsl:choose>
	    	<xsl:when test="string($ac_out_con_str_1)='3P'">
	    		<xsl:value-of select="concat(concat(concat($modinfo_pow_ac_outa/@val,'&#160;'),
	    													concat($modinfo_pow_ac_outb/@val,'&#160;')),concat($modinfo_pow_ac_outc/@val,'&#160;'))"/>
	    	</xsl:when>
	    	<xsl:when test="string($ac_out_con_str_1)='1P'">
	    		<xsl:value-of select="$modinfo_pow_ac_outa/@val + $modinfo_pow_ac_outb/@val + $modinfo_pow_ac_outc/@val"/>
	    	</xsl:when>
	      <xsl:otherwise><xsl:value-of select="'???'"/></xsl:otherwise>
	    </xsl:choose>
		</xsl:variable>
		<xsl:variable name="ac_out_power_consumption_2">
	    <xsl:choose>
	    	<xsl:when test="string($ac_out_con_str_2)='3P'">
	    		<xsl:value-of select="concat(concat(concat($modinfo_pow_ac_outa/@val2,'&#160;'),
	    													concat($modinfo_pow_ac_outb/@val2,'&#160;')),concat($modinfo_pow_ac_outc/@val2,'&#160;'))"/>
	    	</xsl:when>
	    	<xsl:when test="string($ac_out_con_str_2)='1P'">
	    		<xsl:value-of select="$modinfo_pow_ac_outa/@val2 + $modinfo_pow_ac_outb/@val2 + $modinfo_pow_ac_outc/@val2"/>
	    	</xsl:when>
	      <xsl:otherwise><xsl:value-of select="'???'"/></xsl:otherwise>
	    </xsl:choose>
		</xsl:variable>
		<xsl:variable name="ac_out_power_consumption_3">
	    <xsl:choose>
	    	<xsl:when test="string($ac_out_con_str_3)='3P'">
	    		<xsl:value-of select="concat(concat(concat($modinfo_pow_ac_outa/@val3,'&#160;'),
	    													concat($modinfo_pow_ac_outb/@val3,'&#160;')),concat($modinfo_pow_ac_outc/@val3,'&#160;'))"/>
	    	</xsl:when>
	    	<xsl:when test="string($ac_out_con_str_3)='1P'">
	    		<xsl:value-of select="$modinfo_pow_ac_outa/@val3 + $modinfo_pow_ac_outb/@val3 + $modinfo_pow_ac_outc/@val3"/>
	    	</xsl:when>
	      <xsl:otherwise><xsl:value-of select="'???'"/></xsl:otherwise>
	    </xsl:choose>
		</xsl:variable>
		<xsl:variable name="ac_out_power_consumption_4">
	    <xsl:choose>
	    	<xsl:when test="string($ac_out_con_str_4)='3P'">
	    		<xsl:value-of select="concat(concat(concat($modinfo_pow_ac_outa/@val4,'&#160;'),
	    													concat($modinfo_pow_ac_outb/@val4,'&#160;')),concat($modinfo_pow_ac_outc/@val4,'&#160;'))"/>
	    	</xsl:when>
	    	<xsl:when test="string($ac_out_con_str_4)='1P'">
	    		<xsl:value-of select="$modinfo_pow_ac_outa/@val4 + $modinfo_pow_ac_outb/@val4 + $modinfo_pow_ac_outc/@val4"/>
	    	</xsl:when>
	      <xsl:otherwise><xsl:value-of select="'???'"/></xsl:otherwise>
	    </xsl:choose>
		</xsl:variable>
		<xsl:variable name="ac_out_power_consumption_5">
	    <xsl:choose>
	    	<xsl:when test="string($ac_out_con_str_5)='3P'">
	    		<xsl:value-of select="concat(concat(concat($modinfo_pow_ac_outa/@val5,'&#160;'),
	    													concat($modinfo_pow_ac_outb/@val5,'&#160;')),concat($modinfo_pow_ac_outc/@val5,'&#160;'))"/>
	    	</xsl:when>
	    	<xsl:when test="string($ac_out_con_str_5)='1P'">
	    		<xsl:value-of select="$modinfo_pow_ac_outa/@val5 + $modinfo_pow_ac_outb/@val5 + $modinfo_pow_ac_outc/@val5"/>
	    	</xsl:when>
	      <xsl:otherwise><xsl:value-of select="'???'"/></xsl:otherwise>
	    </xsl:choose>
		</xsl:variable>
		<xsl:variable name="ac_out_power_consumption_6">
	    <xsl:choose>
	    	<xsl:when test="string($ac_out_con_str_6)='3P'">
	    		<xsl:value-of select="concat(concat(concat($modinfo_pow_ac_outa/@val6,'&#160;'),
	    													concat($modinfo_pow_ac_outb/@val6,'&#160;')),concat($modinfo_pow_ac_outc/@val6,'&#160;'))"/>
	    	</xsl:when>
	    	<xsl:when test="string($ac_out_con_str_6)='1P'">
	    		<xsl:value-of select="$modinfo_pow_ac_outa/@val6 + $modinfo_pow_ac_outb/@val6 + $modinfo_pow_ac_outc/@val6"/>
	    	</xsl:when>
	      <xsl:otherwise><xsl:value-of select="'???'"/></xsl:otherwise>
	    </xsl:choose>
		</xsl:variable>
		<xsl:variable name="ac_out_power_consumption_7">
	    <xsl:choose>
	    	<xsl:when test="string($ac_out_con_str_7)='3P'">
	    		<xsl:value-of select="concat(concat(concat($modinfo_pow_ac_outa/@val7,'&#160;'),
	    													concat($modinfo_pow_ac_outb/@val7,'&#160;')),concat($modinfo_pow_ac_outc/@val7,'&#160;'))"/>
	    	</xsl:when>
	    	<xsl:when test="string($ac_out_con_str_7)='1P'">
	    		<xsl:value-of select="$modinfo_pow_ac_outa/@val7 + $modinfo_pow_ac_outb/@val7 + $modinfo_pow_ac_outc/@val7"/>
	    	</xsl:when>
	      <xsl:otherwise><xsl:value-of select="'???'"/></xsl:otherwise>
	    </xsl:choose>
		</xsl:variable>
		<xsl:variable name="ac_out_power_consumption_8">
	    <xsl:choose>
	    	<xsl:when test="string($ac_out_con_str_8)='3P'">
	    		<xsl:value-of select="concat(concat(concat($modinfo_pow_ac_outa/@val8,'&#160;'),
	    													concat($modinfo_pow_ac_outb/@val8,'&#160;')),concat($modinfo_pow_ac_outc/@val8,'&#160;'))"/>
	    	</xsl:when>
	    	<xsl:when test="string($ac_out_con_str_8)='1P'">
	    		<xsl:value-of select="$modinfo_pow_ac_outa/@val8 + $modinfo_pow_ac_outb/@val8 + $modinfo_pow_ac_outc/@val8"/>
	    	</xsl:when>
	      <xsl:otherwise><xsl:value-of select="'???'"/></xsl:otherwise>
	    </xsl:choose>
		</xsl:variable>
		<xsl:variable name="ac_out_power_consumption_9">
	    <xsl:choose>
	    	<xsl:when test="string($ac_out_con_str_9)='3P'">
	    		<xsl:value-of select="concat(concat(concat($modinfo_pow_ac_outa/@val9,'&#160;'),
	    													concat($modinfo_pow_ac_outb/@val9,'&#160;')),concat($modinfo_pow_ac_outc/@val9,'&#160;'))"/>
	    	</xsl:when>
	    	<xsl:when test="string($ac_out_con_str_9)='1P'">
	    		<xsl:value-of select="$modinfo_pow_ac_outa/@val9 + $modinfo_pow_ac_outb/@val9 + $modinfo_pow_ac_outc/@val9"/>
	    	</xsl:when>
	      <xsl:otherwise><xsl:value-of select="'???'"/></xsl:otherwise>
	    </xsl:choose>
		</xsl:variable>
		<xsl:variable name="ac_out_power_consumption_10">
	    <xsl:choose>
	    	<xsl:when test="string($ac_out_con_str_10)='3P'">
	    		<xsl:value-of select="concat(concat(concat($modinfo_pow_ac_outa/@val10,'&#160;'),
	    													concat($modinfo_pow_ac_outb/@val10,'&#160;')),concat($modinfo_pow_ac_outc/@val10,'&#160;'))"/>
	    	</xsl:when>
	    	<xsl:when test="string($ac_out_con_str_10)='1P'">
	    		<xsl:value-of select="$modinfo_pow_ac_outa/@val10 + $modinfo_pow_ac_outb/@val10 + $modinfo_pow_ac_outc/@val10"/>
	    	</xsl:when>
	      <xsl:otherwise><xsl:value-of select="'???'"/></xsl:otherwise>
	    </xsl:choose>
		</xsl:variable>
		<xsl:variable name="ac_out_power_consumption_11">
	    <xsl:choose>
	    	<xsl:when test="string($ac_out_con_str_11)='3P'">
	    		<xsl:value-of select="concat(concat(concat($modinfo_pow_ac_outa/@val11,'&#160;'),
	    													concat($modinfo_pow_ac_outb/@val11,'&#160;')),concat($modinfo_pow_ac_outc/@val11,'&#160;'))"/>
	    	</xsl:when>
	    	<xsl:when test="string($ac_out_con_str_11)='1P'">
	    		<xsl:value-of select="$modinfo_pow_ac_outa/@val11 + $modinfo_pow_ac_outb/@val11 + $modinfo_pow_ac_outc/@val11"/>
	    	</xsl:when>
	      <xsl:otherwise><xsl:value-of select="'???'"/></xsl:otherwise>
	    </xsl:choose>
		</xsl:variable>
		<xsl:variable name="ac_out_power_consumption_12">
	    <xsl:choose>
	    	<xsl:when test="string($ac_out_con_str_12)='3P'">
	    		<xsl:value-of select="concat(concat(concat($modinfo_pow_ac_outa/@val12,'&#160;'),
	    													concat($modinfo_pow_ac_outb/@val12,'&#160;')),concat($modinfo_pow_ac_outc/@val12,'&#160;'))"/>
	    	</xsl:when>
	    	<xsl:when test="string($ac_out_con_str_12)='1P'">
	    		<xsl:value-of select="$modinfo_pow_ac_outa/@val12 + $modinfo_pow_ac_outb/@val12 + $modinfo_pow_ac_outc/@val12"/>
	    	</xsl:when>
	      <xsl:otherwise><xsl:value-of select="'???'"/></xsl:otherwise>
	    </xsl:choose>
		</xsl:variable>
		<xsl:variable name="ac_out_power_consumption_13">
	    <xsl:choose>
	    	<xsl:when test="string($ac_out_con_str_13)='3P'">
	    		<xsl:value-of select="concat(concat(concat($modinfo_pow_ac_outa/@val13,'&#160;'),
	    													concat($modinfo_pow_ac_outb/@val13,'&#160;')),concat($modinfo_pow_ac_outc/@val13,'&#160;'))"/>
	    	</xsl:when>
	    	<xsl:when test="string($ac_out_con_str_13)='1P'">
	    		<xsl:value-of select="$modinfo_pow_ac_outa/@val13 + $modinfo_pow_ac_outb/@val13 + $modinfo_pow_ac_outc/@val13"/>
	    	</xsl:when>
	      <xsl:otherwise><xsl:value-of select="'???'"/></xsl:otherwise>
	    </xsl:choose>
		</xsl:variable>
		<xsl:variable name="ac_out_power_consumption_14">
	    <xsl:choose>
	    	<xsl:when test="string($ac_out_con_str_14)='3P'">
	    		<xsl:value-of select="concat(concat(concat($modinfo_pow_ac_outa/@val14,'&#160;'),
	    													concat($modinfo_pow_ac_outb/@val14,'&#160;')),concat($modinfo_pow_ac_outc/@val14,'&#160;'))"/>
	    	</xsl:when>
	    	<xsl:when test="string($ac_out_con_str_14)='1P'">
	    		<xsl:value-of select="$modinfo_pow_ac_outa/@val14 + $modinfo_pow_ac_outb/@val14 + $modinfo_pow_ac_outc/@val14"/>
	    	</xsl:when>
	      <xsl:otherwise><xsl:value-of select="'???'"/></xsl:otherwise>
	    </xsl:choose>
		</xsl:variable>
		<xsl:variable name="ac_out_power_consumption_15">
	    <xsl:choose>
	    	<xsl:when test="string($ac_out_con_str_15)='3P'">
	    		<xsl:value-of select="concat(concat(concat($modinfo_pow_ac_outa/@val15,'&#160;'),
	    													concat($modinfo_pow_ac_outb/@val15,'&#160;')),concat($modinfo_pow_ac_outc/@val15,'&#160;'))"/>
	    	</xsl:when>
	    	<xsl:when test="string($ac_out_con_str_15)='1P'">
	    		<xsl:value-of select="$modinfo_pow_ac_outa/@val15 + $modinfo_pow_ac_outb/@val15 + $modinfo_pow_ac_outc/@val15"/>
	    	</xsl:when>
	      <xsl:otherwise><xsl:value-of select="'???'"/></xsl:otherwise>
	    </xsl:choose>
		</xsl:variable>
		<xsl:variable name="ac_out_power_consumption_16">
	    <xsl:choose>
	    	<xsl:when test="string($ac_out_con_str_16)='3P'">
	    		<xsl:value-of select="concat(concat(concat($modinfo_pow_ac_outa/@val16,'&#160;'),
	    													concat($modinfo_pow_ac_outb/@val16,'&#160;')),concat($modinfo_pow_ac_outc/@val16,'&#160;'))"/>
	    	</xsl:when>
	    	<xsl:when test="string($ac_out_con_str_16)='1P'">
	    		<xsl:value-of select="$modinfo_pow_ac_outa/@val16 + $modinfo_pow_ac_outb/@val16 + $modinfo_pow_ac_outc/@val16"/>
	    	</xsl:when>
	      <xsl:otherwise><xsl:value-of select="'???'"/></xsl:otherwise>
	    </xsl:choose>
		</xsl:variable>
		<xsl:variable name="ac_out_power_consumption_17">
	    <xsl:choose>
	    	<xsl:when test="string($ac_out_con_str_17)='3P'">
	    		<xsl:value-of select="concat(concat(concat($modinfo_pow_ac_outa/@val17,'&#160;'),
	    													concat($modinfo_pow_ac_outb/@val17,'&#160;')),concat($modinfo_pow_ac_outc/@val17,'&#160;'))"/>
	    	</xsl:when>
	    	<xsl:when test="string($ac_out_con_str_17)='1P'">
	    		<xsl:value-of select="$modinfo_pow_ac_outa/@val17 + $modinfo_pow_ac_outb/@val17 + $modinfo_pow_ac_outc/@val17"/>
	    	</xsl:when>
	      <xsl:otherwise><xsl:value-of select="'???'"/></xsl:otherwise>
	    </xsl:choose>
		</xsl:variable>
		<xsl:variable name="ac_out_power_consumption_18">
	    <xsl:choose>
	    	<xsl:when test="string($ac_out_con_str_18)='3P'">
	    		<xsl:value-of select="concat(concat(concat($modinfo_pow_ac_outa/@val18,'&#160;'),
	    													concat($modinfo_pow_ac_outb/@val18,'&#160;')),concat($modinfo_pow_ac_outc/@val18,'&#160;'))"/>
	    	</xsl:when>
	    	<xsl:when test="string($ac_out_con_str_18)='1P'">
	    		<xsl:value-of select="$modinfo_pow_ac_outa/@val18 + $modinfo_pow_ac_outb/@val18 + $modinfo_pow_ac_outc/@val18"/>
	    	</xsl:when>
	      <xsl:otherwise><xsl:value-of select="'???'"/></xsl:otherwise>
	    </xsl:choose>
		</xsl:variable>
		<xsl:variable name="ac_out_power_consumption_19">
	    <xsl:choose>
	    	<xsl:when test="string($ac_out_con_str_19)='3P'">
	    		<xsl:value-of select="concat(concat(concat($modinfo_pow_ac_outa/@val19,'&#160;'),
	    													concat($modinfo_pow_ac_outb/@val19,'&#160;')),concat($modinfo_pow_ac_outc/@val19,'&#160;'))"/>
	    	</xsl:when>
	    	<xsl:when test="string($ac_out_con_str_19)='1P'">
	    		<xsl:value-of select="$modinfo_pow_ac_outa/@val19 + $modinfo_pow_ac_outb/@val19 + $modinfo_pow_ac_outc/@val19"/>
	    	</xsl:when>
	      <xsl:otherwise><xsl:value-of select="'???'"/></xsl:otherwise>
	    </xsl:choose>
		</xsl:variable>
		<xsl:variable name="ac_out_power_consumption_20">
	    <xsl:choose>
	    	<xsl:when test="string($ac_out_con_str_20)='3P'">
	    		<xsl:value-of select="concat(concat(concat($modinfo_pow_ac_outa/@val20,'&#160;'),
	    													concat($modinfo_pow_ac_outb/@val20,'&#160;')),concat($modinfo_pow_ac_outc/@val20,'&#160;'))"/>
	    	</xsl:when>
	    	<xsl:when test="string($ac_out_con_str_20)='1P'">
	    		<xsl:value-of select="$modinfo_pow_ac_outa/@val20 + $modinfo_pow_ac_outb/@val20 + $modinfo_pow_ac_outc/@val20"/>
	    	</xsl:when>
	      <xsl:otherwise><xsl:value-of select="'???'"/></xsl:otherwise>
	    </xsl:choose>
		</xsl:variable>
		<xsl:variable name="ac_out_power_consumption_21">
	    <xsl:choose>
	    	<xsl:when test="string($ac_out_con_str_21)='3P'">
	    		<xsl:value-of select="concat(concat(concat($modinfo_pow_ac_outa/@val,'&#160;'),
	    													concat($modinfo_pow_ac_outb/@val21,'&#160;')),concat($modinfo_pow_ac_outc/@val21,'&#160;'))"/>
	    	</xsl:when>
	    	<xsl:when test="string($ac_out_con_str_21)='1P'">
	    		<xsl:value-of select="$modinfo_pow_ac_outa/@val21 + $modinfo_pow_ac_outb/@val21 + $modinfo_pow_ac_outc/@val21"/>
	    	</xsl:when>
	      <xsl:otherwise><xsl:value-of select="'???'"/></xsl:otherwise>
	    </xsl:choose>
		</xsl:variable>
		<xsl:variable name="ac_out_power_consumption_22">
	    <xsl:choose>
	    	<xsl:when test="string($ac_out_con_str_22)='3P'">
	    		<xsl:value-of select="concat(concat(concat($modinfo_pow_ac_outa/@val22,'&#160;'),
	    													concat($modinfo_pow_ac_outb/@val22,'&#160;')),concat($modinfo_pow_ac_outc/@val22,'&#160;'))"/>
	    	</xsl:when>
	    	<xsl:when test="string($ac_out_con_str_22)='1P'">
	    		<xsl:value-of select="$modinfo_pow_ac_outa/@val22 + $modinfo_pow_ac_outb/@val22 + $modinfo_pow_ac_outc/@val22"/>
	    	</xsl:when>
	      <xsl:otherwise><xsl:value-of select="'???'"/></xsl:otherwise>
	    </xsl:choose>
		</xsl:variable>
		<xsl:variable name="ac_out_power_consumption_23">
	    <xsl:choose>
	    	<xsl:when test="string($ac_out_con_str_23)='3P'">
	    		<xsl:value-of select="concat(concat(concat($modinfo_pow_ac_outa/@valv,'&#160;'),
	    													concat($modinfo_pow_ac_outb/@val23,'&#160;')),concat($modinfo_pow_ac_outc/@val23,'&#160;'))"/>
	    	</xsl:when>
	    	<xsl:when test="string($ac_out_con_str_23)='1P'">
	    		<xsl:value-of select="$modinfo_pow_ac_outa/@val23 + $modinfo_pow_ac_outb/@val23 + $modinfo_pow_ac_outc/@val23"/>
	    	</xsl:when>
	      <xsl:otherwise><xsl:value-of select="'???'"/></xsl:otherwise>
	    </xsl:choose>
		</xsl:variable>
		<xsl:variable name="ac_out_power_consumption_24">
	    <xsl:choose>
	    	<xsl:when test="string($ac_out_con_str_24)='3P'">
	    		<xsl:value-of select="concat(concat(concat($modinfo_pow_ac_outa/@val24,'&#160;'),
	    													concat($modinfo_pow_ac_outb/@val24,'&#160;')),concat($modinfo_pow_ac_outc/@val24,'&#160;'))"/>
	    	</xsl:when>
	    	<xsl:when test="string($ac_out_con_str_24)='1P'">
	    		<xsl:value-of select="$modinfo_pow_ac_outa/@val24 + $modinfo_pow_ac_outb/@val24 + $modinfo_pow_ac_outc/@val24"/>
	    	</xsl:when>
	      <xsl:otherwise><xsl:value-of select="'???'"/></xsl:otherwise>
	    </xsl:choose>
		</xsl:variable>
		<xsl:variable name="ac_out_power_consumption_25">
	    <xsl:choose>
	    	<xsl:when test="string($ac_out_con_str_25)='3P'">
	    		<xsl:value-of select="concat(concat(concat($modinfo_pow_ac_outa/@val25,'&#160;'),
	    													concat($modinfo_pow_ac_outb/@val25,'&#160;')),concat($modinfo_pow_ac_outc/@val25,'&#160;'))"/>
	    	</xsl:when>
	    	<xsl:when test="string($ac_out_con_str_25)='1P'">
	    		<xsl:value-of select="$modinfo_pow_ac_outa/@val25 + $modinfo_pow_ac_outb/@val25 + $modinfo_pow_ac_outc/@val25"/>
	    	</xsl:when>
	      <xsl:otherwise><xsl:value-of select="'???'"/></xsl:otherwise>
	    </xsl:choose>
		</xsl:variable>
		<xsl:variable name="ac_out_power_consumption_26">
	    <xsl:choose>
	    	<xsl:when test="string($ac_out_con_str_26)='3P'">
	    		<xsl:value-of select="concat(concat(concat($modinfo_pow_ac_outa/@val26,'&#160;'),
	    													concat($modinfo_pow_ac_outb/@val26,'&#160;')),concat($modinfo_pow_ac_outc/@val26,'&#160;'))"/>
	    	</xsl:when>
	    	<xsl:when test="string($ac_out_con_str_26)='1P'">
	    		<xsl:value-of select="$modinfo_pow_ac_outa/@val26 + $modinfo_pow_ac_outb/@val26 + $modinfo_pow_ac_outc/@val26"/>
	    	</xsl:when>
	      <xsl:otherwise><xsl:value-of select="'???'"/></xsl:otherwise>
	    </xsl:choose>
		</xsl:variable>
		<xsl:variable name="ac_out_power_consumption_27">
	    <xsl:choose>
	    	<xsl:when test="string($ac_out_con_str_27)='3P'">
	    		<xsl:value-of select="concat(concat(concat($modinfo_pow_ac_outa/@val27,'&#160;'),
	    													concat($modinfo_pow_ac_outb/@val27,'&#160;')),concat($modinfo_pow_ac_outc/@val27,'&#160;'))"/>
	    	</xsl:when>
	    	<xsl:when test="string($ac_out_con_str_27)='1P'">
	    		<xsl:value-of select="$modinfo_pow_ac_outa/@val27 + $modinfo_pow_ac_outb/@val27 + $modinfo_pow_ac_outc/@val27"/>
	    	</xsl:when>
	      <xsl:otherwise><xsl:value-of select="'???'"/></xsl:otherwise>
	    </xsl:choose>
		</xsl:variable>
		<xsl:variable name="ac_out_power_consumption_28">
	    <xsl:choose>
	    	<xsl:when test="string($ac_out_con_str_28)='3P'">
	    		<xsl:value-of select="concat(concat(concat($modinfo_pow_ac_outa/@val28,'&#160;'),
	    													concat($modinfo_pow_ac_outb/@val28,'&#160;')),concat($modinfo_pow_ac_outc/@val28,'&#160;'))"/>
	    	</xsl:when>
	    	<xsl:when test="string($ac_out_con_str_28)='1P'">
	    		<xsl:value-of select="$modinfo_pow_ac_outa/@val28 + $modinfo_pow_ac_outb/@val28 + $modinfo_pow_ac_outc/@val28"/>
	    	</xsl:when>
	      <xsl:otherwise><xsl:value-of select="'???'"/></xsl:otherwise>
	    </xsl:choose>
		</xsl:variable>
		<xsl:variable name="ac_out_power_consumption_29">
	    <xsl:choose>
	    	<xsl:when test="string($ac_out_con_str_29)='3P'">
	    		<xsl:value-of select="concat(concat(concat($modinfo_pow_ac_outa/@val29,'&#160;'),
	    													concat($modinfo_pow_ac_outb/@val29,'&#160;')),concat($modinfo_pow_ac_outc/@val29,'&#160;'))"/>
	    	</xsl:when>
	    	<xsl:when test="string($ac_out_con_str_29)='1P'">
	    		<xsl:value-of select="$modinfo_pow_ac_outa/@val29 + $modinfo_pow_ac_outb/@val29 + $modinfo_pow_ac_outc/@val29"/>
	    	</xsl:when>
	      <xsl:otherwise><xsl:value-of select="'???'"/></xsl:otherwise>
	    </xsl:choose>
		</xsl:variable>
		<xsl:variable name="ac_out_power_consumption_30">
	    <xsl:choose>
	    	<xsl:when test="string($ac_out_con_str_30)='3P'">
	    		<xsl:value-of select="concat(concat(concat($modinfo_pow_ac_outa/@val30,'&#160;'),
	    													concat($modinfo_pow_ac_outb/@val30,'&#160;')),concat($modinfo_pow_ac_outc/@val30,'&#160;'))"/>
	    	</xsl:when>
	    	<xsl:when test="string($ac_out_con_str_30)='1P'">
	    		<xsl:value-of select="$modinfo_pow_ac_outa/@val30 + $modinfo_pow_ac_outb/@val30 + $modinfo_pow_ac_outc/@val30"/>
	    	</xsl:when>
	      <xsl:otherwise><xsl:value-of select="'???'"/></xsl:otherwise>
	    </xsl:choose>
		</xsl:variable>
		<xsl:variable name="ac_out_power_consumption_31">
	    <xsl:choose>
	    	<xsl:when test="string($ac_out_con_str_31)='3P'">
	    		<xsl:value-of select="concat(concat(concat($modinfo_pow_ac_outa/@val31,'&#160;'),
	    													concat($modinfo_pow_ac_outb/@val31,'&#160;')),concat($modinfo_pow_ac_outc/@val31,'&#160;'))"/>
	    	</xsl:when>
	    	<xsl:when test="string($ac_out_con_str_31)='1P'">
	    		<xsl:value-of select="$modinfo_pow_ac_outa/@val31 + $modinfo_pow_ac_outb/@val31 + $modinfo_pow_ac_outc/@val31"/>
	    	</xsl:when>
	      <xsl:otherwise><xsl:value-of select="'???'"/></xsl:otherwise>
	    </xsl:choose>
		</xsl:variable>
		<xsl:variable name="ac_out_power_consumption_32">
	    <xsl:choose>
	    	<xsl:when test="string($ac_out_con_str_32)='3P'">
	    		<xsl:value-of select="concat(concat(concat($modinfo_pow_ac_outa/@val32,'&#160;'),
	    													concat($modinfo_pow_ac_outb/@val32,'&#160;')),concat($modinfo_pow_ac_outc/@val32,'&#160;'))"/>
	    	</xsl:when>
	    	<xsl:when test="string($ac_out_con_str_32)='1P'">
	    		<xsl:value-of select="$modinfo_pow_ac_outa/@val32 + $modinfo_pow_ac_outb/@val32 + $modinfo_pow_ac_outc/@val32"/>
	    	</xsl:when>
	      <xsl:otherwise><xsl:value-of select="'???'"/></xsl:otherwise>
	    </xsl:choose>
		</xsl:variable>

		<!--Ac Out power (VA) consumption variables -->
		<xsl:variable name="ac_out_power_va_consumption_1">
	    <xsl:choose>
	    	<xsl:when test="string($ac_out_con_str_1)='3P'">
	    		<xsl:value-of select="concat(concat(concat($modinfo_pow_va_ac_outa/@val,'&#160;'),
	    													concat($modinfo_pow_va_ac_outb/@val,'&#160;')),concat($modinfo_pow_va_ac_outc/@val,'&#160;'))"/>
	    	</xsl:when>
	    	<xsl:when test="string($ac_out_con_str_1)='1P'">
	    		<xsl:value-of select="$modinfo_pow_va_ac_outa/@val + $modinfo_pow_va_ac_outb/@val + $modinfo_pow_va_ac_outc/@val"/>
	    	</xsl:when>
	      <xsl:otherwise><xsl:value-of select="'???'"/></xsl:otherwise>
	    </xsl:choose>
		</xsl:variable>
		<xsl:variable name="ac_out_power_va_consumption_2">
	    <xsl:choose>
	    	<xsl:when test="string($ac_out_con_str_2)='3P'">
	    		<xsl:value-of select="concat(concat(concat($modinfo_pow_va_ac_outa/@val2,'&#160;'),
	    													concat($modinfo_pow_va_ac_outb/@val2,'&#160;')),concat($modinfo_pow_va_ac_outc/@val2,'&#160;'))"/>
	    	</xsl:when>
	    	<xsl:when test="string($ac_out_con_str_2)='1P'">
	    		<xsl:value-of select="$modinfo_pow_va_ac_outa/@val2 + $modinfo_pow_va_ac_outb/@val2 + $modinfo_pow_va_ac_outc/@val2"/>
	    	</xsl:when>
	      <xsl:otherwise><xsl:value-of select="'???'"/></xsl:otherwise>
	    </xsl:choose>
		</xsl:variable>
		<xsl:variable name="ac_out_power_va_consumption_3">
	    <xsl:choose>
	    	<xsl:when test="string($ac_out_con_str_3)='3P'">
	    		<xsl:value-of select="concat(concat(concat($modinfo_pow_va_ac_outa/@val3,'&#160;'),
	    													concat($modinfo_pow_va_ac_outb/@val3,'&#160;')),concat($modinfo_pow_va_ac_outc/@val3,'&#160;'))"/>
	    	</xsl:when>
	    	<xsl:when test="string($ac_out_con_str_3)='1P'">
	    		<xsl:value-of select="$modinfo_pow_va_ac_outa/@val3 + $modinfo_pow_va_ac_outb/@val3 + $modinfo_pow_va_ac_outc/@val3"/>
	    	</xsl:when>
	      <xsl:otherwise><xsl:value-of select="'???'"/></xsl:otherwise>
	    </xsl:choose>
		</xsl:variable>
		<xsl:variable name="ac_out_power_va_consumption_4">
	    <xsl:choose>
	    	<xsl:when test="string($ac_out_con_str_4)='3P'">
	    		<xsl:value-of select="concat(concat(concat($modinfo_pow_va_ac_outa/@val4,'&#160;'),
	    													concat($modinfo_pow_va_ac_outb/@val4,'&#160;')),concat($modinfo_pow_va_ac_outc/@val4,'&#160;'))"/>
	    	</xsl:when>
	    	<xsl:when test="string($ac_out_con_str_4)='1P'">
	    		<xsl:value-of select="$modinfo_pow_va_ac_outa/@val4 + $modinfo_pow_va_ac_outb/@val4 + $modinfo_pow_va_ac_outc/@val4"/>
	    	</xsl:when>
	      <xsl:otherwise><xsl:value-of select="'???'"/></xsl:otherwise>
	    </xsl:choose>
		</xsl:variable>
		<xsl:variable name="ac_out_power_va_consumption_5">
	    <xsl:choose>
	    	<xsl:when test="string($ac_out_con_str_5)='3P'">
	    		<xsl:value-of select="concat(concat(concat($modinfo_pow_va_ac_outa/@val5,'&#160;'),
	    													concat($modinfo_pow_va_ac_outb/@val5,'&#160;')),concat($modinfo_pow_va_ac_outc/@val5,'&#160;'))"/>
	    	</xsl:when>
	    	<xsl:when test="string($ac_out_con_str_5)='1P'">
	    		<xsl:value-of select="$modinfo_pow_va_ac_outa/@val5 + $modinfo_pow_va_ac_outb/@val5 + $modinfo_pow_va_ac_outc/@val5"/>
	    	</xsl:when>
	      <xsl:otherwise><xsl:value-of select="'???'"/></xsl:otherwise>
	    </xsl:choose>
		</xsl:variable>
		<xsl:variable name="ac_out_power_va_consumption_6">
	    <xsl:choose>
	    	<xsl:when test="string($ac_out_con_str_6)='3P'">
	    		<xsl:value-of select="concat(concat(concat($modinfo_pow_va_ac_outa/@val6,'&#160;'),
	    													concat($modinfo_pow_va_ac_outb/@val6,'&#160;')),concat($modinfo_pow_va_ac_outc/@val6,'&#160;'))"/>
	    	</xsl:when>
	    	<xsl:when test="string($ac_out_con_str_6)='1P'">
	    		<xsl:value-of select="$modinfo_pow_va_ac_outa/@val6 + $modinfo_pow_va_ac_outb/@val6 + $modinfo_pow_va_ac_outc/@val6"/>
	    	</xsl:when>
	      <xsl:otherwise><xsl:value-of select="'???'"/></xsl:otherwise>
	    </xsl:choose>
		</xsl:variable>
		<xsl:variable name="ac_out_power_va_consumption_7">
	    <xsl:choose>
	    	<xsl:when test="string($ac_out_con_str_7)='3P'">
	    		<xsl:value-of select="concat(concat(concat($modinfo_pow_va_ac_outa/@val7,'&#160;'),
	    													concat($modinfo_pow_va_ac_outb/@val7,'&#160;')),concat($modinfo_pow_va_ac_outc/@val7,'&#160;'))"/>
	    	</xsl:when>
	    	<xsl:when test="string($ac_out_con_str_7)='1P'">
	    		<xsl:value-of select="$modinfo_pow_va_ac_outa/@val7 + $modinfo_pow_va_ac_outb/@val7 + $modinfo_pow_va_ac_outc/@val7"/>
	    	</xsl:when>
	      <xsl:otherwise><xsl:value-of select="'???'"/></xsl:otherwise>
	    </xsl:choose>
		</xsl:variable>
		<xsl:variable name="ac_out_power_va_consumption_8">
	    <xsl:choose>
	    	<xsl:when test="string($ac_out_con_str_8)='3P'">
	    		<xsl:value-of select="concat(concat(concat($modinfo_pow_va_ac_outa/@val8,'&#160;'),
	    													concat($modinfo_pow_va_ac_outb/@val8,'&#160;')),concat($modinfo_pow_va_ac_outc/@val8,'&#160;'))"/>
	    	</xsl:when>
	    	<xsl:when test="string($ac_out_con_str_8)='1P'">
	    		<xsl:value-of select="$modinfo_pow_va_ac_outa/@val8 + $modinfo_pow_va_ac_outb/@val8 + $modinfo_pow_va_ac_outc/@val8"/>
	    	</xsl:when>
	      <xsl:otherwise><xsl:value-of select="'???'"/></xsl:otherwise>
	    </xsl:choose>
		</xsl:variable>
		<xsl:variable name="ac_out_power_va_consumption_9">
	    <xsl:choose>
	    	<xsl:when test="string($ac_out_con_str_9)='3P'">
	    		<xsl:value-of select="concat(concat(concat($modinfo_pow_va_ac_outa/@val9,'&#160;'),
	    													concat($modinfo_pow_va_ac_outb/@val9,'&#160;')),concat($modinfo_pow_va_ac_outc/@val9,'&#160;'))"/>
	    	</xsl:when>
	    	<xsl:when test="string($ac_out_con_str_9)='1P'">
	    		<xsl:value-of select="$modinfo_pow_va_ac_outa/@val9 + $modinfo_pow_va_ac_outb/@val9 + $modinfo_pow_va_ac_outc/@val9"/>
	    	</xsl:when>
	      <xsl:otherwise><xsl:value-of select="'???'"/></xsl:otherwise>
	    </xsl:choose>
		</xsl:variable>
		<xsl:variable name="ac_out_power_va_consumption_10">
	    <xsl:choose>
	    	<xsl:when test="string($ac_out_con_str_10)='3P'">
	    		<xsl:value-of select="concat(concat(concat($modinfo_pow_va_ac_outa/@val10,'&#160;'),
	    													concat($modinfo_pow_va_ac_outb/@val10,'&#160;')),concat($modinfo_pow_va_ac_outc/@val10,'&#160;'))"/>
	    	</xsl:when>
	    	<xsl:when test="string($ac_out_con_str_10)='1P'">
	    		<xsl:value-of select="$modinfo_pow_va_ac_outa/@val10 + $modinfo_pow_va_ac_outb/@val10 + $modinfo_pow_va_ac_outc/@val10"/>
	    	</xsl:when>
	      <xsl:otherwise><xsl:value-of select="'???'"/></xsl:otherwise>
	    </xsl:choose>
		</xsl:variable>
		<xsl:variable name="ac_out_power_va_consumption_11">
	    <xsl:choose>
	    	<xsl:when test="string($ac_out_con_str_11)='3P'">
	    		<xsl:value-of select="concat(concat(concat($modinfo_pow_va_ac_outa/@val11,'&#160;'),
	    													concat($modinfo_pow_va_ac_outb/@val11,'&#160;')),concat($modinfo_pow_va_ac_outc/@val11,'&#160;'))"/>
	    	</xsl:when>
	    	<xsl:when test="string($ac_out_con_str_11)='1P'">
	    		<xsl:value-of select="$modinfo_pow_va_ac_outa/@val11 + $modinfo_pow_va_ac_outb/@val11 + $modinfo_pow_va_ac_outc/@val11"/>
	    	</xsl:when>
	      <xsl:otherwise><xsl:value-of select="'???'"/></xsl:otherwise>
	    </xsl:choose>
		</xsl:variable>
		<xsl:variable name="ac_out_power_va_consumption_12">
	    <xsl:choose>
	    	<xsl:when test="string($ac_out_con_str_12)='3P'">
	    		<xsl:value-of select="concat(concat(concat($modinfo_pow_va_ac_outa/@val12,'&#160;'),
	    													concat($modinfo_pow_va_ac_outb/@val12,'&#160;')),concat($modinfo_pow_va_ac_outc/@val12,'&#160;'))"/>
	    	</xsl:when>
	    	<xsl:when test="string($ac_out_con_str_12)='1P'">
	    		<xsl:value-of select="$modinfo_pow_va_ac_outa/@val12 + $modinfo_pow_va_ac_outb/@val12 + $modinfo_pow_va_ac_outc/@val12"/>
	    	</xsl:when>
	      <xsl:otherwise><xsl:value-of select="'???'"/></xsl:otherwise>
	    </xsl:choose>
		</xsl:variable>
		<xsl:variable name="ac_out_power_va_consumption_13">
	    <xsl:choose>
	    	<xsl:when test="string($ac_out_con_str_13)='3P'">
	    		<xsl:value-of select="concat(concat(concat($modinfo_pow_va_ac_outa/@val13,'&#160;'),
	    													concat($modinfo_pow_va_ac_outb/@val13,'&#160;')),concat($modinfo_pow_va_ac_outc/@val13,'&#160;'))"/>
	    	</xsl:when>
	    	<xsl:when test="string($ac_out_con_str_13)='1P'">
	    		<xsl:value-of select="$modinfo_pow_va_ac_outa/@val13 + $modinfo_pow_va_ac_outb/@val13 + $modinfo_pow_va_ac_outc/@val13"/>
	    	</xsl:when>
	      <xsl:otherwise><xsl:value-of select="'???'"/></xsl:otherwise>
	    </xsl:choose>
		</xsl:variable>
		<xsl:variable name="ac_out_power_va_consumption_14">
	    <xsl:choose>
	    	<xsl:when test="string($ac_out_con_str_14)='3P'">
	    		<xsl:value-of select="concat(concat(concat($modinfo_pow_va_ac_outa/@val14,'&#160;'),
	    													concat($modinfo_pow_va_ac_outb/@val14,'&#160;')),concat($modinfo_pow_va_ac_outc/@val14,'&#160;'))"/>
	    	</xsl:when>
	    	<xsl:when test="string($ac_out_con_str_14)='1P'">
	    		<xsl:value-of select="$modinfo_pow_va_ac_outa/@val14 + $modinfo_pow_va_ac_outb/@val14 + $modinfo_pow_va_ac_outc/@val14"/>
	    	</xsl:when>
	      <xsl:otherwise><xsl:value-of select="'???'"/></xsl:otherwise>
	    </xsl:choose>
		</xsl:variable>
		<xsl:variable name="ac_out_power_va_consumption_15">
	    <xsl:choose>
	    	<xsl:when test="string($ac_out_con_str_15)='3P'">
	    		<xsl:value-of select="concat(concat(concat($modinfo_pow_va_ac_outa/@val15,'&#160;'),
	    													concat($modinfo_pow_va_ac_outb/@val15,'&#160;')),concat($modinfo_pow_va_ac_outc/@val15,'&#160;'))"/>
	    	</xsl:when>
	    	<xsl:when test="string($ac_out_con_str_15)='1P'">
	    		<xsl:value-of select="$modinfo_pow_va_ac_outa/@val15 + $modinfo_pow_va_ac_outb/@val15 + $modinfo_pow_va_ac_outc/@val15"/>
	    	</xsl:when>
	      <xsl:otherwise><xsl:value-of select="'???'"/></xsl:otherwise>
	    </xsl:choose>
		</xsl:variable>
		<xsl:variable name="ac_out_power_va_consumption_16">
	    <xsl:choose>
	    	<xsl:when test="string($ac_out_con_str_16)='3P'">
	    		<xsl:value-of select="concat(concat(concat($modinfo_pow_va_ac_outa/@val16,'&#160;'),
	    													concat($modinfo_pow_va_ac_outb/@val16,'&#160;')),concat($modinfo_pow_va_ac_outc/@val16,'&#160;'))"/>
	    	</xsl:when>
	    	<xsl:when test="string($ac_out_con_str_16)='1P'">
	    		<xsl:value-of select="$modinfo_pow_va_ac_outa/@val16 + $modinfo_pow_va_ac_outb/@val16 + $modinfo_pow_va_ac_outc/@val16"/>
	    	</xsl:when>
	      <xsl:otherwise><xsl:value-of select="'???'"/></xsl:otherwise>
	    </xsl:choose>
		</xsl:variable>
		<xsl:variable name="ac_out_power_va_consumption_17">
	    <xsl:choose>
	    	<xsl:when test="string($ac_out_con_str_17)='3P'">
	    		<xsl:value-of select="concat(concat(concat($modinfo_pow_va_ac_outa/@val17,'&#160;'),
	    													concat($modinfo_pow_va_ac_outb/@val17,'&#160;')),concat($modinfo_pow_va_ac_outc/@val17,'&#160;'))"/>
	    	</xsl:when>
	    	<xsl:when test="string($ac_out_con_str_17)='1P'">
	    		<xsl:value-of select="$modinfo_pow_va_ac_outa/@val17 + $modinfo_pow_va_ac_outb/@val17 + $modinfo_pow_va_ac_outc/@val17"/>
	    	</xsl:when>
	      <xsl:otherwise><xsl:value-of select="'???'"/></xsl:otherwise>
	    </xsl:choose>
		</xsl:variable>
		<xsl:variable name="ac_out_power_va_consumption_18">
	    <xsl:choose>
	    	<xsl:when test="string($ac_out_con_str_18)='3P'">
	    		<xsl:value-of select="concat(concat(concat($modinfo_pow_va_ac_outa/@val18,'&#160;'),
	    													concat($modinfo_pow_va_ac_outb/@val18,'&#160;')),concat($modinfo_pow_va_ac_outc/@val18,'&#160;'))"/>
	    	</xsl:when>
	    	<xsl:when test="string($ac_out_con_str_18)='1P'">
	    		<xsl:value-of select="$modinfo_pow_va_ac_outa/@val18 + $modinfo_pow_va_ac_outb/@val18 + $modinfo_pow_va_ac_outc/@val18"/>
	    	</xsl:when>
	      <xsl:otherwise><xsl:value-of select="'???'"/></xsl:otherwise>
	    </xsl:choose>
		</xsl:variable>
		<xsl:variable name="ac_out_power_va_consumption_19">
	    <xsl:choose>
	    	<xsl:when test="string($ac_out_con_str_19)='3P'">
	    		<xsl:value-of select="concat(concat(concat($modinfo_pow_va_ac_outa/@val19,'&#160;'),
	    													concat($modinfo_pow_va_ac_outb/@val19,'&#160;')),concat($modinfo_pow_va_ac_outc/@val19,'&#160;'))"/>
	    	</xsl:when>
	    	<xsl:when test="string($ac_out_con_str_19)='1P'">
	    		<xsl:value-of select="$modinfo_pow_va_ac_outa/@val19 + $modinfo_pow_va_ac_outb/@val19 + $modinfo_pow_va_ac_outc/@val19"/>
	    	</xsl:when>
	      <xsl:otherwise><xsl:value-of select="'???'"/></xsl:otherwise>
	    </xsl:choose>
		</xsl:variable>
		<xsl:variable name="ac_out_power_va_consumption_20">
	    <xsl:choose>
	    	<xsl:when test="string($ac_out_con_str_20)='3P'">
	    		<xsl:value-of select="concat(concat(concat($modinfo_pow_va_ac_outa/@val20,'&#160;'),
	    													concat($modinfo_pow_va_ac_outb/@val20,'&#160;')),concat($modinfo_pow_va_ac_outc/@val20,'&#160;'))"/>
	    	</xsl:when>
	    	<xsl:when test="string($ac_out_con_str_20)='1P'">
	    		<xsl:value-of select="$modinfo_pow_va_ac_outa/@val20 + $modinfo_pow_va_ac_outb/@val20 + $modinfo_pow_va_ac_outc/@val20"/>
	    	</xsl:when>
	      <xsl:otherwise><xsl:value-of select="'???'"/></xsl:otherwise>
	    </xsl:choose>
		</xsl:variable>
		<xsl:variable name="ac_out_power_va_consumption_21">
	    <xsl:choose>
	    	<xsl:when test="string($ac_out_con_str_21)='3P'">
	    		<xsl:value-of select="concat(concat(concat($modinfo_pow_va_ac_outa/@val,'&#160;'),
	    													concat($modinfo_pow_va_ac_outb/@val21,'&#160;')),concat($modinfo_pow_va_ac_outc/@val21,'&#160;'))"/>
	    	</xsl:when>
	    	<xsl:when test="string($ac_out_con_str_21)='1P'">
	    		<xsl:value-of select="$modinfo_pow_va_ac_outa/@val21 + $modinfo_pow_va_ac_outb/@val21 + $modinfo_pow_va_ac_outc/@val21"/>
	    	</xsl:when>
	      <xsl:otherwise><xsl:value-of select="'???'"/></xsl:otherwise>
	    </xsl:choose>
		</xsl:variable>
		<xsl:variable name="ac_out_power_va_consumption_22">
	    <xsl:choose>
	    	<xsl:when test="string($ac_out_con_str_22)='3P'">
	    		<xsl:value-of select="concat(concat(concat($modinfo_pow_va_ac_outa/@val22,'&#160;'),
	    													concat($modinfo_pow_va_ac_outb/@val22,'&#160;')),concat($modinfo_pow_va_ac_outc/@val22,'&#160;'))"/>
	    	</xsl:when>
	    	<xsl:when test="string($ac_out_con_str_22)='1P'">
	    		<xsl:value-of select="$modinfo_pow_va_ac_outa/@val22 + $modinfo_pow_va_ac_outb/@val22 + $modinfo_pow_va_ac_outc/@val22"/>
	    	</xsl:when>
	      <xsl:otherwise><xsl:value-of select="'???'"/></xsl:otherwise>
	    </xsl:choose>
		</xsl:variable>
		<xsl:variable name="ac_out_power_va_consumption_23">
	    <xsl:choose>
	    	<xsl:when test="string($ac_out_con_str_23)='3P'">
	    		<xsl:value-of select="concat(concat(concat($modinfo_pow_va_ac_outa/@valv,'&#160;'),
	    													concat($modinfo_pow_va_ac_outb/@val23,'&#160;')),concat($modinfo_pow_va_ac_outc/@val23,'&#160;'))"/>
	    	</xsl:when>
	    	<xsl:when test="string($ac_out_con_str_23)='1P'">
	    		<xsl:value-of select="$modinfo_pow_va_ac_outa/@val23 + $modinfo_pow_va_ac_outb/@val23 + $modinfo_pow_va_ac_outc/@val23"/>
	    	</xsl:when>
	      <xsl:otherwise><xsl:value-of select="'???'"/></xsl:otherwise>
	    </xsl:choose>
		</xsl:variable>
		<xsl:variable name="ac_out_power_va_consumption_24">
	    <xsl:choose>
	    	<xsl:when test="string($ac_out_con_str_24)='3P'">
	    		<xsl:value-of select="concat(concat(concat($modinfo_pow_va_ac_outa/@val24,'&#160;'),
	    													concat($modinfo_pow_va_ac_outb/@val24,'&#160;')),concat($modinfo_pow_va_ac_outc/@val24,'&#160;'))"/>
	    	</xsl:when>
	    	<xsl:when test="string($ac_out_con_str_24)='1P'">
	    		<xsl:value-of select="$modinfo_pow_va_ac_outa/@val24 + $modinfo_pow_va_ac_outb/@val24 + $modinfo_pow_va_ac_outc/@val24"/>
	    	</xsl:when>
	      <xsl:otherwise><xsl:value-of select="'???'"/></xsl:otherwise>
	    </xsl:choose>
		</xsl:variable>
		<xsl:variable name="ac_out_power_va_consumption_25">
	    <xsl:choose>
	    	<xsl:when test="string($ac_out_con_str_25)='3P'">
	    		<xsl:value-of select="concat(concat(concat($modinfo_pow_va_ac_outa/@val25,'&#160;'),
	    													concat($modinfo_pow_va_ac_outb/@val25,'&#160;')),concat($modinfo_pow_va_ac_outc/@val25,'&#160;'))"/>
	    	</xsl:when>
	    	<xsl:when test="string($ac_out_con_str_25)='1P'">
	    		<xsl:value-of select="$modinfo_pow_va_ac_outa/@val25 + $modinfo_pow_va_ac_outb/@val25 + $modinfo_pow_va_ac_outc/@val25"/>
	    	</xsl:when>
	      <xsl:otherwise><xsl:value-of select="'???'"/></xsl:otherwise>
	    </xsl:choose>
		</xsl:variable>
		<xsl:variable name="ac_out_power_va_consumption_26">
	    <xsl:choose>
	    	<xsl:when test="string($ac_out_con_str_26)='3P'">
	    		<xsl:value-of select="concat(concat(concat($modinfo_pow_va_ac_outa/@val26,'&#160;'),
	    													concat($modinfo_pow_va_ac_outb/@val26,'&#160;')),concat($modinfo_pow_va_ac_outc/@val26,'&#160;'))"/>
	    	</xsl:when>
	    	<xsl:when test="string($ac_out_con_str_26)='1P'">
	    		<xsl:value-of select="$modinfo_pow_va_ac_outa/@val26 + $modinfo_pow_va_ac_outb/@val26 + $modinfo_pow_va_ac_outc/@val26"/>
	    	</xsl:when>
	      <xsl:otherwise><xsl:value-of select="'???'"/></xsl:otherwise>
	    </xsl:choose>
		</xsl:variable>
		<xsl:variable name="ac_out_power_va_consumption_27">
	    <xsl:choose>
	    	<xsl:when test="string($ac_out_con_str_27)='3P'">
	    		<xsl:value-of select="concat(concat(concat($modinfo_pow_va_ac_outa/@val27,'&#160;'),
	    													concat($modinfo_pow_va_ac_outb/@val27,'&#160;')),concat($modinfo_pow_va_ac_outc/@val27,'&#160;'))"/>
	    	</xsl:when>
	    	<xsl:when test="string($ac_out_con_str_27)='1P'">
	    		<xsl:value-of select="$modinfo_pow_va_ac_outa/@val27 + $modinfo_pow_va_ac_outb/@val27 + $modinfo_pow_va_ac_outc/@val27"/>
	    	</xsl:when>
	      <xsl:otherwise><xsl:value-of select="'???'"/></xsl:otherwise>
	    </xsl:choose>
		</xsl:variable>
		<xsl:variable name="ac_out_power_va_consumption_28">
	    <xsl:choose>
	    	<xsl:when test="string($ac_out_con_str_28)='3P'">
	    		<xsl:value-of select="concat(concat(concat($modinfo_pow_va_ac_outa/@val28,'&#160;'),
	    													concat($modinfo_pow_va_ac_outb/@val28,'&#160;')),concat($modinfo_pow_va_ac_outc/@val28,'&#160;'))"/>
	    	</xsl:when>
	    	<xsl:when test="string($ac_out_con_str_28)='1P'">
	    		<xsl:value-of select="$modinfo_pow_va_ac_outa/@val28 + $modinfo_pow_va_ac_outb/@val28 + $modinfo_pow_va_ac_outc/@val28"/>
	    	</xsl:when>
	      <xsl:otherwise><xsl:value-of select="'???'"/></xsl:otherwise>
	    </xsl:choose>
		</xsl:variable>
		<xsl:variable name="ac_out_power_va_consumption_29">
	    <xsl:choose>
	    	<xsl:when test="string($ac_out_con_str_29)='3P'">
	    		<xsl:value-of select="concat(concat(concat($modinfo_pow_va_ac_outa/@val29,'&#160;'),
	    													concat($modinfo_pow_va_ac_outb/@val29,'&#160;')),concat($modinfo_pow_va_ac_outc/@val29,'&#160;'))"/>
	    	</xsl:when>
	    	<xsl:when test="string($ac_out_con_str_29)='1P'">
	    		<xsl:value-of select="$modinfo_pow_va_ac_outa/@val29 + $modinfo_pow_va_ac_outb/@val29 + $modinfo_pow_va_ac_outc/@val29"/>
	    	</xsl:when>
	      <xsl:otherwise><xsl:value-of select="'???'"/></xsl:otherwise>
	    </xsl:choose>
		</xsl:variable>
		<xsl:variable name="ac_out_power_va_consumption_30">
	    <xsl:choose>
	    	<xsl:when test="string($ac_out_con_str_30)='3P'">
	    		<xsl:value-of select="concat(concat(concat($modinfo_pow_va_ac_outa/@val30,'&#160;'),
	    													concat($modinfo_pow_va_ac_outb/@val30,'&#160;')),concat($modinfo_pow_va_ac_outc/@val30,'&#160;'))"/>
	    	</xsl:when>
	    	<xsl:when test="string($ac_out_con_str_30)='1P'">
	    		<xsl:value-of select="$modinfo_pow_va_ac_outa/@val30 + $modinfo_pow_va_ac_outb/@val30 + $modinfo_pow_va_ac_outc/@val30"/>
	    	</xsl:when>
	      <xsl:otherwise><xsl:value-of select="'???'"/></xsl:otherwise>
	    </xsl:choose>
		</xsl:variable>
		<xsl:variable name="ac_out_power_va_consumption_31">
	    <xsl:choose>
	    	<xsl:when test="string($ac_out_con_str_31)='3P'">
	    		<xsl:value-of select="concat(concat(concat($modinfo_pow_va_ac_outa/@val31,'&#160;'),
	    													concat($modinfo_pow_va_ac_outb/@val31,'&#160;')),concat($modinfo_pow_va_ac_outc/@val31,'&#160;'))"/>
	    	</xsl:when>
	    	<xsl:when test="string($ac_out_con_str_31)='1P'">
	    		<xsl:value-of select="$modinfo_pow_va_ac_outa/@val31 + $modinfo_pow_va_ac_outb/@val31 + $modinfo_pow_va_ac_outc/@val31"/>
	    	</xsl:when>
	      <xsl:otherwise><xsl:value-of select="'???'"/></xsl:otherwise>
	    </xsl:choose>
		</xsl:variable>
		<xsl:variable name="ac_out_power_va_consumption_32">
	    <xsl:choose>
	    	<xsl:when test="string($ac_out_con_str_32)='3P'">
	    		<xsl:value-of select="concat(concat(concat($modinfo_pow_va_ac_outa/@val32,'&#160;'),
	    													concat($modinfo_pow_va_ac_outb/@val32,'&#160;')),concat($modinfo_pow_va_ac_outc/@val32,'&#160;'))"/>
	    	</xsl:when>
	    	<xsl:when test="string($ac_out_con_str_32)='1P'">
	    		<xsl:value-of select="$modinfo_pow_va_ac_outa/@val32 + $modinfo_pow_va_ac_outb/@val32 + $modinfo_pow_va_ac_outc/@val32"/>
	    	</xsl:when>
	      <xsl:otherwise><xsl:value-of select="'???'"/></xsl:otherwise>
	    </xsl:choose>
		</xsl:variable>

		<!--Modules status-->
		<xsl:variable name="mod_status_1">
			<xsl:call-template name="get-mod-status-string"><xsl:with-param name="mod_status" select="$mod_status/@val"/></xsl:call-template>
		</xsl:variable>
		<xsl:variable name="mod_status_2">
			<xsl:call-template name="get-mod-status-string"><xsl:with-param name="mod_status" select="$mod_status/@val2"/></xsl:call-template>
		</xsl:variable>
		<xsl:variable name="mod_status_3">
			<xsl:call-template name="get-mod-status-string"><xsl:with-param name="mod_status" select="$mod_status/@val3"/></xsl:call-template>
		</xsl:variable>
		<xsl:variable name="mod_status_4">
			<xsl:call-template name="get-mod-status-string"><xsl:with-param name="mod_status" select="$mod_status/@val4"/></xsl:call-template>
		</xsl:variable>
		<xsl:variable name="mod_status_5">
			<xsl:call-template name="get-mod-status-string"><xsl:with-param name="mod_status" select="$mod_status/@val5"/></xsl:call-template>
		</xsl:variable>
		<xsl:variable name="mod_status_6">
			<xsl:call-template name="get-mod-status-string"><xsl:with-param name="mod_status" select="$mod_status/@val6"/></xsl:call-template>
		</xsl:variable>
		<xsl:variable name="mod_status_7">
			<xsl:call-template name="get-mod-status-string"><xsl:with-param name="mod_status" select="$mod_status/@val7"/></xsl:call-template>
		</xsl:variable>
		<xsl:variable name="mod_status_8">
			<xsl:call-template name="get-mod-status-string"><xsl:with-param name="mod_status" select="$mod_status/@val8"/></xsl:call-template>
		</xsl:variable>
		<xsl:variable name="mod_status_9">
			<xsl:call-template name="get-mod-status-string"><xsl:with-param name="mod_status" select="$mod_status/@val9"/></xsl:call-template>
		</xsl:variable>
		<xsl:variable name="mod_status_10">
			<xsl:call-template name="get-mod-status-string"><xsl:with-param name="mod_status" select="$mod_status/@val10"/></xsl:call-template>
		</xsl:variable>
		<xsl:variable name="mod_status_11">
			<xsl:call-template name="get-mod-status-string"><xsl:with-param name="mod_status" select="$mod_status/@val11"/></xsl:call-template>
		</xsl:variable>
		<xsl:variable name="mod_status_12">
			<xsl:call-template name="get-mod-status-string"><xsl:with-param name="mod_status" select="$mod_status/@val12"/></xsl:call-template>
		</xsl:variable>
		<xsl:variable name="mod_status_13">
			<xsl:call-template name="get-mod-status-string"><xsl:with-param name="mod_status" select="$mod_status/@val13"/></xsl:call-template>
		</xsl:variable>
		<xsl:variable name="mod_status_14">
			<xsl:call-template name="get-mod-status-string"><xsl:with-param name="mod_status" select="$mod_status/@val14"/></xsl:call-template>
		</xsl:variable>
		<xsl:variable name="mod_status_15">
			<xsl:call-template name="get-mod-status-string"><xsl:with-param name="mod_status" select="$mod_status/@val15"/></xsl:call-template>
		</xsl:variable>
		<xsl:variable name="mod_status_16">
			<xsl:call-template name="get-mod-status-string"><xsl:with-param name="mod_status" select="$mod_status/@val16"/></xsl:call-template>
		</xsl:variable>
		<xsl:variable name="mod_status_17">
			<xsl:call-template name="get-mod-status-string"><xsl:with-param name="mod_status" select="$mod_status/@val17"/></xsl:call-template>
		</xsl:variable>
		<xsl:variable name="mod_status_18">
			<xsl:call-template name="get-mod-status-string"><xsl:with-param name="mod_status" select="$mod_status/@val18"/></xsl:call-template>
		</xsl:variable>
		<xsl:variable name="mod_status_19">
			<xsl:call-template name="get-mod-status-string"><xsl:with-param name="mod_status" select="$mod_status/@val19"/></xsl:call-template>
		</xsl:variable>
		<xsl:variable name="mod_status_20">
			<xsl:call-template name="get-mod-status-string"><xsl:with-param name="mod_status" select="$mod_status/@val20"/></xsl:call-template>
		</xsl:variable>
		<xsl:variable name="mod_status_21">
			<xsl:call-template name="get-mod-status-string"><xsl:with-param name="mod_status" select="$mod_status/@val21"/></xsl:call-template>
		</xsl:variable>
		<xsl:variable name="mod_status_22">
			<xsl:call-template name="get-mod-status-string"><xsl:with-param name="mod_status" select="$mod_status/@val22"/></xsl:call-template>
		</xsl:variable>
		<xsl:variable name="mod_status_23">
			<xsl:call-template name="get-mod-status-string"><xsl:with-param name="mod_status" select="$mod_status/@val23"/></xsl:call-template>
		</xsl:variable>
		<xsl:variable name="mod_status_24">
			<xsl:call-template name="get-mod-status-string"><xsl:with-param name="mod_status" select="$mod_status/@val24"/></xsl:call-template>
		</xsl:variable>
		<xsl:variable name="mod_status_25">
			<xsl:call-template name="get-mod-status-string"><xsl:with-param name="mod_status" select="$mod_status/@val25"/></xsl:call-template>
		</xsl:variable>
		<xsl:variable name="mod_status_26">
			<xsl:call-template name="get-mod-status-string"><xsl:with-param name="mod_status" select="$mod_status/@val26"/></xsl:call-template>
		</xsl:variable>
		<xsl:variable name="mod_status_27">
			<xsl:call-template name="get-mod-status-string"><xsl:with-param name="mod_status" select="$mod_status/@val27"/></xsl:call-template>
		</xsl:variable>
		<xsl:variable name="mod_status_28">
			<xsl:call-template name="get-mod-status-string"><xsl:with-param name="mod_status" select="$mod_status/@val28"/></xsl:call-template>
		</xsl:variable>
		<xsl:variable name="mod_status_29">
			<xsl:call-template name="get-mod-status-string"><xsl:with-param name="mod_status" select="$mod_status/@val29"/></xsl:call-template>
		</xsl:variable>
		<xsl:variable name="mod_status_30">
			<xsl:call-template name="get-mod-status-string"><xsl:with-param name="mod_status" select="$mod_status/@val30"/></xsl:call-template>
		</xsl:variable>
		<xsl:variable name="mod_status_31">
			<xsl:call-template name="get-mod-status-string"><xsl:with-param name="mod_status" select="$mod_status/@val31"/></xsl:call-template>
		</xsl:variable>
		<xsl:variable name="mod_status_32">
			<xsl:call-template name="get-mod-status-string"><xsl:with-param name="mod_status" select="$mod_status/@val32"/></xsl:call-template>
		</xsl:variable>

		<!--Modules status color-->
		<xsl:variable name="mod_status_color_1">
			<xsl:call-template name="get-mod-status-color"><xsl:with-param name="mod_status" select="$mod_status/@val"/></xsl:call-template>
		</xsl:variable>
		<xsl:variable name="mod_status_color_2">
			<xsl:call-template name="get-mod-status-color"><xsl:with-param name="mod_status" select="$mod_status/@val2"/></xsl:call-template>
		</xsl:variable>
		<xsl:variable name="mod_status_color_3">
			<xsl:call-template name="get-mod-status-color"><xsl:with-param name="mod_status" select="$mod_status/@val3"/></xsl:call-template>
		</xsl:variable>
		<xsl:variable name="mod_status_color_4">
			<xsl:call-template name="get-mod-status-color"><xsl:with-param name="mod_status" select="$mod_status/@val4"/></xsl:call-template>
		</xsl:variable>
		<xsl:variable name="mod_status_color_5">
			<xsl:call-template name="get-mod-status-color"><xsl:with-param name="mod_status" select="$mod_status/@val5"/></xsl:call-template>
		</xsl:variable>
		<xsl:variable name="mod_status_color_6">
			<xsl:call-template name="get-mod-status-color"><xsl:with-param name="mod_status" select="$mod_status/@val6"/></xsl:call-template>
		</xsl:variable>
		<xsl:variable name="mod_status_color_7">
			<xsl:call-template name="get-mod-status-color"><xsl:with-param name="mod_status" select="$mod_status/@val7"/></xsl:call-template>
		</xsl:variable>
		<xsl:variable name="mod_status_color_8">
			<xsl:call-template name="get-mod-status-color"><xsl:with-param name="mod_status" select="$mod_status/@val8"/></xsl:call-template>
		</xsl:variable>
		<xsl:variable name="mod_status_color_9">
			<xsl:call-template name="get-mod-status-color"><xsl:with-param name="mod_status" select="$mod_status/@val9"/></xsl:call-template>
		</xsl:variable>
		<xsl:variable name="mod_status_color_10">
			<xsl:call-template name="get-mod-status-color"><xsl:with-param name="mod_status" select="$mod_status/@val10"/></xsl:call-template>
		</xsl:variable>
		<xsl:variable name="mod_status_color_11">
			<xsl:call-template name="get-mod-status-color"><xsl:with-param name="mod_status" select="$mod_status/@val11"/></xsl:call-template>
		</xsl:variable>
		<xsl:variable name="mod_status_color_12">
			<xsl:call-template name="get-mod-status-color"><xsl:with-param name="mod_status" select="$mod_status/@val12"/></xsl:call-template>
		</xsl:variable>
		<xsl:variable name="mod_status_color_13">
			<xsl:call-template name="get-mod-status-color"><xsl:with-param name="mod_status" select="$mod_status/@val13"/></xsl:call-template>
		</xsl:variable>
		<xsl:variable name="mod_status_color_14">
			<xsl:call-template name="get-mod-status-color"><xsl:with-param name="mod_status" select="$mod_status/@val14"/></xsl:call-template>
		</xsl:variable>
		<xsl:variable name="mod_status_color_15">
			<xsl:call-template name="get-mod-status-color"><xsl:with-param name="mod_status" select="$mod_status/@val15"/></xsl:call-template>
		</xsl:variable>
		<xsl:variable name="mod_status_color_16">
			<xsl:call-template name="get-mod-status-color"><xsl:with-param name="mod_status" select="$mod_status/@val16"/></xsl:call-template>
		</xsl:variable>
		<xsl:variable name="mod_status_color_17">
			<xsl:call-template name="get-mod-status-color"><xsl:with-param name="mod_status" select="$mod_status/@val17"/></xsl:call-template>
		</xsl:variable>
		<xsl:variable name="mod_status_color_18">
			<xsl:call-template name="get-mod-status-color"><xsl:with-param name="mod_status" select="$mod_status/@val18"/></xsl:call-template>
		</xsl:variable>
		<xsl:variable name="mod_status_color_19">
			<xsl:call-template name="get-mod-status-color"><xsl:with-param name="mod_status" select="$mod_status/@val19"/></xsl:call-template>
		</xsl:variable>
		<xsl:variable name="mod_status_color_20">
			<xsl:call-template name="get-mod-status-color"><xsl:with-param name="mod_status" select="$mod_status/@val20"/></xsl:call-template>
		</xsl:variable>
		<xsl:variable name="mod_status_color_21">
			<xsl:call-template name="get-mod-status-color"><xsl:with-param name="mod_status" select="$mod_status/@val21"/></xsl:call-template>
		</xsl:variable>
		<xsl:variable name="mod_status_color_22">
			<xsl:call-template name="get-mod-status-color"><xsl:with-param name="mod_status" select="$mod_status/@val22"/></xsl:call-template>
		</xsl:variable>
		<xsl:variable name="mod_status_color_23">
			<xsl:call-template name="get-mod-status-color"><xsl:with-param name="mod_status" select="$mod_status/@val23"/></xsl:call-template>
		</xsl:variable>
		<xsl:variable name="mod_status_color_24">
			<xsl:call-template name="get-mod-status-color"><xsl:with-param name="mod_status" select="$mod_status/@val24"/></xsl:call-template>
		</xsl:variable>
		<xsl:variable name="mod_status_color_25">
			<xsl:call-template name="get-mod-status-color"><xsl:with-param name="mod_status" select="$mod_status/@val25"/></xsl:call-template>
		</xsl:variable>
		<xsl:variable name="mod_status_color_26">
			<xsl:call-template name="get-mod-status-color"><xsl:with-param name="mod_status" select="$mod_status/@val26"/></xsl:call-template>
		</xsl:variable>
		<xsl:variable name="mod_status_color_27">
			<xsl:call-template name="get-mod-status-color"><xsl:with-param name="mod_status" select="$mod_status/@val27"/></xsl:call-template>
		</xsl:variable>
		<xsl:variable name="mod_status_color_28">
			<xsl:call-template name="get-mod-status-color"><xsl:with-param name="mod_status" select="$mod_status/@val28"/></xsl:call-template>
		</xsl:variable>
		<xsl:variable name="mod_status_color_29">
			<xsl:call-template name="get-mod-status-color"><xsl:with-param name="mod_status" select="$mod_status/@val29"/></xsl:call-template>
		</xsl:variable>
		<xsl:variable name="mod_status_color_30">
			<xsl:call-template name="get-mod-status-color"><xsl:with-param name="mod_status" select="$mod_status/@val30"/></xsl:call-template>
		</xsl:variable>
		<xsl:variable name="mod_status_color_31">
			<xsl:call-template name="get-mod-status-color"><xsl:with-param name="mod_status" select="$mod_status/@val31"/></xsl:call-template>
		</xsl:variable>
		<xsl:variable name="mod_status_color_32">
			<xsl:call-template name="get-mod-status-color"><xsl:with-param name="mod_status" select="$mod_status/@val32"/></xsl:call-template>
		</xsl:variable>

		<xsl:if test="$mod_present > 0"><!--modules  present-->
	  	<div id="{$id}" onmousedown="dragstart(this)" onclick="bring_to_front('{$id}')" class="{$class}" align="left" style="position:absolute; left:{$x}px; top:{$y}px; width:{$w}px;text-align:left;visibility:{$visibility};z-index:1;">
				<table cellspacing="0" cellpadding="0" border="0"  class="{$class}" style="position:absolute;width:{$w}px; height:20px;color:white;background-color:#800000;">
					<thead>
		  			<tr>
		    			<th align="left"><xsl:value-of select="concat('&#160;&#160;','AGIL&#160;module&#160;table')"/></th>
		    			<th align="right">
								<a href="javascript:show('{$id}');" class="btn">
									<span><span  style="color:white; background-color:#800000;"><xsl:value-of select="'X'"/></span></span>
								</a>
							</th>
		  			</tr>
					</thead>
				</table>
		  	<table border="1" cellspacing="2" cellpadding="2" class="{$class}" style="position:absolute;top:{$y + 10}px; width:{$w}px;color:black;background-color:white;text-align:center;">
					<tbody>
			  		<tr>
							<td><xsl:value-of select="'Ref'"/></td>
							<td><xsl:value-of select="'SN'"/></td>
							<td><xsl:value-of select="'AC&#160;In'"/></td>
							<td><xsl:value-of select="'BAT.'"/></td>
							<td><xsl:value-of select="'AC&#160;Out'"/></td>
							<td><xsl:value-of select="'P&#160;In(W)'"/></td>
							<td><xsl:value-of select="'P&#160;Out(W)'"/></td>
							<td><xsl:value-of select="'P&#160;Out(VA)'"/></td>
							<td><xsl:value-of select="'P&#160;BAT(W)'"/></td>
                     <xsl:if test="value[@name='_tempUnit']/@val=1">
							   <td><xsl:value-of select="'T(&#176;F)'"/></td>
                     </xsl:if>
                     <xsl:if test="value[@name='_tempUnit']/@val=0 or string-length(value[@name='_tempUnit']/@val)=0">
							   <td><xsl:value-of select="'T(&#176;C)'"/></td>
                     </xsl:if>
							<td><xsl:value-of select="'Status'"/></td>
						</tr>

						<xsl:if test="($mod_present mod ($bit0 * 2)) - ($mod_present mod $bit0)"><!--module 1 present-->
				  		<tr>
								<td><xsl:value-of select="'1'"/></td>
								<td><xsl:value-of select="$serial_nm/@val"/></td>
								<td><xsl:value-of select="$ac_in_con_str_1"/></td>
								<td><xsl:value-of select="$modinfo_gr_dc/@val"/></td>
								<td><xsl:value-of select="$ac_out_con_str_1"/></td>
								<td><xsl:value-of select="$ac_in_power_consumption_1"/></td>
								<td><xsl:value-of select="$ac_out_power_consumption_1"/></td>
								<td><xsl:value-of select="$ac_out_power_va_consumption_1"/></td>
								<td><xsl:value-of select="$mod_power_dc/@val"/></td>
								<td>
									<xsl:choose>
						  		  <xsl:when test="$mod_power_temp/@val &lt; -200"><xsl:value-of select="'---'"/></xsl:when>
										<xsl:otherwise>
                                 <xsl:call-template name="get-CET-temperature">
                                    <xsl:with-param name="value" select="$mod_power_temp/@val" />
                                    <xsl:with-param name="precision" select="2"/>
                                 </xsl:call-template>
                              </xsl:otherwise>
						    </xsl:choose>
								</td>
								<td style="background-color:{$mod_status_color_1}"><xsl:value-of select="$mod_status_1"/></td>
							</tr>
						</xsl:if>
						<xsl:if test="($mod_present mod ($bit1 * 2)) - ($mod_present mod $bit1)"><!--module 2 present-->
				  		<tr>
								<td><xsl:value-of select="'2'"/></td>
								<td><xsl:value-of select="$serial_nm/@val2"/></td>
								<td><xsl:value-of select="$ac_in_con_str_2"/></td>
								<td><xsl:value-of select="$modinfo_gr_dc/@val2"/></td>
								<td><xsl:value-of select="$ac_out_con_str_2"/></td>
								<td><xsl:value-of select="$ac_in_power_consumption_2"/></td>
								<td><xsl:value-of select="$ac_out_power_consumption_2"/></td>
								<td><xsl:value-of select="$ac_out_power_va_consumption_2"/></td>
								<td><xsl:value-of select="$mod_power_dc/@val2"/></td>
								<td>
									<xsl:choose>
						  		  <xsl:when test="$mod_power_temp/@val2 &lt; -200"><xsl:value-of select="'---'"/></xsl:when>
										<xsl:otherwise>
                                 <xsl:call-template name="get-CET-temperature">
                                    <xsl:with-param name="value" select="$mod_power_temp/@val2" />
                                    <xsl:with-param name="precision" select="2"/>
                                 </xsl:call-template>
                              </xsl:otherwise>
						    </xsl:choose>
								</td>
								<td style="background-color:{$mod_status_color_2}"><xsl:value-of select="$mod_status_2"/></td>
							</tr>
						</xsl:if>
						<xsl:if test="($mod_present mod ($bit2 * 2)) - ($mod_present mod $bit2)"><!--module 3 present-->
				  		<tr>
								<td><xsl:value-of select="'3'"/></td>
								<td><xsl:value-of select="$serial_nm/@val3"/></td>
								<td><xsl:value-of select="$ac_in_con_str_3"/></td>
								<td><xsl:value-of select="$modinfo_gr_dc/@val3"/></td>
								<td><xsl:value-of select="$ac_out_con_str_3"/></td>
								<td><xsl:value-of select="$ac_in_power_consumption_3"/></td>
								<td><xsl:value-of select="$ac_out_power_consumption_3"/></td>
								<td><xsl:value-of select="$ac_out_power_va_consumption_3"/></td>
								<td><xsl:value-of select="$mod_power_dc/@val3"/></td>
								<td>
									<xsl:choose>
						  		  <xsl:when test="$mod_power_temp/@val3 &lt; -200"><xsl:value-of select="'---'"/></xsl:when>
										<xsl:otherwise>
                                 <xsl:call-template name="get-CET-temperature">
                                    <xsl:with-param name="value" select="$mod_power_temp/@val3" />
                                    <xsl:with-param name="precision" select="2"/>
                                 </xsl:call-template>
                              </xsl:otherwise>
						    </xsl:choose>
								</td>
								<td style="background-color:{$mod_status_color_3}"><xsl:value-of select="$mod_status_3"/></td>
							</tr>
						</xsl:if>
						<xsl:if test="($mod_present mod ($bit3 * 2)) - ($mod_present mod $bit3)"><!--module 4 present-->
				  		<tr>
								<td><xsl:value-of select="'4'"/></td>
								<td><xsl:value-of select="$serial_nm/@val4"/></td>
								<td><xsl:value-of select="$ac_in_con_str_4"/></td>
								<td><xsl:value-of select="$modinfo_gr_dc/@val4"/></td>
								<td><xsl:value-of select="$ac_out_con_str_4"/></td>
								<td><xsl:value-of select="$ac_in_power_consumption_4"/></td>
								<td><xsl:value-of select="$ac_out_power_consumption_4"/></td>
								<td><xsl:value-of select="$ac_out_power_va_consumption_4"/></td>
								<td><xsl:value-of select="$mod_power_dc/@val4"/></td>
								<td>
									<xsl:choose>
						  		  <xsl:when test="$mod_power_temp/@val4 &lt; -200"><xsl:value-of select="'---'"/></xsl:when>
										<xsl:otherwise>
                                 <xsl:call-template name="get-CET-temperature">
                                    <xsl:with-param name="value" select="$mod_power_temp/@val4" />
                                    <xsl:with-param name="precision" select="2"/>
                                 </xsl:call-template>
                              </xsl:otherwise>
						    </xsl:choose>
								</td>
								<td style="background-color:{$mod_status_color_4}"><xsl:value-of select="$mod_status_4"/></td>
							</tr>
						</xsl:if>
						<xsl:if test="($mod_present mod ($bit4 * 2)) - ($mod_present mod $bit4)"><!--module 5 present-->
				  		<tr>
								<td><xsl:value-of select="'5'"/></td>
								<td><xsl:value-of select="$serial_nm/@val5"/></td>
								<td><xsl:value-of select="$ac_in_con_str_5"/></td>
								<td><xsl:value-of select="$modinfo_gr_dc/@val5"/></td>
								<td><xsl:value-of select="$ac_out_con_str_5"/></td>
								<td><xsl:value-of select="$ac_in_power_consumption_5"/></td>
								<td><xsl:value-of select="$ac_out_power_consumption_5"/></td>
								<td><xsl:value-of select="$ac_out_power_va_consumption_5"/></td>
								<td><xsl:value-of select="$mod_power_dc/@val5"/></td>
								<td>
									<xsl:choose>
						  		  <xsl:when test="$mod_power_temp/@val5 &lt; -200"><xsl:value-of select="'---'"/></xsl:when>
										<xsl:otherwise>
                                 <xsl:call-template name="get-CET-temperature">
                                    <xsl:with-param name="value" select="$mod_power_temp/@val5" />
                                    <xsl:with-param name="precision" select="2"/>
                                 </xsl:call-template>
                              </xsl:otherwise>
						    </xsl:choose>
								</td>
								<td style="background-color:{$mod_status_color_5}"><xsl:value-of select="$mod_status_5"/></td>
							</tr>
						</xsl:if>
						<xsl:if test="($mod_present mod ($bit5 * 2)) - ($mod_present mod $bit5)"><!--module 6 present-->
				  		<tr>
								<td><xsl:value-of select="'6'"/></td>
								<td><xsl:value-of select="$serial_nm/@val6"/></td>
								<td><xsl:value-of select="$ac_in_con_str_6"/></td>
								<td><xsl:value-of select="$modinfo_gr_dc/@val6"/></td>
								<td><xsl:value-of select="$ac_out_con_str_6"/></td>
								<td><xsl:value-of select="$ac_in_power_consumption_6"/></td>
								<td><xsl:value-of select="$ac_out_power_consumption_6"/></td>
								<td><xsl:value-of select="$ac_out_power_va_consumption_6"/></td>
								<td><xsl:value-of select="$mod_power_dc/@val6"/></td>
								<td>
									<xsl:choose>
						  		  <xsl:when test="$mod_power_temp/@val6 &lt; -200"><xsl:value-of select="'---'"/></xsl:when>
										<xsl:otherwise>
                                 <xsl:call-template name="get-CET-temperature">
                                    <xsl:with-param name="value" select="$mod_power_temp/@val6" />
                                    <xsl:with-param name="precision" select="2"/>
                                 </xsl:call-template>
                              </xsl:otherwise>
						    </xsl:choose>
								</td>
								<td style="background-color:{$mod_status_color_6}"><xsl:value-of select="$mod_status_6"/></td>
							</tr>
						</xsl:if>
						<xsl:if test="($mod_present mod ($bit6 * 2)) - ($mod_present mod $bit6)"><!--module 7 present-->
				  		<tr>
								<td><xsl:value-of select="'7'"/></td>
								<td><xsl:value-of select="$serial_nm/@val7"/></td>
								<td><xsl:value-of select="$ac_in_con_str_7"/></td>
								<td><xsl:value-of select="$modinfo_gr_dc/@val7"/></td>
								<td><xsl:value-of select="$ac_out_con_str_7"/></td>
								<td><xsl:value-of select="$ac_in_power_consumption_7"/></td>
								<td><xsl:value-of select="$ac_out_power_consumption_7"/></td>
								<td><xsl:value-of select="$ac_out_power_va_consumption_7"/></td>
								<td><xsl:value-of select="$mod_power_dc/@val7"/></td>
								<td>
									<xsl:choose>
						  		  <xsl:when test="$mod_power_temp/@val7 &lt; -200"><xsl:value-of select="'---'"/></xsl:when>
										<xsl:otherwise>
                                 <xsl:call-template name="get-CET-temperature">
                                    <xsl:with-param name="value" select="$mod_power_temp/@val7" />
                                    <xsl:with-param name="precision" select="2"/>
                                 </xsl:call-template>
                              </xsl:otherwise>
						    </xsl:choose>
								</td>
								<td style="background-color:{$mod_status_color_7}"><xsl:value-of select="$mod_status_7"/></td>
							</tr>
						</xsl:if>
						<xsl:if test="($mod_present mod ($bit7 * 2)) - ($mod_present mod $bit7)"><!--module 8 present-->
				  		<tr>
								<td><xsl:value-of select="'8'"/></td>
								<td><xsl:value-of select="$serial_nm/@val8"/></td>
								<td><xsl:value-of select="$ac_in_con_str_8"/></td>
								<td><xsl:value-of select="$modinfo_gr_dc/@val8"/></td>
								<td><xsl:value-of select="$ac_out_con_str_8"/></td>
								<td><xsl:value-of select="$ac_in_power_consumption_8"/></td>
								<td><xsl:value-of select="$ac_out_power_consumption_8"/></td>
								<td><xsl:value-of select="$ac_out_power_va_consumption_8"/></td>
								<td><xsl:value-of select="$mod_power_dc/@val8"/></td>
								<td>
									<xsl:choose>
						  		  <xsl:when test="$mod_power_temp/@val8 &lt; -200"><xsl:value-of select="'---'"/></xsl:when>
										<xsl:otherwise>
                                 <xsl:call-template name="get-CET-temperature">
                                    <xsl:with-param name="value" select="$mod_power_temp/@val8" />
                                    <xsl:with-param name="precision" select="2"/>
                                 </xsl:call-template>
                              </xsl:otherwise>
						    </xsl:choose>
								</td>
								<td style="background-color:{$mod_status_color_8}"><xsl:value-of select="$mod_status_8"/></td>
							</tr>
						</xsl:if>
						<xsl:if test="($mod_present mod ($bit8 * 2)) - ($mod_present mod $bit8)"><!--module 9 present-->
				  		<tr>
								<td><xsl:value-of select="'9'"/></td>
								<td><xsl:value-of select="$serial_nm/@val9"/></td>
								<td><xsl:value-of select="$ac_in_con_str_9"/></td>
								<td><xsl:value-of select="$modinfo_gr_dc/@val9"/></td>
								<td><xsl:value-of select="$ac_out_con_str_9"/></td>
								<td><xsl:value-of select="$ac_in_power_consumption_9"/></td>
								<td><xsl:value-of select="$ac_out_power_consumption_9"/></td>
								<td><xsl:value-of select="$ac_out_power_va_consumption_9"/></td>
								<td><xsl:value-of select="$mod_power_dc/@val9"/></td>
								<td>
									<xsl:choose>
						  		  <xsl:when test="$mod_power_temp/@val9 &lt; -200"><xsl:value-of select="'---'"/></xsl:when>
										<xsl:otherwise>
                                 <xsl:call-template name="get-CET-temperature">
                                    <xsl:with-param name="value" select="$mod_power_temp/@val9" />
                                    <xsl:with-param name="precision" select="2"/>
                                 </xsl:call-template>
                              </xsl:otherwise>
						    </xsl:choose>
								</td>
								<td style="background-color:{$mod_status_color_9}"><xsl:value-of select="$mod_status_9"/></td>
							</tr>
						</xsl:if>
						<xsl:if test="($mod_present mod ($bit9 * 2)) - ($mod_present mod $bit9)"><!--module 10 present-->
				  		<tr>
								<td><xsl:value-of select="'10'"/></td>
								<td><xsl:value-of select="$serial_nm/@val10"/></td>
								<td><xsl:value-of select="$ac_in_con_str_10"/></td>
								<td><xsl:value-of select="$modinfo_gr_dc/@val10"/></td>
								<td><xsl:value-of select="$ac_out_con_str_10"/></td>
								<td><xsl:value-of select="$ac_in_power_consumption_10"/></td>
								<td><xsl:value-of select="$ac_out_power_consumption_10"/></td>
								<td><xsl:value-of select="$ac_out_power_va_consumption_10"/></td>
								<td><xsl:value-of select="$mod_power_dc/@val10"/></td>
								<td>
									<xsl:choose>
						  		  <xsl:when test="$mod_power_temp/@val10 &lt; -200"><xsl:value-of select="'---'"/></xsl:when>
										<xsl:otherwise>
                                 <xsl:call-template name="get-CET-temperature">
                                    <xsl:with-param name="value" select="$mod_power_temp/@val10" />
                                    <xsl:with-param name="precision" select="2"/>
                                 </xsl:call-template>
                              </xsl:otherwise>
						    </xsl:choose>
								</td>
								<td style="background-color:{$mod_status_color_10}"><xsl:value-of select="$mod_status_10"/></td>
							</tr>
						</xsl:if>
						<xsl:if test="($mod_present mod ($bit10 * 2)) - ($mod_present mod $bit10)"><!--module 11 present-->
				  		<tr>
								<td><xsl:value-of select="'11'"/></td>
								<td><xsl:value-of select="$serial_nm/@val11"/></td>
								<td><xsl:value-of select="$ac_in_con_str_11"/></td>
								<td><xsl:value-of select="$modinfo_gr_dc/@val11"/></td>
								<td><xsl:value-of select="$ac_out_con_str_11"/></td>
								<td><xsl:value-of select="$ac_in_power_consumption_11"/></td>
								<td><xsl:value-of select="$ac_out_power_consumption_11"/></td>
								<td><xsl:value-of select="$ac_out_power_va_consumption_11"/></td>
								<td><xsl:value-of select="$mod_power_dc/@val11"/></td>
								<td>
									<xsl:choose>
						  		  <xsl:when test="$mod_power_temp/@val11 &lt; -200"><xsl:value-of select="'---'"/></xsl:when>
										<xsl:otherwise>
                                 <xsl:call-template name="get-CET-temperature">
                                    <xsl:with-param name="value" select="$mod_power_temp/@val11" />
                                    <xsl:with-param name="precision" select="2"/>
                                 </xsl:call-template>
                              </xsl:otherwise>
						    </xsl:choose>
								</td>
								<td style="background-color:{$mod_status_color_11}"><xsl:value-of select="$mod_status_11"/></td>
							</tr>
						</xsl:if>
						<xsl:if test="($mod_present mod ($bit11 * 2)) - ($mod_present mod $bit11)"><!--module 12 present-->
				  		<tr>
								<td><xsl:value-of select="'12'"/></td>
								<td><xsl:value-of select="$serial_nm/@val12"/></td>
								<td><xsl:value-of select="$ac_in_con_str_12"/></td>
								<td><xsl:value-of select="$modinfo_gr_dc/@val12"/></td>
								<td><xsl:value-of select="$ac_out_con_str_12"/></td>
								<td><xsl:value-of select="$ac_in_power_consumption_12"/></td>
								<td><xsl:value-of select="$ac_out_power_consumption_12"/></td>
								<td><xsl:value-of select="$ac_out_power_va_consumption_12"/></td>
								<td><xsl:value-of select="$mod_power_dc/@val12"/></td>
								<td>
									<xsl:choose>
						  		  <xsl:when test="$mod_power_temp/@val12 &lt; -200"><xsl:value-of select="'---'"/></xsl:when>
										<xsl:otherwise>
                                 <xsl:call-template name="get-CET-temperature">
                                    <xsl:with-param name="value" select="$mod_power_temp/@val12" />
                                    <xsl:with-param name="precision" select="2"/>
                                 </xsl:call-template>
                              </xsl:otherwise>
						    </xsl:choose>
								</td>
								<td style="background-color:{$mod_status_color_12}"><xsl:value-of select="$mod_status_12"/></td>
							</tr>
						</xsl:if>
						<xsl:if test="($mod_present mod ($bit12 * 2)) - ($mod_present mod $bit12)"><!--module 13 present-->
				  		<tr>
								<td><xsl:value-of select="'13'"/></td>
								<td><xsl:value-of select="$serial_nm/@val13"/></td>
								<td><xsl:value-of select="$ac_in_con_str_13"/></td>
								<td><xsl:value-of select="$modinfo_gr_dc/@val13"/></td>
								<td><xsl:value-of select="$ac_out_con_str_13"/></td>
								<td><xsl:value-of select="$ac_in_power_consumption_13"/></td>
								<td><xsl:value-of select="$ac_out_power_consumption_13"/></td>
								<td><xsl:value-of select="$ac_out_power_va_consumption_13"/></td>
								<td><xsl:value-of select="$mod_power_dc/@val13"/></td>
								<td>
									<xsl:choose>
						  		  <xsl:when test="$mod_power_temp/@val13 &lt; -200"><xsl:value-of select="'---'"/></xsl:when>
										<xsl:otherwise>
                                 <xsl:call-template name="get-CET-temperature">
                                    <xsl:with-param name="value" select="$mod_power_temp/@val13" />
                                    <xsl:with-param name="precision" select="2"/>
                                 </xsl:call-template>
                              </xsl:otherwise>
						    </xsl:choose>
								</td>
								<td style="background-color:{$mod_status_color_13}"><xsl:value-of select="$mod_status_13"/></td>
							</tr>
						</xsl:if>
						<xsl:if test="($mod_present mod ($bit13 * 2)) - ($mod_present mod $bit13)"><!--module 14 present-->
				  		<tr>
								<td><xsl:value-of select="'14'"/></td>
								<td><xsl:value-of select="$serial_nm/@val14"/></td>
								<td><xsl:value-of select="$ac_in_con_str_14"/></td>
								<td><xsl:value-of select="$modinfo_gr_dc/@val14"/></td>
								<td><xsl:value-of select="$ac_out_con_str_14"/></td>
								<td><xsl:value-of select="$ac_in_power_consumption_14"/></td>
								<td><xsl:value-of select="$ac_out_power_consumption_14"/></td>
								<td><xsl:value-of select="$ac_out_power_va_consumption_14"/></td>
								<td><xsl:value-of select="$mod_power_dc/@val14"/></td>
								<td>
									<xsl:choose>
						  		  <xsl:when test="$mod_power_temp/@val14 &lt; -200"><xsl:value-of select="'---'"/></xsl:when>
										<xsl:otherwise>
                                 <xsl:call-template name="get-CET-temperature">
                                    <xsl:with-param name="value" select="$mod_power_temp/@val14" />
                                    <xsl:with-param name="precision" select="2"/>
                                 </xsl:call-template>
                              </xsl:otherwise>
						    </xsl:choose>
								</td>
								<td style="background-color:{$mod_status_color_14}"><xsl:value-of select="$mod_status_14"/></td>
							</tr>
						</xsl:if>
						<xsl:if test="($mod_present mod ($bit14 * 2)) - ($mod_present mod $bit14)"><!--module 15 present-->
				  		<tr>
								<td><xsl:value-of select="'15'"/></td>
								<td><xsl:value-of select="$serial_nm/@val15"/></td>
								<td><xsl:value-of select="$ac_in_con_str_15"/></td>
								<td><xsl:value-of select="$modinfo_gr_dc/@val15"/></td>
								<td><xsl:value-of select="$ac_out_con_str_15"/></td>
								<td><xsl:value-of select="$ac_in_power_consumption_15"/></td>
								<td><xsl:value-of select="$ac_out_power_consumption_15"/></td>
								<td><xsl:value-of select="$ac_out_power_va_consumption_15"/></td>
								<td><xsl:value-of select="$mod_power_dc/@val15"/></td>
								<td>
									<xsl:choose>
						  		  <xsl:when test="$mod_power_temp/@val15 &lt; -200"><xsl:value-of select="'---'"/></xsl:when>
										<xsl:otherwise>
                                 <xsl:call-template name="get-CET-temperature">
                                    <xsl:with-param name="value" select="$mod_power_temp/@val15" />
                                    <xsl:with-param name="precision" select="2"/>
                                 </xsl:call-template>
                              </xsl:otherwise>
						    </xsl:choose>
								</td>
								<td style="background-color:{$mod_status_color_15}"><xsl:value-of select="$mod_status_15"/></td>
							</tr>
						</xsl:if>
						<xsl:if test="($mod_present mod ($bit15 * 2)) - ($mod_present mod $bit15)"><!--module 16 present-->
				  		<tr>
								<td><xsl:value-of select="'16'"/></td>
								<td><xsl:value-of select="$serial_nm/@val16"/></td>
								<td><xsl:value-of select="$ac_in_con_str_16"/></td>
								<td><xsl:value-of select="$modinfo_gr_dc/@val16"/></td>
								<td><xsl:value-of select="$ac_out_con_str_16"/></td>
								<td><xsl:value-of select="$ac_in_power_consumption_16"/></td>
								<td><xsl:value-of select="$ac_out_power_consumption_16"/></td>
								<td><xsl:value-of select="$ac_out_power_va_consumption_16"/></td>
								<td><xsl:value-of select="$mod_power_dc/@val16"/></td>
								<td>
									<xsl:choose>
						  		  <xsl:when test="$mod_power_temp/@val16 &lt; -200"><xsl:value-of select="'---'"/></xsl:when>
										<xsl:otherwise>
                                 <xsl:call-template name="get-CET-temperature">
                                    <xsl:with-param name="value" select="$mod_power_temp/@val16" />
                                    <xsl:with-param name="precision" select="2"/>
                                 </xsl:call-template>
                              </xsl:otherwise>
						    </xsl:choose>
								</td>
								<td style="background-color:{$mod_status_color_16}"><xsl:value-of select="$mod_status_16"/></td>
							</tr>
						</xsl:if>
						<xsl:if test="($mod_present mod ($bit16 * 2)) - ($mod_present mod $bit16)"><!--module 17 present-->
				  		<tr>
								<td><xsl:value-of select="'17'"/></td>
								<td><xsl:value-of select="$serial_nm/@val17"/></td>
								<td><xsl:value-of select="$ac_in_con_str_17"/></td>
								<td><xsl:value-of select="$modinfo_gr_dc/@val17"/></td>
								<td><xsl:value-of select="$ac_out_con_str_17"/></td>
								<td><xsl:value-of select="$ac_in_power_consumption_17"/></td>
								<td><xsl:value-of select="$ac_out_power_consumption_17"/></td>
								<td><xsl:value-of select="$ac_out_power_va_consumption_17"/></td>
								<td><xsl:value-of select="$mod_power_dc/@val17"/></td>
								<td>
									<xsl:choose>
						  		  <xsl:when test="$mod_power_temp/@val17 &lt; -200"><xsl:value-of select="'---'"/></xsl:when>
										<xsl:otherwise>
                                 <xsl:call-template name="get-CET-temperature">
                                    <xsl:with-param name="value" select="$mod_power_temp/@val17" />
                                    <xsl:with-param name="precision" select="2"/>
                                 </xsl:call-template>
                              </xsl:otherwise>
						    </xsl:choose>
								</td>
								<td style="background-color:{$mod_status_color_17}"><xsl:value-of select="$mod_status_17"/></td>
							</tr>
						</xsl:if>
						<xsl:if test="($mod_present mod ($bit17 * 2)) - ($mod_present mod $bit17)"><!--module 18 present-->
				  		<tr>
								<td><xsl:value-of select="'18'"/></td>
								<td><xsl:value-of select="$serial_nm/@val18"/></td>
								<td><xsl:value-of select="$ac_in_con_str_18"/></td>
								<td><xsl:value-of select="$modinfo_gr_dc/@val18"/></td>
								<td><xsl:value-of select="$ac_out_con_str_18"/></td>
								<td><xsl:value-of select="$ac_in_power_consumption_18"/></td>
								<td><xsl:value-of select="$ac_out_power_consumption_18"/></td>
								<td><xsl:value-of select="$ac_out_power_va_consumption_18"/></td>
								<td><xsl:value-of select="$mod_power_dc/@val18"/></td>
								<td>
									<xsl:choose>
						  		  <xsl:when test="$mod_power_temp/@val18 &lt; -200"><xsl:value-of select="'---'"/></xsl:when>
										<xsl:otherwise>
                                 <xsl:call-template name="get-CET-temperature">
                                    <xsl:with-param name="value" select="$mod_power_temp/@val18" />
                                    <xsl:with-param name="precision" select="2"/>
                                 </xsl:call-template>
                              </xsl:otherwise>
						    </xsl:choose>
								</td>
								<td style="background-color:{$mod_status_color_18}"><xsl:value-of select="$mod_status_18"/></td>
							</tr>
						</xsl:if>
						<xsl:if test="($mod_present mod ($bit18 * 2)) - ($mod_present mod $bit18)"><!--module 19 present-->
				  		<tr>
								<td><xsl:value-of select="'19'"/></td>
								<td><xsl:value-of select="$serial_nm/@val19"/></td>
								<td><xsl:value-of select="$ac_in_con_str_19"/></td>
								<td><xsl:value-of select="$modinfo_gr_dc/@val19"/></td>
								<td><xsl:value-of select="$ac_out_con_str_19"/></td>
								<td><xsl:value-of select="$ac_in_power_consumption_19"/></td>
								<td><xsl:value-of select="$ac_out_power_consumption_19"/></td>
								<td><xsl:value-of select="$ac_out_power_va_consumption_19"/></td>
								<td><xsl:value-of select="$mod_power_dc/@val19"/></td>
								<td>
									<xsl:choose>
						  		  <xsl:when test="$mod_power_temp/@val19 &lt; -200"><xsl:value-of select="'---'"/></xsl:when>
										<xsl:otherwise>
                                 <xsl:call-template name="get-CET-temperature">
                                    <xsl:with-param name="value" select="$mod_power_temp/@val19" />
                                    <xsl:with-param name="precision" select="2"/>
                                 </xsl:call-template>
                              </xsl:otherwise>
						    </xsl:choose>
								</td>
								<td style="background-color:{$mod_status_color_19}"><xsl:value-of select="$mod_status_19"/></td>
							</tr>
						</xsl:if>
						<xsl:if test="($mod_present mod ($bit19 * 2)) - ($mod_present mod $bit19)"><!--module 20 present-->
				  		<tr>
								<td><xsl:value-of select="'20'"/></td>
								<td><xsl:value-of select="$serial_nm/@val20"/></td>
								<td><xsl:value-of select="$ac_in_con_str_20"/></td>
								<td><xsl:value-of select="$modinfo_gr_dc/@val20"/></td>
								<td><xsl:value-of select="$ac_out_con_str_20"/></td>
								<td><xsl:value-of select="$ac_in_power_consumption_20"/></td>
								<td><xsl:value-of select="$ac_out_power_consumption_20"/></td>
								<td><xsl:value-of select="$ac_out_power_va_consumption_20"/></td>
								<td><xsl:value-of select="$mod_power_dc/@val20"/></td>
								<td>
									<xsl:choose>
						  		  <xsl:when test="$mod_power_temp/@val20 &lt; -200"><xsl:value-of select="'---'"/></xsl:when>
										<xsl:otherwise>
                                 <xsl:call-template name="get-CET-temperature">
                                    <xsl:with-param name="value" select="$mod_power_temp/@val20" />
                                    <xsl:with-param name="precision" select="2"/>
                                 </xsl:call-template>
                              </xsl:otherwise>
						    </xsl:choose>
								</td>
								<td style="background-color:{$mod_status_color_20}"><xsl:value-of select="$mod_status_20"/></td>
							</tr>
						</xsl:if>
						<xsl:if test="($mod_present mod ($bit20 * 2)) - ($mod_present mod $bit20)"><!--module 21 present-->
				  		<tr>
								<td><xsl:value-of select="'21'"/></td>
								<td><xsl:value-of select="$serial_nm/@val21"/></td>
								<td><xsl:value-of select="$ac_in_con_str_21"/></td>
								<td><xsl:value-of select="$modinfo_gr_dc/@val21"/></td>
								<td><xsl:value-of select="$ac_out_con_str_21"/></td>
								<td><xsl:value-of select="$ac_in_power_consumption_21"/></td>
								<td><xsl:value-of select="$ac_out_power_consumption_21"/></td>
								<td><xsl:value-of select="$ac_out_power_va_consumption_21"/></td>
								<td><xsl:value-of select="$mod_power_dc/@val21"/></td>
								<td>
									<xsl:choose>
						  		  <xsl:when test="$mod_power_temp/@val21 &lt; -200"><xsl:value-of select="'---'"/></xsl:when>
										<xsl:otherwise>
                                 <xsl:call-template name="get-CET-temperature">
                                    <xsl:with-param name="value" select="$mod_power_temp/@val21" />
                                    <xsl:with-param name="precision" select="2"/>
                                 </xsl:call-template>
                              </xsl:otherwise>
						    </xsl:choose>
								</td>
								<td style="background-color:{$mod_status_color_21}"><xsl:value-of select="$mod_status_21"/></td>
							</tr>
						</xsl:if>
						<xsl:if test="($mod_present mod ($bit21 * 2)) - ($mod_present mod $bit21)"><!--module 22 present-->
				  		<tr>
								<td><xsl:value-of select="'22'"/></td>
								<td><xsl:value-of select="$serial_nm/@val22"/></td>
								<td><xsl:value-of select="$ac_in_con_str_22"/></td>
								<td><xsl:value-of select="$modinfo_gr_dc/@val22"/></td>
								<td><xsl:value-of select="$ac_out_con_str_22"/></td>
								<td><xsl:value-of select="$ac_in_power_consumption_22"/></td>
								<td><xsl:value-of select="$ac_out_power_consumption_22"/></td>
								<td><xsl:value-of select="$ac_out_power_va_consumption_22"/></td>
								<td><xsl:value-of select="$mod_power_dc/@val22"/></td>
								<td>
									<xsl:choose>
						  		  <xsl:when test="$mod_power_temp/@val22 &lt; -200"><xsl:value-of select="'---'"/></xsl:when>
										<xsl:otherwise>
                                 <xsl:call-template name="get-CET-temperature">
                                    <xsl:with-param name="value" select="$mod_power_temp/@val22" />
                                    <xsl:with-param name="precision" select="2"/>
                                 </xsl:call-template>
                              </xsl:otherwise>
						    </xsl:choose>
								</td>
								<td style="background-color:{$mod_status_color_22}"><xsl:value-of select="$mod_status_22"/></td>
							</tr>
						</xsl:if>
						<xsl:if test="($mod_present mod ($bit22 * 2)) - ($mod_present mod $bit22)"><!--module 23 present-->
				  		<tr>
								<td><xsl:value-of select="'23'"/></td>
								<td><xsl:value-of select="$serial_nm/@val23"/></td>
								<td><xsl:value-of select="$ac_in_con_str_23"/></td>
								<td><xsl:value-of select="$modinfo_gr_dc/@val23"/></td>
								<td><xsl:value-of select="$ac_out_con_str_23"/></td>
								<td><xsl:value-of select="$ac_in_power_consumption_23"/></td>
								<td><xsl:value-of select="$ac_out_power_consumption_23"/></td>
								<td><xsl:value-of select="$ac_out_power_va_consumption_23"/></td>
								<td><xsl:value-of select="$mod_power_dc/@val23"/></td>
								<td>
									<xsl:choose>
						  		  <xsl:when test="$mod_power_temp/@val23 &lt; -200"><xsl:value-of select="'---'"/></xsl:when>
										<xsl:otherwise>
                                 <xsl:call-template name="get-CET-temperature">
                                    <xsl:with-param name="value" select="$mod_power_temp/@val23" />
                                    <xsl:with-param name="precision" select="2"/>
                                 </xsl:call-template>
                              </xsl:otherwise>
						    </xsl:choose>
								</td>
								<td style="background-color:{$mod_status_color_23}"><xsl:value-of select="$mod_status_23"/></td>
							</tr>
						</xsl:if>
						<xsl:if test="($mod_present mod ($bit23 * 2)) - ($mod_present mod $bit23)"><!--module 24 present-->
				  		<tr>
								<td><xsl:value-of select="'24'"/></td>
								<td><xsl:value-of select="$serial_nm/@val24"/></td>
								<td><xsl:value-of select="$ac_in_con_str_24"/></td>
								<td><xsl:value-of select="$modinfo_gr_dc/@val24"/></td>
								<td><xsl:value-of select="$ac_out_con_str_24"/></td>
								<td><xsl:value-of select="$ac_in_power_consumption_24"/></td>
								<td><xsl:value-of select="$ac_out_power_consumption_24"/></td>
								<td><xsl:value-of select="$ac_out_power_va_consumption_24"/></td>
								<td><xsl:value-of select="$mod_power_dc/@val24"/></td>
								<td>
									<xsl:choose>
						  		  <xsl:when test="$mod_power_temp/@val24 &lt; -200"><xsl:value-of select="'---'"/></xsl:when>
										<xsl:otherwise>
                                 <xsl:call-template name="get-CET-temperature">
                                    <xsl:with-param name="value" select="$mod_power_temp/@val24" />
                                    <xsl:with-param name="precision" select="2"/>
                                 </xsl:call-template>
                              </xsl:otherwise>
						    </xsl:choose>
								</td>
								<td style="background-color:{$mod_status_color_24}"><xsl:value-of select="$mod_status_24"/></td>
							</tr>
						</xsl:if>
						<xsl:if test="($mod_present mod ($bit24 * 2)) - ($mod_present mod $bit24)"><!--module 25 present-->
				  		<tr>
								<td><xsl:value-of select="'25'"/></td>
								<td><xsl:value-of select="$serial_nm/@val25"/></td>
								<td><xsl:value-of select="$ac_in_con_str_25"/></td>
								<td><xsl:value-of select="$modinfo_gr_dc/@val25"/></td>
								<td><xsl:value-of select="$ac_out_con_str_25"/></td>
								<td><xsl:value-of select="$ac_in_power_consumption_25"/></td>
								<td><xsl:value-of select="$ac_out_power_consumption_25"/></td>
								<td><xsl:value-of select="$ac_out_power_va_consumption_25"/></td>
								<td><xsl:value-of select="$mod_power_dc/@val25"/></td>
								<td>
									<xsl:choose>
						  		  <xsl:when test="$mod_power_temp/@val25 &lt; -200"><xsl:value-of select="'---'"/></xsl:when>
										<xsl:otherwise>
                                 <xsl:call-template name="get-CET-temperature">
                                    <xsl:with-param name="value" select="$mod_power_temp/@val25" />
                                    <xsl:with-param name="precision" select="2"/>
                                 </xsl:call-template>
                              </xsl:otherwise>
						    </xsl:choose>
								</td>
								<td style="background-color:{$mod_status_color_25}"><xsl:value-of select="$mod_status_25"/></td>
							</tr>
						</xsl:if>
						<xsl:if test="($mod_present mod ($bit25 * 2)) - ($mod_present mod $bit25)"><!--module 26 present-->
				  		<tr>
								<td><xsl:value-of select="'26'"/></td>
								<td><xsl:value-of select="$serial_nm/@val26"/></td>
								<td><xsl:value-of select="$ac_in_con_str_26"/></td>
								<td><xsl:value-of select="$modinfo_gr_dc/@val26"/></td>
								<td><xsl:value-of select="$ac_out_con_str_26"/></td>
								<td><xsl:value-of select="$ac_in_power_consumption_26"/></td>
								<td><xsl:value-of select="$ac_out_power_consumption_26"/></td>
								<td><xsl:value-of select="$ac_out_power_va_consumption_26"/></td>
								<td><xsl:value-of select="$mod_power_dc/@val26"/></td>
								<td>
									<xsl:choose>
						  		  <xsl:when test="$mod_power_temp/@val26 &lt; -200"><xsl:value-of select="'---'"/></xsl:when>
										<xsl:otherwise>
                                 <xsl:call-template name="get-CET-temperature">
                                    <xsl:with-param name="value" select="$mod_power_temp/@val26" />
                                    <xsl:with-param name="precision" select="2"/>
                                 </xsl:call-template>
                              </xsl:otherwise>
						    </xsl:choose>
								</td>
								<td style="background-color:{$mod_status_color_26}"><xsl:value-of select="$mod_status_26"/></td>
							</tr>
						</xsl:if>
						<xsl:if test="($mod_present mod ($bit26 * 2)) - ($mod_present mod $bit26)"><!--module 27 present-->
				  		<tr>
								<td><xsl:value-of select="'27'"/></td>
								<td><xsl:value-of select="$serial_nm/@val27"/></td>
								<td><xsl:value-of select="$ac_in_con_str_27"/></td>
								<td><xsl:value-of select="$modinfo_gr_dc/@val27"/></td>
								<td><xsl:value-of select="$ac_out_con_str_27"/></td>
								<td><xsl:value-of select="$ac_in_power_consumption_27"/></td>
								<td><xsl:value-of select="$ac_out_power_consumption_27"/></td>
								<td><xsl:value-of select="$ac_out_power_va_consumption_27"/></td>
								<td><xsl:value-of select="$mod_power_dc/@val27"/></td>
								<td>
									<xsl:choose>
						  		  <xsl:when test="$mod_power_temp/@val27 &lt; -200"><xsl:value-of select="'---'"/></xsl:when>
												<xsl:otherwise>
                                 <xsl:call-template name="get-CET-temperature">
                                    <xsl:with-param name="value" select="$mod_power_temp/@val27" />
                                    <xsl:with-param name="precision" select="2"/>
                                 </xsl:call-template>
                              </xsl:otherwise>
						    </xsl:choose>
								</td>
								<td style="background-color:{$mod_status_color_27}"><xsl:value-of select="$mod_status_27"/></td>
							</tr>
						</xsl:if>
						<xsl:if test="($mod_present mod ($bit27 * 2)) - ($mod_present mod $bit27)"><!--module 28 present-->
				  		<tr>
								<td><xsl:value-of select="'28'"/></td>
								<td><xsl:value-of select="$serial_nm/@val28"/></td>
								<td><xsl:value-of select="$ac_in_con_str_28"/></td>
								<td><xsl:value-of select="$modinfo_gr_dc/@val28"/></td>
								<td><xsl:value-of select="$ac_out_con_str_28"/></td>
								<td><xsl:value-of select="$ac_in_power_consumption_28"/></td>
								<td><xsl:value-of select="$ac_out_power_consumption_28"/></td>
								<td><xsl:value-of select="$ac_out_power_va_consumption_28"/></td>
								<td><xsl:value-of select="$mod_power_dc/@val28"/></td>
								<td>
									<xsl:choose>
						  		  <xsl:when test="$mod_power_temp/@val28 &lt; -200"><xsl:value-of select="'---'"/></xsl:when>
										<xsl:otherwise>
                                 <xsl:call-template name="get-CET-temperature">
                                    <xsl:with-param name="value" select="$mod_power_temp/@val28" />
                                    <xsl:with-param name="precision" select="2"/>
                                 </xsl:call-template>
                              </xsl:otherwise>
						    </xsl:choose>
								</td>
								<td style="background-color:{$mod_status_color_28}"><xsl:value-of select="$mod_status_28"/></td>
							</tr>
						</xsl:if>
						<xsl:if test="($mod_present mod ($bit28 * 2)) - ($mod_present mod $bit28)"><!--module 29 present-->
				  		<tr>
								<td><xsl:value-of select="'29'"/></td>
								<td><xsl:value-of select="$serial_nm/@val29"/></td>
								<td><xsl:value-of select="$ac_in_con_str_29"/></td>
								<td><xsl:value-of select="$modinfo_gr_dc/@val29"/></td>
								<td><xsl:value-of select="$ac_out_con_str_29"/></td>
								<td><xsl:value-of select="$ac_in_power_consumption_29"/></td>
								<td><xsl:value-of select="$ac_out_power_consumption_29"/></td>
								<td><xsl:value-of select="$ac_out_power_va_consumption_29"/></td>
								<td><xsl:value-of select="$mod_power_dc/@val29"/></td>
								<td>
									<xsl:choose>
						  		  <xsl:when test="$mod_power_temp/@val29 &lt; -200"><xsl:value-of select="'---'"/></xsl:when>
										<xsl:otherwise>
                                 <xsl:call-template name="get-CET-temperature">
                                    <xsl:with-param name="value" select="$mod_power_temp/@val29" />
                                    <xsl:with-param name="precision" select="2"/>
                                 </xsl:call-template>
                              </xsl:otherwise>
						    </xsl:choose>
								</td>
								<td style="background-color:{$mod_status_color_29}"><xsl:value-of select="$mod_status_29"/></td>
							</tr>
						</xsl:if>
						<xsl:if test="($mod_present mod ($bit29 * 2)) - ($mod_present mod $bit29)"><!--module 30 present-->
				  		<tr>
								<td><xsl:value-of select="'30'"/></td>
								<td><xsl:value-of select="$serial_nm/@val30"/></td>
								<td><xsl:value-of select="$ac_in_con_str_30"/></td>
								<td><xsl:value-of select="$modinfo_gr_dc/@val30"/></td>
								<td><xsl:value-of select="$ac_out_con_str_30"/></td>
								<td><xsl:value-of select="$ac_in_power_consumption_30"/></td>
								<td><xsl:value-of select="$ac_out_power_consumption_30"/></td>
								<td><xsl:value-of select="$ac_out_power_va_consumption_30"/></td>
								<td><xsl:value-of select="$mod_power_dc/@val30"/></td>
								<td>
									<xsl:choose>
						  		  <xsl:when test="$mod_power_temp/@val30 &lt; -200"><xsl:value-of select="'---'"/></xsl:when>
										<xsl:otherwise>
                                 <xsl:call-template name="get-CET-temperature">
                                    <xsl:with-param name="value" select="$mod_power_temp/@val30" />
                                    <xsl:with-param name="precision" select="2"/>
                                 </xsl:call-template>
                              </xsl:otherwise>
						    </xsl:choose>
								</td>
								<td style="background-color:{$mod_status_color_30}"><xsl:value-of select="$mod_status_30"/></td>
							</tr>
						</xsl:if>
						<xsl:if test="($mod_present mod ($bit30 * 2)) - ($mod_present mod $bit30)"><!--module 31 present-->
				  		<tr>
								<td><xsl:value-of select="'31'"/></td>
								<td><xsl:value-of select="$serial_nm/@val31"/></td>
								<td><xsl:value-of select="$ac_in_con_str_31"/></td>
								<td><xsl:value-of select="$modinfo_gr_dc/@val31"/></td>
								<td><xsl:value-of select="$ac_out_con_str_31"/></td>
								<td><xsl:value-of select="$ac_in_power_consumption_31"/></td>
								<td><xsl:value-of select="$ac_out_power_consumption_31"/></td>
								<td><xsl:value-of select="$ac_out_power_va_consumption_31"/></td>
								<td><xsl:value-of select="$mod_power_dc/@val31"/></td>
								<td>
									<xsl:choose>
						  		  <xsl:when test="$mod_power_temp/@val31 &lt; -200"><xsl:value-of select="'---'"/></xsl:when>
										<xsl:otherwise>
                                 <xsl:call-template name="get-CET-temperature">
                                    <xsl:with-param name="value" select="$mod_power_temp/@val31" />
                                    <xsl:with-param name="precision" select="2"/>
                                 </xsl:call-template>
                              </xsl:otherwise>
						    </xsl:choose>
								</td>
								<td style="background-color:{$mod_status_color_31}"><xsl:value-of select="$mod_status_31"/></td>
							</tr>
						</xsl:if>
						<xsl:if test="($mod_present mod ($bit31 * 2)) - ($mod_present mod $bit31)"><!--module 32 present-->
				  		<tr>
								<td><xsl:value-of select="'32'"/></td>
								<td><xsl:value-of select="$serial_nm/@val32"/></td>
								<td><xsl:value-of select="$ac_in_con_str_32"/></td>
								<td><xsl:value-of select="$modinfo_gr_dc/@val32"/></td>
								<td><xsl:value-of select="$ac_out_con_str_32"/></td>
								<td><xsl:value-of select="$ac_in_power_consumption_32"/></td>
								<td><xsl:value-of select="$ac_out_power_consumption_32"/></td>
								<td><xsl:value-of select="$ac_out_power_va_consumption_32"/></td>
								<td><xsl:value-of select="$mod_power_dc/@val32"/></td>
								<td>
									<xsl:choose>
						  		  <xsl:when test="$mod_power_temp/@val32 &lt; -200"><xsl:value-of select="'---'"/></xsl:when>
										<xsl:otherwise>
                                 <xsl:call-template name="get-CET-temperature">
                                    <xsl:with-param name="value" select="$mod_power_temp/@val32" />
                                    <xsl:with-param name="precision" select="2"/>
                                 </xsl:call-template>
                              </xsl:otherwise>
						    </xsl:choose>
								</td>
								<td style="background-color:{$mod_status_color_32}"><xsl:value-of select="$mod_status_32"/></td>
							</tr>
						</xsl:if>
					</tbody>
				</table>
			</div>
		</xsl:if>
	</xsl:template>

	<!-- CET alarm comm lost pane-->
	<xsl:template name="alarm-comm-lost-pane">
		<xsl:param name="alarm_com_lost" select="'0'"/>
		<xsl:param name="alarm_msg" select="'Communication Lost with UPS'"/>
		<xsl:param name="x" select="320" />
	  <xsl:param name="y" select="40" />
	  <xsl:param name="w" select="190" />
	  <xsl:param name="h" select="40" />
	  <xsl:param name="align" select="'center'" />
	 	<xsl:param name="class" select="'XBold_black'" />
		<xsl:param name="id" select="'alarm_comm_lost_div'" />

		<xsl:if test="$alarm_com_lost=1">
	  	<div id="{$id}" class="{$class}" align="center" style="position:absolute; left:{$x}px; top:{$y}px; width:{$w}px; height:{$h}px;text-align:center;border:1px solid white;background-color:white;">
				<table border="0" cellspacing="2" cellpadding="2" class="{$class}" style="position:relative;top:10px; width:{$w}px;color:black;background-color:white;text-align:center;">
					<tr>
						<td style="color:#C51923;text-align:center;"><xsl:value-of select="$alarm_msg"/></td>
					</tr>
				</table>
			</div>
		</xsl:if>
	 	<!--<script type="text/javascript">
			function toggle()
 			{
 				var div = document.getElementById('<xsl:value-of select="$id"/>');
 				var tmp = div.style.visibility;
 				if(div != null)
 					div.style.visibility=="hidden"?div.style.visibility="visible":div.style.visibility="hidden";
 			}
	 		setInterval("toggle()", 600);
	 	</script>-->

	</xsl:template>

 </xsl:stylesheet>




